/******************************************************************************
 *
 * Copyright (C) 1997-2021 by Dimitri van Heesch.
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation under the terms of the GNU General Public License is hereby
 * granted. No representations are made about the suitability of this software
 * for any purpose. It is provided "as is" without express or implied warranty.
 * See the GNU General Public License for more details.
 *
 * Documents produced by Doxygen are derivative works derived from the
 * input used in their production; they are not affected by this license.
 *
 */

#ifndef HTMLDOCVISITOR_H
#define HTMLDOCVISITOR_H

#include "docvisitor.h"
#include "docnode.h"
#include "dstring.h"

class Definition;
class MemberDef;
class OutputCodeList;
class TextStream;

/*! @brief Concrete visitor implementation for HTML output. */
class HtmlDocVisitor final : public DocVisitor
{
  public:
    HtmlDocVisitor(TextStream &t,OutputCodeList &ci,const Definition *ctx,const DString &fn = DString());

    //--------------------------------------
    // visitor functions for leaf nodes
    //--------------------------------------

    void operator()(const DocWord &);
    void operator()(const DocLinkedWord &);
    void operator()(const DocWhiteSpace &);
    void operator()(const DocSymbol &);
    void operator()(const DocEmoji &);
    void operator()(const DocURL &);
    void operator()(const DocLineBreak &);
    void operator()(const DocHorRuler &);
    void operator()(const DocStyleChange &);
    void operator()(const DocVerbatim &);
    void operator()(const DocAnchor &);
    void operator()(const DocInclude &);
    void operator()(const DocIncOperator &);
    void operator()(const DocFormula &);
    void operator()(const DocIndexEntry &);
    void operator()(const DocSimpleSectSep &);
    void operator()(const DocCite &);
    void operator()(const DocSeparator &);

    //--------------------------------------
    // visitor functions for compound nodes
    //--------------------------------------

    void operator()(const DocAutoList &);
    void operator()(const DocAutoListItem &);
    void operator()(const DocPara &) ;
    void operator()(const DocRoot &);
    void operator()(const DocSimpleSect &);
    void operator()(const DocTitle &);
    void operator()(const DocSimpleList &);
    void operator()(const DocSimpleListItem &);
    void operator()(const DocSection &);
    void operator()(const DocHtmlList &);
    void operator()(const DocHtmlListItem &);
    void operator()(const DocHtmlDescList &);
    void operator()(const DocHtmlDescTitle &);
    void operator()(const DocHtmlDescData &);
    void operator()(const DocHtmlTable &);
    void operator()(const DocHtmlRow &);
    void operator()(const DocHtmlCell &);
    void operator()(const DocHtmlCaption &);
    void operator()(const DocInternal &);
    void operator()(const DocHRef &);
    void operator()(const DocHtmlSummary &);
    void operator()(const DocHtmlDetails &);
    void operator()(const DocHtmlHeader &);
    void operator()(const DocImage &);
    void operator()(const DocDotFile &);
    void operator()(const DocMscFile &);
    void operator()(const DocDiaFile &);
    void operator()(const DocPlantUmlFile &);
    void operator()(const DocMermaidFile &);
    void operator()(const DocLink &);
    void operator()(const DocRef &);
    void operator()(const DocSecRefItem &);
    void operator()(const DocSecRefList &);
    void operator()(const DocParamSect &);
    void operator()(const DocParamList &);
    void operator()(const DocXRefItem &);
    void operator()(const DocInternalRef &);
    void operator()(const DocText &);
    void operator()(const DocHtmlBlockQuote &);
    void operator()(const DocVhdlFlow &);
    void operator()(const DocParBlock &);

  private:

    template<class T>
    void visitChildren(const T &t)
    {
      for (const auto &child : t.children())
      {
        std::visit(*this, child);
      }
    }
    template<class T>
    void visitCaption(TextStream &t,const T &n);

    //--------------------------------------
    // helper functions
    //--------------------------------------

    void writeObfuscatedMailAddress(const DString &url);
    void filter(const DString &str, bool retainNewline = false, bool citeEntry = false);
    DString filterQuotedCdataAttr(const DString &str);
    void startLink(const DString &ref,const DString &file,
                   const DString &relPath,const DString &anchor,
                   const DString &tooltip = "");
    void endLink();
    void writeDotFile(const DString &fileName,const DString &relPath,const DString &context,
                      const DString &srcFile,int srcLine, bool newFile = true);
    void writeMscFile(const DString &fileName,const DString &relPath,const DString &context,
                      const DString &srcFile,int srcLine, bool newFile = true);
    void writeDiaFile(const DString &fileName,const DString &relPath,const DString &context,
                      const DString &srcFile,int srcLine, bool newFile = true);
    void writePlantUMLFile(const DString &fileName,const DString &relPath,const DString &context,
                           const DString &srcFile,int srcLine);
    void writeMermaidFile(const DString &fileName,const DString &relPath,const DString &context,
                          const DString &srcFile,int srcLine);

    template<class DocNode>
    void forceEndParagraph(const DocNode &n);
    template<class DocNode>
    void forceStartParagraph(const DocNode &n);

    //--------------------------------------
    // state variables
    //--------------------------------------

    TextStream &m_t;
    OutputCodeList &m_ci;
    bool m_insidePre = false;
    bool m_hide = false;
    bool m_insideTitle = false;
    const Definition *m_ctx;
    DString m_fileName;
    DString m_langExt;
};

#endif
