/****************************************************************************
** Meta object code from reading C++ file 'expert.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.9.4)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../src/doxygen-1.18.0/addon/doxywizard/expert.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'expert.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.9.4. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_Expert_t {
    QByteArrayData data[16];
    char stringdata0[153];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Expert_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Expert_t qt_meta_stringdata_Expert = {
    {
QT_MOC_LITERAL(0, 0, 6), // "Expert"
QT_MOC_LITERAL(1, 7, 7), // "changed"
QT_MOC_LITERAL(2, 15, 0), // ""
QT_MOC_LITERAL(3, 16, 4), // "done"
QT_MOC_LITERAL(4, 21, 18), // "createMessagesList"
QT_MOC_LITERAL(5, 40, 12), // "QDomElement&"
QT_MOC_LITERAL(6, 53, 4), // "elem"
QT_MOC_LITERAL(7, 58, 7), // "refresh"
QT_MOC_LITERAL(8, 66, 13), // "filterChanged"
QT_MOC_LITERAL(9, 80, 4), // "text"
QT_MOC_LITERAL(10, 85, 13), // "groupSelected"
QT_MOC_LITERAL(11, 99, 16), // "QTreeWidgetItem*"
QT_MOC_LITERAL(12, 116, 7), // "current"
QT_MOC_LITERAL(13, 124, 8), // "previous"
QT_MOC_LITERAL(14, 133, 9), // "prevGroup"
QT_MOC_LITERAL(15, 143, 9) // "nextGroup"

    },
    "Expert\0changed\0\0done\0createMessagesList\0"
    "QDomElement&\0elem\0refresh\0filterChanged\0"
    "text\0groupSelected\0QTreeWidgetItem*\0"
    "current\0previous\0prevGroup\0nextGroup"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Expert[] = {

 // content:
       7,       // revision
       0,       // classname
       0,    0, // classinfo
       8,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   54,    2, 0x06 /* Public */,
       3,    0,   55,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       4,    1,   56,    2, 0x0a /* Public */,
       7,    0,   59,    2, 0x0a /* Public */,
       8,    1,   60,    2, 0x08 /* Private */,
      10,    2,   63,    2, 0x08 /* Private */,
      14,    0,   68,    2, 0x08 /* Private */,
      15,    0,   69,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 5,    6,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    9,
    QMetaType::Void, 0x80000000 | 11, 0x80000000 | 11,   12,   13,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void Expert::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        Expert *_t = static_cast<Expert *>(_o);
        Q_UNUSED(_t)
        switch (_id) {
        case 0: _t->changed(); break;
        case 1: _t->done(); break;
        case 2: _t->createMessagesList((*reinterpret_cast< QDomElement(*)>(_a[1]))); break;
        case 3: _t->refresh(); break;
        case 4: _t->filterChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 5: _t->groupSelected((*reinterpret_cast< QTreeWidgetItem*(*)>(_a[1])),(*reinterpret_cast< QTreeWidgetItem*(*)>(_a[2]))); break;
        case 6: _t->prevGroup(); break;
        case 7: _t->nextGroup(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            typedef void (Expert::*_t)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Expert::changed)) {
                *result = 0;
                return;
            }
        }
        {
            typedef void (Expert::*_t)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Expert::done)) {
                *result = 1;
                return;
            }
        }
    }
}

const QMetaObject Expert::staticMetaObject = {
    { &QWidget::staticMetaObject, qt_meta_stringdata_Expert.data,
      qt_meta_data_Expert,  qt_static_metacall, nullptr, nullptr}
};


const QMetaObject *Expert::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Expert::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Expert.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "DocIntf"))
        return static_cast< DocIntf*>(this);
    return QWidget::qt_metacast(_clname);
}

int Expert::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 8;
    }
    return _id;
}

// SIGNAL 0
void Expert::changed()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void Expert::done()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
