/****************************************************************************
** Meta object code from reading C++ file 'doxywizard.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.9.4)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../src/doxygen-1.18.0/addon/doxywizard/doxywizard.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'doxywizard.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.9.4. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_MainWindow_t {
    QByteArrayData data[28];
    char stringdata0[331];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_MainWindow_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_MainWindow_t qt_meta_stringdata_MainWindow = {
    {
QT_MOC_LITERAL(0, 0, 10), // "MainWindow"
QT_MOC_LITERAL(1, 11, 6), // "manual"
QT_MOC_LITERAL(2, 18, 0), // ""
QT_MOC_LITERAL(3, 19, 5), // "about"
QT_MOC_LITERAL(4, 25, 10), // "openConfig"
QT_MOC_LITERAL(5, 36, 10), // "saveConfig"
QT_MOC_LITERAL(6, 47, 12), // "saveConfigAs"
QT_MOC_LITERAL(7, 60, 12), // "makeDefaults"
QT_MOC_LITERAL(8, 73, 15), // "resetToDefaults"
QT_MOC_LITERAL(9, 89, 9), // "selectTab"
QT_MOC_LITERAL(10, 99, 4), // "quit"
QT_MOC_LITERAL(11, 104, 14), // "switchLanguage"
QT_MOC_LITERAL(12, 119, 26), // "setDocumentationVisibility"
QT_MOC_LITERAL(13, 146, 10), // "openRecent"
QT_MOC_LITERAL(14, 157, 8), // "QAction*"
QT_MOC_LITERAL(15, 166, 6), // "action"
QT_MOC_LITERAL(16, 173, 16), // "selectWorkingDir"
QT_MOC_LITERAL(17, 190, 16), // "updateWorkingDir"
QT_MOC_LITERAL(18, 207, 10), // "runDoxygen"
QT_MOC_LITERAL(19, 218, 14), // "runDoxygenMenu"
QT_MOC_LITERAL(20, 233, 10), // "readStdout"
QT_MOC_LITERAL(21, 244, 11), // "runComplete"
QT_MOC_LITERAL(22, 256, 14), // "showHtmlOutput"
QT_MOC_LITERAL(23, 271, 7), // "saveLog"
QT_MOC_LITERAL(24, 279, 12), // "showSettings"
QT_MOC_LITERAL(25, 292, 13), // "configChanged"
QT_MOC_LITERAL(26, 306, 11), // "clearRecent"
QT_MOC_LITERAL(27, 318, 12) // "selectRunTab"

    },
    "MainWindow\0manual\0\0about\0openConfig\0"
    "saveConfig\0saveConfigAs\0makeDefaults\0"
    "resetToDefaults\0selectTab\0quit\0"
    "switchLanguage\0setDocumentationVisibility\0"
    "openRecent\0QAction*\0action\0selectWorkingDir\0"
    "updateWorkingDir\0runDoxygen\0runDoxygenMenu\0"
    "readStdout\0runComplete\0showHtmlOutput\0"
    "saveLog\0showSettings\0configChanged\0"
    "clearRecent\0selectRunTab"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_MainWindow[] = {

 // content:
       7,       // revision
       0,       // classname
       0,    0, // classinfo
      24,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,  134,    2, 0x0a /* Public */,
       3,    0,  135,    2, 0x0a /* Public */,
       4,    0,  136,    2, 0x0a /* Public */,
       5,    0,  137,    2, 0x0a /* Public */,
       6,    0,  138,    2, 0x0a /* Public */,
       7,    0,  139,    2, 0x0a /* Public */,
       8,    0,  140,    2, 0x0a /* Public */,
       9,    1,  141,    2, 0x0a /* Public */,
      10,    0,  144,    2, 0x0a /* Public */,
      11,    0,  145,    2, 0x0a /* Public */,
      12,    0,  146,    2, 0x0a /* Public */,
      13,    1,  147,    2, 0x08 /* Private */,
      16,    0,  150,    2, 0x08 /* Private */,
      17,    0,  151,    2, 0x08 /* Private */,
      18,    0,  152,    2, 0x08 /* Private */,
      19,    0,  153,    2, 0x08 /* Private */,
      20,    0,  154,    2, 0x08 /* Private */,
      21,    0,  155,    2, 0x08 /* Private */,
      22,    0,  156,    2, 0x08 /* Private */,
      23,    0,  157,    2, 0x08 /* Private */,
      24,    0,  158,    2, 0x08 /* Private */,
      25,    0,  159,    2, 0x08 /* Private */,
      26,    0,  160,    2, 0x08 /* Private */,
      27,    0,  161,    2, 0x08 /* Private */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Bool,
    QMetaType::Bool,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 14,   15,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void MainWindow::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        MainWindow *_t = static_cast<MainWindow *>(_o);
        Q_UNUSED(_t)
        switch (_id) {
        case 0: _t->manual(); break;
        case 1: _t->about(); break;
        case 2: _t->openConfig(); break;
        case 3: { bool _r = _t->saveConfig();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 4: { bool _r = _t->saveConfigAs();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 5: _t->makeDefaults(); break;
        case 6: _t->resetToDefaults(); break;
        case 7: _t->selectTab((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 8: _t->quit(); break;
        case 9: _t->switchLanguage(); break;
        case 10: _t->setDocumentationVisibility(); break;
        case 11: _t->openRecent((*reinterpret_cast< QAction*(*)>(_a[1]))); break;
        case 12: _t->selectWorkingDir(); break;
        case 13: _t->updateWorkingDir(); break;
        case 14: _t->runDoxygen(); break;
        case 15: _t->runDoxygenMenu(); break;
        case 16: _t->readStdout(); break;
        case 17: _t->runComplete(); break;
        case 18: _t->showHtmlOutput(); break;
        case 19: _t->saveLog(); break;
        case 20: _t->showSettings(); break;
        case 21: _t->configChanged(); break;
        case 22: _t->clearRecent(); break;
        case 23: _t->selectRunTab(); break;
        default: ;
        }
    }
}

const QMetaObject MainWindow::staticMetaObject = {
    { &QMainWindow::staticMetaObject, qt_meta_stringdata_MainWindow.data,
      qt_meta_data_MainWindow,  qt_static_metacall, nullptr, nullptr}
};


const QMetaObject *MainWindow::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *MainWindow::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_MainWindow.stringdata0))
        return static_cast<void*>(this);
    return QMainWindow::qt_metacast(_clname);
}

int MainWindow::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QMainWindow::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 24)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 24;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 24)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 24;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
