#ifndef CONFIGSWITCHER_H
#define CONFIGSWITCHER_H

using LangFunc = void (*)(DocIntf *);

void addConfigDocs(DocIntf *doc);
void addConfigDocs_de(DocIntf *doc);
void addConfigDocs_es(DocIntf *doc);
void addConfigDocs_fr(DocIntf *doc);
void addConfigDocs_ja(DocIntf *doc);
void addConfigDocs_ko(DocIntf *doc);
void addConfigDocs_ru(DocIntf *doc);
void addConfigDocs_zh_CN(DocIntf *doc);
void addConfigDocs_zh_TW(DocIntf *doc);

static const std::unordered_map<std::string,LangFunc> langNames =
{
  { "en", addConfigDocs},
  { "de", addConfigDocs_de},
  { "es", addConfigDocs_es},
  { "fr", addConfigDocs_fr},
  { "ja", addConfigDocs_ja},
  { "ko", addConfigDocs_ko},
  { "ru", addConfigDocs_ru},
  { "zh_CN", addConfigDocs_zh_CN},
  { "zh_TW", addConfigDocs_zh_TW},
};

#endif
