/****************************************************************************
** Meta object code from reading C++ file 'inputstring.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.9.4)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../src/doxygen-1.18.0/addon/doxywizard/inputstring.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'inputstring.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.9.4. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_InputString_t {
    QByteArrayData data[11];
    char stringdata0[84];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_InputString_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_InputString_t qt_meta_stringdata_InputString = {
    {
QT_MOC_LITERAL(0, 0, 11), // "InputString"
QT_MOC_LITERAL(1, 12, 7), // "changed"
QT_MOC_LITERAL(2, 20, 0), // ""
QT_MOC_LITERAL(3, 21, 8), // "showHelp"
QT_MOC_LITERAL(4, 30, 6), // "Input*"
QT_MOC_LITERAL(5, 37, 5), // "reset"
QT_MOC_LITERAL(6, 43, 8), // "setValue"
QT_MOC_LITERAL(7, 52, 10), // "browseFile"
QT_MOC_LITERAL(8, 63, 9), // "browseDir"
QT_MOC_LITERAL(9, 73, 5), // "clear"
QT_MOC_LITERAL(10, 79, 4) // "help"

    },
    "InputString\0changed\0\0showHelp\0Input*\0"
    "reset\0setValue\0browseFile\0browseDir\0"
    "clear\0help"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_InputString[] = {

 // content:
       7,       // revision
       0,       // classname
       0,    0, // classinfo
       8,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   54,    2, 0x06 /* Public */,
       3,    1,   55,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       5,    0,   58,    2, 0x0a /* Public */,
       6,    1,   59,    2, 0x0a /* Public */,
       7,    0,   62,    2, 0x08 /* Private */,
       8,    0,   63,    2, 0x08 /* Private */,
       9,    0,   64,    2, 0x08 /* Private */,
      10,    0,   65,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 4,    2,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void InputString::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        InputString *_t = static_cast<InputString *>(_o);
        Q_UNUSED(_t)
        switch (_id) {
        case 0: _t->changed(); break;
        case 1: _t->showHelp((*reinterpret_cast< Input*(*)>(_a[1]))); break;
        case 2: _t->reset(); break;
        case 3: _t->setValue((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 4: _t->browseFile(); break;
        case 5: _t->browseDir(); break;
        case 6: _t->clear(); break;
        case 7: _t->help(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            typedef void (InputString::*_t)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&InputString::changed)) {
                *result = 0;
                return;
            }
        }
        {
            typedef void (InputString::*_t)(Input * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&InputString::showHelp)) {
                *result = 1;
                return;
            }
        }
    }
}

const QMetaObject InputString::staticMetaObject = {
    { &QObject::staticMetaObject, qt_meta_stringdata_InputString.data,
      qt_meta_data_InputString,  qt_static_metacall, nullptr, nullptr}
};


const QMetaObject *InputString::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *InputString::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_InputString.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Input"))
        return static_cast< Input*>(this);
    return QObject::qt_metacast(_clname);
}

int InputString::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 8;
    }
    return _id;
}

// SIGNAL 0
void InputString::changed()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void InputString::showHelp(Input * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(&_t1)) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
