#line 1 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner_intermediate.cpp"
#include <stdint.h>
// forward declare yyscan_t to improve type safety
#define YY_TYPEDEF_YY_SCANNER_T
struct yyguts_t;
typedef yyguts_t *yyscan_t;

#line 8 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner_intermediate.cpp"

#define  YY_INT_ALIGNED short int

/* A lexical scanner generated by flex */

#define FLEX_SCANNER
#define YY_FLEX_MAJOR_VERSION 2
#define YY_FLEX_MINOR_VERSION 6
#define YY_FLEX_SUBMINOR_VERSION 4
#if YY_FLEX_SUBMINOR_VERSION > 0
#define FLEX_BETA
#endif

#ifdef yy_create_buffer
#define pyscannerYY_create_buffer_ALREADY_DEFINED
#else
#define yy_create_buffer pyscannerYY_create_buffer
#endif

#ifdef yy_delete_buffer
#define pyscannerYY_delete_buffer_ALREADY_DEFINED
#else
#define yy_delete_buffer pyscannerYY_delete_buffer
#endif

#ifdef yy_scan_buffer
#define pyscannerYY_scan_buffer_ALREADY_DEFINED
#else
#define yy_scan_buffer pyscannerYY_scan_buffer
#endif

#ifdef yy_scan_string
#define pyscannerYY_scan_string_ALREADY_DEFINED
#else
#define yy_scan_string pyscannerYY_scan_string
#endif

#ifdef yy_scan_bytes
#define pyscannerYY_scan_bytes_ALREADY_DEFINED
#else
#define yy_scan_bytes pyscannerYY_scan_bytes
#endif

#ifdef yy_init_buffer
#define pyscannerYY_init_buffer_ALREADY_DEFINED
#else
#define yy_init_buffer pyscannerYY_init_buffer
#endif

#ifdef yy_flush_buffer
#define pyscannerYY_flush_buffer_ALREADY_DEFINED
#else
#define yy_flush_buffer pyscannerYY_flush_buffer
#endif

#ifdef yy_load_buffer_state
#define pyscannerYY_load_buffer_state_ALREADY_DEFINED
#else
#define yy_load_buffer_state pyscannerYY_load_buffer_state
#endif

#ifdef yy_switch_to_buffer
#define pyscannerYY_switch_to_buffer_ALREADY_DEFINED
#else
#define yy_switch_to_buffer pyscannerYY_switch_to_buffer
#endif

#ifdef yypush_buffer_state
#define pyscannerYYpush_buffer_state_ALREADY_DEFINED
#else
#define yypush_buffer_state pyscannerYYpush_buffer_state
#endif

#ifdef yypop_buffer_state
#define pyscannerYYpop_buffer_state_ALREADY_DEFINED
#else
#define yypop_buffer_state pyscannerYYpop_buffer_state
#endif

#ifdef yyensure_buffer_stack
#define pyscannerYYensure_buffer_stack_ALREADY_DEFINED
#else
#define yyensure_buffer_stack pyscannerYYensure_buffer_stack
#endif

#ifdef yylex
#define pyscannerYYlex_ALREADY_DEFINED
#else
#define yylex pyscannerYYlex
#endif

#ifdef yyrestart
#define pyscannerYYrestart_ALREADY_DEFINED
#else
#define yyrestart pyscannerYYrestart
#endif

#ifdef yylex_init
#define pyscannerYYlex_init_ALREADY_DEFINED
#else
#define yylex_init pyscannerYYlex_init
#endif

#ifdef yylex_init_extra
#define pyscannerYYlex_init_extra_ALREADY_DEFINED
#else
#define yylex_init_extra pyscannerYYlex_init_extra
#endif

#ifdef yylex_destroy
#define pyscannerYYlex_destroy_ALREADY_DEFINED
#else
#define yylex_destroy pyscannerYYlex_destroy
#endif

#ifdef yyget_debug
#define pyscannerYYget_debug_ALREADY_DEFINED
#else
#define yyget_debug pyscannerYYget_debug
#endif

#ifdef yyset_debug
#define pyscannerYYset_debug_ALREADY_DEFINED
#else
#define yyset_debug pyscannerYYset_debug
#endif

#ifdef yyget_extra
#define pyscannerYYget_extra_ALREADY_DEFINED
#else
#define yyget_extra pyscannerYYget_extra
#endif

#ifdef yyset_extra
#define pyscannerYYset_extra_ALREADY_DEFINED
#else
#define yyset_extra pyscannerYYset_extra
#endif

#ifdef yyget_in
#define pyscannerYYget_in_ALREADY_DEFINED
#else
#define yyget_in pyscannerYYget_in
#endif

#ifdef yyset_in
#define pyscannerYYset_in_ALREADY_DEFINED
#else
#define yyset_in pyscannerYYset_in
#endif

#ifdef yyget_out
#define pyscannerYYget_out_ALREADY_DEFINED
#else
#define yyget_out pyscannerYYget_out
#endif

#ifdef yyset_out
#define pyscannerYYset_out_ALREADY_DEFINED
#else
#define yyset_out pyscannerYYset_out
#endif

#ifdef yyget_leng
#define pyscannerYYget_leng_ALREADY_DEFINED
#else
#define yyget_leng pyscannerYYget_leng
#endif

#ifdef yyget_text
#define pyscannerYYget_text_ALREADY_DEFINED
#else
#define yyget_text pyscannerYYget_text
#endif

#ifdef yyget_lineno
#define pyscannerYYget_lineno_ALREADY_DEFINED
#else
#define yyget_lineno pyscannerYYget_lineno
#endif

#ifdef yyset_lineno
#define pyscannerYYset_lineno_ALREADY_DEFINED
#else
#define yyset_lineno pyscannerYYset_lineno
#endif

#ifdef yyget_column
#define pyscannerYYget_column_ALREADY_DEFINED
#else
#define yyget_column pyscannerYYget_column
#endif

#ifdef yyset_column
#define pyscannerYYset_column_ALREADY_DEFINED
#else
#define yyset_column pyscannerYYset_column
#endif

#ifdef yywrap
#define pyscannerYYwrap_ALREADY_DEFINED
#else
#define yywrap pyscannerYYwrap
#endif

#ifdef yyalloc
#define pyscannerYYalloc_ALREADY_DEFINED
#else
#define yyalloc pyscannerYYalloc
#endif

#ifdef yyrealloc
#define pyscannerYYrealloc_ALREADY_DEFINED
#else
#define yyrealloc pyscannerYYrealloc
#endif

#ifdef yyfree
#define pyscannerYYfree_ALREADY_DEFINED
#else
#define yyfree pyscannerYYfree
#endif

/* First, we deal with  platform-specific or compiler-specific issues. */

/* begin standard C headers. */
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <stdlib.h>

/* end standard C headers. */

/* flex integer type definitions */

#ifndef FLEXINT_H
#define FLEXINT_H

/* C99 systems have <inttypes.h>. Non-C99 systems may or may not. */

#if defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L

/* C99 says to define __STDC_LIMIT_MACROS before including stdint.h,
 * if you want the limit (max/min) macros for int types. 
 */
#ifndef __STDC_LIMIT_MACROS
#define __STDC_LIMIT_MACROS 1
#endif

#include <inttypes.h>
typedef int8_t flex_int8_t;
typedef uint8_t flex_uint8_t;
typedef int16_t flex_int16_t;
typedef uint16_t flex_uint16_t;
typedef int32_t flex_int32_t;
typedef uint32_t flex_uint32_t;
#else
typedef signed char flex_int8_t;
typedef short int flex_int16_t;
typedef int flex_int32_t;
typedef unsigned char flex_uint8_t; 
typedef unsigned short int flex_uint16_t;
typedef unsigned int flex_uint32_t;

/* Limits of integral types. */
#ifndef INT8_MIN
#define INT8_MIN               (-128)
#endif
#ifndef INT16_MIN
#define INT16_MIN              (-32767-1)
#endif
#ifndef INT32_MIN
#define INT32_MIN              (-2147483647-1)
#endif
#ifndef INT8_MAX
#define INT8_MAX               (127)
#endif
#ifndef INT16_MAX
#define INT16_MAX              (32767)
#endif
#ifndef INT32_MAX
#define INT32_MAX              (2147483647)
#endif
#ifndef UINT8_MAX
#define UINT8_MAX              (255U)
#endif
#ifndef UINT16_MAX
#define UINT16_MAX             (65535U)
#endif
#ifndef UINT32_MAX
#define UINT32_MAX             (4294967295U)
#endif

#ifndef SIZE_MAX
#define SIZE_MAX               (~(size_t)0)
#endif

#endif /* ! C99 */

#endif /* ! FLEXINT_H */

/* begin standard C++ headers. */

/* TODO: this is always defined, so inline it */
#define yyconst const

#if defined(__GNUC__) && __GNUC__ >= 3
#define yynoreturn __attribute__((__noreturn__))
#else
#define yynoreturn
#endif

/* Returned upon end-of-file. */
#define YY_NULL 0

/* Promotes a possibly negative, possibly signed char to an
 *   integer in range [0..255] for use as an array index.
 */
#define YY_SC_TO_UI(c) ((YY_CHAR) (c))

/* An opaque pointer. */
#ifndef YY_TYPEDEF_YY_SCANNER_T
#define YY_TYPEDEF_YY_SCANNER_T
typedef void* yyscan_t;
#endif

/* For convenience, these vars (plus the bison vars far below)
   are macros in the reentrant scanner. */
#define yyin yyg->yyin_r
#define yyout yyg->yyout_r
#define yyextra yyg->yyextra_r
#define yyleng yyg->yyleng_r
#define yytext yyg->yytext_r
#define yylineno (YY_CURRENT_BUFFER_LVALUE->yy_bs_lineno)
#define yycolumn (YY_CURRENT_BUFFER_LVALUE->yy_bs_column)
#define yy_flex_debug yyg->yy_flex_debug_r

/* Enter a start condition.  This macro really ought to take a parameter,
 * but we do it the disgusting crufty way forced on us by the ()-less
 * definition of BEGIN.
 */
#define BEGIN yyg->yy_start = 1 + 2 *
/* Translate the current start state into a value that can be later handed
 * to BEGIN to return to the state.  The YYSTATE alias is for lex
 * compatibility.
 */
#define YY_START ((yyg->yy_start - 1) / 2)
#define YYSTATE YY_START
/* Action number for EOF rule of a given start state. */
#define YY_STATE_EOF(state) (YY_END_OF_BUFFER + state + 1)
/* Special action meaning "start processing a new file". */
#define YY_NEW_FILE yyrestart( yyin , yyscanner )
#define YY_END_OF_BUFFER_CHAR 0

/* Size of default input buffer. */
#ifndef YY_BUF_SIZE
#ifdef __ia64__
/* On IA-64, the buffer size is 16k, not 8k.
 * Moreover, YY_BUF_SIZE is 2*YY_READ_BUF_SIZE in the general case.
 * Ditto for the __ia64__ case accordingly.
 */
#define YY_BUF_SIZE 32768
#else
#define YY_BUF_SIZE 16384
#endif /* __ia64__ */
#endif

/* The state buf must be large enough to hold one state per character in the main buffer.
 */
#define YY_STATE_BUF_SIZE   ((YY_BUF_SIZE + 2) * sizeof(yy_state_type))

#ifndef YY_TYPEDEF_YY_BUFFER_STATE
#define YY_TYPEDEF_YY_BUFFER_STATE
typedef struct yy_buffer_state *YY_BUFFER_STATE;
#endif

#ifndef YY_TYPEDEF_YY_SIZE_T
#define YY_TYPEDEF_YY_SIZE_T
typedef size_t yy_size_t;
#endif

#define EOB_ACT_CONTINUE_SCAN 0
#define EOB_ACT_END_OF_FILE 1
#define EOB_ACT_LAST_MATCH 2
    
    #define YY_LESS_LINENO(n)
    #define YY_LINENO_REWIND_TO(ptr)
    
/* Return all but the first "n" matched characters back to the input stream. */
#define yyless(n) \
	do \
		{ \
		/* Undo effects of setting up yytext. */ \
        int yyless_macro_arg = (n); \
        YY_LESS_LINENO(yyless_macro_arg);\
		*yy_cp = yyg->yy_hold_char; \
		YY_RESTORE_YY_MORE_OFFSET \
		yyg->yy_c_buf_p = yy_cp = yy_bp + yyless_macro_arg - YY_MORE_ADJ; \
		YY_DO_BEFORE_ACTION; /* set up yytext again */ \
		} \
	while ( 0 )
#define unput(c) yyunput( c, yyg->yytext_ptr , yyscanner )

#ifndef YY_STRUCT_YY_BUFFER_STATE
#define YY_STRUCT_YY_BUFFER_STATE
struct yy_buffer_state
	{
	FILE *yy_input_file;

	char *yy_ch_buf;		/* input buffer */
	char *yy_buf_pos;		/* current position in input buffer */

	/* Size of input buffer in bytes, not including room for EOB
	 * characters.
	 */
	int yy_buf_size;

	/* Number of characters read into yy_ch_buf, not including EOB
	 * characters.
	 */
	int yy_n_chars;

	/* Whether we "own" the buffer - i.e., we know we created it,
	 * and can realloc() it to grow it, and should free() it to
	 * delete it.
	 */
	int yy_is_our_buffer;

	/* Whether this is an "interactive" input source; if so, and
	 * if we're using stdio for input, then we want to use getc()
	 * instead of fread(), to make sure we stop fetching input after
	 * each newline.
	 */
	int yy_is_interactive;

	/* Whether we're considered to be at the beginning of a line.
	 * If so, '^' rules will be active on the next match, otherwise
	 * not.
	 */
	int yy_at_bol;

    int yy_bs_lineno; /**< The line count. */
    int yy_bs_column; /**< The column count. */

	/* Whether to try to fill the input buffer when we reach the
	 * end of it.
	 */
	int yy_fill_buffer;

	int yy_buffer_status;

#define YY_BUFFER_NEW 0
#define YY_BUFFER_NORMAL 1
	/* When an EOF's been seen but there's still some text to process
	 * then we mark the buffer as YY_EOF_PENDING, to indicate that we
	 * shouldn't try reading from the input source any more.  We might
	 * still have a bunch of tokens to match, though, because of
	 * possible backing-up.
	 *
	 * When we actually see the EOF, we change the status to "new"
	 * (via yyrestart()), so that the user can continue scanning by
	 * just pointing yyin at a new input file.
	 */
#define YY_BUFFER_EOF_PENDING 2

	};
#endif /* !YY_STRUCT_YY_BUFFER_STATE */

/* We provide macros for accessing buffer states in case in the
 * future we want to put the buffer states in a more general
 * "scanner state".
 *
 * Returns the top of the stack, or NULL.
 */
#define YY_CURRENT_BUFFER ( yyg->yy_buffer_stack \
                          ? yyg->yy_buffer_stack[yyg->yy_buffer_stack_top] \
                          : NULL)
/* Same as previous macro, but useful when we know that the buffer stack is not
 * NULL or when we need an lvalue. For internal use only.
 */
#define YY_CURRENT_BUFFER_LVALUE yyg->yy_buffer_stack[yyg->yy_buffer_stack_top]

void yyrestart ( FILE *input_file , yyscan_t yyscanner );
void yy_switch_to_buffer ( YY_BUFFER_STATE new_buffer , yyscan_t yyscanner );
YY_BUFFER_STATE yy_create_buffer ( FILE *file, int size , yyscan_t yyscanner );
void yy_delete_buffer ( YY_BUFFER_STATE b , yyscan_t yyscanner );
void yy_flush_buffer ( YY_BUFFER_STATE b , yyscan_t yyscanner );
void yypush_buffer_state ( YY_BUFFER_STATE new_buffer , yyscan_t yyscanner );
void yypop_buffer_state ( yyscan_t yyscanner );

static void yyensure_buffer_stack ( yyscan_t yyscanner );
static void yy_load_buffer_state ( yyscan_t yyscanner );
static void yy_init_buffer ( YY_BUFFER_STATE b, FILE *file , yyscan_t yyscanner );
#define YY_FLUSH_BUFFER yy_flush_buffer( YY_CURRENT_BUFFER , yyscanner)

YY_BUFFER_STATE yy_scan_buffer ( char *base, yy_size_t size , yyscan_t yyscanner );
YY_BUFFER_STATE yy_scan_string ( const char *yy_str , yyscan_t yyscanner );
YY_BUFFER_STATE yy_scan_bytes ( const char *bytes, int len , yyscan_t yyscanner );

void *yyalloc ( yy_size_t , yyscan_t yyscanner );
void *yyrealloc ( void *, yy_size_t , yyscan_t yyscanner );
void yyfree ( void * , yyscan_t yyscanner );

#define yy_new_buffer yy_create_buffer
#define yy_set_interactive(is_interactive) \
	{ \
	if ( ! YY_CURRENT_BUFFER ){ \
        yyensure_buffer_stack (yyscanner); \
		YY_CURRENT_BUFFER_LVALUE =    \
            yy_create_buffer( yyin, YY_BUF_SIZE , yyscanner); \
	} \
	YY_CURRENT_BUFFER_LVALUE->yy_is_interactive = is_interactive; \
	}
#define yy_set_bol(at_bol) \
	{ \
	if ( ! YY_CURRENT_BUFFER ){\
        yyensure_buffer_stack (yyscanner); \
		YY_CURRENT_BUFFER_LVALUE =    \
            yy_create_buffer( yyin, YY_BUF_SIZE , yyscanner); \
	} \
	YY_CURRENT_BUFFER_LVALUE->yy_at_bol = at_bol; \
	}
#define YY_AT_BOL() (YY_CURRENT_BUFFER_LVALUE->yy_at_bol)

/* Begin user sect3 */

#define pyscannerYYwrap(yyscanner) (/*CONSTCOND*/1)
#define YY_SKIP_YYWRAP
typedef flex_uint8_t YY_CHAR;

typedef int yy_state_type;

#define yytext_ptr yytext_r

static yy_state_type yy_get_previous_state ( yyscan_t yyscanner );
static yy_state_type yy_try_NUL_trans ( yy_state_type current_state  , yyscan_t yyscanner);
static int yy_get_next_buffer ( yyscan_t yyscanner );
static void yynoreturn yy_fatal_error ( const char* msg , yyscan_t yyscanner );

/* Done after the current pattern has been matched and before the
 * corresponding action - sets up yytext.
 */
#define YY_DO_BEFORE_ACTION \
	yyg->yytext_ptr = yy_bp; \
	yyleng = (int) (yy_cp - yy_bp); \
	yyg->yy_hold_char = *yy_cp; \
	*yy_cp = '\0'; \
	yyg->yy_c_buf_p = yy_cp;
#define YY_NUM_RULES 265
#define YY_END_OF_BUFFER 266
/* This struct is not used in this scanner,
   but its presence is necessary. */
struct yy_trans_info
	{
	flex_int32_t yy_verify;
	flex_int32_t yy_nxt;
	};
static const flex_int16_t yy_acclist[1717] =
    {   0,
       80,   80,16581,16581,   80,   80,   39,   39,   46,   46,
       54,   54,   80,   80,  266,  264,  265,  262,  265,  263,
      264,  265,   34,  264,  265,   34,  264,  265,   26,  262,
      265,   18,   34,  264,  265,   34,  264,  265,   17,   34,
      263,  264,  265,   30,   34,  264,  265,   31,   34,  264,
      265,   32,   34,  264,  265,   34,  264,  265,   24,   33,
       34,  264,  265,16400,   24,   33,   34,  264,  265,16400,
       24,   33,   34,  264,  265,16400,   24,   33,   34,  264,
      265,16400,   24,   33,   34,  264,  265,16400,   24,   33,
       34,  264,  265,16400,   24,   33,   34,  264,  265,16400,

       24,   33,   34,  264,  265,16400,   24,   33,   34,  264,
      265,16400,   24,   33,   34,  264,  265,16400,   25,   34,
      264,  265,   24,   33,   34,  264,  265,16397,16398,16399,
    16400,   24,   33,   34,  264,  265,16397,16398,16399,16400,
       24,   33,   34,  264,  265,16397,16398,16399,16400,   24,
       33,   34,  264,  265,16397,16398,16399,16400,   24,   33,
       34,  264,  265,16397,16398,16399,16400,   24,   33,   34,
      264,  265,16397,16398,16399,16400,   24,   33,   34,  264,
      265,16397,16398,16399,16400,   24,   33,   34,  264,  265,
    16397,16398,16399,16400,   24,   33,   34,  264,  265,16397,

    16398,16399,16400,   24,   33,   34,  264,  265,16397,16398,
    16399,16400,   68,   69,  264,  265,   65,  262,  265,   64,
       69,  264,  265,   69,  264,  265,   63,   69,  263,  264,
      265,   69,  264,  265,   66,   69,  264,  265,   67,   69,
      264,  265,   67,   69,  264,  265,   67,   69,  264,  265,
       67,   69,  264,  265,   67,   69,  264,  265,   67,   69,
      264,  265,   67,   68,   69,  264,  265,   93,  264,  265,
       80,   93,  264,  265,   96,  262,  265,   85,   93,  264,
      265,   93,  264,  265,   84,   93,  263,  264,  265,   88,
       93,  264,  265,   81,   93,  264,  265,   81,   93,  264,

      265,   79,   93,  264,  265,   92,   93,  264,  265,   92,
       93,  264,  265,   92,   93,  264,  265,   92,   93,  264,
      265,   92,   93,  264,  265,   89,   93,  264,  265,   93,
      264,  265,   92,   93,  264,  265,   90,   93,  264,  265,
      217,  225,  264,  265,  225,  264,  265,  218,  262,  265,
      225,  264,  265,  225,  263,  264,  265,  225,  264,  265,
      225,  264,  265,  216,  225,  264,  265,  234,  236,  264,
      265,  235,  262,  265,  236,  264,  265,  234,  236,  263,
      264,  265,  236,  264,  265,  236,  264,  265,  234,  236,
      264,  265,  226,  236,  264,  265,  264,  265,  114,  264,

      265,  115,  264,  265,  264,  265,  111,  264,  265,  110,
      264,  265,  112,  264,  265,  128,  264,  265,  120,  264,
      265,  128,  264,  265,  128,  263,  264,  265,  125,  264,
      265,  122,  128,  264,  265,  121,  128,  264,  265,  126,
      264,  265,  124,  128,  264,  265,  123,  128,  264,  265,
      103,  107,  264,  265,  107,  264,  265,  106,  262,  265,
    16481,  102,  107,  264,  265,  105,  107,  264,  265,  101,
      107,  263,  264,  265,  103,  107,  264,  265,  103,  107,
      264,  265,  103,  107,  264,  265,  100,  107,  264,  265,
      105,  107,  264,  265,  151,  264,  265,  150,  262,  265,

      149,  151,  264,  265,  148,  151,  263,  264,  265,  142,
      151,  264,  265,  145,  151,  264,  265,  147,  151,  264,
      265,  146,  151,  264,  265,  141,  151,  264,  265,  144,
      151,  264,  265,  140,  151,  264,  265,  143,  151,  264,
      265,  139,  264,  265,  138,  262,  265,  137,  139,  264,
      265,  136,  139,  263,  264,  265,  131,  139,  264,  265,
      134,  139,  264,  265,  135,  139,  264,  265,  130,  139,
      264,  265,  133,  139,  264,  265,  129,  139,  264,  265,
      132,  139,  264,  265,  162,  264,  265,  161,  262,  265,
      160,  162,  264,  265,  159,  162,  263,  264,  265,  154,

      162,  264,  265,  157,  162,  264,  265,  158,  162,  264,
      265,  153,  162,  264,  265,  156,  162,  264,  265,  152,
      162,  264,  265,  155,  162,  264,  265,  180,  264,  265,
      181,  264,  265,  186,  264,  265,  185,  263,  264,  265,
      181,  264,  265,  183,  264,  265,  184,  264,  265,  182,
      264,  265,  264,  265,  191,  262,  265, 8389,  264,  265,
      264,  265, 8389,  264,  265,  263,  264,  265, 8389,  264,
      265, 8389,  264,  265, 8389,  264,  265,  196,  264,  265,
      169,  172,  264,  265,  172,  264,  265,  170,  262,  265,
      168,  172,  264,  265,  172,  264,  265,  167,  172,  263,

      264,  265,  169,  172,  264,  265,  169,  172,  264,  265,
      169,  172,  264,  265,  166,  172,  264,  265,  201,  262,
      265,   93,  200,  264,  265,   93,  264,  265,  213,  264,
      265,  212,  262,  265,  213,  263,  264,  265,  210,  264,
      265,  211,  262,  265,  207,  210,  264,  265,  210,  264,
      265,  206,  210,  263,  264,  265,  202,  210,  264,  265,
      203,  210,  264,  265,  208,  210,  264,  265,  208,  210,
      264,  265,  208,  210,  264,  265,  208,  210,  264,  265,
      241,  242,  264,  265,  242,  264,  265,  240,  242,  263,
      264,  265,  242,  264,  265,  247,  248,  264,  265,  246,

      248,  264,  265,  248,  263,  264,  265,  248,  264,  265,
      251,  253,  264,  265,  251,  252,  262,  265,  251,  253,
      264,  265,  251,  253,  263,  264,  265,  253,  264,  265,
      188,  264,  265,  187,  188,  263,  264,  265,  190,  264,
      265,  189,  190,  264,  265,  190,  263,  264,  265,   40,
      264,  265,   39,   40,  264,  265,   38,  262,  265,   40,
      263,  264,  265,   35,   40,  264,  265,   36,   40,  264,
      265,   36,   40,  264,  265,   51,  264,  265,   46,   51,
      264,  265,   45,  262,  265,   51,  263,  264,  265,   47,
       51,  264,  265,   48,   51,  264,  265,   41,   51,  264,

      265,   49,   51,  264,  265,   44,   51,  264,  265,16426,
    16427,   51,  264,  265,   55,  264,  265,   54,   55,  264,
      265,   53,  262,  265,   55,  263,  264,  265,   52,   55,
      264,  265,   78,  264,  265,   76,  262,  265,   74,   78,
      264,  265,   73,   78,  263,  264,  265,   71,   78,  264,
      265,   72,   78,  264,  265,   70,   78,  264,  265,   75,
       78,  264,  265,   78,  264,  265,  261,  264,  265,  260,
      262,  265,  257,  261,  264,  265,  256,  261,  263,  264,
      265,  258,  261,  264,  265,  259,  261,  264,  265,  261,
      264,  265,  261,  264,  265,  261,  264,  265,  119,  264,

      265,  119,  263,  264,  265,  116,  119,  264,  265,  117,
      119,  264,  265,  118,  119,  264,  265,  177,  264,  265,
      175,  264,  265,  176,  264,  265,   93,  179,  264,  265,
       80,   93,  179,  264,  265,  178,  262,  265,   85,   93,
      179,  264,  265,   84,   93,  179,  263,  264,  265,   88,
       93,  179,  264,  265,   81,   93,  179,  264,  265,   81,
       93,  179,  264,  265,   79,   93,  179,  264,  265,   92,
       93,  179,  264,  265,   92,   93,  179,  264,  265,   92,
       93,  179,  264,  265,   92,   93,  179,  264,  265,   92,
       93,  179,  264,  265,   89,   93,  179,  264,  265,   93,

      179,  264,  265,   92,   93,  179,  264,  265,   90,   93,
      179,  264,  265,16400,   26,   23,   22,   23,   21,   21,
       24,   33,16400,   24,   33,16400,   24,   33,16400,   24,
       33,16400,   24,   33,16400,   24,   33,16400,   24,   33,
    16400,   24,   33,16400,   25,16397,16398,16399,16400,16397,
    16398,16399,16400,16397,16398,16399,16400,16397,16398,16399,
    16400,16397,16398,16399,16400,16397,16398,16399,16400,16397,
    16398,16399,16400,   24,   33,16397,16398,16399,16400, 8207,
       24,   33,16397,16398,16399,16400,   24,   33,16397,16398,
    16399,16400,   24,   33,16397,16398,16399,16400,   24,   33,

    16397,16398,16399,16400,   24,   33,16397,16398,16399,16400,
       24,   33,16397,16398,16399,16400,   24,   33,16397,16398,
    16399,16400,   68,   62,   67,   67,   67,   67,   67,   68,
       80,   95,   81,   81,   81,   92,   92,   85,   84,   92,
       92,   91,  217,  224,  224,  224,  216,  234,  227,  234,
      226,  226,  114,  111,  113,  111,  110,  120,  127,  122,
      126,  123,  103,16481,  105,  103,  100,   99,  104,  105,
      180,  181,  184,  191,  195,  194,  196,  169,  171,  169,
      166,  165,  200,  200,  198,  209,  208,  208,  241,  238,
      238,  237,  247,  244,  244,  243,  251,  251,  251,   39,

       36,   36,   46, 8235, 8234,   44,16426,16427,   50,   54,
       52,   77,  116,  175,16400,   27,   23,   22,   23,   29,
       28,   20,   21,   21, 8208,   24,   33,16400,   24,   33,
    16400,   24,   33,16400,   24,   33,16400,   24,   33,16400,
       24,   33,16400,16397,16398,16399,16400,16397,16398,16399,
    16400,16397,16398,16399,16400,16397,16398,16399,16400,16397,
    16398,16399,16400,16397,16398,16399,16400,16397,16398,16399,
    16400, 8206, 8208, 8206, 8208, 8206, 8208,   24,   33,16397,
    16398,16399,16400,   24,   33,16397,16398,16399,16400,   24,
       33,16397,16398,16399,16400,   24,   33,16397,16398,16399,

    16400,   24,   33,16397,16398,16399,16400,   24,   33,16397,
    16398,16399,16400,   59,   62,   61,   60,   67,   67,   86,
       95,   94,   87,   82,   81,   81,   92,   92,  214,  215,
      127,   98, 8289, 8289,  108,  109,  104,  105,  184,  192,
      195,  194,  193,  163,  173,  171,  174,  199,  200,  198,
      204,  209,  205,  239,  245,  249,  250,   36,   36, 8234,
       52,  254,  255,   27,   28,   21,   21,   24,   33,16400,
       24,   33,16400,    3,   24,   33,16400,   24,   33,16400,
       24,   33,16400,16397,16398,16399,16400,16397,16398,16399,
    16400,16397,16398,16399,16400,16397,16398,16399,16400,16397,

    16398,16399,16400,16397,16398,16399,16400,   24,   33,16397,
    16398,16399,16400,   24,   33,16397,16398,16399,16400,    1,
        3,   24,   33,16397,16398,16399,16400,   24,   33,16397,
    16398,16399,16400,   24,   33,16397,16398,16399,16400,   59,
       60,   86,   87,   82,   81,   92,   83,   92,  223,  223,
      232,  232,  108,  109,  184,  192,  193,  164,  173,  174,
      199,  200,  204,  205,   36,   36,   52,  254,  255,   21,
       21,   24,   33,16400,   24,   33,16400,   10,   24,   33,
    16400,    8,16397,16398,16399,16400,16397,16398,16399,16400,
        1,16397,16398,16399,16400,16397,16398,16399,16400,16397,

    16398,16399,16400,   24,   33,16397,16398,16399,16400,   24,
       33,16397,16398,16399,16400,    9,   10,   24,   33,16397,
    16398,16399,16400,    7,    8,16440,16441,16442,   82,   36,
       21,    6,   24,   33,16400,16397,16398,16399,16400,16397,
    16398,16399,16400,    9,16397,16398,16399,16400,    7,    5,
        6,   24,   33,16397,16398,16399,16400, 8248,16440,16441,
    16442, 8250, 8249,   36,   37,   21,   12,    5,16397,16398,
    16399,16400,   11,   12,  220,  222,  229,  231,   37,   21,
       11,  220,  222,  221,  222,  229,  231,  230,  231,   21,
      219,  220,  220,  220,  220,  228,  229,  229,  229,  229,

      233,   21,    4, 8205,    2,    4,  220,  220,  229,  229,
      233,   21,    2,   21,   19,   21
    } ;

static const flex_int16_t yy_accept[819] =
    {   0,
        1,    1,    1,    1,    1,    1,    1,    2,    3,    3,
        3,    3,    3,    3,    3,    3,    3,    3,    3,    3,
        3,    3,    3,    3,    3,    3,    3,    3,    3,    4,
        5,    5,    5,    6,    7,    7,    7,    7,    7,    7,
        7,    7,    7,    7,    7,    7,    7,    7,    7,    8,
        9,   10,   11,   12,   13,   13,   13,   13,   13,   13,
       13,   13,   13,   14,   15,   16,   18,   20,   23,   26,
       29,   32,   36,   39,   44,   48,   52,   56,   59,   65,
       71,   77,   83,   89,   95,  101,  107,  113,  119,  123,
      132,  141,  150,  159,  168,  177,  186,  195,  204,  213,

      217,  220,  224,  227,  232,  235,  239,  243,  247,  251,
      255,  259,  263,  268,  271,  275,  278,  282,  285,  290,
      294,  298,  302,  306,  310,  314,  318,  322,  326,  330,
      333,  337,  341,  345,  348,  351,  354,  358,  361,  364,
      368,  372,  375,  378,  383,  386,  389,  393,  397,  399,
      402,  405,  407,  410,  413,  416,  419,  422,  425,  429,
      432,  436,  440,  443,  447,  451,  455,  458,  462,  466,
      470,  475,  479,  483,  487,  491,  495,  498,  501,  505,
      510,  514,  518,  522,  526,  530,  534,  538,  542,  545,
      548,  552,  557,  561,  565,  569,  573,  577,  581,  585,

      588,  591,  595,  600,  604,  608,  612,  616,  620,  624,
      628,  631,  634,  637,  641,  644,  647,  650,  653,  655,
      658,  661,  663,  666,  669,  672,  675,  678,  681,  685,
      688,  691,  695,  698,  703,  707,  711,  715,  719,  722,
      726,  729,  732,  735,  739,  742,  745,  749,  752,  757,
      761,  765,  769,  773,  777,  781,  785,  788,  793,  796,
      800,  804,  808,  811,  815,  819,  823,  828,  831,  834,
      839,  842,  846,  850,  853,  857,  860,  864,  868,  872,
      876,  879,  883,  886,  890,  894,  898,  902,  906,  912,
      915,  918,  922,  925,  929,  933,  936,  939,  943,  948,

      952,  956,  960,  964,  967,  970,  973,  977,  982,  986,
      990,  993,  996,  999, 1002, 1006, 1010, 1014, 1018, 1021,
     1024, 1027, 1031, 1036, 1039, 1044, 1050, 1055, 1060, 1065,
     1070, 1075, 1080, 1085, 1090, 1095, 1100, 1104, 1109, 1114,
     1114, 1114, 1115, 1116, 1116, 1117, 1119, 1119, 1119, 1120,
     1121, 1121, 1121, 1121, 1124, 1124, 1124, 1124, 1127, 1130,
     1133, 1136, 1139, 1142, 1145, 1146, 1150, 1154, 1158, 1162,
     1166, 1170, 1174, 1174, 1180, 1181, 1181, 1187, 1193, 1199,
     1205, 1211, 1217, 1223, 1224, 1224, 1225, 1225, 1225, 1226,
     1226, 1226, 1227, 1228, 1229, 1231, 1232, 1232, 1233, 1233,

     1233, 1233, 1234, 1235, 1236, 1236, 1237, 1238, 1239, 1240,
     1241, 1242, 1243, 1244, 1244, 1244, 1244, 1244, 1245, 1246,
     1247, 1248, 1249, 1249, 1250, 1250, 1250, 1250, 1251, 1252,
     1253, 1253, 1254, 1255, 1256, 1257, 1258, 1259, 1260, 1261,
     1262, 1263, 1264, 1265, 1265, 1265, 1265, 1266, 1266, 1266,
     1266, 1267, 1268, 1269, 1271, 1272, 1273, 1273, 1274, 1274,
     1275, 1275, 1276, 1277, 1277, 1277, 1277, 1277, 1278, 1279,
     1279, 1279, 1279, 1280, 1280, 1280, 1280, 1281, 1282, 1283,
     1284, 1285, 1285, 1286, 1286, 1287, 1287, 1288, 1288, 1288,
     1289, 1290, 1290, 1291, 1292, 1293, 1294, 1294, 1295, 1296,

     1297, 1298, 1299, 1300, 1301, 1301, 1301, 1302, 1303, 1304,
     1304, 1305, 1306, 1309, 1309, 1310, 1311, 1311, 1311, 1312,
     1313, 1313, 1313, 1313, 1313, 1313, 1314, 1315, 1316, 1317,
     1318, 1320, 1321, 1322, 1323, 1323, 1324, 1325, 1325, 1325,
     1325, 1326, 1329, 1332, 1335, 1338, 1341, 1344, 1348, 1352,
     1356, 1360, 1364, 1368, 1372, 1374, 1376, 1378, 1384, 1390,
     1396, 1402, 1408, 1414, 1415, 1416, 1417, 1418, 1419, 1420,
     1421, 1422, 1423, 1424, 1425, 1426, 1427, 1428, 1429, 1430,
     1431, 1431, 1431, 1431, 1431, 1431, 1431, 1431, 1432, 1433,
     1434, 1435, 1435, 1436, 1437, 1439, 1440, 1441, 1442, 1443,

     1444, 1444, 1445, 1445, 1446, 1447, 1448, 1450, 1451, 1452,
     1453, 1454, 1455, 1455, 1456, 1456, 1457, 1458, 1458, 1459,
     1460, 1461, 1461, 1462, 1463, 1464, 1465, 1466, 1467, 1468,
     1468, 1471, 1474, 1475, 1478, 1481, 1484, 1488, 1492, 1496,
     1500, 1504, 1508, 1508, 1508, 1508, 1514, 1520, 1522, 1528,
     1534, 1540, 1541, 1542, 1542, 1543, 1544, 1544, 1545, 1546,
     1547, 1549, 1550, 1551, 1551, 1551, 1551, 1551, 1552, 1553,
     1553, 1553, 1553, 1553, 1553, 1554, 1555, 1556, 1557, 1558,
     1559, 1560, 1561, 1563, 1564, 1565, 1566, 1567, 1568, 1569,
     1570, 1571, 1572, 1575, 1578, 1579, 1582, 1583, 1587, 1591,

     1592, 1596, 1600, 1604, 1604, 1610, 1616, 1618, 1624, 1626,
     1629, 1629, 1630, 1630, 1630, 1630, 1630, 1630, 1631, 1632,
     1632, 1633, 1636, 1640, 1644, 1645, 1649, 1650, 1650, 1650,
     1652, 1658, 1658, 1659, 1662, 1663, 1664, 1664, 1664, 1664,
     1664, 1664, 1666, 1667, 1667, 1668, 1668, 1669, 1673, 1673,
     1673, 1675, 1675, 1675, 1676, 1676, 1676, 1676, 1677, 1677,
     1677, 1678, 1678, 1678, 1678, 1679, 1679, 1680, 1681, 1681,
     1681, 1682, 1682, 1682, 1682, 1682, 1683, 1684, 1685, 1686,
     1686, 1686, 1687, 1688, 1689, 1690, 1690, 1691, 1691, 1691,
     1691, 1691, 1692, 1693, 1694, 1695, 1696, 1697, 1698, 1699,

     1700, 1701, 1702, 1703, 1704, 1704, 1705, 1707, 1708, 1709,
     1710, 1711, 1712, 1713, 1714, 1715, 1717, 1717
    } ;

static const YY_CHAR yy_ec[256] =
    {   0,
        1,    1,    1,    1,    1,    1,    1,    1,    2,    3,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    4,    5,    6,    7,    1,    8,    9,   10,   11,
       12,   13,   14,   15,   16,   17,   18,   19,   20,   20,
       20,   20,   20,   20,   20,   21,   21,   22,   23,   24,
       25,   26,    1,   27,   28,   28,   28,   29,   30,   31,
       32,   33,   32,   34,   33,   35,   33,   33,   33,   33,
       33,   36,   33,   37,   38,   33,   33,   39,   33,   33,
       40,   41,   42,    1,   43,   23,   44,   45,   46,   47,

       48,   49,   33,   50,   51,   34,   33,   52,   53,   54,
       55,   56,   33,   57,   58,   59,   60,   33,   33,   39,
       61,   33,   62,   23,   63,    9,    1,   64,   64,   64,
       64,   64,   64,   64,   64,   64,   64,   64,   64,   64,
       64,   64,   64,   64,   64,   64,   64,   64,   64,   64,
       64,   64,   64,   64,   64,   64,   64,   64,   64,   64,
       64,   64,   64,   64,   64,   64,   64,   64,   64,   64,
       64,   64,   64,   64,   64,   64,   64,   64,   64,   64,
       64,   64,   64,   64,   64,   64,   64,   64,   64,   64,
       64,   64,   64,   64,   64,   64,   64,   64,   64,   64,

       64,   64,   64,   64,   64,   64,   64,   64,   64,   64,
       64,   64,   64,   64,   64,   64,   64,   64,   64,   64,
       64,   64,   64,   64,   64,   64,   64,   64,   64,   64,
       64,   64,   64,   64,   64,   64,   64,   64,   64,   64,
       64,   64,   64,   64,   64,   64,   64,   64,   64,   64,
       64,   64,   64,   64,   64
    } ;

static const YY_CHAR yy_meta[65] =
    {   0,
        1,    2,    3,    2,    1,    4,    5,    6,    7,    8,
        9,   10,    1,    7,   10,    7,   11,    7,   12,   12,
       12,   13,    1,    1,   14,    1,   15,   16,   16,   16,
       16,   17,   17,   17,   18,   17,   17,   17,   17,    1,
       19,    1,   17,   16,   16,   16,   16,   16,   16,   17,
       17,   18,   17,   17,   17,   17,   17,   17,   17,   17,
       17,    1,    1,   20
    } ;

static const flex_int16_t yy_base[973] =
    {   0,
        0,    1,   11,   75,  139,    0,  203,    0,  266,  271,
      275,  284,  325,    0,  389,    0,  452,  511,  569,  630,
      691,  752,  813,  874,  937,    0, 1001,    0, 1065, 1125,
     1184, 1243,  277,  293,  280,  294, 1303,    0,  454,  460,
      462,  470,  475,  481,  298,  300,  303,  311, 1367,    0,
     1431,    0, 1495,    0, 1557, 1618,  494,  528, 1681,    0,
     1745,    0, 1809,    0, 2684, 5950, 5950, 5950, 5950,  316,
     2679, 2674,    0, 2668, 5950, 5950, 5950, 2616,  521,  585,
      602,  563,  546,  580,  488,  586,  620,    0,  660,  646,
      674, 1872,  659,  669,  702,  704,  712,  724,    0,    0,

     5950, 2654, 2649, 2645, 5950, 5950,    0,  461,  731, 2601,
     2604,  736, 1929, 5950,    4, 5950, 2641, 2636, 2630, 5950,
      756,  783, 5950,    0, 2592,  469, 2578,  773, 5950, 2628,
      774, 5950,    0, 5950, 5950, 2620, 2612,  469,  502,  320,
        0,  553, 5950,    0,  574,  578,  563, 2610,  721, 5950,
     5950, 2585,  533,    0, 5950, 5950,  540,    0, 5950, 5950,
     2589, 5950,  546, 5950,    0,    0, 5950,  642, 2595,    0,
     2587,  476,  776,  791,  559, 2586, 5950, 5950, 5950, 5950,
     5950, 5950, 5950, 5950, 5950, 5950, 5950, 5950, 5950, 5950,
     5950, 5950, 5950, 5950, 5950, 5950, 5950, 5950, 5950, 5950,

     5950, 5950, 5950, 5950, 5950, 5950, 5950, 5950, 5950, 5950,
        0,  572, 5950, 5950, 5950, 5950, 2572, 5950,  649, 5950,
     5950, 2578, 2573, 2379,  483,  801,  816,  725,    0, 5950,
     2381, 2381,    0, 2376,  677,  826,  833,  838, 5950, 2378,
      686, 5950, 5950, 5950, 5950, 5950, 2378,    0, 2373, 5950,
     5950,    0,  714,  851,  858,    0, 2376, 5950,  842,    0,
     5950, 2370,  847, 5950, 5950, 2371, 2364,    0, 5950, 5950,
     5950, 5950, 5950, 5950,  696, 5950, 5950, 5950,  861, 1077,
     5950,  743, 5950, 5950, 5950, 5950, 5950, 5950,  889,  868,
     5950,  747, 5950, 5950,  893, 5950, 5950, 5950, 5950, 5950,

     5950, 5950, 5950, 2362, 5950, 5950, 2355, 2346, 5950, 5950,
      729, 1064, 1076, 5950, 5950,    0, 5950, 5950, 5950,    0,
     5950, 5950,  764, 5950, 2346, 2337, 5950, 1085, 1119, 5950,
        0, 2299,  746, 2283, 1077, 5950, 2327, 1087, 5950,  852,
     2303,  894, 2280, 2268,    0,    0,    0, 2263,  278,  607,
      909, 1086,  785, 1143,    0, 2264, 2240, 1168, 1144, 1177,
     1194, 1199, 1211, 1227, 1226, 1236, 1568, 1562, 1590, 1889,
     1873, 1898, 1272, 1276, 5950,  877, 1629, 1903, 1630, 1961,
     1973, 1992, 1997,    0, 2239,    0,    0, 2228,    0, 2214,
     2201,  828, 2152, 2154, 2053,  794, 2196,    0,    0, 2191,

      906, 2004, 2066, 5950,    0,    0, 2137, 2178, 2170, 2054,
      882, 5950,    0, 2107, 2094,    0,  425, 5950,    0,  450,
      898,    0,  916, 5950,    0,  605,  611,  928, 2085, 2074,
     1074, 5950,  901, 5950,  927,    0, 1087,    0, 2050, 1105,
        0,    0, 1158, 2045, 2118, 2005,    0, 1980, 1983, 1969,
     1067, 1112, 5950,    0,    0, 1106,    0, 1950, 1139, 5950,
     1951,    0, 1947, 1929, 1930, 1906, 1074, 1147,    0, 1905,
     1162, 1903,    0, 1896, 1890, 1879, 1089, 1173, 5950,    0,
     1855, 1193,    0, 1672,    0, 1647,    0, 1650, 1643, 1194,
        0, 1644, 5950, 1214, 5950,    0, 1626, 5950, 1253, 5950,

     1619, 1608, 5950, 1206, 1569, 1223, 1574, 1605, 1230, 1660,
     5950, 1233, 1661, 1260, 5950, 1264, 1599, 1267, 1606, 5950,
     1594, 1586, 1582, 1575, 1276,    0,    0, 2001, 1557,    0,
        0, 5950, 1297, 5950,    0,  566, 1248, 1602, 1291, 2025,
     5950, 2179, 2183, 2087, 2184, 2188, 2203, 2210, 2227, 2232,
     2244, 2256, 2260, 2275, 5950, 1879, 1242, 2287, 2292, 2311,
     2317, 2323, 2342, 1291,    0, 5950, 1285, 1272,  509, 1255,
        0, 5950, 1207, 2011, 5950,  639, 1141, 1136, 5950, 5950,
     2390, 1106, 1067, 2454, 1066,  866,  861,    0, 5950, 5950,
     2518,    0,  902,  894,    0,  865,  869,    0,  860,  824,

        0, 1609, 1644,  816,    0,  815,    0,    0,  808,    0,
      801, 5950, 1664, 5950, 1667, 5950, 5950, 1645, 1915, 1980,
     1675, 1888, 1964,  773,  766, 5950, 5950,  596,  788, 2215,
     2579, 2583, 2347, 2351, 2588, 2356, 2584, 2608, 2612, 2617,
     2642, 2646, 2004,  713,  702, 2647, 2661, 2673, 2677, 2699,
     2703, 5950, 5950,    0, 5950, 5950, 2089, 5950, 5950,  696,
        0, 5950,    0, 2749,    0,  684,  661, 5950,    0, 2803,
        0,  633,  628,  619, 5950, 5950,  630, 5950, 5950, 5950,
     5950, 5950,    0, 5950, 5950, 1970, 2263, 1998, 5950, 5950,
      849, 1156, 2603, 2708, 2725, 2730, 2734, 2863, 2864, 2739,

     2868, 2869, 2893,  579, 2898, 2899, 2915, 2927, 2931, 2932,
     1941, 2077,  586,  575,  554,  545,  530, 2320, 2032, 2946,
     2958, 2962, 2963, 2977, 2992, 2993, 3007,  511, 3008, 3022,
     3023, 3038, 5950, 3039, 5950, 5950, 3063, 2086, 3066, 2770,
     2824, 2016, 1564,  493, 3074, 3075, 3090, 3091,  470,  471,
     3106, 3121,    0, 3128,    0, 3131, 2980, 3054, 3138,    0,
     3159,    0, 3169, 3162, 3186,  451, 2301, 1910,  458,  444,
     3197,  424,  412,  313, 1881, 2029, 1976, 5950, 3194,  299,
     2175, 2730, 2032, 5950, 3214,  288, 1578, 2040,  230,  210,
     2080, 5950, 2802, 3203, 3206, 3219, 5950, 3221, 3228, 3231,

     3234, 2219, 2284, 2222, 2251, 5950, 2259, 3237, 3246, 3248,
     3259, 2282, 1869, 2304, 2281,  911, 5950, 3300, 3320, 3340,
     3360, 3380, 3400, 3420, 3440, 3460, 3480, 3500, 3520, 3540,
     3560, 3580, 3600, 3620, 3639, 3659, 3664, 3683, 3702, 3721,
     3741, 3761, 3770, 3790, 3799, 3819, 3839, 3859, 3868, 3888,
     3897, 3917, 3936, 3956, 3976, 3985, 3995, 4015, 4035, 4051,
     4071, 4091, 4110, 4130, 4139, 4159, 4179, 4199, 4219, 4239,
     4258, 4277, 4296, 4305, 4314, 4333, 4352, 4372, 4392, 4412,
     4424, 4443, 4462, 4482, 4501, 4520, 4539, 4559, 4579, 4599,
     4619, 4628, 4648, 4668, 1549, 4688, 4703, 4721, 4736, 4743,

     4763, 4772, 4792, 4811, 4831, 4851, 4860, 4865, 4875, 4895,
     4915, 4934, 4954, 4974, 4993, 5002, 5022, 5031, 5051, 5071,
     5090, 5109, 5128, 5147, 5166, 5175, 5184, 5203, 5223, 5243,
     5248, 5260, 5279, 5298, 5317, 5336, 5355, 5375, 5385, 5405,
     2357, 5414, 5434, 5454, 5464, 5484, 5504, 5523, 5543, 5563,
     5572, 5592, 5611, 5630, 5649, 5668, 5687, 5699, 5704, 5723,
     5742, 5761, 5781, 5797, 5817, 5833, 5849, 5865, 5881, 5897,
     5913, 5929
    } ;

static const flex_int16_t yy_def[973] =
    {   0,
      818,  818,  817,  817,  817,    5,  817,    7,  819,  819,
      820,  820,  817,   13,  817,   15,  821,  821,  822,  822,
      823,  823,  824,  824,  817,   25,  817,   27,  825,  825,
      826,  826,    7,    7,  827,  827,  817,   37,  828,  828,
      829,  829,  830,  830,  831,  831,  832,  832,  817,   49,
      817,   51,  817,   53,  833,  833,  834,  834,  817,   59,
      817,   61,  817,   63,  817,  817,  817,  817,  817,  835,
      817,  817,  836,  817,  817,  817,  817,  837,  838,  838,
      838,   81,   81,   81,   81,   81,   81,   81,  839,  840,
      840,  840,   92,   92,   92,   92,   92,   92,   92,  841,

      817,  817,  842,  817,  817,  817,  843,  843,  843,  843,
      843,  843,  843,  817,  817,  817,  817,  844,  817,  817,
      817,  817,  817,  845,  845,  845,  845,  845,  817,  817,
      845,  817,  846,  817,  817,  817,  817,  817,  847,  817,
      848,  817,  817,  848,  817,  817,  848,  817,  817,  817,
      817,  817,  817,  849,  817,  817,  817,  850,  817,  817,
      817,  817,  817,  817,  851,  852,  817,  853,  817,  854,
      817,  852,  852,  852,  817,  855,  817,  817,  817,  817,
      817,  817,  817,  817,  817,  817,  817,  817,  817,  817,
      817,  817,  817,  817,  817,  817,  817,  817,  817,  817,

      817,  817,  817,  817,  817,  817,  817,  817,  817,  817,
      856,  817,  817,  817,  817,  817,  857,  817,  817,  817,
      817,  817,  858,  817,  817,  817,  817,  817,  859,  817,
      860,  817,  861,  817,  859,  859,  859,  817,  817,  862,
      863,  817,  817,  817,  817,  817,  817,  864,  817,  817,
      817,  865,  865,  865,  865,  866,  817,  817,  867,  868,
      817,  817,  869,  817,  817,  817,  817,  870,  817,  817,
      817,  817,  817,  817,  817,  817,  817,  817,  871,  871,
      817,  817,  817,  817,  817,  817,  817,  817,  872,  817,
      817,  817,  817,  817,  873,  817,  817,  817,  817,  817,

      817,  817,  817,  817,  817,  817,  817,  817,  817,  817,
      817,  817,  817,  817,  817,  874,  817,  817,  817,  875,
      817,  817,  817,  817,  817,  817,  817,  817,  817,  817,
      845,  845,  845,  845,  845,  817,  817,  845,  817,  876,
      817,  877,  817,  817,  878,  879,  880,  817,  881,  881,
      817,  817,  882,  883,  884,  817,  817,  883,  883,  883,
      883,  883,  883,  883,  885,  886,  886,  886,  886,  886,
      886,  886,  817,  887,  817,  888,  887,  887,  887,  887,
      887,  887,  887,  889,  817,  890,  891,  817,  892,  817,
      817,  892,  892,  892,  892,  817,  817,  893,  894,  817,

      817,  817,  817,  817,  895,  845,  845,  817,  817,  845,
      845,  817,  896,  817,  817,  897,  817,  817,  897,  817,
      817,  898,  817,  817,  899,  817,  817,  898,  817,  817,
      817,  817,  817,  817,  817,  900,  817,  901,  817,  817,
      902,  903,  904,  817,  817,  817,  905,  817,  817,  817,
      903,  817,  817,  906,  907,  817,  908,  909,  817,  817,
      817,  910,  817,  817,  817,  817,  817,  817,  911,  817,
      912,  817,  913,  817,  817,  817,  911,  817,  817,  914,
      914,  915,  916,  817,  917,  817,  918,  817,  817,  918,
      919,  817,  817,  817,  817,  920,  817,  817,  817,  817,

      817,  817,  817,  817,  817,  921,  922,  922,  817,  817,
      817,  817,  923,  817,  817,  817,  817,  924,  925,  817,
      817,  817,  817,  817,  817,  926,  927,  928,  817,  929,
      930,  817,  817,  817,  931,  932,  932,  817,  933,  934,
      817,  935,  935,  935,  935,  935,  935,  936,  936,  936,
      936,  936,  936,  936,  817,  817,  817,  937,  937,  937,
      937,  937,  937,  817,  938,  817,  817,  939,  939,  817,
      940,  817,  817,  817,  817,  941,  942,  942,  817,  817,
      817,  817,  817,  817,  817,  817,  817,  943,  817,  817,
      817,  591,  817,  817,  944,  945,  817,  946,  817,  817,

      947,  817,  948,  817,  949,  817,  950,  951,  817,  952,
      817,  817,  817,  817,  817,  817,  817,  953,  954,  955,
      817,  956,  957,  817,  817,  817,  817,  958,  932,  934,
      935,  935,  817,  935,  935,  935,  936,  936,  936,  936,
      936,  936,  817,  817,  817,  937,  937,  817,  937,  937,
      937,  817,  817,  959,  817,  817,  817,  817,  817,  942,
      942,  817,  581,  581,  581,  817,  817,  817,  584,  584,
      584,  817,  817,  817,  817,  817,  945,  817,  817,  817,
      817,  817,  950,  817,  817,  954,  955,  957,  817,  817,
      958,  932,  935,  935,  817,  935,  817,  936,  936,  817,

      936,  936,  936,  817,  937,  937,  817,  937,  817,  960,
      817,  817,  817,  817,  817,  817,  817,  955,  932,  817,
      817,  935,  936,  936,  817,  936,  817,  817,  817,  817,
      937,  817,  817,  960,  817,  817,  961,  817,  962,  817,
      817,  955,  932,  817,  817,  817,  817,  936,  817,  817,
      817,  961,  963,  961,  964,  961,  817,  817,  962,  965,
      962,  966,  962,  817,  817,  817,  817,  932,  817,  817,
      817,  817,  817,  963,  967,  968,  817,  817,  817,  965,
      969,  970,  817,  817,  817,  817,  932,  817,  817,  817,
      817,  817,  971,  971,  968,  968,  817,  972,  972,  970,

      970,  817,  932,  817,  817,  817,  817,  971,  971,  972,
      972,  817,  932,  817,  932,  932,    0,  817,  817,  817,
      817,  817,  817,  817,  817,  817,  817,  817,  817,  817,
      817,  817,  817,  817,  817,  817,  817,  817,  817,  817,
      817,  817,  817,  817,  817,  817,  817,  817,  817,  817,
      817,  817,  817,  817,  817,  817,  817,  817,  817,  817,
      817,  817,  817,  817,  817,  817,  817,  817,  817,  817,
      817,  817,  817,  817,  817,  817,  817,  817,  817,  817,
      817,  817,  817,  817,  817,  817,  817,  817,  817,  817,
      817,  817,  817,  817,  817,  817,  817,  817,  817,  817,

      817,  817,  817,  817,  817,  817,  817,  817,  817,  817,
      817,  817,  817,  817,  817,  817,  817,  817,  817,  817,
      817,  817,  817,  817,  817,  817,  817,  817,  817,  817,
      817,  817,  817,  817,  817,  817,  817,  817,  817,  817,
      817,  817,  817,  817,  817,  817,  817,  817,  817,  817,
      817,  817,  817,  817,  817,  817,  817,  817,  817,  817,
      817,  817,  817,  817,  817,  817,  817,  817,  817,  817,
      817,  817
    } ;

static const flex_int16_t yy_nxt[6015] =
    {   0,
      817,  817,   67,   67,  346,  396,  347,  396,  817,   68,
       68,   69,   70,   71,   70,   69,   72,   73,   69,   69,
       74,   75,   76,   69,   69,   69,   69,   69,   69,   69,
       69,   69,   69,   69,   69,   77,   69,   78,   79,   79,
       79,   79,   79,   79,   79,   79,   80,   79,   81,   79,
       69,   69,   69,   79,   82,   79,   83,   84,   79,   85,
       79,   86,   79,   79,   79,   79,   79,   80,   79,   87,
       88,   79,   69,   69,   79,   69,   89,   71,   89,   69,
       72,   73,   69,   69,   74,   75,   76,   69,   69,   69,
       69,   69,   69,   69,   69,   69,   69,   69,   69,   77,

       69,   78,   90,   90,   90,   90,   90,   90,   90,   90,
       91,   90,   92,   90,   69,   69,   69,   90,   93,   90,
       94,   95,   90,   96,   90,   97,   90,   90,   90,   90,
       90,   91,   90,   98,   99,   90,   69,   69,   90,  100,
      100,  101,  100,  100,  102,  103,  100,  100,  104,  100,
      100,  100,  100,  100,  100,  105,  100,  100,  100,  100,
      100,  100,  100,  106,  100,  100,  107,  107,  107,  107,
      107,  107,  107,  107,  108,  107,  109,  107,  100,  100,
      100,  107,  107,  107,  110,  107,  107,  107,  107,  107,
      107,  107,  107,  107,  107,  108,  111,  107,  112,  107,

      100,  100,  113,  114,  115,  116,  115,  114,  117,  118,
      114,  114,  119,  120,  114,  114,  114,  114,  114,  114,
      114,  121,  122,  122,  114,  114,  114,  123,  114,  114,
      124,  124,  124,  125,  124,  124,  124,  124,  126,  127,
      128,  124,  129,  130,  114,  124,  124,  124,  124,  124,
      124,  124,  124,  124,  124,  124,  124,  124,  124,  126,
      124,  124,  131,  124,  132,  114,  124,  134,  135,  134,
      806,  136,  140,  135,  140,  137,  136,  142,  805,  239,
      137,  143,  243,  240,  144,  147,  142,  147,  534,  244,
      148,  802,  138,  144,  535,  239,  243,  138,  241,  240,

       67,  145,   67,  244,  797,   67,  139,  270,  272,  270,
      145,  139,  273,   67,  241,  146,  272,  340,  792,  340,
      273,  421,  341,  421,  146,   66,  149,   67,  149,   66,
       66,   66,   66,   66,   68,  150,  151,   66,   66,   66,
      152,   66,   66,   66,   66,   66,  153,   66,   66,   66,
       66,   66,  154,  154,  154,  154,  154,  154,  154,  154,
      154,  154,  154,  154,  155,   66,   66,  154,  154,  154,
      154,  154,  154,  154,  154,  154,  154,  154,  154,  154,
      154,  154,  154,  154,  154,  154,   66,   66,  154,  156,
      157,   67,  157,  156,  156,  158,  156,  156,  159,   66,

      160,  161,  156,  162,  156,  156,  156,  156,  156,  156,
      163,  156,  156,  164,  156,  156,  165,  165,  165,  165,
      165,  165,  165,  165,  165,  165,  165,  165,  156,  156,
      156,  165,  165,  165,  165,  165,  165,  165,  165,  165,
      165,  165,  165,  165,  165,  165,  165,  165,  165,  165,
      156,  156,  165,  167,  168,  167,   67,  169,  170,  257,
      791,  171,   67,  258,   67,  257,  390,  261,  167,  258,
      391,  262,   67,  582,  408,  261,  583,  265,  409,  262,
      266,  449,  790,  265,  267,  450,  266,  172,  465,  173,
      267,  789,  466,  817,  259,  416,  306,  817,  582,  307,

      259,  583,  263,  308,  309,  310,  788,  786,  172,  416,
      263,  174,  175,  168,  175,  268,  169,  176,  773,  417,
      171,  268,  351,  354,  351,  817,  772,  167,  419,  311,
      306,  312,  352,  307,  435,  353,  435,  308,  309,  310,
      769,  437,  419,  437,  362,  355,  172,  440,  173,  440,
      311,  817,  420,  313,  423,  817,  423,  568,  749,  424,
      452,  453,  452,  311,  428,  312,  428,  172,  817,  429,
      174,  178,  817,  456,  179,  456,  534,  741,  180,  181,
      182,  354,  535,  183,  311,  817,  351,  313,  351,  817,
      356,  817,  740,  184,  357,  817,  352,  360,  354,  353,

      425,  739,  354,  351,  425,  351,  534,  356,  185,  355,
      186,  357,  535,  352,  425,  354,  353,  534,  425,  354,
      359,  354,  738,  535,  426,  817,  355,  361,  427,  817,
      187,  188,  178,  737,  728,  179,  354,  358,  363,  180,
      181,  182,  354,  443,  183,  443,  457,  373,  444,  373,
      459,  460,  459,  585,  184,  354,  586,  352,  358,  585,
      353,  365,  587,  365,  817,  537,  341,  375,  817,  185,
      376,  186,  717,  659,  817,  373,  354,  373,  817,  356,
      364,  716,  475,  357,  715,  352,  476,  482,  353,  482,
      659,  187,  188,  190,  374,  375,  191,  504,  376,  504,

      192,  193,  194,  367,  374,  368,  369,  817,  370,  817,
      371,  817,  195,  817,  714,  374,  378,  817,  372,  488,
      379,  817,  431,  489,  431,  374,  468,  460,  468,  817,
      196,  432,  197,  817,  523,  713,  390,  374,  524,  374,
      391,  390,  433,  661,  509,  391,  509,  374,  516,  380,
      516,  408,  198,  199,  190,  409,  704,  191,  374,  374,
      381,  192,  193,  194,  382,  396,  392,  396,  374,  645,
      690,  392,  401,  195,  402,  402,  403,  689,  408,  408,
      374,  449,  409,  409,  383,  450,  539,  392,  539,  404,
      404,  196,  392,  197,  405,  396,  449,  396,  534,  401,

      450,  403,  403,  403,  535,  685,  465,  404,  411,  411,
      466,  451,  684,  198,  199,  201,  404,  404,  202,  682,
      681,  465,  203,  204,  205,  466,  451,  206,  679,  411,
      411,  475,  451,  390,  404,  476,  467,  391,  475,  478,
      479,  478,  476,  494,  495,  494,  692,  451,  499,  500,
      499,  467,  207,  340,  208,  340,  488,  467,  341,  534,
      489,  477,  505,  488,  505,  535,  599,  489,  477,  514,
      515,  514,  467,  678,  209,  210,  201,  506,  556,  202,
      556,  457,  477,  203,  204,  205,  490,  408,  206,  477,
      510,  409,  510,  490,  517,  351,  517,  351,  676,  421,

      511,  421,  435,  512,  435,  352,  675,  490,  353,  518,
      351,  674,  351,  207,  490,  208,  673,  423,  355,  423,
      352,  534,  424,  353,  574,  574,  574,  535,  435,  428,
      435,  428,  557,  355,  429,  209,  210,   66,   66,   67,
       66,   66,   66,   66,   66,   66,   68,   66,   66,   66,
       66,   66,   66,   66,   66,   66,   66,   66,   66,   66,
       66,   66,   66,   66,  211,  211,  211,  211,  211,  211,
      211,  211,  211,  211,  211,  211,   66,   66,   66,  211,
      211,  211,  211,  211,  211,  211,  211,  211,  211,  211,
      211,  211,  211,  211,  211,  211,  211,  211,   66,   66,

      211,   66,  212,   67,  212,   66,  213,   66,   66,   66,
      214,  215,  215,   66,   66,  215,   66,   66,   66,   66,
       66,   66,  216,   66,   66,   66,   66,   66,  217,  217,
      217,  217,  217,  217,  217,  217,  217,  217,  217,  217,
      218,   66,   66,  217,  217,  217,  217,  217,  217,  217,
      217,  217,  217,  217,  217,  217,  217,  217,  217,  217,
      217,  217,   66,   66,  217,   66,  219,  220,  219,  523,
      222,  223,  449,  524,  224,  431,  450,  431,  505,  465,
      505,  523,  408,  466,  432,  524,  409,  538,  437,  538,
      437,   66,  408,  506,  475,  433,  409,  352,  476,  525,

      225,  401,  226,  402,  402,  403,  440,  456,  440,  456,
      355,  525,  411,  452,  453,  452,  672,  667,  404,  404,
      525,  225,  411,  405,  227,   66,  228,  220,  228,  508,
      222,  223,  525,  411,  224,  401,  404,  403,  403,  403,
      459,  460,  459,  411,  351,  351,  351,  351,  468,  460,
      468,   66,  404,  404,  352,  352,  666,  353,  353,  443,
      225,  443,  226,  602,  444,  602,  534,  355,  355,  351,
      404,  351,  535,  356,  478,  479,  478,  357,  351,  352,
      351,  225,  353,  661,  227,  230,  231,  230,  352,  232,
      233,  353,  355,  234,  482,  351,  482,  351,  660,  488,

      351,  355,  351,  489,  542,  352,  719,  504,  353,  504,
      352,  656,  351,  353,  351,  613,  495,  613,  355,  235,
      543,  236,  352,  355,  618,  353,  618,  365,  351,  365,
      351,  509,  341,  509,  621,  355,  621,  373,  352,  373,
      235,  353,  544,  237,  238,  231,  238,  352,  232,  233,
      353,  355,  234,  545,  615,  500,  615,  375,  534,  655,
      376,  514,  515,  514,  535,  516,  546,  516,  622,  367,
      622,  368,  369,  373,  370,  373,  371,  373,  235,  373,
      236,  523,  547,  352,  372,  524,  353,  352,  654,  653,
      353,  629,  539,  375,  539,  652,  376,  375,  645,  235,

      376,  627,  237,  245,  245,  246,  245,  245,  247,  248,
      245,  245,  249,  250,  251,  245,  245,  245,  245,  245,
      245,  245,  245,  245,  245,  245,  245,  245,  245,  245,
      252,  252,  252,  252,  252,  252,  252,  252,  253,  252,
      254,  252,  250,  245,  251,  252,  252,  252,  252,  252,
      252,  252,  252,  252,  252,  252,  252,  252,  252,  253,
      252,  252,  255,  252,  250,  251,  252,  274,  275,  276,
      275,  274,  274,  274,  274,  274,  277,  274,  274,  274,
      274,  274,  274,  278,  274,  274,  274,  274,  274,  274,
      274,  274,  274,  274,  279,  279,  279,  279,  279,  279,

      279,  279,  279,  279,  279,  279,  274,  274,  274,  279,
      279,  279,  279,  279,  279,  279,  279,  280,  279,  279,
      279,  279,  279,  279,  279,  279,  279,  279,  274,  274,
      279,  281,  282,  283,  282,  281,  281,  281,  281,  281,
      284,  285,  286,  287,  281,  288,  281,  281,  281,  281,
      281,  281,  281,  281,  281,  281,  281,  281,  289,  289,
      289,  289,  289,  289,  289,  289,  289,  289,  289,  289,
      281,  290,  281,  289,  289,  289,  289,  289,  289,  289,
      289,  289,  289,  289,  289,  289,  289,  289,  289,  289,
      289,  289,  281,  281,  289,  291,  292,  293,  292,  291,

      291,  291,  291,  291,  294,  291,  291,  291,  291,  291,
      291,  291,  291,  291,  291,  291,  291,  291,  291,  291,
      291,  291,  295,  295,  295,  295,  295,  295,  295,  295,
      295,  295,  295,  295,  291,  291,  291,  295,  295,  295,
      295,  295,  295,  295,  295,  295,  295,  295,  295,  295,
      295,  295,  295,  295,  295,  295,  291,  291,  295,  297,
      576,  626,  298,  373,  576,  373,  299,  300,  301,  373,
      505,  373,  505,  352,  534,  505,  353,  505,  302,  352,
      535,  303,  353,  375,  522,  506,  376,  521,  534,  375,
      506,  373,  376,  373,  535,  625,  300,  304,  301,  624,

      517,  352,  517,  538,  353,  538,  505,  517,  505,  517,
      602,  375,  602,  550,  376,  518,  768,  617,  300,  301,
      297,  506,  518,  298,  616,  549,  355,  299,  300,  301,
      373,  373,  373,  373,  356,  614,  803,  551,  357,  302,
      352,  352,  303,  353,  353,  602,  618,  602,  618,  612,
      375,  375,  486,  376,  376,  484,  611,  300,  304,  301,
      620,  510,  510,  510,  510,  613,  495,  613,  615,  500,
      615,  511,  511,  559,  512,  512,  621,  609,  621,  300,
      301,  314,  314,   67,  314,  314,  314,  314,  314,  314,
      315,  314,  314,  314,  314,  314,  314,  314,  314,  314,

      314,  314,  314,  314,  314,  314,  314,  314,  316,  316,
      316,  316,  316,  316,  316,  316,  316,  316,  316,  316,
      317,  314,  318,  316,  316,  316,  316,  316,  316,  316,
      316,  316,  316,  316,  316,  316,  316,  316,  316,  316,
      316,  316,  314,  314,  316,   66,   66,   67,   66,   66,
       66,   66,   66,   66,   68,   66,   66,   66,   66,   66,
       66,   66,   66,   66,   66,   66,   66,   66,   66,  319,
       66,   66,  320,  320,  320,  320,  320,  320,  320,  320,
      320,  320,  320,  320,  321,   66,   66,  320,  320,  320,
      320,  320,  320,  320,  320,  320,  320,  320,  320,  320,

      320,  320,  320,  320,  320,  320,   66,   66,  320,  322,
      323,  324,  323,  322,  325,  322,  322,  322,  326,  327,
      322,  322,  322,  322,  322,  322,  322,  328,  329,  329,
      322,  322,  322,  330,  322,  322,  331,  331,  331,  332,
      331,  331,  331,  331,  333,  334,  335,  331,  336,  337,
      322,  331,  331,  331,  331,  331,  331,  331,  331,  331,
      331,  331,  331,  331,  331,  333,  331,  331,  338,  331,
      339,  322,  331,  373,  373,  373,  373,  356,  607,  534,
      643,  357,  643,  352,  352,  535,  353,  353,  474,  622,
      373,  622,  373,  375,  375,  472,  376,  376,  794,  373,

      352,  373,  794,  353,  373,  606,  373,  377,  604,  352,
      375,  601,  353,  376,  352,  464,  505,  353,  505,  375,
      534,  794,  376,  815,  375,  553,  535,  376,  377,  384,
      384,  506,  384,  384,  644,  461,  384,  384,  600,  384,
      384,  384,  384,  384,  384,  552,  384,  395,  395,  395,
      384,  384,  384,  599,  384,  384,  597,  787,  554,  712,
      712,  712,  373,  558,  373,  517,  457,  517,  384,  384,
      384,  505,  352,  505,  373,  353,  373,  777,  448,  777,
      518,  505,  375,  505,  352,  376,  506,  353,  446,  594,
      384,  384,  395,  373,  375,  373,  506,  376,  373,  517,

      373,  517,  351,  352,  351,  643,  353,  643,  352,  560,
      593,  353,  352,  375,  518,  353,  376,  767,  375,  767,
      401,  376,  402,  402,  403,  355,  351,  561,  351,  574,
      574,  574,  506,  783,  687,  783,  352,  404,  575,  353,
      657,  804,  534,  804,  658,  755,  796,  562,  535,  355,
      796,  589,  563,  384,  384,  575,  384,  384,  657,  644,
      384,  384,  439,  384,  384,  384,  384,  384,  384,  796,
      384,  395,  395,  395,  384,  384,  384,  743,  384,  384,
      430,  807,  401,  807,  403,  403,  403,  757,  633,  757,
      633,  430,  384,  384,  384,  712,  712,  712,  352,  404,

      404,  353,  711,  580,  711,  758,  758,  712,  712,  712,
      658,  355,  579,  578,  384,  384,  395,  404,  590,  590,
      590,  590,  590,  590,  590,  590,  590,  590,  590,  590,
      590,  590,  590,  590,  590,  590,  591,  591,  591,  590,
      590,  590,  590,  590,  590,  591,  592,  592,  591,  592,
      591,  591,  592,  592,  592,  591,  591,  590,  590,  590,
      592,  591,  591,  591,  591,  591,  591,  591,  591,  591,
      591,  591,  591,  591,  591,  591,  591,  591,  591,  400,
      351,  591,  351,  397,  351,  351,  351,  351,  577,  351,
      352,  351,  799,  353,  352,  352,  799,  353,  353,  352,

      573,  570,  353,  355,  351,  569,  351,  355,  355,  568,
      388,  373,  355,  373,  352,  799,  351,  353,  351,  385,
      812,  352,  812,  804,  353,  804,  352,  355,  373,  353,
      373,  375,  631,  373,  376,  373,  634,  567,  352,  355,
      632,  353,  635,  352,  564,  373,  353,  373,  375,  348,
      636,  376,  814,  375,  814,  352,  376,  373,  353,  373,
      807,  373,  807,  373,  505,  375,  505,  352,  376,  344,
      353,  352,  533,  529,  353,  638,  373,  375,  373,  506,
      376,  375,  343,  812,  376,  812,  352,  637,  373,  353,
      373,  534,  639,  373,  534,  373,  375,  535,  352,  376,

      535,  353,  767,  352,  767,  814,  353,  814,  375,  347,
      640,  376,  648,  375,  648,  641,  376,  506,  373,  718,
      373,  505,  352,  505,  373,  353,  373,  816,  352,  412,
      642,  353,  375,  813,  352,  376,  506,  353,  375,  410,
      646,  376,  407,  373,  375,  373,  400,  376,  633,  647,
      633,  397,  695,  352,  695,  522,  353,  697,  352,  697,
      521,  353,  352,  375,  520,  353,  376,  352,  576,  649,
      353,  355,  576,  502,  576,  355,  501,  650,  742,  497,
      355,  492,  486,  484,  481,  474,  472,  470,  464,  651,
      662,  662,  662,  662,  662,  662,  662,  663,  662,  662,

      662,  662,  662,  662,  662,  662,  662,  662,  664,  664,
      664,  662,  662,  662,  662,  662,  662,  665,  665,  665,
      665,  665,  665,  665,  665,  665,  665,  665,  665,  662,
      662,  662,  665,  665,  665,  665,  665,  665,  665,  665,
      665,  665,  665,  665,  665,  665,  665,  665,  665,  665,
      665,  662,  662,  662,  668,  668,  668,  668,  668,  668,
      668,  669,  668,  668,  668,  668,  668,  668,  668,  668,
      668,  668,  670,  670,  670,  668,  668,  668,  668,  668,
      668,  671,  671,  671,  671,  671,  671,  671,  671,  671,
      671,  671,  671,  668,  668,  668,  671,  671,  671,  671,

      671,  671,  671,  671,  671,  671,  671,  671,  671,  671,
      671,  671,  671,  671,  671,  668,  668,  668,  590,  590,
      590,  590,  590,  590,  590,  590,  590,  590,  590,  590,
      590,  590,  590,  590,  590,  590,  591,  591,  591,  590,
      590,  590,  590,  590,  590,  591,  592,  592,  591,  592,
      591,  591,  592,  592,  592,  591,  591,  590,  590,  590,
      592,  591,  591,  591,  591,  591,  591,  591,  591,  591,
      591,  591,  591,  591,  591,  591,  591,  591,  591,  463,
      351,  591,  351,  461,  351,  373,  351,  373,  457,  351,
      352,  351,  447,  353,  352,  352,  448,  353,  353,  352,

      446,  439,  353,  355,  720,  375,  720,  355,  376,  373,
      434,  373,  355,  700,  352,  700,  430,  353,  373,  352,
      373,  415,  353,  352,  693,  414,  353,  355,  352,  375,
      412,  353,  376,  375,  410,  407,  376,  698,  375,  400,
      694,  376,  399,  373,  696,  373,  397,  373,  373,  373,
      373,  394,  393,  352,  388,  387,  353,  352,  352,  385,
      353,  353,  373,  375,  373,  699,  376,  375,  375,  701,
      376,  376,  352,  350,  648,  353,  648,  348,  707,  344,
      707,  343,  375,  817,  352,  376,  817,  353,  352,  817,
      817,  353,  705,  703,  375,  817,  702,  376,  375,  817,

      373,  376,  373,  817,  709,  817,  709,  817,  817,  721,
      352,  721,  817,  353,  352,  817,  817,  353,  706,  352,
      375,  817,  353,  376,  375,  817,  695,  376,  695,  817,
      817,  351,  355,  351,  817,  697,  352,  697,  817,  353,
      700,  352,  700,  817,  353,  352,  762,  801,  353,  355,
      352,  801,  817,  353,  355,  708,  662,  817,  355,  817,
      375,  817,  817,  376,  817,  817,  817,  817,  817,  817,
      801,  764,  817,  764,  817,  817,  664,  664,  664,  664,
      664,  664,  664,  664,  664,  664,  664,  664,  722,  765,
      765,  664,  664,  664,  664,  664,  664,  664,  664,  664,

      664,  664,  664,  664,  664,  664,  664,  664,  664,  664,
      668,  817,  817,  817,  817,  817,  817,  817,  775,  809,
      817,  817,  817,  809,  817,  764,  817,  764,  817,  817,
      670,  670,  670,  670,  670,  670,  670,  670,  670,  670,
      670,  670,  809,  765,  765,  670,  670,  670,  670,  670,
      670,  670,  670,  670,  670,  670,  670,  670,  670,  670,
      670,  670,  670,  670,  373,  373,  373,  373,  766,  725,
      373,  725,  373,  817,  352,  352,  817,  353,  353,  352,
      352,  817,  353,  353,  375,  375,  817,  376,  376,  375,
      375,  817,  376,  376,  727,  817,  727,  817,  817,  729,

      730,  729,  730,  817,  352,  817,  817,  353,  723,  352,
      352,  817,  353,  353,  375,  817,  707,  376,  707,  375,
      375,  724,  376,  376,  817,  726,  352,  817,  373,  353,
      373,  817,  709,  732,  709,  732,  375,  817,  352,  376,
      817,  353,  352,  733,  817,  353,  733,  720,  375,  720,
      817,  376,  375,  735,  817,  376,  736,  352,  817,  721,
      353,  721,  817,  745,  746,  745,  746,  817,  817,  352,
      355,  817,  353,  352,  352,  817,  353,  353,  747,  817,
      747,  757,  355,  757,  375,  731,  355,  376,  352,  817,
      817,  353,  744,  725,  373,  725,  373,  817,  375,  758,

      758,  376,  817,  352,  352,  817,  353,  353,  727,  729,
      727,  729,  817,  375,  375,  817,  376,  376,  352,  352,
      817,  353,  353,  730,  751,  730,  751,  817,  375,  375,
      817,  376,  376,  352,  352,  817,  353,  353,  817,  732,
      732,  732,  732,  375,  375,  817,  376,  376,  817,  733,
      733,  748,  733,  733,  750,  777,  778,  777,  817,  735,
      735,  817,  736,  736,  752,  817,  752,  759,  753,  759,
      778,  760,  779,  779,  779,  745,  746,  745,  746,  755,
      756,  817,  762,  763,  756,  352,  352,  763,  353,  353,
      817,  747,  771,  747,  771,  817,  375,  817,  355,  376,

      817,  352,  352,  756,  353,  353,  763,  751,  817,  751,
      817,  375,  375,  817,  376,  376,  817,  352,  817,  817,
      353,  770,  752,  817,  752,  817,  753,  375,  817,  817,
      376,  817,  817,  817,  817,  817,  817,  755,  756,  759,
      817,  759,  756,  760,  775,  756,  817,  755,  756,  756,
      817,  817,  756,  817,  762,  763,  817,  817,  817,  763,
      817,  756,  817,  764,  817,  764,  817,  817,  756,  817,
      817,  756,  817,  817,  817,  781,  763,  817,  763,  817,
      763,  765,  765,  817,  817,  762,  763,  783,  784,  783,
      763,  817,  817,  817,  817,  777,  778,  777,  771,  763,

      771,  817,  784,  817,  785,  785,  785,  817,  352,  763,
      778,  353,  779,  779,  779,  783,  784,  783,  375,  755,
      809,  376,  755,  796,  809,  817,  817,  796,  817,  817,
      784,  817,  785,  785,  785,  755,  796,  781,  811,  817,
      796,  817,  811,  809,  762,  811,  796,  762,  801,  811,
      762,  801,  801,  775,  809,  801,  817,  817,  809,  796,
      817,  811,  755,  809,  781,  811,  817,  809,  811,  811,
      817,  801,  817,  817,  801,  762,  811,  809,  817,  817,
      811,  817,  817,  817,  817,  817,  809,  817,  811,  817,
      817,  817,  817,  817,  817,  817,  817,  817,  817,  811,

       66,   66,   66,   66,   66,   66,   66,   66,   66,   66,
       66,   66,   66,   66,   66,   66,   66,   66,   66,   66,
      133,  133,  133,  133,  133,  133,  133,  133,  133,  133,
      133,  133,  133,  133,  133,  133,  133,  133,  133,  133,
      141,  141,  141,  141,  141,  141,  141,  141,  141,  141,
      141,  141,  141,  141,  141,  141,  141,  141,  141,  141,
      166,  166,  166,  166,  166,  166,  166,  166,  166,  166,
      166,  166,  166,  166,  166,  166,  166,  166,  166,  166,
      177,  177,  177,  177,  177,  177,  177,  177,  177,  177,
      177,  177,  177,  177,  177,  177,  177,  177,  177,  177,

      189,  189,  189,  189,  189,  189,  189,  189,  189,  189,
      189,  189,  189,  189,  189,  189,  189,  189,  189,  189,
      200,  200,  200,  200,  200,  200,  200,  200,  200,  200,
      200,  200,  200,  200,  200,  200,  200,  200,  200,  200,
      221,  221,  221,  221,  221,  221,  221,  221,  221,  221,
      221,  221,  221,  221,  221,  221,  221,  221,  221,  221,
      229,  229,  229,  229,  229,  229,  229,  229,  229,  229,
      229,  229,  229,  229,  229,  229,  229,  229,  229,  229,
      242,  242,  242,  242,  242,  242,  242,  242,  242,  242,
      242,  242,  242,  242,  242,  242,  242,  242,  242,  242,

      256,  256,  256,  256,  256,  256,  256,  256,  256,  256,
      256,  256,  256,  256,  256,  256,  256,  256,  256,  256,
      260,  260,  260,  260,  260,  260,  260,  260,  260,  260,
      260,  260,  260,  260,  260,  260,  260,  260,  260,  260,
      264,  264,  264,  264,  264,  264,  264,  264,  264,  264,
      264,  264,  264,  264,  264,  264,  264,  264,  264,  264,
      269,  269,  269,  269,  269,  269,  269,  269,  269,  269,
      269,  269,  269,  269,  269,  269,  269,  269,  269,  269,
      271,  271,  271,  271,  271,  271,  271,  271,  271,  271,
      271,  271,  271,  271,  271,  271,  271,  271,  271,  271,

      296,  296,  296,  296,  296,  296,  296,  296,  296,  296,
      296,  296,  296,  296,  296,  296,  296,  296,  296,  296,
      305,  305,  305,  305,  305,  305,  305,  305,  305,  305,
      305,  305,  305,  305,  305,  305,  305,  305,  305,  305,
      342,  817,  817,  342,  817,  817,  817,  817,  817,  817,
      817,  817,  817,  817,  342,  342,  342,  817,  342,  345,
      345,  817,  345,  345,  345,  345,  345,  345,  345,  345,
      345,  345,  345,  345,  345,  345,  345,  345,  345,  349,
      349,  349,  817,  349,  354,  817,  817,  817,  817,  817,
      817,  817,  354,  817,  354,  817,  354,  817,  354,  354,

      354,  817,  354,  366,  817,  817,  366,  817,  817,  817,
      817,  817,  817,  817,  817,  817,  817,  366,  366,  366,
      817,  366,  374,  817,  817,  817,  817,  817,  817,  817,
      374,  817,  374,  374,  374,  817,  374,  374,  374,  817,
      374,  384,  384,  817,  817,  817,  384,  384,  817,  384,
      384,  817,  384,  384,  817,  384,  817,  817,  817,  384,
      384,  386,  386,  817,  386,  386,  386,  386,  386,  386,
      386,  386,  386,  386,  386,  386,  386,  386,  386,  386,
      386,  389,  817,  817,  817,  389,  389,  389,  817,  389,
      398,  398,  817,  398,  398,  398,  398,  398,  398,  398,

      398,  398,  398,  398,  398,  398,  398,  398,  398,  398,
      406,  817,  817,  817,  406,  406,  406,  817,  406,  413,
      817,  817,  817,  413,  413,  413,  817,  413,  413,  413,
      413,  413,  413,  817,  413,  413,  413,  817,  413,  418,
      418,  817,  418,  418,  418,  418,  418,  418,  418,  418,
      418,  418,  418,  418,  418,  418,  418,  418,  418,  422,
      422,  817,  422,  817,  422,  422,  422,  422,  422,  422,
      422,  422,  422,  817,  422,  422,  422,  817,  422,  436,
      817,  817,  817,  436,  436,  436,  817,  436,  438,  438,
      817,  438,  817,  438,  438,  438,  438,  438,  438,  438,

      438,  438,  438,  438,  438,  438,  438,  438,  441,  817,
      817,  817,  441,  441,  441,  817,  441,  442,  817,  817,
      817,  817,  442,  442,  817,  442,  442,  817,  442,  442,
      442,  442,  442,  442,  442,  442,  442,  445,  817,  817,
      445,  817,  817,  817,  817,  817,  817,  817,  817,  817,
      817,  445,  445,  445,  817,  445,  447,  447,  817,  447,
      447,  447,  447,  447,  447,  447,  447,  447,  447,  447,
      447,  447,  447,  447,  447,  447,  454,  454,  817,  454,
      454,  454,  454,  454,  454,  454,  454,  454,  454,  454,
      454,  454,  454,  454,  454,  454,  455,  817,  817,  817,

      455,  455,  455,  817,  455,  458,  458,  817,  817,  817,
      458,  458,  458,  817,  458,  462,  462,  817,  462,  462,
      462,  462,  462,  462,  462,  462,  462,  462,  462,  462,
      462,  462,  462,  462,  462,  469,  817,  817,  817,  817,
      469,  469,  817,  469,  469,  469,  469,  469,  469,  469,
      469,  469,  469,  469,  469,  471,  817,  817,  817,  817,
      817,  817,  817,  817,  817,  817,  471,  471,  471,  817,
      471,  473,  473,  817,  473,  817,  473,  473,  473,  473,
      473,  473,  473,  473,  473,  473,  473,  473,  473,  473,
      473,  480,  480,  817,  480,  480,  480,  480,  480,  480,

      480,  480,  480,  480,  480,  480,  480,  480,  480,  480,
      480,  483,  817,  817,  817,  817,  817,  817,  817,  817,
      817,  817,  817,  817,  817,  483,  483,  483,  817,  483,
      485,  485,  817,  485,  817,  485,  485,  485,  485,  485,
      485,  485,  485,  485,  485,  485,  485,  485,  485,  485,
      487,  817,  817,  817,  487,  487,  487,  817,  487,  491,
      491,  817,  817,  491,  491,  491,  817,  491,  491,  491,
      491,  491,  491,  491,  491,  491,  491,  817,  491,  493,
      493,  493,  493,  493,  493,  493,  493,  493,  493,  493,
      493,  493,  493,  493,  493,  493,  493,  493,  493,  496,

      496,  817,  817,  496,  496,  496,  817,  496,  496,  496,
      496,  496,  496,  496,  496,  496,  496,  817,  496,  498,
      498,  498,  498,  498,  498,  498,  498,  498,  498,  498,
      498,  498,  498,  498,  498,  498,  498,  498,  498,  503,
      503,  817,  503,  503,  503,  503,  503,  503,  503,  503,
      503,  503,  503,  503,  503,  503,  503,  503,  503,  507,
      817,  817,  817,  817,  817,  817,  817,  817,  507,  507,
      817,  817,  817,  507,  507,  507,  817,  507,  513,  817,
      817,  817,  817,  817,  817,  817,  513,  817,  513,  817,
      817,  817,  513,  513,  513,  817,  513,  519,  817,  817,

      817,  817,  817,  817,  817,  817,  519,  519,  817,  817,
      817,  519,  519,  519,  817,  519,  526,  817,  817,  817,
      526,  526,  526,  817,  526,  527,  817,  817,  817,  527,
      527,  527,  817,  527,  342,  817,  817,  342,  817,  817,
      817,  817,  817,  817,  817,  817,  817,  817,  342,  342,
      342,  817,  342,  528,  817,  817,  817,  817,  817,  817,
      817,  528,  817,  528,  817,  528,  817,  528,  528,  528,
      817,  528,  530,  530,  817,  530,  530,  530,  530,  530,
      530,  530,  530,  530,  530,  530,  530,  530,  530,  530,
      530,  530,  531,  531,  817,  531,  531,  531,  531,  531,

      531,  531,  531,  531,  531,  531,  531,  531,  531,  531,
      531,  531,  532,  532,  532,  532,  817,  532,  532,  532,
      532,  532,  532,  532,  532,  532,  532,  532,  532,  532,
      532,  532,  536,  817,  536,  536,  817,  817,  817,  536,
      536,  536,  817,  536,  540,  817,  817,  817,  817,  817,
      817,  817,  817,  817,  817,  817,  817,  817,  540,  540,
      540,  817,  540,  354,  817,  817,  817,  817,  817,  817,
      817,  354,  817,  354,  817,  354,  817,  354,  354,  354,
      817,  354,  541,  541,  541,  541,  541,  541,  541,  541,
      541,  541,  541,  541,  541,  817,  541,  541,  541,  541,

      541,  541,  366,  817,  817,  366,  817,  817,  817,  817,
      817,  817,  817,  817,  817,  817,  366,  366,  366,  817,
      366,  548,  817,  817,  817,  817,  817,  817,  817,  548,
      817,  548,  548,  548,  817,  548,  548,  548,  817,  548,
      374,  817,  817,  817,  817,  817,  817,  817,  374,  817,
      374,  374,  374,  817,  374,  374,  374,  817,  374,  555,
      555,  555,  555,  555,  555,  555,  555,  555,  555,  555,
      555,  555,  817,  555,  555,  555,  555,  555,  555,  384,
      384,  817,  817,  817,  384,  384,  817,  384,  384,  817,
      384,  384,  817,  384,  817,  817,  817,  384,  384,  565,

      565,  817,  565,  565,  565,  565,  565,  565,  565,  565,
      565,  565,  565,  565,  565,  565,  565,  565,  565,  566,
      566,  566,  566,  817,  566,  566,  566,  566,  566,  566,
      566,  566,  566,  566,  566,  566,  566,  566,  566,  389,
      817,  817,  817,  389,  389,  389,  817,  389,  571,  571,
      817,  571,  571,  571,  571,  571,  571,  571,  571,  571,
      571,  571,  571,  571,  571,  571,  571,  571,  572,  572,
      572,  572,  817,  572,  572,  572,  572,  572,  572,  572,
      572,  572,  572,  572,  572,  572,  572,  572,  413,  817,
      817,  817,  413,  413,  413,  817,  413,  413,  413,  413,

      413,  413,  817,  413,  413,  413,  817,  413,  581,  817,
      817,  817,  817,  817,  817,  817,  817,  817,  581,  581,
      581,  422,  422,  817,  422,  817,  422,  422,  422,  422,
      422,  422,  422,  422,  422,  817,  422,  422,  422,  817,
      422,  584,  817,  817,  817,  817,  817,  817,  817,  817,
      817,  584,  584,  584,  436,  817,  817,  817,  436,  436,
      436,  817,  436,  588,  588,  817,  588,  588,  588,  588,
      588,  588,  588,  588,  588,  588,  588,  588,  588,  588,
      588,  588,  588,  441,  817,  817,  817,  441,  441,  441,
      817,  441,  442,  817,  817,  817,  817,  442,  442,  817,

      442,  442,  817,  442,  442,  442,  442,  442,  442,  442,
      442,  442,  445,  817,  817,  445,  817,  817,  817,  817,
      817,  817,  817,  817,  817,  817,  445,  445,  445,  817,
      445,  447,  447,  817,  447,  447,  447,  447,  447,  447,
      447,  447,  447,  447,  447,  447,  447,  447,  447,  447,
      447,  595,  595,  817,  595,  595,  595,  595,  595,  595,
      595,  595,  595,  595,  595,  595,  595,  595,  595,  595,
      595,  455,  817,  817,  817,  455,  455,  455,  817,  455,
      596,  596,  596,  817,  596,  458,  458,  817,  817,  817,
      458,  458,  458,  817,  458,  598,  598,  817,  598,  598,

      598,  598,  598,  598,  598,  598,  598,  598,  598,  598,
      598,  598,  598,  598,  598,  469,  817,  817,  817,  817,
      469,  469,  817,  469,  469,  469,  469,  469,  469,  469,
      469,  469,  469,  469,  469,  603,  817,  817,  817,  817,
      817,  817,  817,  817,  817,  603,  817,  817,  817,  603,
      603,  603,  817,  603,  605,  605,  817,  605,  605,  605,
      605,  605,  605,  605,  605,  605,  605,  605,  605,  605,
      605,  605,  605,  605,  480,  480,  817,  480,  480,  480,
      480,  480,  480,  480,  480,  480,  480,  480,  480,  480,
      480,  480,  480,  480,  483,  817,  817,  817,  817,  817,

      817,  817,  817,  817,  817,  817,  817,  817,  483,  483,
      483,  817,  483,  608,  817,  817,  817,  608,  608,  608,
      817,  608,  610,  610,  817,  610,  610,  610,  610,  610,
      610,  610,  610,  610,  610,  610,  610,  610,  610,  610,
      610,  610,  487,  817,  817,  817,  487,  487,  487,  817,
      487,  491,  491,  817,  817,  491,  491,  491,  817,  491,
      491,  491,  491,  491,  491,  491,  491,  491,  491,  817,
      491,  496,  496,  817,  817,  496,  496,  496,  817,  496,
      496,  496,  496,  496,  496,  496,  496,  496,  496,  817,
      496,  619,  817,  817,  817,  817,  817,  817,  817,  817,

      817,  817,  817,  817,  817,  619,  619,  619,  817,  619,
      507,  817,  817,  817,  817,  817,  817,  817,  817,  507,
      507,  817,  817,  817,  507,  507,  507,  817,  507,  513,
      817,  817,  817,  817,  817,  817,  817,  513,  817,  513,
      817,  817,  817,  513,  513,  513,  817,  513,  623,  817,
      817,  817,  817,  817,  817,  817,  817,  817,  817,  817,
      817,  817,  623,  623,  623,  817,  623,  519,  817,  817,
      817,  817,  817,  817,  817,  817,  519,  519,  817,  817,
      817,  519,  519,  519,  817,  519,  526,  817,  817,  817,
      526,  526,  526,  817,  526,  527,  817,  817,  817,  527,

      527,  527,  817,  527,  528,  817,  817,  817,  817,  817,
      817,  817,  528,  817,  528,  817,  528,  817,  528,  528,
      528,  817,  528,  530,  530,  817,  530,  530,  530,  530,
      530,  530,  530,  530,  530,  530,  530,  530,  530,  530,
      530,  530,  530,  531,  531,  817,  531,  531,  531,  531,
      531,  531,  531,  531,  531,  531,  531,  531,  531,  531,
      531,  531,  531,  628,  628,  628,  817,  628,  536,  817,
      536,  536,  817,  817,  817,  536,  536,  536,  817,  536,
      540,  817,  817,  817,  817,  817,  817,  817,  817,  817,
      817,  817,  817,  817,  540,  540,  540,  817,  540,  630,

      817,  817,  817,  817,  817,  817,  817,  630,  817,  630,
      817,  630,  817,  630,  630,  630,  817,  630,  354,  817,
      817,  817,  817,  817,  817,  817,  354,  817,  354,  817,
      354,  817,  354,  354,  354,  817,  354,  548,  817,  817,
      817,  817,  817,  817,  817,  548,  817,  548,  548,  548,
      817,  548,  548,  548,  817,  548,  374,  817,  817,  817,
      817,  817,  817,  817,  374,  817,  374,  374,  374,  817,
      374,  374,  374,  817,  374,  565,  565,  817,  565,  565,
      565,  565,  565,  565,  565,  565,  565,  565,  565,  565,
      565,  565,  565,  565,  565,  389,  389,  817,  817,  817,

      389,  389,  389,  817,  389,  571,  571,  817,  571,  571,
      571,  571,  571,  571,  571,  571,  571,  571,  571,  571,
      571,  571,  571,  571,  571,  406,  817,  817,  817,  406,
      406,  406,  817,  406,  588,  588,  817,  588,  588,  588,
      588,  588,  588,  588,  588,  588,  588,  588,  588,  588,
      588,  588,  588,  588,  595,  595,  817,  595,  595,  595,
      595,  595,  595,  595,  595,  595,  595,  595,  595,  595,
      595,  595,  595,  595,  677,  677,  817,  817,  817,  677,
      677,  677,  817,  677,  598,  598,  817,  598,  598,  598,
      598,  598,  598,  598,  598,  598,  598,  598,  598,  598,

      598,  598,  598,  598,  680,  680,  680,  680,  817,  680,
      680,  680,  680,  680,  680,  680,  680,  680,  680,  680,
      680,  680,  680,  680,  603,  817,  817,  817,  817,  817,
      817,  817,  817,  817,  603,  817,  817,  817,  603,  603,
      603,  817,  603,  605,  605,  817,  605,  605,  605,  605,
      605,  605,  605,  605,  605,  605,  605,  605,  605,  605,
      605,  605,  605,  683,  683,  817,  683,  683,  683,  683,
      683,  683,  683,  683,  683,  683,  683,  683,  683,  683,
      683,  683,  683,  608,  817,  817,  817,  608,  608,  608,
      817,  608,  610,  610,  817,  610,  610,  610,  610,  610,

      610,  610,  610,  610,  610,  610,  610,  610,  610,  610,
      610,  610,  619,  817,  817,  817,  817,  817,  817,  817,
      817,  817,  817,  817,  817,  817,  619,  619,  619,  817,
      619,  686,  817,  817,  817,  817,  817,  817,  817,  817,
      686,  686,  817,  817,  817,  686,  686,  686,  817,  686,
      507,  817,  817,  817,  817,  817,  817,  817,  817,  507,
      507,  817,  817,  817,  507,  507,  507,  817,  507,  623,
      817,  817,  817,  817,  817,  817,  817,  817,  817,  817,
      817,  817,  817,  623,  623,  623,  817,  623,  688,  817,
      817,  817,  817,  817,  817,  817,  817,  688,  688,  817,

      817,  817,  688,  688,  688,  817,  688,  691,  817,  691,
      691,  817,  817,  817,  691,  691,  691,  817,  691,  710,
      710,  710,  817,  710,  734,  817,  817,  817,  817,  817,
      817,  817,  734,  817,  734,  734,  734,  817,  734,  734,
      734,  817,  734,  754,  817,  754,  754,  817,  754,  817,
      817,  817,  754,  754,  754,  754,  754,  754,  754,  754,
      754,  754,  761,  817,  761,  761,  817,  761,  817,  817,
      817,  761,  761,  761,  761,  761,  761,  761,  761,  761,
      761,  774,  774,  817,  817,  774,  774,  774,  774,  774,
      774,  774,  774,  774,  774,  774,  774,  774,  774,  774,

      774,  776,  817,  776,  817,  817,  817,  817,  776,  776,
      776,  776,  776,  776,  776,  776,  776,  780,  780,  817,
      817,  780,  780,  780,  780,  780,  780,  780,  780,  780,
      780,  780,  780,  780,  780,  780,  780,  782,  817,  782,
      817,  817,  817,  817,  782,  782,  782,  782,  782,  782,
      782,  782,  782,  793,  817,  793,  817,  817,  817,  817,
      793,  793,  793,  793,  793,  793,  793,  793,  793,  795,
      817,  795,  817,  817,  817,  795,  795,  795,  795,  795,
      795,  795,  795,  795,  795,  798,  817,  798,  817,  817,
      817,  817,  798,  798,  798,  798,  798,  798,  798,  798,

      798,  800,  817,  800,  817,  817,  817,  800,  800,  800,
      800,  800,  800,  800,  800,  800,  800,  808,  817,  808,
      817,  817,  817,  808,  808,  808,  808,  808,  808,  808,
      808,  808,  808,  810,  817,  810,  817,  817,  817,  810,
      810,  810,  810,  810,  810,  810,  810,  810,  810,   65,
      817,  817,  817,  817,  817,  817,  817,  817,  817,  817,
      817,  817,  817,  817,  817,  817,  817,  817,  817,  817,
      817,  817,  817,  817,  817,  817,  817,  817,  817,  817,
      817,  817,  817,  817,  817,  817,  817,  817,  817,  817,
      817,  817,  817,  817,  817,  817,  817,  817,  817,  817,

      817,  817,  817,  817,  817,  817,  817,  817,  817,  817,
      817,  817,  817,  817
    } ;

static const flex_int16_t yy_chk[6015] =
    {   0,
        0,    0,    1,    2,   73,  115,   73,  115,    0,    1,
        2,    3,    3,    3,    3,    3,    3,    3,    3,    3,
        3,    3,    3,    3,    3,    3,    3,    3,    3,    3,
        3,    3,    3,    3,    3,    3,    3,    3,    3,    3,
        3,    3,    3,    3,    3,    3,    3,    3,    3,    3,
        3,    3,    3,    3,    3,    3,    3,    3,    3,    3,
        3,    3,    3,    3,    3,    3,    3,    3,    3,    3,
        3,    3,    3,    3,    3,    4,    4,    4,    4,    4,
        4,    4,    4,    4,    4,    4,    4,    4,    4,    4,
        4,    4,    4,    4,    4,    4,    4,    4,    4,    4,

        4,    4,    4,    4,    4,    4,    4,    4,    4,    4,
        4,    4,    4,    4,    4,    4,    4,    4,    4,    4,
        4,    4,    4,    4,    4,    4,    4,    4,    4,    4,
        4,    4,    4,    4,    4,    4,    4,    4,    4,    5,
        5,    5,    5,    5,    5,    5,    5,    5,    5,    5,
        5,    5,    5,    5,    5,    5,    5,    5,    5,    5,
        5,    5,    5,    5,    5,    5,    5,    5,    5,    5,
        5,    5,    5,    5,    5,    5,    5,    5,    5,    5,
        5,    5,    5,    5,    5,    5,    5,    5,    5,    5,
        5,    5,    5,    5,    5,    5,    5,    5,    5,    5,

        5,    5,    5,    7,    7,    7,    7,    7,    7,    7,
        7,    7,    7,    7,    7,    7,    7,    7,    7,    7,
        7,    7,    7,    7,    7,    7,    7,    7,    7,    7,
        7,    7,    7,    7,    7,    7,    7,    7,    7,    7,
        7,    7,    7,    7,    7,    7,    7,    7,    7,    7,
        7,    7,    7,    7,    7,    7,    7,    7,    7,    7,
        7,    7,    7,    7,    7,    7,    7,    9,    9,    9,
      790,    9,   10,   10,   10,    9,   10,   11,  789,   33,
       10,   11,   35,   33,   11,   12,   12,   12,  349,   35,
       12,  786,    9,   12,  349,   34,   36,   10,   33,   34,

       45,   11,   46,   36,  780,   47,    9,   45,   47,   46,
       12,   10,   47,   48,   34,   11,   48,   70,  774,   70,
       48,  140,   70,  140,   12,   13,   13,   13,   13,   13,
       13,   13,   13,   13,   13,   13,   13,   13,   13,   13,
       13,   13,   13,   13,   13,   13,   13,   13,   13,   13,
       13,   13,   13,   13,   13,   13,   13,   13,   13,   13,
       13,   13,   13,   13,   13,   13,   13,   13,   13,   13,
       13,   13,   13,   13,   13,   13,   13,   13,   13,   13,
       13,   13,   13,   13,   13,   13,   13,   13,   13,   15,
       15,   15,   15,   15,   15,   15,   15,   15,   15,   15,

       15,   15,   15,   15,   15,   15,   15,   15,   15,   15,
       15,   15,   15,   15,   15,   15,   15,   15,   15,   15,
       15,   15,   15,   15,   15,   15,   15,   15,   15,   15,
       15,   15,   15,   15,   15,   15,   15,   15,   15,   15,
       15,   15,   15,   15,   15,   15,   15,   15,   15,   15,
       15,   15,   15,   17,   17,   17,   39,   17,   17,   39,
      773,   17,   40,   39,   41,   40,  108,   41,   17,   40,
      108,   41,   42,  417,  126,   42,  417,   43,  126,   42,
       43,  172,  772,   44,   43,  172,   44,   17,  225,   17,
       44,  770,  225,   85,   39,  138,   57,   85,  420,   57,

       40,  420,   41,   57,   57,   57,  769,  766,   17,  138,
       42,   17,   18,   18,   18,   43,   18,   18,  750,  138,
       18,   44,   79,   85,   79,  569,  749,   18,  139,   57,
       58,   57,   79,   58,  153,   79,  153,   58,   58,   58,
      744,  157,  139,  157,   85,   79,   18,  163,   18,  163,
       57,   83,  139,   57,  142,   83,  142,  569,  728,  142,
      175,  175,  175,   58,  147,   58,  147,   18,   82,  147,
       18,   19,   82,  212,   19,  212,  536,  717,   19,   19,
       19,   83,  536,   19,   58,   84,   80,   58,   80,   84,
       80,   86,  716,   19,   80,   86,   80,   83,   82,   80,

      145,  715,   83,   81,  146,   81,  628,   81,   19,   80,
       19,   81,  628,   81,  145,   84,   81,  350,  146,   82,
       82,   86,  714,  350,  145,   87,   81,   84,  146,   87,
       19,   19,   20,  713,  704,   20,   84,   81,   86,   20,
       20,   20,   86,  168,   20,  168,  677,   90,  168,   90,
      219,  219,  219,  426,   20,   87,  426,   90,   81,  427,
       90,   89,  427,   89,   93,  350,   89,   90,   93,   20,
       90,   20,  674,  576,   94,   91,   87,   91,   94,   91,
       87,  673,  235,   91,  672,   91,  235,  241,   91,  241,
      576,   20,   20,   21,   93,   91,   21,  275,   91,  275,

       21,   21,   21,   89,   94,   89,   89,   95,   89,   96,
       89,   95,   21,   96,  667,   93,   93,   97,   89,  253,
       94,   97,  149,  253,  149,   94,  228,  228,  228,   98,
       21,  149,   21,   98,  311,  666,  109,   95,  311,   96,
      109,  112,  149,  660,  282,  112,  282,   97,  292,   95,
      292,  333,   21,   21,   22,  333,  645,   22,   95,   98,
       96,   22,   22,   22,   97,  323,  109,  323,   97,  644,
      625,  112,  121,   22,  121,  121,  121,  624,  128,  131,
       98,  173,  128,  131,   98,  173,  353,  109,  353,  121,
      121,   22,  112,   22,  121,  396,  174,  396,  629,  122,

      174,  122,  122,  122,  629,  611,  226,  121,  128,  131,
      226,  173,  609,   22,   22,   23,  122,  122,   23,  606,
      604,  227,   23,   23,   23,  227,  174,   23,  600,  128,
      131,  236,  173,  392,  122,  236,  226,  392,  237,  238,
      238,  238,  237,  259,  259,  259,  629,  174,  263,  263,
      263,  227,   23,  340,   23,  340,  254,  226,  340,  691,
      254,  236,  279,  255,  279,  691,  599,  255,  237,  290,
      290,  290,  227,  597,   23,   23,   24,  279,  376,   24,
      376,  596,  236,   24,   24,   24,  254,  411,   24,  237,
      289,  411,  289,  255,  295,  342,  295,  342,  594,  421,

      289,  421,  433,  289,  433,  342,  593,  254,  342,  295,
      351,  587,  351,   24,  255,   24,  586,  423,  342,  423,
      351,  816,  423,  351,  401,  401,  401,  816,  435,  428,
      435,  428,  376,  351,  428,   24,   24,   25,   25,   25,
       25,   25,   25,   25,   25,   25,   25,   25,   25,   25,
       25,   25,   25,   25,   25,   25,   25,   25,   25,   25,
       25,   25,   25,   25,   25,   25,   25,   25,   25,   25,
       25,   25,   25,   25,   25,   25,   25,   25,   25,   25,
       25,   25,   25,   25,   25,   25,   25,   25,   25,   25,
       25,   25,   25,   25,   25,   25,   25,   25,   25,   25,

       25,   27,   27,   27,   27,   27,   27,   27,   27,   27,
       27,   27,   27,   27,   27,   27,   27,   27,   27,   27,
       27,   27,   27,   27,   27,   27,   27,   27,   27,   27,
       27,   27,   27,   27,   27,   27,   27,   27,   27,   27,
       27,   27,   27,   27,   27,   27,   27,   27,   27,   27,
       27,   27,   27,   27,   27,   27,   27,   27,   27,   27,
       27,   27,   27,   27,   27,   29,   29,   29,   29,  312,
       29,   29,  451,  312,   29,  431,  451,  431,  280,  467,
      280,  313,  335,  467,  431,  313,  335,  352,  437,  352,
      437,   29,  338,  280,  477,  431,  338,  352,  477,  312,

       29,  328,   29,  328,  328,  328,  440,  456,  440,  456,
      352,  313,  335,  452,  452,  452,  585,  583,  328,  328,
      312,   29,  338,  328,   29,   30,   30,   30,   30,  280,
       30,   30,  313,  335,   30,  329,  328,  329,  329,  329,
      459,  459,  459,  338,  354,  359,  354,  359,  468,  468,
      468,   30,  329,  329,  354,  359,  582,  354,  359,  443,
       30,  443,   30,  471,  443,  471,  692,  354,  359,  358,
      329,  358,  692,  358,  478,  478,  478,  358,  360,  358,
      360,   30,  358,  578,   30,   31,   31,   31,  360,   31,
       31,  360,  358,   31,  482,  361,  482,  361,  577,  490,

      362,  360,  362,  490,  359,  361,  692,  504,  361,  504,
      362,  573,  363,  362,  363,  494,  494,  494,  361,   31,
      360,   31,  363,  362,  506,  363,  506,  365,  364,  365,
      364,  509,  365,  509,  512,  363,  512,  366,  364,  366,
       31,  364,  361,   31,   32,   32,   32,  366,   32,   32,
      366,  364,   32,  362,  499,  499,  499,  366,  537,  570,
      366,  514,  514,  514,  537,  516,  363,  516,  518,  365,
      518,  365,  365,  373,  365,  373,  365,  374,   32,  374,
       32,  525,  364,  373,  365,  525,  373,  374,  568,  567,
      374,  537,  539,  373,  539,  564,  373,  374,  557,   32,

      374,  533,   32,   37,   37,   37,   37,   37,   37,   37,
       37,   37,   37,   37,   37,   37,   37,   37,   37,   37,
       37,   37,   37,   37,   37,   37,   37,   37,   37,   37,
       37,   37,   37,   37,   37,   37,   37,   37,   37,   37,
       37,   37,   37,   37,   37,   37,   37,   37,   37,   37,
       37,   37,   37,   37,   37,   37,   37,   37,   37,   37,
       37,   37,   37,   37,   37,   37,   37,   49,   49,   49,
       49,   49,   49,   49,   49,   49,   49,   49,   49,   49,
       49,   49,   49,   49,   49,   49,   49,   49,   49,   49,
       49,   49,   49,   49,   49,   49,   49,   49,   49,   49,

       49,   49,   49,   49,   49,   49,   49,   49,   49,   49,
       49,   49,   49,   49,   49,   49,   49,   49,   49,   49,
       49,   49,   49,   49,   49,   49,   49,   49,   49,   49,
       49,   51,   51,   51,   51,   51,   51,   51,   51,   51,
       51,   51,   51,   51,   51,   51,   51,   51,   51,   51,
       51,   51,   51,   51,   51,   51,   51,   51,   51,   51,
       51,   51,   51,   51,   51,   51,   51,   51,   51,   51,
       51,   51,   51,   51,   51,   51,   51,   51,   51,   51,
       51,   51,   51,   51,   51,   51,   51,   51,   51,   51,
       51,   51,   51,   51,   51,   53,   53,   53,   53,   53,

       53,   53,   53,   53,   53,   53,   53,   53,   53,   53,
       53,   53,   53,   53,   53,   53,   53,   53,   53,   53,
       53,   53,   53,   53,   53,   53,   53,   53,   53,   53,
       53,   53,   53,   53,   53,   53,   53,   53,   53,   53,
       53,   53,   53,   53,   53,   53,   53,   53,   53,   53,
       53,   53,   53,   53,   53,   53,   53,   53,   53,   55,
      895,  529,   55,  368,  895,  368,   55,   55,   55,  367,
      505,  367,  505,  368,  743,  507,  368,  507,   55,  367,
      743,   55,  367,  368,  524,  505,  368,  523,  787,  367,
      507,  369,  367,  369,  787,  522,   55,   55,   55,  521,

      517,  369,  517,  538,  369,  538,  508,  519,  508,  519,
      602,  369,  602,  368,  369,  517,  743,  502,   55,   55,
       56,  508,  519,   56,  501,  367,  538,   56,   56,   56,
      377,  379,  377,  379,  377,  497,  787,  369,  377,   56,
      377,  379,   56,  377,  379,  603,  618,  603,  618,  492,
      377,  379,  489,  377,  379,  488,  486,   56,   56,   56,
      508,  510,  513,  510,  513,  613,  613,  613,  615,  615,
      615,  510,  513,  379,  510,  513,  621,  484,  621,   56,
       56,   59,   59,   59,   59,   59,   59,   59,   59,   59,
       59,   59,   59,   59,   59,   59,   59,   59,   59,   59,

       59,   59,   59,   59,   59,   59,   59,   59,   59,   59,
       59,   59,   59,   59,   59,   59,   59,   59,   59,   59,
       59,   59,   59,   59,   59,   59,   59,   59,   59,   59,
       59,   59,   59,   59,   59,   59,   59,   59,   59,   59,
       59,   59,   59,   59,   59,   61,   61,   61,   61,   61,
       61,   61,   61,   61,   61,   61,   61,   61,   61,   61,
       61,   61,   61,   61,   61,   61,   61,   61,   61,   61,
       61,   61,   61,   61,   61,   61,   61,   61,   61,   61,
       61,   61,   61,   61,   61,   61,   61,   61,   61,   61,
       61,   61,   61,   61,   61,   61,   61,   61,   61,   61,

       61,   61,   61,   61,   61,   61,   61,   61,   61,   63,
       63,   63,   63,   63,   63,   63,   63,   63,   63,   63,
       63,   63,   63,   63,   63,   63,   63,   63,   63,   63,
       63,   63,   63,   63,   63,   63,   63,   63,   63,   63,
       63,   63,   63,   63,   63,   63,   63,   63,   63,   63,
       63,   63,   63,   63,   63,   63,   63,   63,   63,   63,
       63,   63,   63,   63,   63,   63,   63,   63,   63,   63,
       63,   63,   63,   92,  371,   92,  371,   92,  481,  813,
      556,   92,  556,   92,  371,  813,   92,  371,  476,  622,
      370,  622,  370,   92,  371,  475,   92,  371,  775,  372,

      370,  372,  775,  370,  378,  474,  378,   92,  472,  372,
      370,  470,  372,  370,  378,  466,  619,  378,  619,  372,
      768,  775,  372,  813,  378,  371,  768,  378,   92,  113,
      113,  619,  113,  113,  556,  465,  113,  113,  464,  113,
      113,  113,  113,  113,  113,  370,  113,  113,  113,  113,
      113,  113,  113,  463,  113,  113,  461,  768,  372,  711,
      711,  711,  380,  378,  380,  623,  458,  623,  113,  113,
      113,  686,  380,  686,  381,  380,  381,  777,  450,  777,
      623,  620,  380,  620,  381,  380,  686,  381,  449,  448,
      113,  113,  113,  382,  381,  382,  620,  381,  383,  688,

      383,  688,  528,  382,  528,  643,  382,  643,  383,  380,
      446,  383,  528,  382,  688,  528,  382,  742,  383,  742,
      402,  383,  402,  402,  402,  528,  540,  381,  540,  574,
      574,  574,  742,  783,  620,  783,  540,  402,  402,  540,
      574,  788,  719,  788,  574,  776,  776,  382,  719,  540,
      776,  444,  383,  395,  395,  402,  395,  395,  574,  643,
      395,  395,  439,  395,  395,  395,  395,  395,  395,  776,
      395,  395,  395,  395,  395,  395,  395,  719,  395,  395,
      430,  791,  403,  791,  403,  403,  403,  738,  544,  738,
      544,  429,  395,  395,  395,  712,  712,  712,  544,  403,

      403,  544,  657,  415,  657,  738,  738,  657,  657,  657,
      712,  544,  414,  410,  395,  395,  395,  403,  445,  445,
      445,  445,  445,  445,  445,  445,  445,  445,  445,  445,
      445,  445,  445,  445,  445,  445,  445,  445,  445,  445,
      445,  445,  445,  445,  445,  445,  445,  445,  445,  445,
      445,  445,  445,  445,  445,  445,  445,  445,  445,  445,
      445,  445,  445,  445,  445,  445,  445,  445,  445,  445,
      445,  445,  445,  445,  445,  445,  445,  445,  445,  409,
      542,  445,  542,  408,  543,  545,  543,  545,  407,  546,
      542,  546,  781,  542,  543,  545,  781,  543,  545,  546,

      400,  397,  546,  542,  547,  394,  547,  543,  545,  393,
      391,  548,  546,  548,  547,  781,  630,  547,  630,  390,
      802,  548,  802,  804,  548,  804,  630,  547,  549,  630,
      549,  548,  542,  550,  548,  550,  545,  388,  549,  630,
      543,  549,  546,  550,  385,  551,  550,  551,  549,  357,
      547,  549,  805,  550,  805,  551,  550,  552,  551,  552,
      807,  553,  807,  553,  687,  551,  687,  552,  551,  356,
      552,  553,  348,  344,  553,  550,  554,  552,  554,  687,
      552,  553,  343,  812,  553,  812,  554,  549,  558,  554,
      558,  815,  551,  559,  803,  559,  554,  815,  558,  554,

      803,  558,  767,  559,  767,  814,  559,  814,  558,  341,
      552,  558,  560,  559,  560,  553,  559,  767,  561,  687,
      561,  718,  560,  718,  562,  560,  562,  815,  561,  337,
      554,  561,  560,  803,  562,  560,  718,  562,  561,  334,
      558,  561,  332,  563,  562,  563,  326,  562,  633,  559,
      633,  325,  634,  563,  634,  308,  563,  636,  633,  636,
      307,  633,  634,  563,  304,  634,  563,  636,  941,  561,
      636,  633,  941,  267,  941,  634,  266,  562,  718,  262,
      636,  257,  249,  247,  240,  234,  232,  231,  224,  563,
      581,  581,  581,  581,  581,  581,  581,  581,  581,  581,

      581,  581,  581,  581,  581,  581,  581,  581,  581,  581,
      581,  581,  581,  581,  581,  581,  581,  581,  581,  581,
      581,  581,  581,  581,  581,  581,  581,  581,  581,  581,
      581,  581,  581,  581,  581,  581,  581,  581,  581,  581,
      581,  581,  581,  581,  581,  581,  581,  581,  581,  581,
      581,  581,  581,  581,  584,  584,  584,  584,  584,  584,
      584,  584,  584,  584,  584,  584,  584,  584,  584,  584,
      584,  584,  584,  584,  584,  584,  584,  584,  584,  584,
      584,  584,  584,  584,  584,  584,  584,  584,  584,  584,
      584,  584,  584,  584,  584,  584,  584,  584,  584,  584,

      584,  584,  584,  584,  584,  584,  584,  584,  584,  584,
      584,  584,  584,  584,  584,  584,  584,  584,  591,  591,
      591,  591,  591,  591,  591,  591,  591,  591,  591,  591,
      591,  591,  591,  591,  591,  591,  591,  591,  591,  591,
      591,  591,  591,  591,  591,  591,  591,  591,  591,  591,
      591,  591,  591,  591,  591,  591,  591,  591,  591,  591,
      591,  591,  591,  591,  591,  591,  591,  591,  591,  591,
      591,  591,  591,  591,  591,  591,  591,  591,  591,  223,
      631,  591,  631,  222,  632,  637,  632,  637,  217,  635,
      631,  635,  176,  631,  632,  637,  171,  632,  637,  635,

      169,  161,  635,  631,  693,  637,  693,  632,  637,  638,
      152,  638,  635,  639,  693,  639,  148,  693,  640,  638,
      640,  137,  638,  639,  631,  136,  639,  693,  640,  638,
      130,  640,  638,  639,  127,  125,  639,  637,  640,  119,
      632,  640,  118,  641,  635,  641,  117,  642,  646,  642,
      646,  111,  110,  641,  104,  103,  641,  642,  646,  102,
      642,  646,  647,  641,  647,  638,  641,  642,  646,  640,
      642,  646,  647,   78,  648,  647,  648,   74,  649,   72,
      649,   71,  647,   65,  648,  647,    0,  648,  649,    0,
        0,  649,  646,  642,  648,    0,  641,  648,  649,    0,

      650,  649,  650,    0,  651,    0,  651,    0,    0,  694,
      650,  694,    0,  650,  651,    0,    0,  651,  647,  694,
      650,    0,  694,  650,  651,    0,  695,  651,  695,    0,
        0,  696,  694,  696,    0,  697,  695,  697,    0,  695,
      700,  696,  700,    0,  696,  697,  782,  782,  697,  695,
      700,  782,    0,  700,  696,  650,  664,    0,  697,    0,
      700,    0,    0,  700,    0,    0,    0,    0,    0,    0,
      782,  740,    0,  740,    0,    0,  664,  664,  664,  664,
      664,  664,  664,  664,  664,  664,  664,  664,  696,  740,
      740,  664,  664,  664,  664,  664,  664,  664,  664,  664,

      664,  664,  664,  664,  664,  664,  664,  664,  664,  664,
      670,    0,    0,    0,    0,    0,    0,    0,  793,  793,
        0,    0,    0,  793,    0,  741,    0,  741,    0,    0,
      670,  670,  670,  670,  670,  670,  670,  670,  670,  670,
      670,  670,  793,  741,  741,  670,  670,  670,  670,  670,
      670,  670,  670,  670,  670,  670,  670,  670,  670,  670,
      670,  670,  670,  670,  698,  699,  698,  699,  741,  701,
      702,  701,  702,    0,  698,  699,    0,  698,  699,  701,
      702,    0,  701,  702,  698,  699,    0,  698,  699,  701,
      702,    0,  701,  702,  703,    0,  703,    0,    0,  705,

      706,  705,  706,    0,  703,    0,    0,  703,  698,  705,
      706,    0,  705,  706,  703,    0,  707,  703,  707,  705,
      706,  699,  705,  706,    0,  702,  707,    0,  708,  707,
      708,    0,  709,  710,  709,  710,  707,    0,  708,  707,
        0,  708,  709,  710,    0,  709,  710,  720,  708,  720,
        0,  708,  709,  710,    0,  709,  710,  720,    0,  721,
      720,  721,    0,  722,  723,  722,  723,    0,    0,  721,
      720,    0,  721,  722,  723,    0,  722,  723,  724,    0,
      724,  757,  721,  757,  723,  708,  722,  723,  724,    0,
        0,  724,  720,  725,  726,  725,  726,    0,  724,  757,

      757,  724,    0,  725,  726,    0,  725,  726,  727,  729,
      727,  729,    0,  725,  726,    0,  725,  726,  727,  729,
        0,  727,  729,  730,  731,  730,  731,    0,  727,  729,
        0,  727,  729,  730,  731,    0,  730,  731,    0,  732,
      734,  732,  734,  730,  731,    0,  730,  731,    0,  732,
      734,  726,  732,  734,  729,  758,  758,  758,    0,  732,
      734,    0,  732,  734,  737,    0,  737,  739,  737,  739,
      758,  739,  758,  758,  758,  745,  746,  745,  746,  737,
      737,    0,  739,  739,  737,  745,  746,  739,  745,  746,
        0,  747,  748,  747,  748,    0,  746,    0,  745,  746,

        0,  747,  748,  737,  747,  748,  739,  751,    0,  751,
        0,  747,  748,    0,  747,  748,    0,  751,    0,    0,
      751,  746,  752,    0,  752,    0,  752,  751,    0,  754,
      751,  754,  756,  754,  756,    0,  756,  752,  752,  759,
        0,  759,  752,  759,  754,  754,    0,  756,  756,  754,
        0,    0,  756,    0,  759,  759,    0,    0,    0,  759,
      761,  752,  761,  764,  761,  764,    0,    0,  754,    0,
      763,  756,  763,    0,  763,  761,  761,    0,  759,    0,
      761,  764,  764,    0,    0,  763,  763,  765,  765,  765,
      763,    0,    0,    0,    0,  779,  779,  779,  771,  761,

      771,    0,  765,    0,  765,  765,  765,    0,  771,  763,
      779,  771,  779,  779,  779,  785,  785,  785,  771,  794,
      794,  771,  795,  795,  794,    0,    0,  795,    0,    0,
      785,    0,  785,  785,  785,  796,  796,  798,  798,    0,
      796,    0,  798,  794,  799,  799,  795,  800,  800,  799,
      801,  801,  800,  808,  808,  801,    0,    0,  808,  796,
        0,  798,  809,  809,  810,  810,    0,  809,  799,  810,
        0,  800,    0,    0,  801,  811,  811,  808,    0,    0,
      811,    0,    0,    0,    0,    0,  809,    0,  810,    0,
        0,    0,    0,    0,    0,    0,    0,    0,    0,  811,

      818,  818,  818,  818,  818,  818,  818,  818,  818,  818,
      818,  818,  818,  818,  818,  818,  818,  818,  818,  818,
      819,  819,  819,  819,  819,  819,  819,  819,  819,  819,
      819,  819,  819,  819,  819,  819,  819,  819,  819,  819,
      820,  820,  820,  820,  820,  820,  820,  820,  820,  820,
      820,  820,  820,  820,  820,  820,  820,  820,  820,  820,
      821,  821,  821,  821,  821,  821,  821,  821,  821,  821,
      821,  821,  821,  821,  821,  821,  821,  821,  821,  821,
      822,  822,  822,  822,  822,  822,  822,  822,  822,  822,
      822,  822,  822,  822,  822,  822,  822,  822,  822,  822,

      823,  823,  823,  823,  823,  823,  823,  823,  823,  823,
      823,  823,  823,  823,  823,  823,  823,  823,  823,  823,
      824,  824,  824,  824,  824,  824,  824,  824,  824,  824,
      824,  824,  824,  824,  824,  824,  824,  824,  824,  824,
      825,  825,  825,  825,  825,  825,  825,  825,  825,  825,
      825,  825,  825,  825,  825,  825,  825,  825,  825,  825,
      826,  826,  826,  826,  826,  826,  826,  826,  826,  826,
      826,  826,  826,  826,  826,  826,  826,  826,  826,  826,
      827,  827,  827,  827,  827,  827,  827,  827,  827,  827,
      827,  827,  827,  827,  827,  827,  827,  827,  827,  827,

      828,  828,  828,  828,  828,  828,  828,  828,  828,  828,
      828,  828,  828,  828,  828,  828,  828,  828,  828,  828,
      829,  829,  829,  829,  829,  829,  829,  829,  829,  829,
      829,  829,  829,  829,  829,  829,  829,  829,  829,  829,
      830,  830,  830,  830,  830,  830,  830,  830,  830,  830,
      830,  830,  830,  830,  830,  830,  830,  830,  830,  830,
      831,  831,  831,  831,  831,  831,  831,  831,  831,  831,
      831,  831,  831,  831,  831,  831,  831,  831,  831,  831,
      832,  832,  832,  832,  832,  832,  832,  832,  832,  832,
      832,  832,  832,  832,  832,  832,  832,  832,  832,  832,

      833,  833,  833,  833,  833,  833,  833,  833,  833,  833,
      833,  833,  833,  833,  833,  833,  833,  833,  833,  833,
      834,  834,  834,  834,  834,  834,  834,  834,  834,  834,
      834,  834,  834,  834,  834,  834,  834,  834,  834,  834,
      835,    0,    0,  835,    0,    0,    0,    0,    0,    0,
        0,    0,    0,    0,  835,  835,  835,    0,  835,  836,
      836,    0,  836,  836,  836,  836,  836,  836,  836,  836,
      836,  836,  836,  836,  836,  836,  836,  836,  836,  837,
      837,  837,    0,  837,  838,    0,    0,    0,    0,    0,
        0,    0,  838,    0,  838,    0,  838,    0,  838,  838,

      838,    0,  838,  839,    0,    0,  839,    0,    0,    0,
        0,    0,    0,    0,    0,    0,    0,  839,  839,  839,
        0,  839,  840,    0,    0,    0,    0,    0,    0,    0,
      840,    0,  840,  840,  840,    0,  840,  840,  840,    0,
      840,  841,  841,    0,    0,    0,  841,  841,    0,  841,
      841,    0,  841,  841,    0,  841,    0,    0,    0,  841,
      841,  842,  842,    0,  842,  842,  842,  842,  842,  842,
      842,  842,  842,  842,  842,  842,  842,  842,  842,  842,
      842,  843,    0,    0,    0,  843,  843,  843,    0,  843,
      844,  844,    0,  844,  844,  844,  844,  844,  844,  844,

      844,  844,  844,  844,  844,  844,  844,  844,  844,  844,
      845,    0,    0,    0,  845,  845,  845,    0,  845,  846,
        0,    0,    0,  846,  846,  846,    0,  846,  846,  846,
      846,  846,  846,    0,  846,  846,  846,    0,  846,  847,
      847,    0,  847,  847,  847,  847,  847,  847,  847,  847,
      847,  847,  847,  847,  847,  847,  847,  847,  847,  848,
      848,    0,  848,    0,  848,  848,  848,  848,  848,  848,
      848,  848,  848,    0,  848,  848,  848,    0,  848,  849,
        0,    0,    0,  849,  849,  849,    0,  849,  850,  850,
        0,  850,    0,  850,  850,  850,  850,  850,  850,  850,

      850,  850,  850,  850,  850,  850,  850,  850,  851,    0,
        0,    0,  851,  851,  851,    0,  851,  852,    0,    0,
        0,    0,  852,  852,    0,  852,  852,    0,  852,  852,
      852,  852,  852,  852,  852,  852,  852,  853,    0,    0,
      853,    0,    0,    0,    0,    0,    0,    0,    0,    0,
        0,  853,  853,  853,    0,  853,  854,  854,    0,  854,
      854,  854,  854,  854,  854,  854,  854,  854,  854,  854,
      854,  854,  854,  854,  854,  854,  855,  855,    0,  855,
      855,  855,  855,  855,  855,  855,  855,  855,  855,  855,
      855,  855,  855,  855,  855,  855,  856,    0,    0,    0,

      856,  856,  856,    0,  856,  857,  857,    0,    0,    0,
      857,  857,  857,    0,  857,  858,  858,    0,  858,  858,
      858,  858,  858,  858,  858,  858,  858,  858,  858,  858,
      858,  858,  858,  858,  858,  859,    0,    0,    0,    0,
      859,  859,    0,  859,  859,  859,  859,  859,  859,  859,
      859,  859,  859,  859,  859,  860,    0,    0,    0,    0,
        0,    0,    0,    0,    0,    0,  860,  860,  860,    0,
      860,  861,  861,    0,  861,    0,  861,  861,  861,  861,
      861,  861,  861,  861,  861,  861,  861,  861,  861,  861,
      861,  862,  862,    0,  862,  862,  862,  862,  862,  862,

      862,  862,  862,  862,  862,  862,  862,  862,  862,  862,
      862,  863,    0,    0,    0,    0,    0,    0,    0,    0,
        0,    0,    0,    0,    0,  863,  863,  863,    0,  863,
      864,  864,    0,  864,    0,  864,  864,  864,  864,  864,
      864,  864,  864,  864,  864,  864,  864,  864,  864,  864,
      865,    0,    0,    0,  865,  865,  865,    0,  865,  866,
      866,    0,    0,  866,  866,  866,    0,  866,  866,  866,
      866,  866,  866,  866,  866,  866,  866,    0,  866,  867,
      867,  867,  867,  867,  867,  867,  867,  867,  867,  867,
      867,  867,  867,  867,  867,  867,  867,  867,  867,  868,

      868,    0,    0,  868,  868,  868,    0,  868,  868,  868,
      868,  868,  868,  868,  868,  868,  868,    0,  868,  869,
      869,  869,  869,  869,  869,  869,  869,  869,  869,  869,
      869,  869,  869,  869,  869,  869,  869,  869,  869,  870,
      870,    0,  870,  870,  870,  870,  870,  870,  870,  870,
      870,  870,  870,  870,  870,  870,  870,  870,  870,  871,
        0,    0,    0,    0,    0,    0,    0,    0,  871,  871,
        0,    0,    0,  871,  871,  871,    0,  871,  872,    0,
        0,    0,    0,    0,    0,    0,  872,    0,  872,    0,
        0,    0,  872,  872,  872,    0,  872,  873,    0,    0,

        0,    0,    0,    0,    0,    0,  873,  873,    0,    0,
        0,  873,  873,  873,    0,  873,  874,    0,    0,    0,
      874,  874,  874,    0,  874,  875,    0,    0,    0,  875,
      875,  875,    0,  875,  876,    0,    0,  876,    0,    0,
        0,    0,    0,    0,    0,    0,    0,    0,  876,  876,
      876,    0,  876,  877,    0,    0,    0,    0,    0,    0,
        0,  877,    0,  877,    0,  877,    0,  877,  877,  877,
        0,  877,  878,  878,    0,  878,  878,  878,  878,  878,
      878,  878,  878,  878,  878,  878,  878,  878,  878,  878,
      878,  878,  879,  879,    0,  879,  879,  879,  879,  879,

      879,  879,  879,  879,  879,  879,  879,  879,  879,  879,
      879,  879,  880,  880,  880,  880,    0,  880,  880,  880,
      880,  880,  880,  880,  880,  880,  880,  880,  880,  880,
      880,  880,  881,    0,  881,  881,    0,    0,    0,  881,
      881,  881,    0,  881,  882,    0,    0,    0,    0,    0,
        0,    0,    0,    0,    0,    0,    0,    0,  882,  882,
      882,    0,  882,  883,    0,    0,    0,    0,    0,    0,
        0,  883,    0,  883,    0,  883,    0,  883,  883,  883,
        0,  883,  884,  884,  884,  884,  884,  884,  884,  884,
      884,  884,  884,  884,  884,    0,  884,  884,  884,  884,

      884,  884,  885,    0,    0,  885,    0,    0,    0,    0,
        0,    0,    0,    0,    0,    0,  885,  885,  885,    0,
      885,  886,    0,    0,    0,    0,    0,    0,    0,  886,
        0,  886,  886,  886,    0,  886,  886,  886,    0,  886,
      887,    0,    0,    0,    0,    0,    0,    0,  887,    0,
      887,  887,  887,    0,  887,  887,  887,    0,  887,  888,
      888,  888,  888,  888,  888,  888,  888,  888,  888,  888,
      888,  888,    0,  888,  888,  888,  888,  888,  888,  889,
      889,    0,    0,    0,  889,  889,    0,  889,  889,    0,
      889,  889,    0,  889,    0,    0,    0,  889,  889,  890,

      890,    0,  890,  890,  890,  890,  890,  890,  890,  890,
      890,  890,  890,  890,  890,  890,  890,  890,  890,  891,
      891,  891,  891,    0,  891,  891,  891,  891,  891,  891,
      891,  891,  891,  891,  891,  891,  891,  891,  891,  892,
        0,    0,    0,  892,  892,  892,    0,  892,  893,  893,
        0,  893,  893,  893,  893,  893,  893,  893,  893,  893,
      893,  893,  893,  893,  893,  893,  893,  893,  894,  894,
      894,  894,    0,  894,  894,  894,  894,  894,  894,  894,
      894,  894,  894,  894,  894,  894,  894,  894,  896,    0,
        0,    0,  896,  896,  896,    0,  896,  896,  896,  896,

      896,  896,    0,  896,  896,  896,    0,  896,  897,    0,
        0,    0,    0,    0,    0,    0,    0,    0,  897,  897,
      897,  898,  898,    0,  898,    0,  898,  898,  898,  898,
      898,  898,  898,  898,  898,    0,  898,  898,  898,    0,
      898,  899,    0,    0,    0,    0,    0,    0,    0,    0,
        0,  899,  899,  899,  900,    0,    0,    0,  900,  900,
      900,    0,  900,  901,  901,    0,  901,  901,  901,  901,
      901,  901,  901,  901,  901,  901,  901,  901,  901,  901,
      901,  901,  901,  902,    0,    0,    0,  902,  902,  902,
        0,  902,  903,    0,    0,    0,    0,  903,  903,    0,

      903,  903,    0,  903,  903,  903,  903,  903,  903,  903,
      903,  903,  904,    0,    0,  904,    0,    0,    0,    0,
        0,    0,    0,    0,    0,    0,  904,  904,  904,    0,
      904,  905,  905,    0,  905,  905,  905,  905,  905,  905,
      905,  905,  905,  905,  905,  905,  905,  905,  905,  905,
      905,  906,  906,    0,  906,  906,  906,  906,  906,  906,
      906,  906,  906,  906,  906,  906,  906,  906,  906,  906,
      906,  907,    0,    0,    0,  907,  907,  907,    0,  907,
      908,  908,  908,    0,  908,  909,  909,    0,    0,    0,
      909,  909,  909,    0,  909,  910,  910,    0,  910,  910,

      910,  910,  910,  910,  910,  910,  910,  910,  910,  910,
      910,  910,  910,  910,  910,  911,    0,    0,    0,    0,
      911,  911,    0,  911,  911,  911,  911,  911,  911,  911,
      911,  911,  911,  911,  911,  912,    0,    0,    0,    0,
        0,    0,    0,    0,    0,  912,    0,    0,    0,  912,
      912,  912,    0,  912,  913,  913,    0,  913,  913,  913,
      913,  913,  913,  913,  913,  913,  913,  913,  913,  913,
      913,  913,  913,  913,  914,  914,    0,  914,  914,  914,
      914,  914,  914,  914,  914,  914,  914,  914,  914,  914,
      914,  914,  914,  914,  915,    0,    0,    0,    0,    0,

        0,    0,    0,    0,    0,    0,    0,    0,  915,  915,
      915,    0,  915,  916,    0,    0,    0,  916,  916,  916,
        0,  916,  917,  917,    0,  917,  917,  917,  917,  917,
      917,  917,  917,  917,  917,  917,  917,  917,  917,  917,
      917,  917,  918,    0,    0,    0,  918,  918,  918,    0,
      918,  919,  919,    0,    0,  919,  919,  919,    0,  919,
      919,  919,  919,  919,  919,  919,  919,  919,  919,    0,
      919,  920,  920,    0,    0,  920,  920,  920,    0,  920,
      920,  920,  920,  920,  920,  920,  920,  920,  920,    0,
      920,  921,    0,    0,    0,    0,    0,    0,    0,    0,

        0,    0,    0,    0,    0,  921,  921,  921,    0,  921,
      922,    0,    0,    0,    0,    0,    0,    0,    0,  922,
      922,    0,    0,    0,  922,  922,  922,    0,  922,  923,
        0,    0,    0,    0,    0,    0,    0,  923,    0,  923,
        0,    0,    0,  923,  923,  923,    0,  923,  924,    0,
        0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
        0,    0,  924,  924,  924,    0,  924,  925,    0,    0,
        0,    0,    0,    0,    0,    0,  925,  925,    0,    0,
        0,  925,  925,  925,    0,  925,  926,    0,    0,    0,
      926,  926,  926,    0,  926,  927,    0,    0,    0,  927,

      927,  927,    0,  927,  928,    0,    0,    0,    0,    0,
        0,    0,  928,    0,  928,    0,  928,    0,  928,  928,
      928,    0,  928,  929,  929,    0,  929,  929,  929,  929,
      929,  929,  929,  929,  929,  929,  929,  929,  929,  929,
      929,  929,  929,  930,  930,    0,  930,  930,  930,  930,
      930,  930,  930,  930,  930,  930,  930,  930,  930,  930,
      930,  930,  930,  931,  931,  931,    0,  931,  932,    0,
      932,  932,    0,    0,    0,  932,  932,  932,    0,  932,
      933,    0,    0,    0,    0,    0,    0,    0,    0,    0,
        0,    0,    0,    0,  933,  933,  933,    0,  933,  934,

        0,    0,    0,    0,    0,    0,    0,  934,    0,  934,
        0,  934,    0,  934,  934,  934,    0,  934,  935,    0,
        0,    0,    0,    0,    0,    0,  935,    0,  935,    0,
      935,    0,  935,  935,  935,    0,  935,  936,    0,    0,
        0,    0,    0,    0,    0,  936,    0,  936,  936,  936,
        0,  936,  936,  936,    0,  936,  937,    0,    0,    0,
        0,    0,    0,    0,  937,    0,  937,  937,  937,    0,
      937,  937,  937,    0,  937,  938,  938,    0,  938,  938,
      938,  938,  938,  938,  938,  938,  938,  938,  938,  938,
      938,  938,  938,  938,  938,  939,  939,    0,    0,    0,

      939,  939,  939,    0,  939,  940,  940,    0,  940,  940,
      940,  940,  940,  940,  940,  940,  940,  940,  940,  940,
      940,  940,  940,  940,  940,  942,    0,    0,    0,  942,
      942,  942,    0,  942,  943,  943,    0,  943,  943,  943,
      943,  943,  943,  943,  943,  943,  943,  943,  943,  943,
      943,  943,  943,  943,  944,  944,    0,  944,  944,  944,
      944,  944,  944,  944,  944,  944,  944,  944,  944,  944,
      944,  944,  944,  944,  945,  945,    0,    0,    0,  945,
      945,  945,    0,  945,  946,  946,    0,  946,  946,  946,
      946,  946,  946,  946,  946,  946,  946,  946,  946,  946,

      946,  946,  946,  946,  947,  947,  947,  947,    0,  947,
      947,  947,  947,  947,  947,  947,  947,  947,  947,  947,
      947,  947,  947,  947,  948,    0,    0,    0,    0,    0,
        0,    0,    0,    0,  948,    0,    0,    0,  948,  948,
      948,    0,  948,  949,  949,    0,  949,  949,  949,  949,
      949,  949,  949,  949,  949,  949,  949,  949,  949,  949,
      949,  949,  949,  950,  950,    0,  950,  950,  950,  950,
      950,  950,  950,  950,  950,  950,  950,  950,  950,  950,
      950,  950,  950,  951,    0,    0,    0,  951,  951,  951,
        0,  951,  952,  952,    0,  952,  952,  952,  952,  952,

      952,  952,  952,  952,  952,  952,  952,  952,  952,  952,
      952,  952,  953,    0,    0,    0,    0,    0,    0,    0,
        0,    0,    0,    0,    0,    0,  953,  953,  953,    0,
      953,  954,    0,    0,    0,    0,    0,    0,    0,    0,
      954,  954,    0,    0,    0,  954,  954,  954,    0,  954,
      955,    0,    0,    0,    0,    0,    0,    0,    0,  955,
      955,    0,    0,    0,  955,  955,  955,    0,  955,  956,
        0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
        0,    0,    0,  956,  956,  956,    0,  956,  957,    0,
        0,    0,    0,    0,    0,    0,    0,  957,  957,    0,

        0,    0,  957,  957,  957,    0,  957,  958,    0,  958,
      958,    0,    0,    0,  958,  958,  958,    0,  958,  959,
      959,  959,    0,  959,  960,    0,    0,    0,    0,    0,
        0,    0,  960,    0,  960,  960,  960,    0,  960,  960,
      960,    0,  960,  961,    0,  961,  961,    0,  961,    0,
        0,    0,  961,  961,  961,  961,  961,  961,  961,  961,
      961,  961,  962,    0,  962,  962,    0,  962,    0,    0,
        0,  962,  962,  962,  962,  962,  962,  962,  962,  962,
      962,  963,  963,    0,    0,  963,  963,  963,  963,  963,
      963,  963,  963,  963,  963,  963,  963,  963,  963,  963,

      963,  964,    0,  964,    0,    0,    0,    0,  964,  964,
      964,  964,  964,  964,  964,  964,  964,  965,  965,    0,
        0,  965,  965,  965,  965,  965,  965,  965,  965,  965,
      965,  965,  965,  965,  965,  965,  965,  966,    0,  966,
        0,    0,    0,    0,  966,  966,  966,  966,  966,  966,
      966,  966,  966,  967,    0,  967,    0,    0,    0,    0,
      967,  967,  967,  967,  967,  967,  967,  967,  967,  968,
        0,  968,    0,    0,    0,  968,  968,  968,  968,  968,
      968,  968,  968,  968,  968,  969,    0,  969,    0,    0,
        0,    0,  969,  969,  969,  969,  969,  969,  969,  969,

      969,  970,    0,  970,    0,    0,    0,  970,  970,  970,
      970,  970,  970,  970,  970,  970,  970,  971,    0,  971,
        0,    0,    0,  971,  971,  971,  971,  971,  971,  971,
      971,  971,  971,  972,    0,  972,    0,    0,    0,  972,
      972,  972,  972,  972,  972,  972,  972,  972,  972,  817,
      817,  817,  817,  817,  817,  817,  817,  817,  817,  817,
      817,  817,  817,  817,  817,  817,  817,  817,  817,  817,
      817,  817,  817,  817,  817,  817,  817,  817,  817,  817,
      817,  817,  817,  817,  817,  817,  817,  817,  817,  817,
      817,  817,  817,  817,  817,  817,  817,  817,  817,  817,

      817,  817,  817,  817,  817,  817,  817,  817,  817,  817,
      817,  817,  817,  817
    } ;

#define YY_TRAILING_MASK 0x2000
#define YY_TRAILING_HEAD_MASK 0x4000
#define REJECT \
{ \
*yy_cp = yyg->yy_hold_char; /* undo effects of setting up yytext */ \
yy_cp = yyg->yy_full_match; /* restore poss. backed-over text */ \
yyg->yy_lp = yyg->yy_full_lp; /* restore orig. accepting pos. */ \
yyg->yy_state_ptr = yyg->yy_full_state; /* restore orig. state */ \
yy_current_state = *yyg->yy_state_ptr; /* restore curr. state */ \
++yyg->yy_lp; \
goto find_rule; \
}

#define yymore() yymore_used_but_not_detected
#define YY_MORE_ADJ 0
#define YY_RESTORE_YY_MORE_OFFSET
#line 1 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
/******************************************************************************
 *
 * Copyright (C) 1997-2021 by Dimitri van Heesch.
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation under the terms of the GNU General Public License is hereby
 * granted. No representations are made about the suitability of this software
 * for any purpose. It is provided "as is" without express or implied warranty.
 * See the GNU General Public License for more details.
 *
 * Documents produced by Doxygen are derivative works derived from the
 * input used in their production; they are not affected by this license.
 *
 */
/*  This code is based on the work done by the MoxyPyDoxy team
 *  (Linda Leong, Mike Rivera, Kim Truong, and Gabriel Estrada)
 *  in Spring 2005 as part of CS 179E: Compiler Design Project
 *  at the University of California, Riverside; the course was
 *  taught by Peter H. Froehlich <phf@acm.org>.
 */

#line 35 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"

/*
 *      includes
 */

#include <algorithm>

#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <ctype.h>
#include <string.h>

#include "pyscanner.h"
#include "entry.h"
#include "message.h"
#include "config.h"
#include "doxygen.h"
#include "util.h"
#include "defargs.h"
#include "language.h"
#include "commentscan.h"
#include "arguments.h"
#include "markdown.h"
#include "fileinfo.h"
#include "debug.h"
#include "stringutil.h"

// Toggle for some debugging info
//#define DBG_CTX(x) fprintf x
#define DBG_CTX(x) do { } while(0)

#define YY_NO_INPUT 1
#define YY_NO_UNISTD_H 1

#define unput_string(yytext,yyleng) do { for (int i=(int)yyleng-1;i>=0;i--) unput(yytext[i]); } while(0)

/* ----------------------------------------------------------------- */

struct pyscannerYY_state
{
  CommentScanner          commentScanner;
  OutlineParserInterface *thisParser = nullptr;
  const char *            inputString = nullptr;
  int                     inputPosition = 0;
  Protection              protection = Protection::Public;
  std::shared_ptr<Entry>  current_root;
  std::shared_ptr<Entry>  current;
  std::shared_ptr<Entry>  previous;
  std::shared_ptr<Entry>  bodyEntry;
  int                     yyLineNr = 1 ;
  DString                fileName;
  MethodTypes             mtype = MethodTypes::Method;
  bool                    isStatic = false;
  Specifier               virt = Specifier::Normal;
  int                     docBlockContext = 0;
  DString                docBlock;
  bool                    docBlockInBody = false;
  bool                    docBlockJavaStyle = false;
  bool                    docBrief = false;
  bool                    docBlockSpecial = false;
  bool                    doubleQuote = false;
  bool                    specialBlock = false;
  int                     stringContext = 0;
  TextStream *            copyString = nullptr;
  int                     indent = 0;
  int                     curIndent = 0;
  int                     commentIndent = 0;
  bool                    importTuple = false;
  StringUnorderedMap      packageNameCache;
  char                    atomStart = 0;
  char                    atomEnd = 0;
  int                     atomCount = 0;
  int                     atomContext = 0;
  DString                moduleScope;
  DString                packageName;
  TextStream              defVal;
  int                     braceCount = 0;
  bool                    lexInit = false;
  bool                    packageCommentAllowed = false;
  bool                    startInit = false;
  int                     searchCount = 0;
  DString                argType;
  bool                    funcParamsEnd = false;
  std::vector<DString>   decorators;
  DString                programStr;
  TextStream              decoratorCommentStr;
  int                     decoratorRound = 0;
  bool                    checkDupEntry = false;
  bool                    firstPass = true;
  int                     bracketDepth = 0;
  int                     genericsContext;
  DString                genericsValue;
};

//-----------------------------------------------------------------------------
[[maybe_unused]] static const char *stateToString(int state);

static inline int computeIndent(const char *s);

static void initParser(yyscan_t yyscanner);
static void initEntry(yyscan_t yyscanner);
static void newEntry(yyscan_t yyscanner);
static void addEntry(yyscan_t yyscanner);
static void docVariable(yyscan_t yyscanner,const char *s);
static void newVariable(yyscan_t yyscanner);
static void newTypeAlias(yyscan_t yyscanner);
static void addVariable(yyscan_t yyscanner);
static void newFunction(yyscan_t yyscanner);
static DString findPackageScopeFromPath(yyscan_t yyscanner,const DString &path);
static void addFrom(yyscan_t yyscanner,bool all);
static void lineCount(yyscan_t yyscanner);
static void incLineNr(yyscan_t yyscanner);
static void startCommentBlock(yyscan_t yyscanner,bool brief);
static void handleCommentBlock(yyscan_t yyscanner,const DString &doc,bool brief);
static void endOfDef(yyscan_t yyscanner,int correction=0);
static inline void addToString(yyscan_t yyscanner,const char *s);
static void initTriDoubleQuoteBlock(yyscan_t yyscanner);
static void initTriSingleQuoteBlock(yyscan_t yyscanner);
static void initSpecialBlock(yyscan_t yyscanner);
static void searchFoundDef(yyscan_t yyscanner);
static void searchFoundClass(yyscan_t yyscanner);
static void searchFoundTypeAlias(yyscan_t yyscanner);
static DString findPackageScope(yyscan_t yyscanner,const DString &fileName);
static void setProtection(yyscan_t yyscanner);

static int yyread(yyscan_t yyscanner,char *buf,int max_size);

//-----------------------------------------------------------------------------
/* ----------------------------------------------------------------- */
#undef  YY_INPUT
#define YY_INPUT(buf,result,max_size) result=yyread(yyscanner,buf,max_size);

// otherwise the filename would be the name of the converted file (*.cpp instead of *.l)
static inline const char *getLexerFILE() {return __FILE__;}
#include "doxygen_lex.h"

#line 2626 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner_intermediate.cpp"
#line 175 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
       /* start command character */
  /* Main start state */



  /* Mid-comment states */
  /* %x FuncDoubleComment */
  /* %x ClassDoubleComment */


  /* Function states */






  /* Class states */




  /* Variable states */



  /* String states */





  /* import */



  /* TypeHints */

  /* Decorator */

  /* Generics */



#line 2672 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner_intermediate.cpp"

#define INITIAL 0
#define Search 1
#define SearchMemVars 2
#define SearchSkipValue 3
#define TripleComment 4
#define SpecialComment 5
#define FunctionDec 6
#define FunctionParams 7
#define FunctionBody 8
#define FunctionAnnotation 9
#define FunctionTypeAnnotation 10
#define FunctionParamDefVal 11
#define ClassDec 12
#define ClassInheritance 13
#define ClassCaptureIndent 14
#define ClassBody 15
#define VariableDec 16
#define VariableEnd 17
#define VariableAtom 18
#define SingleQuoteString 19
#define DoubleQuoteString 20
#define TripleString 21
#define SingleQuoteStringIgnore 22
#define DoubleQuoteStringIgnore 23
#define FromMod 24
#define FromModItem 25
#define Import 26
#define TypeHint 27
#define Decorator 28
#define Generics 29
#define TypeAlias 30
#define TypeValue 31

#ifndef YY_NO_UNISTD_H
/* Special case for "unistd.h", since it is non-ANSI. We include it way
 * down here because we want the user's section 1 to have been scanned first.
 * The user has a chance to override it with an option.
 */
#include <unistd.h>
#endif

#define YY_EXTRA_TYPE struct pyscannerYY_state *

/* Holds the entire state of the reentrant scanner. */
struct yyguts_t
    {

    /* User-defined. Not touched by flex. */
    YY_EXTRA_TYPE yyextra_r;

    /* The rest are the same as the globals declared in the non-reentrant scanner. */
    FILE *yyin_r, *yyout_r;
    size_t yy_buffer_stack_top; /**< index of top of stack. */
    size_t yy_buffer_stack_max; /**< capacity of stack. */
    YY_BUFFER_STATE * yy_buffer_stack; /**< Stack as an array. */
    char yy_hold_char;
    int yy_n_chars;
    int yyleng_r;
    char *yy_c_buf_p;
    int yy_init;
    int yy_start;
    int yy_did_buffer_switch_on_eof;
    int yy_start_stack_ptr;
    int yy_start_stack_depth;
    int *yy_start_stack;
    yy_state_type yy_last_accepting_state;
    char* yy_last_accepting_cpos;

    int yylineno_r;
    int yy_flex_debug_r;

    yy_state_type *yy_state_buf;
    yy_state_type *yy_state_ptr;
    char *yy_full_match;
    int yy_lp;

    /* These are only needed for trailing context rules,
     * but there's no conditional variable for that yet. */
    int yy_looking_for_trail_begin;
    int yy_full_lp;
    int *yy_full_state;

    char *yytext_r;
    int yy_more_flag;
    int yy_more_len;

    }; /* end struct yyguts_t */

static int yy_init_globals ( yyscan_t yyscanner );

int yylex_init (yyscan_t* scanner);

int yylex_init_extra ( YY_EXTRA_TYPE user_defined, yyscan_t* scanner);

/* Accessor methods to globals.
   These are made visible to non-reentrant scanners for convenience. */

int yylex_destroy ( yyscan_t yyscanner );

int yyget_debug ( yyscan_t yyscanner );

void yyset_debug ( int debug_flag , yyscan_t yyscanner );

YY_EXTRA_TYPE yyget_extra ( yyscan_t yyscanner );

void yyset_extra ( YY_EXTRA_TYPE user_defined , yyscan_t yyscanner );

FILE *yyget_in ( yyscan_t yyscanner );

void yyset_in  ( FILE * _in_str , yyscan_t yyscanner );

FILE *yyget_out ( yyscan_t yyscanner );

void yyset_out  ( FILE * _out_str , yyscan_t yyscanner );

			int yyget_leng ( yyscan_t yyscanner );

char *yyget_text ( yyscan_t yyscanner );

int yyget_lineno ( yyscan_t yyscanner );

void yyset_lineno ( int _line_number , yyscan_t yyscanner );

int yyget_column  ( yyscan_t yyscanner );

void yyset_column ( int _column_no , yyscan_t yyscanner );

/* Macros after this point can all be overridden by user definitions in
 * section 1.
 */

#ifndef YY_SKIP_YYWRAP
#ifdef __cplusplus
extern "C" int yywrap ( yyscan_t yyscanner );
#else
extern int yywrap ( yyscan_t yyscanner );
#endif
#endif

#ifndef YY_NO_UNPUT
    
    static void yyunput ( int c, char *buf_ptr  , yyscan_t yyscanner);
    
#endif

#ifndef yytext_ptr
static void yy_flex_strncpy ( char *, const char *, int , yyscan_t yyscanner);
#endif

#ifdef YY_NEED_STRLEN
static int yy_flex_strlen ( const char * , yyscan_t yyscanner);
#endif

#ifndef YY_NO_INPUT
#ifdef __cplusplus
static int yyinput ( yyscan_t yyscanner );
#else
static int input ( yyscan_t yyscanner );
#endif

#endif

/* Amount of stuff to slurp up with each read. */
#ifndef YY_READ_BUF_SIZE
#ifdef __ia64__
/* On IA-64, the buffer size is 16k, not 8k */
#define YY_READ_BUF_SIZE 16384
#else
#define YY_READ_BUF_SIZE 8192
#endif /* __ia64__ */
#endif

/* Copy whatever the last rule matched to the standard output. */
#ifndef ECHO
/* This used to be an fputs(), but since the string might contain NUL's,
 * we now use fwrite().
 */
#define ECHO do { if (fwrite( yytext, (size_t) yyleng, 1, yyout )) {} } while (0)
#endif

/* Gets input and stuffs it into "buf".  number of characters read, or YY_NULL,
 * is returned in "result".
 */
#ifndef YY_INPUT
#define YY_INPUT(buf,result,max_size) \
	if ( YY_CURRENT_BUFFER_LVALUE->yy_is_interactive ) \
		{ \
		int c = '*'; \
		int n; \
		for ( n = 0; n < max_size && \
			     (c = getc( yyin )) != EOF && c != '\n'; ++n ) \
			buf[n] = (char) c; \
		if ( c == '\n' ) \
			buf[n++] = (char) c; \
		if ( c == EOF && ferror( yyin ) ) \
			YY_FATAL_ERROR( "input in flex scanner failed" ); \
		result = n; \
		} \
	else \
		{ \
		errno=0; \
		while ( (result = (int) fread(buf, 1, (yy_size_t) max_size, yyin)) == 0 && ferror(yyin)) \
			{ \
			if( errno != EINTR) \
				{ \
				YY_FATAL_ERROR( "input in flex scanner failed" ); \
				break; \
				} \
			errno=0; \
			clearerr(yyin); \
			} \
		}\
\

#endif

/* No semi-colon after return; correct usage is to write "yyterminate();" -
 * we don't want an extra ';' after the "return" because that will cause
 * some compilers to complain about unreachable statements.
 */
#ifndef yyterminate
#define yyterminate() return YY_NULL
#endif

/* Number of entries by which start-condition stack grows. */
#ifndef YY_START_STACK_INCR
#define YY_START_STACK_INCR 25
#endif

/* Report a fatal error. */
#ifndef YY_FATAL_ERROR
#define YY_FATAL_ERROR(msg) yy_fatal_error( msg , yyscanner)
#endif

/* end tables serialization structures and prototypes */

/* Default declaration of generated scanner - a define so the user can
 * easily add parameters.
 */
#ifndef YY_DECL
#define YY_DECL_IS_OURS 1

extern int yylex (yyscan_t yyscanner);

#define YY_DECL int yylex (yyscan_t yyscanner)
#endif /* !YY_DECL */

/* Code executed at the beginning of each rule, after yytext and yyleng
 * have been set up.
 */
#ifndef YY_USER_ACTION
#define YY_USER_ACTION
#endif

/* Code executed at the end of each rule. */
#ifndef YY_BREAK
#define YY_BREAK /*LINTED*/break;
#endif

#define YY_RULE_SETUP \
	if ( yyleng > 0 ) \
		YY_CURRENT_BUFFER_LVALUE->yy_at_bol = \
				(yytext[yyleng - 1] == '\n'); \
	YY_USER_ACTION

/** The main scanner function which does all the work.
 */
YY_DECL
{
	yy_state_type yy_current_state;
	char *yy_cp, *yy_bp;
	int yy_act;
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;

	if ( !yyg->yy_init )
		{
		yyg->yy_init = 1;

#ifdef YY_USER_INIT
		YY_USER_INIT;
#endif

        /* Create the reject buffer large enough to save one state per allowed character. */
        if ( ! yyg->yy_state_buf )
            yyg->yy_state_buf = (yy_state_type *)yyalloc(YY_STATE_BUF_SIZE  , yyscanner);
            if ( ! yyg->yy_state_buf )
                YY_FATAL_ERROR( "out of dynamic memory in yylex()" );

		if ( ! yyg->yy_start )
			yyg->yy_start = 1;	/* first start state */

		if ( ! yyin )
			yyin = stdin;

		if ( ! yyout )
			yyout = stdout;

		if ( ! YY_CURRENT_BUFFER ) {
			yyensure_buffer_stack (yyscanner);
			YY_CURRENT_BUFFER_LVALUE =
				yy_create_buffer( yyin, YY_BUF_SIZE , yyscanner);
		}

		yy_load_buffer_state( yyscanner );
		}

	{
#line 284 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"


  /* ------------ Function recognition rules -------------- */

#line 2986 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner_intermediate.cpp"

	while ( /*CONSTCOND*/1 )		/* loops until end-of-file is reached */
		{
		yy_cp = yyg->yy_c_buf_p;

		/* Support of yytext. */
		*yy_cp = yyg->yy_hold_char;

		/* yy_bp points to the position in yy_ch_buf of the start of
		 * the current run.
		 */
		yy_bp = yy_cp;

		yy_current_state = yyg->yy_start;
		yy_current_state += YY_AT_BOL();

		yyg->yy_state_ptr = yyg->yy_state_buf;
		*yyg->yy_state_ptr++ = yy_current_state;

yy_match:
		do
			{
			YY_CHAR yy_c = yy_ec[YY_SC_TO_UI(*yy_cp)] ;
			while ( yy_chk[yy_base[yy_current_state] + yy_c] != yy_current_state )
				{
				yy_current_state = (int) yy_def[yy_current_state];
				if ( yy_current_state >= 818 )
					yy_c = yy_meta[yy_c];
				}
			yy_current_state = yy_nxt[yy_base[yy_current_state] + yy_c];
			*yyg->yy_state_ptr++ = yy_current_state;
			++yy_cp;
			}
		while ( yy_current_state != 817 );

yy_find_action:
		yy_current_state = *--yyg->yy_state_ptr;
		yyg->yy_lp = yy_accept[yy_current_state];
find_rule: /* we branch to this label when backing up */
		for ( ; ; ) /* until we find what rule we matched */
			{
			if ( yyg->yy_lp && yyg->yy_lp < yy_accept[yy_current_state + 1] )
				{
				yy_act = yy_acclist[yyg->yy_lp];
				if ( yy_act & YY_TRAILING_HEAD_MASK ||
				     yyg->yy_looking_for_trail_begin )
					{
					if ( yy_act == yyg->yy_looking_for_trail_begin )
						{
						yyg->yy_looking_for_trail_begin = 0;
						yy_act &= ~YY_TRAILING_HEAD_MASK;
						break;
						}
					}
				else if ( yy_act & YY_TRAILING_MASK )
					{
					yyg->yy_looking_for_trail_begin = yy_act & ~YY_TRAILING_MASK;
					yyg->yy_looking_for_trail_begin |= YY_TRAILING_HEAD_MASK;
					yyg->yy_full_match = yy_cp;
					yyg->yy_full_state = yyg->yy_state_ptr;
					yyg->yy_full_lp = yyg->yy_lp;
					}
				else
					{
					yyg->yy_full_match = yy_cp;
					yyg->yy_full_state = yyg->yy_state_ptr;
					yyg->yy_full_lp = yyg->yy_lp;
					break;
					}
				++yyg->yy_lp;
				goto find_rule;
				}
			--yy_cp;
			yy_current_state = *--yyg->yy_state_ptr;
			yyg->yy_lp = yy_accept[yy_current_state];
			}

		YY_DO_BEFORE_ACTION;

do_action:	/* This label is used only to access EOF actions. */

		switch ( yy_act )
	{ /* beginning of action switch */

case 1:
YY_RULE_SETUP
#line 290 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ // start of a function/method definition with indent
                          DBG_CTX((stderr,"Found def at %d\n",yyextra->yyLineNr));
                          yyextra->indent=computeIndent(yytext);
                          searchFoundDef(yyscanner);
                          BEGIN( FunctionDec );
                        }
	YY_BREAK
case 2:
YY_RULE_SETUP
#line 296 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ // start of an async function/method definition with indent
                          DBG_CTX((stderr,"Found async def at %d\n",yyextra->yyLineNr));
                          yyextra->indent=computeIndent(yytext);
                          searchFoundDef(yyscanner);
                          BEGIN( FunctionDec );
                        }
	YY_BREAK
case 3:
YY_RULE_SETUP
#line 302 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ // start of a function/method definition
                          searchFoundDef(yyscanner);
                          BEGIN( FunctionDec );
                        }
	YY_BREAK
case 4:
YY_RULE_SETUP
#line 306 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ // start of a function/method definition
                          searchFoundDef(yyscanner);
                          BEGIN( FunctionDec );
                        }
	YY_BREAK
case 5:
YY_RULE_SETUP
#line 311 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ // start of a class definition with indent
                          DBG_CTX((stderr,"Found class at %d\n",yyextra->yyLineNr));
                          yyextra->indent=computeIndent(yytext);
                          searchFoundClass(yyscanner);
                          BEGIN( ClassDec ) ;
                        }
	YY_BREAK
case 6:
YY_RULE_SETUP
#line 317 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{  // start of a class definition
                          searchFoundClass(yyscanner);
                          BEGIN( ClassDec ) ;
                       }
	YY_BREAK
case 7:
YY_RULE_SETUP
#line 321 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{
                          yyextra->indent=computeIndent(yytext);
                          searchFoundTypeAlias(yyscanner);
                          BEGIN( TypeAlias );
                       }
	YY_BREAK
case 8:
YY_RULE_SETUP
#line 326 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{
                          searchFoundTypeAlias(yyscanner);
                          BEGIN( TypeAlias );
                       }
	YY_BREAK
case 9:
#line 331 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
case 10:
YY_RULE_SETUP
#line 331 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ // start of an from import
                          yyextra->packageCommentAllowed = false;
                          BEGIN( FromMod );
                       }
	YY_BREAK
case 11:
#line 337 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
case 12:
YY_RULE_SETUP
#line 337 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ // start of an import statement
                          yyextra->packageCommentAllowed = false;
                          BEGIN( Import );
                       }
	YY_BREAK
case 13:
YY_RULE_SETUP
#line 341 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ // property
                        yyextra->current->section   = EntryType::makeVariable();
                        yyextra->current->mtype     = MethodTypes::Property;
                        yyextra->current->name      = DString(yytext).stripWhiteSpace();
                        yyextra->current->fileName  = yyextra->fileName;
                        yyextra->current->startLine = yyextra->yyLineNr;
                        yyextra->current->bodyLine  = yyextra->yyLineNr;
                        yyextra->packageCommentAllowed = false;
                        BEGIN(VariableDec);
                      }
	YY_BREAK
case 14:
/* rule 14 can match eol */
YY_RULE_SETUP
#line 351 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ // variable
                        if (yyextra->searchCount>0) REJECT;
                        yyextra->indent=computeIndent(yytext);
                        yyextra->current->section   = EntryType::makeVariable();
                        yyextra->current->name      = DString(yytext).stripWhiteSpace();
                        yyextra->current->fileName  = yyextra->fileName;
                        yyextra->current->startLine = yyextra->yyLineNr;
                        yyextra->current->bodyLine  = yyextra->yyLineNr;
                        yyextra->packageCommentAllowed = false;
                        BEGIN(VariableDec);
                      }
	YY_BREAK
case 15:
YY_RULE_SETUP
#line 362 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ // variable
                        if (yyextra->searchCount>0) REJECT;
                        DString id = DString(yytext).stripWhiteSpace();
                        if (id =="try" || id == "else" || id == "except" || id == "finally") REJECT;
                        yyextra->indent=computeIndent(yytext);
                        yyextra->current->section   = EntryType::makeVariable();
                        yyextra->current->name      = id;
                        yyextra->current->fileName  = yyextra->fileName;
                        yyextra->current->startLine = yyextra->yyLineNr;
                        yyextra->current->bodyLine  = yyextra->yyLineNr;
                        yyextra->packageCommentAllowed = false;
                        BEGIN(VariableDec);
                      }
	YY_BREAK
case 16:
/* rule 16 can match eol */
YY_RULE_SETUP
#line 375 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ // list of variables, we cannot place the default value
                                                               // so we will skip it later on in a general rule
                                                               // Also note ")" this is to catch also (a,b). the "("
                                                               // is caught in the rule: [(], the ")" will be handled in [)]
                        if (yyextra->searchCount>1) REJECT;
                        yyextra->indent=computeIndent(yytext);
                        yyextra->current->section   = EntryType::makeVariable();
                        yyextra->current->name      = DString(yytext).stripWhiteSpace();
                        yyextra->current->fileName  = yyextra->fileName;
                        yyextra->current->startLine = yyextra->yyLineNr;
                        yyextra->current->bodyLine  = yyextra->yyLineNr;
                        yyextra->packageCommentAllowed = false;
                        addVariable(yyscanner);
                      }
	YY_BREAK
case 17:
YY_RULE_SETUP
#line 389 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ // start of a single quoted string
                        yyextra->stringContext=YY_START;
                        yyextra->copyString=nullptr;
                        yyextra->packageCommentAllowed = false;
                        BEGIN( SingleQuoteString );
                      }
	YY_BREAK
case 18:
YY_RULE_SETUP
#line 395 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ // start of a double quoted string
                        yyextra->stringContext=YY_START;
                        yyextra->copyString=nullptr;
                        yyextra->packageCommentAllowed = false;
                        BEGIN( DoubleQuoteString );
                      }
	YY_BREAK
case 19:
YY_RULE_SETUP
#line 401 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{
                        yyextra->isStatic=true;
                      }
	YY_BREAK
case 20:
YY_RULE_SETUP
#line 404 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ // decorator
                        lineCount(yyscanner);
                        yyextra->decoratorRound = 1;
                        yyextra->copyString=nullptr;
                        BEGIN( Decorator );
                      }
	YY_BREAK
case 21:
YY_RULE_SETUP
#line 410 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ // decorator
                        lineCount(yyscanner);
                      }
	YY_BREAK
case 22:
YY_RULE_SETUP
#line 413 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ // Unix type script comment
                        if (yyextra->yyLineNr != 1) REJECT;
                      }
	YY_BREAK
case 23:
YY_RULE_SETUP
#line 416 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ // normal comment
                        // issue 9672
                        //yyextra->packageCommentAllowed = false;
                      }
	YY_BREAK
case 24:
YY_RULE_SETUP
#line 420 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ // some other identifier
                        yyextra->packageCommentAllowed = false;
                      }
	YY_BREAK
case 25:
YY_RULE_SETUP
#line 423 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{
                        yyextra->curIndent=computeIndent(yytext);
                      }
	YY_BREAK
case 26:
/* rule 26 can match eol */
YY_RULE_SETUP
#line 427 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ // new line
                        lineCount(yyscanner);
                      }
	YY_BREAK
case 27:
YY_RULE_SETUP
#line 431 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ // start of a comment block
                        initTriDoubleQuoteBlock(yyscanner);
                        BEGIN(TripleComment);
                      }
	YY_BREAK
case 28:
YY_RULE_SETUP
#line 436 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ // start of a comment block
                        initTriSingleQuoteBlock(yyscanner);
                        BEGIN(TripleComment);
                      }
	YY_BREAK
case 29:
/* rule 29 can match eol */
*yy_cp = yyg->yy_hold_char; /* undo effects of setting up yytext */
YY_LINENO_REWIND_TO(yy_cp - 1);
yyg->yy_c_buf_p = yy_cp -= 1;
YY_DO_BEFORE_ACTION; /* set up yytext again */
YY_RULE_SETUP
#line 441 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{  // start of a special comment
                        yyextra->curIndent=computeIndent(yytext);
                        yyextra->packageCommentAllowed = false;
                        initSpecialBlock(yyscanner);
                        BEGIN(SpecialComment);
                      }
	YY_BREAK
case 30:
YY_RULE_SETUP
#line 447 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ // we have to do something with (
                        yyextra->searchCount++;
                      }
	YY_BREAK
case 31:
YY_RULE_SETUP
#line 450 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ // we have to do something with )
                        if (yyextra->searchCount>0)
                        {
                          yyextra->searchCount--;
                        }
                      }
	YY_BREAK
case 32:
YY_RULE_SETUP
#line 456 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{
                        yyextra->current->doc.clear();
                        yyextra->current->brief.clear();
                      }
	YY_BREAK
case 33:
YY_RULE_SETUP
#line 460 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{
                      }
	YY_BREAK
case 34:
YY_RULE_SETUP
#line 462 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ // any other character...
                        // This is the major default
                        // that should catch everything
                        // else in Body.
                      }
	YY_BREAK


case 35:
YY_RULE_SETUP
#line 470 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ // python3 style imports
                      }
	YY_BREAK
case 36:
YY_RULE_SETUP
#line 472 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ // from package import
                        yyextra->packageName=yytext;
                      }
	YY_BREAK
case 37:
YY_RULE_SETUP
#line 475 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{
                        BEGIN(FromModItem);
                      }
	YY_BREAK
case 38:
/* rule 38 can match eol */
YY_RULE_SETUP
#line 478 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{
                        incLineNr(yyscanner);
                        BEGIN(Search);
                      }
	YY_BREAK
case 39:
YY_RULE_SETUP
#line 482 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{
                      }
	YY_BREAK
case 40:
YY_RULE_SETUP
#line 484 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{
                        unput(*yytext);
                        BEGIN(Search);
                      }
	YY_BREAK


case 41:
YY_RULE_SETUP
#line 491 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ // import all
                  addFrom(yyscanner,true);
                  BEGIN(Search);
                }
	YY_BREAK
case 42:
YY_RULE_SETUP
#line 495 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{
                  addFrom(yyscanner,false);
                }
	YY_BREAK
case 43:
YY_RULE_SETUP
#line 498 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{
                  addFrom(yyscanner,false);
                }
	YY_BREAK
case 44:
YY_RULE_SETUP
#line 501 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{
                  addFrom(yyscanner,false);
                  if (!yyextra->importTuple)
                  {
                    BEGIN(Search);
                  }
                }
	YY_BREAK
case 45:
/* rule 45 can match eol */
YY_RULE_SETUP
#line 508 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{
                  incLineNr(yyscanner);
                  if (!yyextra->importTuple)
                  {
                    BEGIN(Search);
                  }
                }
	YY_BREAK
case 46:
YY_RULE_SETUP
#line 515 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{
                }
	YY_BREAK
case 47:
YY_RULE_SETUP
#line 517 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{
                  yyextra->importTuple=true;
                }
	YY_BREAK
case 48:
YY_RULE_SETUP
#line 520 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{
                  yyextra->importTuple=false;
                  BEGIN(Search);
                }
	YY_BREAK
case 49:
YY_RULE_SETUP
#line 524 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{
                }
	YY_BREAK
case 50:
/* rule 50 can match eol */
YY_RULE_SETUP
#line 526 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ // line continuation
                  incLineNr(yyscanner);
                }
	YY_BREAK
case 51:
YY_RULE_SETUP
#line 529 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{
                  unput(*yytext);
                  BEGIN(Search);
                }
	YY_BREAK


case 52:
YY_RULE_SETUP
#line 536 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{
                        yyextra->current->name=removeRedundantWhiteSpace(substitute(yytext,".","::"));
                        yyextra->current->fileName = yyextra->fileName;
                        //printf("Adding using declaration: found:%s:%d name=%s\n",qPrint(yyextra->fileName),yyextra->yyLineNr,qPrint(yyextra->current->name));
                        yyextra->current->section=EntryType::makeUsingDecl();
                        yyextra->current_root->moveToSubEntryAndRefresh(yyextra->current);
                        initEntry(yyscanner);
                        BEGIN(Search);
                      }
	YY_BREAK
case 53:
/* rule 53 can match eol */
YY_RULE_SETUP
#line 545 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{
                  incLineNr(yyscanner);
                  BEGIN(Search);
                }
	YY_BREAK
case 54:
YY_RULE_SETUP
#line 549 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{
                }
	YY_BREAK
case 55:
YY_RULE_SETUP
#line 551 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{
                  unput(*yytext);
                  BEGIN(Search);
                }
	YY_BREAK


case 56:
YY_RULE_SETUP
#line 558 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{
                        const char *s = strchr(yytext,'.'); s++;
                        DBG_CTX((stderr,"Found instance method variable %s in %s at %d\n",s,qPrint(yyextra->current_root->name.data(),yyextra->yyLineNr)));
                        docVariable(yyscanner,s);
                        addEntry(yyscanner);
                      }
	YY_BREAK
case 57:
YY_RULE_SETUP
#line 564 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{
                        const char *s = strchr(yytext,'.'); s++;
                        DBG_CTX((stderr,"Found instance method variable %s in %s at %d\n",s,qPrint(yyextra->current_root->name.data(),yyextra->yyLineNr)));
                        docVariable(yyscanner,s);
                        BEGIN( SearchSkipValue  );
                      }
	YY_BREAK
case 58:
YY_RULE_SETUP
#line 570 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ // type hint
                        const char *s = strchr(yytext,'.'); s++;
                        DBG_CTX((stderr,"Found instance method variable %s in %s at %d\n",s,qPrint(yyextra->current_root->name.data(),yyextra->yyLineNr)));
                        docVariable(yyscanner,s);
                        BEGIN(TypeHint);
                      }
	YY_BREAK
case 59:
YY_RULE_SETUP
#line 576 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ // start of a comment block
                        initTriDoubleQuoteBlock(yyscanner);
                        BEGIN(TripleComment);
                      }
	YY_BREAK
case 60:
YY_RULE_SETUP
#line 581 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ // start of a comment block
                        initTriSingleQuoteBlock(yyscanner);
                        BEGIN(TripleComment);
                      }
	YY_BREAK
case 61:
/* rule 61 can match eol */
*yy_cp = yyg->yy_hold_char; /* undo effects of setting up yytext */
YY_LINENO_REWIND_TO(yy_bp + 2);
yyg->yy_c_buf_p = yy_cp = yy_bp + 2;
YY_DO_BEFORE_ACTION; /* set up yytext again */
YY_RULE_SETUP
#line 586 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{  // start of a special comment
                        initSpecialBlock(yyscanner);
                        BEGIN(SpecialComment);
                      }
	YY_BREAK
case 62:
YY_RULE_SETUP
#line 590 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ // #
                      }
	YY_BREAK
case 63:
YY_RULE_SETUP
#line 592 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ // start of a single quoted string
                        yyextra->stringContext=YY_START;
                        yyextra->copyString=nullptr;
                        BEGIN( SingleQuoteString );
                      }
	YY_BREAK
case 64:
YY_RULE_SETUP
#line 597 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ // start of a double quoted string
                        yyextra->stringContext=YY_START;
                        yyextra->copyString=nullptr;
                        BEGIN( DoubleQuoteString );
                      }
	YY_BREAK
case 65:
/* rule 65 can match eol */
YY_RULE_SETUP
#line 602 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ incLineNr(yyscanner); }
	YY_BREAK
case 66:
YY_RULE_SETUP
#line 603 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{
                        yyextra->current->doc.clear();
                        yyextra->current->brief.clear();
                        unput(*yytext);
                        BEGIN( SearchSkipValue  );
                      }
	YY_BREAK
case 67:
YY_RULE_SETUP
#line 609 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
// identifiers
	YY_BREAK
case 68:
YY_RULE_SETUP
#line 610 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
// other uninteresting stuff
	YY_BREAK
case 69:
YY_RULE_SETUP
#line 611 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
// anything else
	YY_BREAK


case 70:
YY_RULE_SETUP
#line 615 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ // skip over start of type hint
                          yyextra->braceCount=0;
                        }
	YY_BREAK
case 71:
YY_RULE_SETUP
#line 618 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{
                          yyextra->current->type+=*yytext;
                          yyextra->braceCount++;
                        }
	YY_BREAK
case 72:
YY_RULE_SETUP
#line 622 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{
                          yyextra->current->type+=*yytext;
                          yyextra->braceCount--;
                        }
	YY_BREAK
case 73:
YY_RULE_SETUP
#line 626 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ // start of a single quoted string
                          yyextra->stringContext=YY_START;
                          yyextra->copyString=nullptr;
                          BEGIN( SingleQuoteString );
                        }
	YY_BREAK
case 74:
YY_RULE_SETUP
#line 631 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ // start of a double quoted string
                          yyextra->stringContext=YY_START;
                          yyextra->copyString=nullptr;
                          BEGIN( DoubleQuoteString );
                        }
	YY_BREAK
case 75:
YY_RULE_SETUP
#line 636 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{
                          if (yyextra->braceCount==0)
                          { // end of the type hint
                            yyextra->current->type = yyextra->current->type.stripWhiteSpace();
                            unput(*yytext);
                            BEGIN(SearchSkipValue);
                          }
                          else
                          {
                            yyextra->current->type+=*yytext;
                          }
                        }
	YY_BREAK
case 76:
/* rule 76 can match eol */
YY_RULE_SETUP
#line 648 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ // end of the type hint
                          yyextra->current->type = yyextra->current->type.stripWhiteSpace();
                          incLineNr(yyscanner);
                          newEntry(yyscanner);
                          BEGIN(SearchMemVars);
                        }
	YY_BREAK
case 77:
/* rule 77 can match eol */
YY_RULE_SETUP
#line 654 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{
                          yyextra->current->type+=' ';
                          incLineNr(yyscanner);
                        }
	YY_BREAK
case 78:
YY_RULE_SETUP
#line 658 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{
                          yyextra->current->type+=*yytext;
                        }
	YY_BREAK


case 79:
YY_RULE_SETUP
#line 664 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ // the assignment operator
                        //printf("====== VariableDec at line %d\n",yyextra->yyLineNr);
                        yyextra->startInit = true;
                        if (YY_START!=TypeValue)
                        {
                          yyextra->current->initializer.str(yytext);
                          yyextra->current->initializer << " ";
                        }
                      }
	YY_BREAK
case 80:
YY_RULE_SETUP
#line 673 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ // spaces
                        yyextra->current->initializer << yytext;
                      }
	YY_BREAK
case 81:
YY_RULE_SETUP
#line 677 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ // integer value
                        if (yyextra->current->type.empty()) yyextra->current->type = "int";
                        yyextra->current->initializer << yytext;
                      }
	YY_BREAK
case 82:
YY_RULE_SETUP
#line 681 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ // floating point value
                        if (yyextra->current->type.empty()) yyextra->current->type = "float";
                        yyextra->current->initializer << yytext;
                      }
	YY_BREAK
case 83:
YY_RULE_SETUP
#line 685 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ // boolean value
                        if (yyextra->current->type.empty()) yyextra->current->type = "bool";
                        yyextra->current->initializer << yytext;
                      }
	YY_BREAK
case 84:
YY_RULE_SETUP
#line 689 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ // string
                        if (yyextra->current->type.empty()) yyextra->current->type = "str";
                        yyextra->current->initializer << yytext;
                        yyextra->copyString=&yyextra->current->initializer;
                        yyextra->stringContext=YY_START;
                        BEGIN( SingleQuoteString );
                      }
	YY_BREAK
case 85:
YY_RULE_SETUP
#line 696 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ // string
                        if (yyextra->current->type.empty()) yyextra->current->type = "str";
                        yyextra->current->initializer << yytext;
                        yyextra->copyString=&yyextra->current->initializer;
                        yyextra->stringContext=YY_START;
                        BEGIN( DoubleQuoteString );
                      }
	YY_BREAK
case 86:
YY_RULE_SETUP
#line 703 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ // start of a comment block
                        if (yyextra->current->type.empty()) yyextra->current->type = "str";
                        yyextra->current->initializer << yytext;
                        yyextra->doubleQuote=true;
                        yyextra->copyString=&yyextra->current->initializer;
                        yyextra->stringContext=YY_START;
                        BEGIN(TripleString);
                      }
	YY_BREAK
case 87:
YY_RULE_SETUP
#line 712 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ // start of a comment block
                        if (yyextra->current->type.empty()) yyextra->current->type = "str";
                        yyextra->current->initializer << yytext;
                        yyextra->doubleQuote=false;
                        yyextra->copyString=&yyextra->current->initializer;
                        yyextra->stringContext=YY_START;
                        BEGIN(TripleString);
                      }
	YY_BREAK
case 88:
YY_RULE_SETUP
#line 720 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ // tuple, only when direct after =
                        if (yyextra->current->mtype!=MethodTypes::Property && yyextra->startInit)
                        {
                          yyextra->current->type = "tuple";
                        }
                        yyextra->current->initializer << *yytext;
                        yyextra->atomStart='(';
                        yyextra->atomEnd=')';
                        yyextra->atomCount=1;
                        yyextra->atomContext=YY_START;
                        BEGIN( VariableAtom );
                      }
	YY_BREAK
case 89:
YY_RULE_SETUP
#line 732 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ // list
                        if (yyextra->startInit) yyextra->current->type = "list";
                        yyextra->current->initializer << *yytext;
                        yyextra->atomStart='[';
                        yyextra->atomEnd=']';
                        yyextra->atomCount=1;
                        yyextra->atomContext=YY_START;
                        BEGIN( VariableAtom );
                      }
	YY_BREAK
case 90:
YY_RULE_SETUP
#line 741 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ // dictionary
                        if (yyextra->startInit) yyextra->current->type = "dict";
                        yyextra->current->initializer << *yytext;
                        yyextra->atomStart='{';
                        yyextra->atomEnd='}';
                        yyextra->atomCount=1;
                        yyextra->atomContext=YY_START;
                        BEGIN( VariableAtom );
                      }
	YY_BREAK
case 91:
/* rule 91 can match eol */
YY_RULE_SETUP
#line 750 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{
                        yyextra->current->initializer << yytext;
                        incLineNr(yyscanner);
                      }
	YY_BREAK
case 92:
YY_RULE_SETUP
#line 754 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{
                        // do something based on the type of the IDENTIFIER
                        if (yyextra->current->type.empty())
                        {
                          for (const auto &child : yyextra->current_root->children())
                          {
                            if (child->name == yytext)
                            {
                               yyextra->current->type = child->type;
                               break;
                            }
                          }
                        }
                        yyextra->startInit = false;
                        yyextra->current->initializer << yytext;
                      }
	YY_BREAK
case 93:
YY_RULE_SETUP
#line 770 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{
                        yyextra->startInit = false;
                        yyextra->current->initializer << *yytext;
                      }
	YY_BREAK


case 94:
/* rule 94 can match eol */
*yy_cp = yyg->yy_hold_char; /* undo effects of setting up yytext */
YY_LINENO_REWIND_TO(yy_bp + 2);
yyg->yy_c_buf_p = yy_cp = yy_bp + 2;
YY_DO_BEFORE_ACTION; /* set up yytext again */
YY_RULE_SETUP
#line 776 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{  // start of a special comment
                        initSpecialBlock(yyscanner);
                        BEGIN(SpecialComment);
                      }
	YY_BREAK
case 95:
YY_RULE_SETUP
#line 780 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ // #
                      }
	YY_BREAK
case 96:
/* rule 96 can match eol */
YY_RULE_SETUP
#line 782 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ incLineNr(yyscanner);
                        newEntry(yyscanner);
                        BEGIN(SearchMemVars);
                      }
	YY_BREAK
case YY_STATE_EOF(SearchSkipValue):
#line 786 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ incLineNr(yyscanner);
                        newEntry(yyscanner);
                        BEGIN(SearchMemVars);
                      }
	YY_BREAK


case 97:
/* rule 97 can match eol */
YY_RULE_SETUP
#line 793 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{
                        DBG_CTX((stderr,"indent %d<=%d\n",computeIndent(&yytext[1]),yyextra->indent));
                        if (computeIndent(&yytext[1])<=yyextra->indent)
                        {
                          unput_string(yytext,yyleng);
                          endOfDef(yyscanner);
                          //YY_CURRENT_BUFFER->yy_at_bol=true;
                          BEGIN(Search);
                        }
                        else
                        {
                          incLineNr(yyscanner);
                          yyextra->current->program << yytext;
                        }
                      }
	YY_BREAK
case 98:
/* rule 98 can match eol */
*yy_cp = yyg->yy_hold_char; /* undo effects of setting up yytext */
YY_LINENO_REWIND_TO(yy_cp - 2);
yyg->yy_c_buf_p = yy_cp -= 2;
YY_DO_BEFORE_ACTION; /* set up yytext again */
YY_RULE_SETUP
#line 808 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{
                        if (computeIndent(&yytext[1])<=yyextra->indent)
                        {
                          unput_string(yytext,yyleng);
                          endOfDef(yyscanner);
                          //YY_CURRENT_BUFFER->yy_at_bol=true;
                          BEGIN(Search);
                        }
                        else
                        {
                          incLineNr(yyscanner);
                          yyextra->current->program << yytext;
                        }
                      }
	YY_BREAK
case YY_STATE_EOF(FunctionBody):
#line 822 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{
                        endOfDef(yyscanner);
                        yyterminate();
                      }
	YY_BREAK
case 99:
/* rule 99 can match eol */
*yy_cp = yyg->yy_hold_char; /* undo effects of setting up yytext */
YY_LINENO_REWIND_TO(yy_cp - 1);
yyg->yy_c_buf_p = yy_cp -= 1;
YY_DO_BEFORE_ACTION; /* set up yytext again */
YY_RULE_SETUP
#line 826 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ // skip empty line
                        yyextra->current->program << yytext;
                      }
	YY_BREAK
case 100:
YY_RULE_SETUP
#line 829 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ // something at indent >0
                        yyextra->curIndent = computeIndent(yytext);
                        if (yyextra->curIndent<=yyextra->indent)
                          // jumped out of the function
                        {
                          endOfDef(yyscanner,1);
                          BEGIN(Search);
                        }
                        else
                        {
                          yyextra->current->program << yytext;
                        }
                      }
	YY_BREAK
case 101:
YY_RULE_SETUP
#line 842 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ // start of a single quoted string
                        yyextra->current->program << yytext;
                        yyextra->stringContext=YY_START;
                        yyextra->specialBlock = false;
                        yyextra->copyString=&yyextra->current->program;
                        BEGIN( SingleQuoteString );
                      }
	YY_BREAK
case 102:
YY_RULE_SETUP
#line 849 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ // start of a double quoted string
                        yyextra->current->program << yytext;
                        yyextra->stringContext=YY_START;
                        yyextra->specialBlock = false;
                        yyextra->copyString=&yyextra->current->program;
                        BEGIN( DoubleQuoteString );
                      }
	YY_BREAK
case 103:
YY_RULE_SETUP
#line 856 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ // non-special stuff
                        yyextra->current->program << yytext;
                        yyextra->specialBlock = false;
                      }
	YY_BREAK
case 104:
YY_RULE_SETUP
#line 860 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ // normal comment
                        yyextra->current->program << yytext;
                      }
	YY_BREAK
case 105:
YY_RULE_SETUP
#line 863 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ // comment half way
                        yyextra->current->program << yytext;
                      }
	YY_BREAK
case 106:
/* rule 106 can match eol */
YY_RULE_SETUP
#line 866 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{
                        incLineNr(yyscanner);
                        yyextra->current->program << yytext;
                      }
	YY_BREAK
case 107:
YY_RULE_SETUP
#line 870 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ // any character
                        yyextra->current->program << *yytext;
                        yyextra->specialBlock = false;
                      }
	YY_BREAK
case 108:
YY_RULE_SETUP
#line 875 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ // start of a comment block
                        yyextra->current->program << yytext;
                        initTriDoubleQuoteBlock(yyscanner);
                        BEGIN(TripleComment);
                      }
	YY_BREAK
case 109:
YY_RULE_SETUP
#line 881 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ // start of a comment block
                        yyextra->current->program << yytext;
                        initTriSingleQuoteBlock(yyscanner);
                        BEGIN(TripleComment);
                      }
	YY_BREAK


case 110:
YY_RULE_SETUP
#line 890 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{
                              //found function name
                              yyextra->current->name = yytext;
                              yyextra->current->name = yyextra->current->name.stripWhiteSpace();
                              newFunction(yyscanner);
                            }
	YY_BREAK
case 111:
YY_RULE_SETUP
#line 896 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ // function without arguments
                              yyextra->specialBlock = true; // expecting a docstring
                              yyextra->bodyEntry = yyextra->current;
                              yyextra->current->bodyLine  = yyextra->yyLineNr;
                              BEGIN(FunctionBody);
                            }
	YY_BREAK
case 112:
YY_RULE_SETUP
#line 902 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ // python 3.12+ style generics
                              yyextra->genericsContext = YY_START;
                              yyextra->bracketDepth = 1;
                              yyextra->genericsValue = "[";
                              BEGIN(Generics);
                            }
	YY_BREAK
case 113:
YY_RULE_SETUP
#line 908 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{
                              yyextra->defVal.str(std::string());
                              yyextra->braceCount = 0;
                              BEGIN(FunctionTypeAnnotation);
                            }
	YY_BREAK
case 114:
YY_RULE_SETUP
#line 913 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{
                              yyextra->funcParamsEnd = false;
                              yyextra->current->bodyLine  = yyextra->yyLineNr;
                              BEGIN(FunctionParams);
                            }
	YY_BREAK
case 115:
YY_RULE_SETUP
#line 918 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ // end of parameter list
                              if (yyextra->current->argList.empty())
                              {
                                yyextra->current->argList.setNoParameters(true);
                              }
                              yyextra->current->args = argListToString(yyextra->current->argList);
                              yyextra->funcParamsEnd = true;
                            }
	YY_BREAK


case 116:
YY_RULE_SETUP
#line 929 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{
                              yyextra->genericsValue+=yytext;
                            }
	YY_BREAK
case 117:
YY_RULE_SETUP
#line 932 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{
                              yyextra->genericsValue+=yytext;
                              yyextra->bracketDepth++;
                            }
	YY_BREAK
case 118:
YY_RULE_SETUP
#line 936 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{
                              yyextra->genericsValue+=yytext;
                              yyextra->bracketDepth--;
                              if (yyextra->bracketDepth==0)
                              {
                                BEGIN(yyextra->genericsContext);
                              }
                            }
	YY_BREAK
case 119:
YY_RULE_SETUP
#line 944 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{
                              yyextra->genericsValue+=yytext;
                            }
	YY_BREAK


case 120:
YY_RULE_SETUP
#line 950 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{
                        }
	YY_BREAK
case 121:
YY_RULE_SETUP
#line 953 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{
                          if (!yyextra->argType.empty())
                          {
                            Argument a;
                            a.name = "";
                            a.type = yyextra->argType;
                            yyextra->current->argList.push_back(a);
                            yyextra->argType = "";
                          }
                        }
	YY_BREAK
case 122:
YY_RULE_SETUP
#line 964 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{
                          yyextra->argType = yytext;
                        }
	YY_BREAK
case 123:
YY_RULE_SETUP
#line 967 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ // Name of parameter
                          lineCount(yyscanner);
                          Argument a;
                          a.name = DString(yytext).stripWhiteSpace();
                          a.type = yyextra->argType;
                          yyextra->current->argList.push_back(a);
                          yyextra->argType = "";
                        }
	YY_BREAK
case 124:
YY_RULE_SETUP
#line 975 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ // default value
                          // TODO: this rule is too simple, need to be able to
                          // match things like =")" as well!
                          yyextra->defVal.str(std::string());
                          yyextra->braceCount = 0;
                          BEGIN(FunctionParamDefVal);
                        }
	YY_BREAK
case 125:
YY_RULE_SETUP
#line 982 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{
                          if (!yyextra->argType.empty())
                          {
                            Argument a;
                            a.name = "";
                            a.type = yyextra->argType;
                            yyextra->current->argList.push_back(a);
                            yyextra->argType = "";
                          }
                          unput(*yytext);
                          BEGIN(FunctionDec);
                        }
	YY_BREAK
case 126:
YY_RULE_SETUP
#line 994 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{
                          yyextra->defVal.str(std::string());
                          yyextra->braceCount = 0;
                          BEGIN(FunctionAnnotation);
                        }
	YY_BREAK
case 127:
YY_RULE_SETUP
#line 999 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ // a comment
                        }
	YY_BREAK
case 128:
YY_RULE_SETUP
#line 1001 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ // Default rule inside arguments.
                        }
	YY_BREAK


case 129:
#line 1008 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
case 130:
#line 1009 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
case 131:
YY_RULE_SETUP
#line 1009 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{
                          ++yyextra->braceCount;
                          yyextra->defVal << *yytext;
                        }
	YY_BREAK
case 132:
#line 1014 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
case 133:
#line 1015 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
case 134:
YY_RULE_SETUP
#line 1015 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{
                          --yyextra->braceCount;
                          yyextra->defVal << *yytext;
                        }
	YY_BREAK
case 135:
YY_RULE_SETUP
#line 1019 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{
                          if (yyextra->braceCount == 0)
                          {
                            yyextra->current->type = yyextra->defVal.str();
                            unput(*yytext);
                            BEGIN(FunctionDec);
                          }
                          else
                            yyextra->defVal << *yytext;
                        }
	YY_BREAK
case 136:
YY_RULE_SETUP
#line 1029 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{
                          yyextra->defVal << *yytext;
                          yyextra->copyString=&yyextra->defVal;
                          yyextra->stringContext=FunctionTypeAnnotation;
                          BEGIN(SingleQuoteString);
                        }
	YY_BREAK
case 137:
YY_RULE_SETUP
#line 1035 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{
                          yyextra->defVal << *yytext;
                          yyextra->copyString=&yyextra->defVal;
                          yyextra->stringContext=FunctionTypeAnnotation;
                          BEGIN(DoubleQuoteString);
                        }
	YY_BREAK
case 138:
/* rule 138 can match eol */
YY_RULE_SETUP
#line 1041 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{
                          yyextra->defVal << *yytext;
                          incLineNr(yyscanner);
                        }
	YY_BREAK
case 139:
YY_RULE_SETUP
#line 1045 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{
                          yyextra->defVal << *yytext;
                        }
	YY_BREAK


case 140:
#line 1052 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
case 141:
#line 1053 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
case 142:
YY_RULE_SETUP
#line 1053 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{
                          ++yyextra->braceCount;
                          yyextra->defVal << *yytext;
                        }
	YY_BREAK
case 143:
#line 1058 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
case 144:
YY_RULE_SETUP
#line 1058 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{
                          --yyextra->braceCount;
                          yyextra->defVal << *yytext;
                        }
	YY_BREAK
case 145:
#line 1063 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
case 146:
#line 1064 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
case 147:
YY_RULE_SETUP
#line 1064 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{
                          if (yyextra->braceCount == 0)
                          {
                            if (!yyextra->current->argList.empty())
                              yyextra->current->argList.back().type += yyextra->defVal.str();
                            if (*yytext != ',')
                              unput(*yytext);
                            BEGIN(FunctionParams);
                          }
                          else
                          {
                            if (*yytext == ')')
                              --yyextra->braceCount;
                            yyextra->defVal << *yytext;
                          }
                        }
	YY_BREAK
case 148:
YY_RULE_SETUP
#line 1080 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{
                          yyextra->defVal << *yytext;
                          yyextra->copyString=&yyextra->defVal;
                          yyextra->stringContext=FunctionAnnotation;
                          BEGIN(SingleQuoteString);
                        }
	YY_BREAK
case 149:
YY_RULE_SETUP
#line 1086 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{
                          yyextra->defVal << *yytext;
                          yyextra->copyString=&yyextra->defVal;
                          yyextra->stringContext=FunctionAnnotation;
                          BEGIN(DoubleQuoteString);
                        }
	YY_BREAK
case 150:
/* rule 150 can match eol */
YY_RULE_SETUP
#line 1092 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{
                          yyextra->defVal << *yytext;
                          incLineNr(yyscanner);
                        }
	YY_BREAK
case 151:
YY_RULE_SETUP
#line 1096 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{
                          yyextra->defVal << *yytext;
                        }
	YY_BREAK


case 152:
#line 1103 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
case 153:
#line 1104 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
case 154:
YY_RULE_SETUP
#line 1104 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ // internal opening brace, assumption is that we have correct code so braces do match
                          ++yyextra->braceCount;
                          yyextra->defVal << *yytext;
                        }
	YY_BREAK
case 155:
#line 1109 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
case 156:
YY_RULE_SETUP
#line 1109 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{
                          --yyextra->braceCount;
                          yyextra->defVal << *yytext;
                        }
	YY_BREAK
case 157:
#line 1114 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
case 158:
YY_RULE_SETUP
#line 1114 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{
                          if (yyextra->braceCount == 0)
                          {
                            if (!yyextra->current->argList.empty())
                              yyextra->current->argList.back().defval=DString(yyextra->defVal.str()).stripWhiteSpace();
                            if (*yytext == ')')
                              unput(*yytext);
                            BEGIN(FunctionParams);
                          }
                          else
                          {
                            if (*yytext == ')')
                              --yyextra->braceCount;
                            yyextra->defVal << *yytext;
                          }
                        }
	YY_BREAK
case 159:
YY_RULE_SETUP
#line 1131 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{
                          yyextra->defVal << *yytext;
                          yyextra->copyString=&yyextra->defVal;
                          yyextra->stringContext=FunctionParamDefVal;
                          BEGIN( SingleQuoteString );
                        }
	YY_BREAK
case 160:
YY_RULE_SETUP
#line 1137 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{
                          yyextra->defVal << *yytext;
                          yyextra->copyString=&yyextra->defVal;
                          yyextra->stringContext=FunctionParamDefVal;
                          BEGIN( DoubleQuoteString );
                        }
	YY_BREAK
case 161:
/* rule 161 can match eol */
YY_RULE_SETUP
#line 1143 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{
                            yyextra->defVal << *yytext;
                            incLineNr(yyscanner);
                        }
	YY_BREAK
case 162:
YY_RULE_SETUP
#line 1147 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{
                            yyextra->defVal << *yytext;
                        }
	YY_BREAK


case 163:
/* rule 163 can match eol */
*yy_cp = yyg->yy_hold_char; /* undo effects of setting up yytext */
YY_LINENO_REWIND_TO(yy_bp + 1);
yyg->yy_c_buf_p = yy_cp = yy_bp + 1;
YY_DO_BEFORE_ACTION; /* set up yytext again */
YY_RULE_SETUP
#line 1154 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ // new def at indent 0
                        if (computeIndent(&yytext[1])<=yyextra->indent)
                        {
                          int i;
                          for (i=(int)yyleng-1;i>=0;i--)
                          {
                            unput(yytext[i]);
                          }
                          endOfDef(yyscanner);
                          //YY_CURRENT_BUFFER->yy_at_bol=true;
                          BEGIN(Search);
                        }
                        else
                        {
                          incLineNr(yyscanner);
                          yyextra->current->program << yytext;
                        }
                      }
	YY_BREAK
case 164:
/* rule 164 can match eol */
*yy_cp = yyg->yy_hold_char; /* undo effects of setting up yytext */
YY_LINENO_REWIND_TO(yy_bp + 1);
yyg->yy_c_buf_p = yy_cp = yy_bp + 1;
YY_DO_BEFORE_ACTION; /* set up yytext again */
YY_RULE_SETUP
#line 1172 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{  // start of a special comment at indent 0
                        if (computeIndent(&yytext[1])<=yyextra->indent)
                        {
                          int i;
                          for (i=(int)yyleng-1;i>=0;i--)
                          {
                            unput(yytext[i]);
                          }
                          endOfDef(yyscanner);
                          //YY_CURRENT_BUFFER->yy_at_bol=true;
                          BEGIN(Search);
                        }
                        else
                        {
                          incLineNr(yyscanner);
                          yyextra->current->program << yytext;
                        }
                      }
	YY_BREAK
case 165:
/* rule 165 can match eol */
*yy_cp = yyg->yy_hold_char; /* undo effects of setting up yytext */
YY_LINENO_REWIND_TO(yy_cp - 1);
yyg->yy_c_buf_p = yy_cp -= 1;
YY_DO_BEFORE_ACTION; /* set up yytext again */
YY_RULE_SETUP
#line 1190 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ // skip empty line
                        yyextra->current->program << yytext;
                      }
	YY_BREAK
case YY_STATE_EOF(ClassBody):
#line 1193 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{
                        endOfDef(yyscanner);
                        yyterminate();
                      }
	YY_BREAK
case 166:
YY_RULE_SETUP
#line 1197 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ // something at indent >0
                        yyextra->curIndent=computeIndent(yytext);
                        DBG_CTX((stderr,"yyextra->curIndent=%d yyextra->indent=%d\n",yyextra->curIndent,yyextra->indent));
                        if (yyextra->curIndent<=yyextra->indent)
                          // jumped out of the class/method
                        {
                          endOfDef(yyscanner,1);
                          yyextra->indent=yyextra->curIndent;
                          // make sure the next rule matches ^...
                          //YY_CURRENT_BUFFER->yy_at_bol=true;
                          //yyextra->hideClassDocs = false;
                          BEGIN(Search);
                        }
                        else
                        {
                          yyextra->current->program << yytext;
                        }
                      }
	YY_BREAK
case 167:
YY_RULE_SETUP
#line 1215 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ // start of a single quoted string
                        yyextra->current->program << *yytext;
                        yyextra->stringContext=YY_START;
                        yyextra->specialBlock = false;
                        yyextra->copyString=&yyextra->current->program;
                        BEGIN( SingleQuoteString );
                      }
	YY_BREAK
case 168:
YY_RULE_SETUP
#line 1222 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ // start of a double quoted string
                        yyextra->current->program << *yytext;
                        yyextra->stringContext=YY_START;
                        yyextra->specialBlock = false;
                        yyextra->copyString=&yyextra->current->program;
                        BEGIN( DoubleQuoteString );
                      }
	YY_BREAK
case 169:
YY_RULE_SETUP
#line 1229 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ // non-special stuff
                        yyextra->current->program << yytext;
                        yyextra->specialBlock = false;
                        //yyextra->hideClassDocs = false;
                      }
	YY_BREAK
case 170:
/* rule 170 can match eol */
YY_RULE_SETUP
#line 1234 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{
                        yyextra->current->program << *yytext;
                        incLineNr(yyscanner);
                      }
	YY_BREAK
case 171:
YY_RULE_SETUP
#line 1238 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ // normal comment
                        yyextra->current->program << yytext;
                      }
	YY_BREAK
case 172:
YY_RULE_SETUP
#line 1241 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ // any character
                        yyextra->specialBlock = false;
                        yyextra->current->program << *yytext;
                      }
	YY_BREAK
case 173:
YY_RULE_SETUP
#line 1245 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ // start of a comment block
                        //if (!yyextra->hideClassDocs)
                        yyextra->current->program << yytext;
                        initTriDoubleQuoteBlock(yyscanner);
                        BEGIN(TripleComment);
                      }
	YY_BREAK
case 174:
YY_RULE_SETUP
#line 1252 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ // start of a comment block
                        //if (!yyextra->hideClassDocs)
                        yyextra->current->program << yytext;
                        initTriSingleQuoteBlock(yyscanner);
                        BEGIN(TripleComment);
                      }
	YY_BREAK


case 175:
YY_RULE_SETUP
#line 1261 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{
                        yyextra->current->name = yytext;
                      }
	YY_BREAK
case 176:
YY_RULE_SETUP
#line 1264 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{
                        yyextra->bracketDepth=1;
                        yyextra->genericsContext = YY_START;
                        yyextra->genericsValue = "[";
                        BEGIN(Generics);
                      }
	YY_BREAK
case 177:
YY_RULE_SETUP
#line 1270 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{
                        yyextra->current->args = yyextra->genericsValue;
                        BEGIN(TypeValue);
                      }
	YY_BREAK


case 178:
/* rule 178 can match eol */
YY_RULE_SETUP
#line 1276 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{
                        incLineNr(yyscanner);
                        if (!stripWhiteSpace(yyextra->current->initializer.str()).empty())
                        {
                          newTypeAlias(yyscanner);
                        }
                        BEGIN(Search);
                      }
	YY_BREAK
case 179:
YY_RULE_SETUP
#line 1284 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{
                        unput(*yytext);
                        newTypeAlias(yyscanner);
                        BEGIN(Search);
                      }
	YY_BREAK
case YY_STATE_EOF(TypeValue):
#line 1289 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ yyterminate();
                      }
	YY_BREAK

case 180:
YY_RULE_SETUP
#line 1293 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{
                         if (yyextra->current->type.empty())
                         {
                             yyextra->current->type = "class";
                         }

                         yyextra->current->section = EntryType::makeClass();
                         yyextra->current->name = yytext;
                         // we need to set the protectiion based on the "local" class name
                         setProtection(yyscanner);

                         // prepend scope in case of nested classes
                         if (yyextra->current_root->section.isScope())
                         {
                           //printf("*** Prepending scope %s to class %s\n",qPrint(yyextra->current_root->name),qPrint(yyextra->current->name));
                           yyextra->current->name.prepend(yyextra->current_root->name+"::");
                         }

                         yyextra->current->name = yyextra->current->name.stripWhiteSpace();
                         yyextra->current->fileName = yyextra->fileName;
                         yyextra->docBlockContext   = YY_START;
                         yyextra->docBlockInBody    = false;
                         yyextra->docBlockJavaStyle = false;
                         yyextra->docBlock.clear();
                         yyextra->bracketDepth = 0;

                         BEGIN(ClassInheritance);
                       }
	YY_BREAK

case 181:
YY_RULE_SETUP
#line 1323 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ // syntactic sugar for the list
                       }
	YY_BREAK
case 182:
YY_RULE_SETUP
#line 1325 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{
                         yyextra->bracketDepth=1;
                         yyextra->genericsContext = YY_START;
                         yyextra->genericsValue = "[";
                         BEGIN(Generics);
                       }
	YY_BREAK
case 183:
YY_RULE_SETUP
#line 1331 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ // begin of the class definition
                         yyextra->specialBlock = true; // expecting a docstring
                         yyextra->current->bodyLine  = yyextra->yyLineNr;
                         yyextra->current->program.str(std::string());
                         BEGIN(ClassCaptureIndent);
                       }
	YY_BREAK
case 184:
YY_RULE_SETUP
#line 1338 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{
                         yyextra->current->extends.emplace_back(
                                                substitute(yytext,".","::"),Protection::Public,Specifier::Normal
                                          );
                       }
	YY_BREAK
case 185:
YY_RULE_SETUP
#line 1343 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ // start of a single quoted string
                         yyextra->stringContext=YY_START;
                         BEGIN( SingleQuoteStringIgnore );
                       }
	YY_BREAK
case 186:
YY_RULE_SETUP
#line 1347 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ // start of a double quoted string
                         yyextra->stringContext=YY_START;
                         BEGIN( DoubleQuoteStringIgnore );
                       }
	YY_BREAK


case 187:
YY_RULE_SETUP
#line 1354 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ // end of a single quoted string
                         BEGIN(yyextra->stringContext);
                       }
	YY_BREAK
case 188:
YY_RULE_SETUP
#line 1357 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ }
	YY_BREAK


case 189:
YY_RULE_SETUP
#line 1360 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ // end of a double quoted string
                         BEGIN(yyextra->stringContext);
                       }
	YY_BREAK
case 190:
YY_RULE_SETUP
#line 1363 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ }
	YY_BREAK


case 191:
/* rule 191 can match eol */
YY_RULE_SETUP
#line 1367 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{
                                 // Blankline - ignore, keep looking for indentation.
                                 lineCount(yyscanner);
                                 yyextra->current->program << yytext;
                               }
	YY_BREAK
case 192:
YY_RULE_SETUP
#line 1373 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ // start of a comment block
                                 initTriDoubleQuoteBlock(yyscanner);
                                 yyextra->current->program << yytext;
                                 BEGIN(TripleComment);
                               }
	YY_BREAK
case 193:
YY_RULE_SETUP
#line 1378 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ // start of a comment block
                                 initTriSingleQuoteBlock(yyscanner);
                                 yyextra->current->program << yytext;
                                 BEGIN(TripleComment);
                               }
	YY_BREAK
case 194:
YY_RULE_SETUP
#line 1383 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{  // start of a special comment
                                 initSpecialBlock(yyscanner);
                                 BEGIN(SpecialComment);
                               }
	YY_BREAK
case 195:
YY_RULE_SETUP
#line 1387 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ // ignore comment with just one #
                               }
	YY_BREAK
case 196:
YY_RULE_SETUP
#line 1389 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{
                                 yyextra->current->program << yytext;
                                 //yyextra->current->startLine = yyextra->yyLineNr;
                                 yyextra->curIndent=computeIndent(yytext);
                                 yyextra->bodyEntry = yyextra->current;
                                 DBG_CTX((stderr,"setting indent %d\n",yyextra->curIndent));
                                 //printf("yyextra->current->program=[%s]\n",qPrint(yyextra->current->program));
                                 //yyextra->hideClassDocs = true;
                                 BEGIN(ClassBody);
                               }
	YY_BREAK
case 197:
YY_RULE_SETUP
#line 1400 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{
                                 // Just pushback an empty class, and
                                 // resume parsing the body.
                                 newEntry(yyscanner);
                                 yyextra->current->program << yytext;

                                 // printf("Failed to find indent - skipping!");
                                 BEGIN( Search );
                               }
	YY_BREAK


case 198:
YY_RULE_SETUP
#line 1413 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ //typing
                        yyextra->startInit = false;
                        yyextra->current->type = substitute(yytext,":","");
                      }
	YY_BREAK
case 199:
*yy_cp = yyg->yy_hold_char; /* undo effects of setting up yytext */
yyg->yy_c_buf_p = yy_cp = yy_bp + 3;
YY_DO_BEFORE_ACTION; /* set up yytext again */
YY_RULE_SETUP
#line 1417 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{  // start of a special comment
                        yyextra->curIndent=computeIndent(yytext);
                        yyextra->packageCommentAllowed = false;
                        initSpecialBlock(yyscanner);
                        yyextra->docBlockContext = VariableEnd;
                        BEGIN(SpecialComment);
                      }
	YY_BREAK
case 200:
YY_RULE_SETUP
#line 1424 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ // comment
                        BEGIN( VariableEnd );
                      }
	YY_BREAK
case 201:
/* rule 201 can match eol */
YY_RULE_SETUP
#line 1427 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{
                        unput('\n');
                        BEGIN( VariableEnd );
                      }
	YY_BREAK


case 202:
YY_RULE_SETUP
#line 1434 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{
                        yyextra->current->initializer << *yytext;
                        if (yyextra->atomStart==*yytext)
                        {
                          yyextra->atomCount++;
                        }
                      }
	YY_BREAK
case 203:
YY_RULE_SETUP
#line 1441 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{
                        yyextra->current->initializer << *yytext;
                        if (yyextra->atomEnd==*yytext)
                        {
                          yyextra->atomCount--;
                        }
                        if (yyextra->atomCount==0)
                        {
                          yyextra->startInit = false;
                          BEGIN(yyextra->atomContext);
                        }
                      }
	YY_BREAK
case 204:
YY_RULE_SETUP
#line 1453 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ // start of a comment block
                        yyextra->specialBlock = false;
                        yyextra->current->program << yytext;
                        initTriDoubleQuoteBlock(yyscanner);
                        BEGIN(TripleComment);
                      }
	YY_BREAK
case 205:
YY_RULE_SETUP
#line 1460 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ // start of a comment block
                        yyextra->specialBlock = false;
                        yyextra->current->program << yytext;
                        initTriSingleQuoteBlock(yyscanner);
                        BEGIN(TripleComment);
                      }
	YY_BREAK
case 206:
YY_RULE_SETUP
#line 1466 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{
                        yyextra->stringContext=YY_START;
                        yyextra->current->initializer << "'";
                        yyextra->copyString=&yyextra->current->initializer;
                        BEGIN( SingleQuoteString );
                      }
	YY_BREAK
case 207:
YY_RULE_SETUP
#line 1472 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{
                        yyextra->stringContext=YY_START;
                        yyextra->current->initializer << "\"";
                        yyextra->copyString=&yyextra->current->initializer;
                        BEGIN( DoubleQuoteString );
                      }
	YY_BREAK
case 208:
YY_RULE_SETUP
#line 1478 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{
                        yyextra->current->initializer << yytext;
                      }
	YY_BREAK
case 209:
YY_RULE_SETUP
#line 1481 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ // normal comment
                        yyextra->current->initializer << yytext;
                      }
	YY_BREAK
case 210:
YY_RULE_SETUP
#line 1484 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{
                        yyextra->current->initializer << *yytext;
                      }
	YY_BREAK
case 211:
/* rule 211 can match eol */
YY_RULE_SETUP
#line 1487 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{
                        yyextra->current->initializer << *yytext;
                        incLineNr(yyscanner);
                      }
	YY_BREAK


case 212:
/* rule 212 can match eol */
YY_RULE_SETUP
#line 1495 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{
                        incLineNr(yyscanner);
                        if (!stripWhiteSpace(yyextra->current->initializer.str()).empty())
                        {
                          newVariable(yyscanner);
                        }
                        BEGIN(Search);
                      }
	YY_BREAK
case 213:
YY_RULE_SETUP
#line 1503 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{
                        unput(*yytext);
                        newVariable(yyscanner);
                        BEGIN(Search);
                      }
	YY_BREAK
case YY_STATE_EOF(VariableEnd):
#line 1508 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ yyterminate();
                      }
	YY_BREAK


case 214:
#line 1514 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
case 215:
YY_RULE_SETUP
#line 1514 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{
                          // printf("Expected module block %d special=%d\n",yyextra->expectModuleDocs,yyextra->specialBlock);
                          if (yyextra->doubleQuote==(yytext[0]=='"'))
                          {
                            if (yyextra->specialBlock) // expecting a docstring
                            {
                              DString actualDoc=yyextra->docBlock;
                              if (!yyextra->docBlockSpecial) // legacy unformatted docstring
                              {
                                if (!actualDoc.empty())
                                {
                                  stripIndentationVerbatim(actualDoc,yyextra->commentIndent);
                                  actualDoc.prepend("@iverbatim\n");
                                  actualDoc.append("@endiverbatim ");
                                }
                              }
                              //printf("-------> yyextra->current=%p yyextra->bodyEntry=%p\n",yyextra->current,yyextra->bodyEntry);
                              handleCommentBlock(yyscanner, actualDoc, false);
                            }
                            else if (yyextra->packageCommentAllowed) // expecting module docs
                            {
                              DString actualDoc=yyextra->docBlock;
                              if (!yyextra->docBlockSpecial) // legacy unformatted docstring
                              {
                                if (!actualDoc.empty())
                                {
                                  stripIndentationVerbatim(actualDoc,yyextra->commentIndent);
                                  actualDoc.prepend("@iverbatim\n");
                                  actualDoc.append("@endiverbatim ");
                                }
                              }
                              if (yyextra->moduleScope.startsWith("__") &&  yyextra->moduleScope.endsWith("__"))
                              {
                                actualDoc.prepend("\\namespace \\"+yyextra->moduleScope+" ");
                              }
                              else
                              {
                                actualDoc.prepend("\\namespace "+yyextra->moduleScope+" ");
                              }
                              handleCommentBlock(yyscanner, actualDoc, false);
                            }
                            if ((yyextra->docBlockContext==ClassBody /*&& !yyextra->hideClassDocs*/) ||
                                yyextra->docBlockContext==FunctionBody)
                            {
                              yyextra->current->program << yyextra->docBlock;
                              yyextra->current->program << yytext;
                            }
                            //if (yyextra->hideClassDocs)
                            //{
                            //  yyextra->current->startLine = yyextra->yyLineNr;
                            //}
                            //yyextra->hideClassDocs=false;
                            BEGIN(yyextra->docBlockContext);
                          }
                          else
                          {
                            yyextra->docBlock += yytext;
                          }
                          yyextra->packageCommentAllowed = false;
                        }
	YY_BREAK
case 216:
YY_RULE_SETUP
#line 1576 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ // leading whitespace, compensate for """! / '''!
                          if (yyextra->firstPass && yyextra->docBlockSpecial && yyleng >= yyextra->curIndent)
                            yyextra->docBlock += yytext + yyextra->curIndent;
                          else
                            yyextra->docBlock += yytext;
                        }
	YY_BREAK
case 217:
YY_RULE_SETUP
#line 1582 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{
                          yyextra->docBlock += yytext;
                        }
	YY_BREAK
case 218:
/* rule 218 can match eol */
YY_RULE_SETUP
#line 1585 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{
                          incLineNr(yyscanner);
                          yyextra->docBlock += yytext;
                        }
	YY_BREAK
case 219:
YY_RULE_SETUP
#line 1589 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{
                                    yyextra->fileName = &yytext[6];
                                    yyextra->fileName = yyextra->fileName.stripWhiteSpace();
                                    yyextra->fileName = yyextra->fileName.mid(1,yyextra->fileName.length()-2);
                                    yyextra->docBlock+=yytext;
                                  }
	YY_BREAK
case 220:
YY_RULE_SETUP
#line 1595 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{
                                    yyextra->fileName = &yytext[6];
                                    yyextra->fileName = yyextra->fileName.stripWhiteSpace();
                                    yyextra->docBlock+=yytext;
                                  }
	YY_BREAK
case 221:
/* rule 221 can match eol */
*yy_cp = yyg->yy_hold_char; /* undo effects of setting up yytext */
YY_LINENO_REWIND_TO(yy_cp - 1);
yyg->yy_c_buf_p = yy_cp -= 1;
YY_DO_BEFORE_ACTION; /* set up yytext again */
#line 1601 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
case 222:
/* rule 222 can match eol */
YY_RULE_SETUP
#line 1601 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{
                                    bool ok = false;
                                    int nr = DString(&yytext[6]).toInt(&ok);
                                    if (!ok)
                                    {
                                      warn(yyextra->fileName,yyextra->yyLineNr,"Invalid line number '{}' for iline command",yytext);
                                    }
                                    else
                                    {
                                      yyextra->yyLineNr = nr;
                                    }
                                    yyextra->docBlock+=yytext;
                                  }
	YY_BREAK
case 223:
/* rule 223 can match eol */
*yy_cp = yyg->yy_hold_char; /* undo effects of setting up yytext */
YY_LINENO_REWIND_TO(yy_cp - 1);
yyg->yy_c_buf_p = yy_cp -= 1;
YY_DO_BEFORE_ACTION; /* set up yytext again */
YY_RULE_SETUP
#line 1614 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ // escaped command
                                     yyextra->docBlock+=yytext;
                                  }
	YY_BREAK
case 224:
YY_RULE_SETUP
#line 1617 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ // escaped char TO be extended
                          yyextra->docBlock += yytext;
                        }
	YY_BREAK
case 225:
YY_RULE_SETUP
#line 1620 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{
                          yyextra->docBlock += yytext;
                        }
	YY_BREAK


case 226:
YY_RULE_SETUP
#line 1626 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ // skip leading hashes
                        }
	YY_BREAK
case 227:
/* rule 227 can match eol */
*yy_cp = yyg->yy_hold_char; /* undo effects of setting up yytext */
YY_LINENO_REWIND_TO(yy_bp + 1);
yyg->yy_c_buf_p = yy_cp = yy_bp + 1;
YY_DO_BEFORE_ACTION; /* set up yytext again */
YY_RULE_SETUP
#line 1628 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ // continuation of the comment on the next line
                          yyextra->docBlock+='\n';
                          yyextra->docBrief = false;
                          incLineNr(yyscanner);
                        }
	YY_BREAK
case 228:
YY_RULE_SETUP
#line 1633 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{
                                    yyextra->fileName = &yytext[6];
                                    yyextra->fileName = yyextra->fileName.stripWhiteSpace();
                                    yyextra->fileName = yyextra->fileName.mid(1,yyextra->fileName.length()-2);
                                    yyextra->docBlock+=yytext;
                                  }
	YY_BREAK
case 229:
YY_RULE_SETUP
#line 1639 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{
                                    yyextra->fileName = &yytext[6];
                                    yyextra->fileName = yyextra->fileName.stripWhiteSpace();
                                    yyextra->docBlock+=yytext;
                                  }
	YY_BREAK
case 230:
/* rule 230 can match eol */
*yy_cp = yyg->yy_hold_char; /* undo effects of setting up yytext */
YY_LINENO_REWIND_TO(yy_cp - 1);
yyg->yy_c_buf_p = yy_cp -= 1;
YY_DO_BEFORE_ACTION; /* set up yytext again */
#line 1645 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
case 231:
/* rule 231 can match eol */
YY_RULE_SETUP
#line 1645 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{
                                    bool ok = false;
                                    int nr = DString(&yytext[6]).toInt(&ok);
                                    if (!ok)
                                    {
                                      warn(yyextra->fileName,yyextra->yyLineNr,"Invalid line number '{}' for iline command",yytext);
                                    }
                                    else
                                    {
                                      yyextra->yyLineNr = nr;
                                    }
                                    yyextra->docBlock+=yytext;
                                  }
	YY_BREAK
case 232:
/* rule 232 can match eol */
*yy_cp = yyg->yy_hold_char; /* undo effects of setting up yytext */
YY_LINENO_REWIND_TO(yy_cp - 1);
yyg->yy_c_buf_p = yy_cp -= 1;
YY_DO_BEFORE_ACTION; /* set up yytext again */
YY_RULE_SETUP
#line 1658 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ // escaped command
                                     yyextra->docBlock+=yytext;
                                  }
	YY_BREAK
case 233:
YY_RULE_SETUP
#line 1661 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{
                          DString indent;
                          int extraSpaces = std::max(0,static_cast<int>(yyleng-9-yyextra->curIndent-2));
                          indent.fill(' ',extraSpaces);
                          //printf("extraSpaces=%d\n",extraSpaces);
                          yyextra->docBlock += "\\ilinebr ";
                          yyextra->docBlock += indent;
                        }
	YY_BREAK
case 234:
YY_RULE_SETUP
#line 1669 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ // any other stuff
                          yyextra->docBlock+=yytext;
                        }
	YY_BREAK
case 235:
/* rule 235 can match eol */
YY_RULE_SETUP
#line 1672 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ // new line that ends the comment
                          handleCommentBlock(yyscanner, yyextra->docBlock, yyextra->docBrief);
                          if (yyextra->docBlockContext == VariableEnd)
                          {
                            unput(*yytext);
                          }
                          else
                          {
                            incLineNr(yyscanner);
                          }
                          BEGIN(yyextra->docBlockContext);
                        }
	YY_BREAK
case 236:
YY_RULE_SETUP
#line 1684 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ // anything we missed
                          yyextra->docBlock+=*yytext;
                        }
	YY_BREAK


case 237:
/* rule 237 can match eol */
YY_RULE_SETUP
#line 1690 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ // line continuation
                                 addToString(yyscanner,yytext);
                                 incLineNr(yyscanner);
                               }
	YY_BREAK
case 238:
YY_RULE_SETUP
#line 1694 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ // escaped char
                                 addToString(yyscanner,yytext);
                               }
	YY_BREAK
case 239:
YY_RULE_SETUP
#line 1697 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ // triple double quotes
                                 addToString(yyscanner,yytext);
                               }
	YY_BREAK
case 240:
YY_RULE_SETUP
#line 1700 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ // end of the string
                                 addToString(yyscanner,yytext);
                                 BEGIN(yyextra->stringContext);
                               }
	YY_BREAK
case 241:
YY_RULE_SETUP
#line 1704 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ // normal chars
                                 addToString(yyscanner,yytext);
                               }
	YY_BREAK
case 242:
YY_RULE_SETUP
#line 1707 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ // normal char
                                 addToString(yyscanner,yytext);
                               }
	YY_BREAK


case 243:
/* rule 243 can match eol */
YY_RULE_SETUP
#line 1713 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ // line continuation
                                 addToString(yyscanner,yytext);
                                 incLineNr(yyscanner);
                               }
	YY_BREAK
case 244:
YY_RULE_SETUP
#line 1717 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ // escaped char
                                 addToString(yyscanner,yytext);
                               }
	YY_BREAK
case 245:
YY_RULE_SETUP
#line 1720 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ // triple single quotes
                                 addToString(yyscanner,yytext);
                               }
	YY_BREAK
case 246:
YY_RULE_SETUP
#line 1723 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ // end of the string
                                 addToString(yyscanner,yytext);
                                 BEGIN(yyextra->stringContext);
                               }
	YY_BREAK
case 247:
YY_RULE_SETUP
#line 1727 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ // normal chars
                                 addToString(yyscanner,yytext);
                               }
	YY_BREAK
case 248:
YY_RULE_SETUP
#line 1730 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ // normal char
                                 addToString(yyscanner,yytext);
                               }
	YY_BREAK


case 249:
#line 1737 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
case 250:
YY_RULE_SETUP
#line 1737 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{
                          *yyextra->copyString << yytext;
                          if (yyextra->doubleQuote==(yytext[0]=='"'))
                          {
                            BEGIN(yyextra->stringContext);
                          }
                        }
	YY_BREAK
case 251:
/* rule 251 can match eol */
YY_RULE_SETUP
#line 1746 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{
                          lineCount(yyscanner);
                          *yyextra->copyString << yytext;
                        }
	YY_BREAK
case 252:
/* rule 252 can match eol */
YY_RULE_SETUP
#line 1750 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{
                          incLineNr(yyscanner);
                          *yyextra->copyString << yytext;
                        }
	YY_BREAK
case 253:
YY_RULE_SETUP
#line 1754 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{
                          *yyextra->copyString << *yytext;
                        }
	YY_BREAK


case 254:
YY_RULE_SETUP
#line 1760 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ // start of a comment block
                          yyextra->doubleQuote=true;
                          yyextra->decoratorCommentStr.str(std::string());
                          yyextra->copyString=&yyextra->decoratorCommentStr;
                          yyextra->stringContext=YY_START;
                          BEGIN(TripleString);
                        }
	YY_BREAK
case 255:
YY_RULE_SETUP
#line 1768 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ // start of a comment block
                          yyextra->doubleQuote=false;
                          yyextra->decoratorCommentStr.str(std::string());
                          yyextra->copyString=&yyextra->decoratorCommentStr;
                          yyextra->stringContext=YY_START;
                          BEGIN(TripleString);
                        }
	YY_BREAK
case 256:
YY_RULE_SETUP
#line 1775 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{
                          yyextra->stringContext=YY_START;
                          yyextra->decoratorCommentStr.str(std::string());
                          yyextra->copyString=&yyextra->decoratorCommentStr;
                          BEGIN( SingleQuoteString );
                        }
	YY_BREAK
case 257:
YY_RULE_SETUP
#line 1781 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{
                          yyextra->stringContext=YY_START;
                          yyextra->decoratorCommentStr.str(std::string());
                          yyextra->copyString=&yyextra->decoratorCommentStr;
                          BEGIN( DoubleQuoteString );
                        }
	YY_BREAK
case 258:
YY_RULE_SETUP
#line 1787 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{
                          yyextra->decoratorRound++;
                        }
	YY_BREAK
case 259:
YY_RULE_SETUP
#line 1790 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{
                          yyextra->decoratorRound--;
                          if (!yyextra->decoratorRound) BEGIN( Search );
                        }
	YY_BREAK
case 260:
/* rule 260 can match eol */
YY_RULE_SETUP
#line 1794 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{
                          incLineNr(yyscanner);
                        }
	YY_BREAK
case 261:
YY_RULE_SETUP
#line 1797 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{ }
	YY_BREAK

/* ------------ End rules -------------- */
/*
<*>({NONEMPTY}|{EXPCHAR}|{BB})           { // This should go one character at a time.
                                 // printf("[pyscanner] '%s' [ state %d ]  [line %d] no match\n",
                                 //       yytext, YY_START, yyextra->yyLineNr);

                               }
  */
case 262:
/* rule 262 can match eol */
YY_RULE_SETUP
#line 1810 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{
                                 //printf("[pyscanner] %d NEWLINE [line %d] no match\n",
                                 //       YY_START, yyextra->yyLineNr);

                                 lineCount(yyscanner);
                               }
	YY_BREAK
case 263:
YY_RULE_SETUP
#line 1817 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{
                                 //fprintf(stderr,"Quote: %d\n",YY_START);
                               }
	YY_BREAK
case 264:
YY_RULE_SETUP
#line 1821 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
{
                                 //printf("[pyscanner] '%s' [ state %d ]  [line %d] no match\n",
                                 //       yytext, YY_START, yyextra->yyLineNr);

                               }
	YY_BREAK
case 265:
YY_RULE_SETUP
#line 1828 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"
ECHO;
	YY_BREAK
#line 5664 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner_intermediate.cpp"
			case YY_STATE_EOF(INITIAL):
			case YY_STATE_EOF(Search):
			case YY_STATE_EOF(SearchMemVars):
			case YY_STATE_EOF(TripleComment):
			case YY_STATE_EOF(SpecialComment):
			case YY_STATE_EOF(FunctionDec):
			case YY_STATE_EOF(FunctionParams):
			case YY_STATE_EOF(FunctionAnnotation):
			case YY_STATE_EOF(FunctionTypeAnnotation):
			case YY_STATE_EOF(FunctionParamDefVal):
			case YY_STATE_EOF(ClassDec):
			case YY_STATE_EOF(ClassInheritance):
			case YY_STATE_EOF(ClassCaptureIndent):
			case YY_STATE_EOF(VariableDec):
			case YY_STATE_EOF(VariableAtom):
			case YY_STATE_EOF(SingleQuoteString):
			case YY_STATE_EOF(DoubleQuoteString):
			case YY_STATE_EOF(TripleString):
			case YY_STATE_EOF(SingleQuoteStringIgnore):
			case YY_STATE_EOF(DoubleQuoteStringIgnore):
			case YY_STATE_EOF(FromMod):
			case YY_STATE_EOF(FromModItem):
			case YY_STATE_EOF(Import):
			case YY_STATE_EOF(TypeHint):
			case YY_STATE_EOF(Decorator):
			case YY_STATE_EOF(Generics):
			case YY_STATE_EOF(TypeAlias):
				yyterminate();

	case YY_END_OF_BUFFER:
		{
		/* Amount of text matched not including the EOB char. */
		int yy_amount_of_matched_text = (int) (yy_cp - yyg->yytext_ptr) - 1;

		/* Undo the effects of YY_DO_BEFORE_ACTION. */
		*yy_cp = yyg->yy_hold_char;
		YY_RESTORE_YY_MORE_OFFSET

		if ( YY_CURRENT_BUFFER_LVALUE->yy_buffer_status == YY_BUFFER_NEW )
			{
			/* We're scanning a new file or input source.  It's
			 * possible that this happened because the user
			 * just pointed yyin at a new source and called
			 * yylex().  If so, then we have to assure
			 * consistency between YY_CURRENT_BUFFER and our
			 * globals.  Here is the right place to do so, because
			 * this is the first action (other than possibly a
			 * back-up) that will match for the new input source.
			 */
			yyg->yy_n_chars = YY_CURRENT_BUFFER_LVALUE->yy_n_chars;
			YY_CURRENT_BUFFER_LVALUE->yy_input_file = yyin;
			YY_CURRENT_BUFFER_LVALUE->yy_buffer_status = YY_BUFFER_NORMAL;
			}

		/* Note that here we test for yy_c_buf_p "<=" to the position
		 * of the first EOB in the buffer, since yy_c_buf_p will
		 * already have been incremented past the NUL character
		 * (since all states make transitions on EOB to the
		 * end-of-buffer state).  Contrast this with the test
		 * in input().
		 */
		if ( yyg->yy_c_buf_p <= &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[yyg->yy_n_chars] )
			{ /* This was really a NUL. */
			yy_state_type yy_next_state;

			yyg->yy_c_buf_p = yyg->yytext_ptr + yy_amount_of_matched_text;

			yy_current_state = yy_get_previous_state( yyscanner );

			/* Okay, we're now positioned to make the NUL
			 * transition.  We couldn't have
			 * yy_get_previous_state() go ahead and do it
			 * for us because it doesn't know how to deal
			 * with the possibility of jamming (and we don't
			 * want to build jamming into it because then it
			 * will run more slowly).
			 */

			yy_next_state = yy_try_NUL_trans( yy_current_state , yyscanner);

			yy_bp = yyg->yytext_ptr + YY_MORE_ADJ;

			if ( yy_next_state )
				{
				/* Consume the NUL. */
				yy_cp = ++yyg->yy_c_buf_p;
				yy_current_state = yy_next_state;
				goto yy_match;
				}

			else
				{
				yy_cp = yyg->yy_c_buf_p;
				goto yy_find_action;
				}
			}

		else switch ( yy_get_next_buffer( yyscanner ) )
			{
			case EOB_ACT_END_OF_FILE:
				{
				yyg->yy_did_buffer_switch_on_eof = 0;

				if ( yywrap( yyscanner ) )
					{
					/* Note: because we've taken care in
					 * yy_get_next_buffer() to have set up
					 * yytext, we can now set up
					 * yy_c_buf_p so that if some total
					 * hoser (like flex itself) wants to
					 * call the scanner after we return the
					 * YY_NULL, it'll still work - another
					 * YY_NULL will get returned.
					 */
					yyg->yy_c_buf_p = yyg->yytext_ptr + YY_MORE_ADJ;

					yy_act = YY_STATE_EOF(YY_START);
					goto do_action;
					}

				else
					{
					if ( ! yyg->yy_did_buffer_switch_on_eof )
						YY_NEW_FILE;
					}
				break;
				}

			case EOB_ACT_CONTINUE_SCAN:
				yyg->yy_c_buf_p =
					yyg->yytext_ptr + yy_amount_of_matched_text;

				yy_current_state = yy_get_previous_state( yyscanner );

				yy_cp = yyg->yy_c_buf_p;
				yy_bp = yyg->yytext_ptr + YY_MORE_ADJ;
				goto yy_match;

			case EOB_ACT_LAST_MATCH:
				yyg->yy_c_buf_p =
				&YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[yyg->yy_n_chars];

				yy_current_state = yy_get_previous_state( yyscanner );

				yy_cp = yyg->yy_c_buf_p;
				yy_bp = yyg->yytext_ptr + YY_MORE_ADJ;
				goto yy_find_action;
			}
		break;
		}

	default:
		YY_FATAL_ERROR(
			"fatal flex scanner internal error--no action found" );
	} /* end of action switch */
		} /* end of scanning one token */
	} /* end of user's declarations */
} /* end of yylex */

/* yy_get_next_buffer - try to read in a new buffer
 *
 * Returns a code representing an action:
 *	EOB_ACT_LAST_MATCH -
 *	EOB_ACT_CONTINUE_SCAN - continue scanning from current position
 *	EOB_ACT_END_OF_FILE - end of file
 */
static int yy_get_next_buffer (yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
	char *dest = YY_CURRENT_BUFFER_LVALUE->yy_ch_buf;
	char *source = yyg->yytext_ptr;
	int number_to_move, i;
	int ret_val;

	if ( yyg->yy_c_buf_p > &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[yyg->yy_n_chars + 1] )
		YY_FATAL_ERROR(
		"fatal flex scanner internal error--end of buffer missed" );

	if ( YY_CURRENT_BUFFER_LVALUE->yy_fill_buffer == 0 )
		{ /* Don't try to fill the buffer, so this is an EOF. */
		if ( yyg->yy_c_buf_p - yyg->yytext_ptr - YY_MORE_ADJ == 1 )
			{
			/* We matched a single character, the EOB, so
			 * treat this as a final EOF.
			 */
			return EOB_ACT_END_OF_FILE;
			}

		else
			{
			/* We matched some text prior to the EOB, first
			 * process it.
			 */
			return EOB_ACT_LAST_MATCH;
			}
		}

	/* Try to read more data. */

	/* First move last chars to start of buffer. */
	number_to_move = (int) (yyg->yy_c_buf_p - yyg->yytext_ptr - 1);

	for ( i = 0; i < number_to_move; ++i )
		*(dest++) = *(source++);

	if ( YY_CURRENT_BUFFER_LVALUE->yy_buffer_status == YY_BUFFER_EOF_PENDING )
		/* don't do the read, it's not guaranteed to return an EOF,
		 * just force an EOF
		 */
		YY_CURRENT_BUFFER_LVALUE->yy_n_chars = yyg->yy_n_chars = 0;

	else
		{
			int num_to_read =
			YY_CURRENT_BUFFER_LVALUE->yy_buf_size - number_to_move - 1;

		while ( num_to_read <= 0 )
			{ /* Not enough room in the buffer - grow it. */

			YY_FATAL_ERROR(
"input buffer overflow, can't enlarge buffer because scanner uses REJECT" );

			}

		if ( num_to_read > YY_READ_BUF_SIZE )
			num_to_read = YY_READ_BUF_SIZE;

		/* Read in more data. */
		YY_INPUT( (&YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[number_to_move]),
			yyg->yy_n_chars, num_to_read );

		YY_CURRENT_BUFFER_LVALUE->yy_n_chars = yyg->yy_n_chars;
		}

	if ( yyg->yy_n_chars == 0 )
		{
		if ( number_to_move == YY_MORE_ADJ )
			{
			ret_val = EOB_ACT_END_OF_FILE;
			yyrestart( yyin  , yyscanner);
			}

		else
			{
			ret_val = EOB_ACT_LAST_MATCH;
			YY_CURRENT_BUFFER_LVALUE->yy_buffer_status =
				YY_BUFFER_EOF_PENDING;
			}
		}

	else
		ret_val = EOB_ACT_CONTINUE_SCAN;

	if ((yyg->yy_n_chars + number_to_move) > YY_CURRENT_BUFFER_LVALUE->yy_buf_size) {
		/* Extend the array by 50%, plus the number we really need. */
		int new_size = yyg->yy_n_chars + number_to_move + (yyg->yy_n_chars >> 1);
		YY_CURRENT_BUFFER_LVALUE->yy_ch_buf = (char *) yyrealloc(
			(void *) YY_CURRENT_BUFFER_LVALUE->yy_ch_buf, (yy_size_t) new_size , yyscanner );
		if ( ! YY_CURRENT_BUFFER_LVALUE->yy_ch_buf )
			YY_FATAL_ERROR( "out of dynamic memory in yy_get_next_buffer()" );
		/* "- 2" to take care of EOB's */
		YY_CURRENT_BUFFER_LVALUE->yy_buf_size = (int) (new_size - 2);
	}

	yyg->yy_n_chars += number_to_move;
	YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[yyg->yy_n_chars] = YY_END_OF_BUFFER_CHAR;
	YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[yyg->yy_n_chars + 1] = YY_END_OF_BUFFER_CHAR;

	yyg->yytext_ptr = &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[0];

	return ret_val;
}

/* yy_get_previous_state - get the state just before the EOB char was reached */

    static yy_state_type yy_get_previous_state (yyscan_t yyscanner)
{
	yy_state_type yy_current_state;
	char *yy_cp;
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;

	yy_current_state = yyg->yy_start;
	yy_current_state += YY_AT_BOL();

	yyg->yy_state_ptr = yyg->yy_state_buf;
	*yyg->yy_state_ptr++ = yy_current_state;

	for ( yy_cp = yyg->yytext_ptr + YY_MORE_ADJ; yy_cp < yyg->yy_c_buf_p; ++yy_cp )
		{
		YY_CHAR yy_c = (*yy_cp ? yy_ec[YY_SC_TO_UI(*yy_cp)] : 1);
		while ( yy_chk[yy_base[yy_current_state] + yy_c] != yy_current_state )
			{
			yy_current_state = (int) yy_def[yy_current_state];
			if ( yy_current_state >= 818 )
				yy_c = yy_meta[yy_c];
			}
		yy_current_state = yy_nxt[yy_base[yy_current_state] + yy_c];
		*yyg->yy_state_ptr++ = yy_current_state;
		}

	return yy_current_state;
}

/* yy_try_NUL_trans - try to make a transition on the NUL character
 *
 * synopsis
 *	next_state = yy_try_NUL_trans( current_state );
 */
    static yy_state_type yy_try_NUL_trans  (yy_state_type yy_current_state , yyscan_t yyscanner)
{
	int yy_is_jam;
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner; /* This var may be unused depending upon options. */

	YY_CHAR yy_c = 1;
	while ( yy_chk[yy_base[yy_current_state] + yy_c] != yy_current_state )
		{
		yy_current_state = (int) yy_def[yy_current_state];
		if ( yy_current_state >= 818 )
			yy_c = yy_meta[yy_c];
		}
	yy_current_state = yy_nxt[yy_base[yy_current_state] + yy_c];
	yy_is_jam = (yy_current_state == 817);
	if ( ! yy_is_jam )
		*yyg->yy_state_ptr++ = yy_current_state;

	(void)yyg;
	return yy_is_jam ? 0 : yy_current_state;
}

#ifndef YY_NO_UNPUT

    static void yyunput (int c, char * yy_bp , yyscan_t yyscanner)
{
	char *yy_cp;
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;

    yy_cp = yyg->yy_c_buf_p;

	/* undo effects of setting up yytext */
	*yy_cp = yyg->yy_hold_char;

	if ( yy_cp < YY_CURRENT_BUFFER_LVALUE->yy_ch_buf + 2 )
		{ /* need to shift things up to make room */
		/* +2 for EOB chars. */
		int number_to_move = yyg->yy_n_chars + 2;
		char *dest = &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[
					YY_CURRENT_BUFFER_LVALUE->yy_buf_size + 2];
		char *source =
				&YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[number_to_move];

		while ( source > YY_CURRENT_BUFFER_LVALUE->yy_ch_buf )
			*--dest = *--source;

		yy_cp += (int) (dest - source);
		yy_bp += (int) (dest - source);
		YY_CURRENT_BUFFER_LVALUE->yy_n_chars =
			yyg->yy_n_chars = (int) YY_CURRENT_BUFFER_LVALUE->yy_buf_size;

		if ( yy_cp < YY_CURRENT_BUFFER_LVALUE->yy_ch_buf + 2 )
			YY_FATAL_ERROR( "flex scanner push-back overflow" );
		}

	*--yy_cp = (char) c;

	yyg->yytext_ptr = yy_bp;
	yyg->yy_hold_char = *yy_cp;
	yyg->yy_c_buf_p = yy_cp;
}

#endif

#ifndef YY_NO_INPUT
#ifdef __cplusplus
    static int yyinput (yyscan_t yyscanner)
#else
    static int input  (yyscan_t yyscanner)
#endif

{
	int c;
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;

	*yyg->yy_c_buf_p = yyg->yy_hold_char;

	if ( *yyg->yy_c_buf_p == YY_END_OF_BUFFER_CHAR )
		{
		/* yy_c_buf_p now points to the character we want to return.
		 * If this occurs *before* the EOB characters, then it's a
		 * valid NUL; if not, then we've hit the end of the buffer.
		 */
		if ( yyg->yy_c_buf_p < &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[yyg->yy_n_chars] )
			/* This was really a NUL. */
			*yyg->yy_c_buf_p = '\0';

		else
			{ /* need more input */
			int offset = (int) (yyg->yy_c_buf_p - yyg->yytext_ptr);
			++yyg->yy_c_buf_p;

			switch ( yy_get_next_buffer( yyscanner ) )
				{
				case EOB_ACT_LAST_MATCH:
					/* This happens because yy_g_n_b()
					 * sees that we've accumulated a
					 * token and flags that we need to
					 * try matching the token before
					 * proceeding.  But for input(),
					 * there's no matching to consider.
					 * So convert the EOB_ACT_LAST_MATCH
					 * to EOB_ACT_END_OF_FILE.
					 */

					/* Reset buffer status. */
					yyrestart( yyin , yyscanner);

					/*FALLTHROUGH*/

				case EOB_ACT_END_OF_FILE:
					{
					if ( yywrap( yyscanner ) )
						return 0;

					if ( ! yyg->yy_did_buffer_switch_on_eof )
						YY_NEW_FILE;
#ifdef __cplusplus
					return yyinput(yyscanner);
#else
					return input(yyscanner);
#endif
					}

				case EOB_ACT_CONTINUE_SCAN:
					yyg->yy_c_buf_p = yyg->yytext_ptr + offset;
					break;
				}
			}
		}

	c = *(unsigned char *) yyg->yy_c_buf_p;	/* cast for 8-bit char's */
	*yyg->yy_c_buf_p = '\0';	/* preserve yytext */
	yyg->yy_hold_char = *++yyg->yy_c_buf_p;

	YY_CURRENT_BUFFER_LVALUE->yy_at_bol = (c == '\n');

	return c;
}
#endif	/* ifndef YY_NO_INPUT */

/** Immediately switch to a different input stream.
 * @param input_file A readable stream.
 * @param yyscanner The scanner object.
 * @note This function does not reset the start condition to @c INITIAL .
 */
    void yyrestart  (FILE * input_file , yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;

	if ( ! YY_CURRENT_BUFFER ){
        yyensure_buffer_stack (yyscanner);
		YY_CURRENT_BUFFER_LVALUE =
            yy_create_buffer( yyin, YY_BUF_SIZE , yyscanner);
	}

	yy_init_buffer( YY_CURRENT_BUFFER, input_file , yyscanner);
	yy_load_buffer_state( yyscanner );
}

/** Switch to a different input buffer.
 * @param new_buffer The new input buffer.
 * @param yyscanner The scanner object.
 */
    void yy_switch_to_buffer  (YY_BUFFER_STATE  new_buffer , yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;

	/* TODO. We should be able to replace this entire function body
	 * with
	 *		yypop_buffer_state();
	 *		yypush_buffer_state(new_buffer);
     */
	yyensure_buffer_stack (yyscanner);
	if ( YY_CURRENT_BUFFER == new_buffer )
		return;

	if ( YY_CURRENT_BUFFER )
		{
		/* Flush out information for old buffer. */
		*yyg->yy_c_buf_p = yyg->yy_hold_char;
		YY_CURRENT_BUFFER_LVALUE->yy_buf_pos = yyg->yy_c_buf_p;
		YY_CURRENT_BUFFER_LVALUE->yy_n_chars = yyg->yy_n_chars;
		}

	YY_CURRENT_BUFFER_LVALUE = new_buffer;
	yy_load_buffer_state( yyscanner );

	/* We don't actually know whether we did this switch during
	 * EOF (yywrap()) processing, but the only time this flag
	 * is looked at is after yywrap() is called, so it's safe
	 * to go ahead and always set it.
	 */
	yyg->yy_did_buffer_switch_on_eof = 1;
}

static void yy_load_buffer_state  (yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
	yyg->yy_n_chars = YY_CURRENT_BUFFER_LVALUE->yy_n_chars;
	yyg->yytext_ptr = yyg->yy_c_buf_p = YY_CURRENT_BUFFER_LVALUE->yy_buf_pos;
	yyin = YY_CURRENT_BUFFER_LVALUE->yy_input_file;
	yyg->yy_hold_char = *yyg->yy_c_buf_p;
}

/** Allocate and initialize an input buffer state.
 * @param file A readable stream.
 * @param size The character buffer size in bytes. When in doubt, use @c YY_BUF_SIZE.
 * @param yyscanner The scanner object.
 * @return the allocated buffer state.
 */
    YY_BUFFER_STATE yy_create_buffer  (FILE * file, int  size , yyscan_t yyscanner)
{
	YY_BUFFER_STATE b;
    
	b = (YY_BUFFER_STATE) yyalloc( sizeof( struct yy_buffer_state ) , yyscanner );
	if ( ! b )
		YY_FATAL_ERROR( "out of dynamic memory in yy_create_buffer()" );

	b->yy_buf_size = size;

	/* yy_ch_buf has to be 2 characters longer than the size given because
	 * we need to put in 2 end-of-buffer characters.
	 */
	b->yy_ch_buf = (char *) yyalloc( (yy_size_t) (b->yy_buf_size + 2) , yyscanner );
	if ( ! b->yy_ch_buf )
		YY_FATAL_ERROR( "out of dynamic memory in yy_create_buffer()" );

	b->yy_is_our_buffer = 1;

	yy_init_buffer( b, file , yyscanner);

	return b;
}

/** Destroy the buffer.
 * @param b a buffer created with yy_create_buffer()
 * @param yyscanner The scanner object.
 */
    void yy_delete_buffer (YY_BUFFER_STATE  b , yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;

	if ( ! b )
		return;

	if ( b == YY_CURRENT_BUFFER ) /* Not sure if we should pop here. */
		YY_CURRENT_BUFFER_LVALUE = (YY_BUFFER_STATE) 0;

	if ( b->yy_is_our_buffer )
		yyfree( (void *) b->yy_ch_buf , yyscanner );

	yyfree( (void *) b , yyscanner );
}

/* Initializes or reinitializes a buffer.
 * This function is sometimes called more than once on the same buffer,
 * such as during a yyrestart() or at EOF.
 */
    static void yy_init_buffer  (YY_BUFFER_STATE  b, FILE * file , yyscan_t yyscanner)

{
	int oerrno = errno;
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;

	yy_flush_buffer( b , yyscanner);

	b->yy_input_file = file;
	b->yy_fill_buffer = 1;

    /* If b is the current buffer, then yy_init_buffer was _probably_
     * called from yyrestart() or through yy_get_next_buffer.
     * In that case, we don't want to reset the lineno or column.
     */
    if (b != YY_CURRENT_BUFFER){
        b->yy_bs_lineno = 1;
        b->yy_bs_column = 0;
    }

        b->yy_is_interactive = 0;
    
	errno = oerrno;
}

/** Discard all buffered characters. On the next scan, YY_INPUT will be called.
 * @param b the buffer state to be flushed, usually @c YY_CURRENT_BUFFER.
 * @param yyscanner The scanner object.
 */
    void yy_flush_buffer (YY_BUFFER_STATE  b , yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
	if ( ! b )
		return;

	b->yy_n_chars = 0;

	/* We always need two end-of-buffer characters.  The first causes
	 * a transition to the end-of-buffer state.  The second causes
	 * a jam in that state.
	 */
	b->yy_ch_buf[0] = YY_END_OF_BUFFER_CHAR;
	b->yy_ch_buf[1] = YY_END_OF_BUFFER_CHAR;

	b->yy_buf_pos = &b->yy_ch_buf[0];

	b->yy_at_bol = 1;
	b->yy_buffer_status = YY_BUFFER_NEW;

	if ( b == YY_CURRENT_BUFFER )
		yy_load_buffer_state( yyscanner );
}

/** Pushes the new state onto the stack. The new state becomes
 *  the current state. This function will allocate the stack
 *  if necessary.
 *  @param new_buffer The new state.
 *  @param yyscanner The scanner object.
 */
void yypush_buffer_state (YY_BUFFER_STATE new_buffer , yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
	if (new_buffer == NULL)
		return;

	yyensure_buffer_stack(yyscanner);

	/* This block is copied from yy_switch_to_buffer. */
	if ( YY_CURRENT_BUFFER )
		{
		/* Flush out information for old buffer. */
		*yyg->yy_c_buf_p = yyg->yy_hold_char;
		YY_CURRENT_BUFFER_LVALUE->yy_buf_pos = yyg->yy_c_buf_p;
		YY_CURRENT_BUFFER_LVALUE->yy_n_chars = yyg->yy_n_chars;
		}

	/* Only push if top exists. Otherwise, replace top. */
	if (YY_CURRENT_BUFFER)
		yyg->yy_buffer_stack_top++;
	YY_CURRENT_BUFFER_LVALUE = new_buffer;

	/* copied from yy_switch_to_buffer. */
	yy_load_buffer_state( yyscanner );
	yyg->yy_did_buffer_switch_on_eof = 1;
}

/** Removes and deletes the top of the stack, if present.
 *  The next element becomes the new top.
 *  @param yyscanner The scanner object.
 */
void yypop_buffer_state (yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
	if (!YY_CURRENT_BUFFER)
		return;

	yy_delete_buffer(YY_CURRENT_BUFFER , yyscanner);
	YY_CURRENT_BUFFER_LVALUE = NULL;
	if (yyg->yy_buffer_stack_top > 0)
		--yyg->yy_buffer_stack_top;

	if (YY_CURRENT_BUFFER) {
		yy_load_buffer_state( yyscanner );
		yyg->yy_did_buffer_switch_on_eof = 1;
	}
}

/* Allocates the stack if it does not exist.
 *  Guarantees space for at least one push.
 */
static void yyensure_buffer_stack (yyscan_t yyscanner)
{
	yy_size_t num_to_alloc;
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;

	if (!yyg->yy_buffer_stack) {

		/* First allocation is just for 2 elements, since we don't know if this
		 * scanner will even need a stack. We use 2 instead of 1 to avoid an
		 * immediate realloc on the next call.
         */
      num_to_alloc = 1; /* After all that talk, this was set to 1 anyways... */
		yyg->yy_buffer_stack = (struct yy_buffer_state**)yyalloc
								(num_to_alloc * sizeof(struct yy_buffer_state*)
								, yyscanner);
		if ( ! yyg->yy_buffer_stack )
			YY_FATAL_ERROR( "out of dynamic memory in yyensure_buffer_stack()" );

		memset(yyg->yy_buffer_stack, 0, num_to_alloc * sizeof(struct yy_buffer_state*));

		yyg->yy_buffer_stack_max = num_to_alloc;
		yyg->yy_buffer_stack_top = 0;
		return;
	}

	if (yyg->yy_buffer_stack_top >= (yyg->yy_buffer_stack_max) - 1){

		/* Increase the buffer to prepare for a possible push. */
		yy_size_t grow_size = 8 /* arbitrary grow size */;

		num_to_alloc = yyg->yy_buffer_stack_max + grow_size;
		yyg->yy_buffer_stack = (struct yy_buffer_state**)yyrealloc
								(yyg->yy_buffer_stack,
								num_to_alloc * sizeof(struct yy_buffer_state*)
								, yyscanner);
		if ( ! yyg->yy_buffer_stack )
			YY_FATAL_ERROR( "out of dynamic memory in yyensure_buffer_stack()" );

		/* zero only the new slots.*/
		memset(yyg->yy_buffer_stack + yyg->yy_buffer_stack_max, 0, grow_size * sizeof(struct yy_buffer_state*));
		yyg->yy_buffer_stack_max = num_to_alloc;
	}
}

/** Setup the input buffer state to scan directly from a user-specified character buffer.
 * @param base the character buffer
 * @param size the size in bytes of the character buffer
 * @param yyscanner The scanner object.
 * @return the newly allocated buffer state object.
 */
YY_BUFFER_STATE yy_scan_buffer  (char * base, yy_size_t  size , yyscan_t yyscanner)
{
	YY_BUFFER_STATE b;
    
	if ( size < 2 ||
	     base[size-2] != YY_END_OF_BUFFER_CHAR ||
	     base[size-1] != YY_END_OF_BUFFER_CHAR )
		/* They forgot to leave room for the EOB's. */
		return NULL;

	b = (YY_BUFFER_STATE) yyalloc( sizeof( struct yy_buffer_state ) , yyscanner );
	if ( ! b )
		YY_FATAL_ERROR( "out of dynamic memory in yy_scan_buffer()" );

	b->yy_buf_size = (int) (size - 2);	/* "- 2" to take care of EOB's */
	b->yy_buf_pos = b->yy_ch_buf = base;
	b->yy_is_our_buffer = 0;
	b->yy_input_file = NULL;
	b->yy_n_chars = b->yy_buf_size;
	b->yy_is_interactive = 0;
	b->yy_at_bol = 1;
	b->yy_fill_buffer = 0;
	b->yy_buffer_status = YY_BUFFER_NEW;

	yy_switch_to_buffer( b , yyscanner );

	return b;
}

/** Setup the input buffer state to scan a string. The next call to yylex() will
 * scan from a @e copy of @a str.
 * @param yystr a NUL-terminated string to scan
 * @param yyscanner The scanner object.
 * @return the newly allocated buffer state object.
 * @note If you want to scan bytes that may contain NUL values, then use
 *       yy_scan_bytes() instead.
 */
YY_BUFFER_STATE yy_scan_string (const char * yystr , yyscan_t yyscanner)
{
    
	return yy_scan_bytes( yystr, (int) strlen(yystr) , yyscanner);
}

/** Setup the input buffer state to scan the given bytes. The next call to yylex() will
 * scan from a @e copy of @a bytes.
 * @param yybytes the byte buffer to scan
 * @param _yybytes_len the number of bytes in the buffer pointed to by @a bytes.
 * @param yyscanner The scanner object.
 * @return the newly allocated buffer state object.
 */
YY_BUFFER_STATE yy_scan_bytes  (const char * yybytes, int  _yybytes_len , yyscan_t yyscanner)
{
	YY_BUFFER_STATE b;
	char *buf;
	yy_size_t n;
	int i;
    
	/* Get memory for full buffer, including space for trailing EOB's. */
	n = (yy_size_t) (_yybytes_len + 2);
	buf = (char *) yyalloc( n , yyscanner );
	if ( ! buf )
		YY_FATAL_ERROR( "out of dynamic memory in yy_scan_bytes()" );

	for ( i = 0; i < _yybytes_len; ++i )
		buf[i] = yybytes[i];

	buf[_yybytes_len] = buf[_yybytes_len+1] = YY_END_OF_BUFFER_CHAR;

	b = yy_scan_buffer( buf, n , yyscanner);
	if ( ! b )
		YY_FATAL_ERROR( "bad buffer in yy_scan_bytes()" );

	/* It's okay to grow etc. this buffer, and we should throw it
	 * away when we're done.
	 */
	b->yy_is_our_buffer = 1;

	return b;
}

#ifndef YY_EXIT_FAILURE
#define YY_EXIT_FAILURE 2
#endif

static void yynoreturn yy_fatal_error (const char* msg , yyscan_t yyscanner)
{
	struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
	(void)yyg;
	fprintf( stderr, "%s\n", msg );
	exit( YY_EXIT_FAILURE );
}

/* Redefine yyless() so it works in section 3 code. */

#undef yyless
#define yyless(n) \
	do \
		{ \
		/* Undo effects of setting up yytext. */ \
        int yyless_macro_arg = (n); \
        YY_LESS_LINENO(yyless_macro_arg);\
		yytext[yyleng] = yyg->yy_hold_char; \
		yyg->yy_c_buf_p = yytext + yyless_macro_arg; \
		yyg->yy_hold_char = *yyg->yy_c_buf_p; \
		*yyg->yy_c_buf_p = '\0'; \
		yyleng = yyless_macro_arg; \
		} \
	while ( 0 )

/* Accessor  methods (get/set functions) to struct members. */

/** Get the user-defined data for this scanner.
 * @param yyscanner The scanner object.
 */
YY_EXTRA_TYPE yyget_extra  (yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    return yyextra;
}

/** Get the current line number.
 * @param yyscanner The scanner object.
 */
int yyget_lineno  (yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;

        if (! YY_CURRENT_BUFFER)
            return 0;
    
    return yylineno;
}

/** Get the current column number.
 * @param yyscanner The scanner object.
 */
int yyget_column  (yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;

        if (! YY_CURRENT_BUFFER)
            return 0;
    
    return yycolumn;
}

/** Get the input stream.
 * @param yyscanner The scanner object.
 */
FILE *yyget_in  (yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    return yyin;
}

/** Get the output stream.
 * @param yyscanner The scanner object.
 */
FILE *yyget_out  (yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    return yyout;
}

/** Get the length of the current token.
 * @param yyscanner The scanner object.
 */
int yyget_leng  (yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    return yyleng;
}

/** Get the current token.
 * @param yyscanner The scanner object.
 */

char *yyget_text  (yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    return yytext;
}

/** Set the user-defined data. This data is never touched by the scanner.
 * @param user_defined The data to be associated with this scanner.
 * @param yyscanner The scanner object.
 */
void yyset_extra (YY_EXTRA_TYPE  user_defined , yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    yyextra = user_defined ;
}

/** Set the current line number.
 * @param _line_number line number
 * @param yyscanner The scanner object.
 */
void yyset_lineno (int  _line_number , yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;

        /* lineno is only valid if an input buffer exists. */
        if (! YY_CURRENT_BUFFER )
           YY_FATAL_ERROR( "yyset_lineno called with no buffer" );
    
    yylineno = _line_number;
}

/** Set the current column.
 * @param _column_no column number
 * @param yyscanner The scanner object.
 */
void yyset_column (int  _column_no , yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;

        /* column is only valid if an input buffer exists. */
        if (! YY_CURRENT_BUFFER )
           YY_FATAL_ERROR( "yyset_column called with no buffer" );
    
    yycolumn = _column_no;
}

/** Set the input stream. This does not discard the current
 * input buffer.
 * @param _in_str A readable stream.
 * @param yyscanner The scanner object.
 * @see yy_switch_to_buffer
 */
void yyset_in (FILE *  _in_str , yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    yyin = _in_str ;
}

void yyset_out (FILE *  _out_str , yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    yyout = _out_str ;
}

int yyget_debug  (yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    return yy_flex_debug;
}

void yyset_debug (int  _bdebug , yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    yy_flex_debug = _bdebug ;
}

/* Accessor methods for yylval and yylloc */

/* User-visible API */

/* yylex_init is special because it creates the scanner itself, so it is
 * the ONLY reentrant function that doesn't take the scanner as the last argument.
 * That's why we explicitly handle the declaration, instead of using our macros.
 */
int yylex_init(yyscan_t* ptr_yy_globals)
{
    if (ptr_yy_globals == NULL){
        errno = EINVAL;
        return 1;
    }

    *ptr_yy_globals = (yyscan_t) yyalloc ( sizeof( struct yyguts_t ), NULL );

    if (*ptr_yy_globals == NULL){
        errno = ENOMEM;
        return 1;
    }

    /* By setting to 0xAA, we expose bugs in yy_init_globals. Leave at 0x00 for releases. */
    memset(*ptr_yy_globals,0x00,sizeof(struct yyguts_t));

    return yy_init_globals ( *ptr_yy_globals );
}

/* yylex_init_extra has the same functionality as yylex_init, but follows the
 * convention of taking the scanner as the last argument. Note however, that
 * this is a *pointer* to a scanner, as it will be allocated by this call (and
 * is the reason, too, why this function also must handle its own declaration).
 * The user defined value in the first argument will be available to yyalloc in
 * the yyextra field.
 */
int yylex_init_extra( YY_EXTRA_TYPE yy_user_defined, yyscan_t* ptr_yy_globals )
{
    struct yyguts_t dummy_yyguts;

    yyset_extra (yy_user_defined, &dummy_yyguts);

    if (ptr_yy_globals == NULL){
        errno = EINVAL;
        return 1;
    }

    *ptr_yy_globals = (yyscan_t) yyalloc ( sizeof( struct yyguts_t ), &dummy_yyguts );

    if (*ptr_yy_globals == NULL){
        errno = ENOMEM;
        return 1;
    }

    /* By setting to 0xAA, we expose bugs in
    yy_init_globals. Leave at 0x00 for releases. */
    memset(*ptr_yy_globals,0x00,sizeof(struct yyguts_t));

    yyset_extra (yy_user_defined, *ptr_yy_globals);

    return yy_init_globals ( *ptr_yy_globals );
}

static int yy_init_globals (yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    /* Initialization is the same as for the non-reentrant scanner.
     * This function is called from yylex_destroy(), so don't allocate here.
     */

    yyg->yy_buffer_stack = NULL;
    yyg->yy_buffer_stack_top = 0;
    yyg->yy_buffer_stack_max = 0;
    yyg->yy_c_buf_p = NULL;
    yyg->yy_init = 0;
    yyg->yy_start = 0;

    yyg->yy_start_stack_ptr = 0;
    yyg->yy_start_stack_depth = 0;
    yyg->yy_start_stack =  NULL;

    yyg->yy_state_buf = 0;
    yyg->yy_state_ptr = 0;
    yyg->yy_full_match = 0;
    yyg->yy_lp = 0;

/* Defined in main.c */
#ifdef YY_STDINIT
    yyin = stdin;
    yyout = stdout;
#else
    yyin = NULL;
    yyout = NULL;
#endif

    /* For future reference: Set errno on error, since we are called by
     * yylex_init()
     */
    return 0;
}

/* yylex_destroy is for both reentrant and non-reentrant scanners. */
int yylex_destroy  (yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;

    /* Pop the buffer stack, destroying each element. */
	while(YY_CURRENT_BUFFER){
		yy_delete_buffer( YY_CURRENT_BUFFER , yyscanner );
		YY_CURRENT_BUFFER_LVALUE = NULL;
		yypop_buffer_state(yyscanner);
	}

	/* Destroy the stack itself. */
	yyfree(yyg->yy_buffer_stack , yyscanner);
	yyg->yy_buffer_stack = NULL;

    /* Destroy the start condition stack. */
        yyfree( yyg->yy_start_stack , yyscanner );
        yyg->yy_start_stack = NULL;

    yyfree ( yyg->yy_state_buf , yyscanner);
    yyg->yy_state_buf  = NULL;

    /* Reset the globals. This is important in a non-reentrant scanner so the next time
     * yylex() is called, initialization will occur. */
    yy_init_globals( yyscanner);

    /* Destroy the main struct (reentrant only). */
    yyfree ( yyscanner , yyscanner );
    yyscanner = NULL;
    return 0;
}

/*
 * Internal utility routines.
 */

#ifndef yytext_ptr
static void yy_flex_strncpy (char* s1, const char * s2, int n , yyscan_t yyscanner)
{
	struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
	(void)yyg;

	int i;
	for ( i = 0; i < n; ++i )
		s1[i] = s2[i];
}
#endif

#ifdef YY_NEED_STRLEN
static int yy_flex_strlen (const char * s , yyscan_t yyscanner)
{
	int n;
	for ( n = 0; s[n]; ++n )
		;

	return n;
}
#endif

void *yyalloc (yy_size_t  size , yyscan_t yyscanner)
{
	struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
	(void)yyg;
	return malloc(size);
}

void *yyrealloc  (void * ptr, yy_size_t  size , yyscan_t yyscanner)
{
	struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
	(void)yyg;

	/* The cast to (char *) in the following accommodates both
	 * implementations that use char* generic pointers, and those
	 * that use void* generic pointers.  It works with the latter
	 * because both ANSI C and C++ allow castless assignment from
	 * any pointer type to void*, and deal with argument conversions
	 * as though doing an assignment.
	 */
	return realloc(ptr, size);
}

void yyfree (void * ptr , yyscan_t yyscanner)
{
	struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
	(void)yyg;
	free( (char *) ptr );	/* see yyrealloc() for (char *) cast */
}

#define YYTABLES_NAME "yytables"

#line 1828 "/cygdrive/d/a/scallywag/doxygen/doxygen-1.18.0-1.x86_64/build/generated_src/pyscanner.l"


//----------------------------------------------------------------------------


static int yyread(yyscan_t yyscanner,char *buf,int max_size)
{
  struct yyguts_t *yyg = (struct yyguts_t*)yyscanner;
  int c=0;
  const char *p = yyextra->inputString + yyextra->inputPosition;
  while ( c < max_size && *p ) { *buf++ = *p++; c++; }
  yyextra->inputPosition+=c;
  return c;
}

static void initParser(yyscan_t yyscanner)
{
  struct yyguts_t *yyg = (struct yyguts_t*)yyscanner;
  yyextra->protection = Protection::Public;
  yyextra->mtype = MethodTypes::Method;
  yyextra->isStatic = false;
  yyextra->virt = Specifier::Normal;
  yyextra->previous = 0;
  yyextra->packageCommentAllowed = true;
}

static void initEntry(yyscan_t yyscanner)
{
  struct yyguts_t *yyg = (struct yyguts_t*)yyscanner;
  //yyextra->current->python = true;
  yyextra->current->protection = yyextra->protection ;
  yyextra->current->mtype      = yyextra->mtype;
  yyextra->current->virt       = yyextra->virt;
  yyextra->current->isStatic   = yyextra->isStatic;
  yyextra->current->lang       = SrcLangExt::Python;
  yyextra->current->type.clear();
  yyextra->current->name.clear();
  yyextra->current->initializer.clear();
  yyextra->commentScanner.initGroupInfo(yyextra->current.get());
  yyextra->isStatic = false;
}

static void newEntry(yyscan_t yyscanner)
{
  struct yyguts_t *yyg = (struct yyguts_t*)yyscanner;
  if (yyextra->current->name.empty())
  {
    initEntry(yyscanner);
    return;
  }
  bool found = false;
  if (yyextra->checkDupEntry)
  {
    for (auto &v : yyextra->current_root->children())
    {
      //printf("candidate %s<->%s section=%s!\n",qPrint(v->name),qPrint(yyextra->current->name), qPrint(v->section.to_string()));
      if (v->name==yyextra->current->name && v->section==yyextra->current->section) // name is already added, don't add it again
      {
        if (v->doc.empty() && !yyextra->current->doc.empty())
        {
          v->doc       = yyextra->current->doc;
          v->docLine   = yyextra->current->docLine;
          v->docFile   = yyextra->current->docFile;
        }
        if (v->brief.empty() && !yyextra->current->brief.empty())
        {
          v->brief     = yyextra->current->brief;
          v->briefLine = yyextra->current->briefLine;
          v->briefFile = yyextra->current->briefFile;
        }
        if (v->type.empty() && !yyextra->current->type.empty())
        {
          //printf("copying type '%s' from '%s' to '%s'\n",qPrint(yyextra->current->type),qPrint(yyextra->current->name),qPrint(v->name));
          v->type      = yyextra->current->type;
        }
        found=true;
        break;
      }
    }
  }
  if (!found)
  {
    yyextra->previous = yyextra->current;
    yyextra->current_root->moveToSubEntryAndRefresh(yyextra->current);
  }
  initEntry(yyscanner);
}

static void addEntry(yyscan_t yyscanner)
{
  struct yyguts_t *yyg = (struct yyguts_t*)yyscanner;

  auto doc        = yyextra->current->doc;
  auto docLine    = yyextra->current->docLine;
  auto docFile    = yyextra->current->docFile;
  auto brief      = yyextra->current->brief;
  auto briefLine  = yyextra->current->briefLine;
  auto briefFile  = yyextra->current->briefFile;

  yyextra->previous = yyextra->current;
  yyextra->current_root->moveToSubEntryAndRefresh(yyextra->current);
  initEntry(yyscanner);

  yyextra->current->doc        = doc;
  yyextra->current->docLine    = docLine;
  yyextra->current->docFile    = docFile;
  yyextra->current->brief      = brief;
  yyextra->current->briefLine  = briefLine;
  yyextra->current->briefFile  = briefFile;
}

static void setProtection(yyscan_t yyscanner)
{
  struct yyguts_t *yyg = (struct yyguts_t*)yyscanner;
  if (!yyextra->current->name.empty() && yyextra->current->name.at(0)=='_')
  {
    if (yyextra->current->name.at(1)=='_') // mark as private
    {
      yyextra->current->protection=Protection::Private;
    }
    else // mark as protected
    {
      yyextra->current->protection=Protection::Protected;
    }
  }
}

static void docVariable(yyscan_t yyscanner,const char *name)
{
  struct yyguts_t *yyg = (struct yyguts_t*)yyscanner;
  yyextra->current->name = name;
  yyextra->current->section=EntryType::makeVariable();
  yyextra->current->fileName  = yyextra->fileName;
  yyextra->current->startLine = yyextra->yyLineNr;
  yyextra->current->bodyLine  = yyextra->yyLineNr;
  yyextra->current->type.clear();
  setProtection(yyscanner);
  yyextra->checkDupEntry = true;
}

static void newVariable(yyscan_t yyscanner)
{
  struct yyguts_t *yyg = (struct yyguts_t*)yyscanner;
  setProtection(yyscanner);
  if (yyextra->current_root->section.isCompound()) // mark as class variable
  {
    yyextra->current->isStatic = true;
  }
  newEntry(yyscanner);
}

static void newTypeAlias(yyscan_t yyscanner)
{
  //struct yyguts_t *yyg = (struct yyguts_t*)yyscanner;
  //printf("*** newTypeAlias %s\n",qPrint(yyextra->current->name));
  newEntry(yyscanner);
}

static void addVariable(yyscan_t yyscanner)
{
  struct yyguts_t *yyg = (struct yyguts_t*)yyscanner;
  setProtection(yyscanner);
  if (yyextra->current_root->section.isCompound()) // mark as class variable
  {
    yyextra->current->isStatic = true;
  }
  addEntry(yyscanner);
}

static void newFunction(yyscan_t yyscanner)
{
  struct yyguts_t *yyg = (struct yyguts_t*)yyscanner;
  if (yyextra->current->name.startsWith("__") && yyextra->current->name.endsWith("__"))
  {
    // special method name, see
    // http://docs.python.org/ref/specialnames.html
    yyextra->current->protection=Protection::Public;
  }
  else
  {
    setProtection(yyscanner);
  }
}

static inline int computeIndent(const char *s)
{
  int col=0;
  int tabSize=Config_getInt(TAB_SIZE);
  const char *p=s;
  char c;
  while ((c=*p++))
  {
    if (c==' ') col++;
    else if (c=='\t') col+=tabSize-(col%tabSize);
    else break;
  }
  return col;
}

static DString findPackageScopeFromPath(yyscan_t yyscanner,const DString &path)
{
  struct yyguts_t *yyg = (struct yyguts_t*)yyscanner;
  auto it = yyextra->packageNameCache.find(path.str());
  if (it!=yyextra->packageNameCache.end())
  {
    return it->second;
  }
  FileInfo pf(path.str()+"/__init__.py"); // found package initialization file
  if (pf.exists())
  {
    size_t i=path.rfind('/');
    if (i!=DString::npos)
    {
      DString scope = findPackageScopeFromPath(yyscanner,path.left(i));
      if (!scope.empty())
      {
        scope+="::";
      }
      scope+=path.mid(i+1);
      yyextra->packageNameCache.emplace(path.str(),scope.str());
      return scope;
    }
  }
  return "";
}

static DString findPackageScope(yyscan_t yyscanner,const DString &fileName)
{
  if (fileName.empty()) return fileName;
  FileInfo fi(fileName.str());
  return findPackageScopeFromPath(yyscanner,fi.dirPath(true));
}

static void addFrom(yyscan_t yyscanner,bool all)
{
  struct yyguts_t *yyg = (struct yyguts_t*)yyscanner;
  DString item=all ? yyextra->packageName : yyextra->packageName+"."+yytext;
  yyextra->current->name=removeRedundantWhiteSpace(substitute(item,".","::"));
  yyextra->current->fileName = yyextra->fileName;
  //printf("Adding using declaration: found:%s:%d name=%s\n",qPrint(yyextra->fileName),yyextra->yyLineNr,qPrint(yyextra->current->name));
  yyextra->current->section=all ? EntryType::makeUsingDir() : EntryType::makeUsingDecl();
  yyextra->current_root->moveToSubEntryAndRefresh(yyextra->current);
  initEntry(yyscanner);
}
//-----------------------------------------------------------------------------

static void lineCount(yyscan_t yyscanner)
{
  struct yyguts_t *yyg = (struct yyguts_t*)yyscanner;
  DBG_CTX((stderr,"yyextra->yyLineNr=%d\n",yyextra->yyLineNr));
  for (const char *p = yytext; *p; ++p)
  {
    yyextra->yyLineNr += (*p == '\n') ;
  }
}

static void incLineNr(yyscan_t yyscanner)
{
  struct yyguts_t *yyg = (struct yyguts_t*)yyscanner;
  DBG_CTX((stderr,"yyextra->yyLineNr=%d\n",yyextra->yyLineNr));
  yyextra->yyLineNr++;
}

//-----------------------------------------------------------------
static void startCommentBlock(yyscan_t yyscanner,bool brief)
{
  struct yyguts_t *yyg = (struct yyguts_t*)yyscanner;
  if (brief)
  {
    yyextra->current->briefFile = yyextra->fileName;
    yyextra->current->briefLine = yyextra->yyLineNr;
  }
  else
  {
    yyextra->current->docFile = yyextra->fileName;
    yyextra->current->docLine = yyextra->yyLineNr;
  }
}

static void handleCommentBlock(yyscan_t yyscanner,const DString &doc,bool brief)
{
  struct yyguts_t *yyg = (struct yyguts_t*)yyscanner;
  //printf("handleCommentBlock(doc=[%s] brief=%d yyextra->docBlockInBody=%d yyextra->docBlockJavaStyle=%d\n",
  //    qPrint(doc),brief,yyextra->docBlockInBody,yyextra->docBlockJavaStyle);

  // TODO: Fix me
  yyextra->docBlockInBody=false;

  if (!yyextra->current->doc.empty())
  {
    yyextra->current->doc=yyextra->current->doc.stripWhiteSpace()+"\n\n";
  }
  if (yyextra->docBlockInBody && yyextra->previous && !yyextra->previous->doc.empty())
  {
    yyextra->previous->doc=yyextra->previous->doc.stripWhiteSpace()+"\n\n";
  }

  int position = 0;
  bool needsEntry = false;
  int lineNr = brief ? yyextra->current->briefLine : yyextra->current->docLine;
  Markdown markdown(yyextra->fileName,lineNr);
  GuardedSectionStack guards;
  DString strippedDoc = stripIndentation(doc,true);
  //printf("stippedDoc=[%s]\n",qPrint(strippedDoc));
  DString processedDoc = Config_getBool(MARKDOWN_SUPPORT) ? markdown.process(strippedDoc,lineNr) : strippedDoc;
  while (yyextra->commentScanner.parseCommentBlock(
        yyextra->thisParser,
        (yyextra->docBlockInBody && yyextra->previous) ? yyextra->previous.get() : yyextra->current.get(),
        processedDoc, // text
        yyextra->fileName,   // file
        lineNr,
        yyextra->docBlockInBody ? false : brief,
        yyextra->docBlockJavaStyle, // javadoc style // or false,
        yyextra->docBlockInBody,
        yyextra->protection,
        position,
        needsEntry,
        Config_getBool(MARKDOWN_SUPPORT),
        &guards
       )
     ) // need to start a new entry
  {
    if (needsEntry)
    {
      newEntry(yyscanner);
    }
  }
  if (needsEntry)
  {
    newEntry(yyscanner);
  }

}

static void endOfDef(yyscan_t yyscanner,int correction)
{
  struct yyguts_t *yyg = (struct yyguts_t*)yyscanner;
  //printf("endOfDef at=%d\n",yyextra->yyLineNr);
  if (yyextra->bodyEntry)
  {
    yyextra->bodyEntry->endBodyLine  = yyextra->yyLineNr-correction;
    yyextra->bodyEntry = 0;
  }
  newEntry(yyscanner);
  //yyextra->insideConstructor = false;
}

static inline void addToString(yyscan_t yyscanner,const char *s)
{
  struct yyguts_t *yyg = (struct yyguts_t*)yyscanner;
  if (yyextra->copyString) (*yyextra->copyString) << s;
}

static void initTriDoubleQuoteBlock(yyscan_t yyscanner)
{
  struct yyguts_t *yyg = (struct yyguts_t*)yyscanner;
  yyextra->docBlockContext   = YY_START;
  yyextra->docBlockInBody    = false;
  yyextra->docBlockJavaStyle = true;
  yyextra->docBlockSpecial   = yytext[strlen(yytext) - 1]=='!' || !Config_getBool(PYTHON_DOCSTRING);
  yyextra->docBlock.clear();
  yyextra->commentIndent = yyextra->curIndent;
  yyextra->doubleQuote = true;
  startCommentBlock(yyscanner,false);
}

static void initTriSingleQuoteBlock(yyscan_t yyscanner)
{
  struct yyguts_t *yyg = (struct yyguts_t*)yyscanner;
  yyextra->docBlockContext   = YY_START;
  yyextra->docBlockInBody    = false;
  yyextra->docBlockJavaStyle = true;
  yyextra->docBlockSpecial   = yytext[strlen(yytext) - 1]=='!' || !Config_getBool(PYTHON_DOCSTRING);
  yyextra->docBlock.clear();
  yyextra->commentIndent = yyextra->curIndent;
  yyextra->doubleQuote = false;
  startCommentBlock(yyscanner,false);
}

static void initSpecialBlock(yyscan_t yyscanner)
{
  struct yyguts_t *yyg = (struct yyguts_t*)yyscanner;
  yyextra->docBlockContext   = YY_START;
  yyextra->docBlockInBody    = false;
  yyextra->docBlockJavaStyle = true;
  yyextra->docBrief = true;
  yyextra->docBlock.clear();
  yyextra->commentIndent = yyextra->curIndent;
  startCommentBlock(yyscanner,false);
}

static void searchFoundDef(yyscan_t yyscanner)
{
  struct yyguts_t *yyg = (struct yyguts_t*)yyscanner;
  yyextra->current->fileName  = yyextra->fileName;
  yyextra->current->startLine = yyextra->yyLineNr;
  yyextra->current->bodyLine  = yyextra->yyLineNr;
  yyextra->current->section = EntryType::makeFunction();
  yyextra->current->lang = SrcLangExt::Python;
  yyextra->current->virt = Specifier::Normal;
  yyextra->current->isStatic = yyextra->isStatic;
  yyextra->current->mtype = yyextra->mtype = MethodTypes::Method;
  yyextra->current->type.clear();
  yyextra->current->name.clear();
  yyextra->current->args.clear();
  yyextra->current->argList.clear();
  yyextra->packageCommentAllowed = false;
  yyextra->isStatic=false;
  //printf("searchFoundDef at=%d\n",yyextra->yyLineNr);
}

static void searchFoundClass(yyscan_t yyscanner)
{
  struct yyguts_t *yyg = (struct yyguts_t*)yyscanner;
  yyextra->current->section = EntryType::makeClass();
  yyextra->current->argList.clear();
  yyextra->current->type += "class" ;
  yyextra->current->fileName  = yyextra->fileName;
  yyextra->current->startLine  = yyextra->yyLineNr;
  yyextra->current->bodyLine  = yyextra->yyLineNr;
  yyextra->packageCommentAllowed = false;
}

static void searchFoundTypeAlias(yyscan_t yyscanner)
{
  struct yyguts_t *yyg = (struct yyguts_t*)yyscanner;
  yyextra->current->section = EntryType::makeVariable();
  yyextra->current->argList.clear();
  yyextra->current->type = "typedef " ;
  yyextra->current->spec.setAlias(true);
  yyextra->current->fileName  = yyextra->fileName;
  yyextra->current->startLine  = yyextra->yyLineNr;
  yyextra->current->bodyLine  = yyextra->yyLineNr;
  yyextra->packageCommentAllowed = false;
}

//----------------------------------------------------------------------------

static void parseCompounds(yyscan_t yyscanner,std::shared_ptr<Entry> rt)
{
  struct yyguts_t *yyg = (struct yyguts_t*)yyscanner;
  //printf("parseCompounds(%s)\n",qPrint(rt->name));
  for (size_t i=0; i<rt->children().size(); ++i)
  {
    std::shared_ptr<Entry> ce = rt->children()[i];
    if (!ce->program.empty())
    {
      //fprintf(stderr,"parseCompounds: -- %s (line %d) ---------\n%s\n---------------\n",
      //  qPrint(ce->name), ce->bodyLine, qPrint(ce->program));
      // init scanner state
      yyextra->programStr = ce->program.str();
      yyextra->inputString = yyextra->programStr.data();
      yyextra->inputPosition = 0;
      yyextra->firstPass     = false;
      pyscannerYYrestart( nullptr, yyscanner );
      if (ce->section.isCompound())
      {
        yyextra->specialBlock = false;
        yyextra->current_root = ce;
        BEGIN( Search );
      }
      else if (ce->parent())
      {
        yyextra->current_root = rt;
        //printf("Searching for member variables in %s parent=%s\n",
        //    qPrint(ce->name),qPrint(ce->parent->name));
        BEGIN( SearchMemVars );
      }
      yyextra->fileName = ce->fileName;
      yyextra->yyLineNr   = ce->bodyLine ;
      yyextra->current = std::make_shared<Entry>();
      initEntry(yyscanner);

      yyextra->checkDupEntry = false;

      DString name = ce->name;
      yyextra->commentScanner.enterCompound(yyextra->fileName,yyextra->yyLineNr,name);

      pyscannerYYlex(yyscanner) ;
      yyextra->lexInit=true;

      yyextra->programStr.clear();
      ce->program.str(std::string());

      yyextra->commentScanner.leaveCompound(yyextra->fileName,yyextra->yyLineNr,name);

    }
    parseCompounds(yyscanner,ce);
  }
}

//----------------------------------------------------------------------------


static void parseMain(yyscan_t yyscanner, const DString &fileName,const char *fileBuf,const std::shared_ptr<Entry> &rt)
{
  struct yyguts_t *yyg = (struct yyguts_t*)yyscanner;
  initParser(yyscanner);

  if (fileBuf==nullptr || fileBuf[0]=='\0') return;

  yyextra->inputString = fileBuf;
  yyextra->inputPosition = 0;

  yyextra->protection    = Protection::Public;
  yyextra->mtype         = MethodTypes::Method;
  yyextra->isStatic      = false;
  yyextra->virt          = Specifier::Normal;
  yyextra->current_root  = rt;
  yyextra->specialBlock  = false;

  yyextra->yyLineNr      = 1 ;
  yyextra->fileName      = fileName;
  yyextra->checkDupEntry = false;
  yyextra->firstPass     = true;
  //setContext();
  msg("Parsing file {}...\n",yyextra->fileName);

  FileInfo fi(fileName.str());
  yyextra->moduleScope = findPackageScope(yyscanner,fileName);
  DString baseName=fi.baseName();
  if (baseName!="__init__") // package initializer file is not a package itself
  {
    if (!yyextra->moduleScope.empty())
    {
      yyextra->moduleScope+="::";
    }
    yyextra->moduleScope+=baseName;
  }

  // add namespaces for each scope
  DString scope = yyextra->moduleScope;
  size_t startPos = 0;
  size_t pos = 0;
  do
  {
    pos = scope.find("::",startPos);
    if (pos==DString::npos)
    {
      startPos = pos = scope.length();
    }
    else
    {
      startPos = pos+2;
    }
    yyextra->current            = std::make_shared<Entry>();
    initEntry(yyscanner);
    yyextra->current->name      = scope.left(pos);
    yyextra->current->section   = EntryType::makeNamespace();
    yyextra->current->type      = "namespace";
    yyextra->current->fileName  = yyextra->fileName;
    yyextra->current->startLine = yyextra->yyLineNr;
    yyextra->current->bodyLine  = yyextra->yyLineNr;
    yyextra->current_root       = yyextra->current;
    rt->moveToSubEntryAndRefresh(yyextra->current);
  } while (pos<scope.length());

  initParser(yyscanner);

  yyextra->commentScanner.enterFile(yyextra->fileName,yyextra->yyLineNr);

  yyextra->current->reset();
  initEntry(yyscanner);
  pyscannerYYrestart(nullptr,yyscanner);
  BEGIN( Search );
  pyscannerYYlex(yyscanner);
  yyextra->lexInit=true;

  yyextra->commentScanner.leaveFile(yyextra->fileName,yyextra->yyLineNr);

  yyextra->programStr.clear();
  yyextra->current_root->program.str(std::string());

  parseCompounds(yyscanner, yyextra->current_root);
}

//----------------------------------------------------------------------------

static void parsePrototype(yyscan_t yyscanner,const DString &text)
{
  struct yyguts_t *yyg = (struct yyguts_t*)yyscanner;
  //printf("**** parsePrototype(%s) begin\n",qPrint(text));
  if (text.empty())
  {
    warn(yyextra->fileName,yyextra->yyLineNr,"Empty prototype found!");
    return;
  }

  yyextra->specialBlock = false;
  yyextra->packageCommentAllowed = false;

  // save scanner state
  YY_BUFFER_STATE orgState = YY_CURRENT_BUFFER;
  yy_switch_to_buffer(yy_create_buffer(nullptr, YY_BUF_SIZE, yyscanner), yyscanner);
  const char *orgInputString = yyextra->inputString;
  int orgInputPosition = yyextra->inputPosition;

  // set new string
  yyextra->inputString = text.data();
  yyextra->inputPosition = 0;
  pyscannerYYrestart( nullptr, yyscanner );

  BEGIN( FunctionDec );

  pyscannerYYlex(yyscanner);
  yyextra->lexInit=true;

  yyextra->current->name = yyextra->current->name.stripWhiteSpace();
  if (yyextra->current->section.isMemberDoc() && yyextra->current->args.empty())
  {
    yyextra->current->section = EntryType::makeVariableDoc();
  }

  // restore original scanner state

  yy_delete_buffer(YY_CURRENT_BUFFER, yyscanner);
  yy_switch_to_buffer(orgState, yyscanner);

  yyextra->inputString = orgInputString;
  yyextra->inputPosition = orgInputPosition;

  //printf("**** parsePrototype end\n");
}

//----------------------------------------------------------------------------

struct PythonOutlineParser::Private
{
  yyscan_t yyscanner;
  pyscannerYY_state state;
};

PythonOutlineParser::PythonOutlineParser() : p(std::make_unique<PythonOutlineParser::Private>())
{
  pyscannerYYlex_init_extra(&p->state,&p->yyscanner);
#ifdef FLEX_DEBUG
  pyscannerYYset_debug(Debug::isFlagSet(Debug::Lex_pyscanner)?1:0,p->yyscanner);
#endif
}

PythonOutlineParser::~PythonOutlineParser()
{
  pyscannerYYlex_destroy(p->yyscanner);
}


void PythonOutlineParser::parseInput(const DString &fileName,
                                     const char *fileBuf,
                                     const std::shared_ptr<Entry> &root,
                                     ClangTUParser * /*clangParser*/)
{
  struct yyguts_t *yyg = (struct yyguts_t*)p->yyscanner;
  yyextra->thisParser = this;
  DebugLex debugLex(Debug::Lex_pyscanner, __FILE__, qPrint(fileName));
  ::parseMain(p->yyscanner, fileName,fileBuf,root);

  // May print the AST for debugging purposes
  // printAST(global_root);
}

bool PythonOutlineParser::needsPreprocessing(const DString &) const
{
  return false;
}

void PythonOutlineParser::parsePrototype(const DString &text)
{
  ::parsePrototype(p->yyscanner,text);
}

//----------------------------------------------------------------------------

#include "pyscanner.l.h"

