/** This file is generated by python3 caseconvert.py. DO NOT EDIT! */

#ifndef CASECONVERT_H
#define CASECONVERT_H

#include <cstdint>
#include <string>

#define BSEQ(...) { static unsigned char s[] = { __VA_ARGS__, 0x00 }; \
                    return reinterpret_cast<const char *>(s); }

inline const char *convertUnicodeToUpper(uint32_t code)
{
  switch(code)
  {
    case 0x61 /* a */: BSEQ(0x41) /* A */
    case 0x62 /* b */: BSEQ(0x42) /* B */
    case 0x63 /* c */: BSEQ(0x43) /* C */
    case 0x64 /* d */: BSEQ(0x44) /* D */
    case 0x65 /* e */: BSEQ(0x45) /* E */
    case 0x66 /* f */: BSEQ(0x46) /* F */
    case 0x67 /* g */: BSEQ(0x47) /* G */
    case 0x68 /* h */: BSEQ(0x48) /* H */
    case 0x69 /* i */: BSEQ(0x49) /* I */
    case 0x6a /* j */: BSEQ(0x4a) /* J */
    case 0x6b /* k */: BSEQ(0x4b) /* K */
    case 0x6c /* l */: BSEQ(0x4c) /* L */
    case 0x6d /* m */: BSEQ(0x4d) /* M */
    case 0x6e /* n */: BSEQ(0x4e) /* N */
    case 0x6f /* o */: BSEQ(0x4f) /* O */
    case 0x70 /* p */: BSEQ(0x50) /* P */
    case 0x71 /* q */: BSEQ(0x51) /* Q */
    case 0x72 /* r */: BSEQ(0x52) /* R */
    case 0x73 /* s */: BSEQ(0x53) /* S */
    case 0x74 /* t */: BSEQ(0x54) /* T */
    case 0x75 /* u */: BSEQ(0x55) /* U */
    case 0x76 /* v */: BSEQ(0x56) /* V */
    case 0x77 /* w */: BSEQ(0x57) /* W */
    case 0x78 /* x */: BSEQ(0x58) /* X */
    case 0x79 /* y */: BSEQ(0x59) /* Y */
    case 0x7a /* z */: BSEQ(0x5a) /* Z */
    case 0xb5 /* µ */: BSEQ(0xce,0x9c) /* Μ */
    case 0xdf /* ß */: BSEQ(0x53,0x53) /* SS */
    case 0xe0 /* à */: BSEQ(0xc3,0x80) /* À */
    case 0xe1 /* á */: BSEQ(0xc3,0x81) /* Á */
    case 0xe2 /* â */: BSEQ(0xc3,0x82) /* Â */
    case 0xe3 /* ã */: BSEQ(0xc3,0x83) /* Ã */
    case 0xe4 /* ä */: BSEQ(0xc3,0x84) /* Ä */
    case 0xe5 /* å */: BSEQ(0xc3,0x85) /* Å */
    case 0xe6 /* æ */: BSEQ(0xc3,0x86) /* Æ */
    case 0xe7 /* ç */: BSEQ(0xc3,0x87) /* Ç */
    case 0xe8 /* è */: BSEQ(0xc3,0x88) /* È */
    case 0xe9 /* é */: BSEQ(0xc3,0x89) /* É */
    case 0xea /* ê */: BSEQ(0xc3,0x8a) /* Ê */
    case 0xeb /* ë */: BSEQ(0xc3,0x8b) /* Ë */
    case 0xec /* ì */: BSEQ(0xc3,0x8c) /* Ì */
    case 0xed /* í */: BSEQ(0xc3,0x8d) /* Í */
    case 0xee /* î */: BSEQ(0xc3,0x8e) /* Î */
    case 0xef /* ï */: BSEQ(0xc3,0x8f) /* Ï */
    case 0xf0 /* ð */: BSEQ(0xc3,0x90) /* Ð */
    case 0xf1 /* ñ */: BSEQ(0xc3,0x91) /* Ñ */
    case 0xf2 /* ò */: BSEQ(0xc3,0x92) /* Ò */
    case 0xf3 /* ó */: BSEQ(0xc3,0x93) /* Ó */
    case 0xf4 /* ô */: BSEQ(0xc3,0x94) /* Ô */
    case 0xf5 /* õ */: BSEQ(0xc3,0x95) /* Õ */
    case 0xf6 /* ö */: BSEQ(0xc3,0x96) /* Ö */
    case 0xf8 /* ø */: BSEQ(0xc3,0x98) /* Ø */
    case 0xf9 /* ù */: BSEQ(0xc3,0x99) /* Ù */
    case 0xfa /* ú */: BSEQ(0xc3,0x9a) /* Ú */
    case 0xfb /* û */: BSEQ(0xc3,0x9b) /* Û */
    case 0xfc /* ü */: BSEQ(0xc3,0x9c) /* Ü */
    case 0xfd /* ý */: BSEQ(0xc3,0x9d) /* Ý */
    case 0xfe /* þ */: BSEQ(0xc3,0x9e) /* Þ */
    case 0xff /* ÿ */: BSEQ(0xc5,0xb8) /* Ÿ */
    case 0x101 /* ā */: BSEQ(0xc4,0x80) /* Ā */
    case 0x103 /* ă */: BSEQ(0xc4,0x82) /* Ă */
    case 0x105 /* ą */: BSEQ(0xc4,0x84) /* Ą */
    case 0x107 /* ć */: BSEQ(0xc4,0x86) /* Ć */
    case 0x109 /* ĉ */: BSEQ(0xc4,0x88) /* Ĉ */
    case 0x10b /* ċ */: BSEQ(0xc4,0x8a) /* Ċ */
    case 0x10d /* č */: BSEQ(0xc4,0x8c) /* Č */
    case 0x10f /* ď */: BSEQ(0xc4,0x8e) /* Ď */
    case 0x111 /* đ */: BSEQ(0xc4,0x90) /* Đ */
    case 0x113 /* ē */: BSEQ(0xc4,0x92) /* Ē */
    case 0x115 /* ĕ */: BSEQ(0xc4,0x94) /* Ĕ */
    case 0x117 /* ė */: BSEQ(0xc4,0x96) /* Ė */
    case 0x119 /* ę */: BSEQ(0xc4,0x98) /* Ę */
    case 0x11b /* ě */: BSEQ(0xc4,0x9a) /* Ě */
    case 0x11d /* ĝ */: BSEQ(0xc4,0x9c) /* Ĝ */
    case 0x11f /* ğ */: BSEQ(0xc4,0x9e) /* Ğ */
    case 0x121 /* ġ */: BSEQ(0xc4,0xa0) /* Ġ */
    case 0x123 /* ģ */: BSEQ(0xc4,0xa2) /* Ģ */
    case 0x125 /* ĥ */: BSEQ(0xc4,0xa4) /* Ĥ */
    case 0x127 /* ħ */: BSEQ(0xc4,0xa6) /* Ħ */
    case 0x129 /* ĩ */: BSEQ(0xc4,0xa8) /* Ĩ */
    case 0x12b /* ī */: BSEQ(0xc4,0xaa) /* Ī */
    case 0x12d /* ĭ */: BSEQ(0xc4,0xac) /* Ĭ */
    case 0x12f /* į */: BSEQ(0xc4,0xae) /* Į */
    case 0x131 /* ı */: BSEQ(0x49) /* I */
    case 0x133 /* ĳ */: BSEQ(0xc4,0xb2) /* Ĳ */
    case 0x135 /* ĵ */: BSEQ(0xc4,0xb4) /* Ĵ */
    case 0x137 /* ķ */: BSEQ(0xc4,0xb6) /* Ķ */
    case 0x13a /* ĺ */: BSEQ(0xc4,0xb9) /* Ĺ */
    case 0x13c /* ļ */: BSEQ(0xc4,0xbb) /* Ļ */
    case 0x13e /* ľ */: BSEQ(0xc4,0xbd) /* Ľ */
    case 0x140 /* ŀ */: BSEQ(0xc4,0xbf) /* Ŀ */
    case 0x142 /* ł */: BSEQ(0xc5,0x81) /* Ł */
    case 0x144 /* ń */: BSEQ(0xc5,0x83) /* Ń */
    case 0x146 /* ņ */: BSEQ(0xc5,0x85) /* Ņ */
    case 0x148 /* ň */: BSEQ(0xc5,0x87) /* Ň */
    case 0x149 /* ŉ */: BSEQ(0xca,0xbc,0x4e) /* ʼN */
    case 0x14b /* ŋ */: BSEQ(0xc5,0x8a) /* Ŋ */
    case 0x14d /* ō */: BSEQ(0xc5,0x8c) /* Ō */
    case 0x14f /* ŏ */: BSEQ(0xc5,0x8e) /* Ŏ */
    case 0x151 /* ő */: BSEQ(0xc5,0x90) /* Ő */
    case 0x153 /* œ */: BSEQ(0xc5,0x92) /* Œ */
    case 0x155 /* ŕ */: BSEQ(0xc5,0x94) /* Ŕ */
    case 0x157 /* ŗ */: BSEQ(0xc5,0x96) /* Ŗ */
    case 0x159 /* ř */: BSEQ(0xc5,0x98) /* Ř */
    case 0x15b /* ś */: BSEQ(0xc5,0x9a) /* Ś */
    case 0x15d /* ŝ */: BSEQ(0xc5,0x9c) /* Ŝ */
    case 0x15f /* ş */: BSEQ(0xc5,0x9e) /* Ş */
    case 0x161 /* š */: BSEQ(0xc5,0xa0) /* Š */
    case 0x163 /* ţ */: BSEQ(0xc5,0xa2) /* Ţ */
    case 0x165 /* ť */: BSEQ(0xc5,0xa4) /* Ť */
    case 0x167 /* ŧ */: BSEQ(0xc5,0xa6) /* Ŧ */
    case 0x169 /* ũ */: BSEQ(0xc5,0xa8) /* Ũ */
    case 0x16b /* ū */: BSEQ(0xc5,0xaa) /* Ū */
    case 0x16d /* ŭ */: BSEQ(0xc5,0xac) /* Ŭ */
    case 0x16f /* ů */: BSEQ(0xc5,0xae) /* Ů */
    case 0x171 /* ű */: BSEQ(0xc5,0xb0) /* Ű */
    case 0x173 /* ų */: BSEQ(0xc5,0xb2) /* Ų */
    case 0x175 /* ŵ */: BSEQ(0xc5,0xb4) /* Ŵ */
    case 0x177 /* ŷ */: BSEQ(0xc5,0xb6) /* Ŷ */
    case 0x17a /* ź */: BSEQ(0xc5,0xb9) /* Ź */
    case 0x17c /* ż */: BSEQ(0xc5,0xbb) /* Ż */
    case 0x17e /* ž */: BSEQ(0xc5,0xbd) /* Ž */
    case 0x17f /* ſ */: BSEQ(0x53) /* S */
    case 0x180 /* ƀ */: BSEQ(0xc9,0x83) /* Ƀ */
    case 0x183 /* ƃ */: BSEQ(0xc6,0x82) /* Ƃ */
    case 0x185 /* ƅ */: BSEQ(0xc6,0x84) /* Ƅ */
    case 0x188 /* ƈ */: BSEQ(0xc6,0x87) /* Ƈ */
    case 0x18c /* ƌ */: BSEQ(0xc6,0x8b) /* Ƌ */
    case 0x192 /* ƒ */: BSEQ(0xc6,0x91) /* Ƒ */
    case 0x195 /* ƕ */: BSEQ(0xc7,0xb6) /* Ƕ */
    case 0x199 /* ƙ */: BSEQ(0xc6,0x98) /* Ƙ */
    case 0x19a /* ƚ */: BSEQ(0xc8,0xbd) /* Ƚ */
    case 0x19e /* ƞ */: BSEQ(0xc8,0xa0) /* Ƞ */
    case 0x1a1 /* ơ */: BSEQ(0xc6,0xa0) /* Ơ */
    case 0x1a3 /* ƣ */: BSEQ(0xc6,0xa2) /* Ƣ */
    case 0x1a5 /* ƥ */: BSEQ(0xc6,0xa4) /* Ƥ */
    case 0x1a8 /* ƨ */: BSEQ(0xc6,0xa7) /* Ƨ */
    case 0x1ad /* ƭ */: BSEQ(0xc6,0xac) /* Ƭ */
    case 0x1b0 /* ư */: BSEQ(0xc6,0xaf) /* Ư */
    case 0x1b4 /* ƴ */: BSEQ(0xc6,0xb3) /* Ƴ */
    case 0x1b6 /* ƶ */: BSEQ(0xc6,0xb5) /* Ƶ */
    case 0x1b9 /* ƹ */: BSEQ(0xc6,0xb8) /* Ƹ */
    case 0x1bd /* ƽ */: BSEQ(0xc6,0xbc) /* Ƽ */
    case 0x1bf /* ƿ */: BSEQ(0xc7,0xb7) /* Ƿ */
    case 0x1c5 /* ǅ */: BSEQ(0xc7,0x84) /* Ǆ */
    case 0x1c6 /* ǆ */: BSEQ(0xc7,0x84) /* Ǆ */
    case 0x1c8 /* ǈ */: BSEQ(0xc7,0x87) /* Ǉ */
    case 0x1c9 /* ǉ */: BSEQ(0xc7,0x87) /* Ǉ */
    case 0x1cb /* ǋ */: BSEQ(0xc7,0x8a) /* Ǌ */
    case 0x1cc /* ǌ */: BSEQ(0xc7,0x8a) /* Ǌ */
    case 0x1ce /* ǎ */: BSEQ(0xc7,0x8d) /* Ǎ */
    case 0x1d0 /* ǐ */: BSEQ(0xc7,0x8f) /* Ǐ */
    case 0x1d2 /* ǒ */: BSEQ(0xc7,0x91) /* Ǒ */
    case 0x1d4 /* ǔ */: BSEQ(0xc7,0x93) /* Ǔ */
    case 0x1d6 /* ǖ */: BSEQ(0xc7,0x95) /* Ǖ */
    case 0x1d8 /* ǘ */: BSEQ(0xc7,0x97) /* Ǘ */
    case 0x1da /* ǚ */: BSEQ(0xc7,0x99) /* Ǚ */
    case 0x1dc /* ǜ */: BSEQ(0xc7,0x9b) /* Ǜ */
    case 0x1dd /* ǝ */: BSEQ(0xc6,0x8e) /* Ǝ */
    case 0x1df /* ǟ */: BSEQ(0xc7,0x9e) /* Ǟ */
    case 0x1e1 /* ǡ */: BSEQ(0xc7,0xa0) /* Ǡ */
    case 0x1e3 /* ǣ */: BSEQ(0xc7,0xa2) /* Ǣ */
    case 0x1e5 /* ǥ */: BSEQ(0xc7,0xa4) /* Ǥ */
    case 0x1e7 /* ǧ */: BSEQ(0xc7,0xa6) /* Ǧ */
    case 0x1e9 /* ǩ */: BSEQ(0xc7,0xa8) /* Ǩ */
    case 0x1eb /* ǫ */: BSEQ(0xc7,0xaa) /* Ǫ */
    case 0x1ed /* ǭ */: BSEQ(0xc7,0xac) /* Ǭ */
    case 0x1ef /* ǯ */: BSEQ(0xc7,0xae) /* Ǯ */
    case 0x1f0 /* ǰ */: BSEQ(0x4a,0xcc,0x8c) /* J̌ */
    case 0x1f2 /* ǲ */: BSEQ(0xc7,0xb1) /* Ǳ */
    case 0x1f3 /* ǳ */: BSEQ(0xc7,0xb1) /* Ǳ */
    case 0x1f5 /* ǵ */: BSEQ(0xc7,0xb4) /* Ǵ */
    case 0x1f9 /* ǹ */: BSEQ(0xc7,0xb8) /* Ǹ */
    case 0x1fb /* ǻ */: BSEQ(0xc7,0xba) /* Ǻ */
    case 0x1fd /* ǽ */: BSEQ(0xc7,0xbc) /* Ǽ */
    case 0x1ff /* ǿ */: BSEQ(0xc7,0xbe) /* Ǿ */
    case 0x201 /* ȁ */: BSEQ(0xc8,0x80) /* Ȁ */
    case 0x203 /* ȃ */: BSEQ(0xc8,0x82) /* Ȃ */
    case 0x205 /* ȅ */: BSEQ(0xc8,0x84) /* Ȅ */
    case 0x207 /* ȇ */: BSEQ(0xc8,0x86) /* Ȇ */
    case 0x209 /* ȉ */: BSEQ(0xc8,0x88) /* Ȉ */
    case 0x20b /* ȋ */: BSEQ(0xc8,0x8a) /* Ȋ */
    case 0x20d /* ȍ */: BSEQ(0xc8,0x8c) /* Ȍ */
    case 0x20f /* ȏ */: BSEQ(0xc8,0x8e) /* Ȏ */
    case 0x211 /* ȑ */: BSEQ(0xc8,0x90) /* Ȑ */
    case 0x213 /* ȓ */: BSEQ(0xc8,0x92) /* Ȓ */
    case 0x215 /* ȕ */: BSEQ(0xc8,0x94) /* Ȕ */
    case 0x217 /* ȗ */: BSEQ(0xc8,0x96) /* Ȗ */
    case 0x219 /* ș */: BSEQ(0xc8,0x98) /* Ș */
    case 0x21b /* ț */: BSEQ(0xc8,0x9a) /* Ț */
    case 0x21d /* ȝ */: BSEQ(0xc8,0x9c) /* Ȝ */
    case 0x21f /* ȟ */: BSEQ(0xc8,0x9e) /* Ȟ */
    case 0x223 /* ȣ */: BSEQ(0xc8,0xa2) /* Ȣ */
    case 0x225 /* ȥ */: BSEQ(0xc8,0xa4) /* Ȥ */
    case 0x227 /* ȧ */: BSEQ(0xc8,0xa6) /* Ȧ */
    case 0x229 /* ȩ */: BSEQ(0xc8,0xa8) /* Ȩ */
    case 0x22b /* ȫ */: BSEQ(0xc8,0xaa) /* Ȫ */
    case 0x22d /* ȭ */: BSEQ(0xc8,0xac) /* Ȭ */
    case 0x22f /* ȯ */: BSEQ(0xc8,0xae) /* Ȯ */
    case 0x231 /* ȱ */: BSEQ(0xc8,0xb0) /* Ȱ */
    case 0x233 /* ȳ */: BSEQ(0xc8,0xb2) /* Ȳ */
    case 0x23c /* ȼ */: BSEQ(0xc8,0xbb) /* Ȼ */
    case 0x23f /* ȿ */: BSEQ(0xe2,0xb1,0xbe) /* Ȿ */
    case 0x240 /* ɀ */: BSEQ(0xe2,0xb1,0xbf) /* Ɀ */
    case 0x242 /* ɂ */: BSEQ(0xc9,0x81) /* Ɂ */
    case 0x247 /* ɇ */: BSEQ(0xc9,0x86) /* Ɇ */
    case 0x249 /* ɉ */: BSEQ(0xc9,0x88) /* Ɉ */
    case 0x24b /* ɋ */: BSEQ(0xc9,0x8a) /* Ɋ */
    case 0x24d /* ɍ */: BSEQ(0xc9,0x8c) /* Ɍ */
    case 0x24f /* ɏ */: BSEQ(0xc9,0x8e) /* Ɏ */
    case 0x250 /* ɐ */: BSEQ(0xe2,0xb1,0xaf) /* Ɐ */
    case 0x251 /* ɑ */: BSEQ(0xe2,0xb1,0xad) /* Ɑ */
    case 0x252 /* ɒ */: BSEQ(0xe2,0xb1,0xb0) /* Ɒ */
    case 0x253 /* ɓ */: BSEQ(0xc6,0x81) /* Ɓ */
    case 0x254 /* ɔ */: BSEQ(0xc6,0x86) /* Ɔ */
    case 0x256 /* ɖ */: BSEQ(0xc6,0x89) /* Ɖ */
    case 0x257 /* ɗ */: BSEQ(0xc6,0x8a) /* Ɗ */
    case 0x259 /* ə */: BSEQ(0xc6,0x8f) /* Ə */
    case 0x25b /* ɛ */: BSEQ(0xc6,0x90) /* Ɛ */
    case 0x25c /* ɜ */: BSEQ(0xea,0x9e,0xab) /* Ɜ */
    case 0x260 /* ɠ */: BSEQ(0xc6,0x93) /* Ɠ */
    case 0x261 /* ɡ */: BSEQ(0xea,0x9e,0xac) /* Ɡ */
    case 0x263 /* ɣ */: BSEQ(0xc6,0x94) /* Ɣ */
    case 0x265 /* ɥ */: BSEQ(0xea,0x9e,0x8d) /* Ɥ */
    case 0x266 /* ɦ */: BSEQ(0xea,0x9e,0xaa) /* Ɦ */
    case 0x268 /* ɨ */: BSEQ(0xc6,0x97) /* Ɨ */
    case 0x269 /* ɩ */: BSEQ(0xc6,0x96) /* Ɩ */
    case 0x26a /* ɪ */: BSEQ(0xea,0x9e,0xae) /* Ɪ */
    case 0x26b /* ɫ */: BSEQ(0xe2,0xb1,0xa2) /* Ɫ */
    case 0x26c /* ɬ */: BSEQ(0xea,0x9e,0xad) /* Ɬ */
    case 0x26f /* ɯ */: BSEQ(0xc6,0x9c) /* Ɯ */
    case 0x271 /* ɱ */: BSEQ(0xe2,0xb1,0xae) /* Ɱ */
    case 0x272 /* ɲ */: BSEQ(0xc6,0x9d) /* Ɲ */
    case 0x275 /* ɵ */: BSEQ(0xc6,0x9f) /* Ɵ */
    case 0x27d /* ɽ */: BSEQ(0xe2,0xb1,0xa4) /* Ɽ */
    case 0x280 /* ʀ */: BSEQ(0xc6,0xa6) /* Ʀ */
    case 0x282 /* ʂ */: BSEQ(0xea,0x9f,0x85) /* Ʂ */
    case 0x283 /* ʃ */: BSEQ(0xc6,0xa9) /* Ʃ */
    case 0x287 /* ʇ */: BSEQ(0xea,0x9e,0xb1) /* Ʇ */
    case 0x288 /* ʈ */: BSEQ(0xc6,0xae) /* Ʈ */
    case 0x289 /* ʉ */: BSEQ(0xc9,0x84) /* Ʉ */
    case 0x28a /* ʊ */: BSEQ(0xc6,0xb1) /* Ʊ */
    case 0x28b /* ʋ */: BSEQ(0xc6,0xb2) /* Ʋ */
    case 0x28c /* ʌ */: BSEQ(0xc9,0x85) /* Ʌ */
    case 0x292 /* ʒ */: BSEQ(0xc6,0xb7) /* Ʒ */
    case 0x29d /* ʝ */: BSEQ(0xea,0x9e,0xb2) /* Ʝ */
    case 0x29e /* ʞ */: BSEQ(0xea,0x9e,0xb0) /* Ʞ */
    case 0x345 /* ͅ */: BSEQ(0xce,0x99) /* Ι */
    case 0x371 /* ͱ */: BSEQ(0xcd,0xb0) /* Ͱ */
    case 0x373 /* ͳ */: BSEQ(0xcd,0xb2) /* Ͳ */
    case 0x377 /* ͷ */: BSEQ(0xcd,0xb6) /* Ͷ */
    case 0x37b /* ͻ */: BSEQ(0xcf,0xbd) /* Ͻ */
    case 0x37c /* ͼ */: BSEQ(0xcf,0xbe) /* Ͼ */
    case 0x37d /* ͽ */: BSEQ(0xcf,0xbf) /* Ͽ */
    case 0x390 /* ΐ */: BSEQ(0xce,0x99,0xcc,0x88,0xcc,0x81) /* Ϊ́ */
    case 0x3ac /* ά */: BSEQ(0xce,0x86) /* Ά */
    case 0x3ad /* έ */: BSEQ(0xce,0x88) /* Έ */
    case 0x3ae /* ή */: BSEQ(0xce,0x89) /* Ή */
    case 0x3af /* ί */: BSEQ(0xce,0x8a) /* Ί */
    case 0x3b0 /* ΰ */: BSEQ(0xce,0xa5,0xcc,0x88,0xcc,0x81) /* Ϋ́ */
    case 0x3b1 /* α */: BSEQ(0xce,0x91) /* Α */
    case 0x3b2 /* β */: BSEQ(0xce,0x92) /* Β */
    case 0x3b3 /* γ */: BSEQ(0xce,0x93) /* Γ */
    case 0x3b4 /* δ */: BSEQ(0xce,0x94) /* Δ */
    case 0x3b5 /* ε */: BSEQ(0xce,0x95) /* Ε */
    case 0x3b6 /* ζ */: BSEQ(0xce,0x96) /* Ζ */
    case 0x3b7 /* η */: BSEQ(0xce,0x97) /* Η */
    case 0x3b8 /* θ */: BSEQ(0xce,0x98) /* Θ */
    case 0x3b9 /* ι */: BSEQ(0xce,0x99) /* Ι */
    case 0x3ba /* κ */: BSEQ(0xce,0x9a) /* Κ */
    case 0x3bb /* λ */: BSEQ(0xce,0x9b) /* Λ */
    case 0x3bc /* μ */: BSEQ(0xce,0x9c) /* Μ */
    case 0x3bd /* ν */: BSEQ(0xce,0x9d) /* Ν */
    case 0x3be /* ξ */: BSEQ(0xce,0x9e) /* Ξ */
    case 0x3bf /* ο */: BSEQ(0xce,0x9f) /* Ο */
    case 0x3c0 /* π */: BSEQ(0xce,0xa0) /* Π */
    case 0x3c1 /* ρ */: BSEQ(0xce,0xa1) /* Ρ */
    case 0x3c2 /* ς */: BSEQ(0xce,0xa3) /* Σ */
    case 0x3c3 /* σ */: BSEQ(0xce,0xa3) /* Σ */
    case 0x3c4 /* τ */: BSEQ(0xce,0xa4) /* Τ */
    case 0x3c5 /* υ */: BSEQ(0xce,0xa5) /* Υ */
    case 0x3c6 /* φ */: BSEQ(0xce,0xa6) /* Φ */
    case 0x3c7 /* χ */: BSEQ(0xce,0xa7) /* Χ */
    case 0x3c8 /* ψ */: BSEQ(0xce,0xa8) /* Ψ */
    case 0x3c9 /* ω */: BSEQ(0xce,0xa9) /* Ω */
    case 0x3ca /* ϊ */: BSEQ(0xce,0xaa) /* Ϊ */
    case 0x3cb /* ϋ */: BSEQ(0xce,0xab) /* Ϋ */
    case 0x3cc /* ό */: BSEQ(0xce,0x8c) /* Ό */
    case 0x3cd /* ύ */: BSEQ(0xce,0x8e) /* Ύ */
    case 0x3ce /* ώ */: BSEQ(0xce,0x8f) /* Ώ */
    case 0x3d0 /* ϐ */: BSEQ(0xce,0x92) /* Β */
    case 0x3d1 /* ϑ */: BSEQ(0xce,0x98) /* Θ */
    case 0x3d5 /* ϕ */: BSEQ(0xce,0xa6) /* Φ */
    case 0x3d6 /* ϖ */: BSEQ(0xce,0xa0) /* Π */
    case 0x3d7 /* ϗ */: BSEQ(0xcf,0x8f) /* Ϗ */
    case 0x3d9 /* ϙ */: BSEQ(0xcf,0x98) /* Ϙ */
    case 0x3db /* ϛ */: BSEQ(0xcf,0x9a) /* Ϛ */
    case 0x3dd /* ϝ */: BSEQ(0xcf,0x9c) /* Ϝ */
    case 0x3df /* ϟ */: BSEQ(0xcf,0x9e) /* Ϟ */
    case 0x3e1 /* ϡ */: BSEQ(0xcf,0xa0) /* Ϡ */
    case 0x3e3 /* ϣ */: BSEQ(0xcf,0xa2) /* Ϣ */
    case 0x3e5 /* ϥ */: BSEQ(0xcf,0xa4) /* Ϥ */
    case 0x3e7 /* ϧ */: BSEQ(0xcf,0xa6) /* Ϧ */
    case 0x3e9 /* ϩ */: BSEQ(0xcf,0xa8) /* Ϩ */
    case 0x3eb /* ϫ */: BSEQ(0xcf,0xaa) /* Ϫ */
    case 0x3ed /* ϭ */: BSEQ(0xcf,0xac) /* Ϭ */
    case 0x3ef /* ϯ */: BSEQ(0xcf,0xae) /* Ϯ */
    case 0x3f0 /* ϰ */: BSEQ(0xce,0x9a) /* Κ */
    case 0x3f1 /* ϱ */: BSEQ(0xce,0xa1) /* Ρ */
    case 0x3f2 /* ϲ */: BSEQ(0xcf,0xb9) /* Ϲ */
    case 0x3f3 /* ϳ */: BSEQ(0xcd,0xbf) /* Ϳ */
    case 0x3f5 /* ϵ */: BSEQ(0xce,0x95) /* Ε */
    case 0x3f8 /* ϸ */: BSEQ(0xcf,0xb7) /* Ϸ */
    case 0x3fb /* ϻ */: BSEQ(0xcf,0xba) /* Ϻ */
    case 0x430 /* а */: BSEQ(0xd0,0x90) /* А */
    case 0x431 /* б */: BSEQ(0xd0,0x91) /* Б */
    case 0x432 /* в */: BSEQ(0xd0,0x92) /* В */
    case 0x433 /* г */: BSEQ(0xd0,0x93) /* Г */
    case 0x434 /* д */: BSEQ(0xd0,0x94) /* Д */
    case 0x435 /* е */: BSEQ(0xd0,0x95) /* Е */
    case 0x436 /* ж */: BSEQ(0xd0,0x96) /* Ж */
    case 0x437 /* з */: BSEQ(0xd0,0x97) /* З */
    case 0x438 /* и */: BSEQ(0xd0,0x98) /* И */
    case 0x439 /* й */: BSEQ(0xd0,0x99) /* Й */
    case 0x43a /* к */: BSEQ(0xd0,0x9a) /* К */
    case 0x43b /* л */: BSEQ(0xd0,0x9b) /* Л */
    case 0x43c /* м */: BSEQ(0xd0,0x9c) /* М */
    case 0x43d /* н */: BSEQ(0xd0,0x9d) /* Н */
    case 0x43e /* о */: BSEQ(0xd0,0x9e) /* О */
    case 0x43f /* п */: BSEQ(0xd0,0x9f) /* П */
    case 0x440 /* р */: BSEQ(0xd0,0xa0) /* Р */
    case 0x441 /* с */: BSEQ(0xd0,0xa1) /* С */
    case 0x442 /* т */: BSEQ(0xd0,0xa2) /* Т */
    case 0x443 /* у */: BSEQ(0xd0,0xa3) /* У */
    case 0x444 /* ф */: BSEQ(0xd0,0xa4) /* Ф */
    case 0x445 /* х */: BSEQ(0xd0,0xa5) /* Х */
    case 0x446 /* ц */: BSEQ(0xd0,0xa6) /* Ц */
    case 0x447 /* ч */: BSEQ(0xd0,0xa7) /* Ч */
    case 0x448 /* ш */: BSEQ(0xd0,0xa8) /* Ш */
    case 0x449 /* щ */: BSEQ(0xd0,0xa9) /* Щ */
    case 0x44a /* ъ */: BSEQ(0xd0,0xaa) /* Ъ */
    case 0x44b /* ы */: BSEQ(0xd0,0xab) /* Ы */
    case 0x44c /* ь */: BSEQ(0xd0,0xac) /* Ь */
    case 0x44d /* э */: BSEQ(0xd0,0xad) /* Э */
    case 0x44e /* ю */: BSEQ(0xd0,0xae) /* Ю */
    case 0x44f /* я */: BSEQ(0xd0,0xaf) /* Я */
    case 0x450 /* ѐ */: BSEQ(0xd0,0x80) /* Ѐ */
    case 0x451 /* ё */: BSEQ(0xd0,0x81) /* Ё */
    case 0x452 /* ђ */: BSEQ(0xd0,0x82) /* Ђ */
    case 0x453 /* ѓ */: BSEQ(0xd0,0x83) /* Ѓ */
    case 0x454 /* є */: BSEQ(0xd0,0x84) /* Є */
    case 0x455 /* ѕ */: BSEQ(0xd0,0x85) /* Ѕ */
    case 0x456 /* і */: BSEQ(0xd0,0x86) /* І */
    case 0x457 /* ї */: BSEQ(0xd0,0x87) /* Ї */
    case 0x458 /* ј */: BSEQ(0xd0,0x88) /* Ј */
    case 0x459 /* љ */: BSEQ(0xd0,0x89) /* Љ */
    case 0x45a /* њ */: BSEQ(0xd0,0x8a) /* Њ */
    case 0x45b /* ћ */: BSEQ(0xd0,0x8b) /* Ћ */
    case 0x45c /* ќ */: BSEQ(0xd0,0x8c) /* Ќ */
    case 0x45d /* ѝ */: BSEQ(0xd0,0x8d) /* Ѝ */
    case 0x45e /* ў */: BSEQ(0xd0,0x8e) /* Ў */
    case 0x45f /* џ */: BSEQ(0xd0,0x8f) /* Џ */
    case 0x461 /* ѡ */: BSEQ(0xd1,0xa0) /* Ѡ */
    case 0x463 /* ѣ */: BSEQ(0xd1,0xa2) /* Ѣ */
    case 0x465 /* ѥ */: BSEQ(0xd1,0xa4) /* Ѥ */
    case 0x467 /* ѧ */: BSEQ(0xd1,0xa6) /* Ѧ */
    case 0x469 /* ѩ */: BSEQ(0xd1,0xa8) /* Ѩ */
    case 0x46b /* ѫ */: BSEQ(0xd1,0xaa) /* Ѫ */
    case 0x46d /* ѭ */: BSEQ(0xd1,0xac) /* Ѭ */
    case 0x46f /* ѯ */: BSEQ(0xd1,0xae) /* Ѯ */
    case 0x471 /* ѱ */: BSEQ(0xd1,0xb0) /* Ѱ */
    case 0x473 /* ѳ */: BSEQ(0xd1,0xb2) /* Ѳ */
    case 0x475 /* ѵ */: BSEQ(0xd1,0xb4) /* Ѵ */
    case 0x477 /* ѷ */: BSEQ(0xd1,0xb6) /* Ѷ */
    case 0x479 /* ѹ */: BSEQ(0xd1,0xb8) /* Ѹ */
    case 0x47b /* ѻ */: BSEQ(0xd1,0xba) /* Ѻ */
    case 0x47d /* ѽ */: BSEQ(0xd1,0xbc) /* Ѽ */
    case 0x47f /* ѿ */: BSEQ(0xd1,0xbe) /* Ѿ */
    case 0x481 /* ҁ */: BSEQ(0xd2,0x80) /* Ҁ */
    case 0x48b /* ҋ */: BSEQ(0xd2,0x8a) /* Ҋ */
    case 0x48d /* ҍ */: BSEQ(0xd2,0x8c) /* Ҍ */
    case 0x48f /* ҏ */: BSEQ(0xd2,0x8e) /* Ҏ */
    case 0x491 /* ґ */: BSEQ(0xd2,0x90) /* Ґ */
    case 0x493 /* ғ */: BSEQ(0xd2,0x92) /* Ғ */
    case 0x495 /* ҕ */: BSEQ(0xd2,0x94) /* Ҕ */
    case 0x497 /* җ */: BSEQ(0xd2,0x96) /* Җ */
    case 0x499 /* ҙ */: BSEQ(0xd2,0x98) /* Ҙ */
    case 0x49b /* қ */: BSEQ(0xd2,0x9a) /* Қ */
    case 0x49d /* ҝ */: BSEQ(0xd2,0x9c) /* Ҝ */
    case 0x49f /* ҟ */: BSEQ(0xd2,0x9e) /* Ҟ */
    case 0x4a1 /* ҡ */: BSEQ(0xd2,0xa0) /* Ҡ */
    case 0x4a3 /* ң */: BSEQ(0xd2,0xa2) /* Ң */
    case 0x4a5 /* ҥ */: BSEQ(0xd2,0xa4) /* Ҥ */
    case 0x4a7 /* ҧ */: BSEQ(0xd2,0xa6) /* Ҧ */
    case 0x4a9 /* ҩ */: BSEQ(0xd2,0xa8) /* Ҩ */
    case 0x4ab /* ҫ */: BSEQ(0xd2,0xaa) /* Ҫ */
    case 0x4ad /* ҭ */: BSEQ(0xd2,0xac) /* Ҭ */
    case 0x4af /* ү */: BSEQ(0xd2,0xae) /* Ү */
    case 0x4b1 /* ұ */: BSEQ(0xd2,0xb0) /* Ұ */
    case 0x4b3 /* ҳ */: BSEQ(0xd2,0xb2) /* Ҳ */
    case 0x4b5 /* ҵ */: BSEQ(0xd2,0xb4) /* Ҵ */
    case 0x4b7 /* ҷ */: BSEQ(0xd2,0xb6) /* Ҷ */
    case 0x4b9 /* ҹ */: BSEQ(0xd2,0xb8) /* Ҹ */
    case 0x4bb /* һ */: BSEQ(0xd2,0xba) /* Һ */
    case 0x4bd /* ҽ */: BSEQ(0xd2,0xbc) /* Ҽ */
    case 0x4bf /* ҿ */: BSEQ(0xd2,0xbe) /* Ҿ */
    case 0x4c2 /* ӂ */: BSEQ(0xd3,0x81) /* Ӂ */
    case 0x4c4 /* ӄ */: BSEQ(0xd3,0x83) /* Ӄ */
    case 0x4c6 /* ӆ */: BSEQ(0xd3,0x85) /* Ӆ */
    case 0x4c8 /* ӈ */: BSEQ(0xd3,0x87) /* Ӈ */
    case 0x4ca /* ӊ */: BSEQ(0xd3,0x89) /* Ӊ */
    case 0x4cc /* ӌ */: BSEQ(0xd3,0x8b) /* Ӌ */
    case 0x4ce /* ӎ */: BSEQ(0xd3,0x8d) /* Ӎ */
    case 0x4cf /* ӏ */: BSEQ(0xd3,0x80) /* Ӏ */
    case 0x4d1 /* ӑ */: BSEQ(0xd3,0x90) /* Ӑ */
    case 0x4d3 /* ӓ */: BSEQ(0xd3,0x92) /* Ӓ */
    case 0x4d5 /* ӕ */: BSEQ(0xd3,0x94) /* Ӕ */
    case 0x4d7 /* ӗ */: BSEQ(0xd3,0x96) /* Ӗ */
    case 0x4d9 /* ә */: BSEQ(0xd3,0x98) /* Ә */
    case 0x4db /* ӛ */: BSEQ(0xd3,0x9a) /* Ӛ */
    case 0x4dd /* ӝ */: BSEQ(0xd3,0x9c) /* Ӝ */
    case 0x4df /* ӟ */: BSEQ(0xd3,0x9e) /* Ӟ */
    case 0x4e1 /* ӡ */: BSEQ(0xd3,0xa0) /* Ӡ */
    case 0x4e3 /* ӣ */: BSEQ(0xd3,0xa2) /* Ӣ */
    case 0x4e5 /* ӥ */: BSEQ(0xd3,0xa4) /* Ӥ */
    case 0x4e7 /* ӧ */: BSEQ(0xd3,0xa6) /* Ӧ */
    case 0x4e9 /* ө */: BSEQ(0xd3,0xa8) /* Ө */
    case 0x4eb /* ӫ */: BSEQ(0xd3,0xaa) /* Ӫ */
    case 0x4ed /* ӭ */: BSEQ(0xd3,0xac) /* Ӭ */
    case 0x4ef /* ӯ */: BSEQ(0xd3,0xae) /* Ӯ */
    case 0x4f1 /* ӱ */: BSEQ(0xd3,0xb0) /* Ӱ */
    case 0x4f3 /* ӳ */: BSEQ(0xd3,0xb2) /* Ӳ */
    case 0x4f5 /* ӵ */: BSEQ(0xd3,0xb4) /* Ӵ */
    case 0x4f7 /* ӷ */: BSEQ(0xd3,0xb6) /* Ӷ */
    case 0x4f9 /* ӹ */: BSEQ(0xd3,0xb8) /* Ӹ */
    case 0x4fb /* ӻ */: BSEQ(0xd3,0xba) /* Ӻ */
    case 0x4fd /* ӽ */: BSEQ(0xd3,0xbc) /* Ӽ */
    case 0x4ff /* ӿ */: BSEQ(0xd3,0xbe) /* Ӿ */
    case 0x501 /* ԁ */: BSEQ(0xd4,0x80) /* Ԁ */
    case 0x503 /* ԃ */: BSEQ(0xd4,0x82) /* Ԃ */
    case 0x505 /* ԅ */: BSEQ(0xd4,0x84) /* Ԅ */
    case 0x507 /* ԇ */: BSEQ(0xd4,0x86) /* Ԇ */
    case 0x509 /* ԉ */: BSEQ(0xd4,0x88) /* Ԉ */
    case 0x50b /* ԋ */: BSEQ(0xd4,0x8a) /* Ԋ */
    case 0x50d /* ԍ */: BSEQ(0xd4,0x8c) /* Ԍ */
    case 0x50f /* ԏ */: BSEQ(0xd4,0x8e) /* Ԏ */
    case 0x511 /* ԑ */: BSEQ(0xd4,0x90) /* Ԑ */
    case 0x513 /* ԓ */: BSEQ(0xd4,0x92) /* Ԓ */
    case 0x515 /* ԕ */: BSEQ(0xd4,0x94) /* Ԕ */
    case 0x517 /* ԗ */: BSEQ(0xd4,0x96) /* Ԗ */
    case 0x519 /* ԙ */: BSEQ(0xd4,0x98) /* Ԙ */
    case 0x51b /* ԛ */: BSEQ(0xd4,0x9a) /* Ԛ */
    case 0x51d /* ԝ */: BSEQ(0xd4,0x9c) /* Ԝ */
    case 0x51f /* ԟ */: BSEQ(0xd4,0x9e) /* Ԟ */
    case 0x521 /* ԡ */: BSEQ(0xd4,0xa0) /* Ԡ */
    case 0x523 /* ԣ */: BSEQ(0xd4,0xa2) /* Ԣ */
    case 0x525 /* ԥ */: BSEQ(0xd4,0xa4) /* Ԥ */
    case 0x527 /* ԧ */: BSEQ(0xd4,0xa6) /* Ԧ */
    case 0x529 /* ԩ */: BSEQ(0xd4,0xa8) /* Ԩ */
    case 0x52b /* ԫ */: BSEQ(0xd4,0xaa) /* Ԫ */
    case 0x52d /* ԭ */: BSEQ(0xd4,0xac) /* Ԭ */
    case 0x52f /* ԯ */: BSEQ(0xd4,0xae) /* Ԯ */
    case 0x561 /* ա */: BSEQ(0xd4,0xb1) /* Ա */
    case 0x562 /* բ */: BSEQ(0xd4,0xb2) /* Բ */
    case 0x563 /* գ */: BSEQ(0xd4,0xb3) /* Գ */
    case 0x564 /* դ */: BSEQ(0xd4,0xb4) /* Դ */
    case 0x565 /* ե */: BSEQ(0xd4,0xb5) /* Ե */
    case 0x566 /* զ */: BSEQ(0xd4,0xb6) /* Զ */
    case 0x567 /* է */: BSEQ(0xd4,0xb7) /* Է */
    case 0x568 /* ը */: BSEQ(0xd4,0xb8) /* Ը */
    case 0x569 /* թ */: BSEQ(0xd4,0xb9) /* Թ */
    case 0x56a /* ժ */: BSEQ(0xd4,0xba) /* Ժ */
    case 0x56b /* ի */: BSEQ(0xd4,0xbb) /* Ի */
    case 0x56c /* լ */: BSEQ(0xd4,0xbc) /* Լ */
    case 0x56d /* խ */: BSEQ(0xd4,0xbd) /* Խ */
    case 0x56e /* ծ */: BSEQ(0xd4,0xbe) /* Ծ */
    case 0x56f /* կ */: BSEQ(0xd4,0xbf) /* Կ */
    case 0x570 /* հ */: BSEQ(0xd5,0x80) /* Հ */
    case 0x571 /* ձ */: BSEQ(0xd5,0x81) /* Ձ */
    case 0x572 /* ղ */: BSEQ(0xd5,0x82) /* Ղ */
    case 0x573 /* ճ */: BSEQ(0xd5,0x83) /* Ճ */
    case 0x574 /* մ */: BSEQ(0xd5,0x84) /* Մ */
    case 0x575 /* յ */: BSEQ(0xd5,0x85) /* Յ */
    case 0x576 /* ն */: BSEQ(0xd5,0x86) /* Ն */
    case 0x577 /* շ */: BSEQ(0xd5,0x87) /* Շ */
    case 0x578 /* ո */: BSEQ(0xd5,0x88) /* Ո */
    case 0x579 /* չ */: BSEQ(0xd5,0x89) /* Չ */
    case 0x57a /* պ */: BSEQ(0xd5,0x8a) /* Պ */
    case 0x57b /* ջ */: BSEQ(0xd5,0x8b) /* Ջ */
    case 0x57c /* ռ */: BSEQ(0xd5,0x8c) /* Ռ */
    case 0x57d /* ս */: BSEQ(0xd5,0x8d) /* Ս */
    case 0x57e /* վ */: BSEQ(0xd5,0x8e) /* Վ */
    case 0x57f /* տ */: BSEQ(0xd5,0x8f) /* Տ */
    case 0x580 /* ր */: BSEQ(0xd5,0x90) /* Ր */
    case 0x581 /* ց */: BSEQ(0xd5,0x91) /* Ց */
    case 0x582 /* ւ */: BSEQ(0xd5,0x92) /* Ւ */
    case 0x583 /* փ */: BSEQ(0xd5,0x93) /* Փ */
    case 0x584 /* ք */: BSEQ(0xd5,0x94) /* Ք */
    case 0x585 /* օ */: BSEQ(0xd5,0x95) /* Օ */
    case 0x586 /* ֆ */: BSEQ(0xd5,0x96) /* Ֆ */
    case 0x587 /* և */: BSEQ(0xd4,0xb5,0xd5,0x92) /* ԵՒ */
    case 0x10d0 /* ა */: BSEQ(0xe1,0xb2,0x90) /* Ა */
    case 0x10d1 /* ბ */: BSEQ(0xe1,0xb2,0x91) /* Ბ */
    case 0x10d2 /* გ */: BSEQ(0xe1,0xb2,0x92) /* Გ */
    case 0x10d3 /* დ */: BSEQ(0xe1,0xb2,0x93) /* Დ */
    case 0x10d4 /* ე */: BSEQ(0xe1,0xb2,0x94) /* Ე */
    case 0x10d5 /* ვ */: BSEQ(0xe1,0xb2,0x95) /* Ვ */
    case 0x10d6 /* ზ */: BSEQ(0xe1,0xb2,0x96) /* Ზ */
    case 0x10d7 /* თ */: BSEQ(0xe1,0xb2,0x97) /* Თ */
    case 0x10d8 /* ი */: BSEQ(0xe1,0xb2,0x98) /* Ი */
    case 0x10d9 /* კ */: BSEQ(0xe1,0xb2,0x99) /* Კ */
    case 0x10da /* ლ */: BSEQ(0xe1,0xb2,0x9a) /* Ლ */
    case 0x10db /* მ */: BSEQ(0xe1,0xb2,0x9b) /* Მ */
    case 0x10dc /* ნ */: BSEQ(0xe1,0xb2,0x9c) /* Ნ */
    case 0x10dd /* ო */: BSEQ(0xe1,0xb2,0x9d) /* Ო */
    case 0x10de /* პ */: BSEQ(0xe1,0xb2,0x9e) /* Პ */
    case 0x10df /* ჟ */: BSEQ(0xe1,0xb2,0x9f) /* Ჟ */
    case 0x10e0 /* რ */: BSEQ(0xe1,0xb2,0xa0) /* Რ */
    case 0x10e1 /* ს */: BSEQ(0xe1,0xb2,0xa1) /* Ს */
    case 0x10e2 /* ტ */: BSEQ(0xe1,0xb2,0xa2) /* Ტ */
    case 0x10e3 /* უ */: BSEQ(0xe1,0xb2,0xa3) /* Უ */
    case 0x10e4 /* ფ */: BSEQ(0xe1,0xb2,0xa4) /* Ფ */
    case 0x10e5 /* ქ */: BSEQ(0xe1,0xb2,0xa5) /* Ქ */
    case 0x10e6 /* ღ */: BSEQ(0xe1,0xb2,0xa6) /* Ღ */
    case 0x10e7 /* ყ */: BSEQ(0xe1,0xb2,0xa7) /* Ყ */
    case 0x10e8 /* შ */: BSEQ(0xe1,0xb2,0xa8) /* Შ */
    case 0x10e9 /* ჩ */: BSEQ(0xe1,0xb2,0xa9) /* Ჩ */
    case 0x10ea /* ც */: BSEQ(0xe1,0xb2,0xaa) /* Ც */
    case 0x10eb /* ძ */: BSEQ(0xe1,0xb2,0xab) /* Ძ */
    case 0x10ec /* წ */: BSEQ(0xe1,0xb2,0xac) /* Წ */
    case 0x10ed /* ჭ */: BSEQ(0xe1,0xb2,0xad) /* Ჭ */
    case 0x10ee /* ხ */: BSEQ(0xe1,0xb2,0xae) /* Ხ */
    case 0x10ef /* ჯ */: BSEQ(0xe1,0xb2,0xaf) /* Ჯ */
    case 0x10f0 /* ჰ */: BSEQ(0xe1,0xb2,0xb0) /* Ჰ */
    case 0x10f1 /* ჱ */: BSEQ(0xe1,0xb2,0xb1) /* Ჱ */
    case 0x10f2 /* ჲ */: BSEQ(0xe1,0xb2,0xb2) /* Ჲ */
    case 0x10f3 /* ჳ */: BSEQ(0xe1,0xb2,0xb3) /* Ჳ */
    case 0x10f4 /* ჴ */: BSEQ(0xe1,0xb2,0xb4) /* Ჴ */
    case 0x10f5 /* ჵ */: BSEQ(0xe1,0xb2,0xb5) /* Ჵ */
    case 0x10f6 /* ჶ */: BSEQ(0xe1,0xb2,0xb6) /* Ჶ */
    case 0x10f7 /* ჷ */: BSEQ(0xe1,0xb2,0xb7) /* Ჷ */
    case 0x10f8 /* ჸ */: BSEQ(0xe1,0xb2,0xb8) /* Ჸ */
    case 0x10f9 /* ჹ */: BSEQ(0xe1,0xb2,0xb9) /* Ჹ */
    case 0x10fa /* ჺ */: BSEQ(0xe1,0xb2,0xba) /* Ჺ */
    case 0x10fd /* ჽ */: BSEQ(0xe1,0xb2,0xbd) /* Ჽ */
    case 0x10fe /* ჾ */: BSEQ(0xe1,0xb2,0xbe) /* Ჾ */
    case 0x10ff /* ჿ */: BSEQ(0xe1,0xb2,0xbf) /* Ჿ */
    case 0x13f8 /* ᏸ */: BSEQ(0xe1,0x8f,0xb0) /* Ᏸ */
    case 0x13f9 /* ᏹ */: BSEQ(0xe1,0x8f,0xb1) /* Ᏹ */
    case 0x13fa /* ᏺ */: BSEQ(0xe1,0x8f,0xb2) /* Ᏺ */
    case 0x13fb /* ᏻ */: BSEQ(0xe1,0x8f,0xb3) /* Ᏻ */
    case 0x13fc /* ᏼ */: BSEQ(0xe1,0x8f,0xb4) /* Ᏼ */
    case 0x13fd /* ᏽ */: BSEQ(0xe1,0x8f,0xb5) /* Ᏽ */
    case 0x1c80 /* ᲀ */: BSEQ(0xd0,0x92) /* В */
    case 0x1c81 /* ᲁ */: BSEQ(0xd0,0x94) /* Д */
    case 0x1c82 /* ᲂ */: BSEQ(0xd0,0x9e) /* О */
    case 0x1c83 /* ᲃ */: BSEQ(0xd0,0xa1) /* С */
    case 0x1c84 /* ᲄ */: BSEQ(0xd0,0xa2) /* Т */
    case 0x1c85 /* ᲅ */: BSEQ(0xd0,0xa2) /* Т */
    case 0x1c86 /* ᲆ */: BSEQ(0xd0,0xaa) /* Ъ */
    case 0x1c87 /* ᲇ */: BSEQ(0xd1,0xa2) /* Ѣ */
    case 0x1c88 /* ᲈ */: BSEQ(0xea,0x99,0x8a) /* Ꙋ */
    case 0x1d79 /* ᵹ */: BSEQ(0xea,0x9d,0xbd) /* Ᵹ */
    case 0x1d7d /* ᵽ */: BSEQ(0xe2,0xb1,0xa3) /* Ᵽ */
    case 0x1d8e /* ᶎ */: BSEQ(0xea,0x9f,0x86) /* Ᶎ */
    case 0x1e01 /* ḁ */: BSEQ(0xe1,0xb8,0x80) /* Ḁ */
    case 0x1e03 /* ḃ */: BSEQ(0xe1,0xb8,0x82) /* Ḃ */
    case 0x1e05 /* ḅ */: BSEQ(0xe1,0xb8,0x84) /* Ḅ */
    case 0x1e07 /* ḇ */: BSEQ(0xe1,0xb8,0x86) /* Ḇ */
    case 0x1e09 /* ḉ */: BSEQ(0xe1,0xb8,0x88) /* Ḉ */
    case 0x1e0b /* ḋ */: BSEQ(0xe1,0xb8,0x8a) /* Ḋ */
    case 0x1e0d /* ḍ */: BSEQ(0xe1,0xb8,0x8c) /* Ḍ */
    case 0x1e0f /* ḏ */: BSEQ(0xe1,0xb8,0x8e) /* Ḏ */
    case 0x1e11 /* ḑ */: BSEQ(0xe1,0xb8,0x90) /* Ḑ */
    case 0x1e13 /* ḓ */: BSEQ(0xe1,0xb8,0x92) /* Ḓ */
    case 0x1e15 /* ḕ */: BSEQ(0xe1,0xb8,0x94) /* Ḕ */
    case 0x1e17 /* ḗ */: BSEQ(0xe1,0xb8,0x96) /* Ḗ */
    case 0x1e19 /* ḙ */: BSEQ(0xe1,0xb8,0x98) /* Ḙ */
    case 0x1e1b /* ḛ */: BSEQ(0xe1,0xb8,0x9a) /* Ḛ */
    case 0x1e1d /* ḝ */: BSEQ(0xe1,0xb8,0x9c) /* Ḝ */
    case 0x1e1f /* ḟ */: BSEQ(0xe1,0xb8,0x9e) /* Ḟ */
    case 0x1e21 /* ḡ */: BSEQ(0xe1,0xb8,0xa0) /* Ḡ */
    case 0x1e23 /* ḣ */: BSEQ(0xe1,0xb8,0xa2) /* Ḣ */
    case 0x1e25 /* ḥ */: BSEQ(0xe1,0xb8,0xa4) /* Ḥ */
    case 0x1e27 /* ḧ */: BSEQ(0xe1,0xb8,0xa6) /* Ḧ */
    case 0x1e29 /* ḩ */: BSEQ(0xe1,0xb8,0xa8) /* Ḩ */
    case 0x1e2b /* ḫ */: BSEQ(0xe1,0xb8,0xaa) /* Ḫ */
    case 0x1e2d /* ḭ */: BSEQ(0xe1,0xb8,0xac) /* Ḭ */
    case 0x1e2f /* ḯ */: BSEQ(0xe1,0xb8,0xae) /* Ḯ */
    case 0x1e31 /* ḱ */: BSEQ(0xe1,0xb8,0xb0) /* Ḱ */
    case 0x1e33 /* ḳ */: BSEQ(0xe1,0xb8,0xb2) /* Ḳ */
    case 0x1e35 /* ḵ */: BSEQ(0xe1,0xb8,0xb4) /* Ḵ */
    case 0x1e37 /* ḷ */: BSEQ(0xe1,0xb8,0xb6) /* Ḷ */
    case 0x1e39 /* ḹ */: BSEQ(0xe1,0xb8,0xb8) /* Ḹ */
    case 0x1e3b /* ḻ */: BSEQ(0xe1,0xb8,0xba) /* Ḻ */
    case 0x1e3d /* ḽ */: BSEQ(0xe1,0xb8,0xbc) /* Ḽ */
    case 0x1e3f /* ḿ */: BSEQ(0xe1,0xb8,0xbe) /* Ḿ */
    case 0x1e41 /* ṁ */: BSEQ(0xe1,0xb9,0x80) /* Ṁ */
    case 0x1e43 /* ṃ */: BSEQ(0xe1,0xb9,0x82) /* Ṃ */
    case 0x1e45 /* ṅ */: BSEQ(0xe1,0xb9,0x84) /* Ṅ */
    case 0x1e47 /* ṇ */: BSEQ(0xe1,0xb9,0x86) /* Ṇ */
    case 0x1e49 /* ṉ */: BSEQ(0xe1,0xb9,0x88) /* Ṉ */
    case 0x1e4b /* ṋ */: BSEQ(0xe1,0xb9,0x8a) /* Ṋ */
    case 0x1e4d /* ṍ */: BSEQ(0xe1,0xb9,0x8c) /* Ṍ */
    case 0x1e4f /* ṏ */: BSEQ(0xe1,0xb9,0x8e) /* Ṏ */
    case 0x1e51 /* ṑ */: BSEQ(0xe1,0xb9,0x90) /* Ṑ */
    case 0x1e53 /* ṓ */: BSEQ(0xe1,0xb9,0x92) /* Ṓ */
    case 0x1e55 /* ṕ */: BSEQ(0xe1,0xb9,0x94) /* Ṕ */
    case 0x1e57 /* ṗ */: BSEQ(0xe1,0xb9,0x96) /* Ṗ */
    case 0x1e59 /* ṙ */: BSEQ(0xe1,0xb9,0x98) /* Ṙ */
    case 0x1e5b /* ṛ */: BSEQ(0xe1,0xb9,0x9a) /* Ṛ */
    case 0x1e5d /* ṝ */: BSEQ(0xe1,0xb9,0x9c) /* Ṝ */
    case 0x1e5f /* ṟ */: BSEQ(0xe1,0xb9,0x9e) /* Ṟ */
    case 0x1e61 /* ṡ */: BSEQ(0xe1,0xb9,0xa0) /* Ṡ */
    case 0x1e63 /* ṣ */: BSEQ(0xe1,0xb9,0xa2) /* Ṣ */
    case 0x1e65 /* ṥ */: BSEQ(0xe1,0xb9,0xa4) /* Ṥ */
    case 0x1e67 /* ṧ */: BSEQ(0xe1,0xb9,0xa6) /* Ṧ */
    case 0x1e69 /* ṩ */: BSEQ(0xe1,0xb9,0xa8) /* Ṩ */
    case 0x1e6b /* ṫ */: BSEQ(0xe1,0xb9,0xaa) /* Ṫ */
    case 0x1e6d /* ṭ */: BSEQ(0xe1,0xb9,0xac) /* Ṭ */
    case 0x1e6f /* ṯ */: BSEQ(0xe1,0xb9,0xae) /* Ṯ */
    case 0x1e71 /* ṱ */: BSEQ(0xe1,0xb9,0xb0) /* Ṱ */
    case 0x1e73 /* ṳ */: BSEQ(0xe1,0xb9,0xb2) /* Ṳ */
    case 0x1e75 /* ṵ */: BSEQ(0xe1,0xb9,0xb4) /* Ṵ */
    case 0x1e77 /* ṷ */: BSEQ(0xe1,0xb9,0xb6) /* Ṷ */
    case 0x1e79 /* ṹ */: BSEQ(0xe1,0xb9,0xb8) /* Ṹ */
    case 0x1e7b /* ṻ */: BSEQ(0xe1,0xb9,0xba) /* Ṻ */
    case 0x1e7d /* ṽ */: BSEQ(0xe1,0xb9,0xbc) /* Ṽ */
    case 0x1e7f /* ṿ */: BSEQ(0xe1,0xb9,0xbe) /* Ṿ */
    case 0x1e81 /* ẁ */: BSEQ(0xe1,0xba,0x80) /* Ẁ */
    case 0x1e83 /* ẃ */: BSEQ(0xe1,0xba,0x82) /* Ẃ */
    case 0x1e85 /* ẅ */: BSEQ(0xe1,0xba,0x84) /* Ẅ */
    case 0x1e87 /* ẇ */: BSEQ(0xe1,0xba,0x86) /* Ẇ */
    case 0x1e89 /* ẉ */: BSEQ(0xe1,0xba,0x88) /* Ẉ */
    case 0x1e8b /* ẋ */: BSEQ(0xe1,0xba,0x8a) /* Ẋ */
    case 0x1e8d /* ẍ */: BSEQ(0xe1,0xba,0x8c) /* Ẍ */
    case 0x1e8f /* ẏ */: BSEQ(0xe1,0xba,0x8e) /* Ẏ */
    case 0x1e91 /* ẑ */: BSEQ(0xe1,0xba,0x90) /* Ẑ */
    case 0x1e93 /* ẓ */: BSEQ(0xe1,0xba,0x92) /* Ẓ */
    case 0x1e95 /* ẕ */: BSEQ(0xe1,0xba,0x94) /* Ẕ */
    case 0x1e96 /* ẖ */: BSEQ(0x48,0xcc,0xb1) /* H̱ */
    case 0x1e97 /* ẗ */: BSEQ(0x54,0xcc,0x88) /* T̈ */
    case 0x1e98 /* ẘ */: BSEQ(0x57,0xcc,0x8a) /* W̊ */
    case 0x1e99 /* ẙ */: BSEQ(0x59,0xcc,0x8a) /* Y̊ */
    case 0x1e9a /* ẚ */: BSEQ(0x41,0xca,0xbe) /* Aʾ */
    case 0x1e9b /* ẛ */: BSEQ(0xe1,0xb9,0xa0) /* Ṡ */
    case 0x1ea1 /* ạ */: BSEQ(0xe1,0xba,0xa0) /* Ạ */
    case 0x1ea3 /* ả */: BSEQ(0xe1,0xba,0xa2) /* Ả */
    case 0x1ea5 /* ấ */: BSEQ(0xe1,0xba,0xa4) /* Ấ */
    case 0x1ea7 /* ầ */: BSEQ(0xe1,0xba,0xa6) /* Ầ */
    case 0x1ea9 /* ẩ */: BSEQ(0xe1,0xba,0xa8) /* Ẩ */
    case 0x1eab /* ẫ */: BSEQ(0xe1,0xba,0xaa) /* Ẫ */
    case 0x1ead /* ậ */: BSEQ(0xe1,0xba,0xac) /* Ậ */
    case 0x1eaf /* ắ */: BSEQ(0xe1,0xba,0xae) /* Ắ */
    case 0x1eb1 /* ằ */: BSEQ(0xe1,0xba,0xb0) /* Ằ */
    case 0x1eb3 /* ẳ */: BSEQ(0xe1,0xba,0xb2) /* Ẳ */
    case 0x1eb5 /* ẵ */: BSEQ(0xe1,0xba,0xb4) /* Ẵ */
    case 0x1eb7 /* ặ */: BSEQ(0xe1,0xba,0xb6) /* Ặ */
    case 0x1eb9 /* ẹ */: BSEQ(0xe1,0xba,0xb8) /* Ẹ */
    case 0x1ebb /* ẻ */: BSEQ(0xe1,0xba,0xba) /* Ẻ */
    case 0x1ebd /* ẽ */: BSEQ(0xe1,0xba,0xbc) /* Ẽ */
    case 0x1ebf /* ế */: BSEQ(0xe1,0xba,0xbe) /* Ế */
    case 0x1ec1 /* ề */: BSEQ(0xe1,0xbb,0x80) /* Ề */
    case 0x1ec3 /* ể */: BSEQ(0xe1,0xbb,0x82) /* Ể */
    case 0x1ec5 /* ễ */: BSEQ(0xe1,0xbb,0x84) /* Ễ */
    case 0x1ec7 /* ệ */: BSEQ(0xe1,0xbb,0x86) /* Ệ */
    case 0x1ec9 /* ỉ */: BSEQ(0xe1,0xbb,0x88) /* Ỉ */
    case 0x1ecb /* ị */: BSEQ(0xe1,0xbb,0x8a) /* Ị */
    case 0x1ecd /* ọ */: BSEQ(0xe1,0xbb,0x8c) /* Ọ */
    case 0x1ecf /* ỏ */: BSEQ(0xe1,0xbb,0x8e) /* Ỏ */
    case 0x1ed1 /* ố */: BSEQ(0xe1,0xbb,0x90) /* Ố */
    case 0x1ed3 /* ồ */: BSEQ(0xe1,0xbb,0x92) /* Ồ */
    case 0x1ed5 /* ổ */: BSEQ(0xe1,0xbb,0x94) /* Ổ */
    case 0x1ed7 /* ỗ */: BSEQ(0xe1,0xbb,0x96) /* Ỗ */
    case 0x1ed9 /* ộ */: BSEQ(0xe1,0xbb,0x98) /* Ộ */
    case 0x1edb /* ớ */: BSEQ(0xe1,0xbb,0x9a) /* Ớ */
    case 0x1edd /* ờ */: BSEQ(0xe1,0xbb,0x9c) /* Ờ */
    case 0x1edf /* ở */: BSEQ(0xe1,0xbb,0x9e) /* Ở */
    case 0x1ee1 /* ỡ */: BSEQ(0xe1,0xbb,0xa0) /* Ỡ */
    case 0x1ee3 /* ợ */: BSEQ(0xe1,0xbb,0xa2) /* Ợ */
    case 0x1ee5 /* ụ */: BSEQ(0xe1,0xbb,0xa4) /* Ụ */
    case 0x1ee7 /* ủ */: BSEQ(0xe1,0xbb,0xa6) /* Ủ */
    case 0x1ee9 /* ứ */: BSEQ(0xe1,0xbb,0xa8) /* Ứ */
    case 0x1eeb /* ừ */: BSEQ(0xe1,0xbb,0xaa) /* Ừ */
    case 0x1eed /* ử */: BSEQ(0xe1,0xbb,0xac) /* Ử */
    case 0x1eef /* ữ */: BSEQ(0xe1,0xbb,0xae) /* Ữ */
    case 0x1ef1 /* ự */: BSEQ(0xe1,0xbb,0xb0) /* Ự */
    case 0x1ef3 /* ỳ */: BSEQ(0xe1,0xbb,0xb2) /* Ỳ */
    case 0x1ef5 /* ỵ */: BSEQ(0xe1,0xbb,0xb4) /* Ỵ */
    case 0x1ef7 /* ỷ */: BSEQ(0xe1,0xbb,0xb6) /* Ỷ */
    case 0x1ef9 /* ỹ */: BSEQ(0xe1,0xbb,0xb8) /* Ỹ */
    case 0x1efb /* ỻ */: BSEQ(0xe1,0xbb,0xba) /* Ỻ */
    case 0x1efd /* ỽ */: BSEQ(0xe1,0xbb,0xbc) /* Ỽ */
    case 0x1eff /* ỿ */: BSEQ(0xe1,0xbb,0xbe) /* Ỿ */
    case 0x1f00 /* ἀ */: BSEQ(0xe1,0xbc,0x88) /* Ἀ */
    case 0x1f01 /* ἁ */: BSEQ(0xe1,0xbc,0x89) /* Ἁ */
    case 0x1f02 /* ἂ */: BSEQ(0xe1,0xbc,0x8a) /* Ἂ */
    case 0x1f03 /* ἃ */: BSEQ(0xe1,0xbc,0x8b) /* Ἃ */
    case 0x1f04 /* ἄ */: BSEQ(0xe1,0xbc,0x8c) /* Ἄ */
    case 0x1f05 /* ἅ */: BSEQ(0xe1,0xbc,0x8d) /* Ἅ */
    case 0x1f06 /* ἆ */: BSEQ(0xe1,0xbc,0x8e) /* Ἆ */
    case 0x1f07 /* ἇ */: BSEQ(0xe1,0xbc,0x8f) /* Ἇ */
    case 0x1f10 /* ἐ */: BSEQ(0xe1,0xbc,0x98) /* Ἐ */
    case 0x1f11 /* ἑ */: BSEQ(0xe1,0xbc,0x99) /* Ἑ */
    case 0x1f12 /* ἒ */: BSEQ(0xe1,0xbc,0x9a) /* Ἒ */
    case 0x1f13 /* ἓ */: BSEQ(0xe1,0xbc,0x9b) /* Ἓ */
    case 0x1f14 /* ἔ */: BSEQ(0xe1,0xbc,0x9c) /* Ἔ */
    case 0x1f15 /* ἕ */: BSEQ(0xe1,0xbc,0x9d) /* Ἕ */
    case 0x1f20 /* ἠ */: BSEQ(0xe1,0xbc,0xa8) /* Ἠ */
    case 0x1f21 /* ἡ */: BSEQ(0xe1,0xbc,0xa9) /* Ἡ */
    case 0x1f22 /* ἢ */: BSEQ(0xe1,0xbc,0xaa) /* Ἢ */
    case 0x1f23 /* ἣ */: BSEQ(0xe1,0xbc,0xab) /* Ἣ */
    case 0x1f24 /* ἤ */: BSEQ(0xe1,0xbc,0xac) /* Ἤ */
    case 0x1f25 /* ἥ */: BSEQ(0xe1,0xbc,0xad) /* Ἥ */
    case 0x1f26 /* ἦ */: BSEQ(0xe1,0xbc,0xae) /* Ἦ */
    case 0x1f27 /* ἧ */: BSEQ(0xe1,0xbc,0xaf) /* Ἧ */
    case 0x1f30 /* ἰ */: BSEQ(0xe1,0xbc,0xb8) /* Ἰ */
    case 0x1f31 /* ἱ */: BSEQ(0xe1,0xbc,0xb9) /* Ἱ */
    case 0x1f32 /* ἲ */: BSEQ(0xe1,0xbc,0xba) /* Ἲ */
    case 0x1f33 /* ἳ */: BSEQ(0xe1,0xbc,0xbb) /* Ἳ */
    case 0x1f34 /* ἴ */: BSEQ(0xe1,0xbc,0xbc) /* Ἴ */
    case 0x1f35 /* ἵ */: BSEQ(0xe1,0xbc,0xbd) /* Ἵ */
    case 0x1f36 /* ἶ */: BSEQ(0xe1,0xbc,0xbe) /* Ἶ */
    case 0x1f37 /* ἷ */: BSEQ(0xe1,0xbc,0xbf) /* Ἷ */
    case 0x1f40 /* ὀ */: BSEQ(0xe1,0xbd,0x88) /* Ὀ */
    case 0x1f41 /* ὁ */: BSEQ(0xe1,0xbd,0x89) /* Ὁ */
    case 0x1f42 /* ὂ */: BSEQ(0xe1,0xbd,0x8a) /* Ὂ */
    case 0x1f43 /* ὃ */: BSEQ(0xe1,0xbd,0x8b) /* Ὃ */
    case 0x1f44 /* ὄ */: BSEQ(0xe1,0xbd,0x8c) /* Ὄ */
    case 0x1f45 /* ὅ */: BSEQ(0xe1,0xbd,0x8d) /* Ὅ */
    case 0x1f50 /* ὐ */: BSEQ(0xce,0xa5,0xcc,0x93) /* Υ̓ */
    case 0x1f51 /* ὑ */: BSEQ(0xe1,0xbd,0x99) /* Ὑ */
    case 0x1f52 /* ὒ */: BSEQ(0xce,0xa5,0xcc,0x93,0xcc,0x80) /* Υ̓̀ */
    case 0x1f53 /* ὓ */: BSEQ(0xe1,0xbd,0x9b) /* Ὓ */
    case 0x1f54 /* ὔ */: BSEQ(0xce,0xa5,0xcc,0x93,0xcc,0x81) /* Υ̓́ */
    case 0x1f55 /* ὕ */: BSEQ(0xe1,0xbd,0x9d) /* Ὕ */
    case 0x1f56 /* ὖ */: BSEQ(0xce,0xa5,0xcc,0x93,0xcd,0x82) /* Υ̓͂ */
    case 0x1f57 /* ὗ */: BSEQ(0xe1,0xbd,0x9f) /* Ὗ */
    case 0x1f60 /* ὠ */: BSEQ(0xe1,0xbd,0xa8) /* Ὠ */
    case 0x1f61 /* ὡ */: BSEQ(0xe1,0xbd,0xa9) /* Ὡ */
    case 0x1f62 /* ὢ */: BSEQ(0xe1,0xbd,0xaa) /* Ὢ */
    case 0x1f63 /* ὣ */: BSEQ(0xe1,0xbd,0xab) /* Ὣ */
    case 0x1f64 /* ὤ */: BSEQ(0xe1,0xbd,0xac) /* Ὤ */
    case 0x1f65 /* ὥ */: BSEQ(0xe1,0xbd,0xad) /* Ὥ */
    case 0x1f66 /* ὦ */: BSEQ(0xe1,0xbd,0xae) /* Ὦ */
    case 0x1f67 /* ὧ */: BSEQ(0xe1,0xbd,0xaf) /* Ὧ */
    case 0x1f70 /* ὰ */: BSEQ(0xe1,0xbe,0xba) /* Ὰ */
    case 0x1f71 /* ά */: BSEQ(0xe1,0xbe,0xbb) /* Ά */
    case 0x1f72 /* ὲ */: BSEQ(0xe1,0xbf,0x88) /* Ὲ */
    case 0x1f73 /* έ */: BSEQ(0xe1,0xbf,0x89) /* Έ */
    case 0x1f74 /* ὴ */: BSEQ(0xe1,0xbf,0x8a) /* Ὴ */
    case 0x1f75 /* ή */: BSEQ(0xe1,0xbf,0x8b) /* Ή */
    case 0x1f76 /* ὶ */: BSEQ(0xe1,0xbf,0x9a) /* Ὶ */
    case 0x1f77 /* ί */: BSEQ(0xe1,0xbf,0x9b) /* Ί */
    case 0x1f78 /* ὸ */: BSEQ(0xe1,0xbf,0xb8) /* Ὸ */
    case 0x1f79 /* ό */: BSEQ(0xe1,0xbf,0xb9) /* Ό */
    case 0x1f7a /* ὺ */: BSEQ(0xe1,0xbf,0xaa) /* Ὺ */
    case 0x1f7b /* ύ */: BSEQ(0xe1,0xbf,0xab) /* Ύ */
    case 0x1f7c /* ὼ */: BSEQ(0xe1,0xbf,0xba) /* Ὼ */
    case 0x1f7d /* ώ */: BSEQ(0xe1,0xbf,0xbb) /* Ώ */
    case 0x1f80 /* ᾀ */: BSEQ(0xe1,0xbc,0x88,0xce,0x99) /* ἈΙ */
    case 0x1f81 /* ᾁ */: BSEQ(0xe1,0xbc,0x89,0xce,0x99) /* ἉΙ */
    case 0x1f82 /* ᾂ */: BSEQ(0xe1,0xbc,0x8a,0xce,0x99) /* ἊΙ */
    case 0x1f83 /* ᾃ */: BSEQ(0xe1,0xbc,0x8b,0xce,0x99) /* ἋΙ */
    case 0x1f84 /* ᾄ */: BSEQ(0xe1,0xbc,0x8c,0xce,0x99) /* ἌΙ */
    case 0x1f85 /* ᾅ */: BSEQ(0xe1,0xbc,0x8d,0xce,0x99) /* ἍΙ */
    case 0x1f86 /* ᾆ */: BSEQ(0xe1,0xbc,0x8e,0xce,0x99) /* ἎΙ */
    case 0x1f87 /* ᾇ */: BSEQ(0xe1,0xbc,0x8f,0xce,0x99) /* ἏΙ */
    case 0x1f88 /* ᾈ */: BSEQ(0xe1,0xbc,0x88,0xce,0x99) /* ἈΙ */
    case 0x1f89 /* ᾉ */: BSEQ(0xe1,0xbc,0x89,0xce,0x99) /* ἉΙ */
    case 0x1f8a /* ᾊ */: BSEQ(0xe1,0xbc,0x8a,0xce,0x99) /* ἊΙ */
    case 0x1f8b /* ᾋ */: BSEQ(0xe1,0xbc,0x8b,0xce,0x99) /* ἋΙ */
    case 0x1f8c /* ᾌ */: BSEQ(0xe1,0xbc,0x8c,0xce,0x99) /* ἌΙ */
    case 0x1f8d /* ᾍ */: BSEQ(0xe1,0xbc,0x8d,0xce,0x99) /* ἍΙ */
    case 0x1f8e /* ᾎ */: BSEQ(0xe1,0xbc,0x8e,0xce,0x99) /* ἎΙ */
    case 0x1f8f /* ᾏ */: BSEQ(0xe1,0xbc,0x8f,0xce,0x99) /* ἏΙ */
    case 0x1f90 /* ᾐ */: BSEQ(0xe1,0xbc,0xa8,0xce,0x99) /* ἨΙ */
    case 0x1f91 /* ᾑ */: BSEQ(0xe1,0xbc,0xa9,0xce,0x99) /* ἩΙ */
    case 0x1f92 /* ᾒ */: BSEQ(0xe1,0xbc,0xaa,0xce,0x99) /* ἪΙ */
    case 0x1f93 /* ᾓ */: BSEQ(0xe1,0xbc,0xab,0xce,0x99) /* ἫΙ */
    case 0x1f94 /* ᾔ */: BSEQ(0xe1,0xbc,0xac,0xce,0x99) /* ἬΙ */
    case 0x1f95 /* ᾕ */: BSEQ(0xe1,0xbc,0xad,0xce,0x99) /* ἭΙ */
    case 0x1f96 /* ᾖ */: BSEQ(0xe1,0xbc,0xae,0xce,0x99) /* ἮΙ */
    case 0x1f97 /* ᾗ */: BSEQ(0xe1,0xbc,0xaf,0xce,0x99) /* ἯΙ */
    case 0x1f98 /* ᾘ */: BSEQ(0xe1,0xbc,0xa8,0xce,0x99) /* ἨΙ */
    case 0x1f99 /* ᾙ */: BSEQ(0xe1,0xbc,0xa9,0xce,0x99) /* ἩΙ */
    case 0x1f9a /* ᾚ */: BSEQ(0xe1,0xbc,0xaa,0xce,0x99) /* ἪΙ */
    case 0x1f9b /* ᾛ */: BSEQ(0xe1,0xbc,0xab,0xce,0x99) /* ἫΙ */
    case 0x1f9c /* ᾜ */: BSEQ(0xe1,0xbc,0xac,0xce,0x99) /* ἬΙ */
    case 0x1f9d /* ᾝ */: BSEQ(0xe1,0xbc,0xad,0xce,0x99) /* ἭΙ */
    case 0x1f9e /* ᾞ */: BSEQ(0xe1,0xbc,0xae,0xce,0x99) /* ἮΙ */
    case 0x1f9f /* ᾟ */: BSEQ(0xe1,0xbc,0xaf,0xce,0x99) /* ἯΙ */
    case 0x1fa0 /* ᾠ */: BSEQ(0xe1,0xbd,0xa8,0xce,0x99) /* ὨΙ */
    case 0x1fa1 /* ᾡ */: BSEQ(0xe1,0xbd,0xa9,0xce,0x99) /* ὩΙ */
    case 0x1fa2 /* ᾢ */: BSEQ(0xe1,0xbd,0xaa,0xce,0x99) /* ὪΙ */
    case 0x1fa3 /* ᾣ */: BSEQ(0xe1,0xbd,0xab,0xce,0x99) /* ὫΙ */
    case 0x1fa4 /* ᾤ */: BSEQ(0xe1,0xbd,0xac,0xce,0x99) /* ὬΙ */
    case 0x1fa5 /* ᾥ */: BSEQ(0xe1,0xbd,0xad,0xce,0x99) /* ὭΙ */
    case 0x1fa6 /* ᾦ */: BSEQ(0xe1,0xbd,0xae,0xce,0x99) /* ὮΙ */
    case 0x1fa7 /* ᾧ */: BSEQ(0xe1,0xbd,0xaf,0xce,0x99) /* ὯΙ */
    case 0x1fa8 /* ᾨ */: BSEQ(0xe1,0xbd,0xa8,0xce,0x99) /* ὨΙ */
    case 0x1fa9 /* ᾩ */: BSEQ(0xe1,0xbd,0xa9,0xce,0x99) /* ὩΙ */
    case 0x1faa /* ᾪ */: BSEQ(0xe1,0xbd,0xaa,0xce,0x99) /* ὪΙ */
    case 0x1fab /* ᾫ */: BSEQ(0xe1,0xbd,0xab,0xce,0x99) /* ὫΙ */
    case 0x1fac /* ᾬ */: BSEQ(0xe1,0xbd,0xac,0xce,0x99) /* ὬΙ */
    case 0x1fad /* ᾭ */: BSEQ(0xe1,0xbd,0xad,0xce,0x99) /* ὭΙ */
    case 0x1fae /* ᾮ */: BSEQ(0xe1,0xbd,0xae,0xce,0x99) /* ὮΙ */
    case 0x1faf /* ᾯ */: BSEQ(0xe1,0xbd,0xaf,0xce,0x99) /* ὯΙ */
    case 0x1fb0 /* ᾰ */: BSEQ(0xe1,0xbe,0xb8) /* Ᾰ */
    case 0x1fb1 /* ᾱ */: BSEQ(0xe1,0xbe,0xb9) /* Ᾱ */
    case 0x1fb2 /* ᾲ */: BSEQ(0xe1,0xbe,0xba,0xce,0x99) /* ᾺΙ */
    case 0x1fb3 /* ᾳ */: BSEQ(0xce,0x91,0xce,0x99) /* ΑΙ */
    case 0x1fb4 /* ᾴ */: BSEQ(0xce,0x86,0xce,0x99) /* ΆΙ */
    case 0x1fb6 /* ᾶ */: BSEQ(0xce,0x91,0xcd,0x82) /* Α͂ */
    case 0x1fb7 /* ᾷ */: BSEQ(0xce,0x91,0xcd,0x82,0xce,0x99) /* Α͂Ι */
    case 0x1fbc /* ᾼ */: BSEQ(0xce,0x91,0xce,0x99) /* ΑΙ */
    case 0x1fbe /* ι */: BSEQ(0xce,0x99) /* Ι */
    case 0x1fc2 /* ῂ */: BSEQ(0xe1,0xbf,0x8a,0xce,0x99) /* ῊΙ */
    case 0x1fc3 /* ῃ */: BSEQ(0xce,0x97,0xce,0x99) /* ΗΙ */
    case 0x1fc4 /* ῄ */: BSEQ(0xce,0x89,0xce,0x99) /* ΉΙ */
    case 0x1fc6 /* ῆ */: BSEQ(0xce,0x97,0xcd,0x82) /* Η͂ */
    case 0x1fc7 /* ῇ */: BSEQ(0xce,0x97,0xcd,0x82,0xce,0x99) /* Η͂Ι */
    case 0x1fcc /* ῌ */: BSEQ(0xce,0x97,0xce,0x99) /* ΗΙ */
    case 0x1fd0 /* ῐ */: BSEQ(0xe1,0xbf,0x98) /* Ῐ */
    case 0x1fd1 /* ῑ */: BSEQ(0xe1,0xbf,0x99) /* Ῑ */
    case 0x1fd2 /* ῒ */: BSEQ(0xce,0x99,0xcc,0x88,0xcc,0x80) /* Ϊ̀ */
    case 0x1fd3 /* ΐ */: BSEQ(0xce,0x99,0xcc,0x88,0xcc,0x81) /* Ϊ́ */
    case 0x1fd6 /* ῖ */: BSEQ(0xce,0x99,0xcd,0x82) /* Ι͂ */
    case 0x1fd7 /* ῗ */: BSEQ(0xce,0x99,0xcc,0x88,0xcd,0x82) /* Ϊ͂ */
    case 0x1fe0 /* ῠ */: BSEQ(0xe1,0xbf,0xa8) /* Ῠ */
    case 0x1fe1 /* ῡ */: BSEQ(0xe1,0xbf,0xa9) /* Ῡ */
    case 0x1fe2 /* ῢ */: BSEQ(0xce,0xa5,0xcc,0x88,0xcc,0x80) /* Ϋ̀ */
    case 0x1fe3 /* ΰ */: BSEQ(0xce,0xa5,0xcc,0x88,0xcc,0x81) /* Ϋ́ */
    case 0x1fe4 /* ῤ */: BSEQ(0xce,0xa1,0xcc,0x93) /* Ρ̓ */
    case 0x1fe5 /* ῥ */: BSEQ(0xe1,0xbf,0xac) /* Ῥ */
    case 0x1fe6 /* ῦ */: BSEQ(0xce,0xa5,0xcd,0x82) /* Υ͂ */
    case 0x1fe7 /* ῧ */: BSEQ(0xce,0xa5,0xcc,0x88,0xcd,0x82) /* Ϋ͂ */
    case 0x1ff2 /* ῲ */: BSEQ(0xe1,0xbf,0xba,0xce,0x99) /* ῺΙ */
    case 0x1ff3 /* ῳ */: BSEQ(0xce,0xa9,0xce,0x99) /* ΩΙ */
    case 0x1ff4 /* ῴ */: BSEQ(0xce,0x8f,0xce,0x99) /* ΏΙ */
    case 0x1ff6 /* ῶ */: BSEQ(0xce,0xa9,0xcd,0x82) /* Ω͂ */
    case 0x1ff7 /* ῷ */: BSEQ(0xce,0xa9,0xcd,0x82,0xce,0x99) /* Ω͂Ι */
    case 0x1ffc /* ῼ */: BSEQ(0xce,0xa9,0xce,0x99) /* ΩΙ */
    case 0x214e /* ⅎ */: BSEQ(0xe2,0x84,0xb2) /* Ⅎ */
    case 0x2170 /* ⅰ */: BSEQ(0xe2,0x85,0xa0) /* Ⅰ */
    case 0x2171 /* ⅱ */: BSEQ(0xe2,0x85,0xa1) /* Ⅱ */
    case 0x2172 /* ⅲ */: BSEQ(0xe2,0x85,0xa2) /* Ⅲ */
    case 0x2173 /* ⅳ */: BSEQ(0xe2,0x85,0xa3) /* Ⅳ */
    case 0x2174 /* ⅴ */: BSEQ(0xe2,0x85,0xa4) /* Ⅴ */
    case 0x2175 /* ⅵ */: BSEQ(0xe2,0x85,0xa5) /* Ⅵ */
    case 0x2176 /* ⅶ */: BSEQ(0xe2,0x85,0xa6) /* Ⅶ */
    case 0x2177 /* ⅷ */: BSEQ(0xe2,0x85,0xa7) /* Ⅷ */
    case 0x2178 /* ⅸ */: BSEQ(0xe2,0x85,0xa8) /* Ⅸ */
    case 0x2179 /* ⅹ */: BSEQ(0xe2,0x85,0xa9) /* Ⅹ */
    case 0x217a /* ⅺ */: BSEQ(0xe2,0x85,0xaa) /* Ⅺ */
    case 0x217b /* ⅻ */: BSEQ(0xe2,0x85,0xab) /* Ⅻ */
    case 0x217c /* ⅼ */: BSEQ(0xe2,0x85,0xac) /* Ⅼ */
    case 0x217d /* ⅽ */: BSEQ(0xe2,0x85,0xad) /* Ⅽ */
    case 0x217e /* ⅾ */: BSEQ(0xe2,0x85,0xae) /* Ⅾ */
    case 0x217f /* ⅿ */: BSEQ(0xe2,0x85,0xaf) /* Ⅿ */
    case 0x2184 /* ↄ */: BSEQ(0xe2,0x86,0x83) /* Ↄ */
    case 0x24d0 /* ⓐ */: BSEQ(0xe2,0x92,0xb6) /* Ⓐ */
    case 0x24d1 /* ⓑ */: BSEQ(0xe2,0x92,0xb7) /* Ⓑ */
    case 0x24d2 /* ⓒ */: BSEQ(0xe2,0x92,0xb8) /* Ⓒ */
    case 0x24d3 /* ⓓ */: BSEQ(0xe2,0x92,0xb9) /* Ⓓ */
    case 0x24d4 /* ⓔ */: BSEQ(0xe2,0x92,0xba) /* Ⓔ */
    case 0x24d5 /* ⓕ */: BSEQ(0xe2,0x92,0xbb) /* Ⓕ */
    case 0x24d6 /* ⓖ */: BSEQ(0xe2,0x92,0xbc) /* Ⓖ */
    case 0x24d7 /* ⓗ */: BSEQ(0xe2,0x92,0xbd) /* Ⓗ */
    case 0x24d8 /* ⓘ */: BSEQ(0xe2,0x92,0xbe) /* Ⓘ */
    case 0x24d9 /* ⓙ */: BSEQ(0xe2,0x92,0xbf) /* Ⓙ */
    case 0x24da /* ⓚ */: BSEQ(0xe2,0x93,0x80) /* Ⓚ */
    case 0x24db /* ⓛ */: BSEQ(0xe2,0x93,0x81) /* Ⓛ */
    case 0x24dc /* ⓜ */: BSEQ(0xe2,0x93,0x82) /* Ⓜ */
    case 0x24dd /* ⓝ */: BSEQ(0xe2,0x93,0x83) /* Ⓝ */
    case 0x24de /* ⓞ */: BSEQ(0xe2,0x93,0x84) /* Ⓞ */
    case 0x24df /* ⓟ */: BSEQ(0xe2,0x93,0x85) /* Ⓟ */
    case 0x24e0 /* ⓠ */: BSEQ(0xe2,0x93,0x86) /* Ⓠ */
    case 0x24e1 /* ⓡ */: BSEQ(0xe2,0x93,0x87) /* Ⓡ */
    case 0x24e2 /* ⓢ */: BSEQ(0xe2,0x93,0x88) /* Ⓢ */
    case 0x24e3 /* ⓣ */: BSEQ(0xe2,0x93,0x89) /* Ⓣ */
    case 0x24e4 /* ⓤ */: BSEQ(0xe2,0x93,0x8a) /* Ⓤ */
    case 0x24e5 /* ⓥ */: BSEQ(0xe2,0x93,0x8b) /* Ⓥ */
    case 0x24e6 /* ⓦ */: BSEQ(0xe2,0x93,0x8c) /* Ⓦ */
    case 0x24e7 /* ⓧ */: BSEQ(0xe2,0x93,0x8d) /* Ⓧ */
    case 0x24e8 /* ⓨ */: BSEQ(0xe2,0x93,0x8e) /* Ⓨ */
    case 0x24e9 /* ⓩ */: BSEQ(0xe2,0x93,0x8f) /* Ⓩ */
    case 0x2c30 /* ⰰ */: BSEQ(0xe2,0xb0,0x80) /* Ⰰ */
    case 0x2c31 /* ⰱ */: BSEQ(0xe2,0xb0,0x81) /* Ⰱ */
    case 0x2c32 /* ⰲ */: BSEQ(0xe2,0xb0,0x82) /* Ⰲ */
    case 0x2c33 /* ⰳ */: BSEQ(0xe2,0xb0,0x83) /* Ⰳ */
    case 0x2c34 /* ⰴ */: BSEQ(0xe2,0xb0,0x84) /* Ⰴ */
    case 0x2c35 /* ⰵ */: BSEQ(0xe2,0xb0,0x85) /* Ⰵ */
    case 0x2c36 /* ⰶ */: BSEQ(0xe2,0xb0,0x86) /* Ⰶ */
    case 0x2c37 /* ⰷ */: BSEQ(0xe2,0xb0,0x87) /* Ⰷ */
    case 0x2c38 /* ⰸ */: BSEQ(0xe2,0xb0,0x88) /* Ⰸ */
    case 0x2c39 /* ⰹ */: BSEQ(0xe2,0xb0,0x89) /* Ⰹ */
    case 0x2c3a /* ⰺ */: BSEQ(0xe2,0xb0,0x8a) /* Ⰺ */
    case 0x2c3b /* ⰻ */: BSEQ(0xe2,0xb0,0x8b) /* Ⰻ */
    case 0x2c3c /* ⰼ */: BSEQ(0xe2,0xb0,0x8c) /* Ⰼ */
    case 0x2c3d /* ⰽ */: BSEQ(0xe2,0xb0,0x8d) /* Ⰽ */
    case 0x2c3e /* ⰾ */: BSEQ(0xe2,0xb0,0x8e) /* Ⰾ */
    case 0x2c3f /* ⰿ */: BSEQ(0xe2,0xb0,0x8f) /* Ⰿ */
    case 0x2c40 /* ⱀ */: BSEQ(0xe2,0xb0,0x90) /* Ⱀ */
    case 0x2c41 /* ⱁ */: BSEQ(0xe2,0xb0,0x91) /* Ⱁ */
    case 0x2c42 /* ⱂ */: BSEQ(0xe2,0xb0,0x92) /* Ⱂ */
    case 0x2c43 /* ⱃ */: BSEQ(0xe2,0xb0,0x93) /* Ⱃ */
    case 0x2c44 /* ⱄ */: BSEQ(0xe2,0xb0,0x94) /* Ⱄ */
    case 0x2c45 /* ⱅ */: BSEQ(0xe2,0xb0,0x95) /* Ⱅ */
    case 0x2c46 /* ⱆ */: BSEQ(0xe2,0xb0,0x96) /* Ⱆ */
    case 0x2c47 /* ⱇ */: BSEQ(0xe2,0xb0,0x97) /* Ⱇ */
    case 0x2c48 /* ⱈ */: BSEQ(0xe2,0xb0,0x98) /* Ⱈ */
    case 0x2c49 /* ⱉ */: BSEQ(0xe2,0xb0,0x99) /* Ⱉ */
    case 0x2c4a /* ⱊ */: BSEQ(0xe2,0xb0,0x9a) /* Ⱊ */
    case 0x2c4b /* ⱋ */: BSEQ(0xe2,0xb0,0x9b) /* Ⱋ */
    case 0x2c4c /* ⱌ */: BSEQ(0xe2,0xb0,0x9c) /* Ⱌ */
    case 0x2c4d /* ⱍ */: BSEQ(0xe2,0xb0,0x9d) /* Ⱍ */
    case 0x2c4e /* ⱎ */: BSEQ(0xe2,0xb0,0x9e) /* Ⱎ */
    case 0x2c4f /* ⱏ */: BSEQ(0xe2,0xb0,0x9f) /* Ⱏ */
    case 0x2c50 /* ⱐ */: BSEQ(0xe2,0xb0,0xa0) /* Ⱐ */
    case 0x2c51 /* ⱑ */: BSEQ(0xe2,0xb0,0xa1) /* Ⱑ */
    case 0x2c52 /* ⱒ */: BSEQ(0xe2,0xb0,0xa2) /* Ⱒ */
    case 0x2c53 /* ⱓ */: BSEQ(0xe2,0xb0,0xa3) /* Ⱓ */
    case 0x2c54 /* ⱔ */: BSEQ(0xe2,0xb0,0xa4) /* Ⱔ */
    case 0x2c55 /* ⱕ */: BSEQ(0xe2,0xb0,0xa5) /* Ⱕ */
    case 0x2c56 /* ⱖ */: BSEQ(0xe2,0xb0,0xa6) /* Ⱖ */
    case 0x2c57 /* ⱗ */: BSEQ(0xe2,0xb0,0xa7) /* Ⱗ */
    case 0x2c58 /* ⱘ */: BSEQ(0xe2,0xb0,0xa8) /* Ⱘ */
    case 0x2c59 /* ⱙ */: BSEQ(0xe2,0xb0,0xa9) /* Ⱙ */
    case 0x2c5a /* ⱚ */: BSEQ(0xe2,0xb0,0xaa) /* Ⱚ */
    case 0x2c5b /* ⱛ */: BSEQ(0xe2,0xb0,0xab) /* Ⱛ */
    case 0x2c5c /* ⱜ */: BSEQ(0xe2,0xb0,0xac) /* Ⱜ */
    case 0x2c5d /* ⱝ */: BSEQ(0xe2,0xb0,0xad) /* Ⱝ */
    case 0x2c5e /* ⱞ */: BSEQ(0xe2,0xb0,0xae) /* Ⱞ */
    case 0x2c61 /* ⱡ */: BSEQ(0xe2,0xb1,0xa0) /* Ⱡ */
    case 0x2c65 /* ⱥ */: BSEQ(0xc8,0xba) /* Ⱥ */
    case 0x2c66 /* ⱦ */: BSEQ(0xc8,0xbe) /* Ⱦ */
    case 0x2c68 /* ⱨ */: BSEQ(0xe2,0xb1,0xa7) /* Ⱨ */
    case 0x2c6a /* ⱪ */: BSEQ(0xe2,0xb1,0xa9) /* Ⱪ */
    case 0x2c6c /* ⱬ */: BSEQ(0xe2,0xb1,0xab) /* Ⱬ */
    case 0x2c73 /* ⱳ */: BSEQ(0xe2,0xb1,0xb2) /* Ⱳ */
    case 0x2c76 /* ⱶ */: BSEQ(0xe2,0xb1,0xb5) /* Ⱶ */
    case 0x2c81 /* ⲁ */: BSEQ(0xe2,0xb2,0x80) /* Ⲁ */
    case 0x2c83 /* ⲃ */: BSEQ(0xe2,0xb2,0x82) /* Ⲃ */
    case 0x2c85 /* ⲅ */: BSEQ(0xe2,0xb2,0x84) /* Ⲅ */
    case 0x2c87 /* ⲇ */: BSEQ(0xe2,0xb2,0x86) /* Ⲇ */
    case 0x2c89 /* ⲉ */: BSEQ(0xe2,0xb2,0x88) /* Ⲉ */
    case 0x2c8b /* ⲋ */: BSEQ(0xe2,0xb2,0x8a) /* Ⲋ */
    case 0x2c8d /* ⲍ */: BSEQ(0xe2,0xb2,0x8c) /* Ⲍ */
    case 0x2c8f /* ⲏ */: BSEQ(0xe2,0xb2,0x8e) /* Ⲏ */
    case 0x2c91 /* ⲑ */: BSEQ(0xe2,0xb2,0x90) /* Ⲑ */
    case 0x2c93 /* ⲓ */: BSEQ(0xe2,0xb2,0x92) /* Ⲓ */
    case 0x2c95 /* ⲕ */: BSEQ(0xe2,0xb2,0x94) /* Ⲕ */
    case 0x2c97 /* ⲗ */: BSEQ(0xe2,0xb2,0x96) /* Ⲗ */
    case 0x2c99 /* ⲙ */: BSEQ(0xe2,0xb2,0x98) /* Ⲙ */
    case 0x2c9b /* ⲛ */: BSEQ(0xe2,0xb2,0x9a) /* Ⲛ */
    case 0x2c9d /* ⲝ */: BSEQ(0xe2,0xb2,0x9c) /* Ⲝ */
    case 0x2c9f /* ⲟ */: BSEQ(0xe2,0xb2,0x9e) /* Ⲟ */
    case 0x2ca1 /* ⲡ */: BSEQ(0xe2,0xb2,0xa0) /* Ⲡ */
    case 0x2ca3 /* ⲣ */: BSEQ(0xe2,0xb2,0xa2) /* Ⲣ */
    case 0x2ca5 /* ⲥ */: BSEQ(0xe2,0xb2,0xa4) /* Ⲥ */
    case 0x2ca7 /* ⲧ */: BSEQ(0xe2,0xb2,0xa6) /* Ⲧ */
    case 0x2ca9 /* ⲩ */: BSEQ(0xe2,0xb2,0xa8) /* Ⲩ */
    case 0x2cab /* ⲫ */: BSEQ(0xe2,0xb2,0xaa) /* Ⲫ */
    case 0x2cad /* ⲭ */: BSEQ(0xe2,0xb2,0xac) /* Ⲭ */
    case 0x2caf /* ⲯ */: BSEQ(0xe2,0xb2,0xae) /* Ⲯ */
    case 0x2cb1 /* ⲱ */: BSEQ(0xe2,0xb2,0xb0) /* Ⲱ */
    case 0x2cb3 /* ⲳ */: BSEQ(0xe2,0xb2,0xb2) /* Ⲳ */
    case 0x2cb5 /* ⲵ */: BSEQ(0xe2,0xb2,0xb4) /* Ⲵ */
    case 0x2cb7 /* ⲷ */: BSEQ(0xe2,0xb2,0xb6) /* Ⲷ */
    case 0x2cb9 /* ⲹ */: BSEQ(0xe2,0xb2,0xb8) /* Ⲹ */
    case 0x2cbb /* ⲻ */: BSEQ(0xe2,0xb2,0xba) /* Ⲻ */
    case 0x2cbd /* ⲽ */: BSEQ(0xe2,0xb2,0xbc) /* Ⲽ */
    case 0x2cbf /* ⲿ */: BSEQ(0xe2,0xb2,0xbe) /* Ⲿ */
    case 0x2cc1 /* ⳁ */: BSEQ(0xe2,0xb3,0x80) /* Ⳁ */
    case 0x2cc3 /* ⳃ */: BSEQ(0xe2,0xb3,0x82) /* Ⳃ */
    case 0x2cc5 /* ⳅ */: BSEQ(0xe2,0xb3,0x84) /* Ⳅ */
    case 0x2cc7 /* ⳇ */: BSEQ(0xe2,0xb3,0x86) /* Ⳇ */
    case 0x2cc9 /* ⳉ */: BSEQ(0xe2,0xb3,0x88) /* Ⳉ */
    case 0x2ccb /* ⳋ */: BSEQ(0xe2,0xb3,0x8a) /* Ⳋ */
    case 0x2ccd /* ⳍ */: BSEQ(0xe2,0xb3,0x8c) /* Ⳍ */
    case 0x2ccf /* ⳏ */: BSEQ(0xe2,0xb3,0x8e) /* Ⳏ */
    case 0x2cd1 /* ⳑ */: BSEQ(0xe2,0xb3,0x90) /* Ⳑ */
    case 0x2cd3 /* ⳓ */: BSEQ(0xe2,0xb3,0x92) /* Ⳓ */
    case 0x2cd5 /* ⳕ */: BSEQ(0xe2,0xb3,0x94) /* Ⳕ */
    case 0x2cd7 /* ⳗ */: BSEQ(0xe2,0xb3,0x96) /* Ⳗ */
    case 0x2cd9 /* ⳙ */: BSEQ(0xe2,0xb3,0x98) /* Ⳙ */
    case 0x2cdb /* ⳛ */: BSEQ(0xe2,0xb3,0x9a) /* Ⳛ */
    case 0x2cdd /* ⳝ */: BSEQ(0xe2,0xb3,0x9c) /* Ⳝ */
    case 0x2cdf /* ⳟ */: BSEQ(0xe2,0xb3,0x9e) /* Ⳟ */
    case 0x2ce1 /* ⳡ */: BSEQ(0xe2,0xb3,0xa0) /* Ⳡ */
    case 0x2ce3 /* ⳣ */: BSEQ(0xe2,0xb3,0xa2) /* Ⳣ */
    case 0x2cec /* ⳬ */: BSEQ(0xe2,0xb3,0xab) /* Ⳬ */
    case 0x2cee /* ⳮ */: BSEQ(0xe2,0xb3,0xad) /* Ⳮ */
    case 0x2cf3 /* ⳳ */: BSEQ(0xe2,0xb3,0xb2) /* Ⳳ */
    case 0x2d00 /* ⴀ */: BSEQ(0xe1,0x82,0xa0) /* Ⴀ */
    case 0x2d01 /* ⴁ */: BSEQ(0xe1,0x82,0xa1) /* Ⴁ */
    case 0x2d02 /* ⴂ */: BSEQ(0xe1,0x82,0xa2) /* Ⴂ */
    case 0x2d03 /* ⴃ */: BSEQ(0xe1,0x82,0xa3) /* Ⴃ */
    case 0x2d04 /* ⴄ */: BSEQ(0xe1,0x82,0xa4) /* Ⴄ */
    case 0x2d05 /* ⴅ */: BSEQ(0xe1,0x82,0xa5) /* Ⴅ */
    case 0x2d06 /* ⴆ */: BSEQ(0xe1,0x82,0xa6) /* Ⴆ */
    case 0x2d07 /* ⴇ */: BSEQ(0xe1,0x82,0xa7) /* Ⴇ */
    case 0x2d08 /* ⴈ */: BSEQ(0xe1,0x82,0xa8) /* Ⴈ */
    case 0x2d09 /* ⴉ */: BSEQ(0xe1,0x82,0xa9) /* Ⴉ */
    case 0x2d0a /* ⴊ */: BSEQ(0xe1,0x82,0xaa) /* Ⴊ */
    case 0x2d0b /* ⴋ */: BSEQ(0xe1,0x82,0xab) /* Ⴋ */
    case 0x2d0c /* ⴌ */: BSEQ(0xe1,0x82,0xac) /* Ⴌ */
    case 0x2d0d /* ⴍ */: BSEQ(0xe1,0x82,0xad) /* Ⴍ */
    case 0x2d0e /* ⴎ */: BSEQ(0xe1,0x82,0xae) /* Ⴎ */
    case 0x2d0f /* ⴏ */: BSEQ(0xe1,0x82,0xaf) /* Ⴏ */
    case 0x2d10 /* ⴐ */: BSEQ(0xe1,0x82,0xb0) /* Ⴐ */
    case 0x2d11 /* ⴑ */: BSEQ(0xe1,0x82,0xb1) /* Ⴑ */
    case 0x2d12 /* ⴒ */: BSEQ(0xe1,0x82,0xb2) /* Ⴒ */
    case 0x2d13 /* ⴓ */: BSEQ(0xe1,0x82,0xb3) /* Ⴓ */
    case 0x2d14 /* ⴔ */: BSEQ(0xe1,0x82,0xb4) /* Ⴔ */
    case 0x2d15 /* ⴕ */: BSEQ(0xe1,0x82,0xb5) /* Ⴕ */
    case 0x2d16 /* ⴖ */: BSEQ(0xe1,0x82,0xb6) /* Ⴖ */
    case 0x2d17 /* ⴗ */: BSEQ(0xe1,0x82,0xb7) /* Ⴗ */
    case 0x2d18 /* ⴘ */: BSEQ(0xe1,0x82,0xb8) /* Ⴘ */
    case 0x2d19 /* ⴙ */: BSEQ(0xe1,0x82,0xb9) /* Ⴙ */
    case 0x2d1a /* ⴚ */: BSEQ(0xe1,0x82,0xba) /* Ⴚ */
    case 0x2d1b /* ⴛ */: BSEQ(0xe1,0x82,0xbb) /* Ⴛ */
    case 0x2d1c /* ⴜ */: BSEQ(0xe1,0x82,0xbc) /* Ⴜ */
    case 0x2d1d /* ⴝ */: BSEQ(0xe1,0x82,0xbd) /* Ⴝ */
    case 0x2d1e /* ⴞ */: BSEQ(0xe1,0x82,0xbe) /* Ⴞ */
    case 0x2d1f /* ⴟ */: BSEQ(0xe1,0x82,0xbf) /* Ⴟ */
    case 0x2d20 /* ⴠ */: BSEQ(0xe1,0x83,0x80) /* Ⴠ */
    case 0x2d21 /* ⴡ */: BSEQ(0xe1,0x83,0x81) /* Ⴡ */
    case 0x2d22 /* ⴢ */: BSEQ(0xe1,0x83,0x82) /* Ⴢ */
    case 0x2d23 /* ⴣ */: BSEQ(0xe1,0x83,0x83) /* Ⴣ */
    case 0x2d24 /* ⴤ */: BSEQ(0xe1,0x83,0x84) /* Ⴤ */
    case 0x2d25 /* ⴥ */: BSEQ(0xe1,0x83,0x85) /* Ⴥ */
    case 0x2d27 /* ⴧ */: BSEQ(0xe1,0x83,0x87) /* Ⴧ */
    case 0x2d2d /* ⴭ */: BSEQ(0xe1,0x83,0x8d) /* Ⴭ */
    case 0xa641 /* ꙁ */: BSEQ(0xea,0x99,0x80) /* Ꙁ */
    case 0xa643 /* ꙃ */: BSEQ(0xea,0x99,0x82) /* Ꙃ */
    case 0xa645 /* ꙅ */: BSEQ(0xea,0x99,0x84) /* Ꙅ */
    case 0xa647 /* ꙇ */: BSEQ(0xea,0x99,0x86) /* Ꙇ */
    case 0xa649 /* ꙉ */: BSEQ(0xea,0x99,0x88) /* Ꙉ */
    case 0xa64b /* ꙋ */: BSEQ(0xea,0x99,0x8a) /* Ꙋ */
    case 0xa64d /* ꙍ */: BSEQ(0xea,0x99,0x8c) /* Ꙍ */
    case 0xa64f /* ꙏ */: BSEQ(0xea,0x99,0x8e) /* Ꙏ */
    case 0xa651 /* ꙑ */: BSEQ(0xea,0x99,0x90) /* Ꙑ */
    case 0xa653 /* ꙓ */: BSEQ(0xea,0x99,0x92) /* Ꙓ */
    case 0xa655 /* ꙕ */: BSEQ(0xea,0x99,0x94) /* Ꙕ */
    case 0xa657 /* ꙗ */: BSEQ(0xea,0x99,0x96) /* Ꙗ */
    case 0xa659 /* ꙙ */: BSEQ(0xea,0x99,0x98) /* Ꙙ */
    case 0xa65b /* ꙛ */: BSEQ(0xea,0x99,0x9a) /* Ꙛ */
    case 0xa65d /* ꙝ */: BSEQ(0xea,0x99,0x9c) /* Ꙝ */
    case 0xa65f /* ꙟ */: BSEQ(0xea,0x99,0x9e) /* Ꙟ */
    case 0xa661 /* ꙡ */: BSEQ(0xea,0x99,0xa0) /* Ꙡ */
    case 0xa663 /* ꙣ */: BSEQ(0xea,0x99,0xa2) /* Ꙣ */
    case 0xa665 /* ꙥ */: BSEQ(0xea,0x99,0xa4) /* Ꙥ */
    case 0xa667 /* ꙧ */: BSEQ(0xea,0x99,0xa6) /* Ꙧ */
    case 0xa669 /* ꙩ */: BSEQ(0xea,0x99,0xa8) /* Ꙩ */
    case 0xa66b /* ꙫ */: BSEQ(0xea,0x99,0xaa) /* Ꙫ */
    case 0xa66d /* ꙭ */: BSEQ(0xea,0x99,0xac) /* Ꙭ */
    case 0xa681 /* ꚁ */: BSEQ(0xea,0x9a,0x80) /* Ꚁ */
    case 0xa683 /* ꚃ */: BSEQ(0xea,0x9a,0x82) /* Ꚃ */
    case 0xa685 /* ꚅ */: BSEQ(0xea,0x9a,0x84) /* Ꚅ */
    case 0xa687 /* ꚇ */: BSEQ(0xea,0x9a,0x86) /* Ꚇ */
    case 0xa689 /* ꚉ */: BSEQ(0xea,0x9a,0x88) /* Ꚉ */
    case 0xa68b /* ꚋ */: BSEQ(0xea,0x9a,0x8a) /* Ꚋ */
    case 0xa68d /* ꚍ */: BSEQ(0xea,0x9a,0x8c) /* Ꚍ */
    case 0xa68f /* ꚏ */: BSEQ(0xea,0x9a,0x8e) /* Ꚏ */
    case 0xa691 /* ꚑ */: BSEQ(0xea,0x9a,0x90) /* Ꚑ */
    case 0xa693 /* ꚓ */: BSEQ(0xea,0x9a,0x92) /* Ꚓ */
    case 0xa695 /* ꚕ */: BSEQ(0xea,0x9a,0x94) /* Ꚕ */
    case 0xa697 /* ꚗ */: BSEQ(0xea,0x9a,0x96) /* Ꚗ */
    case 0xa699 /* ꚙ */: BSEQ(0xea,0x9a,0x98) /* Ꚙ */
    case 0xa69b /* ꚛ */: BSEQ(0xea,0x9a,0x9a) /* Ꚛ */
    case 0xa723 /* ꜣ */: BSEQ(0xea,0x9c,0xa2) /* Ꜣ */
    case 0xa725 /* ꜥ */: BSEQ(0xea,0x9c,0xa4) /* Ꜥ */
    case 0xa727 /* ꜧ */: BSEQ(0xea,0x9c,0xa6) /* Ꜧ */
    case 0xa729 /* ꜩ */: BSEQ(0xea,0x9c,0xa8) /* Ꜩ */
    case 0xa72b /* ꜫ */: BSEQ(0xea,0x9c,0xaa) /* Ꜫ */
    case 0xa72d /* ꜭ */: BSEQ(0xea,0x9c,0xac) /* Ꜭ */
    case 0xa72f /* ꜯ */: BSEQ(0xea,0x9c,0xae) /* Ꜯ */
    case 0xa733 /* ꜳ */: BSEQ(0xea,0x9c,0xb2) /* Ꜳ */
    case 0xa735 /* ꜵ */: BSEQ(0xea,0x9c,0xb4) /* Ꜵ */
    case 0xa737 /* ꜷ */: BSEQ(0xea,0x9c,0xb6) /* Ꜷ */
    case 0xa739 /* ꜹ */: BSEQ(0xea,0x9c,0xb8) /* Ꜹ */
    case 0xa73b /* ꜻ */: BSEQ(0xea,0x9c,0xba) /* Ꜻ */
    case 0xa73d /* ꜽ */: BSEQ(0xea,0x9c,0xbc) /* Ꜽ */
    case 0xa73f /* ꜿ */: BSEQ(0xea,0x9c,0xbe) /* Ꜿ */
    case 0xa741 /* ꝁ */: BSEQ(0xea,0x9d,0x80) /* Ꝁ */
    case 0xa743 /* ꝃ */: BSEQ(0xea,0x9d,0x82) /* Ꝃ */
    case 0xa745 /* ꝅ */: BSEQ(0xea,0x9d,0x84) /* Ꝅ */
    case 0xa747 /* ꝇ */: BSEQ(0xea,0x9d,0x86) /* Ꝇ */
    case 0xa749 /* ꝉ */: BSEQ(0xea,0x9d,0x88) /* Ꝉ */
    case 0xa74b /* ꝋ */: BSEQ(0xea,0x9d,0x8a) /* Ꝋ */
    case 0xa74d /* ꝍ */: BSEQ(0xea,0x9d,0x8c) /* Ꝍ */
    case 0xa74f /* ꝏ */: BSEQ(0xea,0x9d,0x8e) /* Ꝏ */
    case 0xa751 /* ꝑ */: BSEQ(0xea,0x9d,0x90) /* Ꝑ */
    case 0xa753 /* ꝓ */: BSEQ(0xea,0x9d,0x92) /* Ꝓ */
    case 0xa755 /* ꝕ */: BSEQ(0xea,0x9d,0x94) /* Ꝕ */
    case 0xa757 /* ꝗ */: BSEQ(0xea,0x9d,0x96) /* Ꝗ */
    case 0xa759 /* ꝙ */: BSEQ(0xea,0x9d,0x98) /* Ꝙ */
    case 0xa75b /* ꝛ */: BSEQ(0xea,0x9d,0x9a) /* Ꝛ */
    case 0xa75d /* ꝝ */: BSEQ(0xea,0x9d,0x9c) /* Ꝝ */
    case 0xa75f /* ꝟ */: BSEQ(0xea,0x9d,0x9e) /* Ꝟ */
    case 0xa761 /* ꝡ */: BSEQ(0xea,0x9d,0xa0) /* Ꝡ */
    case 0xa763 /* ꝣ */: BSEQ(0xea,0x9d,0xa2) /* Ꝣ */
    case 0xa765 /* ꝥ */: BSEQ(0xea,0x9d,0xa4) /* Ꝥ */
    case 0xa767 /* ꝧ */: BSEQ(0xea,0x9d,0xa6) /* Ꝧ */
    case 0xa769 /* ꝩ */: BSEQ(0xea,0x9d,0xa8) /* Ꝩ */
    case 0xa76b /* ꝫ */: BSEQ(0xea,0x9d,0xaa) /* Ꝫ */
    case 0xa76d /* ꝭ */: BSEQ(0xea,0x9d,0xac) /* Ꝭ */
    case 0xa76f /* ꝯ */: BSEQ(0xea,0x9d,0xae) /* Ꝯ */
    case 0xa77a /* ꝺ */: BSEQ(0xea,0x9d,0xb9) /* Ꝺ */
    case 0xa77c /* ꝼ */: BSEQ(0xea,0x9d,0xbb) /* Ꝼ */
    case 0xa77f /* ꝿ */: BSEQ(0xea,0x9d,0xbe) /* Ꝿ */
    case 0xa781 /* ꞁ */: BSEQ(0xea,0x9e,0x80) /* Ꞁ */
    case 0xa783 /* ꞃ */: BSEQ(0xea,0x9e,0x82) /* Ꞃ */
    case 0xa785 /* ꞅ */: BSEQ(0xea,0x9e,0x84) /* Ꞅ */
    case 0xa787 /* ꞇ */: BSEQ(0xea,0x9e,0x86) /* Ꞇ */
    case 0xa78c /* ꞌ */: BSEQ(0xea,0x9e,0x8b) /* Ꞌ */
    case 0xa791 /* ꞑ */: BSEQ(0xea,0x9e,0x90) /* Ꞑ */
    case 0xa793 /* ꞓ */: BSEQ(0xea,0x9e,0x92) /* Ꞓ */
    case 0xa794 /* ꞔ */: BSEQ(0xea,0x9f,0x84) /* Ꞔ */
    case 0xa797 /* ꞗ */: BSEQ(0xea,0x9e,0x96) /* Ꞗ */
    case 0xa799 /* ꞙ */: BSEQ(0xea,0x9e,0x98) /* Ꞙ */
    case 0xa79b /* ꞛ */: BSEQ(0xea,0x9e,0x9a) /* Ꞛ */
    case 0xa79d /* ꞝ */: BSEQ(0xea,0x9e,0x9c) /* Ꞝ */
    case 0xa79f /* ꞟ */: BSEQ(0xea,0x9e,0x9e) /* Ꞟ */
    case 0xa7a1 /* ꞡ */: BSEQ(0xea,0x9e,0xa0) /* Ꞡ */
    case 0xa7a3 /* ꞣ */: BSEQ(0xea,0x9e,0xa2) /* Ꞣ */
    case 0xa7a5 /* ꞥ */: BSEQ(0xea,0x9e,0xa4) /* Ꞥ */
    case 0xa7a7 /* ꞧ */: BSEQ(0xea,0x9e,0xa6) /* Ꞧ */
    case 0xa7a9 /* ꞩ */: BSEQ(0xea,0x9e,0xa8) /* Ꞩ */
    case 0xa7b5 /* ꞵ */: BSEQ(0xea,0x9e,0xb4) /* Ꞵ */
    case 0xa7b7 /* ꞷ */: BSEQ(0xea,0x9e,0xb6) /* Ꞷ */
    case 0xa7b9 /* ꞹ */: BSEQ(0xea,0x9e,0xb8) /* Ꞹ */
    case 0xa7bb /* ꞻ */: BSEQ(0xea,0x9e,0xba) /* Ꞻ */
    case 0xa7bd /* ꞽ */: BSEQ(0xea,0x9e,0xbc) /* Ꞽ */
    case 0xa7bf /* ꞿ */: BSEQ(0xea,0x9e,0xbe) /* Ꞿ */
    case 0xa7c3 /* ꟃ */: BSEQ(0xea,0x9f,0x82) /* Ꟃ */
    case 0xa7c8 /* ꟈ */: BSEQ(0xea,0x9f,0x87) /* Ꟈ */
    case 0xa7ca /* ꟊ */: BSEQ(0xea,0x9f,0x89) /* Ꟊ */
    case 0xa7f6 /* ꟶ */: BSEQ(0xea,0x9f,0xb5) /* Ꟶ */
    case 0xab53 /* ꭓ */: BSEQ(0xea,0x9e,0xb3) /* Ꭓ */
    case 0xab70 /* ꭰ */: BSEQ(0xe1,0x8e,0xa0) /* Ꭰ */
    case 0xab71 /* ꭱ */: BSEQ(0xe1,0x8e,0xa1) /* Ꭱ */
    case 0xab72 /* ꭲ */: BSEQ(0xe1,0x8e,0xa2) /* Ꭲ */
    case 0xab73 /* ꭳ */: BSEQ(0xe1,0x8e,0xa3) /* Ꭳ */
    case 0xab74 /* ꭴ */: BSEQ(0xe1,0x8e,0xa4) /* Ꭴ */
    case 0xab75 /* ꭵ */: BSEQ(0xe1,0x8e,0xa5) /* Ꭵ */
    case 0xab76 /* ꭶ */: BSEQ(0xe1,0x8e,0xa6) /* Ꭶ */
    case 0xab77 /* ꭷ */: BSEQ(0xe1,0x8e,0xa7) /* Ꭷ */
    case 0xab78 /* ꭸ */: BSEQ(0xe1,0x8e,0xa8) /* Ꭸ */
    case 0xab79 /* ꭹ */: BSEQ(0xe1,0x8e,0xa9) /* Ꭹ */
    case 0xab7a /* ꭺ */: BSEQ(0xe1,0x8e,0xaa) /* Ꭺ */
    case 0xab7b /* ꭻ */: BSEQ(0xe1,0x8e,0xab) /* Ꭻ */
    case 0xab7c /* ꭼ */: BSEQ(0xe1,0x8e,0xac) /* Ꭼ */
    case 0xab7d /* ꭽ */: BSEQ(0xe1,0x8e,0xad) /* Ꭽ */
    case 0xab7e /* ꭾ */: BSEQ(0xe1,0x8e,0xae) /* Ꭾ */
    case 0xab7f /* ꭿ */: BSEQ(0xe1,0x8e,0xaf) /* Ꭿ */
    case 0xab80 /* ꮀ */: BSEQ(0xe1,0x8e,0xb0) /* Ꮀ */
    case 0xab81 /* ꮁ */: BSEQ(0xe1,0x8e,0xb1) /* Ꮁ */
    case 0xab82 /* ꮂ */: BSEQ(0xe1,0x8e,0xb2) /* Ꮂ */
    case 0xab83 /* ꮃ */: BSEQ(0xe1,0x8e,0xb3) /* Ꮃ */
    case 0xab84 /* ꮄ */: BSEQ(0xe1,0x8e,0xb4) /* Ꮄ */
    case 0xab85 /* ꮅ */: BSEQ(0xe1,0x8e,0xb5) /* Ꮅ */
    case 0xab86 /* ꮆ */: BSEQ(0xe1,0x8e,0xb6) /* Ꮆ */
    case 0xab87 /* ꮇ */: BSEQ(0xe1,0x8e,0xb7) /* Ꮇ */
    case 0xab88 /* ꮈ */: BSEQ(0xe1,0x8e,0xb8) /* Ꮈ */
    case 0xab89 /* ꮉ */: BSEQ(0xe1,0x8e,0xb9) /* Ꮉ */
    case 0xab8a /* ꮊ */: BSEQ(0xe1,0x8e,0xba) /* Ꮊ */
    case 0xab8b /* ꮋ */: BSEQ(0xe1,0x8e,0xbb) /* Ꮋ */
    case 0xab8c /* ꮌ */: BSEQ(0xe1,0x8e,0xbc) /* Ꮌ */
    case 0xab8d /* ꮍ */: BSEQ(0xe1,0x8e,0xbd) /* Ꮍ */
    case 0xab8e /* ꮎ */: BSEQ(0xe1,0x8e,0xbe) /* Ꮎ */
    case 0xab8f /* ꮏ */: BSEQ(0xe1,0x8e,0xbf) /* Ꮏ */
    case 0xab90 /* ꮐ */: BSEQ(0xe1,0x8f,0x80) /* Ꮐ */
    case 0xab91 /* ꮑ */: BSEQ(0xe1,0x8f,0x81) /* Ꮑ */
    case 0xab92 /* ꮒ */: BSEQ(0xe1,0x8f,0x82) /* Ꮒ */
    case 0xab93 /* ꮓ */: BSEQ(0xe1,0x8f,0x83) /* Ꮓ */
    case 0xab94 /* ꮔ */: BSEQ(0xe1,0x8f,0x84) /* Ꮔ */
    case 0xab95 /* ꮕ */: BSEQ(0xe1,0x8f,0x85) /* Ꮕ */
    case 0xab96 /* ꮖ */: BSEQ(0xe1,0x8f,0x86) /* Ꮖ */
    case 0xab97 /* ꮗ */: BSEQ(0xe1,0x8f,0x87) /* Ꮗ */
    case 0xab98 /* ꮘ */: BSEQ(0xe1,0x8f,0x88) /* Ꮘ */
    case 0xab99 /* ꮙ */: BSEQ(0xe1,0x8f,0x89) /* Ꮙ */
    case 0xab9a /* ꮚ */: BSEQ(0xe1,0x8f,0x8a) /* Ꮚ */
    case 0xab9b /* ꮛ */: BSEQ(0xe1,0x8f,0x8b) /* Ꮛ */
    case 0xab9c /* ꮜ */: BSEQ(0xe1,0x8f,0x8c) /* Ꮜ */
    case 0xab9d /* ꮝ */: BSEQ(0xe1,0x8f,0x8d) /* Ꮝ */
    case 0xab9e /* ꮞ */: BSEQ(0xe1,0x8f,0x8e) /* Ꮞ */
    case 0xab9f /* ꮟ */: BSEQ(0xe1,0x8f,0x8f) /* Ꮟ */
    case 0xaba0 /* ꮠ */: BSEQ(0xe1,0x8f,0x90) /* Ꮠ */
    case 0xaba1 /* ꮡ */: BSEQ(0xe1,0x8f,0x91) /* Ꮡ */
    case 0xaba2 /* ꮢ */: BSEQ(0xe1,0x8f,0x92) /* Ꮢ */
    case 0xaba3 /* ꮣ */: BSEQ(0xe1,0x8f,0x93) /* Ꮣ */
    case 0xaba4 /* ꮤ */: BSEQ(0xe1,0x8f,0x94) /* Ꮤ */
    case 0xaba5 /* ꮥ */: BSEQ(0xe1,0x8f,0x95) /* Ꮥ */
    case 0xaba6 /* ꮦ */: BSEQ(0xe1,0x8f,0x96) /* Ꮦ */
    case 0xaba7 /* ꮧ */: BSEQ(0xe1,0x8f,0x97) /* Ꮧ */
    case 0xaba8 /* ꮨ */: BSEQ(0xe1,0x8f,0x98) /* Ꮨ */
    case 0xaba9 /* ꮩ */: BSEQ(0xe1,0x8f,0x99) /* Ꮩ */
    case 0xabaa /* ꮪ */: BSEQ(0xe1,0x8f,0x9a) /* Ꮪ */
    case 0xabab /* ꮫ */: BSEQ(0xe1,0x8f,0x9b) /* Ꮫ */
    case 0xabac /* ꮬ */: BSEQ(0xe1,0x8f,0x9c) /* Ꮬ */
    case 0xabad /* ꮭ */: BSEQ(0xe1,0x8f,0x9d) /* Ꮭ */
    case 0xabae /* ꮮ */: BSEQ(0xe1,0x8f,0x9e) /* Ꮮ */
    case 0xabaf /* ꮯ */: BSEQ(0xe1,0x8f,0x9f) /* Ꮯ */
    case 0xabb0 /* ꮰ */: BSEQ(0xe1,0x8f,0xa0) /* Ꮰ */
    case 0xabb1 /* ꮱ */: BSEQ(0xe1,0x8f,0xa1) /* Ꮱ */
    case 0xabb2 /* ꮲ */: BSEQ(0xe1,0x8f,0xa2) /* Ꮲ */
    case 0xabb3 /* ꮳ */: BSEQ(0xe1,0x8f,0xa3) /* Ꮳ */
    case 0xabb4 /* ꮴ */: BSEQ(0xe1,0x8f,0xa4) /* Ꮴ */
    case 0xabb5 /* ꮵ */: BSEQ(0xe1,0x8f,0xa5) /* Ꮵ */
    case 0xabb6 /* ꮶ */: BSEQ(0xe1,0x8f,0xa6) /* Ꮶ */
    case 0xabb7 /* ꮷ */: BSEQ(0xe1,0x8f,0xa7) /* Ꮷ */
    case 0xabb8 /* ꮸ */: BSEQ(0xe1,0x8f,0xa8) /* Ꮸ */
    case 0xabb9 /* ꮹ */: BSEQ(0xe1,0x8f,0xa9) /* Ꮹ */
    case 0xabba /* ꮺ */: BSEQ(0xe1,0x8f,0xaa) /* Ꮺ */
    case 0xabbb /* ꮻ */: BSEQ(0xe1,0x8f,0xab) /* Ꮻ */
    case 0xabbc /* ꮼ */: BSEQ(0xe1,0x8f,0xac) /* Ꮼ */
    case 0xabbd /* ꮽ */: BSEQ(0xe1,0x8f,0xad) /* Ꮽ */
    case 0xabbe /* ꮾ */: BSEQ(0xe1,0x8f,0xae) /* Ꮾ */
    case 0xabbf /* ꮿ */: BSEQ(0xe1,0x8f,0xaf) /* Ꮿ */
    case 0xfb00 /* ﬀ */: BSEQ(0x46,0x46) /* FF */
    case 0xfb01 /* ﬁ */: BSEQ(0x46,0x49) /* FI */
    case 0xfb02 /* ﬂ */: BSEQ(0x46,0x4c) /* FL */
    case 0xfb03 /* ﬃ */: BSEQ(0x46,0x46,0x49) /* FFI */
    case 0xfb04 /* ﬄ */: BSEQ(0x46,0x46,0x4c) /* FFL */
    case 0xfb05 /* ﬅ */: BSEQ(0x53,0x54) /* ST */
    case 0xfb06 /* ﬆ */: BSEQ(0x53,0x54) /* ST */
    case 0xfb13 /* ﬓ */: BSEQ(0xd5,0x84,0xd5,0x86) /* ՄՆ */
    case 0xfb14 /* ﬔ */: BSEQ(0xd5,0x84,0xd4,0xb5) /* ՄԵ */
    case 0xfb15 /* ﬕ */: BSEQ(0xd5,0x84,0xd4,0xbb) /* ՄԻ */
    case 0xfb16 /* ﬖ */: BSEQ(0xd5,0x8e,0xd5,0x86) /* ՎՆ */
    case 0xfb17 /* ﬗ */: BSEQ(0xd5,0x84,0xd4,0xbd) /* ՄԽ */
    case 0xff41 /* ａ */: BSEQ(0xef,0xbc,0xa1) /* Ａ */
    case 0xff42 /* ｂ */: BSEQ(0xef,0xbc,0xa2) /* Ｂ */
    case 0xff43 /* ｃ */: BSEQ(0xef,0xbc,0xa3) /* Ｃ */
    case 0xff44 /* ｄ */: BSEQ(0xef,0xbc,0xa4) /* Ｄ */
    case 0xff45 /* ｅ */: BSEQ(0xef,0xbc,0xa5) /* Ｅ */
    case 0xff46 /* ｆ */: BSEQ(0xef,0xbc,0xa6) /* Ｆ */
    case 0xff47 /* ｇ */: BSEQ(0xef,0xbc,0xa7) /* Ｇ */
    case 0xff48 /* ｈ */: BSEQ(0xef,0xbc,0xa8) /* Ｈ */
    case 0xff49 /* ｉ */: BSEQ(0xef,0xbc,0xa9) /* Ｉ */
    case 0xff4a /* ｊ */: BSEQ(0xef,0xbc,0xaa) /* Ｊ */
    case 0xff4b /* ｋ */: BSEQ(0xef,0xbc,0xab) /* Ｋ */
    case 0xff4c /* ｌ */: BSEQ(0xef,0xbc,0xac) /* Ｌ */
    case 0xff4d /* ｍ */: BSEQ(0xef,0xbc,0xad) /* Ｍ */
    case 0xff4e /* ｎ */: BSEQ(0xef,0xbc,0xae) /* Ｎ */
    case 0xff4f /* ｏ */: BSEQ(0xef,0xbc,0xaf) /* Ｏ */
    case 0xff50 /* ｐ */: BSEQ(0xef,0xbc,0xb0) /* Ｐ */
    case 0xff51 /* ｑ */: BSEQ(0xef,0xbc,0xb1) /* Ｑ */
    case 0xff52 /* ｒ */: BSEQ(0xef,0xbc,0xb2) /* Ｒ */
    case 0xff53 /* ｓ */: BSEQ(0xef,0xbc,0xb3) /* Ｓ */
    case 0xff54 /* ｔ */: BSEQ(0xef,0xbc,0xb4) /* Ｔ */
    case 0xff55 /* ｕ */: BSEQ(0xef,0xbc,0xb5) /* Ｕ */
    case 0xff56 /* ｖ */: BSEQ(0xef,0xbc,0xb6) /* Ｖ */
    case 0xff57 /* ｗ */: BSEQ(0xef,0xbc,0xb7) /* Ｗ */
    case 0xff58 /* ｘ */: BSEQ(0xef,0xbc,0xb8) /* Ｘ */
    case 0xff59 /* ｙ */: BSEQ(0xef,0xbc,0xb9) /* Ｙ */
    case 0xff5a /* ｚ */: BSEQ(0xef,0xbc,0xba) /* Ｚ */
    case 0x10428 /* 𐐨 */: BSEQ(0xf0,0x90,0x90,0x80) /* 𐐀 */
    case 0x10429 /* 𐐩 */: BSEQ(0xf0,0x90,0x90,0x81) /* 𐐁 */
    case 0x1042a /* 𐐪 */: BSEQ(0xf0,0x90,0x90,0x82) /* 𐐂 */
    case 0x1042b /* 𐐫 */: BSEQ(0xf0,0x90,0x90,0x83) /* 𐐃 */
    case 0x1042c /* 𐐬 */: BSEQ(0xf0,0x90,0x90,0x84) /* 𐐄 */
    case 0x1042d /* 𐐭 */: BSEQ(0xf0,0x90,0x90,0x85) /* 𐐅 */
    case 0x1042e /* 𐐮 */: BSEQ(0xf0,0x90,0x90,0x86) /* 𐐆 */
    case 0x1042f /* 𐐯 */: BSEQ(0xf0,0x90,0x90,0x87) /* 𐐇 */
    case 0x10430 /* 𐐰 */: BSEQ(0xf0,0x90,0x90,0x88) /* 𐐈 */
    case 0x10431 /* 𐐱 */: BSEQ(0xf0,0x90,0x90,0x89) /* 𐐉 */
    case 0x10432 /* 𐐲 */: BSEQ(0xf0,0x90,0x90,0x8a) /* 𐐊 */
    case 0x10433 /* 𐐳 */: BSEQ(0xf0,0x90,0x90,0x8b) /* 𐐋 */
    case 0x10434 /* 𐐴 */: BSEQ(0xf0,0x90,0x90,0x8c) /* 𐐌 */
    case 0x10435 /* 𐐵 */: BSEQ(0xf0,0x90,0x90,0x8d) /* 𐐍 */
    case 0x10436 /* 𐐶 */: BSEQ(0xf0,0x90,0x90,0x8e) /* 𐐎 */
    case 0x10437 /* 𐐷 */: BSEQ(0xf0,0x90,0x90,0x8f) /* 𐐏 */
    case 0x10438 /* 𐐸 */: BSEQ(0xf0,0x90,0x90,0x90) /* 𐐐 */
    case 0x10439 /* 𐐹 */: BSEQ(0xf0,0x90,0x90,0x91) /* 𐐑 */
    case 0x1043a /* 𐐺 */: BSEQ(0xf0,0x90,0x90,0x92) /* 𐐒 */
    case 0x1043b /* 𐐻 */: BSEQ(0xf0,0x90,0x90,0x93) /* 𐐓 */
    case 0x1043c /* 𐐼 */: BSEQ(0xf0,0x90,0x90,0x94) /* 𐐔 */
    case 0x1043d /* 𐐽 */: BSEQ(0xf0,0x90,0x90,0x95) /* 𐐕 */
    case 0x1043e /* 𐐾 */: BSEQ(0xf0,0x90,0x90,0x96) /* 𐐖 */
    case 0x1043f /* 𐐿 */: BSEQ(0xf0,0x90,0x90,0x97) /* 𐐗 */
    case 0x10440 /* 𐑀 */: BSEQ(0xf0,0x90,0x90,0x98) /* 𐐘 */
    case 0x10441 /* 𐑁 */: BSEQ(0xf0,0x90,0x90,0x99) /* 𐐙 */
    case 0x10442 /* 𐑂 */: BSEQ(0xf0,0x90,0x90,0x9a) /* 𐐚 */
    case 0x10443 /* 𐑃 */: BSEQ(0xf0,0x90,0x90,0x9b) /* 𐐛 */
    case 0x10444 /* 𐑄 */: BSEQ(0xf0,0x90,0x90,0x9c) /* 𐐜 */
    case 0x10445 /* 𐑅 */: BSEQ(0xf0,0x90,0x90,0x9d) /* 𐐝 */
    case 0x10446 /* 𐑆 */: BSEQ(0xf0,0x90,0x90,0x9e) /* 𐐞 */
    case 0x10447 /* 𐑇 */: BSEQ(0xf0,0x90,0x90,0x9f) /* 𐐟 */
    case 0x10448 /* 𐑈 */: BSEQ(0xf0,0x90,0x90,0xa0) /* 𐐠 */
    case 0x10449 /* 𐑉 */: BSEQ(0xf0,0x90,0x90,0xa1) /* 𐐡 */
    case 0x1044a /* 𐑊 */: BSEQ(0xf0,0x90,0x90,0xa2) /* 𐐢 */
    case 0x1044b /* 𐑋 */: BSEQ(0xf0,0x90,0x90,0xa3) /* 𐐣 */
    case 0x1044c /* 𐑌 */: BSEQ(0xf0,0x90,0x90,0xa4) /* 𐐤 */
    case 0x1044d /* 𐑍 */: BSEQ(0xf0,0x90,0x90,0xa5) /* 𐐥 */
    case 0x1044e /* 𐑎 */: BSEQ(0xf0,0x90,0x90,0xa6) /* 𐐦 */
    case 0x1044f /* 𐑏 */: BSEQ(0xf0,0x90,0x90,0xa7) /* 𐐧 */
    case 0x104d8 /* 𐓘 */: BSEQ(0xf0,0x90,0x92,0xb0) /* 𐒰 */
    case 0x104d9 /* 𐓙 */: BSEQ(0xf0,0x90,0x92,0xb1) /* 𐒱 */
    case 0x104da /* 𐓚 */: BSEQ(0xf0,0x90,0x92,0xb2) /* 𐒲 */
    case 0x104db /* 𐓛 */: BSEQ(0xf0,0x90,0x92,0xb3) /* 𐒳 */
    case 0x104dc /* 𐓜 */: BSEQ(0xf0,0x90,0x92,0xb4) /* 𐒴 */
    case 0x104dd /* 𐓝 */: BSEQ(0xf0,0x90,0x92,0xb5) /* 𐒵 */
    case 0x104de /* 𐓞 */: BSEQ(0xf0,0x90,0x92,0xb6) /* 𐒶 */
    case 0x104df /* 𐓟 */: BSEQ(0xf0,0x90,0x92,0xb7) /* 𐒷 */
    case 0x104e0 /* 𐓠 */: BSEQ(0xf0,0x90,0x92,0xb8) /* 𐒸 */
    case 0x104e1 /* 𐓡 */: BSEQ(0xf0,0x90,0x92,0xb9) /* 𐒹 */
    case 0x104e2 /* 𐓢 */: BSEQ(0xf0,0x90,0x92,0xba) /* 𐒺 */
    case 0x104e3 /* 𐓣 */: BSEQ(0xf0,0x90,0x92,0xbb) /* 𐒻 */
    case 0x104e4 /* 𐓤 */: BSEQ(0xf0,0x90,0x92,0xbc) /* 𐒼 */
    case 0x104e5 /* 𐓥 */: BSEQ(0xf0,0x90,0x92,0xbd) /* 𐒽 */
    case 0x104e6 /* 𐓦 */: BSEQ(0xf0,0x90,0x92,0xbe) /* 𐒾 */
    case 0x104e7 /* 𐓧 */: BSEQ(0xf0,0x90,0x92,0xbf) /* 𐒿 */
    case 0x104e8 /* 𐓨 */: BSEQ(0xf0,0x90,0x93,0x80) /* 𐓀 */
    case 0x104e9 /* 𐓩 */: BSEQ(0xf0,0x90,0x93,0x81) /* 𐓁 */
    case 0x104ea /* 𐓪 */: BSEQ(0xf0,0x90,0x93,0x82) /* 𐓂 */
    case 0x104eb /* 𐓫 */: BSEQ(0xf0,0x90,0x93,0x83) /* 𐓃 */
    case 0x104ec /* 𐓬 */: BSEQ(0xf0,0x90,0x93,0x84) /* 𐓄 */
    case 0x104ed /* 𐓭 */: BSEQ(0xf0,0x90,0x93,0x85) /* 𐓅 */
    case 0x104ee /* 𐓮 */: BSEQ(0xf0,0x90,0x93,0x86) /* 𐓆 */
    case 0x104ef /* 𐓯 */: BSEQ(0xf0,0x90,0x93,0x87) /* 𐓇 */
    case 0x104f0 /* 𐓰 */: BSEQ(0xf0,0x90,0x93,0x88) /* 𐓈 */
    case 0x104f1 /* 𐓱 */: BSEQ(0xf0,0x90,0x93,0x89) /* 𐓉 */
    case 0x104f2 /* 𐓲 */: BSEQ(0xf0,0x90,0x93,0x8a) /* 𐓊 */
    case 0x104f3 /* 𐓳 */: BSEQ(0xf0,0x90,0x93,0x8b) /* 𐓋 */
    case 0x104f4 /* 𐓴 */: BSEQ(0xf0,0x90,0x93,0x8c) /* 𐓌 */
    case 0x104f5 /* 𐓵 */: BSEQ(0xf0,0x90,0x93,0x8d) /* 𐓍 */
    case 0x104f6 /* 𐓶 */: BSEQ(0xf0,0x90,0x93,0x8e) /* 𐓎 */
    case 0x104f7 /* 𐓷 */: BSEQ(0xf0,0x90,0x93,0x8f) /* 𐓏 */
    case 0x104f8 /* 𐓸 */: BSEQ(0xf0,0x90,0x93,0x90) /* 𐓐 */
    case 0x104f9 /* 𐓹 */: BSEQ(0xf0,0x90,0x93,0x91) /* 𐓑 */
    case 0x104fa /* 𐓺 */: BSEQ(0xf0,0x90,0x93,0x92) /* 𐓒 */
    case 0x104fb /* 𐓻 */: BSEQ(0xf0,0x90,0x93,0x93) /* 𐓓 */
    case 0x10cc0 /* 𐳀 */: BSEQ(0xf0,0x90,0xb2,0x80) /* 𐲀 */
    case 0x10cc1 /* 𐳁 */: BSEQ(0xf0,0x90,0xb2,0x81) /* 𐲁 */
    case 0x10cc2 /* 𐳂 */: BSEQ(0xf0,0x90,0xb2,0x82) /* 𐲂 */
    case 0x10cc3 /* 𐳃 */: BSEQ(0xf0,0x90,0xb2,0x83) /* 𐲃 */
    case 0x10cc4 /* 𐳄 */: BSEQ(0xf0,0x90,0xb2,0x84) /* 𐲄 */
    case 0x10cc5 /* 𐳅 */: BSEQ(0xf0,0x90,0xb2,0x85) /* 𐲅 */
    case 0x10cc6 /* 𐳆 */: BSEQ(0xf0,0x90,0xb2,0x86) /* 𐲆 */
    case 0x10cc7 /* 𐳇 */: BSEQ(0xf0,0x90,0xb2,0x87) /* 𐲇 */
    case 0x10cc8 /* 𐳈 */: BSEQ(0xf0,0x90,0xb2,0x88) /* 𐲈 */
    case 0x10cc9 /* 𐳉 */: BSEQ(0xf0,0x90,0xb2,0x89) /* 𐲉 */
    case 0x10cca /* 𐳊 */: BSEQ(0xf0,0x90,0xb2,0x8a) /* 𐲊 */
    case 0x10ccb /* 𐳋 */: BSEQ(0xf0,0x90,0xb2,0x8b) /* 𐲋 */
    case 0x10ccc /* 𐳌 */: BSEQ(0xf0,0x90,0xb2,0x8c) /* 𐲌 */
    case 0x10ccd /* 𐳍 */: BSEQ(0xf0,0x90,0xb2,0x8d) /* 𐲍 */
    case 0x10cce /* 𐳎 */: BSEQ(0xf0,0x90,0xb2,0x8e) /* 𐲎 */
    case 0x10ccf /* 𐳏 */: BSEQ(0xf0,0x90,0xb2,0x8f) /* 𐲏 */
    case 0x10cd0 /* 𐳐 */: BSEQ(0xf0,0x90,0xb2,0x90) /* 𐲐 */
    case 0x10cd1 /* 𐳑 */: BSEQ(0xf0,0x90,0xb2,0x91) /* 𐲑 */
    case 0x10cd2 /* 𐳒 */: BSEQ(0xf0,0x90,0xb2,0x92) /* 𐲒 */
    case 0x10cd3 /* 𐳓 */: BSEQ(0xf0,0x90,0xb2,0x93) /* 𐲓 */
    case 0x10cd4 /* 𐳔 */: BSEQ(0xf0,0x90,0xb2,0x94) /* 𐲔 */
    case 0x10cd5 /* 𐳕 */: BSEQ(0xf0,0x90,0xb2,0x95) /* 𐲕 */
    case 0x10cd6 /* 𐳖 */: BSEQ(0xf0,0x90,0xb2,0x96) /* 𐲖 */
    case 0x10cd7 /* 𐳗 */: BSEQ(0xf0,0x90,0xb2,0x97) /* 𐲗 */
    case 0x10cd8 /* 𐳘 */: BSEQ(0xf0,0x90,0xb2,0x98) /* 𐲘 */
    case 0x10cd9 /* 𐳙 */: BSEQ(0xf0,0x90,0xb2,0x99) /* 𐲙 */
    case 0x10cda /* 𐳚 */: BSEQ(0xf0,0x90,0xb2,0x9a) /* 𐲚 */
    case 0x10cdb /* 𐳛 */: BSEQ(0xf0,0x90,0xb2,0x9b) /* 𐲛 */
    case 0x10cdc /* 𐳜 */: BSEQ(0xf0,0x90,0xb2,0x9c) /* 𐲜 */
    case 0x10cdd /* 𐳝 */: BSEQ(0xf0,0x90,0xb2,0x9d) /* 𐲝 */
    case 0x10cde /* 𐳞 */: BSEQ(0xf0,0x90,0xb2,0x9e) /* 𐲞 */
    case 0x10cdf /* 𐳟 */: BSEQ(0xf0,0x90,0xb2,0x9f) /* 𐲟 */
    case 0x10ce0 /* 𐳠 */: BSEQ(0xf0,0x90,0xb2,0xa0) /* 𐲠 */
    case 0x10ce1 /* 𐳡 */: BSEQ(0xf0,0x90,0xb2,0xa1) /* 𐲡 */
    case 0x10ce2 /* 𐳢 */: BSEQ(0xf0,0x90,0xb2,0xa2) /* 𐲢 */
    case 0x10ce3 /* 𐳣 */: BSEQ(0xf0,0x90,0xb2,0xa3) /* 𐲣 */
    case 0x10ce4 /* 𐳤 */: BSEQ(0xf0,0x90,0xb2,0xa4) /* 𐲤 */
    case 0x10ce5 /* 𐳥 */: BSEQ(0xf0,0x90,0xb2,0xa5) /* 𐲥 */
    case 0x10ce6 /* 𐳦 */: BSEQ(0xf0,0x90,0xb2,0xa6) /* 𐲦 */
    case 0x10ce7 /* 𐳧 */: BSEQ(0xf0,0x90,0xb2,0xa7) /* 𐲧 */
    case 0x10ce8 /* 𐳨 */: BSEQ(0xf0,0x90,0xb2,0xa8) /* 𐲨 */
    case 0x10ce9 /* 𐳩 */: BSEQ(0xf0,0x90,0xb2,0xa9) /* 𐲩 */
    case 0x10cea /* 𐳪 */: BSEQ(0xf0,0x90,0xb2,0xaa) /* 𐲪 */
    case 0x10ceb /* 𐳫 */: BSEQ(0xf0,0x90,0xb2,0xab) /* 𐲫 */
    case 0x10cec /* 𐳬 */: BSEQ(0xf0,0x90,0xb2,0xac) /* 𐲬 */
    case 0x10ced /* 𐳭 */: BSEQ(0xf0,0x90,0xb2,0xad) /* 𐲭 */
    case 0x10cee /* 𐳮 */: BSEQ(0xf0,0x90,0xb2,0xae) /* 𐲮 */
    case 0x10cef /* 𐳯 */: BSEQ(0xf0,0x90,0xb2,0xaf) /* 𐲯 */
    case 0x10cf0 /* 𐳰 */: BSEQ(0xf0,0x90,0xb2,0xb0) /* 𐲰 */
    case 0x10cf1 /* 𐳱 */: BSEQ(0xf0,0x90,0xb2,0xb1) /* 𐲱 */
    case 0x10cf2 /* 𐳲 */: BSEQ(0xf0,0x90,0xb2,0xb2) /* 𐲲 */
    case 0x118c0 /* 𑣀 */: BSEQ(0xf0,0x91,0xa2,0xa0) /* 𑢠 */
    case 0x118c1 /* 𑣁 */: BSEQ(0xf0,0x91,0xa2,0xa1) /* 𑢡 */
    case 0x118c2 /* 𑣂 */: BSEQ(0xf0,0x91,0xa2,0xa2) /* 𑢢 */
    case 0x118c3 /* 𑣃 */: BSEQ(0xf0,0x91,0xa2,0xa3) /* 𑢣 */
    case 0x118c4 /* 𑣄 */: BSEQ(0xf0,0x91,0xa2,0xa4) /* 𑢤 */
    case 0x118c5 /* 𑣅 */: BSEQ(0xf0,0x91,0xa2,0xa5) /* 𑢥 */
    case 0x118c6 /* 𑣆 */: BSEQ(0xf0,0x91,0xa2,0xa6) /* 𑢦 */
    case 0x118c7 /* 𑣇 */: BSEQ(0xf0,0x91,0xa2,0xa7) /* 𑢧 */
    case 0x118c8 /* 𑣈 */: BSEQ(0xf0,0x91,0xa2,0xa8) /* 𑢨 */
    case 0x118c9 /* 𑣉 */: BSEQ(0xf0,0x91,0xa2,0xa9) /* 𑢩 */
    case 0x118ca /* 𑣊 */: BSEQ(0xf0,0x91,0xa2,0xaa) /* 𑢪 */
    case 0x118cb /* 𑣋 */: BSEQ(0xf0,0x91,0xa2,0xab) /* 𑢫 */
    case 0x118cc /* 𑣌 */: BSEQ(0xf0,0x91,0xa2,0xac) /* 𑢬 */
    case 0x118cd /* 𑣍 */: BSEQ(0xf0,0x91,0xa2,0xad) /* 𑢭 */
    case 0x118ce /* 𑣎 */: BSEQ(0xf0,0x91,0xa2,0xae) /* 𑢮 */
    case 0x118cf /* 𑣏 */: BSEQ(0xf0,0x91,0xa2,0xaf) /* 𑢯 */
    case 0x118d0 /* 𑣐 */: BSEQ(0xf0,0x91,0xa2,0xb0) /* 𑢰 */
    case 0x118d1 /* 𑣑 */: BSEQ(0xf0,0x91,0xa2,0xb1) /* 𑢱 */
    case 0x118d2 /* 𑣒 */: BSEQ(0xf0,0x91,0xa2,0xb2) /* 𑢲 */
    case 0x118d3 /* 𑣓 */: BSEQ(0xf0,0x91,0xa2,0xb3) /* 𑢳 */
    case 0x118d4 /* 𑣔 */: BSEQ(0xf0,0x91,0xa2,0xb4) /* 𑢴 */
    case 0x118d5 /* 𑣕 */: BSEQ(0xf0,0x91,0xa2,0xb5) /* 𑢵 */
    case 0x118d6 /* 𑣖 */: BSEQ(0xf0,0x91,0xa2,0xb6) /* 𑢶 */
    case 0x118d7 /* 𑣗 */: BSEQ(0xf0,0x91,0xa2,0xb7) /* 𑢷 */
    case 0x118d8 /* 𑣘 */: BSEQ(0xf0,0x91,0xa2,0xb8) /* 𑢸 */
    case 0x118d9 /* 𑣙 */: BSEQ(0xf0,0x91,0xa2,0xb9) /* 𑢹 */
    case 0x118da /* 𑣚 */: BSEQ(0xf0,0x91,0xa2,0xba) /* 𑢺 */
    case 0x118db /* 𑣛 */: BSEQ(0xf0,0x91,0xa2,0xbb) /* 𑢻 */
    case 0x118dc /* 𑣜 */: BSEQ(0xf0,0x91,0xa2,0xbc) /* 𑢼 */
    case 0x118dd /* 𑣝 */: BSEQ(0xf0,0x91,0xa2,0xbd) /* 𑢽 */
    case 0x118de /* 𑣞 */: BSEQ(0xf0,0x91,0xa2,0xbe) /* 𑢾 */
    case 0x118df /* 𑣟 */: BSEQ(0xf0,0x91,0xa2,0xbf) /* 𑢿 */
    case 0x16e60 /* 𖹠 */: BSEQ(0xf0,0x96,0xb9,0x80) /* 𖹀 */
    case 0x16e61 /* 𖹡 */: BSEQ(0xf0,0x96,0xb9,0x81) /* 𖹁 */
    case 0x16e62 /* 𖹢 */: BSEQ(0xf0,0x96,0xb9,0x82) /* 𖹂 */
    case 0x16e63 /* 𖹣 */: BSEQ(0xf0,0x96,0xb9,0x83) /* 𖹃 */
    case 0x16e64 /* 𖹤 */: BSEQ(0xf0,0x96,0xb9,0x84) /* 𖹄 */
    case 0x16e65 /* 𖹥 */: BSEQ(0xf0,0x96,0xb9,0x85) /* 𖹅 */
    case 0x16e66 /* 𖹦 */: BSEQ(0xf0,0x96,0xb9,0x86) /* 𖹆 */
    case 0x16e67 /* 𖹧 */: BSEQ(0xf0,0x96,0xb9,0x87) /* 𖹇 */
    case 0x16e68 /* 𖹨 */: BSEQ(0xf0,0x96,0xb9,0x88) /* 𖹈 */
    case 0x16e69 /* 𖹩 */: BSEQ(0xf0,0x96,0xb9,0x89) /* 𖹉 */
    case 0x16e6a /* 𖹪 */: BSEQ(0xf0,0x96,0xb9,0x8a) /* 𖹊 */
    case 0x16e6b /* 𖹫 */: BSEQ(0xf0,0x96,0xb9,0x8b) /* 𖹋 */
    case 0x16e6c /* 𖹬 */: BSEQ(0xf0,0x96,0xb9,0x8c) /* 𖹌 */
    case 0x16e6d /* 𖹭 */: BSEQ(0xf0,0x96,0xb9,0x8d) /* 𖹍 */
    case 0x16e6e /* 𖹮 */: BSEQ(0xf0,0x96,0xb9,0x8e) /* 𖹎 */
    case 0x16e6f /* 𖹯 */: BSEQ(0xf0,0x96,0xb9,0x8f) /* 𖹏 */
    case 0x16e70 /* 𖹰 */: BSEQ(0xf0,0x96,0xb9,0x90) /* 𖹐 */
    case 0x16e71 /* 𖹱 */: BSEQ(0xf0,0x96,0xb9,0x91) /* 𖹑 */
    case 0x16e72 /* 𖹲 */: BSEQ(0xf0,0x96,0xb9,0x92) /* 𖹒 */
    case 0x16e73 /* 𖹳 */: BSEQ(0xf0,0x96,0xb9,0x93) /* 𖹓 */
    case 0x16e74 /* 𖹴 */: BSEQ(0xf0,0x96,0xb9,0x94) /* 𖹔 */
    case 0x16e75 /* 𖹵 */: BSEQ(0xf0,0x96,0xb9,0x95) /* 𖹕 */
    case 0x16e76 /* 𖹶 */: BSEQ(0xf0,0x96,0xb9,0x96) /* 𖹖 */
    case 0x16e77 /* 𖹷 */: BSEQ(0xf0,0x96,0xb9,0x97) /* 𖹗 */
    case 0x16e78 /* 𖹸 */: BSEQ(0xf0,0x96,0xb9,0x98) /* 𖹘 */
    case 0x16e79 /* 𖹹 */: BSEQ(0xf0,0x96,0xb9,0x99) /* 𖹙 */
    case 0x16e7a /* 𖹺 */: BSEQ(0xf0,0x96,0xb9,0x9a) /* 𖹚 */
    case 0x16e7b /* 𖹻 */: BSEQ(0xf0,0x96,0xb9,0x9b) /* 𖹛 */
    case 0x16e7c /* 𖹼 */: BSEQ(0xf0,0x96,0xb9,0x9c) /* 𖹜 */
    case 0x16e7d /* 𖹽 */: BSEQ(0xf0,0x96,0xb9,0x9d) /* 𖹝 */
    case 0x16e7e /* 𖹾 */: BSEQ(0xf0,0x96,0xb9,0x9e) /* 𖹞 */
    case 0x16e7f /* 𖹿 */: BSEQ(0xf0,0x96,0xb9,0x9f) /* 𖹟 */
    case 0x1e922 /* 𞤢 */: BSEQ(0xf0,0x9e,0xa4,0x80) /* 𞤀 */
    case 0x1e923 /* 𞤣 */: BSEQ(0xf0,0x9e,0xa4,0x81) /* 𞤁 */
    case 0x1e924 /* 𞤤 */: BSEQ(0xf0,0x9e,0xa4,0x82) /* 𞤂 */
    case 0x1e925 /* 𞤥 */: BSEQ(0xf0,0x9e,0xa4,0x83) /* 𞤃 */
    case 0x1e926 /* 𞤦 */: BSEQ(0xf0,0x9e,0xa4,0x84) /* 𞤄 */
    case 0x1e927 /* 𞤧 */: BSEQ(0xf0,0x9e,0xa4,0x85) /* 𞤅 */
    case 0x1e928 /* 𞤨 */: BSEQ(0xf0,0x9e,0xa4,0x86) /* 𞤆 */
    case 0x1e929 /* 𞤩 */: BSEQ(0xf0,0x9e,0xa4,0x87) /* 𞤇 */
    case 0x1e92a /* 𞤪 */: BSEQ(0xf0,0x9e,0xa4,0x88) /* 𞤈 */
    case 0x1e92b /* 𞤫 */: BSEQ(0xf0,0x9e,0xa4,0x89) /* 𞤉 */
    case 0x1e92c /* 𞤬 */: BSEQ(0xf0,0x9e,0xa4,0x8a) /* 𞤊 */
    case 0x1e92d /* 𞤭 */: BSEQ(0xf0,0x9e,0xa4,0x8b) /* 𞤋 */
    case 0x1e92e /* 𞤮 */: BSEQ(0xf0,0x9e,0xa4,0x8c) /* 𞤌 */
    case 0x1e92f /* 𞤯 */: BSEQ(0xf0,0x9e,0xa4,0x8d) /* 𞤍 */
    case 0x1e930 /* 𞤰 */: BSEQ(0xf0,0x9e,0xa4,0x8e) /* 𞤎 */
    case 0x1e931 /* 𞤱 */: BSEQ(0xf0,0x9e,0xa4,0x8f) /* 𞤏 */
    case 0x1e932 /* 𞤲 */: BSEQ(0xf0,0x9e,0xa4,0x90) /* 𞤐 */
    case 0x1e933 /* 𞤳 */: BSEQ(0xf0,0x9e,0xa4,0x91) /* 𞤑 */
    case 0x1e934 /* 𞤴 */: BSEQ(0xf0,0x9e,0xa4,0x92) /* 𞤒 */
    case 0x1e935 /* 𞤵 */: BSEQ(0xf0,0x9e,0xa4,0x93) /* 𞤓 */
    case 0x1e936 /* 𞤶 */: BSEQ(0xf0,0x9e,0xa4,0x94) /* 𞤔 */
    case 0x1e937 /* 𞤷 */: BSEQ(0xf0,0x9e,0xa4,0x95) /* 𞤕 */
    case 0x1e938 /* 𞤸 */: BSEQ(0xf0,0x9e,0xa4,0x96) /* 𞤖 */
    case 0x1e939 /* 𞤹 */: BSEQ(0xf0,0x9e,0xa4,0x97) /* 𞤗 */
    case 0x1e93a /* 𞤺 */: BSEQ(0xf0,0x9e,0xa4,0x98) /* 𞤘 */
    case 0x1e93b /* 𞤻 */: BSEQ(0xf0,0x9e,0xa4,0x99) /* 𞤙 */
    case 0x1e93c /* 𞤼 */: BSEQ(0xf0,0x9e,0xa4,0x9a) /* 𞤚 */
    case 0x1e93d /* 𞤽 */: BSEQ(0xf0,0x9e,0xa4,0x9b) /* 𞤛 */
    case 0x1e93e /* 𞤾 */: BSEQ(0xf0,0x9e,0xa4,0x9c) /* 𞤜 */
    case 0x1e93f /* 𞤿 */: BSEQ(0xf0,0x9e,0xa4,0x9d) /* 𞤝 */
    case 0x1e940 /* 𞥀 */: BSEQ(0xf0,0x9e,0xa4,0x9e) /* 𞤞 */
    case 0x1e941 /* 𞥁 */: BSEQ(0xf0,0x9e,0xa4,0x9f) /* 𞤟 */
    case 0x1e942 /* 𞥂 */: BSEQ(0xf0,0x9e,0xa4,0xa0) /* 𞤠 */
    case 0x1e943 /* 𞥃 */: BSEQ(0xf0,0x9e,0xa4,0xa1) /* 𞤡 */
    default: return nullptr;
  }
}

inline const char *convertUnicodeToLower(uint32_t code)
{
  switch(code)
  {
    case 0x41 /* A */: BSEQ(0x61) /* a */
    case 0x42 /* B */: BSEQ(0x62) /* b */
    case 0x43 /* C */: BSEQ(0x63) /* c */
    case 0x44 /* D */: BSEQ(0x64) /* d */
    case 0x45 /* E */: BSEQ(0x65) /* e */
    case 0x46 /* F */: BSEQ(0x66) /* f */
    case 0x47 /* G */: BSEQ(0x67) /* g */
    case 0x48 /* H */: BSEQ(0x68) /* h */
    case 0x49 /* I */: BSEQ(0x69) /* i */
    case 0x4a /* J */: BSEQ(0x6a) /* j */
    case 0x4b /* K */: BSEQ(0x6b) /* k */
    case 0x4c /* L */: BSEQ(0x6c) /* l */
    case 0x4d /* M */: BSEQ(0x6d) /* m */
    case 0x4e /* N */: BSEQ(0x6e) /* n */
    case 0x4f /* O */: BSEQ(0x6f) /* o */
    case 0x50 /* P */: BSEQ(0x70) /* p */
    case 0x51 /* Q */: BSEQ(0x71) /* q */
    case 0x52 /* R */: BSEQ(0x72) /* r */
    case 0x53 /* S */: BSEQ(0x73) /* s */
    case 0x54 /* T */: BSEQ(0x74) /* t */
    case 0x55 /* U */: BSEQ(0x75) /* u */
    case 0x56 /* V */: BSEQ(0x76) /* v */
    case 0x57 /* W */: BSEQ(0x77) /* w */
    case 0x58 /* X */: BSEQ(0x78) /* x */
    case 0x59 /* Y */: BSEQ(0x79) /* y */
    case 0x5a /* Z */: BSEQ(0x7a) /* z */
    case 0xc0 /* À */: BSEQ(0xc3,0xa0) /* à */
    case 0xc1 /* Á */: BSEQ(0xc3,0xa1) /* á */
    case 0xc2 /* Â */: BSEQ(0xc3,0xa2) /* â */
    case 0xc3 /* Ã */: BSEQ(0xc3,0xa3) /* ã */
    case 0xc4 /* Ä */: BSEQ(0xc3,0xa4) /* ä */
    case 0xc5 /* Å */: BSEQ(0xc3,0xa5) /* å */
    case 0xc6 /* Æ */: BSEQ(0xc3,0xa6) /* æ */
    case 0xc7 /* Ç */: BSEQ(0xc3,0xa7) /* ç */
    case 0xc8 /* È */: BSEQ(0xc3,0xa8) /* è */
    case 0xc9 /* É */: BSEQ(0xc3,0xa9) /* é */
    case 0xca /* Ê */: BSEQ(0xc3,0xaa) /* ê */
    case 0xcb /* Ë */: BSEQ(0xc3,0xab) /* ë */
    case 0xcc /* Ì */: BSEQ(0xc3,0xac) /* ì */
    case 0xcd /* Í */: BSEQ(0xc3,0xad) /* í */
    case 0xce /* Î */: BSEQ(0xc3,0xae) /* î */
    case 0xcf /* Ï */: BSEQ(0xc3,0xaf) /* ï */
    case 0xd0 /* Ð */: BSEQ(0xc3,0xb0) /* ð */
    case 0xd1 /* Ñ */: BSEQ(0xc3,0xb1) /* ñ */
    case 0xd2 /* Ò */: BSEQ(0xc3,0xb2) /* ò */
    case 0xd3 /* Ó */: BSEQ(0xc3,0xb3) /* ó */
    case 0xd4 /* Ô */: BSEQ(0xc3,0xb4) /* ô */
    case 0xd5 /* Õ */: BSEQ(0xc3,0xb5) /* õ */
    case 0xd6 /* Ö */: BSEQ(0xc3,0xb6) /* ö */
    case 0xd8 /* Ø */: BSEQ(0xc3,0xb8) /* ø */
    case 0xd9 /* Ù */: BSEQ(0xc3,0xb9) /* ù */
    case 0xda /* Ú */: BSEQ(0xc3,0xba) /* ú */
    case 0xdb /* Û */: BSEQ(0xc3,0xbb) /* û */
    case 0xdc /* Ü */: BSEQ(0xc3,0xbc) /* ü */
    case 0xdd /* Ý */: BSEQ(0xc3,0xbd) /* ý */
    case 0xde /* Þ */: BSEQ(0xc3,0xbe) /* þ */
    case 0x100 /* Ā */: BSEQ(0xc4,0x81) /* ā */
    case 0x102 /* Ă */: BSEQ(0xc4,0x83) /* ă */
    case 0x104 /* Ą */: BSEQ(0xc4,0x85) /* ą */
    case 0x106 /* Ć */: BSEQ(0xc4,0x87) /* ć */
    case 0x108 /* Ĉ */: BSEQ(0xc4,0x89) /* ĉ */
    case 0x10a /* Ċ */: BSEQ(0xc4,0x8b) /* ċ */
    case 0x10c /* Č */: BSEQ(0xc4,0x8d) /* č */
    case 0x10e /* Ď */: BSEQ(0xc4,0x8f) /* ď */
    case 0x110 /* Đ */: BSEQ(0xc4,0x91) /* đ */
    case 0x112 /* Ē */: BSEQ(0xc4,0x93) /* ē */
    case 0x114 /* Ĕ */: BSEQ(0xc4,0x95) /* ĕ */
    case 0x116 /* Ė */: BSEQ(0xc4,0x97) /* ė */
    case 0x118 /* Ę */: BSEQ(0xc4,0x99) /* ę */
    case 0x11a /* Ě */: BSEQ(0xc4,0x9b) /* ě */
    case 0x11c /* Ĝ */: BSEQ(0xc4,0x9d) /* ĝ */
    case 0x11e /* Ğ */: BSEQ(0xc4,0x9f) /* ğ */
    case 0x120 /* Ġ */: BSEQ(0xc4,0xa1) /* ġ */
    case 0x122 /* Ģ */: BSEQ(0xc4,0xa3) /* ģ */
    case 0x124 /* Ĥ */: BSEQ(0xc4,0xa5) /* ĥ */
    case 0x126 /* Ħ */: BSEQ(0xc4,0xa7) /* ħ */
    case 0x128 /* Ĩ */: BSEQ(0xc4,0xa9) /* ĩ */
    case 0x12a /* Ī */: BSEQ(0xc4,0xab) /* ī */
    case 0x12c /* Ĭ */: BSEQ(0xc4,0xad) /* ĭ */
    case 0x12e /* Į */: BSEQ(0xc4,0xaf) /* į */
    case 0x130 /* İ */: BSEQ(0x69,0xcc,0x87) /* i̇ */
    case 0x132 /* Ĳ */: BSEQ(0xc4,0xb3) /* ĳ */
    case 0x134 /* Ĵ */: BSEQ(0xc4,0xb5) /* ĵ */
    case 0x136 /* Ķ */: BSEQ(0xc4,0xb7) /* ķ */
    case 0x139 /* Ĺ */: BSEQ(0xc4,0xba) /* ĺ */
    case 0x13b /* Ļ */: BSEQ(0xc4,0xbc) /* ļ */
    case 0x13d /* Ľ */: BSEQ(0xc4,0xbe) /* ľ */
    case 0x13f /* Ŀ */: BSEQ(0xc5,0x80) /* ŀ */
    case 0x141 /* Ł */: BSEQ(0xc5,0x82) /* ł */
    case 0x143 /* Ń */: BSEQ(0xc5,0x84) /* ń */
    case 0x145 /* Ņ */: BSEQ(0xc5,0x86) /* ņ */
    case 0x147 /* Ň */: BSEQ(0xc5,0x88) /* ň */
    case 0x14a /* Ŋ */: BSEQ(0xc5,0x8b) /* ŋ */
    case 0x14c /* Ō */: BSEQ(0xc5,0x8d) /* ō */
    case 0x14e /* Ŏ */: BSEQ(0xc5,0x8f) /* ŏ */
    case 0x150 /* Ő */: BSEQ(0xc5,0x91) /* ő */
    case 0x152 /* Œ */: BSEQ(0xc5,0x93) /* œ */
    case 0x154 /* Ŕ */: BSEQ(0xc5,0x95) /* ŕ */
    case 0x156 /* Ŗ */: BSEQ(0xc5,0x97) /* ŗ */
    case 0x158 /* Ř */: BSEQ(0xc5,0x99) /* ř */
    case 0x15a /* Ś */: BSEQ(0xc5,0x9b) /* ś */
    case 0x15c /* Ŝ */: BSEQ(0xc5,0x9d) /* ŝ */
    case 0x15e /* Ş */: BSEQ(0xc5,0x9f) /* ş */
    case 0x160 /* Š */: BSEQ(0xc5,0xa1) /* š */
    case 0x162 /* Ţ */: BSEQ(0xc5,0xa3) /* ţ */
    case 0x164 /* Ť */: BSEQ(0xc5,0xa5) /* ť */
    case 0x166 /* Ŧ */: BSEQ(0xc5,0xa7) /* ŧ */
    case 0x168 /* Ũ */: BSEQ(0xc5,0xa9) /* ũ */
    case 0x16a /* Ū */: BSEQ(0xc5,0xab) /* ū */
    case 0x16c /* Ŭ */: BSEQ(0xc5,0xad) /* ŭ */
    case 0x16e /* Ů */: BSEQ(0xc5,0xaf) /* ů */
    case 0x170 /* Ű */: BSEQ(0xc5,0xb1) /* ű */
    case 0x172 /* Ų */: BSEQ(0xc5,0xb3) /* ų */
    case 0x174 /* Ŵ */: BSEQ(0xc5,0xb5) /* ŵ */
    case 0x176 /* Ŷ */: BSEQ(0xc5,0xb7) /* ŷ */
    case 0x178 /* Ÿ */: BSEQ(0xc3,0xbf) /* ÿ */
    case 0x179 /* Ź */: BSEQ(0xc5,0xba) /* ź */
    case 0x17b /* Ż */: BSEQ(0xc5,0xbc) /* ż */
    case 0x17d /* Ž */: BSEQ(0xc5,0xbe) /* ž */
    case 0x181 /* Ɓ */: BSEQ(0xc9,0x93) /* ɓ */
    case 0x182 /* Ƃ */: BSEQ(0xc6,0x83) /* ƃ */
    case 0x184 /* Ƅ */: BSEQ(0xc6,0x85) /* ƅ */
    case 0x186 /* Ɔ */: BSEQ(0xc9,0x94) /* ɔ */
    case 0x187 /* Ƈ */: BSEQ(0xc6,0x88) /* ƈ */
    case 0x189 /* Ɖ */: BSEQ(0xc9,0x96) /* ɖ */
    case 0x18a /* Ɗ */: BSEQ(0xc9,0x97) /* ɗ */
    case 0x18b /* Ƌ */: BSEQ(0xc6,0x8c) /* ƌ */
    case 0x18e /* Ǝ */: BSEQ(0xc7,0x9d) /* ǝ */
    case 0x18f /* Ə */: BSEQ(0xc9,0x99) /* ə */
    case 0x190 /* Ɛ */: BSEQ(0xc9,0x9b) /* ɛ */
    case 0x191 /* Ƒ */: BSEQ(0xc6,0x92) /* ƒ */
    case 0x193 /* Ɠ */: BSEQ(0xc9,0xa0) /* ɠ */
    case 0x194 /* Ɣ */: BSEQ(0xc9,0xa3) /* ɣ */
    case 0x196 /* Ɩ */: BSEQ(0xc9,0xa9) /* ɩ */
    case 0x197 /* Ɨ */: BSEQ(0xc9,0xa8) /* ɨ */
    case 0x198 /* Ƙ */: BSEQ(0xc6,0x99) /* ƙ */
    case 0x19c /* Ɯ */: BSEQ(0xc9,0xaf) /* ɯ */
    case 0x19d /* Ɲ */: BSEQ(0xc9,0xb2) /* ɲ */
    case 0x19f /* Ɵ */: BSEQ(0xc9,0xb5) /* ɵ */
    case 0x1a0 /* Ơ */: BSEQ(0xc6,0xa1) /* ơ */
    case 0x1a2 /* Ƣ */: BSEQ(0xc6,0xa3) /* ƣ */
    case 0x1a4 /* Ƥ */: BSEQ(0xc6,0xa5) /* ƥ */
    case 0x1a6 /* Ʀ */: BSEQ(0xca,0x80) /* ʀ */
    case 0x1a7 /* Ƨ */: BSEQ(0xc6,0xa8) /* ƨ */
    case 0x1a9 /* Ʃ */: BSEQ(0xca,0x83) /* ʃ */
    case 0x1ac /* Ƭ */: BSEQ(0xc6,0xad) /* ƭ */
    case 0x1ae /* Ʈ */: BSEQ(0xca,0x88) /* ʈ */
    case 0x1af /* Ư */: BSEQ(0xc6,0xb0) /* ư */
    case 0x1b1 /* Ʊ */: BSEQ(0xca,0x8a) /* ʊ */
    case 0x1b2 /* Ʋ */: BSEQ(0xca,0x8b) /* ʋ */
    case 0x1b3 /* Ƴ */: BSEQ(0xc6,0xb4) /* ƴ */
    case 0x1b5 /* Ƶ */: BSEQ(0xc6,0xb6) /* ƶ */
    case 0x1b7 /* Ʒ */: BSEQ(0xca,0x92) /* ʒ */
    case 0x1b8 /* Ƹ */: BSEQ(0xc6,0xb9) /* ƹ */
    case 0x1bc /* Ƽ */: BSEQ(0xc6,0xbd) /* ƽ */
    case 0x1c4 /* Ǆ */: BSEQ(0xc7,0x86) /* ǆ */
    case 0x1c5 /* ǅ */: BSEQ(0xc7,0x86) /* ǆ */
    case 0x1c7 /* Ǉ */: BSEQ(0xc7,0x89) /* ǉ */
    case 0x1c8 /* ǈ */: BSEQ(0xc7,0x89) /* ǉ */
    case 0x1ca /* Ǌ */: BSEQ(0xc7,0x8c) /* ǌ */
    case 0x1cb /* ǋ */: BSEQ(0xc7,0x8c) /* ǌ */
    case 0x1cd /* Ǎ */: BSEQ(0xc7,0x8e) /* ǎ */
    case 0x1cf /* Ǐ */: BSEQ(0xc7,0x90) /* ǐ */
    case 0x1d1 /* Ǒ */: BSEQ(0xc7,0x92) /* ǒ */
    case 0x1d3 /* Ǔ */: BSEQ(0xc7,0x94) /* ǔ */
    case 0x1d5 /* Ǖ */: BSEQ(0xc7,0x96) /* ǖ */
    case 0x1d7 /* Ǘ */: BSEQ(0xc7,0x98) /* ǘ */
    case 0x1d9 /* Ǚ */: BSEQ(0xc7,0x9a) /* ǚ */
    case 0x1db /* Ǜ */: BSEQ(0xc7,0x9c) /* ǜ */
    case 0x1de /* Ǟ */: BSEQ(0xc7,0x9f) /* ǟ */
    case 0x1e0 /* Ǡ */: BSEQ(0xc7,0xa1) /* ǡ */
    case 0x1e2 /* Ǣ */: BSEQ(0xc7,0xa3) /* ǣ */
    case 0x1e4 /* Ǥ */: BSEQ(0xc7,0xa5) /* ǥ */
    case 0x1e6 /* Ǧ */: BSEQ(0xc7,0xa7) /* ǧ */
    case 0x1e8 /* Ǩ */: BSEQ(0xc7,0xa9) /* ǩ */
    case 0x1ea /* Ǫ */: BSEQ(0xc7,0xab) /* ǫ */
    case 0x1ec /* Ǭ */: BSEQ(0xc7,0xad) /* ǭ */
    case 0x1ee /* Ǯ */: BSEQ(0xc7,0xaf) /* ǯ */
    case 0x1f1 /* Ǳ */: BSEQ(0xc7,0xb3) /* ǳ */
    case 0x1f2 /* ǲ */: BSEQ(0xc7,0xb3) /* ǳ */
    case 0x1f4 /* Ǵ */: BSEQ(0xc7,0xb5) /* ǵ */
    case 0x1f6 /* Ƕ */: BSEQ(0xc6,0x95) /* ƕ */
    case 0x1f7 /* Ƿ */: BSEQ(0xc6,0xbf) /* ƿ */
    case 0x1f8 /* Ǹ */: BSEQ(0xc7,0xb9) /* ǹ */
    case 0x1fa /* Ǻ */: BSEQ(0xc7,0xbb) /* ǻ */
    case 0x1fc /* Ǽ */: BSEQ(0xc7,0xbd) /* ǽ */
    case 0x1fe /* Ǿ */: BSEQ(0xc7,0xbf) /* ǿ */
    case 0x200 /* Ȁ */: BSEQ(0xc8,0x81) /* ȁ */
    case 0x202 /* Ȃ */: BSEQ(0xc8,0x83) /* ȃ */
    case 0x204 /* Ȅ */: BSEQ(0xc8,0x85) /* ȅ */
    case 0x206 /* Ȇ */: BSEQ(0xc8,0x87) /* ȇ */
    case 0x208 /* Ȉ */: BSEQ(0xc8,0x89) /* ȉ */
    case 0x20a /* Ȋ */: BSEQ(0xc8,0x8b) /* ȋ */
    case 0x20c /* Ȍ */: BSEQ(0xc8,0x8d) /* ȍ */
    case 0x20e /* Ȏ */: BSEQ(0xc8,0x8f) /* ȏ */
    case 0x210 /* Ȑ */: BSEQ(0xc8,0x91) /* ȑ */
    case 0x212 /* Ȓ */: BSEQ(0xc8,0x93) /* ȓ */
    case 0x214 /* Ȕ */: BSEQ(0xc8,0x95) /* ȕ */
    case 0x216 /* Ȗ */: BSEQ(0xc8,0x97) /* ȗ */
    case 0x218 /* Ș */: BSEQ(0xc8,0x99) /* ș */
    case 0x21a /* Ț */: BSEQ(0xc8,0x9b) /* ț */
    case 0x21c /* Ȝ */: BSEQ(0xc8,0x9d) /* ȝ */
    case 0x21e /* Ȟ */: BSEQ(0xc8,0x9f) /* ȟ */
    case 0x220 /* Ƞ */: BSEQ(0xc6,0x9e) /* ƞ */
    case 0x222 /* Ȣ */: BSEQ(0xc8,0xa3) /* ȣ */
    case 0x224 /* Ȥ */: BSEQ(0xc8,0xa5) /* ȥ */
    case 0x226 /* Ȧ */: BSEQ(0xc8,0xa7) /* ȧ */
    case 0x228 /* Ȩ */: BSEQ(0xc8,0xa9) /* ȩ */
    case 0x22a /* Ȫ */: BSEQ(0xc8,0xab) /* ȫ */
    case 0x22c /* Ȭ */: BSEQ(0xc8,0xad) /* ȭ */
    case 0x22e /* Ȯ */: BSEQ(0xc8,0xaf) /* ȯ */
    case 0x230 /* Ȱ */: BSEQ(0xc8,0xb1) /* ȱ */
    case 0x232 /* Ȳ */: BSEQ(0xc8,0xb3) /* ȳ */
    case 0x23a /* Ⱥ */: BSEQ(0xe2,0xb1,0xa5) /* ⱥ */
    case 0x23b /* Ȼ */: BSEQ(0xc8,0xbc) /* ȼ */
    case 0x23d /* Ƚ */: BSEQ(0xc6,0x9a) /* ƚ */
    case 0x23e /* Ⱦ */: BSEQ(0xe2,0xb1,0xa6) /* ⱦ */
    case 0x241 /* Ɂ */: BSEQ(0xc9,0x82) /* ɂ */
    case 0x243 /* Ƀ */: BSEQ(0xc6,0x80) /* ƀ */
    case 0x244 /* Ʉ */: BSEQ(0xca,0x89) /* ʉ */
    case 0x245 /* Ʌ */: BSEQ(0xca,0x8c) /* ʌ */
    case 0x246 /* Ɇ */: BSEQ(0xc9,0x87) /* ɇ */
    case 0x248 /* Ɉ */: BSEQ(0xc9,0x89) /* ɉ */
    case 0x24a /* Ɋ */: BSEQ(0xc9,0x8b) /* ɋ */
    case 0x24c /* Ɍ */: BSEQ(0xc9,0x8d) /* ɍ */
    case 0x24e /* Ɏ */: BSEQ(0xc9,0x8f) /* ɏ */
    case 0x370 /* Ͱ */: BSEQ(0xcd,0xb1) /* ͱ */
    case 0x372 /* Ͳ */: BSEQ(0xcd,0xb3) /* ͳ */
    case 0x376 /* Ͷ */: BSEQ(0xcd,0xb7) /* ͷ */
    case 0x37f /* Ϳ */: BSEQ(0xcf,0xb3) /* ϳ */
    case 0x386 /* Ά */: BSEQ(0xce,0xac) /* ά */
    case 0x388 /* Έ */: BSEQ(0xce,0xad) /* έ */
    case 0x389 /* Ή */: BSEQ(0xce,0xae) /* ή */
    case 0x38a /* Ί */: BSEQ(0xce,0xaf) /* ί */
    case 0x38c /* Ό */: BSEQ(0xcf,0x8c) /* ό */
    case 0x38e /* Ύ */: BSEQ(0xcf,0x8d) /* ύ */
    case 0x38f /* Ώ */: BSEQ(0xcf,0x8e) /* ώ */
    case 0x391 /* Α */: BSEQ(0xce,0xb1) /* α */
    case 0x392 /* Β */: BSEQ(0xce,0xb2) /* β */
    case 0x393 /* Γ */: BSEQ(0xce,0xb3) /* γ */
    case 0x394 /* Δ */: BSEQ(0xce,0xb4) /* δ */
    case 0x395 /* Ε */: BSEQ(0xce,0xb5) /* ε */
    case 0x396 /* Ζ */: BSEQ(0xce,0xb6) /* ζ */
    case 0x397 /* Η */: BSEQ(0xce,0xb7) /* η */
    case 0x398 /* Θ */: BSEQ(0xce,0xb8) /* θ */
    case 0x399 /* Ι */: BSEQ(0xce,0xb9) /* ι */
    case 0x39a /* Κ */: BSEQ(0xce,0xba) /* κ */
    case 0x39b /* Λ */: BSEQ(0xce,0xbb) /* λ */
    case 0x39c /* Μ */: BSEQ(0xce,0xbc) /* μ */
    case 0x39d /* Ν */: BSEQ(0xce,0xbd) /* ν */
    case 0x39e /* Ξ */: BSEQ(0xce,0xbe) /* ξ */
    case 0x39f /* Ο */: BSEQ(0xce,0xbf) /* ο */
    case 0x3a0 /* Π */: BSEQ(0xcf,0x80) /* π */
    case 0x3a1 /* Ρ */: BSEQ(0xcf,0x81) /* ρ */
    case 0x3a3 /* Σ */: BSEQ(0xcf,0x83) /* σ */
    case 0x3a4 /* Τ */: BSEQ(0xcf,0x84) /* τ */
    case 0x3a5 /* Υ */: BSEQ(0xcf,0x85) /* υ */
    case 0x3a6 /* Φ */: BSEQ(0xcf,0x86) /* φ */
    case 0x3a7 /* Χ */: BSEQ(0xcf,0x87) /* χ */
    case 0x3a8 /* Ψ */: BSEQ(0xcf,0x88) /* ψ */
    case 0x3a9 /* Ω */: BSEQ(0xcf,0x89) /* ω */
    case 0x3aa /* Ϊ */: BSEQ(0xcf,0x8a) /* ϊ */
    case 0x3ab /* Ϋ */: BSEQ(0xcf,0x8b) /* ϋ */
    case 0x3cf /* Ϗ */: BSEQ(0xcf,0x97) /* ϗ */
    case 0x3d8 /* Ϙ */: BSEQ(0xcf,0x99) /* ϙ */
    case 0x3da /* Ϛ */: BSEQ(0xcf,0x9b) /* ϛ */
    case 0x3dc /* Ϝ */: BSEQ(0xcf,0x9d) /* ϝ */
    case 0x3de /* Ϟ */: BSEQ(0xcf,0x9f) /* ϟ */
    case 0x3e0 /* Ϡ */: BSEQ(0xcf,0xa1) /* ϡ */
    case 0x3e2 /* Ϣ */: BSEQ(0xcf,0xa3) /* ϣ */
    case 0x3e4 /* Ϥ */: BSEQ(0xcf,0xa5) /* ϥ */
    case 0x3e6 /* Ϧ */: BSEQ(0xcf,0xa7) /* ϧ */
    case 0x3e8 /* Ϩ */: BSEQ(0xcf,0xa9) /* ϩ */
    case 0x3ea /* Ϫ */: BSEQ(0xcf,0xab) /* ϫ */
    case 0x3ec /* Ϭ */: BSEQ(0xcf,0xad) /* ϭ */
    case 0x3ee /* Ϯ */: BSEQ(0xcf,0xaf) /* ϯ */
    case 0x3f4 /* ϴ */: BSEQ(0xce,0xb8) /* θ */
    case 0x3f7 /* Ϸ */: BSEQ(0xcf,0xb8) /* ϸ */
    case 0x3f9 /* Ϲ */: BSEQ(0xcf,0xb2) /* ϲ */
    case 0x3fa /* Ϻ */: BSEQ(0xcf,0xbb) /* ϻ */
    case 0x3fd /* Ͻ */: BSEQ(0xcd,0xbb) /* ͻ */
    case 0x3fe /* Ͼ */: BSEQ(0xcd,0xbc) /* ͼ */
    case 0x3ff /* Ͽ */: BSEQ(0xcd,0xbd) /* ͽ */
    case 0x400 /* Ѐ */: BSEQ(0xd1,0x90) /* ѐ */
    case 0x401 /* Ё */: BSEQ(0xd1,0x91) /* ё */
    case 0x402 /* Ђ */: BSEQ(0xd1,0x92) /* ђ */
    case 0x403 /* Ѓ */: BSEQ(0xd1,0x93) /* ѓ */
    case 0x404 /* Є */: BSEQ(0xd1,0x94) /* є */
    case 0x405 /* Ѕ */: BSEQ(0xd1,0x95) /* ѕ */
    case 0x406 /* І */: BSEQ(0xd1,0x96) /* і */
    case 0x407 /* Ї */: BSEQ(0xd1,0x97) /* ї */
    case 0x408 /* Ј */: BSEQ(0xd1,0x98) /* ј */
    case 0x409 /* Љ */: BSEQ(0xd1,0x99) /* љ */
    case 0x40a /* Њ */: BSEQ(0xd1,0x9a) /* њ */
    case 0x40b /* Ћ */: BSEQ(0xd1,0x9b) /* ћ */
    case 0x40c /* Ќ */: BSEQ(0xd1,0x9c) /* ќ */
    case 0x40d /* Ѝ */: BSEQ(0xd1,0x9d) /* ѝ */
    case 0x40e /* Ў */: BSEQ(0xd1,0x9e) /* ў */
    case 0x40f /* Џ */: BSEQ(0xd1,0x9f) /* џ */
    case 0x410 /* А */: BSEQ(0xd0,0xb0) /* а */
    case 0x411 /* Б */: BSEQ(0xd0,0xb1) /* б */
    case 0x412 /* В */: BSEQ(0xd0,0xb2) /* в */
    case 0x413 /* Г */: BSEQ(0xd0,0xb3) /* г */
    case 0x414 /* Д */: BSEQ(0xd0,0xb4) /* д */
    case 0x415 /* Е */: BSEQ(0xd0,0xb5) /* е */
    case 0x416 /* Ж */: BSEQ(0xd0,0xb6) /* ж */
    case 0x417 /* З */: BSEQ(0xd0,0xb7) /* з */
    case 0x418 /* И */: BSEQ(0xd0,0xb8) /* и */
    case 0x419 /* Й */: BSEQ(0xd0,0xb9) /* й */
    case 0x41a /* К */: BSEQ(0xd0,0xba) /* к */
    case 0x41b /* Л */: BSEQ(0xd0,0xbb) /* л */
    case 0x41c /* М */: BSEQ(0xd0,0xbc) /* м */
    case 0x41d /* Н */: BSEQ(0xd0,0xbd) /* н */
    case 0x41e /* О */: BSEQ(0xd0,0xbe) /* о */
    case 0x41f /* П */: BSEQ(0xd0,0xbf) /* п */
    case 0x420 /* Р */: BSEQ(0xd1,0x80) /* р */
    case 0x421 /* С */: BSEQ(0xd1,0x81) /* с */
    case 0x422 /* Т */: BSEQ(0xd1,0x82) /* т */
    case 0x423 /* У */: BSEQ(0xd1,0x83) /* у */
    case 0x424 /* Ф */: BSEQ(0xd1,0x84) /* ф */
    case 0x425 /* Х */: BSEQ(0xd1,0x85) /* х */
    case 0x426 /* Ц */: BSEQ(0xd1,0x86) /* ц */
    case 0x427 /* Ч */: BSEQ(0xd1,0x87) /* ч */
    case 0x428 /* Ш */: BSEQ(0xd1,0x88) /* ш */
    case 0x429 /* Щ */: BSEQ(0xd1,0x89) /* щ */
    case 0x42a /* Ъ */: BSEQ(0xd1,0x8a) /* ъ */
    case 0x42b /* Ы */: BSEQ(0xd1,0x8b) /* ы */
    case 0x42c /* Ь */: BSEQ(0xd1,0x8c) /* ь */
    case 0x42d /* Э */: BSEQ(0xd1,0x8d) /* э */
    case 0x42e /* Ю */: BSEQ(0xd1,0x8e) /* ю */
    case 0x42f /* Я */: BSEQ(0xd1,0x8f) /* я */
    case 0x460 /* Ѡ */: BSEQ(0xd1,0xa1) /* ѡ */
    case 0x462 /* Ѣ */: BSEQ(0xd1,0xa3) /* ѣ */
    case 0x464 /* Ѥ */: BSEQ(0xd1,0xa5) /* ѥ */
    case 0x466 /* Ѧ */: BSEQ(0xd1,0xa7) /* ѧ */
    case 0x468 /* Ѩ */: BSEQ(0xd1,0xa9) /* ѩ */
    case 0x46a /* Ѫ */: BSEQ(0xd1,0xab) /* ѫ */
    case 0x46c /* Ѭ */: BSEQ(0xd1,0xad) /* ѭ */
    case 0x46e /* Ѯ */: BSEQ(0xd1,0xaf) /* ѯ */
    case 0x470 /* Ѱ */: BSEQ(0xd1,0xb1) /* ѱ */
    case 0x472 /* Ѳ */: BSEQ(0xd1,0xb3) /* ѳ */
    case 0x474 /* Ѵ */: BSEQ(0xd1,0xb5) /* ѵ */
    case 0x476 /* Ѷ */: BSEQ(0xd1,0xb7) /* ѷ */
    case 0x478 /* Ѹ */: BSEQ(0xd1,0xb9) /* ѹ */
    case 0x47a /* Ѻ */: BSEQ(0xd1,0xbb) /* ѻ */
    case 0x47c /* Ѽ */: BSEQ(0xd1,0xbd) /* ѽ */
    case 0x47e /* Ѿ */: BSEQ(0xd1,0xbf) /* ѿ */
    case 0x480 /* Ҁ */: BSEQ(0xd2,0x81) /* ҁ */
    case 0x48a /* Ҋ */: BSEQ(0xd2,0x8b) /* ҋ */
    case 0x48c /* Ҍ */: BSEQ(0xd2,0x8d) /* ҍ */
    case 0x48e /* Ҏ */: BSEQ(0xd2,0x8f) /* ҏ */
    case 0x490 /* Ґ */: BSEQ(0xd2,0x91) /* ґ */
    case 0x492 /* Ғ */: BSEQ(0xd2,0x93) /* ғ */
    case 0x494 /* Ҕ */: BSEQ(0xd2,0x95) /* ҕ */
    case 0x496 /* Җ */: BSEQ(0xd2,0x97) /* җ */
    case 0x498 /* Ҙ */: BSEQ(0xd2,0x99) /* ҙ */
    case 0x49a /* Қ */: BSEQ(0xd2,0x9b) /* қ */
    case 0x49c /* Ҝ */: BSEQ(0xd2,0x9d) /* ҝ */
    case 0x49e /* Ҟ */: BSEQ(0xd2,0x9f) /* ҟ */
    case 0x4a0 /* Ҡ */: BSEQ(0xd2,0xa1) /* ҡ */
    case 0x4a2 /* Ң */: BSEQ(0xd2,0xa3) /* ң */
    case 0x4a4 /* Ҥ */: BSEQ(0xd2,0xa5) /* ҥ */
    case 0x4a6 /* Ҧ */: BSEQ(0xd2,0xa7) /* ҧ */
    case 0x4a8 /* Ҩ */: BSEQ(0xd2,0xa9) /* ҩ */
    case 0x4aa /* Ҫ */: BSEQ(0xd2,0xab) /* ҫ */
    case 0x4ac /* Ҭ */: BSEQ(0xd2,0xad) /* ҭ */
    case 0x4ae /* Ү */: BSEQ(0xd2,0xaf) /* ү */
    case 0x4b0 /* Ұ */: BSEQ(0xd2,0xb1) /* ұ */
    case 0x4b2 /* Ҳ */: BSEQ(0xd2,0xb3) /* ҳ */
    case 0x4b4 /* Ҵ */: BSEQ(0xd2,0xb5) /* ҵ */
    case 0x4b6 /* Ҷ */: BSEQ(0xd2,0xb7) /* ҷ */
    case 0x4b8 /* Ҹ */: BSEQ(0xd2,0xb9) /* ҹ */
    case 0x4ba /* Һ */: BSEQ(0xd2,0xbb) /* һ */
    case 0x4bc /* Ҽ */: BSEQ(0xd2,0xbd) /* ҽ */
    case 0x4be /* Ҿ */: BSEQ(0xd2,0xbf) /* ҿ */
    case 0x4c0 /* Ӏ */: BSEQ(0xd3,0x8f) /* ӏ */
    case 0x4c1 /* Ӂ */: BSEQ(0xd3,0x82) /* ӂ */
    case 0x4c3 /* Ӄ */: BSEQ(0xd3,0x84) /* ӄ */
    case 0x4c5 /* Ӆ */: BSEQ(0xd3,0x86) /* ӆ */
    case 0x4c7 /* Ӈ */: BSEQ(0xd3,0x88) /* ӈ */
    case 0x4c9 /* Ӊ */: BSEQ(0xd3,0x8a) /* ӊ */
    case 0x4cb /* Ӌ */: BSEQ(0xd3,0x8c) /* ӌ */
    case 0x4cd /* Ӎ */: BSEQ(0xd3,0x8e) /* ӎ */
    case 0x4d0 /* Ӑ */: BSEQ(0xd3,0x91) /* ӑ */
    case 0x4d2 /* Ӓ */: BSEQ(0xd3,0x93) /* ӓ */
    case 0x4d4 /* Ӕ */: BSEQ(0xd3,0x95) /* ӕ */
    case 0x4d6 /* Ӗ */: BSEQ(0xd3,0x97) /* ӗ */
    case 0x4d8 /* Ә */: BSEQ(0xd3,0x99) /* ә */
    case 0x4da /* Ӛ */: BSEQ(0xd3,0x9b) /* ӛ */
    case 0x4dc /* Ӝ */: BSEQ(0xd3,0x9d) /* ӝ */
    case 0x4de /* Ӟ */: BSEQ(0xd3,0x9f) /* ӟ */
    case 0x4e0 /* Ӡ */: BSEQ(0xd3,0xa1) /* ӡ */
    case 0x4e2 /* Ӣ */: BSEQ(0xd3,0xa3) /* ӣ */
    case 0x4e4 /* Ӥ */: BSEQ(0xd3,0xa5) /* ӥ */
    case 0x4e6 /* Ӧ */: BSEQ(0xd3,0xa7) /* ӧ */
    case 0x4e8 /* Ө */: BSEQ(0xd3,0xa9) /* ө */
    case 0x4ea /* Ӫ */: BSEQ(0xd3,0xab) /* ӫ */
    case 0x4ec /* Ӭ */: BSEQ(0xd3,0xad) /* ӭ */
    case 0x4ee /* Ӯ */: BSEQ(0xd3,0xaf) /* ӯ */
    case 0x4f0 /* Ӱ */: BSEQ(0xd3,0xb1) /* ӱ */
    case 0x4f2 /* Ӳ */: BSEQ(0xd3,0xb3) /* ӳ */
    case 0x4f4 /* Ӵ */: BSEQ(0xd3,0xb5) /* ӵ */
    case 0x4f6 /* Ӷ */: BSEQ(0xd3,0xb7) /* ӷ */
    case 0x4f8 /* Ӹ */: BSEQ(0xd3,0xb9) /* ӹ */
    case 0x4fa /* Ӻ */: BSEQ(0xd3,0xbb) /* ӻ */
    case 0x4fc /* Ӽ */: BSEQ(0xd3,0xbd) /* ӽ */
    case 0x4fe /* Ӿ */: BSEQ(0xd3,0xbf) /* ӿ */
    case 0x500 /* Ԁ */: BSEQ(0xd4,0x81) /* ԁ */
    case 0x502 /* Ԃ */: BSEQ(0xd4,0x83) /* ԃ */
    case 0x504 /* Ԅ */: BSEQ(0xd4,0x85) /* ԅ */
    case 0x506 /* Ԇ */: BSEQ(0xd4,0x87) /* ԇ */
    case 0x508 /* Ԉ */: BSEQ(0xd4,0x89) /* ԉ */
    case 0x50a /* Ԋ */: BSEQ(0xd4,0x8b) /* ԋ */
    case 0x50c /* Ԍ */: BSEQ(0xd4,0x8d) /* ԍ */
    case 0x50e /* Ԏ */: BSEQ(0xd4,0x8f) /* ԏ */
    case 0x510 /* Ԑ */: BSEQ(0xd4,0x91) /* ԑ */
    case 0x512 /* Ԓ */: BSEQ(0xd4,0x93) /* ԓ */
    case 0x514 /* Ԕ */: BSEQ(0xd4,0x95) /* ԕ */
    case 0x516 /* Ԗ */: BSEQ(0xd4,0x97) /* ԗ */
    case 0x518 /* Ԙ */: BSEQ(0xd4,0x99) /* ԙ */
    case 0x51a /* Ԛ */: BSEQ(0xd4,0x9b) /* ԛ */
    case 0x51c /* Ԝ */: BSEQ(0xd4,0x9d) /* ԝ */
    case 0x51e /* Ԟ */: BSEQ(0xd4,0x9f) /* ԟ */
    case 0x520 /* Ԡ */: BSEQ(0xd4,0xa1) /* ԡ */
    case 0x522 /* Ԣ */: BSEQ(0xd4,0xa3) /* ԣ */
    case 0x524 /* Ԥ */: BSEQ(0xd4,0xa5) /* ԥ */
    case 0x526 /* Ԧ */: BSEQ(0xd4,0xa7) /* ԧ */
    case 0x528 /* Ԩ */: BSEQ(0xd4,0xa9) /* ԩ */
    case 0x52a /* Ԫ */: BSEQ(0xd4,0xab) /* ԫ */
    case 0x52c /* Ԭ */: BSEQ(0xd4,0xad) /* ԭ */
    case 0x52e /* Ԯ */: BSEQ(0xd4,0xaf) /* ԯ */
    case 0x531 /* Ա */: BSEQ(0xd5,0xa1) /* ա */
    case 0x532 /* Բ */: BSEQ(0xd5,0xa2) /* բ */
    case 0x533 /* Գ */: BSEQ(0xd5,0xa3) /* գ */
    case 0x534 /* Դ */: BSEQ(0xd5,0xa4) /* դ */
    case 0x535 /* Ե */: BSEQ(0xd5,0xa5) /* ե */
    case 0x536 /* Զ */: BSEQ(0xd5,0xa6) /* զ */
    case 0x537 /* Է */: BSEQ(0xd5,0xa7) /* է */
    case 0x538 /* Ը */: BSEQ(0xd5,0xa8) /* ը */
    case 0x539 /* Թ */: BSEQ(0xd5,0xa9) /* թ */
    case 0x53a /* Ժ */: BSEQ(0xd5,0xaa) /* ժ */
    case 0x53b /* Ի */: BSEQ(0xd5,0xab) /* ի */
    case 0x53c /* Լ */: BSEQ(0xd5,0xac) /* լ */
    case 0x53d /* Խ */: BSEQ(0xd5,0xad) /* խ */
    case 0x53e /* Ծ */: BSEQ(0xd5,0xae) /* ծ */
    case 0x53f /* Կ */: BSEQ(0xd5,0xaf) /* կ */
    case 0x540 /* Հ */: BSEQ(0xd5,0xb0) /* հ */
    case 0x541 /* Ձ */: BSEQ(0xd5,0xb1) /* ձ */
    case 0x542 /* Ղ */: BSEQ(0xd5,0xb2) /* ղ */
    case 0x543 /* Ճ */: BSEQ(0xd5,0xb3) /* ճ */
    case 0x544 /* Մ */: BSEQ(0xd5,0xb4) /* մ */
    case 0x545 /* Յ */: BSEQ(0xd5,0xb5) /* յ */
    case 0x546 /* Ն */: BSEQ(0xd5,0xb6) /* ն */
    case 0x547 /* Շ */: BSEQ(0xd5,0xb7) /* շ */
    case 0x548 /* Ո */: BSEQ(0xd5,0xb8) /* ո */
    case 0x549 /* Չ */: BSEQ(0xd5,0xb9) /* չ */
    case 0x54a /* Պ */: BSEQ(0xd5,0xba) /* պ */
    case 0x54b /* Ջ */: BSEQ(0xd5,0xbb) /* ջ */
    case 0x54c /* Ռ */: BSEQ(0xd5,0xbc) /* ռ */
    case 0x54d /* Ս */: BSEQ(0xd5,0xbd) /* ս */
    case 0x54e /* Վ */: BSEQ(0xd5,0xbe) /* վ */
    case 0x54f /* Տ */: BSEQ(0xd5,0xbf) /* տ */
    case 0x550 /* Ր */: BSEQ(0xd6,0x80) /* ր */
    case 0x551 /* Ց */: BSEQ(0xd6,0x81) /* ց */
    case 0x552 /* Ւ */: BSEQ(0xd6,0x82) /* ւ */
    case 0x553 /* Փ */: BSEQ(0xd6,0x83) /* փ */
    case 0x554 /* Ք */: BSEQ(0xd6,0x84) /* ք */
    case 0x555 /* Օ */: BSEQ(0xd6,0x85) /* օ */
    case 0x556 /* Ֆ */: BSEQ(0xd6,0x86) /* ֆ */
    case 0x10a0 /* Ⴀ */: BSEQ(0xe2,0xb4,0x80) /* ⴀ */
    case 0x10a1 /* Ⴁ */: BSEQ(0xe2,0xb4,0x81) /* ⴁ */
    case 0x10a2 /* Ⴂ */: BSEQ(0xe2,0xb4,0x82) /* ⴂ */
    case 0x10a3 /* Ⴃ */: BSEQ(0xe2,0xb4,0x83) /* ⴃ */
    case 0x10a4 /* Ⴄ */: BSEQ(0xe2,0xb4,0x84) /* ⴄ */
    case 0x10a5 /* Ⴅ */: BSEQ(0xe2,0xb4,0x85) /* ⴅ */
    case 0x10a6 /* Ⴆ */: BSEQ(0xe2,0xb4,0x86) /* ⴆ */
    case 0x10a7 /* Ⴇ */: BSEQ(0xe2,0xb4,0x87) /* ⴇ */
    case 0x10a8 /* Ⴈ */: BSEQ(0xe2,0xb4,0x88) /* ⴈ */
    case 0x10a9 /* Ⴉ */: BSEQ(0xe2,0xb4,0x89) /* ⴉ */
    case 0x10aa /* Ⴊ */: BSEQ(0xe2,0xb4,0x8a) /* ⴊ */
    case 0x10ab /* Ⴋ */: BSEQ(0xe2,0xb4,0x8b) /* ⴋ */
    case 0x10ac /* Ⴌ */: BSEQ(0xe2,0xb4,0x8c) /* ⴌ */
    case 0x10ad /* Ⴍ */: BSEQ(0xe2,0xb4,0x8d) /* ⴍ */
    case 0x10ae /* Ⴎ */: BSEQ(0xe2,0xb4,0x8e) /* ⴎ */
    case 0x10af /* Ⴏ */: BSEQ(0xe2,0xb4,0x8f) /* ⴏ */
    case 0x10b0 /* Ⴐ */: BSEQ(0xe2,0xb4,0x90) /* ⴐ */
    case 0x10b1 /* Ⴑ */: BSEQ(0xe2,0xb4,0x91) /* ⴑ */
    case 0x10b2 /* Ⴒ */: BSEQ(0xe2,0xb4,0x92) /* ⴒ */
    case 0x10b3 /* Ⴓ */: BSEQ(0xe2,0xb4,0x93) /* ⴓ */
    case 0x10b4 /* Ⴔ */: BSEQ(0xe2,0xb4,0x94) /* ⴔ */
    case 0x10b5 /* Ⴕ */: BSEQ(0xe2,0xb4,0x95) /* ⴕ */
    case 0x10b6 /* Ⴖ */: BSEQ(0xe2,0xb4,0x96) /* ⴖ */
    case 0x10b7 /* Ⴗ */: BSEQ(0xe2,0xb4,0x97) /* ⴗ */
    case 0x10b8 /* Ⴘ */: BSEQ(0xe2,0xb4,0x98) /* ⴘ */
    case 0x10b9 /* Ⴙ */: BSEQ(0xe2,0xb4,0x99) /* ⴙ */
    case 0x10ba /* Ⴚ */: BSEQ(0xe2,0xb4,0x9a) /* ⴚ */
    case 0x10bb /* Ⴛ */: BSEQ(0xe2,0xb4,0x9b) /* ⴛ */
    case 0x10bc /* Ⴜ */: BSEQ(0xe2,0xb4,0x9c) /* ⴜ */
    case 0x10bd /* Ⴝ */: BSEQ(0xe2,0xb4,0x9d) /* ⴝ */
    case 0x10be /* Ⴞ */: BSEQ(0xe2,0xb4,0x9e) /* ⴞ */
    case 0x10bf /* Ⴟ */: BSEQ(0xe2,0xb4,0x9f) /* ⴟ */
    case 0x10c0 /* Ⴠ */: BSEQ(0xe2,0xb4,0xa0) /* ⴠ */
    case 0x10c1 /* Ⴡ */: BSEQ(0xe2,0xb4,0xa1) /* ⴡ */
    case 0x10c2 /* Ⴢ */: BSEQ(0xe2,0xb4,0xa2) /* ⴢ */
    case 0x10c3 /* Ⴣ */: BSEQ(0xe2,0xb4,0xa3) /* ⴣ */
    case 0x10c4 /* Ⴤ */: BSEQ(0xe2,0xb4,0xa4) /* ⴤ */
    case 0x10c5 /* Ⴥ */: BSEQ(0xe2,0xb4,0xa5) /* ⴥ */
    case 0x10c7 /* Ⴧ */: BSEQ(0xe2,0xb4,0xa7) /* ⴧ */
    case 0x10cd /* Ⴭ */: BSEQ(0xe2,0xb4,0xad) /* ⴭ */
    case 0x13a0 /* Ꭰ */: BSEQ(0xea,0xad,0xb0) /* ꭰ */
    case 0x13a1 /* Ꭱ */: BSEQ(0xea,0xad,0xb1) /* ꭱ */
    case 0x13a2 /* Ꭲ */: BSEQ(0xea,0xad,0xb2) /* ꭲ */
    case 0x13a3 /* Ꭳ */: BSEQ(0xea,0xad,0xb3) /* ꭳ */
    case 0x13a4 /* Ꭴ */: BSEQ(0xea,0xad,0xb4) /* ꭴ */
    case 0x13a5 /* Ꭵ */: BSEQ(0xea,0xad,0xb5) /* ꭵ */
    case 0x13a6 /* Ꭶ */: BSEQ(0xea,0xad,0xb6) /* ꭶ */
    case 0x13a7 /* Ꭷ */: BSEQ(0xea,0xad,0xb7) /* ꭷ */
    case 0x13a8 /* Ꭸ */: BSEQ(0xea,0xad,0xb8) /* ꭸ */
    case 0x13a9 /* Ꭹ */: BSEQ(0xea,0xad,0xb9) /* ꭹ */
    case 0x13aa /* Ꭺ */: BSEQ(0xea,0xad,0xba) /* ꭺ */
    case 0x13ab /* Ꭻ */: BSEQ(0xea,0xad,0xbb) /* ꭻ */
    case 0x13ac /* Ꭼ */: BSEQ(0xea,0xad,0xbc) /* ꭼ */
    case 0x13ad /* Ꭽ */: BSEQ(0xea,0xad,0xbd) /* ꭽ */
    case 0x13ae /* Ꭾ */: BSEQ(0xea,0xad,0xbe) /* ꭾ */
    case 0x13af /* Ꭿ */: BSEQ(0xea,0xad,0xbf) /* ꭿ */
    case 0x13b0 /* Ꮀ */: BSEQ(0xea,0xae,0x80) /* ꮀ */
    case 0x13b1 /* Ꮁ */: BSEQ(0xea,0xae,0x81) /* ꮁ */
    case 0x13b2 /* Ꮂ */: BSEQ(0xea,0xae,0x82) /* ꮂ */
    case 0x13b3 /* Ꮃ */: BSEQ(0xea,0xae,0x83) /* ꮃ */
    case 0x13b4 /* Ꮄ */: BSEQ(0xea,0xae,0x84) /* ꮄ */
    case 0x13b5 /* Ꮅ */: BSEQ(0xea,0xae,0x85) /* ꮅ */
    case 0x13b6 /* Ꮆ */: BSEQ(0xea,0xae,0x86) /* ꮆ */
    case 0x13b7 /* Ꮇ */: BSEQ(0xea,0xae,0x87) /* ꮇ */
    case 0x13b8 /* Ꮈ */: BSEQ(0xea,0xae,0x88) /* ꮈ */
    case 0x13b9 /* Ꮉ */: BSEQ(0xea,0xae,0x89) /* ꮉ */
    case 0x13ba /* Ꮊ */: BSEQ(0xea,0xae,0x8a) /* ꮊ */
    case 0x13bb /* Ꮋ */: BSEQ(0xea,0xae,0x8b) /* ꮋ */
    case 0x13bc /* Ꮌ */: BSEQ(0xea,0xae,0x8c) /* ꮌ */
    case 0x13bd /* Ꮍ */: BSEQ(0xea,0xae,0x8d) /* ꮍ */
    case 0x13be /* Ꮎ */: BSEQ(0xea,0xae,0x8e) /* ꮎ */
    case 0x13bf /* Ꮏ */: BSEQ(0xea,0xae,0x8f) /* ꮏ */
    case 0x13c0 /* Ꮐ */: BSEQ(0xea,0xae,0x90) /* ꮐ */
    case 0x13c1 /* Ꮑ */: BSEQ(0xea,0xae,0x91) /* ꮑ */
    case 0x13c2 /* Ꮒ */: BSEQ(0xea,0xae,0x92) /* ꮒ */
    case 0x13c3 /* Ꮓ */: BSEQ(0xea,0xae,0x93) /* ꮓ */
    case 0x13c4 /* Ꮔ */: BSEQ(0xea,0xae,0x94) /* ꮔ */
    case 0x13c5 /* Ꮕ */: BSEQ(0xea,0xae,0x95) /* ꮕ */
    case 0x13c6 /* Ꮖ */: BSEQ(0xea,0xae,0x96) /* ꮖ */
    case 0x13c7 /* Ꮗ */: BSEQ(0xea,0xae,0x97) /* ꮗ */
    case 0x13c8 /* Ꮘ */: BSEQ(0xea,0xae,0x98) /* ꮘ */
    case 0x13c9 /* Ꮙ */: BSEQ(0xea,0xae,0x99) /* ꮙ */
    case 0x13ca /* Ꮚ */: BSEQ(0xea,0xae,0x9a) /* ꮚ */
    case 0x13cb /* Ꮛ */: BSEQ(0xea,0xae,0x9b) /* ꮛ */
    case 0x13cc /* Ꮜ */: BSEQ(0xea,0xae,0x9c) /* ꮜ */
    case 0x13cd /* Ꮝ */: BSEQ(0xea,0xae,0x9d) /* ꮝ */
    case 0x13ce /* Ꮞ */: BSEQ(0xea,0xae,0x9e) /* ꮞ */
    case 0x13cf /* Ꮟ */: BSEQ(0xea,0xae,0x9f) /* ꮟ */
    case 0x13d0 /* Ꮠ */: BSEQ(0xea,0xae,0xa0) /* ꮠ */
    case 0x13d1 /* Ꮡ */: BSEQ(0xea,0xae,0xa1) /* ꮡ */
    case 0x13d2 /* Ꮢ */: BSEQ(0xea,0xae,0xa2) /* ꮢ */
    case 0x13d3 /* Ꮣ */: BSEQ(0xea,0xae,0xa3) /* ꮣ */
    case 0x13d4 /* Ꮤ */: BSEQ(0xea,0xae,0xa4) /* ꮤ */
    case 0x13d5 /* Ꮥ */: BSEQ(0xea,0xae,0xa5) /* ꮥ */
    case 0x13d6 /* Ꮦ */: BSEQ(0xea,0xae,0xa6) /* ꮦ */
    case 0x13d7 /* Ꮧ */: BSEQ(0xea,0xae,0xa7) /* ꮧ */
    case 0x13d8 /* Ꮨ */: BSEQ(0xea,0xae,0xa8) /* ꮨ */
    case 0x13d9 /* Ꮩ */: BSEQ(0xea,0xae,0xa9) /* ꮩ */
    case 0x13da /* Ꮪ */: BSEQ(0xea,0xae,0xaa) /* ꮪ */
    case 0x13db /* Ꮫ */: BSEQ(0xea,0xae,0xab) /* ꮫ */
    case 0x13dc /* Ꮬ */: BSEQ(0xea,0xae,0xac) /* ꮬ */
    case 0x13dd /* Ꮭ */: BSEQ(0xea,0xae,0xad) /* ꮭ */
    case 0x13de /* Ꮮ */: BSEQ(0xea,0xae,0xae) /* ꮮ */
    case 0x13df /* Ꮯ */: BSEQ(0xea,0xae,0xaf) /* ꮯ */
    case 0x13e0 /* Ꮰ */: BSEQ(0xea,0xae,0xb0) /* ꮰ */
    case 0x13e1 /* Ꮱ */: BSEQ(0xea,0xae,0xb1) /* ꮱ */
    case 0x13e2 /* Ꮲ */: BSEQ(0xea,0xae,0xb2) /* ꮲ */
    case 0x13e3 /* Ꮳ */: BSEQ(0xea,0xae,0xb3) /* ꮳ */
    case 0x13e4 /* Ꮴ */: BSEQ(0xea,0xae,0xb4) /* ꮴ */
    case 0x13e5 /* Ꮵ */: BSEQ(0xea,0xae,0xb5) /* ꮵ */
    case 0x13e6 /* Ꮶ */: BSEQ(0xea,0xae,0xb6) /* ꮶ */
    case 0x13e7 /* Ꮷ */: BSEQ(0xea,0xae,0xb7) /* ꮷ */
    case 0x13e8 /* Ꮸ */: BSEQ(0xea,0xae,0xb8) /* ꮸ */
    case 0x13e9 /* Ꮹ */: BSEQ(0xea,0xae,0xb9) /* ꮹ */
    case 0x13ea /* Ꮺ */: BSEQ(0xea,0xae,0xba) /* ꮺ */
    case 0x13eb /* Ꮻ */: BSEQ(0xea,0xae,0xbb) /* ꮻ */
    case 0x13ec /* Ꮼ */: BSEQ(0xea,0xae,0xbc) /* ꮼ */
    case 0x13ed /* Ꮽ */: BSEQ(0xea,0xae,0xbd) /* ꮽ */
    case 0x13ee /* Ꮾ */: BSEQ(0xea,0xae,0xbe) /* ꮾ */
    case 0x13ef /* Ꮿ */: BSEQ(0xea,0xae,0xbf) /* ꮿ */
    case 0x13f0 /* Ᏸ */: BSEQ(0xe1,0x8f,0xb8) /* ᏸ */
    case 0x13f1 /* Ᏹ */: BSEQ(0xe1,0x8f,0xb9) /* ᏹ */
    case 0x13f2 /* Ᏺ */: BSEQ(0xe1,0x8f,0xba) /* ᏺ */
    case 0x13f3 /* Ᏻ */: BSEQ(0xe1,0x8f,0xbb) /* ᏻ */
    case 0x13f4 /* Ᏼ */: BSEQ(0xe1,0x8f,0xbc) /* ᏼ */
    case 0x13f5 /* Ᏽ */: BSEQ(0xe1,0x8f,0xbd) /* ᏽ */
    case 0x1c90 /* Ა */: BSEQ(0xe1,0x83,0x90) /* ა */
    case 0x1c91 /* Ბ */: BSEQ(0xe1,0x83,0x91) /* ბ */
    case 0x1c92 /* Გ */: BSEQ(0xe1,0x83,0x92) /* გ */
    case 0x1c93 /* Დ */: BSEQ(0xe1,0x83,0x93) /* დ */
    case 0x1c94 /* Ე */: BSEQ(0xe1,0x83,0x94) /* ე */
    case 0x1c95 /* Ვ */: BSEQ(0xe1,0x83,0x95) /* ვ */
    case 0x1c96 /* Ზ */: BSEQ(0xe1,0x83,0x96) /* ზ */
    case 0x1c97 /* Თ */: BSEQ(0xe1,0x83,0x97) /* თ */
    case 0x1c98 /* Ი */: BSEQ(0xe1,0x83,0x98) /* ი */
    case 0x1c99 /* Კ */: BSEQ(0xe1,0x83,0x99) /* კ */
    case 0x1c9a /* Ლ */: BSEQ(0xe1,0x83,0x9a) /* ლ */
    case 0x1c9b /* Მ */: BSEQ(0xe1,0x83,0x9b) /* მ */
    case 0x1c9c /* Ნ */: BSEQ(0xe1,0x83,0x9c) /* ნ */
    case 0x1c9d /* Ო */: BSEQ(0xe1,0x83,0x9d) /* ო */
    case 0x1c9e /* Პ */: BSEQ(0xe1,0x83,0x9e) /* პ */
    case 0x1c9f /* Ჟ */: BSEQ(0xe1,0x83,0x9f) /* ჟ */
    case 0x1ca0 /* Რ */: BSEQ(0xe1,0x83,0xa0) /* რ */
    case 0x1ca1 /* Ს */: BSEQ(0xe1,0x83,0xa1) /* ს */
    case 0x1ca2 /* Ტ */: BSEQ(0xe1,0x83,0xa2) /* ტ */
    case 0x1ca3 /* Უ */: BSEQ(0xe1,0x83,0xa3) /* უ */
    case 0x1ca4 /* Ფ */: BSEQ(0xe1,0x83,0xa4) /* ფ */
    case 0x1ca5 /* Ქ */: BSEQ(0xe1,0x83,0xa5) /* ქ */
    case 0x1ca6 /* Ღ */: BSEQ(0xe1,0x83,0xa6) /* ღ */
    case 0x1ca7 /* Ყ */: BSEQ(0xe1,0x83,0xa7) /* ყ */
    case 0x1ca8 /* Შ */: BSEQ(0xe1,0x83,0xa8) /* შ */
    case 0x1ca9 /* Ჩ */: BSEQ(0xe1,0x83,0xa9) /* ჩ */
    case 0x1caa /* Ც */: BSEQ(0xe1,0x83,0xaa) /* ც */
    case 0x1cab /* Ძ */: BSEQ(0xe1,0x83,0xab) /* ძ */
    case 0x1cac /* Წ */: BSEQ(0xe1,0x83,0xac) /* წ */
    case 0x1cad /* Ჭ */: BSEQ(0xe1,0x83,0xad) /* ჭ */
    case 0x1cae /* Ხ */: BSEQ(0xe1,0x83,0xae) /* ხ */
    case 0x1caf /* Ჯ */: BSEQ(0xe1,0x83,0xaf) /* ჯ */
    case 0x1cb0 /* Ჰ */: BSEQ(0xe1,0x83,0xb0) /* ჰ */
    case 0x1cb1 /* Ჱ */: BSEQ(0xe1,0x83,0xb1) /* ჱ */
    case 0x1cb2 /* Ჲ */: BSEQ(0xe1,0x83,0xb2) /* ჲ */
    case 0x1cb3 /* Ჳ */: BSEQ(0xe1,0x83,0xb3) /* ჳ */
    case 0x1cb4 /* Ჴ */: BSEQ(0xe1,0x83,0xb4) /* ჴ */
    case 0x1cb5 /* Ჵ */: BSEQ(0xe1,0x83,0xb5) /* ჵ */
    case 0x1cb6 /* Ჶ */: BSEQ(0xe1,0x83,0xb6) /* ჶ */
    case 0x1cb7 /* Ჷ */: BSEQ(0xe1,0x83,0xb7) /* ჷ */
    case 0x1cb8 /* Ჸ */: BSEQ(0xe1,0x83,0xb8) /* ჸ */
    case 0x1cb9 /* Ჹ */: BSEQ(0xe1,0x83,0xb9) /* ჹ */
    case 0x1cba /* Ჺ */: BSEQ(0xe1,0x83,0xba) /* ჺ */
    case 0x1cbd /* Ჽ */: BSEQ(0xe1,0x83,0xbd) /* ჽ */
    case 0x1cbe /* Ჾ */: BSEQ(0xe1,0x83,0xbe) /* ჾ */
    case 0x1cbf /* Ჿ */: BSEQ(0xe1,0x83,0xbf) /* ჿ */
    case 0x1e00 /* Ḁ */: BSEQ(0xe1,0xb8,0x81) /* ḁ */
    case 0x1e02 /* Ḃ */: BSEQ(0xe1,0xb8,0x83) /* ḃ */
    case 0x1e04 /* Ḅ */: BSEQ(0xe1,0xb8,0x85) /* ḅ */
    case 0x1e06 /* Ḇ */: BSEQ(0xe1,0xb8,0x87) /* ḇ */
    case 0x1e08 /* Ḉ */: BSEQ(0xe1,0xb8,0x89) /* ḉ */
    case 0x1e0a /* Ḋ */: BSEQ(0xe1,0xb8,0x8b) /* ḋ */
    case 0x1e0c /* Ḍ */: BSEQ(0xe1,0xb8,0x8d) /* ḍ */
    case 0x1e0e /* Ḏ */: BSEQ(0xe1,0xb8,0x8f) /* ḏ */
    case 0x1e10 /* Ḑ */: BSEQ(0xe1,0xb8,0x91) /* ḑ */
    case 0x1e12 /* Ḓ */: BSEQ(0xe1,0xb8,0x93) /* ḓ */
    case 0x1e14 /* Ḕ */: BSEQ(0xe1,0xb8,0x95) /* ḕ */
    case 0x1e16 /* Ḗ */: BSEQ(0xe1,0xb8,0x97) /* ḗ */
    case 0x1e18 /* Ḙ */: BSEQ(0xe1,0xb8,0x99) /* ḙ */
    case 0x1e1a /* Ḛ */: BSEQ(0xe1,0xb8,0x9b) /* ḛ */
    case 0x1e1c /* Ḝ */: BSEQ(0xe1,0xb8,0x9d) /* ḝ */
    case 0x1e1e /* Ḟ */: BSEQ(0xe1,0xb8,0x9f) /* ḟ */
    case 0x1e20 /* Ḡ */: BSEQ(0xe1,0xb8,0xa1) /* ḡ */
    case 0x1e22 /* Ḣ */: BSEQ(0xe1,0xb8,0xa3) /* ḣ */
    case 0x1e24 /* Ḥ */: BSEQ(0xe1,0xb8,0xa5) /* ḥ */
    case 0x1e26 /* Ḧ */: BSEQ(0xe1,0xb8,0xa7) /* ḧ */
    case 0x1e28 /* Ḩ */: BSEQ(0xe1,0xb8,0xa9) /* ḩ */
    case 0x1e2a /* Ḫ */: BSEQ(0xe1,0xb8,0xab) /* ḫ */
    case 0x1e2c /* Ḭ */: BSEQ(0xe1,0xb8,0xad) /* ḭ */
    case 0x1e2e /* Ḯ */: BSEQ(0xe1,0xb8,0xaf) /* ḯ */
    case 0x1e30 /* Ḱ */: BSEQ(0xe1,0xb8,0xb1) /* ḱ */
    case 0x1e32 /* Ḳ */: BSEQ(0xe1,0xb8,0xb3) /* ḳ */
    case 0x1e34 /* Ḵ */: BSEQ(0xe1,0xb8,0xb5) /* ḵ */
    case 0x1e36 /* Ḷ */: BSEQ(0xe1,0xb8,0xb7) /* ḷ */
    case 0x1e38 /* Ḹ */: BSEQ(0xe1,0xb8,0xb9) /* ḹ */
    case 0x1e3a /* Ḻ */: BSEQ(0xe1,0xb8,0xbb) /* ḻ */
    case 0x1e3c /* Ḽ */: BSEQ(0xe1,0xb8,0xbd) /* ḽ */
    case 0x1e3e /* Ḿ */: BSEQ(0xe1,0xb8,0xbf) /* ḿ */
    case 0x1e40 /* Ṁ */: BSEQ(0xe1,0xb9,0x81) /* ṁ */
    case 0x1e42 /* Ṃ */: BSEQ(0xe1,0xb9,0x83) /* ṃ */
    case 0x1e44 /* Ṅ */: BSEQ(0xe1,0xb9,0x85) /* ṅ */
    case 0x1e46 /* Ṇ */: BSEQ(0xe1,0xb9,0x87) /* ṇ */
    case 0x1e48 /* Ṉ */: BSEQ(0xe1,0xb9,0x89) /* ṉ */
    case 0x1e4a /* Ṋ */: BSEQ(0xe1,0xb9,0x8b) /* ṋ */
    case 0x1e4c /* Ṍ */: BSEQ(0xe1,0xb9,0x8d) /* ṍ */
    case 0x1e4e /* Ṏ */: BSEQ(0xe1,0xb9,0x8f) /* ṏ */
    case 0x1e50 /* Ṑ */: BSEQ(0xe1,0xb9,0x91) /* ṑ */
    case 0x1e52 /* Ṓ */: BSEQ(0xe1,0xb9,0x93) /* ṓ */
    case 0x1e54 /* Ṕ */: BSEQ(0xe1,0xb9,0x95) /* ṕ */
    case 0x1e56 /* Ṗ */: BSEQ(0xe1,0xb9,0x97) /* ṗ */
    case 0x1e58 /* Ṙ */: BSEQ(0xe1,0xb9,0x99) /* ṙ */
    case 0x1e5a /* Ṛ */: BSEQ(0xe1,0xb9,0x9b) /* ṛ */
    case 0x1e5c /* Ṝ */: BSEQ(0xe1,0xb9,0x9d) /* ṝ */
    case 0x1e5e /* Ṟ */: BSEQ(0xe1,0xb9,0x9f) /* ṟ */
    case 0x1e60 /* Ṡ */: BSEQ(0xe1,0xb9,0xa1) /* ṡ */
    case 0x1e62 /* Ṣ */: BSEQ(0xe1,0xb9,0xa3) /* ṣ */
    case 0x1e64 /* Ṥ */: BSEQ(0xe1,0xb9,0xa5) /* ṥ */
    case 0x1e66 /* Ṧ */: BSEQ(0xe1,0xb9,0xa7) /* ṧ */
    case 0x1e68 /* Ṩ */: BSEQ(0xe1,0xb9,0xa9) /* ṩ */
    case 0x1e6a /* Ṫ */: BSEQ(0xe1,0xb9,0xab) /* ṫ */
    case 0x1e6c /* Ṭ */: BSEQ(0xe1,0xb9,0xad) /* ṭ */
    case 0x1e6e /* Ṯ */: BSEQ(0xe1,0xb9,0xaf) /* ṯ */
    case 0x1e70 /* Ṱ */: BSEQ(0xe1,0xb9,0xb1) /* ṱ */
    case 0x1e72 /* Ṳ */: BSEQ(0xe1,0xb9,0xb3) /* ṳ */
    case 0x1e74 /* Ṵ */: BSEQ(0xe1,0xb9,0xb5) /* ṵ */
    case 0x1e76 /* Ṷ */: BSEQ(0xe1,0xb9,0xb7) /* ṷ */
    case 0x1e78 /* Ṹ */: BSEQ(0xe1,0xb9,0xb9) /* ṹ */
    case 0x1e7a /* Ṻ */: BSEQ(0xe1,0xb9,0xbb) /* ṻ */
    case 0x1e7c /* Ṽ */: BSEQ(0xe1,0xb9,0xbd) /* ṽ */
    case 0x1e7e /* Ṿ */: BSEQ(0xe1,0xb9,0xbf) /* ṿ */
    case 0x1e80 /* Ẁ */: BSEQ(0xe1,0xba,0x81) /* ẁ */
    case 0x1e82 /* Ẃ */: BSEQ(0xe1,0xba,0x83) /* ẃ */
    case 0x1e84 /* Ẅ */: BSEQ(0xe1,0xba,0x85) /* ẅ */
    case 0x1e86 /* Ẇ */: BSEQ(0xe1,0xba,0x87) /* ẇ */
    case 0x1e88 /* Ẉ */: BSEQ(0xe1,0xba,0x89) /* ẉ */
    case 0x1e8a /* Ẋ */: BSEQ(0xe1,0xba,0x8b) /* ẋ */
    case 0x1e8c /* Ẍ */: BSEQ(0xe1,0xba,0x8d) /* ẍ */
    case 0x1e8e /* Ẏ */: BSEQ(0xe1,0xba,0x8f) /* ẏ */
    case 0x1e90 /* Ẑ */: BSEQ(0xe1,0xba,0x91) /* ẑ */
    case 0x1e92 /* Ẓ */: BSEQ(0xe1,0xba,0x93) /* ẓ */
    case 0x1e94 /* Ẕ */: BSEQ(0xe1,0xba,0x95) /* ẕ */
    case 0x1e9e /* ẞ */: BSEQ(0xc3,0x9f) /* ß */
    case 0x1ea0 /* Ạ */: BSEQ(0xe1,0xba,0xa1) /* ạ */
    case 0x1ea2 /* Ả */: BSEQ(0xe1,0xba,0xa3) /* ả */
    case 0x1ea4 /* Ấ */: BSEQ(0xe1,0xba,0xa5) /* ấ */
    case 0x1ea6 /* Ầ */: BSEQ(0xe1,0xba,0xa7) /* ầ */
    case 0x1ea8 /* Ẩ */: BSEQ(0xe1,0xba,0xa9) /* ẩ */
    case 0x1eaa /* Ẫ */: BSEQ(0xe1,0xba,0xab) /* ẫ */
    case 0x1eac /* Ậ */: BSEQ(0xe1,0xba,0xad) /* ậ */
    case 0x1eae /* Ắ */: BSEQ(0xe1,0xba,0xaf) /* ắ */
    case 0x1eb0 /* Ằ */: BSEQ(0xe1,0xba,0xb1) /* ằ */
    case 0x1eb2 /* Ẳ */: BSEQ(0xe1,0xba,0xb3) /* ẳ */
    case 0x1eb4 /* Ẵ */: BSEQ(0xe1,0xba,0xb5) /* ẵ */
    case 0x1eb6 /* Ặ */: BSEQ(0xe1,0xba,0xb7) /* ặ */
    case 0x1eb8 /* Ẹ */: BSEQ(0xe1,0xba,0xb9) /* ẹ */
    case 0x1eba /* Ẻ */: BSEQ(0xe1,0xba,0xbb) /* ẻ */
    case 0x1ebc /* Ẽ */: BSEQ(0xe1,0xba,0xbd) /* ẽ */
    case 0x1ebe /* Ế */: BSEQ(0xe1,0xba,0xbf) /* ế */
    case 0x1ec0 /* Ề */: BSEQ(0xe1,0xbb,0x81) /* ề */
    case 0x1ec2 /* Ể */: BSEQ(0xe1,0xbb,0x83) /* ể */
    case 0x1ec4 /* Ễ */: BSEQ(0xe1,0xbb,0x85) /* ễ */
    case 0x1ec6 /* Ệ */: BSEQ(0xe1,0xbb,0x87) /* ệ */
    case 0x1ec8 /* Ỉ */: BSEQ(0xe1,0xbb,0x89) /* ỉ */
    case 0x1eca /* Ị */: BSEQ(0xe1,0xbb,0x8b) /* ị */
    case 0x1ecc /* Ọ */: BSEQ(0xe1,0xbb,0x8d) /* ọ */
    case 0x1ece /* Ỏ */: BSEQ(0xe1,0xbb,0x8f) /* ỏ */
    case 0x1ed0 /* Ố */: BSEQ(0xe1,0xbb,0x91) /* ố */
    case 0x1ed2 /* Ồ */: BSEQ(0xe1,0xbb,0x93) /* ồ */
    case 0x1ed4 /* Ổ */: BSEQ(0xe1,0xbb,0x95) /* ổ */
    case 0x1ed6 /* Ỗ */: BSEQ(0xe1,0xbb,0x97) /* ỗ */
    case 0x1ed8 /* Ộ */: BSEQ(0xe1,0xbb,0x99) /* ộ */
    case 0x1eda /* Ớ */: BSEQ(0xe1,0xbb,0x9b) /* ớ */
    case 0x1edc /* Ờ */: BSEQ(0xe1,0xbb,0x9d) /* ờ */
    case 0x1ede /* Ở */: BSEQ(0xe1,0xbb,0x9f) /* ở */
    case 0x1ee0 /* Ỡ */: BSEQ(0xe1,0xbb,0xa1) /* ỡ */
    case 0x1ee2 /* Ợ */: BSEQ(0xe1,0xbb,0xa3) /* ợ */
    case 0x1ee4 /* Ụ */: BSEQ(0xe1,0xbb,0xa5) /* ụ */
    case 0x1ee6 /* Ủ */: BSEQ(0xe1,0xbb,0xa7) /* ủ */
    case 0x1ee8 /* Ứ */: BSEQ(0xe1,0xbb,0xa9) /* ứ */
    case 0x1eea /* Ừ */: BSEQ(0xe1,0xbb,0xab) /* ừ */
    case 0x1eec /* Ử */: BSEQ(0xe1,0xbb,0xad) /* ử */
    case 0x1eee /* Ữ */: BSEQ(0xe1,0xbb,0xaf) /* ữ */
    case 0x1ef0 /* Ự */: BSEQ(0xe1,0xbb,0xb1) /* ự */
    case 0x1ef2 /* Ỳ */: BSEQ(0xe1,0xbb,0xb3) /* ỳ */
    case 0x1ef4 /* Ỵ */: BSEQ(0xe1,0xbb,0xb5) /* ỵ */
    case 0x1ef6 /* Ỷ */: BSEQ(0xe1,0xbb,0xb7) /* ỷ */
    case 0x1ef8 /* Ỹ */: BSEQ(0xe1,0xbb,0xb9) /* ỹ */
    case 0x1efa /* Ỻ */: BSEQ(0xe1,0xbb,0xbb) /* ỻ */
    case 0x1efc /* Ỽ */: BSEQ(0xe1,0xbb,0xbd) /* ỽ */
    case 0x1efe /* Ỿ */: BSEQ(0xe1,0xbb,0xbf) /* ỿ */
    case 0x1f08 /* Ἀ */: BSEQ(0xe1,0xbc,0x80) /* ἀ */
    case 0x1f09 /* Ἁ */: BSEQ(0xe1,0xbc,0x81) /* ἁ */
    case 0x1f0a /* Ἂ */: BSEQ(0xe1,0xbc,0x82) /* ἂ */
    case 0x1f0b /* Ἃ */: BSEQ(0xe1,0xbc,0x83) /* ἃ */
    case 0x1f0c /* Ἄ */: BSEQ(0xe1,0xbc,0x84) /* ἄ */
    case 0x1f0d /* Ἅ */: BSEQ(0xe1,0xbc,0x85) /* ἅ */
    case 0x1f0e /* Ἆ */: BSEQ(0xe1,0xbc,0x86) /* ἆ */
    case 0x1f0f /* Ἇ */: BSEQ(0xe1,0xbc,0x87) /* ἇ */
    case 0x1f18 /* Ἐ */: BSEQ(0xe1,0xbc,0x90) /* ἐ */
    case 0x1f19 /* Ἑ */: BSEQ(0xe1,0xbc,0x91) /* ἑ */
    case 0x1f1a /* Ἒ */: BSEQ(0xe1,0xbc,0x92) /* ἒ */
    case 0x1f1b /* Ἓ */: BSEQ(0xe1,0xbc,0x93) /* ἓ */
    case 0x1f1c /* Ἔ */: BSEQ(0xe1,0xbc,0x94) /* ἔ */
    case 0x1f1d /* Ἕ */: BSEQ(0xe1,0xbc,0x95) /* ἕ */
    case 0x1f28 /* Ἠ */: BSEQ(0xe1,0xbc,0xa0) /* ἠ */
    case 0x1f29 /* Ἡ */: BSEQ(0xe1,0xbc,0xa1) /* ἡ */
    case 0x1f2a /* Ἢ */: BSEQ(0xe1,0xbc,0xa2) /* ἢ */
    case 0x1f2b /* Ἣ */: BSEQ(0xe1,0xbc,0xa3) /* ἣ */
    case 0x1f2c /* Ἤ */: BSEQ(0xe1,0xbc,0xa4) /* ἤ */
    case 0x1f2d /* Ἥ */: BSEQ(0xe1,0xbc,0xa5) /* ἥ */
    case 0x1f2e /* Ἦ */: BSEQ(0xe1,0xbc,0xa6) /* ἦ */
    case 0x1f2f /* Ἧ */: BSEQ(0xe1,0xbc,0xa7) /* ἧ */
    case 0x1f38 /* Ἰ */: BSEQ(0xe1,0xbc,0xb0) /* ἰ */
    case 0x1f39 /* Ἱ */: BSEQ(0xe1,0xbc,0xb1) /* ἱ */
    case 0x1f3a /* Ἲ */: BSEQ(0xe1,0xbc,0xb2) /* ἲ */
    case 0x1f3b /* Ἳ */: BSEQ(0xe1,0xbc,0xb3) /* ἳ */
    case 0x1f3c /* Ἴ */: BSEQ(0xe1,0xbc,0xb4) /* ἴ */
    case 0x1f3d /* Ἵ */: BSEQ(0xe1,0xbc,0xb5) /* ἵ */
    case 0x1f3e /* Ἶ */: BSEQ(0xe1,0xbc,0xb6) /* ἶ */
    case 0x1f3f /* Ἷ */: BSEQ(0xe1,0xbc,0xb7) /* ἷ */
    case 0x1f48 /* Ὀ */: BSEQ(0xe1,0xbd,0x80) /* ὀ */
    case 0x1f49 /* Ὁ */: BSEQ(0xe1,0xbd,0x81) /* ὁ */
    case 0x1f4a /* Ὂ */: BSEQ(0xe1,0xbd,0x82) /* ὂ */
    case 0x1f4b /* Ὃ */: BSEQ(0xe1,0xbd,0x83) /* ὃ */
    case 0x1f4c /* Ὄ */: BSEQ(0xe1,0xbd,0x84) /* ὄ */
    case 0x1f4d /* Ὅ */: BSEQ(0xe1,0xbd,0x85) /* ὅ */
    case 0x1f59 /* Ὑ */: BSEQ(0xe1,0xbd,0x91) /* ὑ */
    case 0x1f5b /* Ὓ */: BSEQ(0xe1,0xbd,0x93) /* ὓ */
    case 0x1f5d /* Ὕ */: BSEQ(0xe1,0xbd,0x95) /* ὕ */
    case 0x1f5f /* Ὗ */: BSEQ(0xe1,0xbd,0x97) /* ὗ */
    case 0x1f68 /* Ὠ */: BSEQ(0xe1,0xbd,0xa0) /* ὠ */
    case 0x1f69 /* Ὡ */: BSEQ(0xe1,0xbd,0xa1) /* ὡ */
    case 0x1f6a /* Ὢ */: BSEQ(0xe1,0xbd,0xa2) /* ὢ */
    case 0x1f6b /* Ὣ */: BSEQ(0xe1,0xbd,0xa3) /* ὣ */
    case 0x1f6c /* Ὤ */: BSEQ(0xe1,0xbd,0xa4) /* ὤ */
    case 0x1f6d /* Ὥ */: BSEQ(0xe1,0xbd,0xa5) /* ὥ */
    case 0x1f6e /* Ὦ */: BSEQ(0xe1,0xbd,0xa6) /* ὦ */
    case 0x1f6f /* Ὧ */: BSEQ(0xe1,0xbd,0xa7) /* ὧ */
    case 0x1f88 /* ᾈ */: BSEQ(0xe1,0xbe,0x80) /* ᾀ */
    case 0x1f89 /* ᾉ */: BSEQ(0xe1,0xbe,0x81) /* ᾁ */
    case 0x1f8a /* ᾊ */: BSEQ(0xe1,0xbe,0x82) /* ᾂ */
    case 0x1f8b /* ᾋ */: BSEQ(0xe1,0xbe,0x83) /* ᾃ */
    case 0x1f8c /* ᾌ */: BSEQ(0xe1,0xbe,0x84) /* ᾄ */
    case 0x1f8d /* ᾍ */: BSEQ(0xe1,0xbe,0x85) /* ᾅ */
    case 0x1f8e /* ᾎ */: BSEQ(0xe1,0xbe,0x86) /* ᾆ */
    case 0x1f8f /* ᾏ */: BSEQ(0xe1,0xbe,0x87) /* ᾇ */
    case 0x1f98 /* ᾘ */: BSEQ(0xe1,0xbe,0x90) /* ᾐ */
    case 0x1f99 /* ᾙ */: BSEQ(0xe1,0xbe,0x91) /* ᾑ */
    case 0x1f9a /* ᾚ */: BSEQ(0xe1,0xbe,0x92) /* ᾒ */
    case 0x1f9b /* ᾛ */: BSEQ(0xe1,0xbe,0x93) /* ᾓ */
    case 0x1f9c /* ᾜ */: BSEQ(0xe1,0xbe,0x94) /* ᾔ */
    case 0x1f9d /* ᾝ */: BSEQ(0xe1,0xbe,0x95) /* ᾕ */
    case 0x1f9e /* ᾞ */: BSEQ(0xe1,0xbe,0x96) /* ᾖ */
    case 0x1f9f /* ᾟ */: BSEQ(0xe1,0xbe,0x97) /* ᾗ */
    case 0x1fa8 /* ᾨ */: BSEQ(0xe1,0xbe,0xa0) /* ᾠ */
    case 0x1fa9 /* ᾩ */: BSEQ(0xe1,0xbe,0xa1) /* ᾡ */
    case 0x1faa /* ᾪ */: BSEQ(0xe1,0xbe,0xa2) /* ᾢ */
    case 0x1fab /* ᾫ */: BSEQ(0xe1,0xbe,0xa3) /* ᾣ */
    case 0x1fac /* ᾬ */: BSEQ(0xe1,0xbe,0xa4) /* ᾤ */
    case 0x1fad /* ᾭ */: BSEQ(0xe1,0xbe,0xa5) /* ᾥ */
    case 0x1fae /* ᾮ */: BSEQ(0xe1,0xbe,0xa6) /* ᾦ */
    case 0x1faf /* ᾯ */: BSEQ(0xe1,0xbe,0xa7) /* ᾧ */
    case 0x1fb8 /* Ᾰ */: BSEQ(0xe1,0xbe,0xb0) /* ᾰ */
    case 0x1fb9 /* Ᾱ */: BSEQ(0xe1,0xbe,0xb1) /* ᾱ */
    case 0x1fba /* Ὰ */: BSEQ(0xe1,0xbd,0xb0) /* ὰ */
    case 0x1fbb /* Ά */: BSEQ(0xe1,0xbd,0xb1) /* ά */
    case 0x1fbc /* ᾼ */: BSEQ(0xe1,0xbe,0xb3) /* ᾳ */
    case 0x1fc8 /* Ὲ */: BSEQ(0xe1,0xbd,0xb2) /* ὲ */
    case 0x1fc9 /* Έ */: BSEQ(0xe1,0xbd,0xb3) /* έ */
    case 0x1fca /* Ὴ */: BSEQ(0xe1,0xbd,0xb4) /* ὴ */
    case 0x1fcb /* Ή */: BSEQ(0xe1,0xbd,0xb5) /* ή */
    case 0x1fcc /* ῌ */: BSEQ(0xe1,0xbf,0x83) /* ῃ */
    case 0x1fd8 /* Ῐ */: BSEQ(0xe1,0xbf,0x90) /* ῐ */
    case 0x1fd9 /* Ῑ */: BSEQ(0xe1,0xbf,0x91) /* ῑ */
    case 0x1fda /* Ὶ */: BSEQ(0xe1,0xbd,0xb6) /* ὶ */
    case 0x1fdb /* Ί */: BSEQ(0xe1,0xbd,0xb7) /* ί */
    case 0x1fe8 /* Ῠ */: BSEQ(0xe1,0xbf,0xa0) /* ῠ */
    case 0x1fe9 /* Ῡ */: BSEQ(0xe1,0xbf,0xa1) /* ῡ */
    case 0x1fea /* Ὺ */: BSEQ(0xe1,0xbd,0xba) /* ὺ */
    case 0x1feb /* Ύ */: BSEQ(0xe1,0xbd,0xbb) /* ύ */
    case 0x1fec /* Ῥ */: BSEQ(0xe1,0xbf,0xa5) /* ῥ */
    case 0x1ff8 /* Ὸ */: BSEQ(0xe1,0xbd,0xb8) /* ὸ */
    case 0x1ff9 /* Ό */: BSEQ(0xe1,0xbd,0xb9) /* ό */
    case 0x1ffa /* Ὼ */: BSEQ(0xe1,0xbd,0xbc) /* ὼ */
    case 0x1ffb /* Ώ */: BSEQ(0xe1,0xbd,0xbd) /* ώ */
    case 0x1ffc /* ῼ */: BSEQ(0xe1,0xbf,0xb3) /* ῳ */
    case 0x2126 /* Ω */: BSEQ(0xcf,0x89) /* ω */
    case 0x212a /* K */: BSEQ(0x6b) /* k */
    case 0x212b /* Å */: BSEQ(0xc3,0xa5) /* å */
    case 0x2132 /* Ⅎ */: BSEQ(0xe2,0x85,0x8e) /* ⅎ */
    case 0x2160 /* Ⅰ */: BSEQ(0xe2,0x85,0xb0) /* ⅰ */
    case 0x2161 /* Ⅱ */: BSEQ(0xe2,0x85,0xb1) /* ⅱ */
    case 0x2162 /* Ⅲ */: BSEQ(0xe2,0x85,0xb2) /* ⅲ */
    case 0x2163 /* Ⅳ */: BSEQ(0xe2,0x85,0xb3) /* ⅳ */
    case 0x2164 /* Ⅴ */: BSEQ(0xe2,0x85,0xb4) /* ⅴ */
    case 0x2165 /* Ⅵ */: BSEQ(0xe2,0x85,0xb5) /* ⅵ */
    case 0x2166 /* Ⅶ */: BSEQ(0xe2,0x85,0xb6) /* ⅶ */
    case 0x2167 /* Ⅷ */: BSEQ(0xe2,0x85,0xb7) /* ⅷ */
    case 0x2168 /* Ⅸ */: BSEQ(0xe2,0x85,0xb8) /* ⅸ */
    case 0x2169 /* Ⅹ */: BSEQ(0xe2,0x85,0xb9) /* ⅹ */
    case 0x216a /* Ⅺ */: BSEQ(0xe2,0x85,0xba) /* ⅺ */
    case 0x216b /* Ⅻ */: BSEQ(0xe2,0x85,0xbb) /* ⅻ */
    case 0x216c /* Ⅼ */: BSEQ(0xe2,0x85,0xbc) /* ⅼ */
    case 0x216d /* Ⅽ */: BSEQ(0xe2,0x85,0xbd) /* ⅽ */
    case 0x216e /* Ⅾ */: BSEQ(0xe2,0x85,0xbe) /* ⅾ */
    case 0x216f /* Ⅿ */: BSEQ(0xe2,0x85,0xbf) /* ⅿ */
    case 0x2183 /* Ↄ */: BSEQ(0xe2,0x86,0x84) /* ↄ */
    case 0x24b6 /* Ⓐ */: BSEQ(0xe2,0x93,0x90) /* ⓐ */
    case 0x24b7 /* Ⓑ */: BSEQ(0xe2,0x93,0x91) /* ⓑ */
    case 0x24b8 /* Ⓒ */: BSEQ(0xe2,0x93,0x92) /* ⓒ */
    case 0x24b9 /* Ⓓ */: BSEQ(0xe2,0x93,0x93) /* ⓓ */
    case 0x24ba /* Ⓔ */: BSEQ(0xe2,0x93,0x94) /* ⓔ */
    case 0x24bb /* Ⓕ */: BSEQ(0xe2,0x93,0x95) /* ⓕ */
    case 0x24bc /* Ⓖ */: BSEQ(0xe2,0x93,0x96) /* ⓖ */
    case 0x24bd /* Ⓗ */: BSEQ(0xe2,0x93,0x97) /* ⓗ */
    case 0x24be /* Ⓘ */: BSEQ(0xe2,0x93,0x98) /* ⓘ */
    case 0x24bf /* Ⓙ */: BSEQ(0xe2,0x93,0x99) /* ⓙ */
    case 0x24c0 /* Ⓚ */: BSEQ(0xe2,0x93,0x9a) /* ⓚ */
    case 0x24c1 /* Ⓛ */: BSEQ(0xe2,0x93,0x9b) /* ⓛ */
    case 0x24c2 /* Ⓜ */: BSEQ(0xe2,0x93,0x9c) /* ⓜ */
    case 0x24c3 /* Ⓝ */: BSEQ(0xe2,0x93,0x9d) /* ⓝ */
    case 0x24c4 /* Ⓞ */: BSEQ(0xe2,0x93,0x9e) /* ⓞ */
    case 0x24c5 /* Ⓟ */: BSEQ(0xe2,0x93,0x9f) /* ⓟ */
    case 0x24c6 /* Ⓠ */: BSEQ(0xe2,0x93,0xa0) /* ⓠ */
    case 0x24c7 /* Ⓡ */: BSEQ(0xe2,0x93,0xa1) /* ⓡ */
    case 0x24c8 /* Ⓢ */: BSEQ(0xe2,0x93,0xa2) /* ⓢ */
    case 0x24c9 /* Ⓣ */: BSEQ(0xe2,0x93,0xa3) /* ⓣ */
    case 0x24ca /* Ⓤ */: BSEQ(0xe2,0x93,0xa4) /* ⓤ */
    case 0x24cb /* Ⓥ */: BSEQ(0xe2,0x93,0xa5) /* ⓥ */
    case 0x24cc /* Ⓦ */: BSEQ(0xe2,0x93,0xa6) /* ⓦ */
    case 0x24cd /* Ⓧ */: BSEQ(0xe2,0x93,0xa7) /* ⓧ */
    case 0x24ce /* Ⓨ */: BSEQ(0xe2,0x93,0xa8) /* ⓨ */
    case 0x24cf /* Ⓩ */: BSEQ(0xe2,0x93,0xa9) /* ⓩ */
    case 0x2c00 /* Ⰰ */: BSEQ(0xe2,0xb0,0xb0) /* ⰰ */
    case 0x2c01 /* Ⰱ */: BSEQ(0xe2,0xb0,0xb1) /* ⰱ */
    case 0x2c02 /* Ⰲ */: BSEQ(0xe2,0xb0,0xb2) /* ⰲ */
    case 0x2c03 /* Ⰳ */: BSEQ(0xe2,0xb0,0xb3) /* ⰳ */
    case 0x2c04 /* Ⰴ */: BSEQ(0xe2,0xb0,0xb4) /* ⰴ */
    case 0x2c05 /* Ⰵ */: BSEQ(0xe2,0xb0,0xb5) /* ⰵ */
    case 0x2c06 /* Ⰶ */: BSEQ(0xe2,0xb0,0xb6) /* ⰶ */
    case 0x2c07 /* Ⰷ */: BSEQ(0xe2,0xb0,0xb7) /* ⰷ */
    case 0x2c08 /* Ⰸ */: BSEQ(0xe2,0xb0,0xb8) /* ⰸ */
    case 0x2c09 /* Ⰹ */: BSEQ(0xe2,0xb0,0xb9) /* ⰹ */
    case 0x2c0a /* Ⰺ */: BSEQ(0xe2,0xb0,0xba) /* ⰺ */
    case 0x2c0b /* Ⰻ */: BSEQ(0xe2,0xb0,0xbb) /* ⰻ */
    case 0x2c0c /* Ⰼ */: BSEQ(0xe2,0xb0,0xbc) /* ⰼ */
    case 0x2c0d /* Ⰽ */: BSEQ(0xe2,0xb0,0xbd) /* ⰽ */
    case 0x2c0e /* Ⰾ */: BSEQ(0xe2,0xb0,0xbe) /* ⰾ */
    case 0x2c0f /* Ⰿ */: BSEQ(0xe2,0xb0,0xbf) /* ⰿ */
    case 0x2c10 /* Ⱀ */: BSEQ(0xe2,0xb1,0x80) /* ⱀ */
    case 0x2c11 /* Ⱁ */: BSEQ(0xe2,0xb1,0x81) /* ⱁ */
    case 0x2c12 /* Ⱂ */: BSEQ(0xe2,0xb1,0x82) /* ⱂ */
    case 0x2c13 /* Ⱃ */: BSEQ(0xe2,0xb1,0x83) /* ⱃ */
    case 0x2c14 /* Ⱄ */: BSEQ(0xe2,0xb1,0x84) /* ⱄ */
    case 0x2c15 /* Ⱅ */: BSEQ(0xe2,0xb1,0x85) /* ⱅ */
    case 0x2c16 /* Ⱆ */: BSEQ(0xe2,0xb1,0x86) /* ⱆ */
    case 0x2c17 /* Ⱇ */: BSEQ(0xe2,0xb1,0x87) /* ⱇ */
    case 0x2c18 /* Ⱈ */: BSEQ(0xe2,0xb1,0x88) /* ⱈ */
    case 0x2c19 /* Ⱉ */: BSEQ(0xe2,0xb1,0x89) /* ⱉ */
    case 0x2c1a /* Ⱊ */: BSEQ(0xe2,0xb1,0x8a) /* ⱊ */
    case 0x2c1b /* Ⱋ */: BSEQ(0xe2,0xb1,0x8b) /* ⱋ */
    case 0x2c1c /* Ⱌ */: BSEQ(0xe2,0xb1,0x8c) /* ⱌ */
    case 0x2c1d /* Ⱍ */: BSEQ(0xe2,0xb1,0x8d) /* ⱍ */
    case 0x2c1e /* Ⱎ */: BSEQ(0xe2,0xb1,0x8e) /* ⱎ */
    case 0x2c1f /* Ⱏ */: BSEQ(0xe2,0xb1,0x8f) /* ⱏ */
    case 0x2c20 /* Ⱐ */: BSEQ(0xe2,0xb1,0x90) /* ⱐ */
    case 0x2c21 /* Ⱑ */: BSEQ(0xe2,0xb1,0x91) /* ⱑ */
    case 0x2c22 /* Ⱒ */: BSEQ(0xe2,0xb1,0x92) /* ⱒ */
    case 0x2c23 /* Ⱓ */: BSEQ(0xe2,0xb1,0x93) /* ⱓ */
    case 0x2c24 /* Ⱔ */: BSEQ(0xe2,0xb1,0x94) /* ⱔ */
    case 0x2c25 /* Ⱕ */: BSEQ(0xe2,0xb1,0x95) /* ⱕ */
    case 0x2c26 /* Ⱖ */: BSEQ(0xe2,0xb1,0x96) /* ⱖ */
    case 0x2c27 /* Ⱗ */: BSEQ(0xe2,0xb1,0x97) /* ⱗ */
    case 0x2c28 /* Ⱘ */: BSEQ(0xe2,0xb1,0x98) /* ⱘ */
    case 0x2c29 /* Ⱙ */: BSEQ(0xe2,0xb1,0x99) /* ⱙ */
    case 0x2c2a /* Ⱚ */: BSEQ(0xe2,0xb1,0x9a) /* ⱚ */
    case 0x2c2b /* Ⱛ */: BSEQ(0xe2,0xb1,0x9b) /* ⱛ */
    case 0x2c2c /* Ⱜ */: BSEQ(0xe2,0xb1,0x9c) /* ⱜ */
    case 0x2c2d /* Ⱝ */: BSEQ(0xe2,0xb1,0x9d) /* ⱝ */
    case 0x2c2e /* Ⱞ */: BSEQ(0xe2,0xb1,0x9e) /* ⱞ */
    case 0x2c60 /* Ⱡ */: BSEQ(0xe2,0xb1,0xa1) /* ⱡ */
    case 0x2c62 /* Ɫ */: BSEQ(0xc9,0xab) /* ɫ */
    case 0x2c63 /* Ᵽ */: BSEQ(0xe1,0xb5,0xbd) /* ᵽ */
    case 0x2c64 /* Ɽ */: BSEQ(0xc9,0xbd) /* ɽ */
    case 0x2c67 /* Ⱨ */: BSEQ(0xe2,0xb1,0xa8) /* ⱨ */
    case 0x2c69 /* Ⱪ */: BSEQ(0xe2,0xb1,0xaa) /* ⱪ */
    case 0x2c6b /* Ⱬ */: BSEQ(0xe2,0xb1,0xac) /* ⱬ */
    case 0x2c6d /* Ɑ */: BSEQ(0xc9,0x91) /* ɑ */
    case 0x2c6e /* Ɱ */: BSEQ(0xc9,0xb1) /* ɱ */
    case 0x2c6f /* Ɐ */: BSEQ(0xc9,0x90) /* ɐ */
    case 0x2c70 /* Ɒ */: BSEQ(0xc9,0x92) /* ɒ */
    case 0x2c72 /* Ⱳ */: BSEQ(0xe2,0xb1,0xb3) /* ⱳ */
    case 0x2c75 /* Ⱶ */: BSEQ(0xe2,0xb1,0xb6) /* ⱶ */
    case 0x2c7e /* Ȿ */: BSEQ(0xc8,0xbf) /* ȿ */
    case 0x2c7f /* Ɀ */: BSEQ(0xc9,0x80) /* ɀ */
    case 0x2c80 /* Ⲁ */: BSEQ(0xe2,0xb2,0x81) /* ⲁ */
    case 0x2c82 /* Ⲃ */: BSEQ(0xe2,0xb2,0x83) /* ⲃ */
    case 0x2c84 /* Ⲅ */: BSEQ(0xe2,0xb2,0x85) /* ⲅ */
    case 0x2c86 /* Ⲇ */: BSEQ(0xe2,0xb2,0x87) /* ⲇ */
    case 0x2c88 /* Ⲉ */: BSEQ(0xe2,0xb2,0x89) /* ⲉ */
    case 0x2c8a /* Ⲋ */: BSEQ(0xe2,0xb2,0x8b) /* ⲋ */
    case 0x2c8c /* Ⲍ */: BSEQ(0xe2,0xb2,0x8d) /* ⲍ */
    case 0x2c8e /* Ⲏ */: BSEQ(0xe2,0xb2,0x8f) /* ⲏ */
    case 0x2c90 /* Ⲑ */: BSEQ(0xe2,0xb2,0x91) /* ⲑ */
    case 0x2c92 /* Ⲓ */: BSEQ(0xe2,0xb2,0x93) /* ⲓ */
    case 0x2c94 /* Ⲕ */: BSEQ(0xe2,0xb2,0x95) /* ⲕ */
    case 0x2c96 /* Ⲗ */: BSEQ(0xe2,0xb2,0x97) /* ⲗ */
    case 0x2c98 /* Ⲙ */: BSEQ(0xe2,0xb2,0x99) /* ⲙ */
    case 0x2c9a /* Ⲛ */: BSEQ(0xe2,0xb2,0x9b) /* ⲛ */
    case 0x2c9c /* Ⲝ */: BSEQ(0xe2,0xb2,0x9d) /* ⲝ */
    case 0x2c9e /* Ⲟ */: BSEQ(0xe2,0xb2,0x9f) /* ⲟ */
    case 0x2ca0 /* Ⲡ */: BSEQ(0xe2,0xb2,0xa1) /* ⲡ */
    case 0x2ca2 /* Ⲣ */: BSEQ(0xe2,0xb2,0xa3) /* ⲣ */
    case 0x2ca4 /* Ⲥ */: BSEQ(0xe2,0xb2,0xa5) /* ⲥ */
    case 0x2ca6 /* Ⲧ */: BSEQ(0xe2,0xb2,0xa7) /* ⲧ */
    case 0x2ca8 /* Ⲩ */: BSEQ(0xe2,0xb2,0xa9) /* ⲩ */
    case 0x2caa /* Ⲫ */: BSEQ(0xe2,0xb2,0xab) /* ⲫ */
    case 0x2cac /* Ⲭ */: BSEQ(0xe2,0xb2,0xad) /* ⲭ */
    case 0x2cae /* Ⲯ */: BSEQ(0xe2,0xb2,0xaf) /* ⲯ */
    case 0x2cb0 /* Ⲱ */: BSEQ(0xe2,0xb2,0xb1) /* ⲱ */
    case 0x2cb2 /* Ⲳ */: BSEQ(0xe2,0xb2,0xb3) /* ⲳ */
    case 0x2cb4 /* Ⲵ */: BSEQ(0xe2,0xb2,0xb5) /* ⲵ */
    case 0x2cb6 /* Ⲷ */: BSEQ(0xe2,0xb2,0xb7) /* ⲷ */
    case 0x2cb8 /* Ⲹ */: BSEQ(0xe2,0xb2,0xb9) /* ⲹ */
    case 0x2cba /* Ⲻ */: BSEQ(0xe2,0xb2,0xbb) /* ⲻ */
    case 0x2cbc /* Ⲽ */: BSEQ(0xe2,0xb2,0xbd) /* ⲽ */
    case 0x2cbe /* Ⲿ */: BSEQ(0xe2,0xb2,0xbf) /* ⲿ */
    case 0x2cc0 /* Ⳁ */: BSEQ(0xe2,0xb3,0x81) /* ⳁ */
    case 0x2cc2 /* Ⳃ */: BSEQ(0xe2,0xb3,0x83) /* ⳃ */
    case 0x2cc4 /* Ⳅ */: BSEQ(0xe2,0xb3,0x85) /* ⳅ */
    case 0x2cc6 /* Ⳇ */: BSEQ(0xe2,0xb3,0x87) /* ⳇ */
    case 0x2cc8 /* Ⳉ */: BSEQ(0xe2,0xb3,0x89) /* ⳉ */
    case 0x2cca /* Ⳋ */: BSEQ(0xe2,0xb3,0x8b) /* ⳋ */
    case 0x2ccc /* Ⳍ */: BSEQ(0xe2,0xb3,0x8d) /* ⳍ */
    case 0x2cce /* Ⳏ */: BSEQ(0xe2,0xb3,0x8f) /* ⳏ */
    case 0x2cd0 /* Ⳑ */: BSEQ(0xe2,0xb3,0x91) /* ⳑ */
    case 0x2cd2 /* Ⳓ */: BSEQ(0xe2,0xb3,0x93) /* ⳓ */
    case 0x2cd4 /* Ⳕ */: BSEQ(0xe2,0xb3,0x95) /* ⳕ */
    case 0x2cd6 /* Ⳗ */: BSEQ(0xe2,0xb3,0x97) /* ⳗ */
    case 0x2cd8 /* Ⳙ */: BSEQ(0xe2,0xb3,0x99) /* ⳙ */
    case 0x2cda /* Ⳛ */: BSEQ(0xe2,0xb3,0x9b) /* ⳛ */
    case 0x2cdc /* Ⳝ */: BSEQ(0xe2,0xb3,0x9d) /* ⳝ */
    case 0x2cde /* Ⳟ */: BSEQ(0xe2,0xb3,0x9f) /* ⳟ */
    case 0x2ce0 /* Ⳡ */: BSEQ(0xe2,0xb3,0xa1) /* ⳡ */
    case 0x2ce2 /* Ⳣ */: BSEQ(0xe2,0xb3,0xa3) /* ⳣ */
    case 0x2ceb /* Ⳬ */: BSEQ(0xe2,0xb3,0xac) /* ⳬ */
    case 0x2ced /* Ⳮ */: BSEQ(0xe2,0xb3,0xae) /* ⳮ */
    case 0x2cf2 /* Ⳳ */: BSEQ(0xe2,0xb3,0xb3) /* ⳳ */
    case 0xa640 /* Ꙁ */: BSEQ(0xea,0x99,0x81) /* ꙁ */
    case 0xa642 /* Ꙃ */: BSEQ(0xea,0x99,0x83) /* ꙃ */
    case 0xa644 /* Ꙅ */: BSEQ(0xea,0x99,0x85) /* ꙅ */
    case 0xa646 /* Ꙇ */: BSEQ(0xea,0x99,0x87) /* ꙇ */
    case 0xa648 /* Ꙉ */: BSEQ(0xea,0x99,0x89) /* ꙉ */
    case 0xa64a /* Ꙋ */: BSEQ(0xea,0x99,0x8b) /* ꙋ */
    case 0xa64c /* Ꙍ */: BSEQ(0xea,0x99,0x8d) /* ꙍ */
    case 0xa64e /* Ꙏ */: BSEQ(0xea,0x99,0x8f) /* ꙏ */
    case 0xa650 /* Ꙑ */: BSEQ(0xea,0x99,0x91) /* ꙑ */
    case 0xa652 /* Ꙓ */: BSEQ(0xea,0x99,0x93) /* ꙓ */
    case 0xa654 /* Ꙕ */: BSEQ(0xea,0x99,0x95) /* ꙕ */
    case 0xa656 /* Ꙗ */: BSEQ(0xea,0x99,0x97) /* ꙗ */
    case 0xa658 /* Ꙙ */: BSEQ(0xea,0x99,0x99) /* ꙙ */
    case 0xa65a /* Ꙛ */: BSEQ(0xea,0x99,0x9b) /* ꙛ */
    case 0xa65c /* Ꙝ */: BSEQ(0xea,0x99,0x9d) /* ꙝ */
    case 0xa65e /* Ꙟ */: BSEQ(0xea,0x99,0x9f) /* ꙟ */
    case 0xa660 /* Ꙡ */: BSEQ(0xea,0x99,0xa1) /* ꙡ */
    case 0xa662 /* Ꙣ */: BSEQ(0xea,0x99,0xa3) /* ꙣ */
    case 0xa664 /* Ꙥ */: BSEQ(0xea,0x99,0xa5) /* ꙥ */
    case 0xa666 /* Ꙧ */: BSEQ(0xea,0x99,0xa7) /* ꙧ */
    case 0xa668 /* Ꙩ */: BSEQ(0xea,0x99,0xa9) /* ꙩ */
    case 0xa66a /* Ꙫ */: BSEQ(0xea,0x99,0xab) /* ꙫ */
    case 0xa66c /* Ꙭ */: BSEQ(0xea,0x99,0xad) /* ꙭ */
    case 0xa680 /* Ꚁ */: BSEQ(0xea,0x9a,0x81) /* ꚁ */
    case 0xa682 /* Ꚃ */: BSEQ(0xea,0x9a,0x83) /* ꚃ */
    case 0xa684 /* Ꚅ */: BSEQ(0xea,0x9a,0x85) /* ꚅ */
    case 0xa686 /* Ꚇ */: BSEQ(0xea,0x9a,0x87) /* ꚇ */
    case 0xa688 /* Ꚉ */: BSEQ(0xea,0x9a,0x89) /* ꚉ */
    case 0xa68a /* Ꚋ */: BSEQ(0xea,0x9a,0x8b) /* ꚋ */
    case 0xa68c /* Ꚍ */: BSEQ(0xea,0x9a,0x8d) /* ꚍ */
    case 0xa68e /* Ꚏ */: BSEQ(0xea,0x9a,0x8f) /* ꚏ */
    case 0xa690 /* Ꚑ */: BSEQ(0xea,0x9a,0x91) /* ꚑ */
    case 0xa692 /* Ꚓ */: BSEQ(0xea,0x9a,0x93) /* ꚓ */
    case 0xa694 /* Ꚕ */: BSEQ(0xea,0x9a,0x95) /* ꚕ */
    case 0xa696 /* Ꚗ */: BSEQ(0xea,0x9a,0x97) /* ꚗ */
    case 0xa698 /* Ꚙ */: BSEQ(0xea,0x9a,0x99) /* ꚙ */
    case 0xa69a /* Ꚛ */: BSEQ(0xea,0x9a,0x9b) /* ꚛ */
    case 0xa722 /* Ꜣ */: BSEQ(0xea,0x9c,0xa3) /* ꜣ */
    case 0xa724 /* Ꜥ */: BSEQ(0xea,0x9c,0xa5) /* ꜥ */
    case 0xa726 /* Ꜧ */: BSEQ(0xea,0x9c,0xa7) /* ꜧ */
    case 0xa728 /* Ꜩ */: BSEQ(0xea,0x9c,0xa9) /* ꜩ */
    case 0xa72a /* Ꜫ */: BSEQ(0xea,0x9c,0xab) /* ꜫ */
    case 0xa72c /* Ꜭ */: BSEQ(0xea,0x9c,0xad) /* ꜭ */
    case 0xa72e /* Ꜯ */: BSEQ(0xea,0x9c,0xaf) /* ꜯ */
    case 0xa732 /* Ꜳ */: BSEQ(0xea,0x9c,0xb3) /* ꜳ */
    case 0xa734 /* Ꜵ */: BSEQ(0xea,0x9c,0xb5) /* ꜵ */
    case 0xa736 /* Ꜷ */: BSEQ(0xea,0x9c,0xb7) /* ꜷ */
    case 0xa738 /* Ꜹ */: BSEQ(0xea,0x9c,0xb9) /* ꜹ */
    case 0xa73a /* Ꜻ */: BSEQ(0xea,0x9c,0xbb) /* ꜻ */
    case 0xa73c /* Ꜽ */: BSEQ(0xea,0x9c,0xbd) /* ꜽ */
    case 0xa73e /* Ꜿ */: BSEQ(0xea,0x9c,0xbf) /* ꜿ */
    case 0xa740 /* Ꝁ */: BSEQ(0xea,0x9d,0x81) /* ꝁ */
    case 0xa742 /* Ꝃ */: BSEQ(0xea,0x9d,0x83) /* ꝃ */
    case 0xa744 /* Ꝅ */: BSEQ(0xea,0x9d,0x85) /* ꝅ */
    case 0xa746 /* Ꝇ */: BSEQ(0xea,0x9d,0x87) /* ꝇ */
    case 0xa748 /* Ꝉ */: BSEQ(0xea,0x9d,0x89) /* ꝉ */
    case 0xa74a /* Ꝋ */: BSEQ(0xea,0x9d,0x8b) /* ꝋ */
    case 0xa74c /* Ꝍ */: BSEQ(0xea,0x9d,0x8d) /* ꝍ */
    case 0xa74e /* Ꝏ */: BSEQ(0xea,0x9d,0x8f) /* ꝏ */
    case 0xa750 /* Ꝑ */: BSEQ(0xea,0x9d,0x91) /* ꝑ */
    case 0xa752 /* Ꝓ */: BSEQ(0xea,0x9d,0x93) /* ꝓ */
    case 0xa754 /* Ꝕ */: BSEQ(0xea,0x9d,0x95) /* ꝕ */
    case 0xa756 /* Ꝗ */: BSEQ(0xea,0x9d,0x97) /* ꝗ */
    case 0xa758 /* Ꝙ */: BSEQ(0xea,0x9d,0x99) /* ꝙ */
    case 0xa75a /* Ꝛ */: BSEQ(0xea,0x9d,0x9b) /* ꝛ */
    case 0xa75c /* Ꝝ */: BSEQ(0xea,0x9d,0x9d) /* ꝝ */
    case 0xa75e /* Ꝟ */: BSEQ(0xea,0x9d,0x9f) /* ꝟ */
    case 0xa760 /* Ꝡ */: BSEQ(0xea,0x9d,0xa1) /* ꝡ */
    case 0xa762 /* Ꝣ */: BSEQ(0xea,0x9d,0xa3) /* ꝣ */
    case 0xa764 /* Ꝥ */: BSEQ(0xea,0x9d,0xa5) /* ꝥ */
    case 0xa766 /* Ꝧ */: BSEQ(0xea,0x9d,0xa7) /* ꝧ */
    case 0xa768 /* Ꝩ */: BSEQ(0xea,0x9d,0xa9) /* ꝩ */
    case 0xa76a /* Ꝫ */: BSEQ(0xea,0x9d,0xab) /* ꝫ */
    case 0xa76c /* Ꝭ */: BSEQ(0xea,0x9d,0xad) /* ꝭ */
    case 0xa76e /* Ꝯ */: BSEQ(0xea,0x9d,0xaf) /* ꝯ */
    case 0xa779 /* Ꝺ */: BSEQ(0xea,0x9d,0xba) /* ꝺ */
    case 0xa77b /* Ꝼ */: BSEQ(0xea,0x9d,0xbc) /* ꝼ */
    case 0xa77d /* Ᵹ */: BSEQ(0xe1,0xb5,0xb9) /* ᵹ */
    case 0xa77e /* Ꝿ */: BSEQ(0xea,0x9d,0xbf) /* ꝿ */
    case 0xa780 /* Ꞁ */: BSEQ(0xea,0x9e,0x81) /* ꞁ */
    case 0xa782 /* Ꞃ */: BSEQ(0xea,0x9e,0x83) /* ꞃ */
    case 0xa784 /* Ꞅ */: BSEQ(0xea,0x9e,0x85) /* ꞅ */
    case 0xa786 /* Ꞇ */: BSEQ(0xea,0x9e,0x87) /* ꞇ */
    case 0xa78b /* Ꞌ */: BSEQ(0xea,0x9e,0x8c) /* ꞌ */
    case 0xa78d /* Ɥ */: BSEQ(0xc9,0xa5) /* ɥ */
    case 0xa790 /* Ꞑ */: BSEQ(0xea,0x9e,0x91) /* ꞑ */
    case 0xa792 /* Ꞓ */: BSEQ(0xea,0x9e,0x93) /* ꞓ */
    case 0xa796 /* Ꞗ */: BSEQ(0xea,0x9e,0x97) /* ꞗ */
    case 0xa798 /* Ꞙ */: BSEQ(0xea,0x9e,0x99) /* ꞙ */
    case 0xa79a /* Ꞛ */: BSEQ(0xea,0x9e,0x9b) /* ꞛ */
    case 0xa79c /* Ꞝ */: BSEQ(0xea,0x9e,0x9d) /* ꞝ */
    case 0xa79e /* Ꞟ */: BSEQ(0xea,0x9e,0x9f) /* ꞟ */
    case 0xa7a0 /* Ꞡ */: BSEQ(0xea,0x9e,0xa1) /* ꞡ */
    case 0xa7a2 /* Ꞣ */: BSEQ(0xea,0x9e,0xa3) /* ꞣ */
    case 0xa7a4 /* Ꞥ */: BSEQ(0xea,0x9e,0xa5) /* ꞥ */
    case 0xa7a6 /* Ꞧ */: BSEQ(0xea,0x9e,0xa7) /* ꞧ */
    case 0xa7a8 /* Ꞩ */: BSEQ(0xea,0x9e,0xa9) /* ꞩ */
    case 0xa7aa /* Ɦ */: BSEQ(0xc9,0xa6) /* ɦ */
    case 0xa7ab /* Ɜ */: BSEQ(0xc9,0x9c) /* ɜ */
    case 0xa7ac /* Ɡ */: BSEQ(0xc9,0xa1) /* ɡ */
    case 0xa7ad /* Ɬ */: BSEQ(0xc9,0xac) /* ɬ */
    case 0xa7ae /* Ɪ */: BSEQ(0xc9,0xaa) /* ɪ */
    case 0xa7b0 /* Ʞ */: BSEQ(0xca,0x9e) /* ʞ */
    case 0xa7b1 /* Ʇ */: BSEQ(0xca,0x87) /* ʇ */
    case 0xa7b2 /* Ʝ */: BSEQ(0xca,0x9d) /* ʝ */
    case 0xa7b3 /* Ꭓ */: BSEQ(0xea,0xad,0x93) /* ꭓ */
    case 0xa7b4 /* Ꞵ */: BSEQ(0xea,0x9e,0xb5) /* ꞵ */
    case 0xa7b6 /* Ꞷ */: BSEQ(0xea,0x9e,0xb7) /* ꞷ */
    case 0xa7b8 /* Ꞹ */: BSEQ(0xea,0x9e,0xb9) /* ꞹ */
    case 0xa7ba /* Ꞻ */: BSEQ(0xea,0x9e,0xbb) /* ꞻ */
    case 0xa7bc /* Ꞽ */: BSEQ(0xea,0x9e,0xbd) /* ꞽ */
    case 0xa7be /* Ꞿ */: BSEQ(0xea,0x9e,0xbf) /* ꞿ */
    case 0xa7c2 /* Ꟃ */: BSEQ(0xea,0x9f,0x83) /* ꟃ */
    case 0xa7c4 /* Ꞔ */: BSEQ(0xea,0x9e,0x94) /* ꞔ */
    case 0xa7c5 /* Ʂ */: BSEQ(0xca,0x82) /* ʂ */
    case 0xa7c6 /* Ᶎ */: BSEQ(0xe1,0xb6,0x8e) /* ᶎ */
    case 0xa7c7 /* Ꟈ */: BSEQ(0xea,0x9f,0x88) /* ꟈ */
    case 0xa7c9 /* Ꟊ */: BSEQ(0xea,0x9f,0x8a) /* ꟊ */
    case 0xa7f5 /* Ꟶ */: BSEQ(0xea,0x9f,0xb6) /* ꟶ */
    case 0xff21 /* Ａ */: BSEQ(0xef,0xbd,0x81) /* ａ */
    case 0xff22 /* Ｂ */: BSEQ(0xef,0xbd,0x82) /* ｂ */
    case 0xff23 /* Ｃ */: BSEQ(0xef,0xbd,0x83) /* ｃ */
    case 0xff24 /* Ｄ */: BSEQ(0xef,0xbd,0x84) /* ｄ */
    case 0xff25 /* Ｅ */: BSEQ(0xef,0xbd,0x85) /* ｅ */
    case 0xff26 /* Ｆ */: BSEQ(0xef,0xbd,0x86) /* ｆ */
    case 0xff27 /* Ｇ */: BSEQ(0xef,0xbd,0x87) /* ｇ */
    case 0xff28 /* Ｈ */: BSEQ(0xef,0xbd,0x88) /* ｈ */
    case 0xff29 /* Ｉ */: BSEQ(0xef,0xbd,0x89) /* ｉ */
    case 0xff2a /* Ｊ */: BSEQ(0xef,0xbd,0x8a) /* ｊ */
    case 0xff2b /* Ｋ */: BSEQ(0xef,0xbd,0x8b) /* ｋ */
    case 0xff2c /* Ｌ */: BSEQ(0xef,0xbd,0x8c) /* ｌ */
    case 0xff2d /* Ｍ */: BSEQ(0xef,0xbd,0x8d) /* ｍ */
    case 0xff2e /* Ｎ */: BSEQ(0xef,0xbd,0x8e) /* ｎ */
    case 0xff2f /* Ｏ */: BSEQ(0xef,0xbd,0x8f) /* ｏ */
    case 0xff30 /* Ｐ */: BSEQ(0xef,0xbd,0x90) /* ｐ */
    case 0xff31 /* Ｑ */: BSEQ(0xef,0xbd,0x91) /* ｑ */
    case 0xff32 /* Ｒ */: BSEQ(0xef,0xbd,0x92) /* ｒ */
    case 0xff33 /* Ｓ */: BSEQ(0xef,0xbd,0x93) /* ｓ */
    case 0xff34 /* Ｔ */: BSEQ(0xef,0xbd,0x94) /* ｔ */
    case 0xff35 /* Ｕ */: BSEQ(0xef,0xbd,0x95) /* ｕ */
    case 0xff36 /* Ｖ */: BSEQ(0xef,0xbd,0x96) /* ｖ */
    case 0xff37 /* Ｗ */: BSEQ(0xef,0xbd,0x97) /* ｗ */
    case 0xff38 /* Ｘ */: BSEQ(0xef,0xbd,0x98) /* ｘ */
    case 0xff39 /* Ｙ */: BSEQ(0xef,0xbd,0x99) /* ｙ */
    case 0xff3a /* Ｚ */: BSEQ(0xef,0xbd,0x9a) /* ｚ */
    case 0x10400 /* 𐐀 */: BSEQ(0xf0,0x90,0x90,0xa8) /* 𐐨 */
    case 0x10401 /* 𐐁 */: BSEQ(0xf0,0x90,0x90,0xa9) /* 𐐩 */
    case 0x10402 /* 𐐂 */: BSEQ(0xf0,0x90,0x90,0xaa) /* 𐐪 */
    case 0x10403 /* 𐐃 */: BSEQ(0xf0,0x90,0x90,0xab) /* 𐐫 */
    case 0x10404 /* 𐐄 */: BSEQ(0xf0,0x90,0x90,0xac) /* 𐐬 */
    case 0x10405 /* 𐐅 */: BSEQ(0xf0,0x90,0x90,0xad) /* 𐐭 */
    case 0x10406 /* 𐐆 */: BSEQ(0xf0,0x90,0x90,0xae) /* 𐐮 */
    case 0x10407 /* 𐐇 */: BSEQ(0xf0,0x90,0x90,0xaf) /* 𐐯 */
    case 0x10408 /* 𐐈 */: BSEQ(0xf0,0x90,0x90,0xb0) /* 𐐰 */
    case 0x10409 /* 𐐉 */: BSEQ(0xf0,0x90,0x90,0xb1) /* 𐐱 */
    case 0x1040a /* 𐐊 */: BSEQ(0xf0,0x90,0x90,0xb2) /* 𐐲 */
    case 0x1040b /* 𐐋 */: BSEQ(0xf0,0x90,0x90,0xb3) /* 𐐳 */
    case 0x1040c /* 𐐌 */: BSEQ(0xf0,0x90,0x90,0xb4) /* 𐐴 */
    case 0x1040d /* 𐐍 */: BSEQ(0xf0,0x90,0x90,0xb5) /* 𐐵 */
    case 0x1040e /* 𐐎 */: BSEQ(0xf0,0x90,0x90,0xb6) /* 𐐶 */
    case 0x1040f /* 𐐏 */: BSEQ(0xf0,0x90,0x90,0xb7) /* 𐐷 */
    case 0x10410 /* 𐐐 */: BSEQ(0xf0,0x90,0x90,0xb8) /* 𐐸 */
    case 0x10411 /* 𐐑 */: BSEQ(0xf0,0x90,0x90,0xb9) /* 𐐹 */
    case 0x10412 /* 𐐒 */: BSEQ(0xf0,0x90,0x90,0xba) /* 𐐺 */
    case 0x10413 /* 𐐓 */: BSEQ(0xf0,0x90,0x90,0xbb) /* 𐐻 */
    case 0x10414 /* 𐐔 */: BSEQ(0xf0,0x90,0x90,0xbc) /* 𐐼 */
    case 0x10415 /* 𐐕 */: BSEQ(0xf0,0x90,0x90,0xbd) /* 𐐽 */
    case 0x10416 /* 𐐖 */: BSEQ(0xf0,0x90,0x90,0xbe) /* 𐐾 */
    case 0x10417 /* 𐐗 */: BSEQ(0xf0,0x90,0x90,0xbf) /* 𐐿 */
    case 0x10418 /* 𐐘 */: BSEQ(0xf0,0x90,0x91,0x80) /* 𐑀 */
    case 0x10419 /* 𐐙 */: BSEQ(0xf0,0x90,0x91,0x81) /* 𐑁 */
    case 0x1041a /* 𐐚 */: BSEQ(0xf0,0x90,0x91,0x82) /* 𐑂 */
    case 0x1041b /* 𐐛 */: BSEQ(0xf0,0x90,0x91,0x83) /* 𐑃 */
    case 0x1041c /* 𐐜 */: BSEQ(0xf0,0x90,0x91,0x84) /* 𐑄 */
    case 0x1041d /* 𐐝 */: BSEQ(0xf0,0x90,0x91,0x85) /* 𐑅 */
    case 0x1041e /* 𐐞 */: BSEQ(0xf0,0x90,0x91,0x86) /* 𐑆 */
    case 0x1041f /* 𐐟 */: BSEQ(0xf0,0x90,0x91,0x87) /* 𐑇 */
    case 0x10420 /* 𐐠 */: BSEQ(0xf0,0x90,0x91,0x88) /* 𐑈 */
    case 0x10421 /* 𐐡 */: BSEQ(0xf0,0x90,0x91,0x89) /* 𐑉 */
    case 0x10422 /* 𐐢 */: BSEQ(0xf0,0x90,0x91,0x8a) /* 𐑊 */
    case 0x10423 /* 𐐣 */: BSEQ(0xf0,0x90,0x91,0x8b) /* 𐑋 */
    case 0x10424 /* 𐐤 */: BSEQ(0xf0,0x90,0x91,0x8c) /* 𐑌 */
    case 0x10425 /* 𐐥 */: BSEQ(0xf0,0x90,0x91,0x8d) /* 𐑍 */
    case 0x10426 /* 𐐦 */: BSEQ(0xf0,0x90,0x91,0x8e) /* 𐑎 */
    case 0x10427 /* 𐐧 */: BSEQ(0xf0,0x90,0x91,0x8f) /* 𐑏 */
    case 0x104b0 /* 𐒰 */: BSEQ(0xf0,0x90,0x93,0x98) /* 𐓘 */
    case 0x104b1 /* 𐒱 */: BSEQ(0xf0,0x90,0x93,0x99) /* 𐓙 */
    case 0x104b2 /* 𐒲 */: BSEQ(0xf0,0x90,0x93,0x9a) /* 𐓚 */
    case 0x104b3 /* 𐒳 */: BSEQ(0xf0,0x90,0x93,0x9b) /* 𐓛 */
    case 0x104b4 /* 𐒴 */: BSEQ(0xf0,0x90,0x93,0x9c) /* 𐓜 */
    case 0x104b5 /* 𐒵 */: BSEQ(0xf0,0x90,0x93,0x9d) /* 𐓝 */
    case 0x104b6 /* 𐒶 */: BSEQ(0xf0,0x90,0x93,0x9e) /* 𐓞 */
    case 0x104b7 /* 𐒷 */: BSEQ(0xf0,0x90,0x93,0x9f) /* 𐓟 */
    case 0x104b8 /* 𐒸 */: BSEQ(0xf0,0x90,0x93,0xa0) /* 𐓠 */
    case 0x104b9 /* 𐒹 */: BSEQ(0xf0,0x90,0x93,0xa1) /* 𐓡 */
    case 0x104ba /* 𐒺 */: BSEQ(0xf0,0x90,0x93,0xa2) /* 𐓢 */
    case 0x104bb /* 𐒻 */: BSEQ(0xf0,0x90,0x93,0xa3) /* 𐓣 */
    case 0x104bc /* 𐒼 */: BSEQ(0xf0,0x90,0x93,0xa4) /* 𐓤 */
    case 0x104bd /* 𐒽 */: BSEQ(0xf0,0x90,0x93,0xa5) /* 𐓥 */
    case 0x104be /* 𐒾 */: BSEQ(0xf0,0x90,0x93,0xa6) /* 𐓦 */
    case 0x104bf /* 𐒿 */: BSEQ(0xf0,0x90,0x93,0xa7) /* 𐓧 */
    case 0x104c0 /* 𐓀 */: BSEQ(0xf0,0x90,0x93,0xa8) /* 𐓨 */
    case 0x104c1 /* 𐓁 */: BSEQ(0xf0,0x90,0x93,0xa9) /* 𐓩 */
    case 0x104c2 /* 𐓂 */: BSEQ(0xf0,0x90,0x93,0xaa) /* 𐓪 */
    case 0x104c3 /* 𐓃 */: BSEQ(0xf0,0x90,0x93,0xab) /* 𐓫 */
    case 0x104c4 /* 𐓄 */: BSEQ(0xf0,0x90,0x93,0xac) /* 𐓬 */
    case 0x104c5 /* 𐓅 */: BSEQ(0xf0,0x90,0x93,0xad) /* 𐓭 */
    case 0x104c6 /* 𐓆 */: BSEQ(0xf0,0x90,0x93,0xae) /* 𐓮 */
    case 0x104c7 /* 𐓇 */: BSEQ(0xf0,0x90,0x93,0xaf) /* 𐓯 */
    case 0x104c8 /* 𐓈 */: BSEQ(0xf0,0x90,0x93,0xb0) /* 𐓰 */
    case 0x104c9 /* 𐓉 */: BSEQ(0xf0,0x90,0x93,0xb1) /* 𐓱 */
    case 0x104ca /* 𐓊 */: BSEQ(0xf0,0x90,0x93,0xb2) /* 𐓲 */
    case 0x104cb /* 𐓋 */: BSEQ(0xf0,0x90,0x93,0xb3) /* 𐓳 */
    case 0x104cc /* 𐓌 */: BSEQ(0xf0,0x90,0x93,0xb4) /* 𐓴 */
    case 0x104cd /* 𐓍 */: BSEQ(0xf0,0x90,0x93,0xb5) /* 𐓵 */
    case 0x104ce /* 𐓎 */: BSEQ(0xf0,0x90,0x93,0xb6) /* 𐓶 */
    case 0x104cf /* 𐓏 */: BSEQ(0xf0,0x90,0x93,0xb7) /* 𐓷 */
    case 0x104d0 /* 𐓐 */: BSEQ(0xf0,0x90,0x93,0xb8) /* 𐓸 */
    case 0x104d1 /* 𐓑 */: BSEQ(0xf0,0x90,0x93,0xb9) /* 𐓹 */
    case 0x104d2 /* 𐓒 */: BSEQ(0xf0,0x90,0x93,0xba) /* 𐓺 */
    case 0x104d3 /* 𐓓 */: BSEQ(0xf0,0x90,0x93,0xbb) /* 𐓻 */
    case 0x10c80 /* 𐲀 */: BSEQ(0xf0,0x90,0xb3,0x80) /* 𐳀 */
    case 0x10c81 /* 𐲁 */: BSEQ(0xf0,0x90,0xb3,0x81) /* 𐳁 */
    case 0x10c82 /* 𐲂 */: BSEQ(0xf0,0x90,0xb3,0x82) /* 𐳂 */
    case 0x10c83 /* 𐲃 */: BSEQ(0xf0,0x90,0xb3,0x83) /* 𐳃 */
    case 0x10c84 /* 𐲄 */: BSEQ(0xf0,0x90,0xb3,0x84) /* 𐳄 */
    case 0x10c85 /* 𐲅 */: BSEQ(0xf0,0x90,0xb3,0x85) /* 𐳅 */
    case 0x10c86 /* 𐲆 */: BSEQ(0xf0,0x90,0xb3,0x86) /* 𐳆 */
    case 0x10c87 /* 𐲇 */: BSEQ(0xf0,0x90,0xb3,0x87) /* 𐳇 */
    case 0x10c88 /* 𐲈 */: BSEQ(0xf0,0x90,0xb3,0x88) /* 𐳈 */
    case 0x10c89 /* 𐲉 */: BSEQ(0xf0,0x90,0xb3,0x89) /* 𐳉 */
    case 0x10c8a /* 𐲊 */: BSEQ(0xf0,0x90,0xb3,0x8a) /* 𐳊 */
    case 0x10c8b /* 𐲋 */: BSEQ(0xf0,0x90,0xb3,0x8b) /* 𐳋 */
    case 0x10c8c /* 𐲌 */: BSEQ(0xf0,0x90,0xb3,0x8c) /* 𐳌 */
    case 0x10c8d /* 𐲍 */: BSEQ(0xf0,0x90,0xb3,0x8d) /* 𐳍 */
    case 0x10c8e /* 𐲎 */: BSEQ(0xf0,0x90,0xb3,0x8e) /* 𐳎 */
    case 0x10c8f /* 𐲏 */: BSEQ(0xf0,0x90,0xb3,0x8f) /* 𐳏 */
    case 0x10c90 /* 𐲐 */: BSEQ(0xf0,0x90,0xb3,0x90) /* 𐳐 */
    case 0x10c91 /* 𐲑 */: BSEQ(0xf0,0x90,0xb3,0x91) /* 𐳑 */
    case 0x10c92 /* 𐲒 */: BSEQ(0xf0,0x90,0xb3,0x92) /* 𐳒 */
    case 0x10c93 /* 𐲓 */: BSEQ(0xf0,0x90,0xb3,0x93) /* 𐳓 */
    case 0x10c94 /* 𐲔 */: BSEQ(0xf0,0x90,0xb3,0x94) /* 𐳔 */
    case 0x10c95 /* 𐲕 */: BSEQ(0xf0,0x90,0xb3,0x95) /* 𐳕 */
    case 0x10c96 /* 𐲖 */: BSEQ(0xf0,0x90,0xb3,0x96) /* 𐳖 */
    case 0x10c97 /* 𐲗 */: BSEQ(0xf0,0x90,0xb3,0x97) /* 𐳗 */
    case 0x10c98 /* 𐲘 */: BSEQ(0xf0,0x90,0xb3,0x98) /* 𐳘 */
    case 0x10c99 /* 𐲙 */: BSEQ(0xf0,0x90,0xb3,0x99) /* 𐳙 */
    case 0x10c9a /* 𐲚 */: BSEQ(0xf0,0x90,0xb3,0x9a) /* 𐳚 */
    case 0x10c9b /* 𐲛 */: BSEQ(0xf0,0x90,0xb3,0x9b) /* 𐳛 */
    case 0x10c9c /* 𐲜 */: BSEQ(0xf0,0x90,0xb3,0x9c) /* 𐳜 */
    case 0x10c9d /* 𐲝 */: BSEQ(0xf0,0x90,0xb3,0x9d) /* 𐳝 */
    case 0x10c9e /* 𐲞 */: BSEQ(0xf0,0x90,0xb3,0x9e) /* 𐳞 */
    case 0x10c9f /* 𐲟 */: BSEQ(0xf0,0x90,0xb3,0x9f) /* 𐳟 */
    case 0x10ca0 /* 𐲠 */: BSEQ(0xf0,0x90,0xb3,0xa0) /* 𐳠 */
    case 0x10ca1 /* 𐲡 */: BSEQ(0xf0,0x90,0xb3,0xa1) /* 𐳡 */
    case 0x10ca2 /* 𐲢 */: BSEQ(0xf0,0x90,0xb3,0xa2) /* 𐳢 */
    case 0x10ca3 /* 𐲣 */: BSEQ(0xf0,0x90,0xb3,0xa3) /* 𐳣 */
    case 0x10ca4 /* 𐲤 */: BSEQ(0xf0,0x90,0xb3,0xa4) /* 𐳤 */
    case 0x10ca5 /* 𐲥 */: BSEQ(0xf0,0x90,0xb3,0xa5) /* 𐳥 */
    case 0x10ca6 /* 𐲦 */: BSEQ(0xf0,0x90,0xb3,0xa6) /* 𐳦 */
    case 0x10ca7 /* 𐲧 */: BSEQ(0xf0,0x90,0xb3,0xa7) /* 𐳧 */
    case 0x10ca8 /* 𐲨 */: BSEQ(0xf0,0x90,0xb3,0xa8) /* 𐳨 */
    case 0x10ca9 /* 𐲩 */: BSEQ(0xf0,0x90,0xb3,0xa9) /* 𐳩 */
    case 0x10caa /* 𐲪 */: BSEQ(0xf0,0x90,0xb3,0xaa) /* 𐳪 */
    case 0x10cab /* 𐲫 */: BSEQ(0xf0,0x90,0xb3,0xab) /* 𐳫 */
    case 0x10cac /* 𐲬 */: BSEQ(0xf0,0x90,0xb3,0xac) /* 𐳬 */
    case 0x10cad /* 𐲭 */: BSEQ(0xf0,0x90,0xb3,0xad) /* 𐳭 */
    case 0x10cae /* 𐲮 */: BSEQ(0xf0,0x90,0xb3,0xae) /* 𐳮 */
    case 0x10caf /* 𐲯 */: BSEQ(0xf0,0x90,0xb3,0xaf) /* 𐳯 */
    case 0x10cb0 /* 𐲰 */: BSEQ(0xf0,0x90,0xb3,0xb0) /* 𐳰 */
    case 0x10cb1 /* 𐲱 */: BSEQ(0xf0,0x90,0xb3,0xb1) /* 𐳱 */
    case 0x10cb2 /* 𐲲 */: BSEQ(0xf0,0x90,0xb3,0xb2) /* 𐳲 */
    case 0x118a0 /* 𑢠 */: BSEQ(0xf0,0x91,0xa3,0x80) /* 𑣀 */
    case 0x118a1 /* 𑢡 */: BSEQ(0xf0,0x91,0xa3,0x81) /* 𑣁 */
    case 0x118a2 /* 𑢢 */: BSEQ(0xf0,0x91,0xa3,0x82) /* 𑣂 */
    case 0x118a3 /* 𑢣 */: BSEQ(0xf0,0x91,0xa3,0x83) /* 𑣃 */
    case 0x118a4 /* 𑢤 */: BSEQ(0xf0,0x91,0xa3,0x84) /* 𑣄 */
    case 0x118a5 /* 𑢥 */: BSEQ(0xf0,0x91,0xa3,0x85) /* 𑣅 */
    case 0x118a6 /* 𑢦 */: BSEQ(0xf0,0x91,0xa3,0x86) /* 𑣆 */
    case 0x118a7 /* 𑢧 */: BSEQ(0xf0,0x91,0xa3,0x87) /* 𑣇 */
    case 0x118a8 /* 𑢨 */: BSEQ(0xf0,0x91,0xa3,0x88) /* 𑣈 */
    case 0x118a9 /* 𑢩 */: BSEQ(0xf0,0x91,0xa3,0x89) /* 𑣉 */
    case 0x118aa /* 𑢪 */: BSEQ(0xf0,0x91,0xa3,0x8a) /* 𑣊 */
    case 0x118ab /* 𑢫 */: BSEQ(0xf0,0x91,0xa3,0x8b) /* 𑣋 */
    case 0x118ac /* 𑢬 */: BSEQ(0xf0,0x91,0xa3,0x8c) /* 𑣌 */
    case 0x118ad /* 𑢭 */: BSEQ(0xf0,0x91,0xa3,0x8d) /* 𑣍 */
    case 0x118ae /* 𑢮 */: BSEQ(0xf0,0x91,0xa3,0x8e) /* 𑣎 */
    case 0x118af /* 𑢯 */: BSEQ(0xf0,0x91,0xa3,0x8f) /* 𑣏 */
    case 0x118b0 /* 𑢰 */: BSEQ(0xf0,0x91,0xa3,0x90) /* 𑣐 */
    case 0x118b1 /* 𑢱 */: BSEQ(0xf0,0x91,0xa3,0x91) /* 𑣑 */
    case 0x118b2 /* 𑢲 */: BSEQ(0xf0,0x91,0xa3,0x92) /* 𑣒 */
    case 0x118b3 /* 𑢳 */: BSEQ(0xf0,0x91,0xa3,0x93) /* 𑣓 */
    case 0x118b4 /* 𑢴 */: BSEQ(0xf0,0x91,0xa3,0x94) /* 𑣔 */
    case 0x118b5 /* 𑢵 */: BSEQ(0xf0,0x91,0xa3,0x95) /* 𑣕 */
    case 0x118b6 /* 𑢶 */: BSEQ(0xf0,0x91,0xa3,0x96) /* 𑣖 */
    case 0x118b7 /* 𑢷 */: BSEQ(0xf0,0x91,0xa3,0x97) /* 𑣗 */
    case 0x118b8 /* 𑢸 */: BSEQ(0xf0,0x91,0xa3,0x98) /* 𑣘 */
    case 0x118b9 /* 𑢹 */: BSEQ(0xf0,0x91,0xa3,0x99) /* 𑣙 */
    case 0x118ba /* 𑢺 */: BSEQ(0xf0,0x91,0xa3,0x9a) /* 𑣚 */
    case 0x118bb /* 𑢻 */: BSEQ(0xf0,0x91,0xa3,0x9b) /* 𑣛 */
    case 0x118bc /* 𑢼 */: BSEQ(0xf0,0x91,0xa3,0x9c) /* 𑣜 */
    case 0x118bd /* 𑢽 */: BSEQ(0xf0,0x91,0xa3,0x9d) /* 𑣝 */
    case 0x118be /* 𑢾 */: BSEQ(0xf0,0x91,0xa3,0x9e) /* 𑣞 */
    case 0x118bf /* 𑢿 */: BSEQ(0xf0,0x91,0xa3,0x9f) /* 𑣟 */
    case 0x16e40 /* 𖹀 */: BSEQ(0xf0,0x96,0xb9,0xa0) /* 𖹠 */
    case 0x16e41 /* 𖹁 */: BSEQ(0xf0,0x96,0xb9,0xa1) /* 𖹡 */
    case 0x16e42 /* 𖹂 */: BSEQ(0xf0,0x96,0xb9,0xa2) /* 𖹢 */
    case 0x16e43 /* 𖹃 */: BSEQ(0xf0,0x96,0xb9,0xa3) /* 𖹣 */
    case 0x16e44 /* 𖹄 */: BSEQ(0xf0,0x96,0xb9,0xa4) /* 𖹤 */
    case 0x16e45 /* 𖹅 */: BSEQ(0xf0,0x96,0xb9,0xa5) /* 𖹥 */
    case 0x16e46 /* 𖹆 */: BSEQ(0xf0,0x96,0xb9,0xa6) /* 𖹦 */
    case 0x16e47 /* 𖹇 */: BSEQ(0xf0,0x96,0xb9,0xa7) /* 𖹧 */
    case 0x16e48 /* 𖹈 */: BSEQ(0xf0,0x96,0xb9,0xa8) /* 𖹨 */
    case 0x16e49 /* 𖹉 */: BSEQ(0xf0,0x96,0xb9,0xa9) /* 𖹩 */
    case 0x16e4a /* 𖹊 */: BSEQ(0xf0,0x96,0xb9,0xaa) /* 𖹪 */
    case 0x16e4b /* 𖹋 */: BSEQ(0xf0,0x96,0xb9,0xab) /* 𖹫 */
    case 0x16e4c /* 𖹌 */: BSEQ(0xf0,0x96,0xb9,0xac) /* 𖹬 */
    case 0x16e4d /* 𖹍 */: BSEQ(0xf0,0x96,0xb9,0xad) /* 𖹭 */
    case 0x16e4e /* 𖹎 */: BSEQ(0xf0,0x96,0xb9,0xae) /* 𖹮 */
    case 0x16e4f /* 𖹏 */: BSEQ(0xf0,0x96,0xb9,0xaf) /* 𖹯 */
    case 0x16e50 /* 𖹐 */: BSEQ(0xf0,0x96,0xb9,0xb0) /* 𖹰 */
    case 0x16e51 /* 𖹑 */: BSEQ(0xf0,0x96,0xb9,0xb1) /* 𖹱 */
    case 0x16e52 /* 𖹒 */: BSEQ(0xf0,0x96,0xb9,0xb2) /* 𖹲 */
    case 0x16e53 /* 𖹓 */: BSEQ(0xf0,0x96,0xb9,0xb3) /* 𖹳 */
    case 0x16e54 /* 𖹔 */: BSEQ(0xf0,0x96,0xb9,0xb4) /* 𖹴 */
    case 0x16e55 /* 𖹕 */: BSEQ(0xf0,0x96,0xb9,0xb5) /* 𖹵 */
    case 0x16e56 /* 𖹖 */: BSEQ(0xf0,0x96,0xb9,0xb6) /* 𖹶 */
    case 0x16e57 /* 𖹗 */: BSEQ(0xf0,0x96,0xb9,0xb7) /* 𖹷 */
    case 0x16e58 /* 𖹘 */: BSEQ(0xf0,0x96,0xb9,0xb8) /* 𖹸 */
    case 0x16e59 /* 𖹙 */: BSEQ(0xf0,0x96,0xb9,0xb9) /* 𖹹 */
    case 0x16e5a /* 𖹚 */: BSEQ(0xf0,0x96,0xb9,0xba) /* 𖹺 */
    case 0x16e5b /* 𖹛 */: BSEQ(0xf0,0x96,0xb9,0xbb) /* 𖹻 */
    case 0x16e5c /* 𖹜 */: BSEQ(0xf0,0x96,0xb9,0xbc) /* 𖹼 */
    case 0x16e5d /* 𖹝 */: BSEQ(0xf0,0x96,0xb9,0xbd) /* 𖹽 */
    case 0x16e5e /* 𖹞 */: BSEQ(0xf0,0x96,0xb9,0xbe) /* 𖹾 */
    case 0x16e5f /* 𖹟 */: BSEQ(0xf0,0x96,0xb9,0xbf) /* 𖹿 */
    case 0x1e900 /* 𞤀 */: BSEQ(0xf0,0x9e,0xa4,0xa2) /* 𞤢 */
    case 0x1e901 /* 𞤁 */: BSEQ(0xf0,0x9e,0xa4,0xa3) /* 𞤣 */
    case 0x1e902 /* 𞤂 */: BSEQ(0xf0,0x9e,0xa4,0xa4) /* 𞤤 */
    case 0x1e903 /* 𞤃 */: BSEQ(0xf0,0x9e,0xa4,0xa5) /* 𞤥 */
    case 0x1e904 /* 𞤄 */: BSEQ(0xf0,0x9e,0xa4,0xa6) /* 𞤦 */
    case 0x1e905 /* 𞤅 */: BSEQ(0xf0,0x9e,0xa4,0xa7) /* 𞤧 */
    case 0x1e906 /* 𞤆 */: BSEQ(0xf0,0x9e,0xa4,0xa8) /* 𞤨 */
    case 0x1e907 /* 𞤇 */: BSEQ(0xf0,0x9e,0xa4,0xa9) /* 𞤩 */
    case 0x1e908 /* 𞤈 */: BSEQ(0xf0,0x9e,0xa4,0xaa) /* 𞤪 */
    case 0x1e909 /* 𞤉 */: BSEQ(0xf0,0x9e,0xa4,0xab) /* 𞤫 */
    case 0x1e90a /* 𞤊 */: BSEQ(0xf0,0x9e,0xa4,0xac) /* 𞤬 */
    case 0x1e90b /* 𞤋 */: BSEQ(0xf0,0x9e,0xa4,0xad) /* 𞤭 */
    case 0x1e90c /* 𞤌 */: BSEQ(0xf0,0x9e,0xa4,0xae) /* 𞤮 */
    case 0x1e90d /* 𞤍 */: BSEQ(0xf0,0x9e,0xa4,0xaf) /* 𞤯 */
    case 0x1e90e /* 𞤎 */: BSEQ(0xf0,0x9e,0xa4,0xb0) /* 𞤰 */
    case 0x1e90f /* 𞤏 */: BSEQ(0xf0,0x9e,0xa4,0xb1) /* 𞤱 */
    case 0x1e910 /* 𞤐 */: BSEQ(0xf0,0x9e,0xa4,0xb2) /* 𞤲 */
    case 0x1e911 /* 𞤑 */: BSEQ(0xf0,0x9e,0xa4,0xb3) /* 𞤳 */
    case 0x1e912 /* 𞤒 */: BSEQ(0xf0,0x9e,0xa4,0xb4) /* 𞤴 */
    case 0x1e913 /* 𞤓 */: BSEQ(0xf0,0x9e,0xa4,0xb5) /* 𞤵 */
    case 0x1e914 /* 𞤔 */: BSEQ(0xf0,0x9e,0xa4,0xb6) /* 𞤶 */
    case 0x1e915 /* 𞤕 */: BSEQ(0xf0,0x9e,0xa4,0xb7) /* 𞤷 */
    case 0x1e916 /* 𞤖 */: BSEQ(0xf0,0x9e,0xa4,0xb8) /* 𞤸 */
    case 0x1e917 /* 𞤗 */: BSEQ(0xf0,0x9e,0xa4,0xb9) /* 𞤹 */
    case 0x1e918 /* 𞤘 */: BSEQ(0xf0,0x9e,0xa4,0xba) /* 𞤺 */
    case 0x1e919 /* 𞤙 */: BSEQ(0xf0,0x9e,0xa4,0xbb) /* 𞤻 */
    case 0x1e91a /* 𞤚 */: BSEQ(0xf0,0x9e,0xa4,0xbc) /* 𞤼 */
    case 0x1e91b /* 𞤛 */: BSEQ(0xf0,0x9e,0xa4,0xbd) /* 𞤽 */
    case 0x1e91c /* 𞤜 */: BSEQ(0xf0,0x9e,0xa4,0xbe) /* 𞤾 */
    case 0x1e91d /* 𞤝 */: BSEQ(0xf0,0x9e,0xa4,0xbf) /* 𞤿 */
    case 0x1e91e /* 𞤞 */: BSEQ(0xf0,0x9e,0xa5,0x80) /* 𞥀 */
    case 0x1e91f /* 𞤟 */: BSEQ(0xf0,0x9e,0xa5,0x81) /* 𞥁 */
    case 0x1e920 /* 𞤠 */: BSEQ(0xf0,0x9e,0xa5,0x82) /* 𞥂 */
    case 0x1e921 /* 𞤡 */: BSEQ(0xf0,0x9e,0xa5,0x83) /* 𞥃 */
    default: return nullptr;
  }
}

inline bool isPunctuationCharacter(uint32_t code)
{
  switch(code)
  {
    case 0x21: return true; /* ! */
    case 0x22: return true; /* " */
    case 0x23: return true; /* # */
    case 0x24: return true; /* $ */
    case 0x25: return true; /* % */
    case 0x26: return true; /* & */
    case 0x27: return true; /* ' */
    case 0x28: return true; /* ( */
    case 0x29: return true; /* ) */
    case 0x2a: return true; /* * */
    case 0x2b: return true; /* + */
    case 0x2c: return true; /* , */
    case 0x2d: return true; /* - */
    case 0x2e: return true; /* . */
    case 0x2f: return true; /* / */
    case 0x3a: return true; /* : */
    case 0x3b: return true; /* ; */
    case 0x3c: return true; /* < */
    case 0x3d: return true; /* = */
    case 0x3e: return true; /* > */
    case 0x3f: return true; /* ? */
    case 0x40: return true; /* @ */
    case 0x5b: return true; /* [ */
    case 0x5c: return true; /* \ */
    case 0x5d: return true; /* ] */
    case 0x5e: return true; /* ^ */
    case 0x5f: return true; /* _ */
    case 0x60: return true; /* ` */
    case 0x7b: return true; /* { */
    case 0x7c: return true; /* | */
    case 0x7d: return true; /* } */
    case 0x7e: return true; /* ~ */
    case 0xa1: return true; /* ¡ */
    case 0xa2: return true; /* ¢ */
    case 0xa3: return true; /* £ */
    case 0xa4: return true; /* ¤ */
    case 0xa5: return true; /* ¥ */
    case 0xa6: return true; /* ¦ */
    case 0xa7: return true; /* § */
    case 0xa8: return true; /* ¨ */
    case 0xa9: return true; /* © */
    case 0xab: return true; /* « */
    case 0xac: return true; /* ¬ */
    case 0xae: return true; /* ® */
    case 0xaf: return true; /* ¯ */
    case 0xb0: return true; /* ° */
    case 0xb1: return true; /* ± */
    case 0xb4: return true; /* ´ */
    case 0xb6: return true; /* ¶ */
    case 0xb7: return true; /* · */
    case 0xb8: return true; /* ¸ */
    case 0xbb: return true; /* » */
    case 0xbf: return true; /* ¿ */
    case 0xd7: return true; /* × */
    case 0xf7: return true; /* ÷ */
    case 0x2c2: return true; /* ˂ */
    case 0x2c3: return true; /* ˃ */
    case 0x2c4: return true; /* ˄ */
    case 0x2c5: return true; /* ˅ */
    case 0x2d2: return true; /* ˒ */
    case 0x2d3: return true; /* ˓ */
    case 0x2d4: return true; /* ˔ */
    case 0x2d5: return true; /* ˕ */
    case 0x2d6: return true; /* ˖ */
    case 0x2d7: return true; /* ˗ */
    case 0x2d8: return true; /* ˘ */
    case 0x2d9: return true; /* ˙ */
    case 0x2da: return true; /* ˚ */
    case 0x2db: return true; /* ˛ */
    case 0x2dc: return true; /* ˜ */
    case 0x2dd: return true; /* ˝ */
    case 0x2de: return true; /* ˞ */
    case 0x2df: return true; /* ˟ */
    case 0x2e5: return true; /* ˥ */
    case 0x2e6: return true; /* ˦ */
    case 0x2e7: return true; /* ˧ */
    case 0x2e8: return true; /* ˨ */
    case 0x2e9: return true; /* ˩ */
    case 0x2ea: return true; /* ˪ */
    case 0x2eb: return true; /* ˫ */
    case 0x2ed: return true; /* ˭ */
    case 0x2ef: return true; /* ˯ */
    case 0x2f0: return true; /* ˰ */
    case 0x2f1: return true; /* ˱ */
    case 0x2f2: return true; /* ˲ */
    case 0x2f3: return true; /* ˳ */
    case 0x2f4: return true; /* ˴ */
    case 0x2f5: return true; /* ˵ */
    case 0x2f6: return true; /* ˶ */
    case 0x2f7: return true; /* ˷ */
    case 0x2f8: return true; /* ˸ */
    case 0x2f9: return true; /* ˹ */
    case 0x2fa: return true; /* ˺ */
    case 0x2fb: return true; /* ˻ */
    case 0x2fc: return true; /* ˼ */
    case 0x2fd: return true; /* ˽ */
    case 0x2fe: return true; /* ˾ */
    case 0x2ff: return true; /* ˿ */
    case 0x375: return true; /* ͵ */
    case 0x37e: return true; /* ; */
    case 0x384: return true; /* ΄ */
    case 0x385: return true; /* ΅ */
    case 0x387: return true; /* · */
    case 0x3f6: return true; /* ϶ */
    case 0x482: return true; /* ҂ */
    case 0x55a: return true; /* ՚ */
    case 0x55b: return true; /* ՛ */
    case 0x55c: return true; /* ՜ */
    case 0x55d: return true; /* ՝ */
    case 0x55e: return true; /* ՞ */
    case 0x55f: return true; /* ՟ */
    case 0x589: return true; /* ։ */
    case 0x58a: return true; /* ֊ */
    case 0x58d: return true; /* ֍ */
    case 0x58e: return true; /* ֎ */
    case 0x58f: return true; /* ֏ */
    case 0x5be: return true; /* ־ */
    case 0x5c0: return true; /* ׀ */
    case 0x5c3: return true; /* ׃ */
    case 0x5c6: return true; /* ׆ */
    case 0x5f3: return true; /* ׳ */
    case 0x5f4: return true; /* ״ */
    case 0x606: return true; /* ؆ */
    case 0x607: return true; /* ؇ */
    case 0x608: return true; /* ؈ */
    case 0x609: return true; /* ؉ */
    case 0x60a: return true; /* ؊ */
    case 0x60b: return true; /* ؋ */
    case 0x60c: return true; /* ، */
    case 0x60d: return true; /* ؍ */
    case 0x60e: return true; /* ؎ */
    case 0x60f: return true; /* ؏ */
    case 0x61b: return true; /* ؛ */
    case 0x61e: return true; /* ؞ */
    case 0x61f: return true; /* ؟ */
    case 0x66a: return true; /* ٪ */
    case 0x66b: return true; /* ٫ */
    case 0x66c: return true; /* ٬ */
    case 0x66d: return true; /* ٭ */
    case 0x6d4: return true; /* ۔ */
    case 0x6de: return true; /* ۞ */
    case 0x6e9: return true; /* ۩ */
    case 0x6fd: return true; /* ۽ */
    case 0x6fe: return true; /* ۾ */
    case 0x700: return true; /* ܀ */
    case 0x701: return true; /* ܁ */
    case 0x702: return true; /* ܂ */
    case 0x703: return true; /* ܃ */
    case 0x704: return true; /* ܄ */
    case 0x705: return true; /* ܅ */
    case 0x706: return true; /* ܆ */
    case 0x707: return true; /* ܇ */
    case 0x708: return true; /* ܈ */
    case 0x709: return true; /* ܉ */
    case 0x70a: return true; /* ܊ */
    case 0x70b: return true; /* ܋ */
    case 0x70c: return true; /* ܌ */
    case 0x70d: return true; /* ܍ */
    case 0x7f6: return true; /* ߶ */
    case 0x7f7: return true; /* ߷ */
    case 0x7f8: return true; /* ߸ */
    case 0x7f9: return true; /* ߹ */
    case 0x7fe: return true; /* ߾ */
    case 0x7ff: return true; /* ߿ */
    case 0x830: return true; /* ࠰ */
    case 0x831: return true; /* ࠱ */
    case 0x832: return true; /* ࠲ */
    case 0x833: return true; /* ࠳ */
    case 0x834: return true; /* ࠴ */
    case 0x835: return true; /* ࠵ */
    case 0x836: return true; /* ࠶ */
    case 0x837: return true; /* ࠷ */
    case 0x838: return true; /* ࠸ */
    case 0x839: return true; /* ࠹ */
    case 0x83a: return true; /* ࠺ */
    case 0x83b: return true; /* ࠻ */
    case 0x83c: return true; /* ࠼ */
    case 0x83d: return true; /* ࠽ */
    case 0x83e: return true; /* ࠾ */
    case 0x85e: return true; /* ࡞ */
    case 0x964: return true; /* । */
    case 0x965: return true; /* ॥ */
    case 0x970: return true; /* ॰ */
    case 0x9f2: return true; /* ৲ */
    case 0x9f3: return true; /* ৳ */
    case 0x9fa: return true; /* ৺ */
    case 0x9fb: return true; /* ৻ */
    case 0x9fd: return true; /* ৽ */
    case 0xa76: return true; /* ੶ */
    case 0xaf0: return true; /* ૰ */
    case 0xaf1: return true; /* ૱ */
    case 0xb70: return true; /* ୰ */
    case 0xbf3: return true; /* ௳ */
    case 0xbf4: return true; /* ௴ */
    case 0xbf5: return true; /* ௵ */
    case 0xbf6: return true; /* ௶ */
    case 0xbf7: return true; /* ௷ */
    case 0xbf8: return true; /* ௸ */
    case 0xbf9: return true; /* ௹ */
    case 0xbfa: return true; /* ௺ */
    case 0xc77: return true; /* ౷ */
    case 0xc7f: return true; /* ౿ */
    case 0xc84: return true; /* ಄ */
    case 0xd4f: return true; /* ൏ */
    case 0xd79: return true; /* ൹ */
    case 0xdf4: return true; /* ෴ */
    case 0xe3f: return true; /* ฿ */
    case 0xe4f: return true; /* ๏ */
    case 0xe5a: return true; /* ๚ */
    case 0xe5b: return true; /* ๛ */
    case 0xf01: return true; /* ༁ */
    case 0xf02: return true; /* ༂ */
    case 0xf03: return true; /* ༃ */
    case 0xf04: return true; /* ༄ */
    case 0xf05: return true; /* ༅ */
    case 0xf06: return true; /* ༆ */
    case 0xf07: return true; /* ༇ */
    case 0xf08: return true; /* ༈ */
    case 0xf09: return true; /* ༉ */
    case 0xf0a: return true; /* ༊ */
    case 0xf0b: return true; /* ་ */
    case 0xf0c: return true; /* ༌ */
    case 0xf0d: return true; /* ། */
    case 0xf0e: return true; /* ༎ */
    case 0xf0f: return true; /* ༏ */
    case 0xf10: return true; /* ༐ */
    case 0xf11: return true; /* ༑ */
    case 0xf12: return true; /* ༒ */
    case 0xf13: return true; /* ༓ */
    case 0xf14: return true; /* ༔ */
    case 0xf15: return true; /* ༕ */
    case 0xf16: return true; /* ༖ */
    case 0xf17: return true; /* ༗ */
    case 0xf1a: return true; /* ༚ */
    case 0xf1b: return true; /* ༛ */
    case 0xf1c: return true; /* ༜ */
    case 0xf1d: return true; /* ༝ */
    case 0xf1e: return true; /* ༞ */
    case 0xf1f: return true; /* ༟ */
    case 0xf34: return true; /* ༴ */
    case 0xf36: return true; /* ༶ */
    case 0xf38: return true; /* ༸ */
    case 0xf3a: return true; /* ༺ */
    case 0xf3b: return true; /* ༻ */
    case 0xf3c: return true; /* ༼ */
    case 0xf3d: return true; /* ༽ */
    case 0xf85: return true; /* ྅ */
    case 0xfbe: return true; /* ྾ */
    case 0xfbf: return true; /* ྿ */
    case 0xfc0: return true; /* ࿀ */
    case 0xfc1: return true; /* ࿁ */
    case 0xfc2: return true; /* ࿂ */
    case 0xfc3: return true; /* ࿃ */
    case 0xfc4: return true; /* ࿄ */
    case 0xfc5: return true; /* ࿅ */
    case 0xfc7: return true; /* ࿇ */
    case 0xfc8: return true; /* ࿈ */
    case 0xfc9: return true; /* ࿉ */
    case 0xfca: return true; /* ࿊ */
    case 0xfcb: return true; /* ࿋ */
    case 0xfcc: return true; /* ࿌ */
    case 0xfce: return true; /* ࿎ */
    case 0xfcf: return true; /* ࿏ */
    case 0xfd0: return true; /* ࿐ */
    case 0xfd1: return true; /* ࿑ */
    case 0xfd2: return true; /* ࿒ */
    case 0xfd3: return true; /* ࿓ */
    case 0xfd4: return true; /* ࿔ */
    case 0xfd5: return true; /* ࿕ */
    case 0xfd6: return true; /* ࿖ */
    case 0xfd7: return true; /* ࿗ */
    case 0xfd8: return true; /* ࿘ */
    case 0xfd9: return true; /* ࿙ */
    case 0xfda: return true; /* ࿚ */
    case 0x104a: return true; /* ၊ */
    case 0x104b: return true; /* ။ */
    case 0x104c: return true; /* ၌ */
    case 0x104d: return true; /* ၍ */
    case 0x104e: return true; /* ၎ */
    case 0x104f: return true; /* ၏ */
    case 0x109e: return true; /* ႞ */
    case 0x109f: return true; /* ႟ */
    case 0x10fb: return true; /* ჻ */
    case 0x1360: return true; /* ፠ */
    case 0x1361: return true; /* ፡ */
    case 0x1362: return true; /* ። */
    case 0x1363: return true; /* ፣ */
    case 0x1364: return true; /* ፤ */
    case 0x1365: return true; /* ፥ */
    case 0x1366: return true; /* ፦ */
    case 0x1367: return true; /* ፧ */
    case 0x1368: return true; /* ፨ */
    case 0x1390: return true; /* ᎐ */
    case 0x1391: return true; /* ᎑ */
    case 0x1392: return true; /* ᎒ */
    case 0x1393: return true; /* ᎓ */
    case 0x1394: return true; /* ᎔ */
    case 0x1395: return true; /* ᎕ */
    case 0x1396: return true; /* ᎖ */
    case 0x1397: return true; /* ᎗ */
    case 0x1398: return true; /* ᎘ */
    case 0x1399: return true; /* ᎙ */
    case 0x1400: return true; /* ᐀ */
    case 0x166d: return true; /* ᙭ */
    case 0x166e: return true; /* ᙮ */
    case 0x169b: return true; /* ᚛ */
    case 0x169c: return true; /* ᚜ */
    case 0x16eb: return true; /* ᛫ */
    case 0x16ec: return true; /* ᛬ */
    case 0x16ed: return true; /* ᛭ */
    case 0x1735: return true; /* ᜵ */
    case 0x1736: return true; /* ᜶ */
    case 0x17d4: return true; /* ។ */
    case 0x17d5: return true; /* ៕ */
    case 0x17d6: return true; /* ៖ */
    case 0x17d8: return true; /* ៘ */
    case 0x17d9: return true; /* ៙ */
    case 0x17da: return true; /* ៚ */
    case 0x17db: return true; /* ៛ */
    case 0x1800: return true; /* ᠀ */
    case 0x1801: return true; /* ᠁ */
    case 0x1802: return true; /* ᠂ */
    case 0x1803: return true; /* ᠃ */
    case 0x1804: return true; /* ᠄ */
    case 0x1805: return true; /* ᠅ */
    case 0x1806: return true; /* ᠆ */
    case 0x1807: return true; /* ᠇ */
    case 0x1808: return true; /* ᠈ */
    case 0x1809: return true; /* ᠉ */
    case 0x180a: return true; /* ᠊ */
    case 0x1940: return true; /* ᥀ */
    case 0x1944: return true; /* ᥄ */
    case 0x1945: return true; /* ᥅ */
    case 0x19de: return true; /* ᧞ */
    case 0x19df: return true; /* ᧟ */
    case 0x19e0: return true; /* ᧠ */
    case 0x19e1: return true; /* ᧡ */
    case 0x19e2: return true; /* ᧢ */
    case 0x19e3: return true; /* ᧣ */
    case 0x19e4: return true; /* ᧤ */
    case 0x19e5: return true; /* ᧥ */
    case 0x19e6: return true; /* ᧦ */
    case 0x19e7: return true; /* ᧧ */
    case 0x19e8: return true; /* ᧨ */
    case 0x19e9: return true; /* ᧩ */
    case 0x19ea: return true; /* ᧪ */
    case 0x19eb: return true; /* ᧫ */
    case 0x19ec: return true; /* ᧬ */
    case 0x19ed: return true; /* ᧭ */
    case 0x19ee: return true; /* ᧮ */
    case 0x19ef: return true; /* ᧯ */
    case 0x19f0: return true; /* ᧰ */
    case 0x19f1: return true; /* ᧱ */
    case 0x19f2: return true; /* ᧲ */
    case 0x19f3: return true; /* ᧳ */
    case 0x19f4: return true; /* ᧴ */
    case 0x19f5: return true; /* ᧵ */
    case 0x19f6: return true; /* ᧶ */
    case 0x19f7: return true; /* ᧷ */
    case 0x19f8: return true; /* ᧸ */
    case 0x19f9: return true; /* ᧹ */
    case 0x19fa: return true; /* ᧺ */
    case 0x19fb: return true; /* ᧻ */
    case 0x19fc: return true; /* ᧼ */
    case 0x19fd: return true; /* ᧽ */
    case 0x19fe: return true; /* ᧾ */
    case 0x19ff: return true; /* ᧿ */
    case 0x1a1e: return true; /* ᨞ */
    case 0x1a1f: return true; /* ᨟ */
    case 0x1aa0: return true; /* ᪠ */
    case 0x1aa1: return true; /* ᪡ */
    case 0x1aa2: return true; /* ᪢ */
    case 0x1aa3: return true; /* ᪣ */
    case 0x1aa4: return true; /* ᪤ */
    case 0x1aa5: return true; /* ᪥ */
    case 0x1aa6: return true; /* ᪦ */
    case 0x1aa8: return true; /* ᪨ */
    case 0x1aa9: return true; /* ᪩ */
    case 0x1aaa: return true; /* ᪪ */
    case 0x1aab: return true; /* ᪫ */
    case 0x1aac: return true; /* ᪬ */
    case 0x1aad: return true; /* ᪭ */
    case 0x1b5a: return true; /* ᭚ */
    case 0x1b5b: return true; /* ᭛ */
    case 0x1b5c: return true; /* ᭜ */
    case 0x1b5d: return true; /* ᭝ */
    case 0x1b5e: return true; /* ᭞ */
    case 0x1b5f: return true; /* ᭟ */
    case 0x1b60: return true; /* ᭠ */
    case 0x1b61: return true; /* ᭡ */
    case 0x1b62: return true; /* ᭢ */
    case 0x1b63: return true; /* ᭣ */
    case 0x1b64: return true; /* ᭤ */
    case 0x1b65: return true; /* ᭥ */
    case 0x1b66: return true; /* ᭦ */
    case 0x1b67: return true; /* ᭧ */
    case 0x1b68: return true; /* ᭨ */
    case 0x1b69: return true; /* ᭩ */
    case 0x1b6a: return true; /* ᭪ */
    case 0x1b74: return true; /* ᭴ */
    case 0x1b75: return true; /* ᭵ */
    case 0x1b76: return true; /* ᭶ */
    case 0x1b77: return true; /* ᭷ */
    case 0x1b78: return true; /* ᭸ */
    case 0x1b79: return true; /* ᭹ */
    case 0x1b7a: return true; /* ᭺ */
    case 0x1b7b: return true; /* ᭻ */
    case 0x1b7c: return true; /* ᭼ */
    case 0x1bfc: return true; /* ᯼ */
    case 0x1bfd: return true; /* ᯽ */
    case 0x1bfe: return true; /* ᯾ */
    case 0x1bff: return true; /* ᯿ */
    case 0x1c3b: return true; /* ᰻ */
    case 0x1c3c: return true; /* ᰼ */
    case 0x1c3d: return true; /* ᰽ */
    case 0x1c3e: return true; /* ᰾ */
    case 0x1c3f: return true; /* ᰿ */
    case 0x1c7e: return true; /* ᱾ */
    case 0x1c7f: return true; /* ᱿ */
    case 0x1cc0: return true; /* ᳀ */
    case 0x1cc1: return true; /* ᳁ */
    case 0x1cc2: return true; /* ᳂ */
    case 0x1cc3: return true; /* ᳃ */
    case 0x1cc4: return true; /* ᳄ */
    case 0x1cc5: return true; /* ᳅ */
    case 0x1cc6: return true; /* ᳆ */
    case 0x1cc7: return true; /* ᳇ */
    case 0x1cd3: return true; /* ᳓ */
    case 0x1fbd: return true; /* ᾽ */
    case 0x1fbf: return true; /* ᾿ */
    case 0x1fc0: return true; /* ῀ */
    case 0x1fc1: return true; /* ῁ */
    case 0x1fcd: return true; /* ῍ */
    case 0x1fce: return true; /* ῎ */
    case 0x1fcf: return true; /* ῏ */
    case 0x1fdd: return true; /* ῝ */
    case 0x1fde: return true; /* ῞ */
    case 0x1fdf: return true; /* ῟ */
    case 0x1fed: return true; /* ῭ */
    case 0x1fee: return true; /* ΅ */
    case 0x1fef: return true; /* ` */
    case 0x1ffd: return true; /* ´ */
    case 0x1ffe: return true; /* ῾ */
    case 0x2010: return true; /* ‐ */
    case 0x2011: return true; /* ‑ */
    case 0x2012: return true; /* ‒ */
    case 0x2013: return true; /* – */
    case 0x2014: return true; /* — */
    case 0x2015: return true; /* ― */
    case 0x2016: return true; /* ‖ */
    case 0x2017: return true; /* ‗ */
    case 0x2018: return true; /* ‘ */
    case 0x2019: return true; /* ’ */
    case 0x201a: return true; /* ‚ */
    case 0x201b: return true; /* ‛ */
    case 0x201c: return true; /* “ */
    case 0x201d: return true; /* ” */
    case 0x201e: return true; /* „ */
    case 0x201f: return true; /* ‟ */
    case 0x2020: return true; /* † */
    case 0x2021: return true; /* ‡ */
    case 0x2022: return true; /* • */
    case 0x2023: return true; /* ‣ */
    case 0x2024: return true; /* ․ */
    case 0x2025: return true; /* ‥ */
    case 0x2026: return true; /* … */
    case 0x2027: return true; /* ‧ */
    case 0x2030: return true; /* ‰ */
    case 0x2031: return true; /* ‱ */
    case 0x2032: return true; /* ′ */
    case 0x2033: return true; /* ″ */
    case 0x2034: return true; /* ‴ */
    case 0x2035: return true; /* ‵ */
    case 0x2036: return true; /* ‶ */
    case 0x2037: return true; /* ‷ */
    case 0x2038: return true; /* ‸ */
    case 0x2039: return true; /* ‹ */
    case 0x203a: return true; /* › */
    case 0x203b: return true; /* ※ */
    case 0x203c: return true; /* ‼ */
    case 0x203d: return true; /* ‽ */
    case 0x203e: return true; /* ‾ */
    case 0x203f: return true; /* ‿ */
    case 0x2040: return true; /* ⁀ */
    case 0x2041: return true; /* ⁁ */
    case 0x2042: return true; /* ⁂ */
    case 0x2043: return true; /* ⁃ */
    case 0x2044: return true; /* ⁄ */
    case 0x2045: return true; /* ⁅ */
    case 0x2046: return true; /* ⁆ */
    case 0x2047: return true; /* ⁇ */
    case 0x2048: return true; /* ⁈ */
    case 0x2049: return true; /* ⁉ */
    case 0x204a: return true; /* ⁊ */
    case 0x204b: return true; /* ⁋ */
    case 0x204c: return true; /* ⁌ */
    case 0x204d: return true; /* ⁍ */
    case 0x204e: return true; /* ⁎ */
    case 0x204f: return true; /* ⁏ */
    case 0x2050: return true; /* ⁐ */
    case 0x2051: return true; /* ⁑ */
    case 0x2052: return true; /* ⁒ */
    case 0x2053: return true; /* ⁓ */
    case 0x2054: return true; /* ⁔ */
    case 0x2055: return true; /* ⁕ */
    case 0x2056: return true; /* ⁖ */
    case 0x2057: return true; /* ⁗ */
    case 0x2058: return true; /* ⁘ */
    case 0x2059: return true; /* ⁙ */
    case 0x205a: return true; /* ⁚ */
    case 0x205b: return true; /* ⁛ */
    case 0x205c: return true; /* ⁜ */
    case 0x205d: return true; /* ⁝ */
    case 0x205e: return true; /* ⁞ */
    case 0x207a: return true; /* ⁺ */
    case 0x207b: return true; /* ⁻ */
    case 0x207c: return true; /* ⁼ */
    case 0x207d: return true; /* ⁽ */
    case 0x207e: return true; /* ⁾ */
    case 0x208a: return true; /* ₊ */
    case 0x208b: return true; /* ₋ */
    case 0x208c: return true; /* ₌ */
    case 0x208d: return true; /* ₍ */
    case 0x208e: return true; /* ₎ */
    case 0x20a0: return true; /* ₠ */
    case 0x20a1: return true; /* ₡ */
    case 0x20a2: return true; /* ₢ */
    case 0x20a3: return true; /* ₣ */
    case 0x20a4: return true; /* ₤ */
    case 0x20a5: return true; /* ₥ */
    case 0x20a6: return true; /* ₦ */
    case 0x20a7: return true; /* ₧ */
    case 0x20a8: return true; /* ₨ */
    case 0x20a9: return true; /* ₩ */
    case 0x20aa: return true; /* ₪ */
    case 0x20ab: return true; /* ₫ */
    case 0x20ac: return true; /* € */
    case 0x20ad: return true; /* ₭ */
    case 0x20ae: return true; /* ₮ */
    case 0x20af: return true; /* ₯ */
    case 0x20b0: return true; /* ₰ */
    case 0x20b1: return true; /* ₱ */
    case 0x20b2: return true; /* ₲ */
    case 0x20b3: return true; /* ₳ */
    case 0x20b4: return true; /* ₴ */
    case 0x20b5: return true; /* ₵ */
    case 0x20b6: return true; /* ₶ */
    case 0x20b7: return true; /* ₷ */
    case 0x20b8: return true; /* ₸ */
    case 0x20b9: return true; /* ₹ */
    case 0x20ba: return true; /* ₺ */
    case 0x20bb: return true; /* ₻ */
    case 0x20bc: return true; /* ₼ */
    case 0x20bd: return true; /* ₽ */
    case 0x20be: return true; /* ₾ */
    case 0x20bf: return true; /* ₿ */
    case 0x2100: return true; /* ℀ */
    case 0x2101: return true; /* ℁ */
    case 0x2103: return true; /* ℃ */
    case 0x2104: return true; /* ℄ */
    case 0x2105: return true; /* ℅ */
    case 0x2106: return true; /* ℆ */
    case 0x2108: return true; /* ℈ */
    case 0x2109: return true; /* ℉ */
    case 0x2114: return true; /* ℔ */
    case 0x2116: return true; /* № */
    case 0x2117: return true; /* ℗ */
    case 0x2118: return true; /* ℘ */
    case 0x211e: return true; /* ℞ */
    case 0x211f: return true; /* ℟ */
    case 0x2120: return true; /* ℠ */
    case 0x2121: return true; /* ℡ */
    case 0x2122: return true; /* ™ */
    case 0x2123: return true; /* ℣ */
    case 0x2125: return true; /* ℥ */
    case 0x2127: return true; /* ℧ */
    case 0x2129: return true; /* ℩ */
    case 0x212e: return true; /* ℮ */
    case 0x213a: return true; /* ℺ */
    case 0x213b: return true; /* ℻ */
    case 0x2140: return true; /* ⅀ */
    case 0x2141: return true; /* ⅁ */
    case 0x2142: return true; /* ⅂ */
    case 0x2143: return true; /* ⅃ */
    case 0x2144: return true; /* ⅄ */
    case 0x214a: return true; /* ⅊ */
    case 0x214b: return true; /* ⅋ */
    case 0x214c: return true; /* ⅌ */
    case 0x214d: return true; /* ⅍ */
    case 0x214f: return true; /* ⅏ */
    case 0x218a: return true; /* ↊ */
    case 0x218b: return true; /* ↋ */
    case 0x2190: return true; /* ← */
    case 0x2191: return true; /* ↑ */
    case 0x2192: return true; /* → */
    case 0x2193: return true; /* ↓ */
    case 0x2194: return true; /* ↔ */
    case 0x2195: return true; /* ↕ */
    case 0x2196: return true; /* ↖ */
    case 0x2197: return true; /* ↗ */
    case 0x2198: return true; /* ↘ */
    case 0x2199: return true; /* ↙ */
    case 0x219a: return true; /* ↚ */
    case 0x219b: return true; /* ↛ */
    case 0x219c: return true; /* ↜ */
    case 0x219d: return true; /* ↝ */
    case 0x219e: return true; /* ↞ */
    case 0x219f: return true; /* ↟ */
    case 0x21a0: return true; /* ↠ */
    case 0x21a1: return true; /* ↡ */
    case 0x21a2: return true; /* ↢ */
    case 0x21a3: return true; /* ↣ */
    case 0x21a4: return true; /* ↤ */
    case 0x21a5: return true; /* ↥ */
    case 0x21a6: return true; /* ↦ */
    case 0x21a7: return true; /* ↧ */
    case 0x21a8: return true; /* ↨ */
    case 0x21a9: return true; /* ↩ */
    case 0x21aa: return true; /* ↪ */
    case 0x21ab: return true; /* ↫ */
    case 0x21ac: return true; /* ↬ */
    case 0x21ad: return true; /* ↭ */
    case 0x21ae: return true; /* ↮ */
    case 0x21af: return true; /* ↯ */
    case 0x21b0: return true; /* ↰ */
    case 0x21b1: return true; /* ↱ */
    case 0x21b2: return true; /* ↲ */
    case 0x21b3: return true; /* ↳ */
    case 0x21b4: return true; /* ↴ */
    case 0x21b5: return true; /* ↵ */
    case 0x21b6: return true; /* ↶ */
    case 0x21b7: return true; /* ↷ */
    case 0x21b8: return true; /* ↸ */
    case 0x21b9: return true; /* ↹ */
    case 0x21ba: return true; /* ↺ */
    case 0x21bb: return true; /* ↻ */
    case 0x21bc: return true; /* ↼ */
    case 0x21bd: return true; /* ↽ */
    case 0x21be: return true; /* ↾ */
    case 0x21bf: return true; /* ↿ */
    case 0x21c0: return true; /* ⇀ */
    case 0x21c1: return true; /* ⇁ */
    case 0x21c2: return true; /* ⇂ */
    case 0x21c3: return true; /* ⇃ */
    case 0x21c4: return true; /* ⇄ */
    case 0x21c5: return true; /* ⇅ */
    case 0x21c6: return true; /* ⇆ */
    case 0x21c7: return true; /* ⇇ */
    case 0x21c8: return true; /* ⇈ */
    case 0x21c9: return true; /* ⇉ */
    case 0x21ca: return true; /* ⇊ */
    case 0x21cb: return true; /* ⇋ */
    case 0x21cc: return true; /* ⇌ */
    case 0x21cd: return true; /* ⇍ */
    case 0x21ce: return true; /* ⇎ */
    case 0x21cf: return true; /* ⇏ */
    case 0x21d0: return true; /* ⇐ */
    case 0x21d1: return true; /* ⇑ */
    case 0x21d2: return true; /* ⇒ */
    case 0x21d3: return true; /* ⇓ */
    case 0x21d4: return true; /* ⇔ */
    case 0x21d5: return true; /* ⇕ */
    case 0x21d6: return true; /* ⇖ */
    case 0x21d7: return true; /* ⇗ */
    case 0x21d8: return true; /* ⇘ */
    case 0x21d9: return true; /* ⇙ */
    case 0x21da: return true; /* ⇚ */
    case 0x21db: return true; /* ⇛ */
    case 0x21dc: return true; /* ⇜ */
    case 0x21dd: return true; /* ⇝ */
    case 0x21de: return true; /* ⇞ */
    case 0x21df: return true; /* ⇟ */
    case 0x21e0: return true; /* ⇠ */
    case 0x21e1: return true; /* ⇡ */
    case 0x21e2: return true; /* ⇢ */
    case 0x21e3: return true; /* ⇣ */
    case 0x21e4: return true; /* ⇤ */
    case 0x21e5: return true; /* ⇥ */
    case 0x21e6: return true; /* ⇦ */
    case 0x21e7: return true; /* ⇧ */
    case 0x21e8: return true; /* ⇨ */
    case 0x21e9: return true; /* ⇩ */
    case 0x21ea: return true; /* ⇪ */
    case 0x21eb: return true; /* ⇫ */
    case 0x21ec: return true; /* ⇬ */
    case 0x21ed: return true; /* ⇭ */
    case 0x21ee: return true; /* ⇮ */
    case 0x21ef: return true; /* ⇯ */
    case 0x21f0: return true; /* ⇰ */
    case 0x21f1: return true; /* ⇱ */
    case 0x21f2: return true; /* ⇲ */
    case 0x21f3: return true; /* ⇳ */
    case 0x21f4: return true; /* ⇴ */
    case 0x21f5: return true; /* ⇵ */
    case 0x21f6: return true; /* ⇶ */
    case 0x21f7: return true; /* ⇷ */
    case 0x21f8: return true; /* ⇸ */
    case 0x21f9: return true; /* ⇹ */
    case 0x21fa: return true; /* ⇺ */
    case 0x21fb: return true; /* ⇻ */
    case 0x21fc: return true; /* ⇼ */
    case 0x21fd: return true; /* ⇽ */
    case 0x21fe: return true; /* ⇾ */
    case 0x21ff: return true; /* ⇿ */
    case 0x2200: return true; /* ∀ */
    case 0x2201: return true; /* ∁ */
    case 0x2202: return true; /* ∂ */
    case 0x2203: return true; /* ∃ */
    case 0x2204: return true; /* ∄ */
    case 0x2205: return true; /* ∅ */
    case 0x2206: return true; /* ∆ */
    case 0x2207: return true; /* ∇ */
    case 0x2208: return true; /* ∈ */
    case 0x2209: return true; /* ∉ */
    case 0x220a: return true; /* ∊ */
    case 0x220b: return true; /* ∋ */
    case 0x220c: return true; /* ∌ */
    case 0x220d: return true; /* ∍ */
    case 0x220e: return true; /* ∎ */
    case 0x220f: return true; /* ∏ */
    case 0x2210: return true; /* ∐ */
    case 0x2211: return true; /* ∑ */
    case 0x2212: return true; /* − */
    case 0x2213: return true; /* ∓ */
    case 0x2214: return true; /* ∔ */
    case 0x2215: return true; /* ∕ */
    case 0x2216: return true; /* ∖ */
    case 0x2217: return true; /* ∗ */
    case 0x2218: return true; /* ∘ */
    case 0x2219: return true; /* ∙ */
    case 0x221a: return true; /* √ */
    case 0x221b: return true; /* ∛ */
    case 0x221c: return true; /* ∜ */
    case 0x221d: return true; /* ∝ */
    case 0x221e: return true; /* ∞ */
    case 0x221f: return true; /* ∟ */
    case 0x2220: return true; /* ∠ */
    case 0x2221: return true; /* ∡ */
    case 0x2222: return true; /* ∢ */
    case 0x2223: return true; /* ∣ */
    case 0x2224: return true; /* ∤ */
    case 0x2225: return true; /* ∥ */
    case 0x2226: return true; /* ∦ */
    case 0x2227: return true; /* ∧ */
    case 0x2228: return true; /* ∨ */
    case 0x2229: return true; /* ∩ */
    case 0x222a: return true; /* ∪ */
    case 0x222b: return true; /* ∫ */
    case 0x222c: return true; /* ∬ */
    case 0x222d: return true; /* ∭ */
    case 0x222e: return true; /* ∮ */
    case 0x222f: return true; /* ∯ */
    case 0x2230: return true; /* ∰ */
    case 0x2231: return true; /* ∱ */
    case 0x2232: return true; /* ∲ */
    case 0x2233: return true; /* ∳ */
    case 0x2234: return true; /* ∴ */
    case 0x2235: return true; /* ∵ */
    case 0x2236: return true; /* ∶ */
    case 0x2237: return true; /* ∷ */
    case 0x2238: return true; /* ∸ */
    case 0x2239: return true; /* ∹ */
    case 0x223a: return true; /* ∺ */
    case 0x223b: return true; /* ∻ */
    case 0x223c: return true; /* ∼ */
    case 0x223d: return true; /* ∽ */
    case 0x223e: return true; /* ∾ */
    case 0x223f: return true; /* ∿ */
    case 0x2240: return true; /* ≀ */
    case 0x2241: return true; /* ≁ */
    case 0x2242: return true; /* ≂ */
    case 0x2243: return true; /* ≃ */
    case 0x2244: return true; /* ≄ */
    case 0x2245: return true; /* ≅ */
    case 0x2246: return true; /* ≆ */
    case 0x2247: return true; /* ≇ */
    case 0x2248: return true; /* ≈ */
    case 0x2249: return true; /* ≉ */
    case 0x224a: return true; /* ≊ */
    case 0x224b: return true; /* ≋ */
    case 0x224c: return true; /* ≌ */
    case 0x224d: return true; /* ≍ */
    case 0x224e: return true; /* ≎ */
    case 0x224f: return true; /* ≏ */
    case 0x2250: return true; /* ≐ */
    case 0x2251: return true; /* ≑ */
    case 0x2252: return true; /* ≒ */
    case 0x2253: return true; /* ≓ */
    case 0x2254: return true; /* ≔ */
    case 0x2255: return true; /* ≕ */
    case 0x2256: return true; /* ≖ */
    case 0x2257: return true; /* ≗ */
    case 0x2258: return true; /* ≘ */
    case 0x2259: return true; /* ≙ */
    case 0x225a: return true; /* ≚ */
    case 0x225b: return true; /* ≛ */
    case 0x225c: return true; /* ≜ */
    case 0x225d: return true; /* ≝ */
    case 0x225e: return true; /* ≞ */
    case 0x225f: return true; /* ≟ */
    case 0x2260: return true; /* ≠ */
    case 0x2261: return true; /* ≡ */
    case 0x2262: return true; /* ≢ */
    case 0x2263: return true; /* ≣ */
    case 0x2264: return true; /* ≤ */
    case 0x2265: return true; /* ≥ */
    case 0x2266: return true; /* ≦ */
    case 0x2267: return true; /* ≧ */
    case 0x2268: return true; /* ≨ */
    case 0x2269: return true; /* ≩ */
    case 0x226a: return true; /* ≪ */
    case 0x226b: return true; /* ≫ */
    case 0x226c: return true; /* ≬ */
    case 0x226d: return true; /* ≭ */
    case 0x226e: return true; /* ≮ */
    case 0x226f: return true; /* ≯ */
    case 0x2270: return true; /* ≰ */
    case 0x2271: return true; /* ≱ */
    case 0x2272: return true; /* ≲ */
    case 0x2273: return true; /* ≳ */
    case 0x2274: return true; /* ≴ */
    case 0x2275: return true; /* ≵ */
    case 0x2276: return true; /* ≶ */
    case 0x2277: return true; /* ≷ */
    case 0x2278: return true; /* ≸ */
    case 0x2279: return true; /* ≹ */
    case 0x227a: return true; /* ≺ */
    case 0x227b: return true; /* ≻ */
    case 0x227c: return true; /* ≼ */
    case 0x227d: return true; /* ≽ */
    case 0x227e: return true; /* ≾ */
    case 0x227f: return true; /* ≿ */
    case 0x2280: return true; /* ⊀ */
    case 0x2281: return true; /* ⊁ */
    case 0x2282: return true; /* ⊂ */
    case 0x2283: return true; /* ⊃ */
    case 0x2284: return true; /* ⊄ */
    case 0x2285: return true; /* ⊅ */
    case 0x2286: return true; /* ⊆ */
    case 0x2287: return true; /* ⊇ */
    case 0x2288: return true; /* ⊈ */
    case 0x2289: return true; /* ⊉ */
    case 0x228a: return true; /* ⊊ */
    case 0x228b: return true; /* ⊋ */
    case 0x228c: return true; /* ⊌ */
    case 0x228d: return true; /* ⊍ */
    case 0x228e: return true; /* ⊎ */
    case 0x228f: return true; /* ⊏ */
    case 0x2290: return true; /* ⊐ */
    case 0x2291: return true; /* ⊑ */
    case 0x2292: return true; /* ⊒ */
    case 0x2293: return true; /* ⊓ */
    case 0x2294: return true; /* ⊔ */
    case 0x2295: return true; /* ⊕ */
    case 0x2296: return true; /* ⊖ */
    case 0x2297: return true; /* ⊗ */
    case 0x2298: return true; /* ⊘ */
    case 0x2299: return true; /* ⊙ */
    case 0x229a: return true; /* ⊚ */
    case 0x229b: return true; /* ⊛ */
    case 0x229c: return true; /* ⊜ */
    case 0x229d: return true; /* ⊝ */
    case 0x229e: return true; /* ⊞ */
    case 0x229f: return true; /* ⊟ */
    case 0x22a0: return true; /* ⊠ */
    case 0x22a1: return true; /* ⊡ */
    case 0x22a2: return true; /* ⊢ */
    case 0x22a3: return true; /* ⊣ */
    case 0x22a4: return true; /* ⊤ */
    case 0x22a5: return true; /* ⊥ */
    case 0x22a6: return true; /* ⊦ */
    case 0x22a7: return true; /* ⊧ */
    case 0x22a8: return true; /* ⊨ */
    case 0x22a9: return true; /* ⊩ */
    case 0x22aa: return true; /* ⊪ */
    case 0x22ab: return true; /* ⊫ */
    case 0x22ac: return true; /* ⊬ */
    case 0x22ad: return true; /* ⊭ */
    case 0x22ae: return true; /* ⊮ */
    case 0x22af: return true; /* ⊯ */
    case 0x22b0: return true; /* ⊰ */
    case 0x22b1: return true; /* ⊱ */
    case 0x22b2: return true; /* ⊲ */
    case 0x22b3: return true; /* ⊳ */
    case 0x22b4: return true; /* ⊴ */
    case 0x22b5: return true; /* ⊵ */
    case 0x22b6: return true; /* ⊶ */
    case 0x22b7: return true; /* ⊷ */
    case 0x22b8: return true; /* ⊸ */
    case 0x22b9: return true; /* ⊹ */
    case 0x22ba: return true; /* ⊺ */
    case 0x22bb: return true; /* ⊻ */
    case 0x22bc: return true; /* ⊼ */
    case 0x22bd: return true; /* ⊽ */
    case 0x22be: return true; /* ⊾ */
    case 0x22bf: return true; /* ⊿ */
    case 0x22c0: return true; /* ⋀ */
    case 0x22c1: return true; /* ⋁ */
    case 0x22c2: return true; /* ⋂ */
    case 0x22c3: return true; /* ⋃ */
    case 0x22c4: return true; /* ⋄ */
    case 0x22c5: return true; /* ⋅ */
    case 0x22c6: return true; /* ⋆ */
    case 0x22c7: return true; /* ⋇ */
    case 0x22c8: return true; /* ⋈ */
    case 0x22c9: return true; /* ⋉ */
    case 0x22ca: return true; /* ⋊ */
    case 0x22cb: return true; /* ⋋ */
    case 0x22cc: return true; /* ⋌ */
    case 0x22cd: return true; /* ⋍ */
    case 0x22ce: return true; /* ⋎ */
    case 0x22cf: return true; /* ⋏ */
    case 0x22d0: return true; /* ⋐ */
    case 0x22d1: return true; /* ⋑ */
    case 0x22d2: return true; /* ⋒ */
    case 0x22d3: return true; /* ⋓ */
    case 0x22d4: return true; /* ⋔ */
    case 0x22d5: return true; /* ⋕ */
    case 0x22d6: return true; /* ⋖ */
    case 0x22d7: return true; /* ⋗ */
    case 0x22d8: return true; /* ⋘ */
    case 0x22d9: return true; /* ⋙ */
    case 0x22da: return true; /* ⋚ */
    case 0x22db: return true; /* ⋛ */
    case 0x22dc: return true; /* ⋜ */
    case 0x22dd: return true; /* ⋝ */
    case 0x22de: return true; /* ⋞ */
    case 0x22df: return true; /* ⋟ */
    case 0x22e0: return true; /* ⋠ */
    case 0x22e1: return true; /* ⋡ */
    case 0x22e2: return true; /* ⋢ */
    case 0x22e3: return true; /* ⋣ */
    case 0x22e4: return true; /* ⋤ */
    case 0x22e5: return true; /* ⋥ */
    case 0x22e6: return true; /* ⋦ */
    case 0x22e7: return true; /* ⋧ */
    case 0x22e8: return true; /* ⋨ */
    case 0x22e9: return true; /* ⋩ */
    case 0x22ea: return true; /* ⋪ */
    case 0x22eb: return true; /* ⋫ */
    case 0x22ec: return true; /* ⋬ */
    case 0x22ed: return true; /* ⋭ */
    case 0x22ee: return true; /* ⋮ */
    case 0x22ef: return true; /* ⋯ */
    case 0x22f0: return true; /* ⋰ */
    case 0x22f1: return true; /* ⋱ */
    case 0x22f2: return true; /* ⋲ */
    case 0x22f3: return true; /* ⋳ */
    case 0x22f4: return true; /* ⋴ */
    case 0x22f5: return true; /* ⋵ */
    case 0x22f6: return true; /* ⋶ */
    case 0x22f7: return true; /* ⋷ */
    case 0x22f8: return true; /* ⋸ */
    case 0x22f9: return true; /* ⋹ */
    case 0x22fa: return true; /* ⋺ */
    case 0x22fb: return true; /* ⋻ */
    case 0x22fc: return true; /* ⋼ */
    case 0x22fd: return true; /* ⋽ */
    case 0x22fe: return true; /* ⋾ */
    case 0x22ff: return true; /* ⋿ */
    case 0x2300: return true; /* ⌀ */
    case 0x2301: return true; /* ⌁ */
    case 0x2302: return true; /* ⌂ */
    case 0x2303: return true; /* ⌃ */
    case 0x2304: return true; /* ⌄ */
    case 0x2305: return true; /* ⌅ */
    case 0x2306: return true; /* ⌆ */
    case 0x2307: return true; /* ⌇ */
    case 0x2308: return true; /* ⌈ */
    case 0x2309: return true; /* ⌉ */
    case 0x230a: return true; /* ⌊ */
    case 0x230b: return true; /* ⌋ */
    case 0x230c: return true; /* ⌌ */
    case 0x230d: return true; /* ⌍ */
    case 0x230e: return true; /* ⌎ */
    case 0x230f: return true; /* ⌏ */
    case 0x2310: return true; /* ⌐ */
    case 0x2311: return true; /* ⌑ */
    case 0x2312: return true; /* ⌒ */
    case 0x2313: return true; /* ⌓ */
    case 0x2314: return true; /* ⌔ */
    case 0x2315: return true; /* ⌕ */
    case 0x2316: return true; /* ⌖ */
    case 0x2317: return true; /* ⌗ */
    case 0x2318: return true; /* ⌘ */
    case 0x2319: return true; /* ⌙ */
    case 0x231a: return true; /* ⌚ */
    case 0x231b: return true; /* ⌛ */
    case 0x231c: return true; /* ⌜ */
    case 0x231d: return true; /* ⌝ */
    case 0x231e: return true; /* ⌞ */
    case 0x231f: return true; /* ⌟ */
    case 0x2320: return true; /* ⌠ */
    case 0x2321: return true; /* ⌡ */
    case 0x2322: return true; /* ⌢ */
    case 0x2323: return true; /* ⌣ */
    case 0x2324: return true; /* ⌤ */
    case 0x2325: return true; /* ⌥ */
    case 0x2326: return true; /* ⌦ */
    case 0x2327: return true; /* ⌧ */
    case 0x2328: return true; /* ⌨ */
    case 0x2329: return true; /* 〈 */
    case 0x232a: return true; /* 〉 */
    case 0x232b: return true; /* ⌫ */
    case 0x232c: return true; /* ⌬ */
    case 0x232d: return true; /* ⌭ */
    case 0x232e: return true; /* ⌮ */
    case 0x232f: return true; /* ⌯ */
    case 0x2330: return true; /* ⌰ */
    case 0x2331: return true; /* ⌱ */
    case 0x2332: return true; /* ⌲ */
    case 0x2333: return true; /* ⌳ */
    case 0x2334: return true; /* ⌴ */
    case 0x2335: return true; /* ⌵ */
    case 0x2336: return true; /* ⌶ */
    case 0x2337: return true; /* ⌷ */
    case 0x2338: return true; /* ⌸ */
    case 0x2339: return true; /* ⌹ */
    case 0x233a: return true; /* ⌺ */
    case 0x233b: return true; /* ⌻ */
    case 0x233c: return true; /* ⌼ */
    case 0x233d: return true; /* ⌽ */
    case 0x233e: return true; /* ⌾ */
    case 0x233f: return true; /* ⌿ */
    case 0x2340: return true; /* ⍀ */
    case 0x2341: return true; /* ⍁ */
    case 0x2342: return true; /* ⍂ */
    case 0x2343: return true; /* ⍃ */
    case 0x2344: return true; /* ⍄ */
    case 0x2345: return true; /* ⍅ */
    case 0x2346: return true; /* ⍆ */
    case 0x2347: return true; /* ⍇ */
    case 0x2348: return true; /* ⍈ */
    case 0x2349: return true; /* ⍉ */
    case 0x234a: return true; /* ⍊ */
    case 0x234b: return true; /* ⍋ */
    case 0x234c: return true; /* ⍌ */
    case 0x234d: return true; /* ⍍ */
    case 0x234e: return true; /* ⍎ */
    case 0x234f: return true; /* ⍏ */
    case 0x2350: return true; /* ⍐ */
    case 0x2351: return true; /* ⍑ */
    case 0x2352: return true; /* ⍒ */
    case 0x2353: return true; /* ⍓ */
    case 0x2354: return true; /* ⍔ */
    case 0x2355: return true; /* ⍕ */
    case 0x2356: return true; /* ⍖ */
    case 0x2357: return true; /* ⍗ */
    case 0x2358: return true; /* ⍘ */
    case 0x2359: return true; /* ⍙ */
    case 0x235a: return true; /* ⍚ */
    case 0x235b: return true; /* ⍛ */
    case 0x235c: return true; /* ⍜ */
    case 0x235d: return true; /* ⍝ */
    case 0x235e: return true; /* ⍞ */
    case 0x235f: return true; /* ⍟ */
    case 0x2360: return true; /* ⍠ */
    case 0x2361: return true; /* ⍡ */
    case 0x2362: return true; /* ⍢ */
    case 0x2363: return true; /* ⍣ */
    case 0x2364: return true; /* ⍤ */
    case 0x2365: return true; /* ⍥ */
    case 0x2366: return true; /* ⍦ */
    case 0x2367: return true; /* ⍧ */
    case 0x2368: return true; /* ⍨ */
    case 0x2369: return true; /* ⍩ */
    case 0x236a: return true; /* ⍪ */
    case 0x236b: return true; /* ⍫ */
    case 0x236c: return true; /* ⍬ */
    case 0x236d: return true; /* ⍭ */
    case 0x236e: return true; /* ⍮ */
    case 0x236f: return true; /* ⍯ */
    case 0x2370: return true; /* ⍰ */
    case 0x2371: return true; /* ⍱ */
    case 0x2372: return true; /* ⍲ */
    case 0x2373: return true; /* ⍳ */
    case 0x2374: return true; /* ⍴ */
    case 0x2375: return true; /* ⍵ */
    case 0x2376: return true; /* ⍶ */
    case 0x2377: return true; /* ⍷ */
    case 0x2378: return true; /* ⍸ */
    case 0x2379: return true; /* ⍹ */
    case 0x237a: return true; /* ⍺ */
    case 0x237b: return true; /* ⍻ */
    case 0x237c: return true; /* ⍼ */
    case 0x237d: return true; /* ⍽ */
    case 0x237e: return true; /* ⍾ */
    case 0x237f: return true; /* ⍿ */
    case 0x2380: return true; /* ⎀ */
    case 0x2381: return true; /* ⎁ */
    case 0x2382: return true; /* ⎂ */
    case 0x2383: return true; /* ⎃ */
    case 0x2384: return true; /* ⎄ */
    case 0x2385: return true; /* ⎅ */
    case 0x2386: return true; /* ⎆ */
    case 0x2387: return true; /* ⎇ */
    case 0x2388: return true; /* ⎈ */
    case 0x2389: return true; /* ⎉ */
    case 0x238a: return true; /* ⎊ */
    case 0x238b: return true; /* ⎋ */
    case 0x238c: return true; /* ⎌ */
    case 0x238d: return true; /* ⎍ */
    case 0x238e: return true; /* ⎎ */
    case 0x238f: return true; /* ⎏ */
    case 0x2390: return true; /* ⎐ */
    case 0x2391: return true; /* ⎑ */
    case 0x2392: return true; /* ⎒ */
    case 0x2393: return true; /* ⎓ */
    case 0x2394: return true; /* ⎔ */
    case 0x2395: return true; /* ⎕ */
    case 0x2396: return true; /* ⎖ */
    case 0x2397: return true; /* ⎗ */
    case 0x2398: return true; /* ⎘ */
    case 0x2399: return true; /* ⎙ */
    case 0x239a: return true; /* ⎚ */
    case 0x239b: return true; /* ⎛ */
    case 0x239c: return true; /* ⎜ */
    case 0x239d: return true; /* ⎝ */
    case 0x239e: return true; /* ⎞ */
    case 0x239f: return true; /* ⎟ */
    case 0x23a0: return true; /* ⎠ */
    case 0x23a1: return true; /* ⎡ */
    case 0x23a2: return true; /* ⎢ */
    case 0x23a3: return true; /* ⎣ */
    case 0x23a4: return true; /* ⎤ */
    case 0x23a5: return true; /* ⎥ */
    case 0x23a6: return true; /* ⎦ */
    case 0x23a7: return true; /* ⎧ */
    case 0x23a8: return true; /* ⎨ */
    case 0x23a9: return true; /* ⎩ */
    case 0x23aa: return true; /* ⎪ */
    case 0x23ab: return true; /* ⎫ */
    case 0x23ac: return true; /* ⎬ */
    case 0x23ad: return true; /* ⎭ */
    case 0x23ae: return true; /* ⎮ */
    case 0x23af: return true; /* ⎯ */
    case 0x23b0: return true; /* ⎰ */
    case 0x23b1: return true; /* ⎱ */
    case 0x23b2: return true; /* ⎲ */
    case 0x23b3: return true; /* ⎳ */
    case 0x23b4: return true; /* ⎴ */
    case 0x23b5: return true; /* ⎵ */
    case 0x23b6: return true; /* ⎶ */
    case 0x23b7: return true; /* ⎷ */
    case 0x23b8: return true; /* ⎸ */
    case 0x23b9: return true; /* ⎹ */
    case 0x23ba: return true; /* ⎺ */
    case 0x23bb: return true; /* ⎻ */
    case 0x23bc: return true; /* ⎼ */
    case 0x23bd: return true; /* ⎽ */
    case 0x23be: return true; /* ⎾ */
    case 0x23bf: return true; /* ⎿ */
    case 0x23c0: return true; /* ⏀ */
    case 0x23c1: return true; /* ⏁ */
    case 0x23c2: return true; /* ⏂ */
    case 0x23c3: return true; /* ⏃ */
    case 0x23c4: return true; /* ⏄ */
    case 0x23c5: return true; /* ⏅ */
    case 0x23c6: return true; /* ⏆ */
    case 0x23c7: return true; /* ⏇ */
    case 0x23c8: return true; /* ⏈ */
    case 0x23c9: return true; /* ⏉ */
    case 0x23ca: return true; /* ⏊ */
    case 0x23cb: return true; /* ⏋ */
    case 0x23cc: return true; /* ⏌ */
    case 0x23cd: return true; /* ⏍ */
    case 0x23ce: return true; /* ⏎ */
    case 0x23cf: return true; /* ⏏ */
    case 0x23d0: return true; /* ⏐ */
    case 0x23d1: return true; /* ⏑ */
    case 0x23d2: return true; /* ⏒ */
    case 0x23d3: return true; /* ⏓ */
    case 0x23d4: return true; /* ⏔ */
    case 0x23d5: return true; /* ⏕ */
    case 0x23d6: return true; /* ⏖ */
    case 0x23d7: return true; /* ⏗ */
    case 0x23d8: return true; /* ⏘ */
    case 0x23d9: return true; /* ⏙ */
    case 0x23da: return true; /* ⏚ */
    case 0x23db: return true; /* ⏛ */
    case 0x23dc: return true; /* ⏜ */
    case 0x23dd: return true; /* ⏝ */
    case 0x23de: return true; /* ⏞ */
    case 0x23df: return true; /* ⏟ */
    case 0x23e0: return true; /* ⏠ */
    case 0x23e1: return true; /* ⏡ */
    case 0x23e2: return true; /* ⏢ */
    case 0x23e3: return true; /* ⏣ */
    case 0x23e4: return true; /* ⏤ */
    case 0x23e5: return true; /* ⏥ */
    case 0x23e6: return true; /* ⏦ */
    case 0x23e7: return true; /* ⏧ */
    case 0x23e8: return true; /* ⏨ */
    case 0x23e9: return true; /* ⏩ */
    case 0x23ea: return true; /* ⏪ */
    case 0x23eb: return true; /* ⏫ */
    case 0x23ec: return true; /* ⏬ */
    case 0x23ed: return true; /* ⏭ */
    case 0x23ee: return true; /* ⏮ */
    case 0x23ef: return true; /* ⏯ */
    case 0x23f0: return true; /* ⏰ */
    case 0x23f1: return true; /* ⏱ */
    case 0x23f2: return true; /* ⏲ */
    case 0x23f3: return true; /* ⏳ */
    case 0x23f4: return true; /* ⏴ */
    case 0x23f5: return true; /* ⏵ */
    case 0x23f6: return true; /* ⏶ */
    case 0x23f7: return true; /* ⏷ */
    case 0x23f8: return true; /* ⏸ */
    case 0x23f9: return true; /* ⏹ */
    case 0x23fa: return true; /* ⏺ */
    case 0x23fb: return true; /* ⏻ */
    case 0x23fc: return true; /* ⏼ */
    case 0x23fd: return true; /* ⏽ */
    case 0x23fe: return true; /* ⏾ */
    case 0x23ff: return true; /* ⏿ */
    case 0x2400: return true; /* ␀ */
    case 0x2401: return true; /* ␁ */
    case 0x2402: return true; /* ␂ */
    case 0x2403: return true; /* ␃ */
    case 0x2404: return true; /* ␄ */
    case 0x2405: return true; /* ␅ */
    case 0x2406: return true; /* ␆ */
    case 0x2407: return true; /* ␇ */
    case 0x2408: return true; /* ␈ */
    case 0x2409: return true; /* ␉ */
    case 0x240a: return true; /* ␊ */
    case 0x240b: return true; /* ␋ */
    case 0x240c: return true; /* ␌ */
    case 0x240d: return true; /* ␍ */
    case 0x240e: return true; /* ␎ */
    case 0x240f: return true; /* ␏ */
    case 0x2410: return true; /* ␐ */
    case 0x2411: return true; /* ␑ */
    case 0x2412: return true; /* ␒ */
    case 0x2413: return true; /* ␓ */
    case 0x2414: return true; /* ␔ */
    case 0x2415: return true; /* ␕ */
    case 0x2416: return true; /* ␖ */
    case 0x2417: return true; /* ␗ */
    case 0x2418: return true; /* ␘ */
    case 0x2419: return true; /* ␙ */
    case 0x241a: return true; /* ␚ */
    case 0x241b: return true; /* ␛ */
    case 0x241c: return true; /* ␜ */
    case 0x241d: return true; /* ␝ */
    case 0x241e: return true; /* ␞ */
    case 0x241f: return true; /* ␟ */
    case 0x2420: return true; /* ␠ */
    case 0x2421: return true; /* ␡ */
    case 0x2422: return true; /* ␢ */
    case 0x2423: return true; /* ␣ */
    case 0x2424: return true; /* ␤ */
    case 0x2425: return true; /* ␥ */
    case 0x2426: return true; /* ␦ */
    case 0x2440: return true; /* ⑀ */
    case 0x2441: return true; /* ⑁ */
    case 0x2442: return true; /* ⑂ */
    case 0x2443: return true; /* ⑃ */
    case 0x2444: return true; /* ⑄ */
    case 0x2445: return true; /* ⑅ */
    case 0x2446: return true; /* ⑆ */
    case 0x2447: return true; /* ⑇ */
    case 0x2448: return true; /* ⑈ */
    case 0x2449: return true; /* ⑉ */
    case 0x244a: return true; /* ⑊ */
    case 0x249c: return true; /* ⒜ */
    case 0x249d: return true; /* ⒝ */
    case 0x249e: return true; /* ⒞ */
    case 0x249f: return true; /* ⒟ */
    case 0x24a0: return true; /* ⒠ */
    case 0x24a1: return true; /* ⒡ */
    case 0x24a2: return true; /* ⒢ */
    case 0x24a3: return true; /* ⒣ */
    case 0x24a4: return true; /* ⒤ */
    case 0x24a5: return true; /* ⒥ */
    case 0x24a6: return true; /* ⒦ */
    case 0x24a7: return true; /* ⒧ */
    case 0x24a8: return true; /* ⒨ */
    case 0x24a9: return true; /* ⒩ */
    case 0x24aa: return true; /* ⒪ */
    case 0x24ab: return true; /* ⒫ */
    case 0x24ac: return true; /* ⒬ */
    case 0x24ad: return true; /* ⒭ */
    case 0x24ae: return true; /* ⒮ */
    case 0x24af: return true; /* ⒯ */
    case 0x24b0: return true; /* ⒰ */
    case 0x24b1: return true; /* ⒱ */
    case 0x24b2: return true; /* ⒲ */
    case 0x24b3: return true; /* ⒳ */
    case 0x24b4: return true; /* ⒴ */
    case 0x24b5: return true; /* ⒵ */
    case 0x24b6: return true; /* Ⓐ */
    case 0x24b7: return true; /* Ⓑ */
    case 0x24b8: return true; /* Ⓒ */
    case 0x24b9: return true; /* Ⓓ */
    case 0x24ba: return true; /* Ⓔ */
    case 0x24bb: return true; /* Ⓕ */
    case 0x24bc: return true; /* Ⓖ */
    case 0x24bd: return true; /* Ⓗ */
    case 0x24be: return true; /* Ⓘ */
    case 0x24bf: return true; /* Ⓙ */
    case 0x24c0: return true; /* Ⓚ */
    case 0x24c1: return true; /* Ⓛ */
    case 0x24c2: return true; /* Ⓜ */
    case 0x24c3: return true; /* Ⓝ */
    case 0x24c4: return true; /* Ⓞ */
    case 0x24c5: return true; /* Ⓟ */
    case 0x24c6: return true; /* Ⓠ */
    case 0x24c7: return true; /* Ⓡ */
    case 0x24c8: return true; /* Ⓢ */
    case 0x24c9: return true; /* Ⓣ */
    case 0x24ca: return true; /* Ⓤ */
    case 0x24cb: return true; /* Ⓥ */
    case 0x24cc: return true; /* Ⓦ */
    case 0x24cd: return true; /* Ⓧ */
    case 0x24ce: return true; /* Ⓨ */
    case 0x24cf: return true; /* Ⓩ */
    case 0x24d0: return true; /* ⓐ */
    case 0x24d1: return true; /* ⓑ */
    case 0x24d2: return true; /* ⓒ */
    case 0x24d3: return true; /* ⓓ */
    case 0x24d4: return true; /* ⓔ */
    case 0x24d5: return true; /* ⓕ */
    case 0x24d6: return true; /* ⓖ */
    case 0x24d7: return true; /* ⓗ */
    case 0x24d8: return true; /* ⓘ */
    case 0x24d9: return true; /* ⓙ */
    case 0x24da: return true; /* ⓚ */
    case 0x24db: return true; /* ⓛ */
    case 0x24dc: return true; /* ⓜ */
    case 0x24dd: return true; /* ⓝ */
    case 0x24de: return true; /* ⓞ */
    case 0x24df: return true; /* ⓟ */
    case 0x24e0: return true; /* ⓠ */
    case 0x24e1: return true; /* ⓡ */
    case 0x24e2: return true; /* ⓢ */
    case 0x24e3: return true; /* ⓣ */
    case 0x24e4: return true; /* ⓤ */
    case 0x24e5: return true; /* ⓥ */
    case 0x24e6: return true; /* ⓦ */
    case 0x24e7: return true; /* ⓧ */
    case 0x24e8: return true; /* ⓨ */
    case 0x24e9: return true; /* ⓩ */
    case 0x2500: return true; /* ─ */
    case 0x2501: return true; /* ━ */
    case 0x2502: return true; /* │ */
    case 0x2503: return true; /* ┃ */
    case 0x2504: return true; /* ┄ */
    case 0x2505: return true; /* ┅ */
    case 0x2506: return true; /* ┆ */
    case 0x2507: return true; /* ┇ */
    case 0x2508: return true; /* ┈ */
    case 0x2509: return true; /* ┉ */
    case 0x250a: return true; /* ┊ */
    case 0x250b: return true; /* ┋ */
    case 0x250c: return true; /* ┌ */
    case 0x250d: return true; /* ┍ */
    case 0x250e: return true; /* ┎ */
    case 0x250f: return true; /* ┏ */
    case 0x2510: return true; /* ┐ */
    case 0x2511: return true; /* ┑ */
    case 0x2512: return true; /* ┒ */
    case 0x2513: return true; /* ┓ */
    case 0x2514: return true; /* └ */
    case 0x2515: return true; /* ┕ */
    case 0x2516: return true; /* ┖ */
    case 0x2517: return true; /* ┗ */
    case 0x2518: return true; /* ┘ */
    case 0x2519: return true; /* ┙ */
    case 0x251a: return true; /* ┚ */
    case 0x251b: return true; /* ┛ */
    case 0x251c: return true; /* ├ */
    case 0x251d: return true; /* ┝ */
    case 0x251e: return true; /* ┞ */
    case 0x251f: return true; /* ┟ */
    case 0x2520: return true; /* ┠ */
    case 0x2521: return true; /* ┡ */
    case 0x2522: return true; /* ┢ */
    case 0x2523: return true; /* ┣ */
    case 0x2524: return true; /* ┤ */
    case 0x2525: return true; /* ┥ */
    case 0x2526: return true; /* ┦ */
    case 0x2527: return true; /* ┧ */
    case 0x2528: return true; /* ┨ */
    case 0x2529: return true; /* ┩ */
    case 0x252a: return true; /* ┪ */
    case 0x252b: return true; /* ┫ */
    case 0x252c: return true; /* ┬ */
    case 0x252d: return true; /* ┭ */
    case 0x252e: return true; /* ┮ */
    case 0x252f: return true; /* ┯ */
    case 0x2530: return true; /* ┰ */
    case 0x2531: return true; /* ┱ */
    case 0x2532: return true; /* ┲ */
    case 0x2533: return true; /* ┳ */
    case 0x2534: return true; /* ┴ */
    case 0x2535: return true; /* ┵ */
    case 0x2536: return true; /* ┶ */
    case 0x2537: return true; /* ┷ */
    case 0x2538: return true; /* ┸ */
    case 0x2539: return true; /* ┹ */
    case 0x253a: return true; /* ┺ */
    case 0x253b: return true; /* ┻ */
    case 0x253c: return true; /* ┼ */
    case 0x253d: return true; /* ┽ */
    case 0x253e: return true; /* ┾ */
    case 0x253f: return true; /* ┿ */
    case 0x2540: return true; /* ╀ */
    case 0x2541: return true; /* ╁ */
    case 0x2542: return true; /* ╂ */
    case 0x2543: return true; /* ╃ */
    case 0x2544: return true; /* ╄ */
    case 0x2545: return true; /* ╅ */
    case 0x2546: return true; /* ╆ */
    case 0x2547: return true; /* ╇ */
    case 0x2548: return true; /* ╈ */
    case 0x2549: return true; /* ╉ */
    case 0x254a: return true; /* ╊ */
    case 0x254b: return true; /* ╋ */
    case 0x254c: return true; /* ╌ */
    case 0x254d: return true; /* ╍ */
    case 0x254e: return true; /* ╎ */
    case 0x254f: return true; /* ╏ */
    case 0x2550: return true; /* ═ */
    case 0x2551: return true; /* ║ */
    case 0x2552: return true; /* ╒ */
    case 0x2553: return true; /* ╓ */
    case 0x2554: return true; /* ╔ */
    case 0x2555: return true; /* ╕ */
    case 0x2556: return true; /* ╖ */
    case 0x2557: return true; /* ╗ */
    case 0x2558: return true; /* ╘ */
    case 0x2559: return true; /* ╙ */
    case 0x255a: return true; /* ╚ */
    case 0x255b: return true; /* ╛ */
    case 0x255c: return true; /* ╜ */
    case 0x255d: return true; /* ╝ */
    case 0x255e: return true; /* ╞ */
    case 0x255f: return true; /* ╟ */
    case 0x2560: return true; /* ╠ */
    case 0x2561: return true; /* ╡ */
    case 0x2562: return true; /* ╢ */
    case 0x2563: return true; /* ╣ */
    case 0x2564: return true; /* ╤ */
    case 0x2565: return true; /* ╥ */
    case 0x2566: return true; /* ╦ */
    case 0x2567: return true; /* ╧ */
    case 0x2568: return true; /* ╨ */
    case 0x2569: return true; /* ╩ */
    case 0x256a: return true; /* ╪ */
    case 0x256b: return true; /* ╫ */
    case 0x256c: return true; /* ╬ */
    case 0x256d: return true; /* ╭ */
    case 0x256e: return true; /* ╮ */
    case 0x256f: return true; /* ╯ */
    case 0x2570: return true; /* ╰ */
    case 0x2571: return true; /* ╱ */
    case 0x2572: return true; /* ╲ */
    case 0x2573: return true; /* ╳ */
    case 0x2574: return true; /* ╴ */
    case 0x2575: return true; /* ╵ */
    case 0x2576: return true; /* ╶ */
    case 0x2577: return true; /* ╷ */
    case 0x2578: return true; /* ╸ */
    case 0x2579: return true; /* ╹ */
    case 0x257a: return true; /* ╺ */
    case 0x257b: return true; /* ╻ */
    case 0x257c: return true; /* ╼ */
    case 0x257d: return true; /* ╽ */
    case 0x257e: return true; /* ╾ */
    case 0x257f: return true; /* ╿ */
    case 0x2580: return true; /* ▀ */
    case 0x2581: return true; /* ▁ */
    case 0x2582: return true; /* ▂ */
    case 0x2583: return true; /* ▃ */
    case 0x2584: return true; /* ▄ */
    case 0x2585: return true; /* ▅ */
    case 0x2586: return true; /* ▆ */
    case 0x2587: return true; /* ▇ */
    case 0x2588: return true; /* █ */
    case 0x2589: return true; /* ▉ */
    case 0x258a: return true; /* ▊ */
    case 0x258b: return true; /* ▋ */
    case 0x258c: return true; /* ▌ */
    case 0x258d: return true; /* ▍ */
    case 0x258e: return true; /* ▎ */
    case 0x258f: return true; /* ▏ */
    case 0x2590: return true; /* ▐ */
    case 0x2591: return true; /* ░ */
    case 0x2592: return true; /* ▒ */
    case 0x2593: return true; /* ▓ */
    case 0x2594: return true; /* ▔ */
    case 0x2595: return true; /* ▕ */
    case 0x2596: return true; /* ▖ */
    case 0x2597: return true; /* ▗ */
    case 0x2598: return true; /* ▘ */
    case 0x2599: return true; /* ▙ */
    case 0x259a: return true; /* ▚ */
    case 0x259b: return true; /* ▛ */
    case 0x259c: return true; /* ▜ */
    case 0x259d: return true; /* ▝ */
    case 0x259e: return true; /* ▞ */
    case 0x259f: return true; /* ▟ */
    case 0x25a0: return true; /* ■ */
    case 0x25a1: return true; /* □ */
    case 0x25a2: return true; /* ▢ */
    case 0x25a3: return true; /* ▣ */
    case 0x25a4: return true; /* ▤ */
    case 0x25a5: return true; /* ▥ */
    case 0x25a6: return true; /* ▦ */
    case 0x25a7: return true; /* ▧ */
    case 0x25a8: return true; /* ▨ */
    case 0x25a9: return true; /* ▩ */
    case 0x25aa: return true; /* ▪ */
    case 0x25ab: return true; /* ▫ */
    case 0x25ac: return true; /* ▬ */
    case 0x25ad: return true; /* ▭ */
    case 0x25ae: return true; /* ▮ */
    case 0x25af: return true; /* ▯ */
    case 0x25b0: return true; /* ▰ */
    case 0x25b1: return true; /* ▱ */
    case 0x25b2: return true; /* ▲ */
    case 0x25b3: return true; /* △ */
    case 0x25b4: return true; /* ▴ */
    case 0x25b5: return true; /* ▵ */
    case 0x25b6: return true; /* ▶ */
    case 0x25b7: return true; /* ▷ */
    case 0x25b8: return true; /* ▸ */
    case 0x25b9: return true; /* ▹ */
    case 0x25ba: return true; /* ► */
    case 0x25bb: return true; /* ▻ */
    case 0x25bc: return true; /* ▼ */
    case 0x25bd: return true; /* ▽ */
    case 0x25be: return true; /* ▾ */
    case 0x25bf: return true; /* ▿ */
    case 0x25c0: return true; /* ◀ */
    case 0x25c1: return true; /* ◁ */
    case 0x25c2: return true; /* ◂ */
    case 0x25c3: return true; /* ◃ */
    case 0x25c4: return true; /* ◄ */
    case 0x25c5: return true; /* ◅ */
    case 0x25c6: return true; /* ◆ */
    case 0x25c7: return true; /* ◇ */
    case 0x25c8: return true; /* ◈ */
    case 0x25c9: return true; /* ◉ */
    case 0x25ca: return true; /* ◊ */
    case 0x25cb: return true; /* ○ */
    case 0x25cc: return true; /* ◌ */
    case 0x25cd: return true; /* ◍ */
    case 0x25ce: return true; /* ◎ */
    case 0x25cf: return true; /* ● */
    case 0x25d0: return true; /* ◐ */
    case 0x25d1: return true; /* ◑ */
    case 0x25d2: return true; /* ◒ */
    case 0x25d3: return true; /* ◓ */
    case 0x25d4: return true; /* ◔ */
    case 0x25d5: return true; /* ◕ */
    case 0x25d6: return true; /* ◖ */
    case 0x25d7: return true; /* ◗ */
    case 0x25d8: return true; /* ◘ */
    case 0x25d9: return true; /* ◙ */
    case 0x25da: return true; /* ◚ */
    case 0x25db: return true; /* ◛ */
    case 0x25dc: return true; /* ◜ */
    case 0x25dd: return true; /* ◝ */
    case 0x25de: return true; /* ◞ */
    case 0x25df: return true; /* ◟ */
    case 0x25e0: return true; /* ◠ */
    case 0x25e1: return true; /* ◡ */
    case 0x25e2: return true; /* ◢ */
    case 0x25e3: return true; /* ◣ */
    case 0x25e4: return true; /* ◤ */
    case 0x25e5: return true; /* ◥ */
    case 0x25e6: return true; /* ◦ */
    case 0x25e7: return true; /* ◧ */
    case 0x25e8: return true; /* ◨ */
    case 0x25e9: return true; /* ◩ */
    case 0x25ea: return true; /* ◪ */
    case 0x25eb: return true; /* ◫ */
    case 0x25ec: return true; /* ◬ */
    case 0x25ed: return true; /* ◭ */
    case 0x25ee: return true; /* ◮ */
    case 0x25ef: return true; /* ◯ */
    case 0x25f0: return true; /* ◰ */
    case 0x25f1: return true; /* ◱ */
    case 0x25f2: return true; /* ◲ */
    case 0x25f3: return true; /* ◳ */
    case 0x25f4: return true; /* ◴ */
    case 0x25f5: return true; /* ◵ */
    case 0x25f6: return true; /* ◶ */
    case 0x25f7: return true; /* ◷ */
    case 0x25f8: return true; /* ◸ */
    case 0x25f9: return true; /* ◹ */
    case 0x25fa: return true; /* ◺ */
    case 0x25fb: return true; /* ◻ */
    case 0x25fc: return true; /* ◼ */
    case 0x25fd: return true; /* ◽ */
    case 0x25fe: return true; /* ◾ */
    case 0x25ff: return true; /* ◿ */
    case 0x2600: return true; /* ☀ */
    case 0x2601: return true; /* ☁ */
    case 0x2602: return true; /* ☂ */
    case 0x2603: return true; /* ☃ */
    case 0x2604: return true; /* ☄ */
    case 0x2605: return true; /* ★ */
    case 0x2606: return true; /* ☆ */
    case 0x2607: return true; /* ☇ */
    case 0x2608: return true; /* ☈ */
    case 0x2609: return true; /* ☉ */
    case 0x260a: return true; /* ☊ */
    case 0x260b: return true; /* ☋ */
    case 0x260c: return true; /* ☌ */
    case 0x260d: return true; /* ☍ */
    case 0x260e: return true; /* ☎ */
    case 0x260f: return true; /* ☏ */
    case 0x2610: return true; /* ☐ */
    case 0x2611: return true; /* ☑ */
    case 0x2612: return true; /* ☒ */
    case 0x2613: return true; /* ☓ */
    case 0x2614: return true; /* ☔ */
    case 0x2615: return true; /* ☕ */
    case 0x2616: return true; /* ☖ */
    case 0x2617: return true; /* ☗ */
    case 0x2618: return true; /* ☘ */
    case 0x2619: return true; /* ☙ */
    case 0x261a: return true; /* ☚ */
    case 0x261b: return true; /* ☛ */
    case 0x261c: return true; /* ☜ */
    case 0x261d: return true; /* ☝ */
    case 0x261e: return true; /* ☞ */
    case 0x261f: return true; /* ☟ */
    case 0x2620: return true; /* ☠ */
    case 0x2621: return true; /* ☡ */
    case 0x2622: return true; /* ☢ */
    case 0x2623: return true; /* ☣ */
    case 0x2624: return true; /* ☤ */
    case 0x2625: return true; /* ☥ */
    case 0x2626: return true; /* ☦ */
    case 0x2627: return true; /* ☧ */
    case 0x2628: return true; /* ☨ */
    case 0x2629: return true; /* ☩ */
    case 0x262a: return true; /* ☪ */
    case 0x262b: return true; /* ☫ */
    case 0x262c: return true; /* ☬ */
    case 0x262d: return true; /* ☭ */
    case 0x262e: return true; /* ☮ */
    case 0x262f: return true; /* ☯ */
    case 0x2630: return true; /* ☰ */
    case 0x2631: return true; /* ☱ */
    case 0x2632: return true; /* ☲ */
    case 0x2633: return true; /* ☳ */
    case 0x2634: return true; /* ☴ */
    case 0x2635: return true; /* ☵ */
    case 0x2636: return true; /* ☶ */
    case 0x2637: return true; /* ☷ */
    case 0x2638: return true; /* ☸ */
    case 0x2639: return true; /* ☹ */
    case 0x263a: return true; /* ☺ */
    case 0x263b: return true; /* ☻ */
    case 0x263c: return true; /* ☼ */
    case 0x263d: return true; /* ☽ */
    case 0x263e: return true; /* ☾ */
    case 0x263f: return true; /* ☿ */
    case 0x2640: return true; /* ♀ */
    case 0x2641: return true; /* ♁ */
    case 0x2642: return true; /* ♂ */
    case 0x2643: return true; /* ♃ */
    case 0x2644: return true; /* ♄ */
    case 0x2645: return true; /* ♅ */
    case 0x2646: return true; /* ♆ */
    case 0x2647: return true; /* ♇ */
    case 0x2648: return true; /* ♈ */
    case 0x2649: return true; /* ♉ */
    case 0x264a: return true; /* ♊ */
    case 0x264b: return true; /* ♋ */
    case 0x264c: return true; /* ♌ */
    case 0x264d: return true; /* ♍ */
    case 0x264e: return true; /* ♎ */
    case 0x264f: return true; /* ♏ */
    case 0x2650: return true; /* ♐ */
    case 0x2651: return true; /* ♑ */
    case 0x2652: return true; /* ♒ */
    case 0x2653: return true; /* ♓ */
    case 0x2654: return true; /* ♔ */
    case 0x2655: return true; /* ♕ */
    case 0x2656: return true; /* ♖ */
    case 0x2657: return true; /* ♗ */
    case 0x2658: return true; /* ♘ */
    case 0x2659: return true; /* ♙ */
    case 0x265a: return true; /* ♚ */
    case 0x265b: return true; /* ♛ */
    case 0x265c: return true; /* ♜ */
    case 0x265d: return true; /* ♝ */
    case 0x265e: return true; /* ♞ */
    case 0x265f: return true; /* ♟ */
    case 0x2660: return true; /* ♠ */
    case 0x2661: return true; /* ♡ */
    case 0x2662: return true; /* ♢ */
    case 0x2663: return true; /* ♣ */
    case 0x2664: return true; /* ♤ */
    case 0x2665: return true; /* ♥ */
    case 0x2666: return true; /* ♦ */
    case 0x2667: return true; /* ♧ */
    case 0x2668: return true; /* ♨ */
    case 0x2669: return true; /* ♩ */
    case 0x266a: return true; /* ♪ */
    case 0x266b: return true; /* ♫ */
    case 0x266c: return true; /* ♬ */
    case 0x266d: return true; /* ♭ */
    case 0x266e: return true; /* ♮ */
    case 0x266f: return true; /* ♯ */
    case 0x2670: return true; /* ♰ */
    case 0x2671: return true; /* ♱ */
    case 0x2672: return true; /* ♲ */
    case 0x2673: return true; /* ♳ */
    case 0x2674: return true; /* ♴ */
    case 0x2675: return true; /* ♵ */
    case 0x2676: return true; /* ♶ */
    case 0x2677: return true; /* ♷ */
    case 0x2678: return true; /* ♸ */
    case 0x2679: return true; /* ♹ */
    case 0x267a: return true; /* ♺ */
    case 0x267b: return true; /* ♻ */
    case 0x267c: return true; /* ♼ */
    case 0x267d: return true; /* ♽ */
    case 0x267e: return true; /* ♾ */
    case 0x267f: return true; /* ♿ */
    case 0x2680: return true; /* ⚀ */
    case 0x2681: return true; /* ⚁ */
    case 0x2682: return true; /* ⚂ */
    case 0x2683: return true; /* ⚃ */
    case 0x2684: return true; /* ⚄ */
    case 0x2685: return true; /* ⚅ */
    case 0x2686: return true; /* ⚆ */
    case 0x2687: return true; /* ⚇ */
    case 0x2688: return true; /* ⚈ */
    case 0x2689: return true; /* ⚉ */
    case 0x268a: return true; /* ⚊ */
    case 0x268b: return true; /* ⚋ */
    case 0x268c: return true; /* ⚌ */
    case 0x268d: return true; /* ⚍ */
    case 0x268e: return true; /* ⚎ */
    case 0x268f: return true; /* ⚏ */
    case 0x2690: return true; /* ⚐ */
    case 0x2691: return true; /* ⚑ */
    case 0x2692: return true; /* ⚒ */
    case 0x2693: return true; /* ⚓ */
    case 0x2694: return true; /* ⚔ */
    case 0x2695: return true; /* ⚕ */
    case 0x2696: return true; /* ⚖ */
    case 0x2697: return true; /* ⚗ */
    case 0x2698: return true; /* ⚘ */
    case 0x2699: return true; /* ⚙ */
    case 0x269a: return true; /* ⚚ */
    case 0x269b: return true; /* ⚛ */
    case 0x269c: return true; /* ⚜ */
    case 0x269d: return true; /* ⚝ */
    case 0x269e: return true; /* ⚞ */
    case 0x269f: return true; /* ⚟ */
    case 0x26a0: return true; /* ⚠ */
    case 0x26a1: return true; /* ⚡ */
    case 0x26a2: return true; /* ⚢ */
    case 0x26a3: return true; /* ⚣ */
    case 0x26a4: return true; /* ⚤ */
    case 0x26a5: return true; /* ⚥ */
    case 0x26a6: return true; /* ⚦ */
    case 0x26a7: return true; /* ⚧ */
    case 0x26a8: return true; /* ⚨ */
    case 0x26a9: return true; /* ⚩ */
    case 0x26aa: return true; /* ⚪ */
    case 0x26ab: return true; /* ⚫ */
    case 0x26ac: return true; /* ⚬ */
    case 0x26ad: return true; /* ⚭ */
    case 0x26ae: return true; /* ⚮ */
    case 0x26af: return true; /* ⚯ */
    case 0x26b0: return true; /* ⚰ */
    case 0x26b1: return true; /* ⚱ */
    case 0x26b2: return true; /* ⚲ */
    case 0x26b3: return true; /* ⚳ */
    case 0x26b4: return true; /* ⚴ */
    case 0x26b5: return true; /* ⚵ */
    case 0x26b6: return true; /* ⚶ */
    case 0x26b7: return true; /* ⚷ */
    case 0x26b8: return true; /* ⚸ */
    case 0x26b9: return true; /* ⚹ */
    case 0x26ba: return true; /* ⚺ */
    case 0x26bb: return true; /* ⚻ */
    case 0x26bc: return true; /* ⚼ */
    case 0x26bd: return true; /* ⚽ */
    case 0x26be: return true; /* ⚾ */
    case 0x26bf: return true; /* ⚿ */
    case 0x26c0: return true; /* ⛀ */
    case 0x26c1: return true; /* ⛁ */
    case 0x26c2: return true; /* ⛂ */
    case 0x26c3: return true; /* ⛃ */
    case 0x26c4: return true; /* ⛄ */
    case 0x26c5: return true; /* ⛅ */
    case 0x26c6: return true; /* ⛆ */
    case 0x26c7: return true; /* ⛇ */
    case 0x26c8: return true; /* ⛈ */
    case 0x26c9: return true; /* ⛉ */
    case 0x26ca: return true; /* ⛊ */
    case 0x26cb: return true; /* ⛋ */
    case 0x26cc: return true; /* ⛌ */
    case 0x26cd: return true; /* ⛍ */
    case 0x26ce: return true; /* ⛎ */
    case 0x26cf: return true; /* ⛏ */
    case 0x26d0: return true; /* ⛐ */
    case 0x26d1: return true; /* ⛑ */
    case 0x26d2: return true; /* ⛒ */
    case 0x26d3: return true; /* ⛓ */
    case 0x26d4: return true; /* ⛔ */
    case 0x26d5: return true; /* ⛕ */
    case 0x26d6: return true; /* ⛖ */
    case 0x26d7: return true; /* ⛗ */
    case 0x26d8: return true; /* ⛘ */
    case 0x26d9: return true; /* ⛙ */
    case 0x26da: return true; /* ⛚ */
    case 0x26db: return true; /* ⛛ */
    case 0x26dc: return true; /* ⛜ */
    case 0x26dd: return true; /* ⛝ */
    case 0x26de: return true; /* ⛞ */
    case 0x26df: return true; /* ⛟ */
    case 0x26e0: return true; /* ⛠ */
    case 0x26e1: return true; /* ⛡ */
    case 0x26e2: return true; /* ⛢ */
    case 0x26e3: return true; /* ⛣ */
    case 0x26e4: return true; /* ⛤ */
    case 0x26e5: return true; /* ⛥ */
    case 0x26e6: return true; /* ⛦ */
    case 0x26e7: return true; /* ⛧ */
    case 0x26e8: return true; /* ⛨ */
    case 0x26e9: return true; /* ⛩ */
    case 0x26ea: return true; /* ⛪ */
    case 0x26eb: return true; /* ⛫ */
    case 0x26ec: return true; /* ⛬ */
    case 0x26ed: return true; /* ⛭ */
    case 0x26ee: return true; /* ⛮ */
    case 0x26ef: return true; /* ⛯ */
    case 0x26f0: return true; /* ⛰ */
    case 0x26f1: return true; /* ⛱ */
    case 0x26f2: return true; /* ⛲ */
    case 0x26f3: return true; /* ⛳ */
    case 0x26f4: return true; /* ⛴ */
    case 0x26f5: return true; /* ⛵ */
    case 0x26f6: return true; /* ⛶ */
    case 0x26f7: return true; /* ⛷ */
    case 0x26f8: return true; /* ⛸ */
    case 0x26f9: return true; /* ⛹ */
    case 0x26fa: return true; /* ⛺ */
    case 0x26fb: return true; /* ⛻ */
    case 0x26fc: return true; /* ⛼ */
    case 0x26fd: return true; /* ⛽ */
    case 0x26fe: return true; /* ⛾ */
    case 0x26ff: return true; /* ⛿ */
    case 0x2700: return true; /* ✀ */
    case 0x2701: return true; /* ✁ */
    case 0x2702: return true; /* ✂ */
    case 0x2703: return true; /* ✃ */
    case 0x2704: return true; /* ✄ */
    case 0x2705: return true; /* ✅ */
    case 0x2706: return true; /* ✆ */
    case 0x2707: return true; /* ✇ */
    case 0x2708: return true; /* ✈ */
    case 0x2709: return true; /* ✉ */
    case 0x270a: return true; /* ✊ */
    case 0x270b: return true; /* ✋ */
    case 0x270c: return true; /* ✌ */
    case 0x270d: return true; /* ✍ */
    case 0x270e: return true; /* ✎ */
    case 0x270f: return true; /* ✏ */
    case 0x2710: return true; /* ✐ */
    case 0x2711: return true; /* ✑ */
    case 0x2712: return true; /* ✒ */
    case 0x2713: return true; /* ✓ */
    case 0x2714: return true; /* ✔ */
    case 0x2715: return true; /* ✕ */
    case 0x2716: return true; /* ✖ */
    case 0x2717: return true; /* ✗ */
    case 0x2718: return true; /* ✘ */
    case 0x2719: return true; /* ✙ */
    case 0x271a: return true; /* ✚ */
    case 0x271b: return true; /* ✛ */
    case 0x271c: return true; /* ✜ */
    case 0x271d: return true; /* ✝ */
    case 0x271e: return true; /* ✞ */
    case 0x271f: return true; /* ✟ */
    case 0x2720: return true; /* ✠ */
    case 0x2721: return true; /* ✡ */
    case 0x2722: return true; /* ✢ */
    case 0x2723: return true; /* ✣ */
    case 0x2724: return true; /* ✤ */
    case 0x2725: return true; /* ✥ */
    case 0x2726: return true; /* ✦ */
    case 0x2727: return true; /* ✧ */
    case 0x2728: return true; /* ✨ */
    case 0x2729: return true; /* ✩ */
    case 0x272a: return true; /* ✪ */
    case 0x272b: return true; /* ✫ */
    case 0x272c: return true; /* ✬ */
    case 0x272d: return true; /* ✭ */
    case 0x272e: return true; /* ✮ */
    case 0x272f: return true; /* ✯ */
    case 0x2730: return true; /* ✰ */
    case 0x2731: return true; /* ✱ */
    case 0x2732: return true; /* ✲ */
    case 0x2733: return true; /* ✳ */
    case 0x2734: return true; /* ✴ */
    case 0x2735: return true; /* ✵ */
    case 0x2736: return true; /* ✶ */
    case 0x2737: return true; /* ✷ */
    case 0x2738: return true; /* ✸ */
    case 0x2739: return true; /* ✹ */
    case 0x273a: return true; /* ✺ */
    case 0x273b: return true; /* ✻ */
    case 0x273c: return true; /* ✼ */
    case 0x273d: return true; /* ✽ */
    case 0x273e: return true; /* ✾ */
    case 0x273f: return true; /* ✿ */
    case 0x2740: return true; /* ❀ */
    case 0x2741: return true; /* ❁ */
    case 0x2742: return true; /* ❂ */
    case 0x2743: return true; /* ❃ */
    case 0x2744: return true; /* ❄ */
    case 0x2745: return true; /* ❅ */
    case 0x2746: return true; /* ❆ */
    case 0x2747: return true; /* ❇ */
    case 0x2748: return true; /* ❈ */
    case 0x2749: return true; /* ❉ */
    case 0x274a: return true; /* ❊ */
    case 0x274b: return true; /* ❋ */
    case 0x274c: return true; /* ❌ */
    case 0x274d: return true; /* ❍ */
    case 0x274e: return true; /* ❎ */
    case 0x274f: return true; /* ❏ */
    case 0x2750: return true; /* ❐ */
    case 0x2751: return true; /* ❑ */
    case 0x2752: return true; /* ❒ */
    case 0x2753: return true; /* ❓ */
    case 0x2754: return true; /* ❔ */
    case 0x2755: return true; /* ❕ */
    case 0x2756: return true; /* ❖ */
    case 0x2757: return true; /* ❗ */
    case 0x2758: return true; /* ❘ */
    case 0x2759: return true; /* ❙ */
    case 0x275a: return true; /* ❚ */
    case 0x275b: return true; /* ❛ */
    case 0x275c: return true; /* ❜ */
    case 0x275d: return true; /* ❝ */
    case 0x275e: return true; /* ❞ */
    case 0x275f: return true; /* ❟ */
    case 0x2760: return true; /* ❠ */
    case 0x2761: return true; /* ❡ */
    case 0x2762: return true; /* ❢ */
    case 0x2763: return true; /* ❣ */
    case 0x2764: return true; /* ❤ */
    case 0x2765: return true; /* ❥ */
    case 0x2766: return true; /* ❦ */
    case 0x2767: return true; /* ❧ */
    case 0x2768: return true; /* ❨ */
    case 0x2769: return true; /* ❩ */
    case 0x276a: return true; /* ❪ */
    case 0x276b: return true; /* ❫ */
    case 0x276c: return true; /* ❬ */
    case 0x276d: return true; /* ❭ */
    case 0x276e: return true; /* ❮ */
    case 0x276f: return true; /* ❯ */
    case 0x2770: return true; /* ❰ */
    case 0x2771: return true; /* ❱ */
    case 0x2772: return true; /* ❲ */
    case 0x2773: return true; /* ❳ */
    case 0x2774: return true; /* ❴ */
    case 0x2775: return true; /* ❵ */
    case 0x2794: return true; /* ➔ */
    case 0x2795: return true; /* ➕ */
    case 0x2796: return true; /* ➖ */
    case 0x2797: return true; /* ➗ */
    case 0x2798: return true; /* ➘ */
    case 0x2799: return true; /* ➙ */
    case 0x279a: return true; /* ➚ */
    case 0x279b: return true; /* ➛ */
    case 0x279c: return true; /* ➜ */
    case 0x279d: return true; /* ➝ */
    case 0x279e: return true; /* ➞ */
    case 0x279f: return true; /* ➟ */
    case 0x27a0: return true; /* ➠ */
    case 0x27a1: return true; /* ➡ */
    case 0x27a2: return true; /* ➢ */
    case 0x27a3: return true; /* ➣ */
    case 0x27a4: return true; /* ➤ */
    case 0x27a5: return true; /* ➥ */
    case 0x27a6: return true; /* ➦ */
    case 0x27a7: return true; /* ➧ */
    case 0x27a8: return true; /* ➨ */
    case 0x27a9: return true; /* ➩ */
    case 0x27aa: return true; /* ➪ */
    case 0x27ab: return true; /* ➫ */
    case 0x27ac: return true; /* ➬ */
    case 0x27ad: return true; /* ➭ */
    case 0x27ae: return true; /* ➮ */
    case 0x27af: return true; /* ➯ */
    case 0x27b0: return true; /* ➰ */
    case 0x27b1: return true; /* ➱ */
    case 0x27b2: return true; /* ➲ */
    case 0x27b3: return true; /* ➳ */
    case 0x27b4: return true; /* ➴ */
    case 0x27b5: return true; /* ➵ */
    case 0x27b6: return true; /* ➶ */
    case 0x27b7: return true; /* ➷ */
    case 0x27b8: return true; /* ➸ */
    case 0x27b9: return true; /* ➹ */
    case 0x27ba: return true; /* ➺ */
    case 0x27bb: return true; /* ➻ */
    case 0x27bc: return true; /* ➼ */
    case 0x27bd: return true; /* ➽ */
    case 0x27be: return true; /* ➾ */
    case 0x27bf: return true; /* ➿ */
    case 0x27c0: return true; /* ⟀ */
    case 0x27c1: return true; /* ⟁ */
    case 0x27c2: return true; /* ⟂ */
    case 0x27c3: return true; /* ⟃ */
    case 0x27c4: return true; /* ⟄ */
    case 0x27c5: return true; /* ⟅ */
    case 0x27c6: return true; /* ⟆ */
    case 0x27c7: return true; /* ⟇ */
    case 0x27c8: return true; /* ⟈ */
    case 0x27c9: return true; /* ⟉ */
    case 0x27ca: return true; /* ⟊ */
    case 0x27cb: return true; /* ⟋ */
    case 0x27cc: return true; /* ⟌ */
    case 0x27cd: return true; /* ⟍ */
    case 0x27ce: return true; /* ⟎ */
    case 0x27cf: return true; /* ⟏ */
    case 0x27d0: return true; /* ⟐ */
    case 0x27d1: return true; /* ⟑ */
    case 0x27d2: return true; /* ⟒ */
    case 0x27d3: return true; /* ⟓ */
    case 0x27d4: return true; /* ⟔ */
    case 0x27d5: return true; /* ⟕ */
    case 0x27d6: return true; /* ⟖ */
    case 0x27d7: return true; /* ⟗ */
    case 0x27d8: return true; /* ⟘ */
    case 0x27d9: return true; /* ⟙ */
    case 0x27da: return true; /* ⟚ */
    case 0x27db: return true; /* ⟛ */
    case 0x27dc: return true; /* ⟜ */
    case 0x27dd: return true; /* ⟝ */
    case 0x27de: return true; /* ⟞ */
    case 0x27df: return true; /* ⟟ */
    case 0x27e0: return true; /* ⟠ */
    case 0x27e1: return true; /* ⟡ */
    case 0x27e2: return true; /* ⟢ */
    case 0x27e3: return true; /* ⟣ */
    case 0x27e4: return true; /* ⟤ */
    case 0x27e5: return true; /* ⟥ */
    case 0x27e6: return true; /* ⟦ */
    case 0x27e7: return true; /* ⟧ */
    case 0x27e8: return true; /* ⟨ */
    case 0x27e9: return true; /* ⟩ */
    case 0x27ea: return true; /* ⟪ */
    case 0x27eb: return true; /* ⟫ */
    case 0x27ec: return true; /* ⟬ */
    case 0x27ed: return true; /* ⟭ */
    case 0x27ee: return true; /* ⟮ */
    case 0x27ef: return true; /* ⟯ */
    case 0x27f0: return true; /* ⟰ */
    case 0x27f1: return true; /* ⟱ */
    case 0x27f2: return true; /* ⟲ */
    case 0x27f3: return true; /* ⟳ */
    case 0x27f4: return true; /* ⟴ */
    case 0x27f5: return true; /* ⟵ */
    case 0x27f6: return true; /* ⟶ */
    case 0x27f7: return true; /* ⟷ */
    case 0x27f8: return true; /* ⟸ */
    case 0x27f9: return true; /* ⟹ */
    case 0x27fa: return true; /* ⟺ */
    case 0x27fb: return true; /* ⟻ */
    case 0x27fc: return true; /* ⟼ */
    case 0x27fd: return true; /* ⟽ */
    case 0x27fe: return true; /* ⟾ */
    case 0x27ff: return true; /* ⟿ */
    case 0x2800: return true; /* ⠀ */
    case 0x2801: return true; /* ⠁ */
    case 0x2802: return true; /* ⠂ */
    case 0x2803: return true; /* ⠃ */
    case 0x2804: return true; /* ⠄ */
    case 0x2805: return true; /* ⠅ */
    case 0x2806: return true; /* ⠆ */
    case 0x2807: return true; /* ⠇ */
    case 0x2808: return true; /* ⠈ */
    case 0x2809: return true; /* ⠉ */
    case 0x280a: return true; /* ⠊ */
    case 0x280b: return true; /* ⠋ */
    case 0x280c: return true; /* ⠌ */
    case 0x280d: return true; /* ⠍ */
    case 0x280e: return true; /* ⠎ */
    case 0x280f: return true; /* ⠏ */
    case 0x2810: return true; /* ⠐ */
    case 0x2811: return true; /* ⠑ */
    case 0x2812: return true; /* ⠒ */
    case 0x2813: return true; /* ⠓ */
    case 0x2814: return true; /* ⠔ */
    case 0x2815: return true; /* ⠕ */
    case 0x2816: return true; /* ⠖ */
    case 0x2817: return true; /* ⠗ */
    case 0x2818: return true; /* ⠘ */
    case 0x2819: return true; /* ⠙ */
    case 0x281a: return true; /* ⠚ */
    case 0x281b: return true; /* ⠛ */
    case 0x281c: return true; /* ⠜ */
    case 0x281d: return true; /* ⠝ */
    case 0x281e: return true; /* ⠞ */
    case 0x281f: return true; /* ⠟ */
    case 0x2820: return true; /* ⠠ */
    case 0x2821: return true; /* ⠡ */
    case 0x2822: return true; /* ⠢ */
    case 0x2823: return true; /* ⠣ */
    case 0x2824: return true; /* ⠤ */
    case 0x2825: return true; /* ⠥ */
    case 0x2826: return true; /* ⠦ */
    case 0x2827: return true; /* ⠧ */
    case 0x2828: return true; /* ⠨ */
    case 0x2829: return true; /* ⠩ */
    case 0x282a: return true; /* ⠪ */
    case 0x282b: return true; /* ⠫ */
    case 0x282c: return true; /* ⠬ */
    case 0x282d: return true; /* ⠭ */
    case 0x282e: return true; /* ⠮ */
    case 0x282f: return true; /* ⠯ */
    case 0x2830: return true; /* ⠰ */
    case 0x2831: return true; /* ⠱ */
    case 0x2832: return true; /* ⠲ */
    case 0x2833: return true; /* ⠳ */
    case 0x2834: return true; /* ⠴ */
    case 0x2835: return true; /* ⠵ */
    case 0x2836: return true; /* ⠶ */
    case 0x2837: return true; /* ⠷ */
    case 0x2838: return true; /* ⠸ */
    case 0x2839: return true; /* ⠹ */
    case 0x283a: return true; /* ⠺ */
    case 0x283b: return true; /* ⠻ */
    case 0x283c: return true; /* ⠼ */
    case 0x283d: return true; /* ⠽ */
    case 0x283e: return true; /* ⠾ */
    case 0x283f: return true; /* ⠿ */
    case 0x2840: return true; /* ⡀ */
    case 0x2841: return true; /* ⡁ */
    case 0x2842: return true; /* ⡂ */
    case 0x2843: return true; /* ⡃ */
    case 0x2844: return true; /* ⡄ */
    case 0x2845: return true; /* ⡅ */
    case 0x2846: return true; /* ⡆ */
    case 0x2847: return true; /* ⡇ */
    case 0x2848: return true; /* ⡈ */
    case 0x2849: return true; /* ⡉ */
    case 0x284a: return true; /* ⡊ */
    case 0x284b: return true; /* ⡋ */
    case 0x284c: return true; /* ⡌ */
    case 0x284d: return true; /* ⡍ */
    case 0x284e: return true; /* ⡎ */
    case 0x284f: return true; /* ⡏ */
    case 0x2850: return true; /* ⡐ */
    case 0x2851: return true; /* ⡑ */
    case 0x2852: return true; /* ⡒ */
    case 0x2853: return true; /* ⡓ */
    case 0x2854: return true; /* ⡔ */
    case 0x2855: return true; /* ⡕ */
    case 0x2856: return true; /* ⡖ */
    case 0x2857: return true; /* ⡗ */
    case 0x2858: return true; /* ⡘ */
    case 0x2859: return true; /* ⡙ */
    case 0x285a: return true; /* ⡚ */
    case 0x285b: return true; /* ⡛ */
    case 0x285c: return true; /* ⡜ */
    case 0x285d: return true; /* ⡝ */
    case 0x285e: return true; /* ⡞ */
    case 0x285f: return true; /* ⡟ */
    case 0x2860: return true; /* ⡠ */
    case 0x2861: return true; /* ⡡ */
    case 0x2862: return true; /* ⡢ */
    case 0x2863: return true; /* ⡣ */
    case 0x2864: return true; /* ⡤ */
    case 0x2865: return true; /* ⡥ */
    case 0x2866: return true; /* ⡦ */
    case 0x2867: return true; /* ⡧ */
    case 0x2868: return true; /* ⡨ */
    case 0x2869: return true; /* ⡩ */
    case 0x286a: return true; /* ⡪ */
    case 0x286b: return true; /* ⡫ */
    case 0x286c: return true; /* ⡬ */
    case 0x286d: return true; /* ⡭ */
    case 0x286e: return true; /* ⡮ */
    case 0x286f: return true; /* ⡯ */
    case 0x2870: return true; /* ⡰ */
    case 0x2871: return true; /* ⡱ */
    case 0x2872: return true; /* ⡲ */
    case 0x2873: return true; /* ⡳ */
    case 0x2874: return true; /* ⡴ */
    case 0x2875: return true; /* ⡵ */
    case 0x2876: return true; /* ⡶ */
    case 0x2877: return true; /* ⡷ */
    case 0x2878: return true; /* ⡸ */
    case 0x2879: return true; /* ⡹ */
    case 0x287a: return true; /* ⡺ */
    case 0x287b: return true; /* ⡻ */
    case 0x287c: return true; /* ⡼ */
    case 0x287d: return true; /* ⡽ */
    case 0x287e: return true; /* ⡾ */
    case 0x287f: return true; /* ⡿ */
    case 0x2880: return true; /* ⢀ */
    case 0x2881: return true; /* ⢁ */
    case 0x2882: return true; /* ⢂ */
    case 0x2883: return true; /* ⢃ */
    case 0x2884: return true; /* ⢄ */
    case 0x2885: return true; /* ⢅ */
    case 0x2886: return true; /* ⢆ */
    case 0x2887: return true; /* ⢇ */
    case 0x2888: return true; /* ⢈ */
    case 0x2889: return true; /* ⢉ */
    case 0x288a: return true; /* ⢊ */
    case 0x288b: return true; /* ⢋ */
    case 0x288c: return true; /* ⢌ */
    case 0x288d: return true; /* ⢍ */
    case 0x288e: return true; /* ⢎ */
    case 0x288f: return true; /* ⢏ */
    case 0x2890: return true; /* ⢐ */
    case 0x2891: return true; /* ⢑ */
    case 0x2892: return true; /* ⢒ */
    case 0x2893: return true; /* ⢓ */
    case 0x2894: return true; /* ⢔ */
    case 0x2895: return true; /* ⢕ */
    case 0x2896: return true; /* ⢖ */
    case 0x2897: return true; /* ⢗ */
    case 0x2898: return true; /* ⢘ */
    case 0x2899: return true; /* ⢙ */
    case 0x289a: return true; /* ⢚ */
    case 0x289b: return true; /* ⢛ */
    case 0x289c: return true; /* ⢜ */
    case 0x289d: return true; /* ⢝ */
    case 0x289e: return true; /* ⢞ */
    case 0x289f: return true; /* ⢟ */
    case 0x28a0: return true; /* ⢠ */
    case 0x28a1: return true; /* ⢡ */
    case 0x28a2: return true; /* ⢢ */
    case 0x28a3: return true; /* ⢣ */
    case 0x28a4: return true; /* ⢤ */
    case 0x28a5: return true; /* ⢥ */
    case 0x28a6: return true; /* ⢦ */
    case 0x28a7: return true; /* ⢧ */
    case 0x28a8: return true; /* ⢨ */
    case 0x28a9: return true; /* ⢩ */
    case 0x28aa: return true; /* ⢪ */
    case 0x28ab: return true; /* ⢫ */
    case 0x28ac: return true; /* ⢬ */
    case 0x28ad: return true; /* ⢭ */
    case 0x28ae: return true; /* ⢮ */
    case 0x28af: return true; /* ⢯ */
    case 0x28b0: return true; /* ⢰ */
    case 0x28b1: return true; /* ⢱ */
    case 0x28b2: return true; /* ⢲ */
    case 0x28b3: return true; /* ⢳ */
    case 0x28b4: return true; /* ⢴ */
    case 0x28b5: return true; /* ⢵ */
    case 0x28b6: return true; /* ⢶ */
    case 0x28b7: return true; /* ⢷ */
    case 0x28b8: return true; /* ⢸ */
    case 0x28b9: return true; /* ⢹ */
    case 0x28ba: return true; /* ⢺ */
    case 0x28bb: return true; /* ⢻ */
    case 0x28bc: return true; /* ⢼ */
    case 0x28bd: return true; /* ⢽ */
    case 0x28be: return true; /* ⢾ */
    case 0x28bf: return true; /* ⢿ */
    case 0x28c0: return true; /* ⣀ */
    case 0x28c1: return true; /* ⣁ */
    case 0x28c2: return true; /* ⣂ */
    case 0x28c3: return true; /* ⣃ */
    case 0x28c4: return true; /* ⣄ */
    case 0x28c5: return true; /* ⣅ */
    case 0x28c6: return true; /* ⣆ */
    case 0x28c7: return true; /* ⣇ */
    case 0x28c8: return true; /* ⣈ */
    case 0x28c9: return true; /* ⣉ */
    case 0x28ca: return true; /* ⣊ */
    case 0x28cb: return true; /* ⣋ */
    case 0x28cc: return true; /* ⣌ */
    case 0x28cd: return true; /* ⣍ */
    case 0x28ce: return true; /* ⣎ */
    case 0x28cf: return true; /* ⣏ */
    case 0x28d0: return true; /* ⣐ */
    case 0x28d1: return true; /* ⣑ */
    case 0x28d2: return true; /* ⣒ */
    case 0x28d3: return true; /* ⣓ */
    case 0x28d4: return true; /* ⣔ */
    case 0x28d5: return true; /* ⣕ */
    case 0x28d6: return true; /* ⣖ */
    case 0x28d7: return true; /* ⣗ */
    case 0x28d8: return true; /* ⣘ */
    case 0x28d9: return true; /* ⣙ */
    case 0x28da: return true; /* ⣚ */
    case 0x28db: return true; /* ⣛ */
    case 0x28dc: return true; /* ⣜ */
    case 0x28dd: return true; /* ⣝ */
    case 0x28de: return true; /* ⣞ */
    case 0x28df: return true; /* ⣟ */
    case 0x28e0: return true; /* ⣠ */
    case 0x28e1: return true; /* ⣡ */
    case 0x28e2: return true; /* ⣢ */
    case 0x28e3: return true; /* ⣣ */
    case 0x28e4: return true; /* ⣤ */
    case 0x28e5: return true; /* ⣥ */
    case 0x28e6: return true; /* ⣦ */
    case 0x28e7: return true; /* ⣧ */
    case 0x28e8: return true; /* ⣨ */
    case 0x28e9: return true; /* ⣩ */
    case 0x28ea: return true; /* ⣪ */
    case 0x28eb: return true; /* ⣫ */
    case 0x28ec: return true; /* ⣬ */
    case 0x28ed: return true; /* ⣭ */
    case 0x28ee: return true; /* ⣮ */
    case 0x28ef: return true; /* ⣯ */
    case 0x28f0: return true; /* ⣰ */
    case 0x28f1: return true; /* ⣱ */
    case 0x28f2: return true; /* ⣲ */
    case 0x28f3: return true; /* ⣳ */
    case 0x28f4: return true; /* ⣴ */
    case 0x28f5: return true; /* ⣵ */
    case 0x28f6: return true; /* ⣶ */
    case 0x28f7: return true; /* ⣷ */
    case 0x28f8: return true; /* ⣸ */
    case 0x28f9: return true; /* ⣹ */
    case 0x28fa: return true; /* ⣺ */
    case 0x28fb: return true; /* ⣻ */
    case 0x28fc: return true; /* ⣼ */
    case 0x28fd: return true; /* ⣽ */
    case 0x28fe: return true; /* ⣾ */
    case 0x28ff: return true; /* ⣿ */
    case 0x2900: return true; /* ⤀ */
    case 0x2901: return true; /* ⤁ */
    case 0x2902: return true; /* ⤂ */
    case 0x2903: return true; /* ⤃ */
    case 0x2904: return true; /* ⤄ */
    case 0x2905: return true; /* ⤅ */
    case 0x2906: return true; /* ⤆ */
    case 0x2907: return true; /* ⤇ */
    case 0x2908: return true; /* ⤈ */
    case 0x2909: return true; /* ⤉ */
    case 0x290a: return true; /* ⤊ */
    case 0x290b: return true; /* ⤋ */
    case 0x290c: return true; /* ⤌ */
    case 0x290d: return true; /* ⤍ */
    case 0x290e: return true; /* ⤎ */
    case 0x290f: return true; /* ⤏ */
    case 0x2910: return true; /* ⤐ */
    case 0x2911: return true; /* ⤑ */
    case 0x2912: return true; /* ⤒ */
    case 0x2913: return true; /* ⤓ */
    case 0x2914: return true; /* ⤔ */
    case 0x2915: return true; /* ⤕ */
    case 0x2916: return true; /* ⤖ */
    case 0x2917: return true; /* ⤗ */
    case 0x2918: return true; /* ⤘ */
    case 0x2919: return true; /* ⤙ */
    case 0x291a: return true; /* ⤚ */
    case 0x291b: return true; /* ⤛ */
    case 0x291c: return true; /* ⤜ */
    case 0x291d: return true; /* ⤝ */
    case 0x291e: return true; /* ⤞ */
    case 0x291f: return true; /* ⤟ */
    case 0x2920: return true; /* ⤠ */
    case 0x2921: return true; /* ⤡ */
    case 0x2922: return true; /* ⤢ */
    case 0x2923: return true; /* ⤣ */
    case 0x2924: return true; /* ⤤ */
    case 0x2925: return true; /* ⤥ */
    case 0x2926: return true; /* ⤦ */
    case 0x2927: return true; /* ⤧ */
    case 0x2928: return true; /* ⤨ */
    case 0x2929: return true; /* ⤩ */
    case 0x292a: return true; /* ⤪ */
    case 0x292b: return true; /* ⤫ */
    case 0x292c: return true; /* ⤬ */
    case 0x292d: return true; /* ⤭ */
    case 0x292e: return true; /* ⤮ */
    case 0x292f: return true; /* ⤯ */
    case 0x2930: return true; /* ⤰ */
    case 0x2931: return true; /* ⤱ */
    case 0x2932: return true; /* ⤲ */
    case 0x2933: return true; /* ⤳ */
    case 0x2934: return true; /* ⤴ */
    case 0x2935: return true; /* ⤵ */
    case 0x2936: return true; /* ⤶ */
    case 0x2937: return true; /* ⤷ */
    case 0x2938: return true; /* ⤸ */
    case 0x2939: return true; /* ⤹ */
    case 0x293a: return true; /* ⤺ */
    case 0x293b: return true; /* ⤻ */
    case 0x293c: return true; /* ⤼ */
    case 0x293d: return true; /* ⤽ */
    case 0x293e: return true; /* ⤾ */
    case 0x293f: return true; /* ⤿ */
    case 0x2940: return true; /* ⥀ */
    case 0x2941: return true; /* ⥁ */
    case 0x2942: return true; /* ⥂ */
    case 0x2943: return true; /* ⥃ */
    case 0x2944: return true; /* ⥄ */
    case 0x2945: return true; /* ⥅ */
    case 0x2946: return true; /* ⥆ */
    case 0x2947: return true; /* ⥇ */
    case 0x2948: return true; /* ⥈ */
    case 0x2949: return true; /* ⥉ */
    case 0x294a: return true; /* ⥊ */
    case 0x294b: return true; /* ⥋ */
    case 0x294c: return true; /* ⥌ */
    case 0x294d: return true; /* ⥍ */
    case 0x294e: return true; /* ⥎ */
    case 0x294f: return true; /* ⥏ */
    case 0x2950: return true; /* ⥐ */
    case 0x2951: return true; /* ⥑ */
    case 0x2952: return true; /* ⥒ */
    case 0x2953: return true; /* ⥓ */
    case 0x2954: return true; /* ⥔ */
    case 0x2955: return true; /* ⥕ */
    case 0x2956: return true; /* ⥖ */
    case 0x2957: return true; /* ⥗ */
    case 0x2958: return true; /* ⥘ */
    case 0x2959: return true; /* ⥙ */
    case 0x295a: return true; /* ⥚ */
    case 0x295b: return true; /* ⥛ */
    case 0x295c: return true; /* ⥜ */
    case 0x295d: return true; /* ⥝ */
    case 0x295e: return true; /* ⥞ */
    case 0x295f: return true; /* ⥟ */
    case 0x2960: return true; /* ⥠ */
    case 0x2961: return true; /* ⥡ */
    case 0x2962: return true; /* ⥢ */
    case 0x2963: return true; /* ⥣ */
    case 0x2964: return true; /* ⥤ */
    case 0x2965: return true; /* ⥥ */
    case 0x2966: return true; /* ⥦ */
    case 0x2967: return true; /* ⥧ */
    case 0x2968: return true; /* ⥨ */
    case 0x2969: return true; /* ⥩ */
    case 0x296a: return true; /* ⥪ */
    case 0x296b: return true; /* ⥫ */
    case 0x296c: return true; /* ⥬ */
    case 0x296d: return true; /* ⥭ */
    case 0x296e: return true; /* ⥮ */
    case 0x296f: return true; /* ⥯ */
    case 0x2970: return true; /* ⥰ */
    case 0x2971: return true; /* ⥱ */
    case 0x2972: return true; /* ⥲ */
    case 0x2973: return true; /* ⥳ */
    case 0x2974: return true; /* ⥴ */
    case 0x2975: return true; /* ⥵ */
    case 0x2976: return true; /* ⥶ */
    case 0x2977: return true; /* ⥷ */
    case 0x2978: return true; /* ⥸ */
    case 0x2979: return true; /* ⥹ */
    case 0x297a: return true; /* ⥺ */
    case 0x297b: return true; /* ⥻ */
    case 0x297c: return true; /* ⥼ */
    case 0x297d: return true; /* ⥽ */
    case 0x297e: return true; /* ⥾ */
    case 0x297f: return true; /* ⥿ */
    case 0x2980: return true; /* ⦀ */
    case 0x2981: return true; /* ⦁ */
    case 0x2982: return true; /* ⦂ */
    case 0x2983: return true; /* ⦃ */
    case 0x2984: return true; /* ⦄ */
    case 0x2985: return true; /* ⦅ */
    case 0x2986: return true; /* ⦆ */
    case 0x2987: return true; /* ⦇ */
    case 0x2988: return true; /* ⦈ */
    case 0x2989: return true; /* ⦉ */
    case 0x298a: return true; /* ⦊ */
    case 0x298b: return true; /* ⦋ */
    case 0x298c: return true; /* ⦌ */
    case 0x298d: return true; /* ⦍ */
    case 0x298e: return true; /* ⦎ */
    case 0x298f: return true; /* ⦏ */
    case 0x2990: return true; /* ⦐ */
    case 0x2991: return true; /* ⦑ */
    case 0x2992: return true; /* ⦒ */
    case 0x2993: return true; /* ⦓ */
    case 0x2994: return true; /* ⦔ */
    case 0x2995: return true; /* ⦕ */
    case 0x2996: return true; /* ⦖ */
    case 0x2997: return true; /* ⦗ */
    case 0x2998: return true; /* ⦘ */
    case 0x2999: return true; /* ⦙ */
    case 0x299a: return true; /* ⦚ */
    case 0x299b: return true; /* ⦛ */
    case 0x299c: return true; /* ⦜ */
    case 0x299d: return true; /* ⦝ */
    case 0x299e: return true; /* ⦞ */
    case 0x299f: return true; /* ⦟ */
    case 0x29a0: return true; /* ⦠ */
    case 0x29a1: return true; /* ⦡ */
    case 0x29a2: return true; /* ⦢ */
    case 0x29a3: return true; /* ⦣ */
    case 0x29a4: return true; /* ⦤ */
    case 0x29a5: return true; /* ⦥ */
    case 0x29a6: return true; /* ⦦ */
    case 0x29a7: return true; /* ⦧ */
    case 0x29a8: return true; /* ⦨ */
    case 0x29a9: return true; /* ⦩ */
    case 0x29aa: return true; /* ⦪ */
    case 0x29ab: return true; /* ⦫ */
    case 0x29ac: return true; /* ⦬ */
    case 0x29ad: return true; /* ⦭ */
    case 0x29ae: return true; /* ⦮ */
    case 0x29af: return true; /* ⦯ */
    case 0x29b0: return true; /* ⦰ */
    case 0x29b1: return true; /* ⦱ */
    case 0x29b2: return true; /* ⦲ */
    case 0x29b3: return true; /* ⦳ */
    case 0x29b4: return true; /* ⦴ */
    case 0x29b5: return true; /* ⦵ */
    case 0x29b6: return true; /* ⦶ */
    case 0x29b7: return true; /* ⦷ */
    case 0x29b8: return true; /* ⦸ */
    case 0x29b9: return true; /* ⦹ */
    case 0x29ba: return true; /* ⦺ */
    case 0x29bb: return true; /* ⦻ */
    case 0x29bc: return true; /* ⦼ */
    case 0x29bd: return true; /* ⦽ */
    case 0x29be: return true; /* ⦾ */
    case 0x29bf: return true; /* ⦿ */
    case 0x29c0: return true; /* ⧀ */
    case 0x29c1: return true; /* ⧁ */
    case 0x29c2: return true; /* ⧂ */
    case 0x29c3: return true; /* ⧃ */
    case 0x29c4: return true; /* ⧄ */
    case 0x29c5: return true; /* ⧅ */
    case 0x29c6: return true; /* ⧆ */
    case 0x29c7: return true; /* ⧇ */
    case 0x29c8: return true; /* ⧈ */
    case 0x29c9: return true; /* ⧉ */
    case 0x29ca: return true; /* ⧊ */
    case 0x29cb: return true; /* ⧋ */
    case 0x29cc: return true; /* ⧌ */
    case 0x29cd: return true; /* ⧍ */
    case 0x29ce: return true; /* ⧎ */
    case 0x29cf: return true; /* ⧏ */
    case 0x29d0: return true; /* ⧐ */
    case 0x29d1: return true; /* ⧑ */
    case 0x29d2: return true; /* ⧒ */
    case 0x29d3: return true; /* ⧓ */
    case 0x29d4: return true; /* ⧔ */
    case 0x29d5: return true; /* ⧕ */
    case 0x29d6: return true; /* ⧖ */
    case 0x29d7: return true; /* ⧗ */
    case 0x29d8: return true; /* ⧘ */
    case 0x29d9: return true; /* ⧙ */
    case 0x29da: return true; /* ⧚ */
    case 0x29db: return true; /* ⧛ */
    case 0x29dc: return true; /* ⧜ */
    case 0x29dd: return true; /* ⧝ */
    case 0x29de: return true; /* ⧞ */
    case 0x29df: return true; /* ⧟ */
    case 0x29e0: return true; /* ⧠ */
    case 0x29e1: return true; /* ⧡ */
    case 0x29e2: return true; /* ⧢ */
    case 0x29e3: return true; /* ⧣ */
    case 0x29e4: return true; /* ⧤ */
    case 0x29e5: return true; /* ⧥ */
    case 0x29e6: return true; /* ⧦ */
    case 0x29e7: return true; /* ⧧ */
    case 0x29e8: return true; /* ⧨ */
    case 0x29e9: return true; /* ⧩ */
    case 0x29ea: return true; /* ⧪ */
    case 0x29eb: return true; /* ⧫ */
    case 0x29ec: return true; /* ⧬ */
    case 0x29ed: return true; /* ⧭ */
    case 0x29ee: return true; /* ⧮ */
    case 0x29ef: return true; /* ⧯ */
    case 0x29f0: return true; /* ⧰ */
    case 0x29f1: return true; /* ⧱ */
    case 0x29f2: return true; /* ⧲ */
    case 0x29f3: return true; /* ⧳ */
    case 0x29f4: return true; /* ⧴ */
    case 0x29f5: return true; /* ⧵ */
    case 0x29f6: return true; /* ⧶ */
    case 0x29f7: return true; /* ⧷ */
    case 0x29f8: return true; /* ⧸ */
    case 0x29f9: return true; /* ⧹ */
    case 0x29fa: return true; /* ⧺ */
    case 0x29fb: return true; /* ⧻ */
    case 0x29fc: return true; /* ⧼ */
    case 0x29fd: return true; /* ⧽ */
    case 0x29fe: return true; /* ⧾ */
    case 0x29ff: return true; /* ⧿ */
    case 0x2a00: return true; /* ⨀ */
    case 0x2a01: return true; /* ⨁ */
    case 0x2a02: return true; /* ⨂ */
    case 0x2a03: return true; /* ⨃ */
    case 0x2a04: return true; /* ⨄ */
    case 0x2a05: return true; /* ⨅ */
    case 0x2a06: return true; /* ⨆ */
    case 0x2a07: return true; /* ⨇ */
    case 0x2a08: return true; /* ⨈ */
    case 0x2a09: return true; /* ⨉ */
    case 0x2a0a: return true; /* ⨊ */
    case 0x2a0b: return true; /* ⨋ */
    case 0x2a0c: return true; /* ⨌ */
    case 0x2a0d: return true; /* ⨍ */
    case 0x2a0e: return true; /* ⨎ */
    case 0x2a0f: return true; /* ⨏ */
    case 0x2a10: return true; /* ⨐ */
    case 0x2a11: return true; /* ⨑ */
    case 0x2a12: return true; /* ⨒ */
    case 0x2a13: return true; /* ⨓ */
    case 0x2a14: return true; /* ⨔ */
    case 0x2a15: return true; /* ⨕ */
    case 0x2a16: return true; /* ⨖ */
    case 0x2a17: return true; /* ⨗ */
    case 0x2a18: return true; /* ⨘ */
    case 0x2a19: return true; /* ⨙ */
    case 0x2a1a: return true; /* ⨚ */
    case 0x2a1b: return true; /* ⨛ */
    case 0x2a1c: return true; /* ⨜ */
    case 0x2a1d: return true; /* ⨝ */
    case 0x2a1e: return true; /* ⨞ */
    case 0x2a1f: return true; /* ⨟ */
    case 0x2a20: return true; /* ⨠ */
    case 0x2a21: return true; /* ⨡ */
    case 0x2a22: return true; /* ⨢ */
    case 0x2a23: return true; /* ⨣ */
    case 0x2a24: return true; /* ⨤ */
    case 0x2a25: return true; /* ⨥ */
    case 0x2a26: return true; /* ⨦ */
    case 0x2a27: return true; /* ⨧ */
    case 0x2a28: return true; /* ⨨ */
    case 0x2a29: return true; /* ⨩ */
    case 0x2a2a: return true; /* ⨪ */
    case 0x2a2b: return true; /* ⨫ */
    case 0x2a2c: return true; /* ⨬ */
    case 0x2a2d: return true; /* ⨭ */
    case 0x2a2e: return true; /* ⨮ */
    case 0x2a2f: return true; /* ⨯ */
    case 0x2a30: return true; /* ⨰ */
    case 0x2a31: return true; /* ⨱ */
    case 0x2a32: return true; /* ⨲ */
    case 0x2a33: return true; /* ⨳ */
    case 0x2a34: return true; /* ⨴ */
    case 0x2a35: return true; /* ⨵ */
    case 0x2a36: return true; /* ⨶ */
    case 0x2a37: return true; /* ⨷ */
    case 0x2a38: return true; /* ⨸ */
    case 0x2a39: return true; /* ⨹ */
    case 0x2a3a: return true; /* ⨺ */
    case 0x2a3b: return true; /* ⨻ */
    case 0x2a3c: return true; /* ⨼ */
    case 0x2a3d: return true; /* ⨽ */
    case 0x2a3e: return true; /* ⨾ */
    case 0x2a3f: return true; /* ⨿ */
    case 0x2a40: return true; /* ⩀ */
    case 0x2a41: return true; /* ⩁ */
    case 0x2a42: return true; /* ⩂ */
    case 0x2a43: return true; /* ⩃ */
    case 0x2a44: return true; /* ⩄ */
    case 0x2a45: return true; /* ⩅ */
    case 0x2a46: return true; /* ⩆ */
    case 0x2a47: return true; /* ⩇ */
    case 0x2a48: return true; /* ⩈ */
    case 0x2a49: return true; /* ⩉ */
    case 0x2a4a: return true; /* ⩊ */
    case 0x2a4b: return true; /* ⩋ */
    case 0x2a4c: return true; /* ⩌ */
    case 0x2a4d: return true; /* ⩍ */
    case 0x2a4e: return true; /* ⩎ */
    case 0x2a4f: return true; /* ⩏ */
    case 0x2a50: return true; /* ⩐ */
    case 0x2a51: return true; /* ⩑ */
    case 0x2a52: return true; /* ⩒ */
    case 0x2a53: return true; /* ⩓ */
    case 0x2a54: return true; /* ⩔ */
    case 0x2a55: return true; /* ⩕ */
    case 0x2a56: return true; /* ⩖ */
    case 0x2a57: return true; /* ⩗ */
    case 0x2a58: return true; /* ⩘ */
    case 0x2a59: return true; /* ⩙ */
    case 0x2a5a: return true; /* ⩚ */
    case 0x2a5b: return true; /* ⩛ */
    case 0x2a5c: return true; /* ⩜ */
    case 0x2a5d: return true; /* ⩝ */
    case 0x2a5e: return true; /* ⩞ */
    case 0x2a5f: return true; /* ⩟ */
    case 0x2a60: return true; /* ⩠ */
    case 0x2a61: return true; /* ⩡ */
    case 0x2a62: return true; /* ⩢ */
    case 0x2a63: return true; /* ⩣ */
    case 0x2a64: return true; /* ⩤ */
    case 0x2a65: return true; /* ⩥ */
    case 0x2a66: return true; /* ⩦ */
    case 0x2a67: return true; /* ⩧ */
    case 0x2a68: return true; /* ⩨ */
    case 0x2a69: return true; /* ⩩ */
    case 0x2a6a: return true; /* ⩪ */
    case 0x2a6b: return true; /* ⩫ */
    case 0x2a6c: return true; /* ⩬ */
    case 0x2a6d: return true; /* ⩭ */
    case 0x2a6e: return true; /* ⩮ */
    case 0x2a6f: return true; /* ⩯ */
    case 0x2a70: return true; /* ⩰ */
    case 0x2a71: return true; /* ⩱ */
    case 0x2a72: return true; /* ⩲ */
    case 0x2a73: return true; /* ⩳ */
    case 0x2a74: return true; /* ⩴ */
    case 0x2a75: return true; /* ⩵ */
    case 0x2a76: return true; /* ⩶ */
    case 0x2a77: return true; /* ⩷ */
    case 0x2a78: return true; /* ⩸ */
    case 0x2a79: return true; /* ⩹ */
    case 0x2a7a: return true; /* ⩺ */
    case 0x2a7b: return true; /* ⩻ */
    case 0x2a7c: return true; /* ⩼ */
    case 0x2a7d: return true; /* ⩽ */
    case 0x2a7e: return true; /* ⩾ */
    case 0x2a7f: return true; /* ⩿ */
    case 0x2a80: return true; /* ⪀ */
    case 0x2a81: return true; /* ⪁ */
    case 0x2a82: return true; /* ⪂ */
    case 0x2a83: return true; /* ⪃ */
    case 0x2a84: return true; /* ⪄ */
    case 0x2a85: return true; /* ⪅ */
    case 0x2a86: return true; /* ⪆ */
    case 0x2a87: return true; /* ⪇ */
    case 0x2a88: return true; /* ⪈ */
    case 0x2a89: return true; /* ⪉ */
    case 0x2a8a: return true; /* ⪊ */
    case 0x2a8b: return true; /* ⪋ */
    case 0x2a8c: return true; /* ⪌ */
    case 0x2a8d: return true; /* ⪍ */
    case 0x2a8e: return true; /* ⪎ */
    case 0x2a8f: return true; /* ⪏ */
    case 0x2a90: return true; /* ⪐ */
    case 0x2a91: return true; /* ⪑ */
    case 0x2a92: return true; /* ⪒ */
    case 0x2a93: return true; /* ⪓ */
    case 0x2a94: return true; /* ⪔ */
    case 0x2a95: return true; /* ⪕ */
    case 0x2a96: return true; /* ⪖ */
    case 0x2a97: return true; /* ⪗ */
    case 0x2a98: return true; /* ⪘ */
    case 0x2a99: return true; /* ⪙ */
    case 0x2a9a: return true; /* ⪚ */
    case 0x2a9b: return true; /* ⪛ */
    case 0x2a9c: return true; /* ⪜ */
    case 0x2a9d: return true; /* ⪝ */
    case 0x2a9e: return true; /* ⪞ */
    case 0x2a9f: return true; /* ⪟ */
    case 0x2aa0: return true; /* ⪠ */
    case 0x2aa1: return true; /* ⪡ */
    case 0x2aa2: return true; /* ⪢ */
    case 0x2aa3: return true; /* ⪣ */
    case 0x2aa4: return true; /* ⪤ */
    case 0x2aa5: return true; /* ⪥ */
    case 0x2aa6: return true; /* ⪦ */
    case 0x2aa7: return true; /* ⪧ */
    case 0x2aa8: return true; /* ⪨ */
    case 0x2aa9: return true; /* ⪩ */
    case 0x2aaa: return true; /* ⪪ */
    case 0x2aab: return true; /* ⪫ */
    case 0x2aac: return true; /* ⪬ */
    case 0x2aad: return true; /* ⪭ */
    case 0x2aae: return true; /* ⪮ */
    case 0x2aaf: return true; /* ⪯ */
    case 0x2ab0: return true; /* ⪰ */
    case 0x2ab1: return true; /* ⪱ */
    case 0x2ab2: return true; /* ⪲ */
    case 0x2ab3: return true; /* ⪳ */
    case 0x2ab4: return true; /* ⪴ */
    case 0x2ab5: return true; /* ⪵ */
    case 0x2ab6: return true; /* ⪶ */
    case 0x2ab7: return true; /* ⪷ */
    case 0x2ab8: return true; /* ⪸ */
    case 0x2ab9: return true; /* ⪹ */
    case 0x2aba: return true; /* ⪺ */
    case 0x2abb: return true; /* ⪻ */
    case 0x2abc: return true; /* ⪼ */
    case 0x2abd: return true; /* ⪽ */
    case 0x2abe: return true; /* ⪾ */
    case 0x2abf: return true; /* ⪿ */
    case 0x2ac0: return true; /* ⫀ */
    case 0x2ac1: return true; /* ⫁ */
    case 0x2ac2: return true; /* ⫂ */
    case 0x2ac3: return true; /* ⫃ */
    case 0x2ac4: return true; /* ⫄ */
    case 0x2ac5: return true; /* ⫅ */
    case 0x2ac6: return true; /* ⫆ */
    case 0x2ac7: return true; /* ⫇ */
    case 0x2ac8: return true; /* ⫈ */
    case 0x2ac9: return true; /* ⫉ */
    case 0x2aca: return true; /* ⫊ */
    case 0x2acb: return true; /* ⫋ */
    case 0x2acc: return true; /* ⫌ */
    case 0x2acd: return true; /* ⫍ */
    case 0x2ace: return true; /* ⫎ */
    case 0x2acf: return true; /* ⫏ */
    case 0x2ad0: return true; /* ⫐ */
    case 0x2ad1: return true; /* ⫑ */
    case 0x2ad2: return true; /* ⫒ */
    case 0x2ad3: return true; /* ⫓ */
    case 0x2ad4: return true; /* ⫔ */
    case 0x2ad5: return true; /* ⫕ */
    case 0x2ad6: return true; /* ⫖ */
    case 0x2ad7: return true; /* ⫗ */
    case 0x2ad8: return true; /* ⫘ */
    case 0x2ad9: return true; /* ⫙ */
    case 0x2ada: return true; /* ⫚ */
    case 0x2adb: return true; /* ⫛ */
    case 0x2adc: return true; /* ⫝̸ */
    case 0x2add: return true; /* ⫝ */
    case 0x2ade: return true; /* ⫞ */
    case 0x2adf: return true; /* ⫟ */
    case 0x2ae0: return true; /* ⫠ */
    case 0x2ae1: return true; /* ⫡ */
    case 0x2ae2: return true; /* ⫢ */
    case 0x2ae3: return true; /* ⫣ */
    case 0x2ae4: return true; /* ⫤ */
    case 0x2ae5: return true; /* ⫥ */
    case 0x2ae6: return true; /* ⫦ */
    case 0x2ae7: return true; /* ⫧ */
    case 0x2ae8: return true; /* ⫨ */
    case 0x2ae9: return true; /* ⫩ */
    case 0x2aea: return true; /* ⫪ */
    case 0x2aeb: return true; /* ⫫ */
    case 0x2aec: return true; /* ⫬ */
    case 0x2aed: return true; /* ⫭ */
    case 0x2aee: return true; /* ⫮ */
    case 0x2aef: return true; /* ⫯ */
    case 0x2af0: return true; /* ⫰ */
    case 0x2af1: return true; /* ⫱ */
    case 0x2af2: return true; /* ⫲ */
    case 0x2af3: return true; /* ⫳ */
    case 0x2af4: return true; /* ⫴ */
    case 0x2af5: return true; /* ⫵ */
    case 0x2af6: return true; /* ⫶ */
    case 0x2af7: return true; /* ⫷ */
    case 0x2af8: return true; /* ⫸ */
    case 0x2af9: return true; /* ⫹ */
    case 0x2afa: return true; /* ⫺ */
    case 0x2afb: return true; /* ⫻ */
    case 0x2afc: return true; /* ⫼ */
    case 0x2afd: return true; /* ⫽ */
    case 0x2afe: return true; /* ⫾ */
    case 0x2aff: return true; /* ⫿ */
    case 0x2b00: return true; /* ⬀ */
    case 0x2b01: return true; /* ⬁ */
    case 0x2b02: return true; /* ⬂ */
    case 0x2b03: return true; /* ⬃ */
    case 0x2b04: return true; /* ⬄ */
    case 0x2b05: return true; /* ⬅ */
    case 0x2b06: return true; /* ⬆ */
    case 0x2b07: return true; /* ⬇ */
    case 0x2b08: return true; /* ⬈ */
    case 0x2b09: return true; /* ⬉ */
    case 0x2b0a: return true; /* ⬊ */
    case 0x2b0b: return true; /* ⬋ */
    case 0x2b0c: return true; /* ⬌ */
    case 0x2b0d: return true; /* ⬍ */
    case 0x2b0e: return true; /* ⬎ */
    case 0x2b0f: return true; /* ⬏ */
    case 0x2b10: return true; /* ⬐ */
    case 0x2b11: return true; /* ⬑ */
    case 0x2b12: return true; /* ⬒ */
    case 0x2b13: return true; /* ⬓ */
    case 0x2b14: return true; /* ⬔ */
    case 0x2b15: return true; /* ⬕ */
    case 0x2b16: return true; /* ⬖ */
    case 0x2b17: return true; /* ⬗ */
    case 0x2b18: return true; /* ⬘ */
    case 0x2b19: return true; /* ⬙ */
    case 0x2b1a: return true; /* ⬚ */
    case 0x2b1b: return true; /* ⬛ */
    case 0x2b1c: return true; /* ⬜ */
    case 0x2b1d: return true; /* ⬝ */
    case 0x2b1e: return true; /* ⬞ */
    case 0x2b1f: return true; /* ⬟ */
    case 0x2b20: return true; /* ⬠ */
    case 0x2b21: return true; /* ⬡ */
    case 0x2b22: return true; /* ⬢ */
    case 0x2b23: return true; /* ⬣ */
    case 0x2b24: return true; /* ⬤ */
    case 0x2b25: return true; /* ⬥ */
    case 0x2b26: return true; /* ⬦ */
    case 0x2b27: return true; /* ⬧ */
    case 0x2b28: return true; /* ⬨ */
    case 0x2b29: return true; /* ⬩ */
    case 0x2b2a: return true; /* ⬪ */
    case 0x2b2b: return true; /* ⬫ */
    case 0x2b2c: return true; /* ⬬ */
    case 0x2b2d: return true; /* ⬭ */
    case 0x2b2e: return true; /* ⬮ */
    case 0x2b2f: return true; /* ⬯ */
    case 0x2b30: return true; /* ⬰ */
    case 0x2b31: return true; /* ⬱ */
    case 0x2b32: return true; /* ⬲ */
    case 0x2b33: return true; /* ⬳ */
    case 0x2b34: return true; /* ⬴ */
    case 0x2b35: return true; /* ⬵ */
    case 0x2b36: return true; /* ⬶ */
    case 0x2b37: return true; /* ⬷ */
    case 0x2b38: return true; /* ⬸ */
    case 0x2b39: return true; /* ⬹ */
    case 0x2b3a: return true; /* ⬺ */
    case 0x2b3b: return true; /* ⬻ */
    case 0x2b3c: return true; /* ⬼ */
    case 0x2b3d: return true; /* ⬽ */
    case 0x2b3e: return true; /* ⬾ */
    case 0x2b3f: return true; /* ⬿ */
    case 0x2b40: return true; /* ⭀ */
    case 0x2b41: return true; /* ⭁ */
    case 0x2b42: return true; /* ⭂ */
    case 0x2b43: return true; /* ⭃ */
    case 0x2b44: return true; /* ⭄ */
    case 0x2b45: return true; /* ⭅ */
    case 0x2b46: return true; /* ⭆ */
    case 0x2b47: return true; /* ⭇ */
    case 0x2b48: return true; /* ⭈ */
    case 0x2b49: return true; /* ⭉ */
    case 0x2b4a: return true; /* ⭊ */
    case 0x2b4b: return true; /* ⭋ */
    case 0x2b4c: return true; /* ⭌ */
    case 0x2b4d: return true; /* ⭍ */
    case 0x2b4e: return true; /* ⭎ */
    case 0x2b4f: return true; /* ⭏ */
    case 0x2b50: return true; /* ⭐ */
    case 0x2b51: return true; /* ⭑ */
    case 0x2b52: return true; /* ⭒ */
    case 0x2b53: return true; /* ⭓ */
    case 0x2b54: return true; /* ⭔ */
    case 0x2b55: return true; /* ⭕ */
    case 0x2b56: return true; /* ⭖ */
    case 0x2b57: return true; /* ⭗ */
    case 0x2b58: return true; /* ⭘ */
    case 0x2b59: return true; /* ⭙ */
    case 0x2b5a: return true; /* ⭚ */
    case 0x2b5b: return true; /* ⭛ */
    case 0x2b5c: return true; /* ⭜ */
    case 0x2b5d: return true; /* ⭝ */
    case 0x2b5e: return true; /* ⭞ */
    case 0x2b5f: return true; /* ⭟ */
    case 0x2b60: return true; /* ⭠ */
    case 0x2b61: return true; /* ⭡ */
    case 0x2b62: return true; /* ⭢ */
    case 0x2b63: return true; /* ⭣ */
    case 0x2b64: return true; /* ⭤ */
    case 0x2b65: return true; /* ⭥ */
    case 0x2b66: return true; /* ⭦ */
    case 0x2b67: return true; /* ⭧ */
    case 0x2b68: return true; /* ⭨ */
    case 0x2b69: return true; /* ⭩ */
    case 0x2b6a: return true; /* ⭪ */
    case 0x2b6b: return true; /* ⭫ */
    case 0x2b6c: return true; /* ⭬ */
    case 0x2b6d: return true; /* ⭭ */
    case 0x2b6e: return true; /* ⭮ */
    case 0x2b6f: return true; /* ⭯ */
    case 0x2b70: return true; /* ⭰ */
    case 0x2b71: return true; /* ⭱ */
    case 0x2b72: return true; /* ⭲ */
    case 0x2b73: return true; /* ⭳ */
    case 0x2b76: return true; /* ⭶ */
    case 0x2b77: return true; /* ⭷ */
    case 0x2b78: return true; /* ⭸ */
    case 0x2b79: return true; /* ⭹ */
    case 0x2b7a: return true; /* ⭺ */
    case 0x2b7b: return true; /* ⭻ */
    case 0x2b7c: return true; /* ⭼ */
    case 0x2b7d: return true; /* ⭽ */
    case 0x2b7e: return true; /* ⭾ */
    case 0x2b7f: return true; /* ⭿ */
    case 0x2b80: return true; /* ⮀ */
    case 0x2b81: return true; /* ⮁ */
    case 0x2b82: return true; /* ⮂ */
    case 0x2b83: return true; /* ⮃ */
    case 0x2b84: return true; /* ⮄ */
    case 0x2b85: return true; /* ⮅ */
    case 0x2b86: return true; /* ⮆ */
    case 0x2b87: return true; /* ⮇ */
    case 0x2b88: return true; /* ⮈ */
    case 0x2b89: return true; /* ⮉ */
    case 0x2b8a: return true; /* ⮊ */
    case 0x2b8b: return true; /* ⮋ */
    case 0x2b8c: return true; /* ⮌ */
    case 0x2b8d: return true; /* ⮍ */
    case 0x2b8e: return true; /* ⮎ */
    case 0x2b8f: return true; /* ⮏ */
    case 0x2b90: return true; /* ⮐ */
    case 0x2b91: return true; /* ⮑ */
    case 0x2b92: return true; /* ⮒ */
    case 0x2b93: return true; /* ⮓ */
    case 0x2b94: return true; /* ⮔ */
    case 0x2b95: return true; /* ⮕ */
    case 0x2b97: return true; /* ⮗ */
    case 0x2b98: return true; /* ⮘ */
    case 0x2b99: return true; /* ⮙ */
    case 0x2b9a: return true; /* ⮚ */
    case 0x2b9b: return true; /* ⮛ */
    case 0x2b9c: return true; /* ⮜ */
    case 0x2b9d: return true; /* ⮝ */
    case 0x2b9e: return true; /* ⮞ */
    case 0x2b9f: return true; /* ⮟ */
    case 0x2ba0: return true; /* ⮠ */
    case 0x2ba1: return true; /* ⮡ */
    case 0x2ba2: return true; /* ⮢ */
    case 0x2ba3: return true; /* ⮣ */
    case 0x2ba4: return true; /* ⮤ */
    case 0x2ba5: return true; /* ⮥ */
    case 0x2ba6: return true; /* ⮦ */
    case 0x2ba7: return true; /* ⮧ */
    case 0x2ba8: return true; /* ⮨ */
    case 0x2ba9: return true; /* ⮩ */
    case 0x2baa: return true; /* ⮪ */
    case 0x2bab: return true; /* ⮫ */
    case 0x2bac: return true; /* ⮬ */
    case 0x2bad: return true; /* ⮭ */
    case 0x2bae: return true; /* ⮮ */
    case 0x2baf: return true; /* ⮯ */
    case 0x2bb0: return true; /* ⮰ */
    case 0x2bb1: return true; /* ⮱ */
    case 0x2bb2: return true; /* ⮲ */
    case 0x2bb3: return true; /* ⮳ */
    case 0x2bb4: return true; /* ⮴ */
    case 0x2bb5: return true; /* ⮵ */
    case 0x2bb6: return true; /* ⮶ */
    case 0x2bb7: return true; /* ⮷ */
    case 0x2bb8: return true; /* ⮸ */
    case 0x2bb9: return true; /* ⮹ */
    case 0x2bba: return true; /* ⮺ */
    case 0x2bbb: return true; /* ⮻ */
    case 0x2bbc: return true; /* ⮼ */
    case 0x2bbd: return true; /* ⮽ */
    case 0x2bbe: return true; /* ⮾ */
    case 0x2bbf: return true; /* ⮿ */
    case 0x2bc0: return true; /* ⯀ */
    case 0x2bc1: return true; /* ⯁ */
    case 0x2bc2: return true; /* ⯂ */
    case 0x2bc3: return true; /* ⯃ */
    case 0x2bc4: return true; /* ⯄ */
    case 0x2bc5: return true; /* ⯅ */
    case 0x2bc6: return true; /* ⯆ */
    case 0x2bc7: return true; /* ⯇ */
    case 0x2bc8: return true; /* ⯈ */
    case 0x2bc9: return true; /* ⯉ */
    case 0x2bca: return true; /* ⯊ */
    case 0x2bcb: return true; /* ⯋ */
    case 0x2bcc: return true; /* ⯌ */
    case 0x2bcd: return true; /* ⯍ */
    case 0x2bce: return true; /* ⯎ */
    case 0x2bcf: return true; /* ⯏ */
    case 0x2bd0: return true; /* ⯐ */
    case 0x2bd1: return true; /* ⯑ */
    case 0x2bd2: return true; /* ⯒ */
    case 0x2bd3: return true; /* ⯓ */
    case 0x2bd4: return true; /* ⯔ */
    case 0x2bd5: return true; /* ⯕ */
    case 0x2bd6: return true; /* ⯖ */
    case 0x2bd7: return true; /* ⯗ */
    case 0x2bd8: return true; /* ⯘ */
    case 0x2bd9: return true; /* ⯙ */
    case 0x2bda: return true; /* ⯚ */
    case 0x2bdb: return true; /* ⯛ */
    case 0x2bdc: return true; /* ⯜ */
    case 0x2bdd: return true; /* ⯝ */
    case 0x2bde: return true; /* ⯞ */
    case 0x2bdf: return true; /* ⯟ */
    case 0x2be0: return true; /* ⯠ */
    case 0x2be1: return true; /* ⯡ */
    case 0x2be2: return true; /* ⯢ */
    case 0x2be3: return true; /* ⯣ */
    case 0x2be4: return true; /* ⯤ */
    case 0x2be5: return true; /* ⯥ */
    case 0x2be6: return true; /* ⯦ */
    case 0x2be7: return true; /* ⯧ */
    case 0x2be8: return true; /* ⯨ */
    case 0x2be9: return true; /* ⯩ */
    case 0x2bea: return true; /* ⯪ */
    case 0x2beb: return true; /* ⯫ */
    case 0x2bec: return true; /* ⯬ */
    case 0x2bed: return true; /* ⯭ */
    case 0x2bee: return true; /* ⯮ */
    case 0x2bef: return true; /* ⯯ */
    case 0x2bf0: return true; /* ⯰ */
    case 0x2bf1: return true; /* ⯱ */
    case 0x2bf2: return true; /* ⯲ */
    case 0x2bf3: return true; /* ⯳ */
    case 0x2bf4: return true; /* ⯴ */
    case 0x2bf5: return true; /* ⯵ */
    case 0x2bf6: return true; /* ⯶ */
    case 0x2bf7: return true; /* ⯷ */
    case 0x2bf8: return true; /* ⯸ */
    case 0x2bf9: return true; /* ⯹ */
    case 0x2bfa: return true; /* ⯺ */
    case 0x2bfb: return true; /* ⯻ */
    case 0x2bfc: return true; /* ⯼ */
    case 0x2bfd: return true; /* ⯽ */
    case 0x2bfe: return true; /* ⯾ */
    case 0x2bff: return true; /* ⯿ */
    case 0x2ce5: return true; /* ⳥ */
    case 0x2ce6: return true; /* ⳦ */
    case 0x2ce7: return true; /* ⳧ */
    case 0x2ce8: return true; /* ⳨ */
    case 0x2ce9: return true; /* ⳩ */
    case 0x2cea: return true; /* ⳪ */
    case 0x2cf9: return true; /* ⳹ */
    case 0x2cfa: return true; /* ⳺ */
    case 0x2cfb: return true; /* ⳻ */
    case 0x2cfc: return true; /* ⳼ */
    case 0x2cfe: return true; /* ⳾ */
    case 0x2cff: return true; /* ⳿ */
    case 0x2d70: return true; /* ⵰ */
    case 0x2e00: return true; /* ⸀ */
    case 0x2e01: return true; /* ⸁ */
    case 0x2e02: return true; /* ⸂ */
    case 0x2e03: return true; /* ⸃ */
    case 0x2e04: return true; /* ⸄ */
    case 0x2e05: return true; /* ⸅ */
    case 0x2e06: return true; /* ⸆ */
    case 0x2e07: return true; /* ⸇ */
    case 0x2e08: return true; /* ⸈ */
    case 0x2e09: return true; /* ⸉ */
    case 0x2e0a: return true; /* ⸊ */
    case 0x2e0b: return true; /* ⸋ */
    case 0x2e0c: return true; /* ⸌ */
    case 0x2e0d: return true; /* ⸍ */
    case 0x2e0e: return true; /* ⸎ */
    case 0x2e0f: return true; /* ⸏ */
    case 0x2e10: return true; /* ⸐ */
    case 0x2e11: return true; /* ⸑ */
    case 0x2e12: return true; /* ⸒ */
    case 0x2e13: return true; /* ⸓ */
    case 0x2e14: return true; /* ⸔ */
    case 0x2e15: return true; /* ⸕ */
    case 0x2e16: return true; /* ⸖ */
    case 0x2e17: return true; /* ⸗ */
    case 0x2e18: return true; /* ⸘ */
    case 0x2e19: return true; /* ⸙ */
    case 0x2e1a: return true; /* ⸚ */
    case 0x2e1b: return true; /* ⸛ */
    case 0x2e1c: return true; /* ⸜ */
    case 0x2e1d: return true; /* ⸝ */
    case 0x2e1e: return true; /* ⸞ */
    case 0x2e1f: return true; /* ⸟ */
    case 0x2e20: return true; /* ⸠ */
    case 0x2e21: return true; /* ⸡ */
    case 0x2e22: return true; /* ⸢ */
    case 0x2e23: return true; /* ⸣ */
    case 0x2e24: return true; /* ⸤ */
    case 0x2e25: return true; /* ⸥ */
    case 0x2e26: return true; /* ⸦ */
    case 0x2e27: return true; /* ⸧ */
    case 0x2e28: return true; /* ⸨ */
    case 0x2e29: return true; /* ⸩ */
    case 0x2e2a: return true; /* ⸪ */
    case 0x2e2b: return true; /* ⸫ */
    case 0x2e2c: return true; /* ⸬ */
    case 0x2e2d: return true; /* ⸭ */
    case 0x2e2e: return true; /* ⸮ */
    case 0x2e30: return true; /* ⸰ */
    case 0x2e31: return true; /* ⸱ */
    case 0x2e32: return true; /* ⸲ */
    case 0x2e33: return true; /* ⸳ */
    case 0x2e34: return true; /* ⸴ */
    case 0x2e35: return true; /* ⸵ */
    case 0x2e36: return true; /* ⸶ */
    case 0x2e37: return true; /* ⸷ */
    case 0x2e38: return true; /* ⸸ */
    case 0x2e39: return true; /* ⸹ */
    case 0x2e3a: return true; /* ⸺ */
    case 0x2e3b: return true; /* ⸻ */
    case 0x2e3c: return true; /* ⸼ */
    case 0x2e3d: return true; /* ⸽ */
    case 0x2e3e: return true; /* ⸾ */
    case 0x2e3f: return true; /* ⸿ */
    case 0x2e40: return true; /* ⹀ */
    case 0x2e41: return true; /* ⹁ */
    case 0x2e42: return true; /* ⹂ */
    case 0x2e43: return true; /* ⹃ */
    case 0x2e44: return true; /* ⹄ */
    case 0x2e45: return true; /* ⹅ */
    case 0x2e46: return true; /* ⹆ */
    case 0x2e47: return true; /* ⹇ */
    case 0x2e48: return true; /* ⹈ */
    case 0x2e49: return true; /* ⹉ */
    case 0x2e4a: return true; /* ⹊ */
    case 0x2e4b: return true; /* ⹋ */
    case 0x2e4c: return true; /* ⹌ */
    case 0x2e4d: return true; /* ⹍ */
    case 0x2e4e: return true; /* ⹎ */
    case 0x2e4f: return true; /* ⹏ */
    case 0x2e50: return true; /* ⹐ */
    case 0x2e51: return true; /* ⹑ */
    case 0x2e52: return true; /* ⹒ */
    case 0x2e80: return true; /* ⺀ */
    case 0x2e81: return true; /* ⺁ */
    case 0x2e82: return true; /* ⺂ */
    case 0x2e83: return true; /* ⺃ */
    case 0x2e84: return true; /* ⺄ */
    case 0x2e85: return true; /* ⺅ */
    case 0x2e86: return true; /* ⺆ */
    case 0x2e87: return true; /* ⺇ */
    case 0x2e88: return true; /* ⺈ */
    case 0x2e89: return true; /* ⺉ */
    case 0x2e8a: return true; /* ⺊ */
    case 0x2e8b: return true; /* ⺋ */
    case 0x2e8c: return true; /* ⺌ */
    case 0x2e8d: return true; /* ⺍ */
    case 0x2e8e: return true; /* ⺎ */
    case 0x2e8f: return true; /* ⺏ */
    case 0x2e90: return true; /* ⺐ */
    case 0x2e91: return true; /* ⺑ */
    case 0x2e92: return true; /* ⺒ */
    case 0x2e93: return true; /* ⺓ */
    case 0x2e94: return true; /* ⺔ */
    case 0x2e95: return true; /* ⺕ */
    case 0x2e96: return true; /* ⺖ */
    case 0x2e97: return true; /* ⺗ */
    case 0x2e98: return true; /* ⺘ */
    case 0x2e99: return true; /* ⺙ */
    case 0x2e9b: return true; /* ⺛ */
    case 0x2e9c: return true; /* ⺜ */
    case 0x2e9d: return true; /* ⺝ */
    case 0x2e9e: return true; /* ⺞ */
    case 0x2e9f: return true; /* ⺟ */
    case 0x2ea0: return true; /* ⺠ */
    case 0x2ea1: return true; /* ⺡ */
    case 0x2ea2: return true; /* ⺢ */
    case 0x2ea3: return true; /* ⺣ */
    case 0x2ea4: return true; /* ⺤ */
    case 0x2ea5: return true; /* ⺥ */
    case 0x2ea6: return true; /* ⺦ */
    case 0x2ea7: return true; /* ⺧ */
    case 0x2ea8: return true; /* ⺨ */
    case 0x2ea9: return true; /* ⺩ */
    case 0x2eaa: return true; /* ⺪ */
    case 0x2eab: return true; /* ⺫ */
    case 0x2eac: return true; /* ⺬ */
    case 0x2ead: return true; /* ⺭ */
    case 0x2eae: return true; /* ⺮ */
    case 0x2eaf: return true; /* ⺯ */
    case 0x2eb0: return true; /* ⺰ */
    case 0x2eb1: return true; /* ⺱ */
    case 0x2eb2: return true; /* ⺲ */
    case 0x2eb3: return true; /* ⺳ */
    case 0x2eb4: return true; /* ⺴ */
    case 0x2eb5: return true; /* ⺵ */
    case 0x2eb6: return true; /* ⺶ */
    case 0x2eb7: return true; /* ⺷ */
    case 0x2eb8: return true; /* ⺸ */
    case 0x2eb9: return true; /* ⺹ */
    case 0x2eba: return true; /* ⺺ */
    case 0x2ebb: return true; /* ⺻ */
    case 0x2ebc: return true; /* ⺼ */
    case 0x2ebd: return true; /* ⺽ */
    case 0x2ebe: return true; /* ⺾ */
    case 0x2ebf: return true; /* ⺿ */
    case 0x2ec0: return true; /* ⻀ */
    case 0x2ec1: return true; /* ⻁ */
    case 0x2ec2: return true; /* ⻂ */
    case 0x2ec3: return true; /* ⻃ */
    case 0x2ec4: return true; /* ⻄ */
    case 0x2ec5: return true; /* ⻅ */
    case 0x2ec6: return true; /* ⻆ */
    case 0x2ec7: return true; /* ⻇ */
    case 0x2ec8: return true; /* ⻈ */
    case 0x2ec9: return true; /* ⻉ */
    case 0x2eca: return true; /* ⻊ */
    case 0x2ecb: return true; /* ⻋ */
    case 0x2ecc: return true; /* ⻌ */
    case 0x2ecd: return true; /* ⻍ */
    case 0x2ece: return true; /* ⻎ */
    case 0x2ecf: return true; /* ⻏ */
    case 0x2ed0: return true; /* ⻐ */
    case 0x2ed1: return true; /* ⻑ */
    case 0x2ed2: return true; /* ⻒ */
    case 0x2ed3: return true; /* ⻓ */
    case 0x2ed4: return true; /* ⻔ */
    case 0x2ed5: return true; /* ⻕ */
    case 0x2ed6: return true; /* ⻖ */
    case 0x2ed7: return true; /* ⻗ */
    case 0x2ed8: return true; /* ⻘ */
    case 0x2ed9: return true; /* ⻙ */
    case 0x2eda: return true; /* ⻚ */
    case 0x2edb: return true; /* ⻛ */
    case 0x2edc: return true; /* ⻜ */
    case 0x2edd: return true; /* ⻝ */
    case 0x2ede: return true; /* ⻞ */
    case 0x2edf: return true; /* ⻟ */
    case 0x2ee0: return true; /* ⻠ */
    case 0x2ee1: return true; /* ⻡ */
    case 0x2ee2: return true; /* ⻢ */
    case 0x2ee3: return true; /* ⻣ */
    case 0x2ee4: return true; /* ⻤ */
    case 0x2ee5: return true; /* ⻥ */
    case 0x2ee6: return true; /* ⻦ */
    case 0x2ee7: return true; /* ⻧ */
    case 0x2ee8: return true; /* ⻨ */
    case 0x2ee9: return true; /* ⻩ */
    case 0x2eea: return true; /* ⻪ */
    case 0x2eeb: return true; /* ⻫ */
    case 0x2eec: return true; /* ⻬ */
    case 0x2eed: return true; /* ⻭ */
    case 0x2eee: return true; /* ⻮ */
    case 0x2eef: return true; /* ⻯ */
    case 0x2ef0: return true; /* ⻰ */
    case 0x2ef1: return true; /* ⻱ */
    case 0x2ef2: return true; /* ⻲ */
    case 0x2ef3: return true; /* ⻳ */
    case 0x2f00: return true; /* ⼀ */
    case 0x2f01: return true; /* ⼁ */
    case 0x2f02: return true; /* ⼂ */
    case 0x2f03: return true; /* ⼃ */
    case 0x2f04: return true; /* ⼄ */
    case 0x2f05: return true; /* ⼅ */
    case 0x2f06: return true; /* ⼆ */
    case 0x2f07: return true; /* ⼇ */
    case 0x2f08: return true; /* ⼈ */
    case 0x2f09: return true; /* ⼉ */
    case 0x2f0a: return true; /* ⼊ */
    case 0x2f0b: return true; /* ⼋ */
    case 0x2f0c: return true; /* ⼌ */
    case 0x2f0d: return true; /* ⼍ */
    case 0x2f0e: return true; /* ⼎ */
    case 0x2f0f: return true; /* ⼏ */
    case 0x2f10: return true; /* ⼐ */
    case 0x2f11: return true; /* ⼑ */
    case 0x2f12: return true; /* ⼒ */
    case 0x2f13: return true; /* ⼓ */
    case 0x2f14: return true; /* ⼔ */
    case 0x2f15: return true; /* ⼕ */
    case 0x2f16: return true; /* ⼖ */
    case 0x2f17: return true; /* ⼗ */
    case 0x2f18: return true; /* ⼘ */
    case 0x2f19: return true; /* ⼙ */
    case 0x2f1a: return true; /* ⼚ */
    case 0x2f1b: return true; /* ⼛ */
    case 0x2f1c: return true; /* ⼜ */
    case 0x2f1d: return true; /* ⼝ */
    case 0x2f1e: return true; /* ⼞ */
    case 0x2f1f: return true; /* ⼟ */
    case 0x2f20: return true; /* ⼠ */
    case 0x2f21: return true; /* ⼡ */
    case 0x2f22: return true; /* ⼢ */
    case 0x2f23: return true; /* ⼣ */
    case 0x2f24: return true; /* ⼤ */
    case 0x2f25: return true; /* ⼥ */
    case 0x2f26: return true; /* ⼦ */
    case 0x2f27: return true; /* ⼧ */
    case 0x2f28: return true; /* ⼨ */
    case 0x2f29: return true; /* ⼩ */
    case 0x2f2a: return true; /* ⼪ */
    case 0x2f2b: return true; /* ⼫ */
    case 0x2f2c: return true; /* ⼬ */
    case 0x2f2d: return true; /* ⼭ */
    case 0x2f2e: return true; /* ⼮ */
    case 0x2f2f: return true; /* ⼯ */
    case 0x2f30: return true; /* ⼰ */
    case 0x2f31: return true; /* ⼱ */
    case 0x2f32: return true; /* ⼲ */
    case 0x2f33: return true; /* ⼳ */
    case 0x2f34: return true; /* ⼴ */
    case 0x2f35: return true; /* ⼵ */
    case 0x2f36: return true; /* ⼶ */
    case 0x2f37: return true; /* ⼷ */
    case 0x2f38: return true; /* ⼸ */
    case 0x2f39: return true; /* ⼹ */
    case 0x2f3a: return true; /* ⼺ */
    case 0x2f3b: return true; /* ⼻ */
    case 0x2f3c: return true; /* ⼼ */
    case 0x2f3d: return true; /* ⼽ */
    case 0x2f3e: return true; /* ⼾ */
    case 0x2f3f: return true; /* ⼿ */
    case 0x2f40: return true; /* ⽀ */
    case 0x2f41: return true; /* ⽁ */
    case 0x2f42: return true; /* ⽂ */
    case 0x2f43: return true; /* ⽃ */
    case 0x2f44: return true; /* ⽄ */
    case 0x2f45: return true; /* ⽅ */
    case 0x2f46: return true; /* ⽆ */
    case 0x2f47: return true; /* ⽇ */
    case 0x2f48: return true; /* ⽈ */
    case 0x2f49: return true; /* ⽉ */
    case 0x2f4a: return true; /* ⽊ */
    case 0x2f4b: return true; /* ⽋ */
    case 0x2f4c: return true; /* ⽌ */
    case 0x2f4d: return true; /* ⽍ */
    case 0x2f4e: return true; /* ⽎ */
    case 0x2f4f: return true; /* ⽏ */
    case 0x2f50: return true; /* ⽐ */
    case 0x2f51: return true; /* ⽑ */
    case 0x2f52: return true; /* ⽒ */
    case 0x2f53: return true; /* ⽓ */
    case 0x2f54: return true; /* ⽔ */
    case 0x2f55: return true; /* ⽕ */
    case 0x2f56: return true; /* ⽖ */
    case 0x2f57: return true; /* ⽗ */
    case 0x2f58: return true; /* ⽘ */
    case 0x2f59: return true; /* ⽙ */
    case 0x2f5a: return true; /* ⽚ */
    case 0x2f5b: return true; /* ⽛ */
    case 0x2f5c: return true; /* ⽜ */
    case 0x2f5d: return true; /* ⽝ */
    case 0x2f5e: return true; /* ⽞ */
    case 0x2f5f: return true; /* ⽟ */
    case 0x2f60: return true; /* ⽠ */
    case 0x2f61: return true; /* ⽡ */
    case 0x2f62: return true; /* ⽢ */
    case 0x2f63: return true; /* ⽣ */
    case 0x2f64: return true; /* ⽤ */
    case 0x2f65: return true; /* ⽥ */
    case 0x2f66: return true; /* ⽦ */
    case 0x2f67: return true; /* ⽧ */
    case 0x2f68: return true; /* ⽨ */
    case 0x2f69: return true; /* ⽩ */
    case 0x2f6a: return true; /* ⽪ */
    case 0x2f6b: return true; /* ⽫ */
    case 0x2f6c: return true; /* ⽬ */
    case 0x2f6d: return true; /* ⽭ */
    case 0x2f6e: return true; /* ⽮ */
    case 0x2f6f: return true; /* ⽯ */
    case 0x2f70: return true; /* ⽰ */
    case 0x2f71: return true; /* ⽱ */
    case 0x2f72: return true; /* ⽲ */
    case 0x2f73: return true; /* ⽳ */
    case 0x2f74: return true; /* ⽴ */
    case 0x2f75: return true; /* ⽵ */
    case 0x2f76: return true; /* ⽶ */
    case 0x2f77: return true; /* ⽷ */
    case 0x2f78: return true; /* ⽸ */
    case 0x2f79: return true; /* ⽹ */
    case 0x2f7a: return true; /* ⽺ */
    case 0x2f7b: return true; /* ⽻ */
    case 0x2f7c: return true; /* ⽼ */
    case 0x2f7d: return true; /* ⽽ */
    case 0x2f7e: return true; /* ⽾ */
    case 0x2f7f: return true; /* ⽿ */
    case 0x2f80: return true; /* ⾀ */
    case 0x2f81: return true; /* ⾁ */
    case 0x2f82: return true; /* ⾂ */
    case 0x2f83: return true; /* ⾃ */
    case 0x2f84: return true; /* ⾄ */
    case 0x2f85: return true; /* ⾅ */
    case 0x2f86: return true; /* ⾆ */
    case 0x2f87: return true; /* ⾇ */
    case 0x2f88: return true; /* ⾈ */
    case 0x2f89: return true; /* ⾉ */
    case 0x2f8a: return true; /* ⾊ */
    case 0x2f8b: return true; /* ⾋ */
    case 0x2f8c: return true; /* ⾌ */
    case 0x2f8d: return true; /* ⾍ */
    case 0x2f8e: return true; /* ⾎ */
    case 0x2f8f: return true; /* ⾏ */
    case 0x2f90: return true; /* ⾐ */
    case 0x2f91: return true; /* ⾑ */
    case 0x2f92: return true; /* ⾒ */
    case 0x2f93: return true; /* ⾓ */
    case 0x2f94: return true; /* ⾔ */
    case 0x2f95: return true; /* ⾕ */
    case 0x2f96: return true; /* ⾖ */
    case 0x2f97: return true; /* ⾗ */
    case 0x2f98: return true; /* ⾘ */
    case 0x2f99: return true; /* ⾙ */
    case 0x2f9a: return true; /* ⾚ */
    case 0x2f9b: return true; /* ⾛ */
    case 0x2f9c: return true; /* ⾜ */
    case 0x2f9d: return true; /* ⾝ */
    case 0x2f9e: return true; /* ⾞ */
    case 0x2f9f: return true; /* ⾟ */
    case 0x2fa0: return true; /* ⾠ */
    case 0x2fa1: return true; /* ⾡ */
    case 0x2fa2: return true; /* ⾢ */
    case 0x2fa3: return true; /* ⾣ */
    case 0x2fa4: return true; /* ⾤ */
    case 0x2fa5: return true; /* ⾥ */
    case 0x2fa6: return true; /* ⾦ */
    case 0x2fa7: return true; /* ⾧ */
    case 0x2fa8: return true; /* ⾨ */
    case 0x2fa9: return true; /* ⾩ */
    case 0x2faa: return true; /* ⾪ */
    case 0x2fab: return true; /* ⾫ */
    case 0x2fac: return true; /* ⾬ */
    case 0x2fad: return true; /* ⾭ */
    case 0x2fae: return true; /* ⾮ */
    case 0x2faf: return true; /* ⾯ */
    case 0x2fb0: return true; /* ⾰ */
    case 0x2fb1: return true; /* ⾱ */
    case 0x2fb2: return true; /* ⾲ */
    case 0x2fb3: return true; /* ⾳ */
    case 0x2fb4: return true; /* ⾴ */
    case 0x2fb5: return true; /* ⾵ */
    case 0x2fb6: return true; /* ⾶ */
    case 0x2fb7: return true; /* ⾷ */
    case 0x2fb8: return true; /* ⾸ */
    case 0x2fb9: return true; /* ⾹ */
    case 0x2fba: return true; /* ⾺ */
    case 0x2fbb: return true; /* ⾻ */
    case 0x2fbc: return true; /* ⾼ */
    case 0x2fbd: return true; /* ⾽ */
    case 0x2fbe: return true; /* ⾾ */
    case 0x2fbf: return true; /* ⾿ */
    case 0x2fc0: return true; /* ⿀ */
    case 0x2fc1: return true; /* ⿁ */
    case 0x2fc2: return true; /* ⿂ */
    case 0x2fc3: return true; /* ⿃ */
    case 0x2fc4: return true; /* ⿄ */
    case 0x2fc5: return true; /* ⿅ */
    case 0x2fc6: return true; /* ⿆ */
    case 0x2fc7: return true; /* ⿇ */
    case 0x2fc8: return true; /* ⿈ */
    case 0x2fc9: return true; /* ⿉ */
    case 0x2fca: return true; /* ⿊ */
    case 0x2fcb: return true; /* ⿋ */
    case 0x2fcc: return true; /* ⿌ */
    case 0x2fcd: return true; /* ⿍ */
    case 0x2fce: return true; /* ⿎ */
    case 0x2fcf: return true; /* ⿏ */
    case 0x2fd0: return true; /* ⿐ */
    case 0x2fd1: return true; /* ⿑ */
    case 0x2fd2: return true; /* ⿒ */
    case 0x2fd3: return true; /* ⿓ */
    case 0x2fd4: return true; /* ⿔ */
    case 0x2fd5: return true; /* ⿕ */
    case 0x2ff0: return true; /* ⿰ */
    case 0x2ff1: return true; /* ⿱ */
    case 0x2ff2: return true; /* ⿲ */
    case 0x2ff3: return true; /* ⿳ */
    case 0x2ff4: return true; /* ⿴ */
    case 0x2ff5: return true; /* ⿵ */
    case 0x2ff6: return true; /* ⿶ */
    case 0x2ff7: return true; /* ⿷ */
    case 0x2ff8: return true; /* ⿸ */
    case 0x2ff9: return true; /* ⿹ */
    case 0x2ffa: return true; /* ⿺ */
    case 0x2ffb: return true; /* ⿻ */
    case 0x3001: return true; /* 、 */
    case 0x3002: return true; /* 。 */
    case 0x3003: return true; /* 〃 */
    case 0x3004: return true; /* 〄 */
    case 0x3008: return true; /* 〈 */
    case 0x3009: return true; /* 〉 */
    case 0x300a: return true; /* 《 */
    case 0x300b: return true; /* 》 */
    case 0x300c: return true; /* 「 */
    case 0x300d: return true; /* 」 */
    case 0x300e: return true; /* 『 */
    case 0x300f: return true; /* 』 */
    case 0x3010: return true; /* 【 */
    case 0x3011: return true; /* 】 */
    case 0x3012: return true; /* 〒 */
    case 0x3013: return true; /* 〓 */
    case 0x3014: return true; /* 〔 */
    case 0x3015: return true; /* 〕 */
    case 0x3016: return true; /* 〖 */
    case 0x3017: return true; /* 〗 */
    case 0x3018: return true; /* 〘 */
    case 0x3019: return true; /* 〙 */
    case 0x301a: return true; /* 〚 */
    case 0x301b: return true; /* 〛 */
    case 0x301c: return true; /* 〜 */
    case 0x301d: return true; /* 〝 */
    case 0x301e: return true; /* 〞 */
    case 0x301f: return true; /* 〟 */
    case 0x3020: return true; /* 〠 */
    case 0x3030: return true; /* 〰 */
    case 0x3036: return true; /* 〶 */
    case 0x3037: return true; /* 〷 */
    case 0x303d: return true; /* 〽 */
    case 0x303e: return true; /* 〾 */
    case 0x303f: return true; /* 〿 */
    case 0x309b: return true; /* ゛ */
    case 0x309c: return true; /* ゜ */
    case 0x30a0: return true; /* ゠ */
    case 0x30fb: return true; /* ・ */
    case 0x3190: return true; /* ㆐ */
    case 0x3191: return true; /* ㆑ */
    case 0x3196: return true; /* ㆖ */
    case 0x3197: return true; /* ㆗ */
    case 0x3198: return true; /* ㆘ */
    case 0x3199: return true; /* ㆙ */
    case 0x319a: return true; /* ㆚ */
    case 0x319b: return true; /* ㆛ */
    case 0x319c: return true; /* ㆜ */
    case 0x319d: return true; /* ㆝ */
    case 0x319e: return true; /* ㆞ */
    case 0x319f: return true; /* ㆟ */
    case 0x31c0: return true; /* ㇀ */
    case 0x31c1: return true; /* ㇁ */
    case 0x31c2: return true; /* ㇂ */
    case 0x31c3: return true; /* ㇃ */
    case 0x31c4: return true; /* ㇄ */
    case 0x31c5: return true; /* ㇅ */
    case 0x31c6: return true; /* ㇆ */
    case 0x31c7: return true; /* ㇇ */
    case 0x31c8: return true; /* ㇈ */
    case 0x31c9: return true; /* ㇉ */
    case 0x31ca: return true; /* ㇊ */
    case 0x31cb: return true; /* ㇋ */
    case 0x31cc: return true; /* ㇌ */
    case 0x31cd: return true; /* ㇍ */
    case 0x31ce: return true; /* ㇎ */
    case 0x31cf: return true; /* ㇏ */
    case 0x31d0: return true; /* ㇐ */
    case 0x31d1: return true; /* ㇑ */
    case 0x31d2: return true; /* ㇒ */
    case 0x31d3: return true; /* ㇓ */
    case 0x31d4: return true; /* ㇔ */
    case 0x31d5: return true; /* ㇕ */
    case 0x31d6: return true; /* ㇖ */
    case 0x31d7: return true; /* ㇗ */
    case 0x31d8: return true; /* ㇘ */
    case 0x31d9: return true; /* ㇙ */
    case 0x31da: return true; /* ㇚ */
    case 0x31db: return true; /* ㇛ */
    case 0x31dc: return true; /* ㇜ */
    case 0x31dd: return true; /* ㇝ */
    case 0x31de: return true; /* ㇞ */
    case 0x31df: return true; /* ㇟ */
    case 0x31e0: return true; /* ㇠ */
    case 0x31e1: return true; /* ㇡ */
    case 0x31e2: return true; /* ㇢ */
    case 0x31e3: return true; /* ㇣ */
    case 0x3200: return true; /* ㈀ */
    case 0x3201: return true; /* ㈁ */
    case 0x3202: return true; /* ㈂ */
    case 0x3203: return true; /* ㈃ */
    case 0x3204: return true; /* ㈄ */
    case 0x3205: return true; /* ㈅ */
    case 0x3206: return true; /* ㈆ */
    case 0x3207: return true; /* ㈇ */
    case 0x3208: return true; /* ㈈ */
    case 0x3209: return true; /* ㈉ */
    case 0x320a: return true; /* ㈊ */
    case 0x320b: return true; /* ㈋ */
    case 0x320c: return true; /* ㈌ */
    case 0x320d: return true; /* ㈍ */
    case 0x320e: return true; /* ㈎ */
    case 0x320f: return true; /* ㈏ */
    case 0x3210: return true; /* ㈐ */
    case 0x3211: return true; /* ㈑ */
    case 0x3212: return true; /* ㈒ */
    case 0x3213: return true; /* ㈓ */
    case 0x3214: return true; /* ㈔ */
    case 0x3215: return true; /* ㈕ */
    case 0x3216: return true; /* ㈖ */
    case 0x3217: return true; /* ㈗ */
    case 0x3218: return true; /* ㈘ */
    case 0x3219: return true; /* ㈙ */
    case 0x321a: return true; /* ㈚ */
    case 0x321b: return true; /* ㈛ */
    case 0x321c: return true; /* ㈜ */
    case 0x321d: return true; /* ㈝ */
    case 0x321e: return true; /* ㈞ */
    case 0x322a: return true; /* ㈪ */
    case 0x322b: return true; /* ㈫ */
    case 0x322c: return true; /* ㈬ */
    case 0x322d: return true; /* ㈭ */
    case 0x322e: return true; /* ㈮ */
    case 0x322f: return true; /* ㈯ */
    case 0x3230: return true; /* ㈰ */
    case 0x3231: return true; /* ㈱ */
    case 0x3232: return true; /* ㈲ */
    case 0x3233: return true; /* ㈳ */
    case 0x3234: return true; /* ㈴ */
    case 0x3235: return true; /* ㈵ */
    case 0x3236: return true; /* ㈶ */
    case 0x3237: return true; /* ㈷ */
    case 0x3238: return true; /* ㈸ */
    case 0x3239: return true; /* ㈹ */
    case 0x323a: return true; /* ㈺ */
    case 0x323b: return true; /* ㈻ */
    case 0x323c: return true; /* ㈼ */
    case 0x323d: return true; /* ㈽ */
    case 0x323e: return true; /* ㈾ */
    case 0x323f: return true; /* ㈿ */
    case 0x3240: return true; /* ㉀ */
    case 0x3241: return true; /* ㉁ */
    case 0x3242: return true; /* ㉂ */
    case 0x3243: return true; /* ㉃ */
    case 0x3244: return true; /* ㉄ */
    case 0x3245: return true; /* ㉅ */
    case 0x3246: return true; /* ㉆ */
    case 0x3247: return true; /* ㉇ */
    case 0x3250: return true; /* ㉐ */
    case 0x3260: return true; /* ㉠ */
    case 0x3261: return true; /* ㉡ */
    case 0x3262: return true; /* ㉢ */
    case 0x3263: return true; /* ㉣ */
    case 0x3264: return true; /* ㉤ */
    case 0x3265: return true; /* ㉥ */
    case 0x3266: return true; /* ㉦ */
    case 0x3267: return true; /* ㉧ */
    case 0x3268: return true; /* ㉨ */
    case 0x3269: return true; /* ㉩ */
    case 0x326a: return true; /* ㉪ */
    case 0x326b: return true; /* ㉫ */
    case 0x326c: return true; /* ㉬ */
    case 0x326d: return true; /* ㉭ */
    case 0x326e: return true; /* ㉮ */
    case 0x326f: return true; /* ㉯ */
    case 0x3270: return true; /* ㉰ */
    case 0x3271: return true; /* ㉱ */
    case 0x3272: return true; /* ㉲ */
    case 0x3273: return true; /* ㉳ */
    case 0x3274: return true; /* ㉴ */
    case 0x3275: return true; /* ㉵ */
    case 0x3276: return true; /* ㉶ */
    case 0x3277: return true; /* ㉷ */
    case 0x3278: return true; /* ㉸ */
    case 0x3279: return true; /* ㉹ */
    case 0x327a: return true; /* ㉺ */
    case 0x327b: return true; /* ㉻ */
    case 0x327c: return true; /* ㉼ */
    case 0x327d: return true; /* ㉽ */
    case 0x327e: return true; /* ㉾ */
    case 0x327f: return true; /* ㉿ */
    case 0x328a: return true; /* ㊊ */
    case 0x328b: return true; /* ㊋ */
    case 0x328c: return true; /* ㊌ */
    case 0x328d: return true; /* ㊍ */
    case 0x328e: return true; /* ㊎ */
    case 0x328f: return true; /* ㊏ */
    case 0x3290: return true; /* ㊐ */
    case 0x3291: return true; /* ㊑ */
    case 0x3292: return true; /* ㊒ */
    case 0x3293: return true; /* ㊓ */
    case 0x3294: return true; /* ㊔ */
    case 0x3295: return true; /* ㊕ */
    case 0x3296: return true; /* ㊖ */
    case 0x3297: return true; /* ㊗ */
    case 0x3298: return true; /* ㊘ */
    case 0x3299: return true; /* ㊙ */
    case 0x329a: return true; /* ㊚ */
    case 0x329b: return true; /* ㊛ */
    case 0x329c: return true; /* ㊜ */
    case 0x329d: return true; /* ㊝ */
    case 0x329e: return true; /* ㊞ */
    case 0x329f: return true; /* ㊟ */
    case 0x32a0: return true; /* ㊠ */
    case 0x32a1: return true; /* ㊡ */
    case 0x32a2: return true; /* ㊢ */
    case 0x32a3: return true; /* ㊣ */
    case 0x32a4: return true; /* ㊤ */
    case 0x32a5: return true; /* ㊥ */
    case 0x32a6: return true; /* ㊦ */
    case 0x32a7: return true; /* ㊧ */
    case 0x32a8: return true; /* ㊨ */
    case 0x32a9: return true; /* ㊩ */
    case 0x32aa: return true; /* ㊪ */
    case 0x32ab: return true; /* ㊫ */
    case 0x32ac: return true; /* ㊬ */
    case 0x32ad: return true; /* ㊭ */
    case 0x32ae: return true; /* ㊮ */
    case 0x32af: return true; /* ㊯ */
    case 0x32b0: return true; /* ㊰ */
    case 0x32c0: return true; /* ㋀ */
    case 0x32c1: return true; /* ㋁ */
    case 0x32c2: return true; /* ㋂ */
    case 0x32c3: return true; /* ㋃ */
    case 0x32c4: return true; /* ㋄ */
    case 0x32c5: return true; /* ㋅ */
    case 0x32c6: return true; /* ㋆ */
    case 0x32c7: return true; /* ㋇ */
    case 0x32c8: return true; /* ㋈ */
    case 0x32c9: return true; /* ㋉ */
    case 0x32ca: return true; /* ㋊ */
    case 0x32cb: return true; /* ㋋ */
    case 0x32cc: return true; /* ㋌ */
    case 0x32cd: return true; /* ㋍ */
    case 0x32ce: return true; /* ㋎ */
    case 0x32cf: return true; /* ㋏ */
    case 0x32d0: return true; /* ㋐ */
    case 0x32d1: return true; /* ㋑ */
    case 0x32d2: return true; /* ㋒ */
    case 0x32d3: return true; /* ㋓ */
    case 0x32d4: return true; /* ㋔ */
    case 0x32d5: return true; /* ㋕ */
    case 0x32d6: return true; /* ㋖ */
    case 0x32d7: return true; /* ㋗ */
    case 0x32d8: return true; /* ㋘ */
    case 0x32d9: return true; /* ㋙ */
    case 0x32da: return true; /* ㋚ */
    case 0x32db: return true; /* ㋛ */
    case 0x32dc: return true; /* ㋜ */
    case 0x32dd: return true; /* ㋝ */
    case 0x32de: return true; /* ㋞ */
    case 0x32df: return true; /* ㋟ */
    case 0x32e0: return true; /* ㋠ */
    case 0x32e1: return true; /* ㋡ */
    case 0x32e2: return true; /* ㋢ */
    case 0x32e3: return true; /* ㋣ */
    case 0x32e4: return true; /* ㋤ */
    case 0x32e5: return true; /* ㋥ */
    case 0x32e6: return true; /* ㋦ */
    case 0x32e7: return true; /* ㋧ */
    case 0x32e8: return true; /* ㋨ */
    case 0x32e9: return true; /* ㋩ */
    case 0x32ea: return true; /* ㋪ */
    case 0x32eb: return true; /* ㋫ */
    case 0x32ec: return true; /* ㋬ */
    case 0x32ed: return true; /* ㋭ */
    case 0x32ee: return true; /* ㋮ */
    case 0x32ef: return true; /* ㋯ */
    case 0x32f0: return true; /* ㋰ */
    case 0x32f1: return true; /* ㋱ */
    case 0x32f2: return true; /* ㋲ */
    case 0x32f3: return true; /* ㋳ */
    case 0x32f4: return true; /* ㋴ */
    case 0x32f5: return true; /* ㋵ */
    case 0x32f6: return true; /* ㋶ */
    case 0x32f7: return true; /* ㋷ */
    case 0x32f8: return true; /* ㋸ */
    case 0x32f9: return true; /* ㋹ */
    case 0x32fa: return true; /* ㋺ */
    case 0x32fb: return true; /* ㋻ */
    case 0x32fc: return true; /* ㋼ */
    case 0x32fd: return true; /* ㋽ */
    case 0x32fe: return true; /* ㋾ */
    case 0x32ff: return true; /* ㋿ */
    case 0x3300: return true; /* ㌀ */
    case 0x3301: return true; /* ㌁ */
    case 0x3302: return true; /* ㌂ */
    case 0x3303: return true; /* ㌃ */
    case 0x3304: return true; /* ㌄ */
    case 0x3305: return true; /* ㌅ */
    case 0x3306: return true; /* ㌆ */
    case 0x3307: return true; /* ㌇ */
    case 0x3308: return true; /* ㌈ */
    case 0x3309: return true; /* ㌉ */
    case 0x330a: return true; /* ㌊ */
    case 0x330b: return true; /* ㌋ */
    case 0x330c: return true; /* ㌌ */
    case 0x330d: return true; /* ㌍ */
    case 0x330e: return true; /* ㌎ */
    case 0x330f: return true; /* ㌏ */
    case 0x3310: return true; /* ㌐ */
    case 0x3311: return true; /* ㌑ */
    case 0x3312: return true; /* ㌒ */
    case 0x3313: return true; /* ㌓ */
    case 0x3314: return true; /* ㌔ */
    case 0x3315: return true; /* ㌕ */
    case 0x3316: return true; /* ㌖ */
    case 0x3317: return true; /* ㌗ */
    case 0x3318: return true; /* ㌘ */
    case 0x3319: return true; /* ㌙ */
    case 0x331a: return true; /* ㌚ */
    case 0x331b: return true; /* ㌛ */
    case 0x331c: return true; /* ㌜ */
    case 0x331d: return true; /* ㌝ */
    case 0x331e: return true; /* ㌞ */
    case 0x331f: return true; /* ㌟ */
    case 0x3320: return true; /* ㌠ */
    case 0x3321: return true; /* ㌡ */
    case 0x3322: return true; /* ㌢ */
    case 0x3323: return true; /* ㌣ */
    case 0x3324: return true; /* ㌤ */
    case 0x3325: return true; /* ㌥ */
    case 0x3326: return true; /* ㌦ */
    case 0x3327: return true; /* ㌧ */
    case 0x3328: return true; /* ㌨ */
    case 0x3329: return true; /* ㌩ */
    case 0x332a: return true; /* ㌪ */
    case 0x332b: return true; /* ㌫ */
    case 0x332c: return true; /* ㌬ */
    case 0x332d: return true; /* ㌭ */
    case 0x332e: return true; /* ㌮ */
    case 0x332f: return true; /* ㌯ */
    case 0x3330: return true; /* ㌰ */
    case 0x3331: return true; /* ㌱ */
    case 0x3332: return true; /* ㌲ */
    case 0x3333: return true; /* ㌳ */
    case 0x3334: return true; /* ㌴ */
    case 0x3335: return true; /* ㌵ */
    case 0x3336: return true; /* ㌶ */
    case 0x3337: return true; /* ㌷ */
    case 0x3338: return true; /* ㌸ */
    case 0x3339: return true; /* ㌹ */
    case 0x333a: return true; /* ㌺ */
    case 0x333b: return true; /* ㌻ */
    case 0x333c: return true; /* ㌼ */
    case 0x333d: return true; /* ㌽ */
    case 0x333e: return true; /* ㌾ */
    case 0x333f: return true; /* ㌿ */
    case 0x3340: return true; /* ㍀ */
    case 0x3341: return true; /* ㍁ */
    case 0x3342: return true; /* ㍂ */
    case 0x3343: return true; /* ㍃ */
    case 0x3344: return true; /* ㍄ */
    case 0x3345: return true; /* ㍅ */
    case 0x3346: return true; /* ㍆ */
    case 0x3347: return true; /* ㍇ */
    case 0x3348: return true; /* ㍈ */
    case 0x3349: return true; /* ㍉ */
    case 0x334a: return true; /* ㍊ */
    case 0x334b: return true; /* ㍋ */
    case 0x334c: return true; /* ㍌ */
    case 0x334d: return true; /* ㍍ */
    case 0x334e: return true; /* ㍎ */
    case 0x334f: return true; /* ㍏ */
    case 0x3350: return true; /* ㍐ */
    case 0x3351: return true; /* ㍑ */
    case 0x3352: return true; /* ㍒ */
    case 0x3353: return true; /* ㍓ */
    case 0x3354: return true; /* ㍔ */
    case 0x3355: return true; /* ㍕ */
    case 0x3356: return true; /* ㍖ */
    case 0x3357: return true; /* ㍗ */
    case 0x3358: return true; /* ㍘ */
    case 0x3359: return true; /* ㍙ */
    case 0x335a: return true; /* ㍚ */
    case 0x335b: return true; /* ㍛ */
    case 0x335c: return true; /* ㍜ */
    case 0x335d: return true; /* ㍝ */
    case 0x335e: return true; /* ㍞ */
    case 0x335f: return true; /* ㍟ */
    case 0x3360: return true; /* ㍠ */
    case 0x3361: return true; /* ㍡ */
    case 0x3362: return true; /* ㍢ */
    case 0x3363: return true; /* ㍣ */
    case 0x3364: return true; /* ㍤ */
    case 0x3365: return true; /* ㍥ */
    case 0x3366: return true; /* ㍦ */
    case 0x3367: return true; /* ㍧ */
    case 0x3368: return true; /* ㍨ */
    case 0x3369: return true; /* ㍩ */
    case 0x336a: return true; /* ㍪ */
    case 0x336b: return true; /* ㍫ */
    case 0x336c: return true; /* ㍬ */
    case 0x336d: return true; /* ㍭ */
    case 0x336e: return true; /* ㍮ */
    case 0x336f: return true; /* ㍯ */
    case 0x3370: return true; /* ㍰ */
    case 0x3371: return true; /* ㍱ */
    case 0x3372: return true; /* ㍲ */
    case 0x3373: return true; /* ㍳ */
    case 0x3374: return true; /* ㍴ */
    case 0x3375: return true; /* ㍵ */
    case 0x3376: return true; /* ㍶ */
    case 0x3377: return true; /* ㍷ */
    case 0x3378: return true; /* ㍸ */
    case 0x3379: return true; /* ㍹ */
    case 0x337a: return true; /* ㍺ */
    case 0x337b: return true; /* ㍻ */
    case 0x337c: return true; /* ㍼ */
    case 0x337d: return true; /* ㍽ */
    case 0x337e: return true; /* ㍾ */
    case 0x337f: return true; /* ㍿ */
    case 0x3380: return true; /* ㎀ */
    case 0x3381: return true; /* ㎁ */
    case 0x3382: return true; /* ㎂ */
    case 0x3383: return true; /* ㎃ */
    case 0x3384: return true; /* ㎄ */
    case 0x3385: return true; /* ㎅ */
    case 0x3386: return true; /* ㎆ */
    case 0x3387: return true; /* ㎇ */
    case 0x3388: return true; /* ㎈ */
    case 0x3389: return true; /* ㎉ */
    case 0x338a: return true; /* ㎊ */
    case 0x338b: return true; /* ㎋ */
    case 0x338c: return true; /* ㎌ */
    case 0x338d: return true; /* ㎍ */
    case 0x338e: return true; /* ㎎ */
    case 0x338f: return true; /* ㎏ */
    case 0x3390: return true; /* ㎐ */
    case 0x3391: return true; /* ㎑ */
    case 0x3392: return true; /* ㎒ */
    case 0x3393: return true; /* ㎓ */
    case 0x3394: return true; /* ㎔ */
    case 0x3395: return true; /* ㎕ */
    case 0x3396: return true; /* ㎖ */
    case 0x3397: return true; /* ㎗ */
    case 0x3398: return true; /* ㎘ */
    case 0x3399: return true; /* ㎙ */
    case 0x339a: return true; /* ㎚ */
    case 0x339b: return true; /* ㎛ */
    case 0x339c: return true; /* ㎜ */
    case 0x339d: return true; /* ㎝ */
    case 0x339e: return true; /* ㎞ */
    case 0x339f: return true; /* ㎟ */
    case 0x33a0: return true; /* ㎠ */
    case 0x33a1: return true; /* ㎡ */
    case 0x33a2: return true; /* ㎢ */
    case 0x33a3: return true; /* ㎣ */
    case 0x33a4: return true; /* ㎤ */
    case 0x33a5: return true; /* ㎥ */
    case 0x33a6: return true; /* ㎦ */
    case 0x33a7: return true; /* ㎧ */
    case 0x33a8: return true; /* ㎨ */
    case 0x33a9: return true; /* ㎩ */
    case 0x33aa: return true; /* ㎪ */
    case 0x33ab: return true; /* ㎫ */
    case 0x33ac: return true; /* ㎬ */
    case 0x33ad: return true; /* ㎭ */
    case 0x33ae: return true; /* ㎮ */
    case 0x33af: return true; /* ㎯ */
    case 0x33b0: return true; /* ㎰ */
    case 0x33b1: return true; /* ㎱ */
    case 0x33b2: return true; /* ㎲ */
    case 0x33b3: return true; /* ㎳ */
    case 0x33b4: return true; /* ㎴ */
    case 0x33b5: return true; /* ㎵ */
    case 0x33b6: return true; /* ㎶ */
    case 0x33b7: return true; /* ㎷ */
    case 0x33b8: return true; /* ㎸ */
    case 0x33b9: return true; /* ㎹ */
    case 0x33ba: return true; /* ㎺ */
    case 0x33bb: return true; /* ㎻ */
    case 0x33bc: return true; /* ㎼ */
    case 0x33bd: return true; /* ㎽ */
    case 0x33be: return true; /* ㎾ */
    case 0x33bf: return true; /* ㎿ */
    case 0x33c0: return true; /* ㏀ */
    case 0x33c1: return true; /* ㏁ */
    case 0x33c2: return true; /* ㏂ */
    case 0x33c3: return true; /* ㏃ */
    case 0x33c4: return true; /* ㏄ */
    case 0x33c5: return true; /* ㏅ */
    case 0x33c6: return true; /* ㏆ */
    case 0x33c7: return true; /* ㏇ */
    case 0x33c8: return true; /* ㏈ */
    case 0x33c9: return true; /* ㏉ */
    case 0x33ca: return true; /* ㏊ */
    case 0x33cb: return true; /* ㏋ */
    case 0x33cc: return true; /* ㏌ */
    case 0x33cd: return true; /* ㏍ */
    case 0x33ce: return true; /* ㏎ */
    case 0x33cf: return true; /* ㏏ */
    case 0x33d0: return true; /* ㏐ */
    case 0x33d1: return true; /* ㏑ */
    case 0x33d2: return true; /* ㏒ */
    case 0x33d3: return true; /* ㏓ */
    case 0x33d4: return true; /* ㏔ */
    case 0x33d5: return true; /* ㏕ */
    case 0x33d6: return true; /* ㏖ */
    case 0x33d7: return true; /* ㏗ */
    case 0x33d8: return true; /* ㏘ */
    case 0x33d9: return true; /* ㏙ */
    case 0x33da: return true; /* ㏚ */
    case 0x33db: return true; /* ㏛ */
    case 0x33dc: return true; /* ㏜ */
    case 0x33dd: return true; /* ㏝ */
    case 0x33de: return true; /* ㏞ */
    case 0x33df: return true; /* ㏟ */
    case 0x33e0: return true; /* ㏠ */
    case 0x33e1: return true; /* ㏡ */
    case 0x33e2: return true; /* ㏢ */
    case 0x33e3: return true; /* ㏣ */
    case 0x33e4: return true; /* ㏤ */
    case 0x33e5: return true; /* ㏥ */
    case 0x33e6: return true; /* ㏦ */
    case 0x33e7: return true; /* ㏧ */
    case 0x33e8: return true; /* ㏨ */
    case 0x33e9: return true; /* ㏩ */
    case 0x33ea: return true; /* ㏪ */
    case 0x33eb: return true; /* ㏫ */
    case 0x33ec: return true; /* ㏬ */
    case 0x33ed: return true; /* ㏭ */
    case 0x33ee: return true; /* ㏮ */
    case 0x33ef: return true; /* ㏯ */
    case 0x33f0: return true; /* ㏰ */
    case 0x33f1: return true; /* ㏱ */
    case 0x33f2: return true; /* ㏲ */
    case 0x33f3: return true; /* ㏳ */
    case 0x33f4: return true; /* ㏴ */
    case 0x33f5: return true; /* ㏵ */
    case 0x33f6: return true; /* ㏶ */
    case 0x33f7: return true; /* ㏷ */
    case 0x33f8: return true; /* ㏸ */
    case 0x33f9: return true; /* ㏹ */
    case 0x33fa: return true; /* ㏺ */
    case 0x33fb: return true; /* ㏻ */
    case 0x33fc: return true; /* ㏼ */
    case 0x33fd: return true; /* ㏽ */
    case 0x33fe: return true; /* ㏾ */
    case 0x33ff: return true; /* ㏿ */
    case 0x4dc0: return true; /* ䷀ */
    case 0x4dc1: return true; /* ䷁ */
    case 0x4dc2: return true; /* ䷂ */
    case 0x4dc3: return true; /* ䷃ */
    case 0x4dc4: return true; /* ䷄ */
    case 0x4dc5: return true; /* ䷅ */
    case 0x4dc6: return true; /* ䷆ */
    case 0x4dc7: return true; /* ䷇ */
    case 0x4dc8: return true; /* ䷈ */
    case 0x4dc9: return true; /* ䷉ */
    case 0x4dca: return true; /* ䷊ */
    case 0x4dcb: return true; /* ䷋ */
    case 0x4dcc: return true; /* ䷌ */
    case 0x4dcd: return true; /* ䷍ */
    case 0x4dce: return true; /* ䷎ */
    case 0x4dcf: return true; /* ䷏ */
    case 0x4dd0: return true; /* ䷐ */
    case 0x4dd1: return true; /* ䷑ */
    case 0x4dd2: return true; /* ䷒ */
    case 0x4dd3: return true; /* ䷓ */
    case 0x4dd4: return true; /* ䷔ */
    case 0x4dd5: return true; /* ䷕ */
    case 0x4dd6: return true; /* ䷖ */
    case 0x4dd7: return true; /* ䷗ */
    case 0x4dd8: return true; /* ䷘ */
    case 0x4dd9: return true; /* ䷙ */
    case 0x4dda: return true; /* ䷚ */
    case 0x4ddb: return true; /* ䷛ */
    case 0x4ddc: return true; /* ䷜ */
    case 0x4ddd: return true; /* ䷝ */
    case 0x4dde: return true; /* ䷞ */
    case 0x4ddf: return true; /* ䷟ */
    case 0x4de0: return true; /* ䷠ */
    case 0x4de1: return true; /* ䷡ */
    case 0x4de2: return true; /* ䷢ */
    case 0x4de3: return true; /* ䷣ */
    case 0x4de4: return true; /* ䷤ */
    case 0x4de5: return true; /* ䷥ */
    case 0x4de6: return true; /* ䷦ */
    case 0x4de7: return true; /* ䷧ */
    case 0x4de8: return true; /* ䷨ */
    case 0x4de9: return true; /* ䷩ */
    case 0x4dea: return true; /* ䷪ */
    case 0x4deb: return true; /* ䷫ */
    case 0x4dec: return true; /* ䷬ */
    case 0x4ded: return true; /* ䷭ */
    case 0x4dee: return true; /* ䷮ */
    case 0x4def: return true; /* ䷯ */
    case 0x4df0: return true; /* ䷰ */
    case 0x4df1: return true; /* ䷱ */
    case 0x4df2: return true; /* ䷲ */
    case 0x4df3: return true; /* ䷳ */
    case 0x4df4: return true; /* ䷴ */
    case 0x4df5: return true; /* ䷵ */
    case 0x4df6: return true; /* ䷶ */
    case 0x4df7: return true; /* ䷷ */
    case 0x4df8: return true; /* ䷸ */
    case 0x4df9: return true; /* ䷹ */
    case 0x4dfa: return true; /* ䷺ */
    case 0x4dfb: return true; /* ䷻ */
    case 0x4dfc: return true; /* ䷼ */
    case 0x4dfd: return true; /* ䷽ */
    case 0x4dfe: return true; /* ䷾ */
    case 0x4dff: return true; /* ䷿ */
    case 0xa490: return true; /* ꒐ */
    case 0xa491: return true; /* ꒑ */
    case 0xa492: return true; /* ꒒ */
    case 0xa493: return true; /* ꒓ */
    case 0xa494: return true; /* ꒔ */
    case 0xa495: return true; /* ꒕ */
    case 0xa496: return true; /* ꒖ */
    case 0xa497: return true; /* ꒗ */
    case 0xa498: return true; /* ꒘ */
    case 0xa499: return true; /* ꒙ */
    case 0xa49a: return true; /* ꒚ */
    case 0xa49b: return true; /* ꒛ */
    case 0xa49c: return true; /* ꒜ */
    case 0xa49d: return true; /* ꒝ */
    case 0xa49e: return true; /* ꒞ */
    case 0xa49f: return true; /* ꒟ */
    case 0xa4a0: return true; /* ꒠ */
    case 0xa4a1: return true; /* ꒡ */
    case 0xa4a2: return true; /* ꒢ */
    case 0xa4a3: return true; /* ꒣ */
    case 0xa4a4: return true; /* ꒤ */
    case 0xa4a5: return true; /* ꒥ */
    case 0xa4a6: return true; /* ꒦ */
    case 0xa4a7: return true; /* ꒧ */
    case 0xa4a8: return true; /* ꒨ */
    case 0xa4a9: return true; /* ꒩ */
    case 0xa4aa: return true; /* ꒪ */
    case 0xa4ab: return true; /* ꒫ */
    case 0xa4ac: return true; /* ꒬ */
    case 0xa4ad: return true; /* ꒭ */
    case 0xa4ae: return true; /* ꒮ */
    case 0xa4af: return true; /* ꒯ */
    case 0xa4b0: return true; /* ꒰ */
    case 0xa4b1: return true; /* ꒱ */
    case 0xa4b2: return true; /* ꒲ */
    case 0xa4b3: return true; /* ꒳ */
    case 0xa4b4: return true; /* ꒴ */
    case 0xa4b5: return true; /* ꒵ */
    case 0xa4b6: return true; /* ꒶ */
    case 0xa4b7: return true; /* ꒷ */
    case 0xa4b8: return true; /* ꒸ */
    case 0xa4b9: return true; /* ꒹ */
    case 0xa4ba: return true; /* ꒺ */
    case 0xa4bb: return true; /* ꒻ */
    case 0xa4bc: return true; /* ꒼ */
    case 0xa4bd: return true; /* ꒽ */
    case 0xa4be: return true; /* ꒾ */
    case 0xa4bf: return true; /* ꒿ */
    case 0xa4c0: return true; /* ꓀ */
    case 0xa4c1: return true; /* ꓁ */
    case 0xa4c2: return true; /* ꓂ */
    case 0xa4c3: return true; /* ꓃ */
    case 0xa4c4: return true; /* ꓄ */
    case 0xa4c5: return true; /* ꓅ */
    case 0xa4c6: return true; /* ꓆ */
    case 0xa4fe: return true; /* ꓾ */
    case 0xa4ff: return true; /* ꓿ */
    case 0xa60d: return true; /* ꘍ */
    case 0xa60e: return true; /* ꘎ */
    case 0xa60f: return true; /* ꘏ */
    case 0xa673: return true; /* ꙳ */
    case 0xa67e: return true; /* ꙾ */
    case 0xa6f2: return true; /* ꛲ */
    case 0xa6f3: return true; /* ꛳ */
    case 0xa6f4: return true; /* ꛴ */
    case 0xa6f5: return true; /* ꛵ */
    case 0xa6f6: return true; /* ꛶ */
    case 0xa6f7: return true; /* ꛷ */
    case 0xa700: return true; /* ꜀ */
    case 0xa701: return true; /* ꜁ */
    case 0xa702: return true; /* ꜂ */
    case 0xa703: return true; /* ꜃ */
    case 0xa704: return true; /* ꜄ */
    case 0xa705: return true; /* ꜅ */
    case 0xa706: return true; /* ꜆ */
    case 0xa707: return true; /* ꜇ */
    case 0xa708: return true; /* ꜈ */
    case 0xa709: return true; /* ꜉ */
    case 0xa70a: return true; /* ꜊ */
    case 0xa70b: return true; /* ꜋ */
    case 0xa70c: return true; /* ꜌ */
    case 0xa70d: return true; /* ꜍ */
    case 0xa70e: return true; /* ꜎ */
    case 0xa70f: return true; /* ꜏ */
    case 0xa710: return true; /* ꜐ */
    case 0xa711: return true; /* ꜑ */
    case 0xa712: return true; /* ꜒ */
    case 0xa713: return true; /* ꜓ */
    case 0xa714: return true; /* ꜔ */
    case 0xa715: return true; /* ꜕ */
    case 0xa716: return true; /* ꜖ */
    case 0xa720: return true; /* ꜠ */
    case 0xa721: return true; /* ꜡ */
    case 0xa789: return true; /* ꞉ */
    case 0xa78a: return true; /* ꞊ */
    case 0xa828: return true; /* ꠨ */
    case 0xa829: return true; /* ꠩ */
    case 0xa82a: return true; /* ꠪ */
    case 0xa82b: return true; /* ꠫ */
    case 0xa836: return true; /* ꠶ */
    case 0xa837: return true; /* ꠷ */
    case 0xa838: return true; /* ꠸ */
    case 0xa839: return true; /* ꠹ */
    case 0xa874: return true; /* ꡴ */
    case 0xa875: return true; /* ꡵ */
    case 0xa876: return true; /* ꡶ */
    case 0xa877: return true; /* ꡷ */
    case 0xa8ce: return true; /* ꣎ */
    case 0xa8cf: return true; /* ꣏ */
    case 0xa8f8: return true; /* ꣸ */
    case 0xa8f9: return true; /* ꣹ */
    case 0xa8fa: return true; /* ꣺ */
    case 0xa8fc: return true; /* ꣼ */
    case 0xa92e: return true; /* ꤮ */
    case 0xa92f: return true; /* ꤯ */
    case 0xa95f: return true; /* ꥟ */
    case 0xa9c1: return true; /* ꧁ */
    case 0xa9c2: return true; /* ꧂ */
    case 0xa9c3: return true; /* ꧃ */
    case 0xa9c4: return true; /* ꧄ */
    case 0xa9c5: return true; /* ꧅ */
    case 0xa9c6: return true; /* ꧆ */
    case 0xa9c7: return true; /* ꧇ */
    case 0xa9c8: return true; /* ꧈ */
    case 0xa9c9: return true; /* ꧉ */
    case 0xa9ca: return true; /* ꧊ */
    case 0xa9cb: return true; /* ꧋ */
    case 0xa9cc: return true; /* ꧌ */
    case 0xa9cd: return true; /* ꧍ */
    case 0xa9de: return true; /* ꧞ */
    case 0xa9df: return true; /* ꧟ */
    case 0xaa5c: return true; /* ꩜ */
    case 0xaa5d: return true; /* ꩝ */
    case 0xaa5e: return true; /* ꩞ */
    case 0xaa5f: return true; /* ꩟ */
    case 0xaa77: return true; /* ꩷ */
    case 0xaa78: return true; /* ꩸ */
    case 0xaa79: return true; /* ꩹ */
    case 0xaade: return true; /* ꫞ */
    case 0xaadf: return true; /* ꫟ */
    case 0xaaf0: return true; /* ꫰ */
    case 0xaaf1: return true; /* ꫱ */
    case 0xab5b: return true; /* ꭛ */
    case 0xab6a: return true; /* ꭪ */
    case 0xab6b: return true; /* ꭫ */
    case 0xabeb: return true; /* ꯫ */
    case 0xfb29: return true; /* ﬩ */
    case 0xfbb2: return true; /* ﮲ */
    case 0xfbb3: return true; /* ﮳ */
    case 0xfbb4: return true; /* ﮴ */
    case 0xfbb5: return true; /* ﮵ */
    case 0xfbb6: return true; /* ﮶ */
    case 0xfbb7: return true; /* ﮷ */
    case 0xfbb8: return true; /* ﮸ */
    case 0xfbb9: return true; /* ﮹ */
    case 0xfbba: return true; /* ﮺ */
    case 0xfbbb: return true; /* ﮻ */
    case 0xfbbc: return true; /* ﮼ */
    case 0xfbbd: return true; /* ﮽ */
    case 0xfbbe: return true; /* ﮾ */
    case 0xfbbf: return true; /* ﮿ */
    case 0xfbc0: return true; /* ﯀ */
    case 0xfbc1: return true; /* ﯁ */
    case 0xfd3e: return true; /* ﴾ */
    case 0xfd3f: return true; /* ﴿ */
    case 0xfdfc: return true; /* ﷼ */
    case 0xfdfd: return true; /* ﷽ */
    case 0xfe10: return true; /* ︐ */
    case 0xfe11: return true; /* ︑ */
    case 0xfe12: return true; /* ︒ */
    case 0xfe13: return true; /* ︓ */
    case 0xfe14: return true; /* ︔ */
    case 0xfe15: return true; /* ︕ */
    case 0xfe16: return true; /* ︖ */
    case 0xfe17: return true; /* ︗ */
    case 0xfe18: return true; /* ︘ */
    case 0xfe19: return true; /* ︙ */
    case 0xfe30: return true; /* ︰ */
    case 0xfe31: return true; /* ︱ */
    case 0xfe32: return true; /* ︲ */
    case 0xfe33: return true; /* ︳ */
    case 0xfe34: return true; /* ︴ */
    case 0xfe35: return true; /* ︵ */
    case 0xfe36: return true; /* ︶ */
    case 0xfe37: return true; /* ︷ */
    case 0xfe38: return true; /* ︸ */
    case 0xfe39: return true; /* ︹ */
    case 0xfe3a: return true; /* ︺ */
    case 0xfe3b: return true; /* ︻ */
    case 0xfe3c: return true; /* ︼ */
    case 0xfe3d: return true; /* ︽ */
    case 0xfe3e: return true; /* ︾ */
    case 0xfe3f: return true; /* ︿ */
    case 0xfe40: return true; /* ﹀ */
    case 0xfe41: return true; /* ﹁ */
    case 0xfe42: return true; /* ﹂ */
    case 0xfe43: return true; /* ﹃ */
    case 0xfe44: return true; /* ﹄ */
    case 0xfe45: return true; /* ﹅ */
    case 0xfe46: return true; /* ﹆ */
    case 0xfe47: return true; /* ﹇ */
    case 0xfe48: return true; /* ﹈ */
    case 0xfe49: return true; /* ﹉ */
    case 0xfe4a: return true; /* ﹊ */
    case 0xfe4b: return true; /* ﹋ */
    case 0xfe4c: return true; /* ﹌ */
    case 0xfe4d: return true; /* ﹍ */
    case 0xfe4e: return true; /* ﹎ */
    case 0xfe4f: return true; /* ﹏ */
    case 0xfe50: return true; /* ﹐ */
    case 0xfe51: return true; /* ﹑ */
    case 0xfe52: return true; /* ﹒ */
    case 0xfe54: return true; /* ﹔ */
    case 0xfe55: return true; /* ﹕ */
    case 0xfe56: return true; /* ﹖ */
    case 0xfe57: return true; /* ﹗ */
    case 0xfe58: return true; /* ﹘ */
    case 0xfe59: return true; /* ﹙ */
    case 0xfe5a: return true; /* ﹚ */
    case 0xfe5b: return true; /* ﹛ */
    case 0xfe5c: return true; /* ﹜ */
    case 0xfe5d: return true; /* ﹝ */
    case 0xfe5e: return true; /* ﹞ */
    case 0xfe5f: return true; /* ﹟ */
    case 0xfe60: return true; /* ﹠ */
    case 0xfe61: return true; /* ﹡ */
    case 0xfe62: return true; /* ﹢ */
    case 0xfe63: return true; /* ﹣ */
    case 0xfe64: return true; /* ﹤ */
    case 0xfe65: return true; /* ﹥ */
    case 0xfe66: return true; /* ﹦ */
    case 0xfe68: return true; /* ﹨ */
    case 0xfe69: return true; /* ﹩ */
    case 0xfe6a: return true; /* ﹪ */
    case 0xfe6b: return true; /* ﹫ */
    case 0xff01: return true; /* ！ */
    case 0xff02: return true; /* ＂ */
    case 0xff03: return true; /* ＃ */
    case 0xff04: return true; /* ＄ */
    case 0xff05: return true; /* ％ */
    case 0xff06: return true; /* ＆ */
    case 0xff07: return true; /* ＇ */
    case 0xff08: return true; /* （ */
    case 0xff09: return true; /* ） */
    case 0xff0a: return true; /* ＊ */
    case 0xff0b: return true; /* ＋ */
    case 0xff0c: return true; /* ， */
    case 0xff0d: return true; /* － */
    case 0xff0e: return true; /* ． */
    case 0xff0f: return true; /* ／ */
    case 0xff1a: return true; /* ： */
    case 0xff1b: return true; /* ； */
    case 0xff1c: return true; /* ＜ */
    case 0xff1d: return true; /* ＝ */
    case 0xff1e: return true; /* ＞ */
    case 0xff1f: return true; /* ？ */
    case 0xff20: return true; /* ＠ */
    case 0xff3b: return true; /* ［ */
    case 0xff3c: return true; /* ＼ */
    case 0xff3d: return true; /* ］ */
    case 0xff3e: return true; /* ＾ */
    case 0xff3f: return true; /* ＿ */
    case 0xff40: return true; /* ｀ */
    case 0xff5b: return true; /* ｛ */
    case 0xff5c: return true; /* ｜ */
    case 0xff5d: return true; /* ｝ */
    case 0xff5e: return true; /* ～ */
    case 0xff5f: return true; /* ｟ */
    case 0xff60: return true; /* ｠ */
    case 0xff61: return true; /* ｡ */
    case 0xff62: return true; /* ｢ */
    case 0xff63: return true; /* ｣ */
    case 0xff64: return true; /* ､ */
    case 0xff65: return true; /* ･ */
    case 0xffe0: return true; /* ￠ */
    case 0xffe1: return true; /* ￡ */
    case 0xffe2: return true; /* ￢ */
    case 0xffe3: return true; /* ￣ */
    case 0xffe4: return true; /* ￤ */
    case 0xffe5: return true; /* ￥ */
    case 0xffe6: return true; /* ￦ */
    case 0xffe8: return true; /* ￨ */
    case 0xffe9: return true; /* ￩ */
    case 0xffea: return true; /* ￪ */
    case 0xffeb: return true; /* ￫ */
    case 0xffec: return true; /* ￬ */
    case 0xffed: return true; /* ￭ */
    case 0xffee: return true; /* ￮ */
    case 0xfffc: return true; /* ￼ */
    case 0xfffd: return true; /* � */
    case 0x10100: return true; /* 𐄀 */
    case 0x10101: return true; /* 𐄁 */
    case 0x10102: return true; /* 𐄂 */
    case 0x10137: return true; /* 𐄷 */
    case 0x10138: return true; /* 𐄸 */
    case 0x10139: return true; /* 𐄹 */
    case 0x1013a: return true; /* 𐄺 */
    case 0x1013b: return true; /* 𐄻 */
    case 0x1013c: return true; /* 𐄼 */
    case 0x1013d: return true; /* 𐄽 */
    case 0x1013e: return true; /* 𐄾 */
    case 0x1013f: return true; /* 𐄿 */
    case 0x10179: return true; /* 𐅹 */
    case 0x1017a: return true; /* 𐅺 */
    case 0x1017b: return true; /* 𐅻 */
    case 0x1017c: return true; /* 𐅼 */
    case 0x1017d: return true; /* 𐅽 */
    case 0x1017e: return true; /* 𐅾 */
    case 0x1017f: return true; /* 𐅿 */
    case 0x10180: return true; /* 𐆀 */
    case 0x10181: return true; /* 𐆁 */
    case 0x10182: return true; /* 𐆂 */
    case 0x10183: return true; /* 𐆃 */
    case 0x10184: return true; /* 𐆄 */
    case 0x10185: return true; /* 𐆅 */
    case 0x10186: return true; /* 𐆆 */
    case 0x10187: return true; /* 𐆇 */
    case 0x10188: return true; /* 𐆈 */
    case 0x10189: return true; /* 𐆉 */
    case 0x1018c: return true; /* 𐆌 */
    case 0x1018d: return true; /* 𐆍 */
    case 0x1018e: return true; /* 𐆎 */
    case 0x10190: return true; /* 𐆐 */
    case 0x10191: return true; /* 𐆑 */
    case 0x10192: return true; /* 𐆒 */
    case 0x10193: return true; /* 𐆓 */
    case 0x10194: return true; /* 𐆔 */
    case 0x10195: return true; /* 𐆕 */
    case 0x10196: return true; /* 𐆖 */
    case 0x10197: return true; /* 𐆗 */
    case 0x10198: return true; /* 𐆘 */
    case 0x10199: return true; /* 𐆙 */
    case 0x1019a: return true; /* 𐆚 */
    case 0x1019b: return true; /* 𐆛 */
    case 0x1019c: return true; /* 𐆜 */
    case 0x101a0: return true; /* 𐆠 */
    case 0x101d0: return true; /* 𐇐 */
    case 0x101d1: return true; /* 𐇑 */
    case 0x101d2: return true; /* 𐇒 */
    case 0x101d3: return true; /* 𐇓 */
    case 0x101d4: return true; /* 𐇔 */
    case 0x101d5: return true; /* 𐇕 */
    case 0x101d6: return true; /* 𐇖 */
    case 0x101d7: return true; /* 𐇗 */
    case 0x101d8: return true; /* 𐇘 */
    case 0x101d9: return true; /* 𐇙 */
    case 0x101da: return true; /* 𐇚 */
    case 0x101db: return true; /* 𐇛 */
    case 0x101dc: return true; /* 𐇜 */
    case 0x101dd: return true; /* 𐇝 */
    case 0x101de: return true; /* 𐇞 */
    case 0x101df: return true; /* 𐇟 */
    case 0x101e0: return true; /* 𐇠 */
    case 0x101e1: return true; /* 𐇡 */
    case 0x101e2: return true; /* 𐇢 */
    case 0x101e3: return true; /* 𐇣 */
    case 0x101e4: return true; /* 𐇤 */
    case 0x101e5: return true; /* 𐇥 */
    case 0x101e6: return true; /* 𐇦 */
    case 0x101e7: return true; /* 𐇧 */
    case 0x101e8: return true; /* 𐇨 */
    case 0x101e9: return true; /* 𐇩 */
    case 0x101ea: return true; /* 𐇪 */
    case 0x101eb: return true; /* 𐇫 */
    case 0x101ec: return true; /* 𐇬 */
    case 0x101ed: return true; /* 𐇭 */
    case 0x101ee: return true; /* 𐇮 */
    case 0x101ef: return true; /* 𐇯 */
    case 0x101f0: return true; /* 𐇰 */
    case 0x101f1: return true; /* 𐇱 */
    case 0x101f2: return true; /* 𐇲 */
    case 0x101f3: return true; /* 𐇳 */
    case 0x101f4: return true; /* 𐇴 */
    case 0x101f5: return true; /* 𐇵 */
    case 0x101f6: return true; /* 𐇶 */
    case 0x101f7: return true; /* 𐇷 */
    case 0x101f8: return true; /* 𐇸 */
    case 0x101f9: return true; /* 𐇹 */
    case 0x101fa: return true; /* 𐇺 */
    case 0x101fb: return true; /* 𐇻 */
    case 0x101fc: return true; /* 𐇼 */
    case 0x1039f: return true; /* 𐎟 */
    case 0x103d0: return true; /* 𐏐 */
    case 0x1056f: return true; /* 𐕯 */
    case 0x10857: return true; /* 𐡗 */
    case 0x10877: return true; /* 𐡷 */
    case 0x10878: return true; /* 𐡸 */
    case 0x1091f: return true; /* 𐤟 */
    case 0x1093f: return true; /* 𐤿 */
    case 0x10a50: return true; /* 𐩐 */
    case 0x10a51: return true; /* 𐩑 */
    case 0x10a52: return true; /* 𐩒 */
    case 0x10a53: return true; /* 𐩓 */
    case 0x10a54: return true; /* 𐩔 */
    case 0x10a55: return true; /* 𐩕 */
    case 0x10a56: return true; /* 𐩖 */
    case 0x10a57: return true; /* 𐩗 */
    case 0x10a58: return true; /* 𐩘 */
    case 0x10a7f: return true; /* 𐩿 */
    case 0x10ac8: return true; /* 𐫈 */
    case 0x10af0: return true; /* 𐫰 */
    case 0x10af1: return true; /* 𐫱 */
    case 0x10af2: return true; /* 𐫲 */
    case 0x10af3: return true; /* 𐫳 */
    case 0x10af4: return true; /* 𐫴 */
    case 0x10af5: return true; /* 𐫵 */
    case 0x10af6: return true; /* 𐫶 */
    case 0x10b39: return true; /* 𐬹 */
    case 0x10b3a: return true; /* 𐬺 */
    case 0x10b3b: return true; /* 𐬻 */
    case 0x10b3c: return true; /* 𐬼 */
    case 0x10b3d: return true; /* 𐬽 */
    case 0x10b3e: return true; /* 𐬾 */
    case 0x10b3f: return true; /* 𐬿 */
    case 0x10b99: return true; /* 𐮙 */
    case 0x10b9a: return true; /* 𐮚 */
    case 0x10b9b: return true; /* 𐮛 */
    case 0x10b9c: return true; /* 𐮜 */
    case 0x10ead: return true; /* 𐺭 */
    case 0x10f55: return true; /* 𐽕 */
    case 0x10f56: return true; /* 𐽖 */
    case 0x10f57: return true; /* 𐽗 */
    case 0x10f58: return true; /* 𐽘 */
    case 0x10f59: return true; /* 𐽙 */
    case 0x11047: return true; /* 𑁇 */
    case 0x11048: return true; /* 𑁈 */
    case 0x11049: return true; /* 𑁉 */
    case 0x1104a: return true; /* 𑁊 */
    case 0x1104b: return true; /* 𑁋 */
    case 0x1104c: return true; /* 𑁌 */
    case 0x1104d: return true; /* 𑁍 */
    case 0x110bb: return true; /* 𑂻 */
    case 0x110bc: return true; /* 𑂼 */
    case 0x110be: return true; /* 𑂾 */
    case 0x110bf: return true; /* 𑂿 */
    case 0x110c0: return true; /* 𑃀 */
    case 0x110c1: return true; /* 𑃁 */
    case 0x11140: return true; /* 𑅀 */
    case 0x11141: return true; /* 𑅁 */
    case 0x11142: return true; /* 𑅂 */
    case 0x11143: return true; /* 𑅃 */
    case 0x11174: return true; /* 𑅴 */
    case 0x11175: return true; /* 𑅵 */
    case 0x111c5: return true; /* 𑇅 */
    case 0x111c6: return true; /* 𑇆 */
    case 0x111c7: return true; /* 𑇇 */
    case 0x111c8: return true; /* 𑇈 */
    case 0x111cd: return true; /* 𑇍 */
    case 0x111db: return true; /* 𑇛 */
    case 0x111dd: return true; /* 𑇝 */
    case 0x111de: return true; /* 𑇞 */
    case 0x111df: return true; /* 𑇟 */
    case 0x11238: return true; /* 𑈸 */
    case 0x11239: return true; /* 𑈹 */
    case 0x1123a: return true; /* 𑈺 */
    case 0x1123b: return true; /* 𑈻 */
    case 0x1123c: return true; /* 𑈼 */
    case 0x1123d: return true; /* 𑈽 */
    case 0x112a9: return true; /* 𑊩 */
    case 0x1144b: return true; /* 𑑋 */
    case 0x1144c: return true; /* 𑑌 */
    case 0x1144d: return true; /* 𑑍 */
    case 0x1144e: return true; /* 𑑎 */
    case 0x1144f: return true; /* 𑑏 */
    case 0x1145a: return true; /* 𑑚 */
    case 0x1145b: return true; /* 𑑛 */
    case 0x1145d: return true; /* 𑑝 */
    case 0x114c6: return true; /* 𑓆 */
    case 0x115c1: return true; /* 𑗁 */
    case 0x115c2: return true; /* 𑗂 */
    case 0x115c3: return true; /* 𑗃 */
    case 0x115c4: return true; /* 𑗄 */
    case 0x115c5: return true; /* 𑗅 */
    case 0x115c6: return true; /* 𑗆 */
    case 0x115c7: return true; /* 𑗇 */
    case 0x115c8: return true; /* 𑗈 */
    case 0x115c9: return true; /* 𑗉 */
    case 0x115ca: return true; /* 𑗊 */
    case 0x115cb: return true; /* 𑗋 */
    case 0x115cc: return true; /* 𑗌 */
    case 0x115cd: return true; /* 𑗍 */
    case 0x115ce: return true; /* 𑗎 */
    case 0x115cf: return true; /* 𑗏 */
    case 0x115d0: return true; /* 𑗐 */
    case 0x115d1: return true; /* 𑗑 */
    case 0x115d2: return true; /* 𑗒 */
    case 0x115d3: return true; /* 𑗓 */
    case 0x115d4: return true; /* 𑗔 */
    case 0x115d5: return true; /* 𑗕 */
    case 0x115d6: return true; /* 𑗖 */
    case 0x115d7: return true; /* 𑗗 */
    case 0x11641: return true; /* 𑙁 */
    case 0x11642: return true; /* 𑙂 */
    case 0x11643: return true; /* 𑙃 */
    case 0x11660: return true; /* 𑙠 */
    case 0x11661: return true; /* 𑙡 */
    case 0x11662: return true; /* 𑙢 */
    case 0x11663: return true; /* 𑙣 */
    case 0x11664: return true; /* 𑙤 */
    case 0x11665: return true; /* 𑙥 */
    case 0x11666: return true; /* 𑙦 */
    case 0x11667: return true; /* 𑙧 */
    case 0x11668: return true; /* 𑙨 */
    case 0x11669: return true; /* 𑙩 */
    case 0x1166a: return true; /* 𑙪 */
    case 0x1166b: return true; /* 𑙫 */
    case 0x1166c: return true; /* 𑙬 */
    case 0x1173c: return true; /* 𑜼 */
    case 0x1173d: return true; /* 𑜽 */
    case 0x1173e: return true; /* 𑜾 */
    case 0x1173f: return true; /* 𑜿 */
    case 0x1183b: return true; /* 𑠻 */
    case 0x11944: return true; /* 𑥄 */
    case 0x11945: return true; /* 𑥅 */
    case 0x11946: return true; /* 𑥆 */
    case 0x119e2: return true; /* 𑧢 */
    case 0x11a3f: return true; /* 𑨿 */
    case 0x11a40: return true; /* 𑩀 */
    case 0x11a41: return true; /* 𑩁 */
    case 0x11a42: return true; /* 𑩂 */
    case 0x11a43: return true; /* 𑩃 */
    case 0x11a44: return true; /* 𑩄 */
    case 0x11a45: return true; /* 𑩅 */
    case 0x11a46: return true; /* 𑩆 */
    case 0x11a9a: return true; /* 𑪚 */
    case 0x11a9b: return true; /* 𑪛 */
    case 0x11a9c: return true; /* 𑪜 */
    case 0x11a9e: return true; /* 𑪞 */
    case 0x11a9f: return true; /* 𑪟 */
    case 0x11aa0: return true; /* 𑪠 */
    case 0x11aa1: return true; /* 𑪡 */
    case 0x11aa2: return true; /* 𑪢 */
    case 0x11c41: return true; /* 𑱁 */
    case 0x11c42: return true; /* 𑱂 */
    case 0x11c43: return true; /* 𑱃 */
    case 0x11c44: return true; /* 𑱄 */
    case 0x11c45: return true; /* 𑱅 */
    case 0x11c70: return true; /* 𑱰 */
    case 0x11c71: return true; /* 𑱱 */
    case 0x11ef7: return true; /* 𑻷 */
    case 0x11ef8: return true; /* 𑻸 */
    case 0x11fd5: return true; /* 𑿕 */
    case 0x11fd6: return true; /* 𑿖 */
    case 0x11fd7: return true; /* 𑿗 */
    case 0x11fd8: return true; /* 𑿘 */
    case 0x11fd9: return true; /* 𑿙 */
    case 0x11fda: return true; /* 𑿚 */
    case 0x11fdb: return true; /* 𑿛 */
    case 0x11fdc: return true; /* 𑿜 */
    case 0x11fdd: return true; /* 𑿝 */
    case 0x11fde: return true; /* 𑿞 */
    case 0x11fdf: return true; /* 𑿟 */
    case 0x11fe0: return true; /* 𑿠 */
    case 0x11fe1: return true; /* 𑿡 */
    case 0x11fe2: return true; /* 𑿢 */
    case 0x11fe3: return true; /* 𑿣 */
    case 0x11fe4: return true; /* 𑿤 */
    case 0x11fe5: return true; /* 𑿥 */
    case 0x11fe6: return true; /* 𑿦 */
    case 0x11fe7: return true; /* 𑿧 */
    case 0x11fe8: return true; /* 𑿨 */
    case 0x11fe9: return true; /* 𑿩 */
    case 0x11fea: return true; /* 𑿪 */
    case 0x11feb: return true; /* 𑿫 */
    case 0x11fec: return true; /* 𑿬 */
    case 0x11fed: return true; /* 𑿭 */
    case 0x11fee: return true; /* 𑿮 */
    case 0x11fef: return true; /* 𑿯 */
    case 0x11ff0: return true; /* 𑿰 */
    case 0x11ff1: return true; /* 𑿱 */
    case 0x11fff: return true; /* 𑿿 */
    case 0x12470: return true; /* 𒑰 */
    case 0x12471: return true; /* 𒑱 */
    case 0x12472: return true; /* 𒑲 */
    case 0x12473: return true; /* 𒑳 */
    case 0x12474: return true; /* 𒑴 */
    case 0x16a6e: return true; /* 𖩮 */
    case 0x16a6f: return true; /* 𖩯 */
    case 0x16af5: return true; /* 𖫵 */
    case 0x16b37: return true; /* 𖬷 */
    case 0x16b38: return true; /* 𖬸 */
    case 0x16b39: return true; /* 𖬹 */
    case 0x16b3a: return true; /* 𖬺 */
    case 0x16b3b: return true; /* 𖬻 */
    case 0x16b3c: return true; /* 𖬼 */
    case 0x16b3d: return true; /* 𖬽 */
    case 0x16b3e: return true; /* 𖬾 */
    case 0x16b3f: return true; /* 𖬿 */
    case 0x16b44: return true; /* 𖭄 */
    case 0x16b45: return true; /* 𖭅 */
    case 0x16e97: return true; /* 𖺗 */
    case 0x16e98: return true; /* 𖺘 */
    case 0x16e99: return true; /* 𖺙 */
    case 0x16e9a: return true; /* 𖺚 */
    case 0x16fe2: return true; /* 𖿢 */
    case 0x1bc9c: return true; /* 𛲜 */
    case 0x1bc9f: return true; /* 𛲟 */
    case 0x1d000: return true; /* 𝀀 */
    case 0x1d001: return true; /* 𝀁 */
    case 0x1d002: return true; /* 𝀂 */
    case 0x1d003: return true; /* 𝀃 */
    case 0x1d004: return true; /* 𝀄 */
    case 0x1d005: return true; /* 𝀅 */
    case 0x1d006: return true; /* 𝀆 */
    case 0x1d007: return true; /* 𝀇 */
    case 0x1d008: return true; /* 𝀈 */
    case 0x1d009: return true; /* 𝀉 */
    case 0x1d00a: return true; /* 𝀊 */
    case 0x1d00b: return true; /* 𝀋 */
    case 0x1d00c: return true; /* 𝀌 */
    case 0x1d00d: return true; /* 𝀍 */
    case 0x1d00e: return true; /* 𝀎 */
    case 0x1d00f: return true; /* 𝀏 */
    case 0x1d010: return true; /* 𝀐 */
    case 0x1d011: return true; /* 𝀑 */
    case 0x1d012: return true; /* 𝀒 */
    case 0x1d013: return true; /* 𝀓 */
    case 0x1d014: return true; /* 𝀔 */
    case 0x1d015: return true; /* 𝀕 */
    case 0x1d016: return true; /* 𝀖 */
    case 0x1d017: return true; /* 𝀗 */
    case 0x1d018: return true; /* 𝀘 */
    case 0x1d019: return true; /* 𝀙 */
    case 0x1d01a: return true; /* 𝀚 */
    case 0x1d01b: return true; /* 𝀛 */
    case 0x1d01c: return true; /* 𝀜 */
    case 0x1d01d: return true; /* 𝀝 */
    case 0x1d01e: return true; /* 𝀞 */
    case 0x1d01f: return true; /* 𝀟 */
    case 0x1d020: return true; /* 𝀠 */
    case 0x1d021: return true; /* 𝀡 */
    case 0x1d022: return true; /* 𝀢 */
    case 0x1d023: return true; /* 𝀣 */
    case 0x1d024: return true; /* 𝀤 */
    case 0x1d025: return true; /* 𝀥 */
    case 0x1d026: return true; /* 𝀦 */
    case 0x1d027: return true; /* 𝀧 */
    case 0x1d028: return true; /* 𝀨 */
    case 0x1d029: return true; /* 𝀩 */
    case 0x1d02a: return true; /* 𝀪 */
    case 0x1d02b: return true; /* 𝀫 */
    case 0x1d02c: return true; /* 𝀬 */
    case 0x1d02d: return true; /* 𝀭 */
    case 0x1d02e: return true; /* 𝀮 */
    case 0x1d02f: return true; /* 𝀯 */
    case 0x1d030: return true; /* 𝀰 */
    case 0x1d031: return true; /* 𝀱 */
    case 0x1d032: return true; /* 𝀲 */
    case 0x1d033: return true; /* 𝀳 */
    case 0x1d034: return true; /* 𝀴 */
    case 0x1d035: return true; /* 𝀵 */
    case 0x1d036: return true; /* 𝀶 */
    case 0x1d037: return true; /* 𝀷 */
    case 0x1d038: return true; /* 𝀸 */
    case 0x1d039: return true; /* 𝀹 */
    case 0x1d03a: return true; /* 𝀺 */
    case 0x1d03b: return true; /* 𝀻 */
    case 0x1d03c: return true; /* 𝀼 */
    case 0x1d03d: return true; /* 𝀽 */
    case 0x1d03e: return true; /* 𝀾 */
    case 0x1d03f: return true; /* 𝀿 */
    case 0x1d040: return true; /* 𝁀 */
    case 0x1d041: return true; /* 𝁁 */
    case 0x1d042: return true; /* 𝁂 */
    case 0x1d043: return true; /* 𝁃 */
    case 0x1d044: return true; /* 𝁄 */
    case 0x1d045: return true; /* 𝁅 */
    case 0x1d046: return true; /* 𝁆 */
    case 0x1d047: return true; /* 𝁇 */
    case 0x1d048: return true; /* 𝁈 */
    case 0x1d049: return true; /* 𝁉 */
    case 0x1d04a: return true; /* 𝁊 */
    case 0x1d04b: return true; /* 𝁋 */
    case 0x1d04c: return true; /* 𝁌 */
    case 0x1d04d: return true; /* 𝁍 */
    case 0x1d04e: return true; /* 𝁎 */
    case 0x1d04f: return true; /* 𝁏 */
    case 0x1d050: return true; /* 𝁐 */
    case 0x1d051: return true; /* 𝁑 */
    case 0x1d052: return true; /* 𝁒 */
    case 0x1d053: return true; /* 𝁓 */
    case 0x1d054: return true; /* 𝁔 */
    case 0x1d055: return true; /* 𝁕 */
    case 0x1d056: return true; /* 𝁖 */
    case 0x1d057: return true; /* 𝁗 */
    case 0x1d058: return true; /* 𝁘 */
    case 0x1d059: return true; /* 𝁙 */
    case 0x1d05a: return true; /* 𝁚 */
    case 0x1d05b: return true; /* 𝁛 */
    case 0x1d05c: return true; /* 𝁜 */
    case 0x1d05d: return true; /* 𝁝 */
    case 0x1d05e: return true; /* 𝁞 */
    case 0x1d05f: return true; /* 𝁟 */
    case 0x1d060: return true; /* 𝁠 */
    case 0x1d061: return true; /* 𝁡 */
    case 0x1d062: return true; /* 𝁢 */
    case 0x1d063: return true; /* 𝁣 */
    case 0x1d064: return true; /* 𝁤 */
    case 0x1d065: return true; /* 𝁥 */
    case 0x1d066: return true; /* 𝁦 */
    case 0x1d067: return true; /* 𝁧 */
    case 0x1d068: return true; /* 𝁨 */
    case 0x1d069: return true; /* 𝁩 */
    case 0x1d06a: return true; /* 𝁪 */
    case 0x1d06b: return true; /* 𝁫 */
    case 0x1d06c: return true; /* 𝁬 */
    case 0x1d06d: return true; /* 𝁭 */
    case 0x1d06e: return true; /* 𝁮 */
    case 0x1d06f: return true; /* 𝁯 */
    case 0x1d070: return true; /* 𝁰 */
    case 0x1d071: return true; /* 𝁱 */
    case 0x1d072: return true; /* 𝁲 */
    case 0x1d073: return true; /* 𝁳 */
    case 0x1d074: return true; /* 𝁴 */
    case 0x1d075: return true; /* 𝁵 */
    case 0x1d076: return true; /* 𝁶 */
    case 0x1d077: return true; /* 𝁷 */
    case 0x1d078: return true; /* 𝁸 */
    case 0x1d079: return true; /* 𝁹 */
    case 0x1d07a: return true; /* 𝁺 */
    case 0x1d07b: return true; /* 𝁻 */
    case 0x1d07c: return true; /* 𝁼 */
    case 0x1d07d: return true; /* 𝁽 */
    case 0x1d07e: return true; /* 𝁾 */
    case 0x1d07f: return true; /* 𝁿 */
    case 0x1d080: return true; /* 𝂀 */
    case 0x1d081: return true; /* 𝂁 */
    case 0x1d082: return true; /* 𝂂 */
    case 0x1d083: return true; /* 𝂃 */
    case 0x1d084: return true; /* 𝂄 */
    case 0x1d085: return true; /* 𝂅 */
    case 0x1d086: return true; /* 𝂆 */
    case 0x1d087: return true; /* 𝂇 */
    case 0x1d088: return true; /* 𝂈 */
    case 0x1d089: return true; /* 𝂉 */
    case 0x1d08a: return true; /* 𝂊 */
    case 0x1d08b: return true; /* 𝂋 */
    case 0x1d08c: return true; /* 𝂌 */
    case 0x1d08d: return true; /* 𝂍 */
    case 0x1d08e: return true; /* 𝂎 */
    case 0x1d08f: return true; /* 𝂏 */
    case 0x1d090: return true; /* 𝂐 */
    case 0x1d091: return true; /* 𝂑 */
    case 0x1d092: return true; /* 𝂒 */
    case 0x1d093: return true; /* 𝂓 */
    case 0x1d094: return true; /* 𝂔 */
    case 0x1d095: return true; /* 𝂕 */
    case 0x1d096: return true; /* 𝂖 */
    case 0x1d097: return true; /* 𝂗 */
    case 0x1d098: return true; /* 𝂘 */
    case 0x1d099: return true; /* 𝂙 */
    case 0x1d09a: return true; /* 𝂚 */
    case 0x1d09b: return true; /* 𝂛 */
    case 0x1d09c: return true; /* 𝂜 */
    case 0x1d09d: return true; /* 𝂝 */
    case 0x1d09e: return true; /* 𝂞 */
    case 0x1d09f: return true; /* 𝂟 */
    case 0x1d0a0: return true; /* 𝂠 */
    case 0x1d0a1: return true; /* 𝂡 */
    case 0x1d0a2: return true; /* 𝂢 */
    case 0x1d0a3: return true; /* 𝂣 */
    case 0x1d0a4: return true; /* 𝂤 */
    case 0x1d0a5: return true; /* 𝂥 */
    case 0x1d0a6: return true; /* 𝂦 */
    case 0x1d0a7: return true; /* 𝂧 */
    case 0x1d0a8: return true; /* 𝂨 */
    case 0x1d0a9: return true; /* 𝂩 */
    case 0x1d0aa: return true; /* 𝂪 */
    case 0x1d0ab: return true; /* 𝂫 */
    case 0x1d0ac: return true; /* 𝂬 */
    case 0x1d0ad: return true; /* 𝂭 */
    case 0x1d0ae: return true; /* 𝂮 */
    case 0x1d0af: return true; /* 𝂯 */
    case 0x1d0b0: return true; /* 𝂰 */
    case 0x1d0b1: return true; /* 𝂱 */
    case 0x1d0b2: return true; /* 𝂲 */
    case 0x1d0b3: return true; /* 𝂳 */
    case 0x1d0b4: return true; /* 𝂴 */
    case 0x1d0b5: return true; /* 𝂵 */
    case 0x1d0b6: return true; /* 𝂶 */
    case 0x1d0b7: return true; /* 𝂷 */
    case 0x1d0b8: return true; /* 𝂸 */
    case 0x1d0b9: return true; /* 𝂹 */
    case 0x1d0ba: return true; /* 𝂺 */
    case 0x1d0bb: return true; /* 𝂻 */
    case 0x1d0bc: return true; /* 𝂼 */
    case 0x1d0bd: return true; /* 𝂽 */
    case 0x1d0be: return true; /* 𝂾 */
    case 0x1d0bf: return true; /* 𝂿 */
    case 0x1d0c0: return true; /* 𝃀 */
    case 0x1d0c1: return true; /* 𝃁 */
    case 0x1d0c2: return true; /* 𝃂 */
    case 0x1d0c3: return true; /* 𝃃 */
    case 0x1d0c4: return true; /* 𝃄 */
    case 0x1d0c5: return true; /* 𝃅 */
    case 0x1d0c6: return true; /* 𝃆 */
    case 0x1d0c7: return true; /* 𝃇 */
    case 0x1d0c8: return true; /* 𝃈 */
    case 0x1d0c9: return true; /* 𝃉 */
    case 0x1d0ca: return true; /* 𝃊 */
    case 0x1d0cb: return true; /* 𝃋 */
    case 0x1d0cc: return true; /* 𝃌 */
    case 0x1d0cd: return true; /* 𝃍 */
    case 0x1d0ce: return true; /* 𝃎 */
    case 0x1d0cf: return true; /* 𝃏 */
    case 0x1d0d0: return true; /* 𝃐 */
    case 0x1d0d1: return true; /* 𝃑 */
    case 0x1d0d2: return true; /* 𝃒 */
    case 0x1d0d3: return true; /* 𝃓 */
    case 0x1d0d4: return true; /* 𝃔 */
    case 0x1d0d5: return true; /* 𝃕 */
    case 0x1d0d6: return true; /* 𝃖 */
    case 0x1d0d7: return true; /* 𝃗 */
    case 0x1d0d8: return true; /* 𝃘 */
    case 0x1d0d9: return true; /* 𝃙 */
    case 0x1d0da: return true; /* 𝃚 */
    case 0x1d0db: return true; /* 𝃛 */
    case 0x1d0dc: return true; /* 𝃜 */
    case 0x1d0dd: return true; /* 𝃝 */
    case 0x1d0de: return true; /* 𝃞 */
    case 0x1d0df: return true; /* 𝃟 */
    case 0x1d0e0: return true; /* 𝃠 */
    case 0x1d0e1: return true; /* 𝃡 */
    case 0x1d0e2: return true; /* 𝃢 */
    case 0x1d0e3: return true; /* 𝃣 */
    case 0x1d0e4: return true; /* 𝃤 */
    case 0x1d0e5: return true; /* 𝃥 */
    case 0x1d0e6: return true; /* 𝃦 */
    case 0x1d0e7: return true; /* 𝃧 */
    case 0x1d0e8: return true; /* 𝃨 */
    case 0x1d0e9: return true; /* 𝃩 */
    case 0x1d0ea: return true; /* 𝃪 */
    case 0x1d0eb: return true; /* 𝃫 */
    case 0x1d0ec: return true; /* 𝃬 */
    case 0x1d0ed: return true; /* 𝃭 */
    case 0x1d0ee: return true; /* 𝃮 */
    case 0x1d0ef: return true; /* 𝃯 */
    case 0x1d0f0: return true; /* 𝃰 */
    case 0x1d0f1: return true; /* 𝃱 */
    case 0x1d0f2: return true; /* 𝃲 */
    case 0x1d0f3: return true; /* 𝃳 */
    case 0x1d0f4: return true; /* 𝃴 */
    case 0x1d0f5: return true; /* 𝃵 */
    case 0x1d100: return true; /* 𝄀 */
    case 0x1d101: return true; /* 𝄁 */
    case 0x1d102: return true; /* 𝄂 */
    case 0x1d103: return true; /* 𝄃 */
    case 0x1d104: return true; /* 𝄄 */
    case 0x1d105: return true; /* 𝄅 */
    case 0x1d106: return true; /* 𝄆 */
    case 0x1d107: return true; /* 𝄇 */
    case 0x1d108: return true; /* 𝄈 */
    case 0x1d109: return true; /* 𝄉 */
    case 0x1d10a: return true; /* 𝄊 */
    case 0x1d10b: return true; /* 𝄋 */
    case 0x1d10c: return true; /* 𝄌 */
    case 0x1d10d: return true; /* 𝄍 */
    case 0x1d10e: return true; /* 𝄎 */
    case 0x1d10f: return true; /* 𝄏 */
    case 0x1d110: return true; /* 𝄐 */
    case 0x1d111: return true; /* 𝄑 */
    case 0x1d112: return true; /* 𝄒 */
    case 0x1d113: return true; /* 𝄓 */
    case 0x1d114: return true; /* 𝄔 */
    case 0x1d115: return true; /* 𝄕 */
    case 0x1d116: return true; /* 𝄖 */
    case 0x1d117: return true; /* 𝄗 */
    case 0x1d118: return true; /* 𝄘 */
    case 0x1d119: return true; /* 𝄙 */
    case 0x1d11a: return true; /* 𝄚 */
    case 0x1d11b: return true; /* 𝄛 */
    case 0x1d11c: return true; /* 𝄜 */
    case 0x1d11d: return true; /* 𝄝 */
    case 0x1d11e: return true; /* 𝄞 */
    case 0x1d11f: return true; /* 𝄟 */
    case 0x1d120: return true; /* 𝄠 */
    case 0x1d121: return true; /* 𝄡 */
    case 0x1d122: return true; /* 𝄢 */
    case 0x1d123: return true; /* 𝄣 */
    case 0x1d124: return true; /* 𝄤 */
    case 0x1d125: return true; /* 𝄥 */
    case 0x1d126: return true; /* 𝄦 */
    case 0x1d129: return true; /* 𝄩 */
    case 0x1d12a: return true; /* 𝄪 */
    case 0x1d12b: return true; /* 𝄫 */
    case 0x1d12c: return true; /* 𝄬 */
    case 0x1d12d: return true; /* 𝄭 */
    case 0x1d12e: return true; /* 𝄮 */
    case 0x1d12f: return true; /* 𝄯 */
    case 0x1d130: return true; /* 𝄰 */
    case 0x1d131: return true; /* 𝄱 */
    case 0x1d132: return true; /* 𝄲 */
    case 0x1d133: return true; /* 𝄳 */
    case 0x1d134: return true; /* 𝄴 */
    case 0x1d135: return true; /* 𝄵 */
    case 0x1d136: return true; /* 𝄶 */
    case 0x1d137: return true; /* 𝄷 */
    case 0x1d138: return true; /* 𝄸 */
    case 0x1d139: return true; /* 𝄹 */
    case 0x1d13a: return true; /* 𝄺 */
    case 0x1d13b: return true; /* 𝄻 */
    case 0x1d13c: return true; /* 𝄼 */
    case 0x1d13d: return true; /* 𝄽 */
    case 0x1d13e: return true; /* 𝄾 */
    case 0x1d13f: return true; /* 𝄿 */
    case 0x1d140: return true; /* 𝅀 */
    case 0x1d141: return true; /* 𝅁 */
    case 0x1d142: return true; /* 𝅂 */
    case 0x1d143: return true; /* 𝅃 */
    case 0x1d144: return true; /* 𝅄 */
    case 0x1d145: return true; /* 𝅅 */
    case 0x1d146: return true; /* 𝅆 */
    case 0x1d147: return true; /* 𝅇 */
    case 0x1d148: return true; /* 𝅈 */
    case 0x1d149: return true; /* 𝅉 */
    case 0x1d14a: return true; /* 𝅊 */
    case 0x1d14b: return true; /* 𝅋 */
    case 0x1d14c: return true; /* 𝅌 */
    case 0x1d14d: return true; /* 𝅍 */
    case 0x1d14e: return true; /* 𝅎 */
    case 0x1d14f: return true; /* 𝅏 */
    case 0x1d150: return true; /* 𝅐 */
    case 0x1d151: return true; /* 𝅑 */
    case 0x1d152: return true; /* 𝅒 */
    case 0x1d153: return true; /* 𝅓 */
    case 0x1d154: return true; /* 𝅔 */
    case 0x1d155: return true; /* 𝅕 */
    case 0x1d156: return true; /* 𝅖 */
    case 0x1d157: return true; /* 𝅗 */
    case 0x1d158: return true; /* 𝅘 */
    case 0x1d159: return true; /* 𝅙 */
    case 0x1d15a: return true; /* 𝅚 */
    case 0x1d15b: return true; /* 𝅛 */
    case 0x1d15c: return true; /* 𝅜 */
    case 0x1d15d: return true; /* 𝅝 */
    case 0x1d15e: return true; /* 𝅗𝅥 */
    case 0x1d15f: return true; /* 𝅘𝅥 */
    case 0x1d160: return true; /* 𝅘𝅥𝅮 */
    case 0x1d161: return true; /* 𝅘𝅥𝅯 */
    case 0x1d162: return true; /* 𝅘𝅥𝅰 */
    case 0x1d163: return true; /* 𝅘𝅥𝅱 */
    case 0x1d164: return true; /* 𝅘𝅥𝅲 */
    case 0x1d16a: return true; /* 𝅪 */
    case 0x1d16b: return true; /* 𝅫 */
    case 0x1d16c: return true; /* 𝅬 */
    case 0x1d183: return true; /* 𝆃 */
    case 0x1d184: return true; /* 𝆄 */
    case 0x1d18c: return true; /* 𝆌 */
    case 0x1d18d: return true; /* 𝆍 */
    case 0x1d18e: return true; /* 𝆎 */
    case 0x1d18f: return true; /* 𝆏 */
    case 0x1d190: return true; /* 𝆐 */
    case 0x1d191: return true; /* 𝆑 */
    case 0x1d192: return true; /* 𝆒 */
    case 0x1d193: return true; /* 𝆓 */
    case 0x1d194: return true; /* 𝆔 */
    case 0x1d195: return true; /* 𝆕 */
    case 0x1d196: return true; /* 𝆖 */
    case 0x1d197: return true; /* 𝆗 */
    case 0x1d198: return true; /* 𝆘 */
    case 0x1d199: return true; /* 𝆙 */
    case 0x1d19a: return true; /* 𝆚 */
    case 0x1d19b: return true; /* 𝆛 */
    case 0x1d19c: return true; /* 𝆜 */
    case 0x1d19d: return true; /* 𝆝 */
    case 0x1d19e: return true; /* 𝆞 */
    case 0x1d19f: return true; /* 𝆟 */
    case 0x1d1a0: return true; /* 𝆠 */
    case 0x1d1a1: return true; /* 𝆡 */
    case 0x1d1a2: return true; /* 𝆢 */
    case 0x1d1a3: return true; /* 𝆣 */
    case 0x1d1a4: return true; /* 𝆤 */
    case 0x1d1a5: return true; /* 𝆥 */
    case 0x1d1a6: return true; /* 𝆦 */
    case 0x1d1a7: return true; /* 𝆧 */
    case 0x1d1a8: return true; /* 𝆨 */
    case 0x1d1a9: return true; /* 𝆩 */
    case 0x1d1ae: return true; /* 𝆮 */
    case 0x1d1af: return true; /* 𝆯 */
    case 0x1d1b0: return true; /* 𝆰 */
    case 0x1d1b1: return true; /* 𝆱 */
    case 0x1d1b2: return true; /* 𝆲 */
    case 0x1d1b3: return true; /* 𝆳 */
    case 0x1d1b4: return true; /* 𝆴 */
    case 0x1d1b5: return true; /* 𝆵 */
    case 0x1d1b6: return true; /* 𝆶 */
    case 0x1d1b7: return true; /* 𝆷 */
    case 0x1d1b8: return true; /* 𝆸 */
    case 0x1d1b9: return true; /* 𝆹 */
    case 0x1d1ba: return true; /* 𝆺 */
    case 0x1d1bb: return true; /* 𝆹𝅥 */
    case 0x1d1bc: return true; /* 𝆺𝅥 */
    case 0x1d1bd: return true; /* 𝆹𝅥𝅮 */
    case 0x1d1be: return true; /* 𝆺𝅥𝅮 */
    case 0x1d1bf: return true; /* 𝆹𝅥𝅯 */
    case 0x1d1c0: return true; /* 𝆺𝅥𝅯 */
    case 0x1d1c1: return true; /* 𝇁 */
    case 0x1d1c2: return true; /* 𝇂 */
    case 0x1d1c3: return true; /* 𝇃 */
    case 0x1d1c4: return true; /* 𝇄 */
    case 0x1d1c5: return true; /* 𝇅 */
    case 0x1d1c6: return true; /* 𝇆 */
    case 0x1d1c7: return true; /* 𝇇 */
    case 0x1d1c8: return true; /* 𝇈 */
    case 0x1d1c9: return true; /* 𝇉 */
    case 0x1d1ca: return true; /* 𝇊 */
    case 0x1d1cb: return true; /* 𝇋 */
    case 0x1d1cc: return true; /* 𝇌 */
    case 0x1d1cd: return true; /* 𝇍 */
    case 0x1d1ce: return true; /* 𝇎 */
    case 0x1d1cf: return true; /* 𝇏 */
    case 0x1d1d0: return true; /* 𝇐 */
    case 0x1d1d1: return true; /* 𝇑 */
    case 0x1d1d2: return true; /* 𝇒 */
    case 0x1d1d3: return true; /* 𝇓 */
    case 0x1d1d4: return true; /* 𝇔 */
    case 0x1d1d5: return true; /* 𝇕 */
    case 0x1d1d6: return true; /* 𝇖 */
    case 0x1d1d7: return true; /* 𝇗 */
    case 0x1d1d8: return true; /* 𝇘 */
    case 0x1d1d9: return true; /* 𝇙 */
    case 0x1d1da: return true; /* 𝇚 */
    case 0x1d1db: return true; /* 𝇛 */
    case 0x1d1dc: return true; /* 𝇜 */
    case 0x1d1dd: return true; /* 𝇝 */
    case 0x1d1de: return true; /* 𝇞 */
    case 0x1d1df: return true; /* 𝇟 */
    case 0x1d1e0: return true; /* 𝇠 */
    case 0x1d1e1: return true; /* 𝇡 */
    case 0x1d1e2: return true; /* 𝇢 */
    case 0x1d1e3: return true; /* 𝇣 */
    case 0x1d1e4: return true; /* 𝇤 */
    case 0x1d1e5: return true; /* 𝇥 */
    case 0x1d1e6: return true; /* 𝇦 */
    case 0x1d1e7: return true; /* 𝇧 */
    case 0x1d1e8: return true; /* 𝇨 */
    case 0x1d200: return true; /* 𝈀 */
    case 0x1d201: return true; /* 𝈁 */
    case 0x1d202: return true; /* 𝈂 */
    case 0x1d203: return true; /* 𝈃 */
    case 0x1d204: return true; /* 𝈄 */
    case 0x1d205: return true; /* 𝈅 */
    case 0x1d206: return true; /* 𝈆 */
    case 0x1d207: return true; /* 𝈇 */
    case 0x1d208: return true; /* 𝈈 */
    case 0x1d209: return true; /* 𝈉 */
    case 0x1d20a: return true; /* 𝈊 */
    case 0x1d20b: return true; /* 𝈋 */
    case 0x1d20c: return true; /* 𝈌 */
    case 0x1d20d: return true; /* 𝈍 */
    case 0x1d20e: return true; /* 𝈎 */
    case 0x1d20f: return true; /* 𝈏 */
    case 0x1d210: return true; /* 𝈐 */
    case 0x1d211: return true; /* 𝈑 */
    case 0x1d212: return true; /* 𝈒 */
    case 0x1d213: return true; /* 𝈓 */
    case 0x1d214: return true; /* 𝈔 */
    case 0x1d215: return true; /* 𝈕 */
    case 0x1d216: return true; /* 𝈖 */
    case 0x1d217: return true; /* 𝈗 */
    case 0x1d218: return true; /* 𝈘 */
    case 0x1d219: return true; /* 𝈙 */
    case 0x1d21a: return true; /* 𝈚 */
    case 0x1d21b: return true; /* 𝈛 */
    case 0x1d21c: return true; /* 𝈜 */
    case 0x1d21d: return true; /* 𝈝 */
    case 0x1d21e: return true; /* 𝈞 */
    case 0x1d21f: return true; /* 𝈟 */
    case 0x1d220: return true; /* 𝈠 */
    case 0x1d221: return true; /* 𝈡 */
    case 0x1d222: return true; /* 𝈢 */
    case 0x1d223: return true; /* 𝈣 */
    case 0x1d224: return true; /* 𝈤 */
    case 0x1d225: return true; /* 𝈥 */
    case 0x1d226: return true; /* 𝈦 */
    case 0x1d227: return true; /* 𝈧 */
    case 0x1d228: return true; /* 𝈨 */
    case 0x1d229: return true; /* 𝈩 */
    case 0x1d22a: return true; /* 𝈪 */
    case 0x1d22b: return true; /* 𝈫 */
    case 0x1d22c: return true; /* 𝈬 */
    case 0x1d22d: return true; /* 𝈭 */
    case 0x1d22e: return true; /* 𝈮 */
    case 0x1d22f: return true; /* 𝈯 */
    case 0x1d230: return true; /* 𝈰 */
    case 0x1d231: return true; /* 𝈱 */
    case 0x1d232: return true; /* 𝈲 */
    case 0x1d233: return true; /* 𝈳 */
    case 0x1d234: return true; /* 𝈴 */
    case 0x1d235: return true; /* 𝈵 */
    case 0x1d236: return true; /* 𝈶 */
    case 0x1d237: return true; /* 𝈷 */
    case 0x1d238: return true; /* 𝈸 */
    case 0x1d239: return true; /* 𝈹 */
    case 0x1d23a: return true; /* 𝈺 */
    case 0x1d23b: return true; /* 𝈻 */
    case 0x1d23c: return true; /* 𝈼 */
    case 0x1d23d: return true; /* 𝈽 */
    case 0x1d23e: return true; /* 𝈾 */
    case 0x1d23f: return true; /* 𝈿 */
    case 0x1d240: return true; /* 𝉀 */
    case 0x1d241: return true; /* 𝉁 */
    case 0x1d245: return true; /* 𝉅 */
    case 0x1d300: return true; /* 𝌀 */
    case 0x1d301: return true; /* 𝌁 */
    case 0x1d302: return true; /* 𝌂 */
    case 0x1d303: return true; /* 𝌃 */
    case 0x1d304: return true; /* 𝌄 */
    case 0x1d305: return true; /* 𝌅 */
    case 0x1d306: return true; /* 𝌆 */
    case 0x1d307: return true; /* 𝌇 */
    case 0x1d308: return true; /* 𝌈 */
    case 0x1d309: return true; /* 𝌉 */
    case 0x1d30a: return true; /* 𝌊 */
    case 0x1d30b: return true; /* 𝌋 */
    case 0x1d30c: return true; /* 𝌌 */
    case 0x1d30d: return true; /* 𝌍 */
    case 0x1d30e: return true; /* 𝌎 */
    case 0x1d30f: return true; /* 𝌏 */
    case 0x1d310: return true; /* 𝌐 */
    case 0x1d311: return true; /* 𝌑 */
    case 0x1d312: return true; /* 𝌒 */
    case 0x1d313: return true; /* 𝌓 */
    case 0x1d314: return true; /* 𝌔 */
    case 0x1d315: return true; /* 𝌕 */
    case 0x1d316: return true; /* 𝌖 */
    case 0x1d317: return true; /* 𝌗 */
    case 0x1d318: return true; /* 𝌘 */
    case 0x1d319: return true; /* 𝌙 */
    case 0x1d31a: return true; /* 𝌚 */
    case 0x1d31b: return true; /* 𝌛 */
    case 0x1d31c: return true; /* 𝌜 */
    case 0x1d31d: return true; /* 𝌝 */
    case 0x1d31e: return true; /* 𝌞 */
    case 0x1d31f: return true; /* 𝌟 */
    case 0x1d320: return true; /* 𝌠 */
    case 0x1d321: return true; /* 𝌡 */
    case 0x1d322: return true; /* 𝌢 */
    case 0x1d323: return true; /* 𝌣 */
    case 0x1d324: return true; /* 𝌤 */
    case 0x1d325: return true; /* 𝌥 */
    case 0x1d326: return true; /* 𝌦 */
    case 0x1d327: return true; /* 𝌧 */
    case 0x1d328: return true; /* 𝌨 */
    case 0x1d329: return true; /* 𝌩 */
    case 0x1d32a: return true; /* 𝌪 */
    case 0x1d32b: return true; /* 𝌫 */
    case 0x1d32c: return true; /* 𝌬 */
    case 0x1d32d: return true; /* 𝌭 */
    case 0x1d32e: return true; /* 𝌮 */
    case 0x1d32f: return true; /* 𝌯 */
    case 0x1d330: return true; /* 𝌰 */
    case 0x1d331: return true; /* 𝌱 */
    case 0x1d332: return true; /* 𝌲 */
    case 0x1d333: return true; /* 𝌳 */
    case 0x1d334: return true; /* 𝌴 */
    case 0x1d335: return true; /* 𝌵 */
    case 0x1d336: return true; /* 𝌶 */
    case 0x1d337: return true; /* 𝌷 */
    case 0x1d338: return true; /* 𝌸 */
    case 0x1d339: return true; /* 𝌹 */
    case 0x1d33a: return true; /* 𝌺 */
    case 0x1d33b: return true; /* 𝌻 */
    case 0x1d33c: return true; /* 𝌼 */
    case 0x1d33d: return true; /* 𝌽 */
    case 0x1d33e: return true; /* 𝌾 */
    case 0x1d33f: return true; /* 𝌿 */
    case 0x1d340: return true; /* 𝍀 */
    case 0x1d341: return true; /* 𝍁 */
    case 0x1d342: return true; /* 𝍂 */
    case 0x1d343: return true; /* 𝍃 */
    case 0x1d344: return true; /* 𝍄 */
    case 0x1d345: return true; /* 𝍅 */
    case 0x1d346: return true; /* 𝍆 */
    case 0x1d347: return true; /* 𝍇 */
    case 0x1d348: return true; /* 𝍈 */
    case 0x1d349: return true; /* 𝍉 */
    case 0x1d34a: return true; /* 𝍊 */
    case 0x1d34b: return true; /* 𝍋 */
    case 0x1d34c: return true; /* 𝍌 */
    case 0x1d34d: return true; /* 𝍍 */
    case 0x1d34e: return true; /* 𝍎 */
    case 0x1d34f: return true; /* 𝍏 */
    case 0x1d350: return true; /* 𝍐 */
    case 0x1d351: return true; /* 𝍑 */
    case 0x1d352: return true; /* 𝍒 */
    case 0x1d353: return true; /* 𝍓 */
    case 0x1d354: return true; /* 𝍔 */
    case 0x1d355: return true; /* 𝍕 */
    case 0x1d356: return true; /* 𝍖 */
    case 0x1d6c1: return true; /* 𝛁 */
    case 0x1d6db: return true; /* 𝛛 */
    case 0x1d6fb: return true; /* 𝛻 */
    case 0x1d715: return true; /* 𝜕 */
    case 0x1d735: return true; /* 𝜵 */
    case 0x1d74f: return true; /* 𝝏 */
    case 0x1d76f: return true; /* 𝝯 */
    case 0x1d789: return true; /* 𝞉 */
    case 0x1d7a9: return true; /* 𝞩 */
    case 0x1d7c3: return true; /* 𝟃 */
    case 0x1d800: return true; /* 𝠀 */
    case 0x1d801: return true; /* 𝠁 */
    case 0x1d802: return true; /* 𝠂 */
    case 0x1d803: return true; /* 𝠃 */
    case 0x1d804: return true; /* 𝠄 */
    case 0x1d805: return true; /* 𝠅 */
    case 0x1d806: return true; /* 𝠆 */
    case 0x1d807: return true; /* 𝠇 */
    case 0x1d808: return true; /* 𝠈 */
    case 0x1d809: return true; /* 𝠉 */
    case 0x1d80a: return true; /* 𝠊 */
    case 0x1d80b: return true; /* 𝠋 */
    case 0x1d80c: return true; /* 𝠌 */
    case 0x1d80d: return true; /* 𝠍 */
    case 0x1d80e: return true; /* 𝠎 */
    case 0x1d80f: return true; /* 𝠏 */
    case 0x1d810: return true; /* 𝠐 */
    case 0x1d811: return true; /* 𝠑 */
    case 0x1d812: return true; /* 𝠒 */
    case 0x1d813: return true; /* 𝠓 */
    case 0x1d814: return true; /* 𝠔 */
    case 0x1d815: return true; /* 𝠕 */
    case 0x1d816: return true; /* 𝠖 */
    case 0x1d817: return true; /* 𝠗 */
    case 0x1d818: return true; /* 𝠘 */
    case 0x1d819: return true; /* 𝠙 */
    case 0x1d81a: return true; /* 𝠚 */
    case 0x1d81b: return true; /* 𝠛 */
    case 0x1d81c: return true; /* 𝠜 */
    case 0x1d81d: return true; /* 𝠝 */
    case 0x1d81e: return true; /* 𝠞 */
    case 0x1d81f: return true; /* 𝠟 */
    case 0x1d820: return true; /* 𝠠 */
    case 0x1d821: return true; /* 𝠡 */
    case 0x1d822: return true; /* 𝠢 */
    case 0x1d823: return true; /* 𝠣 */
    case 0x1d824: return true; /* 𝠤 */
    case 0x1d825: return true; /* 𝠥 */
    case 0x1d826: return true; /* 𝠦 */
    case 0x1d827: return true; /* 𝠧 */
    case 0x1d828: return true; /* 𝠨 */
    case 0x1d829: return true; /* 𝠩 */
    case 0x1d82a: return true; /* 𝠪 */
    case 0x1d82b: return true; /* 𝠫 */
    case 0x1d82c: return true; /* 𝠬 */
    case 0x1d82d: return true; /* 𝠭 */
    case 0x1d82e: return true; /* 𝠮 */
    case 0x1d82f: return true; /* 𝠯 */
    case 0x1d830: return true; /* 𝠰 */
    case 0x1d831: return true; /* 𝠱 */
    case 0x1d832: return true; /* 𝠲 */
    case 0x1d833: return true; /* 𝠳 */
    case 0x1d834: return true; /* 𝠴 */
    case 0x1d835: return true; /* 𝠵 */
    case 0x1d836: return true; /* 𝠶 */
    case 0x1d837: return true; /* 𝠷 */
    case 0x1d838: return true; /* 𝠸 */
    case 0x1d839: return true; /* 𝠹 */
    case 0x1d83a: return true; /* 𝠺 */
    case 0x1d83b: return true; /* 𝠻 */
    case 0x1d83c: return true; /* 𝠼 */
    case 0x1d83d: return true; /* 𝠽 */
    case 0x1d83e: return true; /* 𝠾 */
    case 0x1d83f: return true; /* 𝠿 */
    case 0x1d840: return true; /* 𝡀 */
    case 0x1d841: return true; /* 𝡁 */
    case 0x1d842: return true; /* 𝡂 */
    case 0x1d843: return true; /* 𝡃 */
    case 0x1d844: return true; /* 𝡄 */
    case 0x1d845: return true; /* 𝡅 */
    case 0x1d846: return true; /* 𝡆 */
    case 0x1d847: return true; /* 𝡇 */
    case 0x1d848: return true; /* 𝡈 */
    case 0x1d849: return true; /* 𝡉 */
    case 0x1d84a: return true; /* 𝡊 */
    case 0x1d84b: return true; /* 𝡋 */
    case 0x1d84c: return true; /* 𝡌 */
    case 0x1d84d: return true; /* 𝡍 */
    case 0x1d84e: return true; /* 𝡎 */
    case 0x1d84f: return true; /* 𝡏 */
    case 0x1d850: return true; /* 𝡐 */
    case 0x1d851: return true; /* 𝡑 */
    case 0x1d852: return true; /* 𝡒 */
    case 0x1d853: return true; /* 𝡓 */
    case 0x1d854: return true; /* 𝡔 */
    case 0x1d855: return true; /* 𝡕 */
    case 0x1d856: return true; /* 𝡖 */
    case 0x1d857: return true; /* 𝡗 */
    case 0x1d858: return true; /* 𝡘 */
    case 0x1d859: return true; /* 𝡙 */
    case 0x1d85a: return true; /* 𝡚 */
    case 0x1d85b: return true; /* 𝡛 */
    case 0x1d85c: return true; /* 𝡜 */
    case 0x1d85d: return true; /* 𝡝 */
    case 0x1d85e: return true; /* 𝡞 */
    case 0x1d85f: return true; /* 𝡟 */
    case 0x1d860: return true; /* 𝡠 */
    case 0x1d861: return true; /* 𝡡 */
    case 0x1d862: return true; /* 𝡢 */
    case 0x1d863: return true; /* 𝡣 */
    case 0x1d864: return true; /* 𝡤 */
    case 0x1d865: return true; /* 𝡥 */
    case 0x1d866: return true; /* 𝡦 */
    case 0x1d867: return true; /* 𝡧 */
    case 0x1d868: return true; /* 𝡨 */
    case 0x1d869: return true; /* 𝡩 */
    case 0x1d86a: return true; /* 𝡪 */
    case 0x1d86b: return true; /* 𝡫 */
    case 0x1d86c: return true; /* 𝡬 */
    case 0x1d86d: return true; /* 𝡭 */
    case 0x1d86e: return true; /* 𝡮 */
    case 0x1d86f: return true; /* 𝡯 */
    case 0x1d870: return true; /* 𝡰 */
    case 0x1d871: return true; /* 𝡱 */
    case 0x1d872: return true; /* 𝡲 */
    case 0x1d873: return true; /* 𝡳 */
    case 0x1d874: return true; /* 𝡴 */
    case 0x1d875: return true; /* 𝡵 */
    case 0x1d876: return true; /* 𝡶 */
    case 0x1d877: return true; /* 𝡷 */
    case 0x1d878: return true; /* 𝡸 */
    case 0x1d879: return true; /* 𝡹 */
    case 0x1d87a: return true; /* 𝡺 */
    case 0x1d87b: return true; /* 𝡻 */
    case 0x1d87c: return true; /* 𝡼 */
    case 0x1d87d: return true; /* 𝡽 */
    case 0x1d87e: return true; /* 𝡾 */
    case 0x1d87f: return true; /* 𝡿 */
    case 0x1d880: return true; /* 𝢀 */
    case 0x1d881: return true; /* 𝢁 */
    case 0x1d882: return true; /* 𝢂 */
    case 0x1d883: return true; /* 𝢃 */
    case 0x1d884: return true; /* 𝢄 */
    case 0x1d885: return true; /* 𝢅 */
    case 0x1d886: return true; /* 𝢆 */
    case 0x1d887: return true; /* 𝢇 */
    case 0x1d888: return true; /* 𝢈 */
    case 0x1d889: return true; /* 𝢉 */
    case 0x1d88a: return true; /* 𝢊 */
    case 0x1d88b: return true; /* 𝢋 */
    case 0x1d88c: return true; /* 𝢌 */
    case 0x1d88d: return true; /* 𝢍 */
    case 0x1d88e: return true; /* 𝢎 */
    case 0x1d88f: return true; /* 𝢏 */
    case 0x1d890: return true; /* 𝢐 */
    case 0x1d891: return true; /* 𝢑 */
    case 0x1d892: return true; /* 𝢒 */
    case 0x1d893: return true; /* 𝢓 */
    case 0x1d894: return true; /* 𝢔 */
    case 0x1d895: return true; /* 𝢕 */
    case 0x1d896: return true; /* 𝢖 */
    case 0x1d897: return true; /* 𝢗 */
    case 0x1d898: return true; /* 𝢘 */
    case 0x1d899: return true; /* 𝢙 */
    case 0x1d89a: return true; /* 𝢚 */
    case 0x1d89b: return true; /* 𝢛 */
    case 0x1d89c: return true; /* 𝢜 */
    case 0x1d89d: return true; /* 𝢝 */
    case 0x1d89e: return true; /* 𝢞 */
    case 0x1d89f: return true; /* 𝢟 */
    case 0x1d8a0: return true; /* 𝢠 */
    case 0x1d8a1: return true; /* 𝢡 */
    case 0x1d8a2: return true; /* 𝢢 */
    case 0x1d8a3: return true; /* 𝢣 */
    case 0x1d8a4: return true; /* 𝢤 */
    case 0x1d8a5: return true; /* 𝢥 */
    case 0x1d8a6: return true; /* 𝢦 */
    case 0x1d8a7: return true; /* 𝢧 */
    case 0x1d8a8: return true; /* 𝢨 */
    case 0x1d8a9: return true; /* 𝢩 */
    case 0x1d8aa: return true; /* 𝢪 */
    case 0x1d8ab: return true; /* 𝢫 */
    case 0x1d8ac: return true; /* 𝢬 */
    case 0x1d8ad: return true; /* 𝢭 */
    case 0x1d8ae: return true; /* 𝢮 */
    case 0x1d8af: return true; /* 𝢯 */
    case 0x1d8b0: return true; /* 𝢰 */
    case 0x1d8b1: return true; /* 𝢱 */
    case 0x1d8b2: return true; /* 𝢲 */
    case 0x1d8b3: return true; /* 𝢳 */
    case 0x1d8b4: return true; /* 𝢴 */
    case 0x1d8b5: return true; /* 𝢵 */
    case 0x1d8b6: return true; /* 𝢶 */
    case 0x1d8b7: return true; /* 𝢷 */
    case 0x1d8b8: return true; /* 𝢸 */
    case 0x1d8b9: return true; /* 𝢹 */
    case 0x1d8ba: return true; /* 𝢺 */
    case 0x1d8bb: return true; /* 𝢻 */
    case 0x1d8bc: return true; /* 𝢼 */
    case 0x1d8bd: return true; /* 𝢽 */
    case 0x1d8be: return true; /* 𝢾 */
    case 0x1d8bf: return true; /* 𝢿 */
    case 0x1d8c0: return true; /* 𝣀 */
    case 0x1d8c1: return true; /* 𝣁 */
    case 0x1d8c2: return true; /* 𝣂 */
    case 0x1d8c3: return true; /* 𝣃 */
    case 0x1d8c4: return true; /* 𝣄 */
    case 0x1d8c5: return true; /* 𝣅 */
    case 0x1d8c6: return true; /* 𝣆 */
    case 0x1d8c7: return true; /* 𝣇 */
    case 0x1d8c8: return true; /* 𝣈 */
    case 0x1d8c9: return true; /* 𝣉 */
    case 0x1d8ca: return true; /* 𝣊 */
    case 0x1d8cb: return true; /* 𝣋 */
    case 0x1d8cc: return true; /* 𝣌 */
    case 0x1d8cd: return true; /* 𝣍 */
    case 0x1d8ce: return true; /* 𝣎 */
    case 0x1d8cf: return true; /* 𝣏 */
    case 0x1d8d0: return true; /* 𝣐 */
    case 0x1d8d1: return true; /* 𝣑 */
    case 0x1d8d2: return true; /* 𝣒 */
    case 0x1d8d3: return true; /* 𝣓 */
    case 0x1d8d4: return true; /* 𝣔 */
    case 0x1d8d5: return true; /* 𝣕 */
    case 0x1d8d6: return true; /* 𝣖 */
    case 0x1d8d7: return true; /* 𝣗 */
    case 0x1d8d8: return true; /* 𝣘 */
    case 0x1d8d9: return true; /* 𝣙 */
    case 0x1d8da: return true; /* 𝣚 */
    case 0x1d8db: return true; /* 𝣛 */
    case 0x1d8dc: return true; /* 𝣜 */
    case 0x1d8dd: return true; /* 𝣝 */
    case 0x1d8de: return true; /* 𝣞 */
    case 0x1d8df: return true; /* 𝣟 */
    case 0x1d8e0: return true; /* 𝣠 */
    case 0x1d8e1: return true; /* 𝣡 */
    case 0x1d8e2: return true; /* 𝣢 */
    case 0x1d8e3: return true; /* 𝣣 */
    case 0x1d8e4: return true; /* 𝣤 */
    case 0x1d8e5: return true; /* 𝣥 */
    case 0x1d8e6: return true; /* 𝣦 */
    case 0x1d8e7: return true; /* 𝣧 */
    case 0x1d8e8: return true; /* 𝣨 */
    case 0x1d8e9: return true; /* 𝣩 */
    case 0x1d8ea: return true; /* 𝣪 */
    case 0x1d8eb: return true; /* 𝣫 */
    case 0x1d8ec: return true; /* 𝣬 */
    case 0x1d8ed: return true; /* 𝣭 */
    case 0x1d8ee: return true; /* 𝣮 */
    case 0x1d8ef: return true; /* 𝣯 */
    case 0x1d8f0: return true; /* 𝣰 */
    case 0x1d8f1: return true; /* 𝣱 */
    case 0x1d8f2: return true; /* 𝣲 */
    case 0x1d8f3: return true; /* 𝣳 */
    case 0x1d8f4: return true; /* 𝣴 */
    case 0x1d8f5: return true; /* 𝣵 */
    case 0x1d8f6: return true; /* 𝣶 */
    case 0x1d8f7: return true; /* 𝣷 */
    case 0x1d8f8: return true; /* 𝣸 */
    case 0x1d8f9: return true; /* 𝣹 */
    case 0x1d8fa: return true; /* 𝣺 */
    case 0x1d8fb: return true; /* 𝣻 */
    case 0x1d8fc: return true; /* 𝣼 */
    case 0x1d8fd: return true; /* 𝣽 */
    case 0x1d8fe: return true; /* 𝣾 */
    case 0x1d8ff: return true; /* 𝣿 */
    case 0x1d900: return true; /* 𝤀 */
    case 0x1d901: return true; /* 𝤁 */
    case 0x1d902: return true; /* 𝤂 */
    case 0x1d903: return true; /* 𝤃 */
    case 0x1d904: return true; /* 𝤄 */
    case 0x1d905: return true; /* 𝤅 */
    case 0x1d906: return true; /* 𝤆 */
    case 0x1d907: return true; /* 𝤇 */
    case 0x1d908: return true; /* 𝤈 */
    case 0x1d909: return true; /* 𝤉 */
    case 0x1d90a: return true; /* 𝤊 */
    case 0x1d90b: return true; /* 𝤋 */
    case 0x1d90c: return true; /* 𝤌 */
    case 0x1d90d: return true; /* 𝤍 */
    case 0x1d90e: return true; /* 𝤎 */
    case 0x1d90f: return true; /* 𝤏 */
    case 0x1d910: return true; /* 𝤐 */
    case 0x1d911: return true; /* 𝤑 */
    case 0x1d912: return true; /* 𝤒 */
    case 0x1d913: return true; /* 𝤓 */
    case 0x1d914: return true; /* 𝤔 */
    case 0x1d915: return true; /* 𝤕 */
    case 0x1d916: return true; /* 𝤖 */
    case 0x1d917: return true; /* 𝤗 */
    case 0x1d918: return true; /* 𝤘 */
    case 0x1d919: return true; /* 𝤙 */
    case 0x1d91a: return true; /* 𝤚 */
    case 0x1d91b: return true; /* 𝤛 */
    case 0x1d91c: return true; /* 𝤜 */
    case 0x1d91d: return true; /* 𝤝 */
    case 0x1d91e: return true; /* 𝤞 */
    case 0x1d91f: return true; /* 𝤟 */
    case 0x1d920: return true; /* 𝤠 */
    case 0x1d921: return true; /* 𝤡 */
    case 0x1d922: return true; /* 𝤢 */
    case 0x1d923: return true; /* 𝤣 */
    case 0x1d924: return true; /* 𝤤 */
    case 0x1d925: return true; /* 𝤥 */
    case 0x1d926: return true; /* 𝤦 */
    case 0x1d927: return true; /* 𝤧 */
    case 0x1d928: return true; /* 𝤨 */
    case 0x1d929: return true; /* 𝤩 */
    case 0x1d92a: return true; /* 𝤪 */
    case 0x1d92b: return true; /* 𝤫 */
    case 0x1d92c: return true; /* 𝤬 */
    case 0x1d92d: return true; /* 𝤭 */
    case 0x1d92e: return true; /* 𝤮 */
    case 0x1d92f: return true; /* 𝤯 */
    case 0x1d930: return true; /* 𝤰 */
    case 0x1d931: return true; /* 𝤱 */
    case 0x1d932: return true; /* 𝤲 */
    case 0x1d933: return true; /* 𝤳 */
    case 0x1d934: return true; /* 𝤴 */
    case 0x1d935: return true; /* 𝤵 */
    case 0x1d936: return true; /* 𝤶 */
    case 0x1d937: return true; /* 𝤷 */
    case 0x1d938: return true; /* 𝤸 */
    case 0x1d939: return true; /* 𝤹 */
    case 0x1d93a: return true; /* 𝤺 */
    case 0x1d93b: return true; /* 𝤻 */
    case 0x1d93c: return true; /* 𝤼 */
    case 0x1d93d: return true; /* 𝤽 */
    case 0x1d93e: return true; /* 𝤾 */
    case 0x1d93f: return true; /* 𝤿 */
    case 0x1d940: return true; /* 𝥀 */
    case 0x1d941: return true; /* 𝥁 */
    case 0x1d942: return true; /* 𝥂 */
    case 0x1d943: return true; /* 𝥃 */
    case 0x1d944: return true; /* 𝥄 */
    case 0x1d945: return true; /* 𝥅 */
    case 0x1d946: return true; /* 𝥆 */
    case 0x1d947: return true; /* 𝥇 */
    case 0x1d948: return true; /* 𝥈 */
    case 0x1d949: return true; /* 𝥉 */
    case 0x1d94a: return true; /* 𝥊 */
    case 0x1d94b: return true; /* 𝥋 */
    case 0x1d94c: return true; /* 𝥌 */
    case 0x1d94d: return true; /* 𝥍 */
    case 0x1d94e: return true; /* 𝥎 */
    case 0x1d94f: return true; /* 𝥏 */
    case 0x1d950: return true; /* 𝥐 */
    case 0x1d951: return true; /* 𝥑 */
    case 0x1d952: return true; /* 𝥒 */
    case 0x1d953: return true; /* 𝥓 */
    case 0x1d954: return true; /* 𝥔 */
    case 0x1d955: return true; /* 𝥕 */
    case 0x1d956: return true; /* 𝥖 */
    case 0x1d957: return true; /* 𝥗 */
    case 0x1d958: return true; /* 𝥘 */
    case 0x1d959: return true; /* 𝥙 */
    case 0x1d95a: return true; /* 𝥚 */
    case 0x1d95b: return true; /* 𝥛 */
    case 0x1d95c: return true; /* 𝥜 */
    case 0x1d95d: return true; /* 𝥝 */
    case 0x1d95e: return true; /* 𝥞 */
    case 0x1d95f: return true; /* 𝥟 */
    case 0x1d960: return true; /* 𝥠 */
    case 0x1d961: return true; /* 𝥡 */
    case 0x1d962: return true; /* 𝥢 */
    case 0x1d963: return true; /* 𝥣 */
    case 0x1d964: return true; /* 𝥤 */
    case 0x1d965: return true; /* 𝥥 */
    case 0x1d966: return true; /* 𝥦 */
    case 0x1d967: return true; /* 𝥧 */
    case 0x1d968: return true; /* 𝥨 */
    case 0x1d969: return true; /* 𝥩 */
    case 0x1d96a: return true; /* 𝥪 */
    case 0x1d96b: return true; /* 𝥫 */
    case 0x1d96c: return true; /* 𝥬 */
    case 0x1d96d: return true; /* 𝥭 */
    case 0x1d96e: return true; /* 𝥮 */
    case 0x1d96f: return true; /* 𝥯 */
    case 0x1d970: return true; /* 𝥰 */
    case 0x1d971: return true; /* 𝥱 */
    case 0x1d972: return true; /* 𝥲 */
    case 0x1d973: return true; /* 𝥳 */
    case 0x1d974: return true; /* 𝥴 */
    case 0x1d975: return true; /* 𝥵 */
    case 0x1d976: return true; /* 𝥶 */
    case 0x1d977: return true; /* 𝥷 */
    case 0x1d978: return true; /* 𝥸 */
    case 0x1d979: return true; /* 𝥹 */
    case 0x1d97a: return true; /* 𝥺 */
    case 0x1d97b: return true; /* 𝥻 */
    case 0x1d97c: return true; /* 𝥼 */
    case 0x1d97d: return true; /* 𝥽 */
    case 0x1d97e: return true; /* 𝥾 */
    case 0x1d97f: return true; /* 𝥿 */
    case 0x1d980: return true; /* 𝦀 */
    case 0x1d981: return true; /* 𝦁 */
    case 0x1d982: return true; /* 𝦂 */
    case 0x1d983: return true; /* 𝦃 */
    case 0x1d984: return true; /* 𝦄 */
    case 0x1d985: return true; /* 𝦅 */
    case 0x1d986: return true; /* 𝦆 */
    case 0x1d987: return true; /* 𝦇 */
    case 0x1d988: return true; /* 𝦈 */
    case 0x1d989: return true; /* 𝦉 */
    case 0x1d98a: return true; /* 𝦊 */
    case 0x1d98b: return true; /* 𝦋 */
    case 0x1d98c: return true; /* 𝦌 */
    case 0x1d98d: return true; /* 𝦍 */
    case 0x1d98e: return true; /* 𝦎 */
    case 0x1d98f: return true; /* 𝦏 */
    case 0x1d990: return true; /* 𝦐 */
    case 0x1d991: return true; /* 𝦑 */
    case 0x1d992: return true; /* 𝦒 */
    case 0x1d993: return true; /* 𝦓 */
    case 0x1d994: return true; /* 𝦔 */
    case 0x1d995: return true; /* 𝦕 */
    case 0x1d996: return true; /* 𝦖 */
    case 0x1d997: return true; /* 𝦗 */
    case 0x1d998: return true; /* 𝦘 */
    case 0x1d999: return true; /* 𝦙 */
    case 0x1d99a: return true; /* 𝦚 */
    case 0x1d99b: return true; /* 𝦛 */
    case 0x1d99c: return true; /* 𝦜 */
    case 0x1d99d: return true; /* 𝦝 */
    case 0x1d99e: return true; /* 𝦞 */
    case 0x1d99f: return true; /* 𝦟 */
    case 0x1d9a0: return true; /* 𝦠 */
    case 0x1d9a1: return true; /* 𝦡 */
    case 0x1d9a2: return true; /* 𝦢 */
    case 0x1d9a3: return true; /* 𝦣 */
    case 0x1d9a4: return true; /* 𝦤 */
    case 0x1d9a5: return true; /* 𝦥 */
    case 0x1d9a6: return true; /* 𝦦 */
    case 0x1d9a7: return true; /* 𝦧 */
    case 0x1d9a8: return true; /* 𝦨 */
    case 0x1d9a9: return true; /* 𝦩 */
    case 0x1d9aa: return true; /* 𝦪 */
    case 0x1d9ab: return true; /* 𝦫 */
    case 0x1d9ac: return true; /* 𝦬 */
    case 0x1d9ad: return true; /* 𝦭 */
    case 0x1d9ae: return true; /* 𝦮 */
    case 0x1d9af: return true; /* 𝦯 */
    case 0x1d9b0: return true; /* 𝦰 */
    case 0x1d9b1: return true; /* 𝦱 */
    case 0x1d9b2: return true; /* 𝦲 */
    case 0x1d9b3: return true; /* 𝦳 */
    case 0x1d9b4: return true; /* 𝦴 */
    case 0x1d9b5: return true; /* 𝦵 */
    case 0x1d9b6: return true; /* 𝦶 */
    case 0x1d9b7: return true; /* 𝦷 */
    case 0x1d9b8: return true; /* 𝦸 */
    case 0x1d9b9: return true; /* 𝦹 */
    case 0x1d9ba: return true; /* 𝦺 */
    case 0x1d9bb: return true; /* 𝦻 */
    case 0x1d9bc: return true; /* 𝦼 */
    case 0x1d9bd: return true; /* 𝦽 */
    case 0x1d9be: return true; /* 𝦾 */
    case 0x1d9bf: return true; /* 𝦿 */
    case 0x1d9c0: return true; /* 𝧀 */
    case 0x1d9c1: return true; /* 𝧁 */
    case 0x1d9c2: return true; /* 𝧂 */
    case 0x1d9c3: return true; /* 𝧃 */
    case 0x1d9c4: return true; /* 𝧄 */
    case 0x1d9c5: return true; /* 𝧅 */
    case 0x1d9c6: return true; /* 𝧆 */
    case 0x1d9c7: return true; /* 𝧇 */
    case 0x1d9c8: return true; /* 𝧈 */
    case 0x1d9c9: return true; /* 𝧉 */
    case 0x1d9ca: return true; /* 𝧊 */
    case 0x1d9cb: return true; /* 𝧋 */
    case 0x1d9cc: return true; /* 𝧌 */
    case 0x1d9cd: return true; /* 𝧍 */
    case 0x1d9ce: return true; /* 𝧎 */
    case 0x1d9cf: return true; /* 𝧏 */
    case 0x1d9d0: return true; /* 𝧐 */
    case 0x1d9d1: return true; /* 𝧑 */
    case 0x1d9d2: return true; /* 𝧒 */
    case 0x1d9d3: return true; /* 𝧓 */
    case 0x1d9d4: return true; /* 𝧔 */
    case 0x1d9d5: return true; /* 𝧕 */
    case 0x1d9d6: return true; /* 𝧖 */
    case 0x1d9d7: return true; /* 𝧗 */
    case 0x1d9d8: return true; /* 𝧘 */
    case 0x1d9d9: return true; /* 𝧙 */
    case 0x1d9da: return true; /* 𝧚 */
    case 0x1d9db: return true; /* 𝧛 */
    case 0x1d9dc: return true; /* 𝧜 */
    case 0x1d9dd: return true; /* 𝧝 */
    case 0x1d9de: return true; /* 𝧞 */
    case 0x1d9df: return true; /* 𝧟 */
    case 0x1d9e0: return true; /* 𝧠 */
    case 0x1d9e1: return true; /* 𝧡 */
    case 0x1d9e2: return true; /* 𝧢 */
    case 0x1d9e3: return true; /* 𝧣 */
    case 0x1d9e4: return true; /* 𝧤 */
    case 0x1d9e5: return true; /* 𝧥 */
    case 0x1d9e6: return true; /* 𝧦 */
    case 0x1d9e7: return true; /* 𝧧 */
    case 0x1d9e8: return true; /* 𝧨 */
    case 0x1d9e9: return true; /* 𝧩 */
    case 0x1d9ea: return true; /* 𝧪 */
    case 0x1d9eb: return true; /* 𝧫 */
    case 0x1d9ec: return true; /* 𝧬 */
    case 0x1d9ed: return true; /* 𝧭 */
    case 0x1d9ee: return true; /* 𝧮 */
    case 0x1d9ef: return true; /* 𝧯 */
    case 0x1d9f0: return true; /* 𝧰 */
    case 0x1d9f1: return true; /* 𝧱 */
    case 0x1d9f2: return true; /* 𝧲 */
    case 0x1d9f3: return true; /* 𝧳 */
    case 0x1d9f4: return true; /* 𝧴 */
    case 0x1d9f5: return true; /* 𝧵 */
    case 0x1d9f6: return true; /* 𝧶 */
    case 0x1d9f7: return true; /* 𝧷 */
    case 0x1d9f8: return true; /* 𝧸 */
    case 0x1d9f9: return true; /* 𝧹 */
    case 0x1d9fa: return true; /* 𝧺 */
    case 0x1d9fb: return true; /* 𝧻 */
    case 0x1d9fc: return true; /* 𝧼 */
    case 0x1d9fd: return true; /* 𝧽 */
    case 0x1d9fe: return true; /* 𝧾 */
    case 0x1d9ff: return true; /* 𝧿 */
    case 0x1da37: return true; /* 𝨷 */
    case 0x1da38: return true; /* 𝨸 */
    case 0x1da39: return true; /* 𝨹 */
    case 0x1da3a: return true; /* 𝨺 */
    case 0x1da6d: return true; /* 𝩭 */
    case 0x1da6e: return true; /* 𝩮 */
    case 0x1da6f: return true; /* 𝩯 */
    case 0x1da70: return true; /* 𝩰 */
    case 0x1da71: return true; /* 𝩱 */
    case 0x1da72: return true; /* 𝩲 */
    case 0x1da73: return true; /* 𝩳 */
    case 0x1da74: return true; /* 𝩴 */
    case 0x1da76: return true; /* 𝩶 */
    case 0x1da77: return true; /* 𝩷 */
    case 0x1da78: return true; /* 𝩸 */
    case 0x1da79: return true; /* 𝩹 */
    case 0x1da7a: return true; /* 𝩺 */
    case 0x1da7b: return true; /* 𝩻 */
    case 0x1da7c: return true; /* 𝩼 */
    case 0x1da7d: return true; /* 𝩽 */
    case 0x1da7e: return true; /* 𝩾 */
    case 0x1da7f: return true; /* 𝩿 */
    case 0x1da80: return true; /* 𝪀 */
    case 0x1da81: return true; /* 𝪁 */
    case 0x1da82: return true; /* 𝪂 */
    case 0x1da83: return true; /* 𝪃 */
    case 0x1da85: return true; /* 𝪅 */
    case 0x1da86: return true; /* 𝪆 */
    case 0x1da87: return true; /* 𝪇 */
    case 0x1da88: return true; /* 𝪈 */
    case 0x1da89: return true; /* 𝪉 */
    case 0x1da8a: return true; /* 𝪊 */
    case 0x1da8b: return true; /* 𝪋 */
    case 0x1e14f: return true; /* 𞅏 */
    case 0x1e2ff: return true; /* 𞋿 */
    case 0x1e95e: return true; /* 𞥞 */
    case 0x1e95f: return true; /* 𞥟 */
    case 0x1ecac: return true; /* 𞲬 */
    case 0x1ecb0: return true; /* 𞲰 */
    case 0x1ed2e: return true; /* 𞴮 */
    case 0x1eef0: return true; /* 𞻰 */
    case 0x1eef1: return true; /* 𞻱 */
    case 0x1f000: return true; /* 🀀 */
    case 0x1f001: return true; /* 🀁 */
    case 0x1f002: return true; /* 🀂 */
    case 0x1f003: return true; /* 🀃 */
    case 0x1f004: return true; /* 🀄 */
    case 0x1f005: return true; /* 🀅 */
    case 0x1f006: return true; /* 🀆 */
    case 0x1f007: return true; /* 🀇 */
    case 0x1f008: return true; /* 🀈 */
    case 0x1f009: return true; /* 🀉 */
    case 0x1f00a: return true; /* 🀊 */
    case 0x1f00b: return true; /* 🀋 */
    case 0x1f00c: return true; /* 🀌 */
    case 0x1f00d: return true; /* 🀍 */
    case 0x1f00e: return true; /* 🀎 */
    case 0x1f00f: return true; /* 🀏 */
    case 0x1f010: return true; /* 🀐 */
    case 0x1f011: return true; /* 🀑 */
    case 0x1f012: return true; /* 🀒 */
    case 0x1f013: return true; /* 🀓 */
    case 0x1f014: return true; /* 🀔 */
    case 0x1f015: return true; /* 🀕 */
    case 0x1f016: return true; /* 🀖 */
    case 0x1f017: return true; /* 🀗 */
    case 0x1f018: return true; /* 🀘 */
    case 0x1f019: return true; /* 🀙 */
    case 0x1f01a: return true; /* 🀚 */
    case 0x1f01b: return true; /* 🀛 */
    case 0x1f01c: return true; /* 🀜 */
    case 0x1f01d: return true; /* 🀝 */
    case 0x1f01e: return true; /* 🀞 */
    case 0x1f01f: return true; /* 🀟 */
    case 0x1f020: return true; /* 🀠 */
    case 0x1f021: return true; /* 🀡 */
    case 0x1f022: return true; /* 🀢 */
    case 0x1f023: return true; /* 🀣 */
    case 0x1f024: return true; /* 🀤 */
    case 0x1f025: return true; /* 🀥 */
    case 0x1f026: return true; /* 🀦 */
    case 0x1f027: return true; /* 🀧 */
    case 0x1f028: return true; /* 🀨 */
    case 0x1f029: return true; /* 🀩 */
    case 0x1f02a: return true; /* 🀪 */
    case 0x1f02b: return true; /* 🀫 */
    case 0x1f030: return true; /* 🀰 */
    case 0x1f031: return true; /* 🀱 */
    case 0x1f032: return true; /* 🀲 */
    case 0x1f033: return true; /* 🀳 */
    case 0x1f034: return true; /* 🀴 */
    case 0x1f035: return true; /* 🀵 */
    case 0x1f036: return true; /* 🀶 */
    case 0x1f037: return true; /* 🀷 */
    case 0x1f038: return true; /* 🀸 */
    case 0x1f039: return true; /* 🀹 */
    case 0x1f03a: return true; /* 🀺 */
    case 0x1f03b: return true; /* 🀻 */
    case 0x1f03c: return true; /* 🀼 */
    case 0x1f03d: return true; /* 🀽 */
    case 0x1f03e: return true; /* 🀾 */
    case 0x1f03f: return true; /* 🀿 */
    case 0x1f040: return true; /* 🁀 */
    case 0x1f041: return true; /* 🁁 */
    case 0x1f042: return true; /* 🁂 */
    case 0x1f043: return true; /* 🁃 */
    case 0x1f044: return true; /* 🁄 */
    case 0x1f045: return true; /* 🁅 */
    case 0x1f046: return true; /* 🁆 */
    case 0x1f047: return true; /* 🁇 */
    case 0x1f048: return true; /* 🁈 */
    case 0x1f049: return true; /* 🁉 */
    case 0x1f04a: return true; /* 🁊 */
    case 0x1f04b: return true; /* 🁋 */
    case 0x1f04c: return true; /* 🁌 */
    case 0x1f04d: return true; /* 🁍 */
    case 0x1f04e: return true; /* 🁎 */
    case 0x1f04f: return true; /* 🁏 */
    case 0x1f050: return true; /* 🁐 */
    case 0x1f051: return true; /* 🁑 */
    case 0x1f052: return true; /* 🁒 */
    case 0x1f053: return true; /* 🁓 */
    case 0x1f054: return true; /* 🁔 */
    case 0x1f055: return true; /* 🁕 */
    case 0x1f056: return true; /* 🁖 */
    case 0x1f057: return true; /* 🁗 */
    case 0x1f058: return true; /* 🁘 */
    case 0x1f059: return true; /* 🁙 */
    case 0x1f05a: return true; /* 🁚 */
    case 0x1f05b: return true; /* 🁛 */
    case 0x1f05c: return true; /* 🁜 */
    case 0x1f05d: return true; /* 🁝 */
    case 0x1f05e: return true; /* 🁞 */
    case 0x1f05f: return true; /* 🁟 */
    case 0x1f060: return true; /* 🁠 */
    case 0x1f061: return true; /* 🁡 */
    case 0x1f062: return true; /* 🁢 */
    case 0x1f063: return true; /* 🁣 */
    case 0x1f064: return true; /* 🁤 */
    case 0x1f065: return true; /* 🁥 */
    case 0x1f066: return true; /* 🁦 */
    case 0x1f067: return true; /* 🁧 */
    case 0x1f068: return true; /* 🁨 */
    case 0x1f069: return true; /* 🁩 */
    case 0x1f06a: return true; /* 🁪 */
    case 0x1f06b: return true; /* 🁫 */
    case 0x1f06c: return true; /* 🁬 */
    case 0x1f06d: return true; /* 🁭 */
    case 0x1f06e: return true; /* 🁮 */
    case 0x1f06f: return true; /* 🁯 */
    case 0x1f070: return true; /* 🁰 */
    case 0x1f071: return true; /* 🁱 */
    case 0x1f072: return true; /* 🁲 */
    case 0x1f073: return true; /* 🁳 */
    case 0x1f074: return true; /* 🁴 */
    case 0x1f075: return true; /* 🁵 */
    case 0x1f076: return true; /* 🁶 */
    case 0x1f077: return true; /* 🁷 */
    case 0x1f078: return true; /* 🁸 */
    case 0x1f079: return true; /* 🁹 */
    case 0x1f07a: return true; /* 🁺 */
    case 0x1f07b: return true; /* 🁻 */
    case 0x1f07c: return true; /* 🁼 */
    case 0x1f07d: return true; /* 🁽 */
    case 0x1f07e: return true; /* 🁾 */
    case 0x1f07f: return true; /* 🁿 */
    case 0x1f080: return true; /* 🂀 */
    case 0x1f081: return true; /* 🂁 */
    case 0x1f082: return true; /* 🂂 */
    case 0x1f083: return true; /* 🂃 */
    case 0x1f084: return true; /* 🂄 */
    case 0x1f085: return true; /* 🂅 */
    case 0x1f086: return true; /* 🂆 */
    case 0x1f087: return true; /* 🂇 */
    case 0x1f088: return true; /* 🂈 */
    case 0x1f089: return true; /* 🂉 */
    case 0x1f08a: return true; /* 🂊 */
    case 0x1f08b: return true; /* 🂋 */
    case 0x1f08c: return true; /* 🂌 */
    case 0x1f08d: return true; /* 🂍 */
    case 0x1f08e: return true; /* 🂎 */
    case 0x1f08f: return true; /* 🂏 */
    case 0x1f090: return true; /* 🂐 */
    case 0x1f091: return true; /* 🂑 */
    case 0x1f092: return true; /* 🂒 */
    case 0x1f093: return true; /* 🂓 */
    case 0x1f0a0: return true; /* 🂠 */
    case 0x1f0a1: return true; /* 🂡 */
    case 0x1f0a2: return true; /* 🂢 */
    case 0x1f0a3: return true; /* 🂣 */
    case 0x1f0a4: return true; /* 🂤 */
    case 0x1f0a5: return true; /* 🂥 */
    case 0x1f0a6: return true; /* 🂦 */
    case 0x1f0a7: return true; /* 🂧 */
    case 0x1f0a8: return true; /* 🂨 */
    case 0x1f0a9: return true; /* 🂩 */
    case 0x1f0aa: return true; /* 🂪 */
    case 0x1f0ab: return true; /* 🂫 */
    case 0x1f0ac: return true; /* 🂬 */
    case 0x1f0ad: return true; /* 🂭 */
    case 0x1f0ae: return true; /* 🂮 */
    case 0x1f0b1: return true; /* 🂱 */
    case 0x1f0b2: return true; /* 🂲 */
    case 0x1f0b3: return true; /* 🂳 */
    case 0x1f0b4: return true; /* 🂴 */
    case 0x1f0b5: return true; /* 🂵 */
    case 0x1f0b6: return true; /* 🂶 */
    case 0x1f0b7: return true; /* 🂷 */
    case 0x1f0b8: return true; /* 🂸 */
    case 0x1f0b9: return true; /* 🂹 */
    case 0x1f0ba: return true; /* 🂺 */
    case 0x1f0bb: return true; /* 🂻 */
    case 0x1f0bc: return true; /* 🂼 */
    case 0x1f0bd: return true; /* 🂽 */
    case 0x1f0be: return true; /* 🂾 */
    case 0x1f0bf: return true; /* 🂿 */
    case 0x1f0c1: return true; /* 🃁 */
    case 0x1f0c2: return true; /* 🃂 */
    case 0x1f0c3: return true; /* 🃃 */
    case 0x1f0c4: return true; /* 🃄 */
    case 0x1f0c5: return true; /* 🃅 */
    case 0x1f0c6: return true; /* 🃆 */
    case 0x1f0c7: return true; /* 🃇 */
    case 0x1f0c8: return true; /* 🃈 */
    case 0x1f0c9: return true; /* 🃉 */
    case 0x1f0ca: return true; /* 🃊 */
    case 0x1f0cb: return true; /* 🃋 */
    case 0x1f0cc: return true; /* 🃌 */
    case 0x1f0cd: return true; /* 🃍 */
    case 0x1f0ce: return true; /* 🃎 */
    case 0x1f0cf: return true; /* 🃏 */
    case 0x1f0d1: return true; /* 🃑 */
    case 0x1f0d2: return true; /* 🃒 */
    case 0x1f0d3: return true; /* 🃓 */
    case 0x1f0d4: return true; /* 🃔 */
    case 0x1f0d5: return true; /* 🃕 */
    case 0x1f0d6: return true; /* 🃖 */
    case 0x1f0d7: return true; /* 🃗 */
    case 0x1f0d8: return true; /* 🃘 */
    case 0x1f0d9: return true; /* 🃙 */
    case 0x1f0da: return true; /* 🃚 */
    case 0x1f0db: return true; /* 🃛 */
    case 0x1f0dc: return true; /* 🃜 */
    case 0x1f0dd: return true; /* 🃝 */
    case 0x1f0de: return true; /* 🃞 */
    case 0x1f0df: return true; /* 🃟 */
    case 0x1f0e0: return true; /* 🃠 */
    case 0x1f0e1: return true; /* 🃡 */
    case 0x1f0e2: return true; /* 🃢 */
    case 0x1f0e3: return true; /* 🃣 */
    case 0x1f0e4: return true; /* 🃤 */
    case 0x1f0e5: return true; /* 🃥 */
    case 0x1f0e6: return true; /* 🃦 */
    case 0x1f0e7: return true; /* 🃧 */
    case 0x1f0e8: return true; /* 🃨 */
    case 0x1f0e9: return true; /* 🃩 */
    case 0x1f0ea: return true; /* 🃪 */
    case 0x1f0eb: return true; /* 🃫 */
    case 0x1f0ec: return true; /* 🃬 */
    case 0x1f0ed: return true; /* 🃭 */
    case 0x1f0ee: return true; /* 🃮 */
    case 0x1f0ef: return true; /* 🃯 */
    case 0x1f0f0: return true; /* 🃰 */
    case 0x1f0f1: return true; /* 🃱 */
    case 0x1f0f2: return true; /* 🃲 */
    case 0x1f0f3: return true; /* 🃳 */
    case 0x1f0f4: return true; /* 🃴 */
    case 0x1f0f5: return true; /* 🃵 */
    case 0x1f10d: return true; /* 🄍 */
    case 0x1f10e: return true; /* 🄎 */
    case 0x1f10f: return true; /* 🄏 */
    case 0x1f110: return true; /* 🄐 */
    case 0x1f111: return true; /* 🄑 */
    case 0x1f112: return true; /* 🄒 */
    case 0x1f113: return true; /* 🄓 */
    case 0x1f114: return true; /* 🄔 */
    case 0x1f115: return true; /* 🄕 */
    case 0x1f116: return true; /* 🄖 */
    case 0x1f117: return true; /* 🄗 */
    case 0x1f118: return true; /* 🄘 */
    case 0x1f119: return true; /* 🄙 */
    case 0x1f11a: return true; /* 🄚 */
    case 0x1f11b: return true; /* 🄛 */
    case 0x1f11c: return true; /* 🄜 */
    case 0x1f11d: return true; /* 🄝 */
    case 0x1f11e: return true; /* 🄞 */
    case 0x1f11f: return true; /* 🄟 */
    case 0x1f120: return true; /* 🄠 */
    case 0x1f121: return true; /* 🄡 */
    case 0x1f122: return true; /* 🄢 */
    case 0x1f123: return true; /* 🄣 */
    case 0x1f124: return true; /* 🄤 */
    case 0x1f125: return true; /* 🄥 */
    case 0x1f126: return true; /* 🄦 */
    case 0x1f127: return true; /* 🄧 */
    case 0x1f128: return true; /* 🄨 */
    case 0x1f129: return true; /* 🄩 */
    case 0x1f12a: return true; /* 🄪 */
    case 0x1f12b: return true; /* 🄫 */
    case 0x1f12c: return true; /* 🄬 */
    case 0x1f12d: return true; /* 🄭 */
    case 0x1f12e: return true; /* 🄮 */
    case 0x1f12f: return true; /* 🄯 */
    case 0x1f130: return true; /* 🄰 */
    case 0x1f131: return true; /* 🄱 */
    case 0x1f132: return true; /* 🄲 */
    case 0x1f133: return true; /* 🄳 */
    case 0x1f134: return true; /* 🄴 */
    case 0x1f135: return true; /* 🄵 */
    case 0x1f136: return true; /* 🄶 */
    case 0x1f137: return true; /* 🄷 */
    case 0x1f138: return true; /* 🄸 */
    case 0x1f139: return true; /* 🄹 */
    case 0x1f13a: return true; /* 🄺 */
    case 0x1f13b: return true; /* 🄻 */
    case 0x1f13c: return true; /* 🄼 */
    case 0x1f13d: return true; /* 🄽 */
    case 0x1f13e: return true; /* 🄾 */
    case 0x1f13f: return true; /* 🄿 */
    case 0x1f140: return true; /* 🅀 */
    case 0x1f141: return true; /* 🅁 */
    case 0x1f142: return true; /* 🅂 */
    case 0x1f143: return true; /* 🅃 */
    case 0x1f144: return true; /* 🅄 */
    case 0x1f145: return true; /* 🅅 */
    case 0x1f146: return true; /* 🅆 */
    case 0x1f147: return true; /* 🅇 */
    case 0x1f148: return true; /* 🅈 */
    case 0x1f149: return true; /* 🅉 */
    case 0x1f14a: return true; /* 🅊 */
    case 0x1f14b: return true; /* 🅋 */
    case 0x1f14c: return true; /* 🅌 */
    case 0x1f14d: return true; /* 🅍 */
    case 0x1f14e: return true; /* 🅎 */
    case 0x1f14f: return true; /* 🅏 */
    case 0x1f150: return true; /* 🅐 */
    case 0x1f151: return true; /* 🅑 */
    case 0x1f152: return true; /* 🅒 */
    case 0x1f153: return true; /* 🅓 */
    case 0x1f154: return true; /* 🅔 */
    case 0x1f155: return true; /* 🅕 */
    case 0x1f156: return true; /* 🅖 */
    case 0x1f157: return true; /* 🅗 */
    case 0x1f158: return true; /* 🅘 */
    case 0x1f159: return true; /* 🅙 */
    case 0x1f15a: return true; /* 🅚 */
    case 0x1f15b: return true; /* 🅛 */
    case 0x1f15c: return true; /* 🅜 */
    case 0x1f15d: return true; /* 🅝 */
    case 0x1f15e: return true; /* 🅞 */
    case 0x1f15f: return true; /* 🅟 */
    case 0x1f160: return true; /* 🅠 */
    case 0x1f161: return true; /* 🅡 */
    case 0x1f162: return true; /* 🅢 */
    case 0x1f163: return true; /* 🅣 */
    case 0x1f164: return true; /* 🅤 */
    case 0x1f165: return true; /* 🅥 */
    case 0x1f166: return true; /* 🅦 */
    case 0x1f167: return true; /* 🅧 */
    case 0x1f168: return true; /* 🅨 */
    case 0x1f169: return true; /* 🅩 */
    case 0x1f16a: return true; /* 🅪 */
    case 0x1f16b: return true; /* 🅫 */
    case 0x1f16c: return true; /* 🅬 */
    case 0x1f16d: return true; /* 🅭 */
    case 0x1f16e: return true; /* 🅮 */
    case 0x1f16f: return true; /* 🅯 */
    case 0x1f170: return true; /* 🅰 */
    case 0x1f171: return true; /* 🅱 */
    case 0x1f172: return true; /* 🅲 */
    case 0x1f173: return true; /* 🅳 */
    case 0x1f174: return true; /* 🅴 */
    case 0x1f175: return true; /* 🅵 */
    case 0x1f176: return true; /* 🅶 */
    case 0x1f177: return true; /* 🅷 */
    case 0x1f178: return true; /* 🅸 */
    case 0x1f179: return true; /* 🅹 */
    case 0x1f17a: return true; /* 🅺 */
    case 0x1f17b: return true; /* 🅻 */
    case 0x1f17c: return true; /* 🅼 */
    case 0x1f17d: return true; /* 🅽 */
    case 0x1f17e: return true; /* 🅾 */
    case 0x1f17f: return true; /* 🅿 */
    case 0x1f180: return true; /* 🆀 */
    case 0x1f181: return true; /* 🆁 */
    case 0x1f182: return true; /* 🆂 */
    case 0x1f183: return true; /* 🆃 */
    case 0x1f184: return true; /* 🆄 */
    case 0x1f185: return true; /* 🆅 */
    case 0x1f186: return true; /* 🆆 */
    case 0x1f187: return true; /* 🆇 */
    case 0x1f188: return true; /* 🆈 */
    case 0x1f189: return true; /* 🆉 */
    case 0x1f18a: return true; /* 🆊 */
    case 0x1f18b: return true; /* 🆋 */
    case 0x1f18c: return true; /* 🆌 */
    case 0x1f18d: return true; /* 🆍 */
    case 0x1f18e: return true; /* 🆎 */
    case 0x1f18f: return true; /* 🆏 */
    case 0x1f190: return true; /* 🆐 */
    case 0x1f191: return true; /* 🆑 */
    case 0x1f192: return true; /* 🆒 */
    case 0x1f193: return true; /* 🆓 */
    case 0x1f194: return true; /* 🆔 */
    case 0x1f195: return true; /* 🆕 */
    case 0x1f196: return true; /* 🆖 */
    case 0x1f197: return true; /* 🆗 */
    case 0x1f198: return true; /* 🆘 */
    case 0x1f199: return true; /* 🆙 */
    case 0x1f19a: return true; /* 🆚 */
    case 0x1f19b: return true; /* 🆛 */
    case 0x1f19c: return true; /* 🆜 */
    case 0x1f19d: return true; /* 🆝 */
    case 0x1f19e: return true; /* 🆞 */
    case 0x1f19f: return true; /* 🆟 */
    case 0x1f1a0: return true; /* 🆠 */
    case 0x1f1a1: return true; /* 🆡 */
    case 0x1f1a2: return true; /* 🆢 */
    case 0x1f1a3: return true; /* 🆣 */
    case 0x1f1a4: return true; /* 🆤 */
    case 0x1f1a5: return true; /* 🆥 */
    case 0x1f1a6: return true; /* 🆦 */
    case 0x1f1a7: return true; /* 🆧 */
    case 0x1f1a8: return true; /* 🆨 */
    case 0x1f1a9: return true; /* 🆩 */
    case 0x1f1aa: return true; /* 🆪 */
    case 0x1f1ab: return true; /* 🆫 */
    case 0x1f1ac: return true; /* 🆬 */
    case 0x1f1ad: return true; /* 🆭 */
    case 0x1f1e6: return true; /* 🇦 */
    case 0x1f1e7: return true; /* 🇧 */
    case 0x1f1e8: return true; /* 🇨 */
    case 0x1f1e9: return true; /* 🇩 */
    case 0x1f1ea: return true; /* 🇪 */
    case 0x1f1eb: return true; /* 🇫 */
    case 0x1f1ec: return true; /* 🇬 */
    case 0x1f1ed: return true; /* 🇭 */
    case 0x1f1ee: return true; /* 🇮 */
    case 0x1f1ef: return true; /* 🇯 */
    case 0x1f1f0: return true; /* 🇰 */
    case 0x1f1f1: return true; /* 🇱 */
    case 0x1f1f2: return true; /* 🇲 */
    case 0x1f1f3: return true; /* 🇳 */
    case 0x1f1f4: return true; /* 🇴 */
    case 0x1f1f5: return true; /* 🇵 */
    case 0x1f1f6: return true; /* 🇶 */
    case 0x1f1f7: return true; /* 🇷 */
    case 0x1f1f8: return true; /* 🇸 */
    case 0x1f1f9: return true; /* 🇹 */
    case 0x1f1fa: return true; /* 🇺 */
    case 0x1f1fb: return true; /* 🇻 */
    case 0x1f1fc: return true; /* 🇼 */
    case 0x1f1fd: return true; /* 🇽 */
    case 0x1f1fe: return true; /* 🇾 */
    case 0x1f1ff: return true; /* 🇿 */
    case 0x1f200: return true; /* 🈀 */
    case 0x1f201: return true; /* 🈁 */
    case 0x1f202: return true; /* 🈂 */
    case 0x1f210: return true; /* 🈐 */
    case 0x1f211: return true; /* 🈑 */
    case 0x1f212: return true; /* 🈒 */
    case 0x1f213: return true; /* 🈓 */
    case 0x1f214: return true; /* 🈔 */
    case 0x1f215: return true; /* 🈕 */
    case 0x1f216: return true; /* 🈖 */
    case 0x1f217: return true; /* 🈗 */
    case 0x1f218: return true; /* 🈘 */
    case 0x1f219: return true; /* 🈙 */
    case 0x1f21a: return true; /* 🈚 */
    case 0x1f21b: return true; /* 🈛 */
    case 0x1f21c: return true; /* 🈜 */
    case 0x1f21d: return true; /* 🈝 */
    case 0x1f21e: return true; /* 🈞 */
    case 0x1f21f: return true; /* 🈟 */
    case 0x1f220: return true; /* 🈠 */
    case 0x1f221: return true; /* 🈡 */
    case 0x1f222: return true; /* 🈢 */
    case 0x1f223: return true; /* 🈣 */
    case 0x1f224: return true; /* 🈤 */
    case 0x1f225: return true; /* 🈥 */
    case 0x1f226: return true; /* 🈦 */
    case 0x1f227: return true; /* 🈧 */
    case 0x1f228: return true; /* 🈨 */
    case 0x1f229: return true; /* 🈩 */
    case 0x1f22a: return true; /* 🈪 */
    case 0x1f22b: return true; /* 🈫 */
    case 0x1f22c: return true; /* 🈬 */
    case 0x1f22d: return true; /* 🈭 */
    case 0x1f22e: return true; /* 🈮 */
    case 0x1f22f: return true; /* 🈯 */
    case 0x1f230: return true; /* 🈰 */
    case 0x1f231: return true; /* 🈱 */
    case 0x1f232: return true; /* 🈲 */
    case 0x1f233: return true; /* 🈳 */
    case 0x1f234: return true; /* 🈴 */
    case 0x1f235: return true; /* 🈵 */
    case 0x1f236: return true; /* 🈶 */
    case 0x1f237: return true; /* 🈷 */
    case 0x1f238: return true; /* 🈸 */
    case 0x1f239: return true; /* 🈹 */
    case 0x1f23a: return true; /* 🈺 */
    case 0x1f23b: return true; /* 🈻 */
    case 0x1f240: return true; /* 🉀 */
    case 0x1f241: return true; /* 🉁 */
    case 0x1f242: return true; /* 🉂 */
    case 0x1f243: return true; /* 🉃 */
    case 0x1f244: return true; /* 🉄 */
    case 0x1f245: return true; /* 🉅 */
    case 0x1f246: return true; /* 🉆 */
    case 0x1f247: return true; /* 🉇 */
    case 0x1f248: return true; /* 🉈 */
    case 0x1f250: return true; /* 🉐 */
    case 0x1f251: return true; /* 🉑 */
    case 0x1f260: return true; /* 🉠 */
    case 0x1f261: return true; /* 🉡 */
    case 0x1f262: return true; /* 🉢 */
    case 0x1f263: return true; /* 🉣 */
    case 0x1f264: return true; /* 🉤 */
    case 0x1f265: return true; /* 🉥 */
    case 0x1f300: return true; /* 🌀 */
    case 0x1f301: return true; /* 🌁 */
    case 0x1f302: return true; /* 🌂 */
    case 0x1f303: return true; /* 🌃 */
    case 0x1f304: return true; /* 🌄 */
    case 0x1f305: return true; /* 🌅 */
    case 0x1f306: return true; /* 🌆 */
    case 0x1f307: return true; /* 🌇 */
    case 0x1f308: return true; /* 🌈 */
    case 0x1f309: return true; /* 🌉 */
    case 0x1f30a: return true; /* 🌊 */
    case 0x1f30b: return true; /* 🌋 */
    case 0x1f30c: return true; /* 🌌 */
    case 0x1f30d: return true; /* 🌍 */
    case 0x1f30e: return true; /* 🌎 */
    case 0x1f30f: return true; /* 🌏 */
    case 0x1f310: return true; /* 🌐 */
    case 0x1f311: return true; /* 🌑 */
    case 0x1f312: return true; /* 🌒 */
    case 0x1f313: return true; /* 🌓 */
    case 0x1f314: return true; /* 🌔 */
    case 0x1f315: return true; /* 🌕 */
    case 0x1f316: return true; /* 🌖 */
    case 0x1f317: return true; /* 🌗 */
    case 0x1f318: return true; /* 🌘 */
    case 0x1f319: return true; /* 🌙 */
    case 0x1f31a: return true; /* 🌚 */
    case 0x1f31b: return true; /* 🌛 */
    case 0x1f31c: return true; /* 🌜 */
    case 0x1f31d: return true; /* 🌝 */
    case 0x1f31e: return true; /* 🌞 */
    case 0x1f31f: return true; /* 🌟 */
    case 0x1f320: return true; /* 🌠 */
    case 0x1f321: return true; /* 🌡 */
    case 0x1f322: return true; /* 🌢 */
    case 0x1f323: return true; /* 🌣 */
    case 0x1f324: return true; /* 🌤 */
    case 0x1f325: return true; /* 🌥 */
    case 0x1f326: return true; /* 🌦 */
    case 0x1f327: return true; /* 🌧 */
    case 0x1f328: return true; /* 🌨 */
    case 0x1f329: return true; /* 🌩 */
    case 0x1f32a: return true; /* 🌪 */
    case 0x1f32b: return true; /* 🌫 */
    case 0x1f32c: return true; /* 🌬 */
    case 0x1f32d: return true; /* 🌭 */
    case 0x1f32e: return true; /* 🌮 */
    case 0x1f32f: return true; /* 🌯 */
    case 0x1f330: return true; /* 🌰 */
    case 0x1f331: return true; /* 🌱 */
    case 0x1f332: return true; /* 🌲 */
    case 0x1f333: return true; /* 🌳 */
    case 0x1f334: return true; /* 🌴 */
    case 0x1f335: return true; /* 🌵 */
    case 0x1f336: return true; /* 🌶 */
    case 0x1f337: return true; /* 🌷 */
    case 0x1f338: return true; /* 🌸 */
    case 0x1f339: return true; /* 🌹 */
    case 0x1f33a: return true; /* 🌺 */
    case 0x1f33b: return true; /* 🌻 */
    case 0x1f33c: return true; /* 🌼 */
    case 0x1f33d: return true; /* 🌽 */
    case 0x1f33e: return true; /* 🌾 */
    case 0x1f33f: return true; /* 🌿 */
    case 0x1f340: return true; /* 🍀 */
    case 0x1f341: return true; /* 🍁 */
    case 0x1f342: return true; /* 🍂 */
    case 0x1f343: return true; /* 🍃 */
    case 0x1f344: return true; /* 🍄 */
    case 0x1f345: return true; /* 🍅 */
    case 0x1f346: return true; /* 🍆 */
    case 0x1f347: return true; /* 🍇 */
    case 0x1f348: return true; /* 🍈 */
    case 0x1f349: return true; /* 🍉 */
    case 0x1f34a: return true; /* 🍊 */
    case 0x1f34b: return true; /* 🍋 */
    case 0x1f34c: return true; /* 🍌 */
    case 0x1f34d: return true; /* 🍍 */
    case 0x1f34e: return true; /* 🍎 */
    case 0x1f34f: return true; /* 🍏 */
    case 0x1f350: return true; /* 🍐 */
    case 0x1f351: return true; /* 🍑 */
    case 0x1f352: return true; /* 🍒 */
    case 0x1f353: return true; /* 🍓 */
    case 0x1f354: return true; /* 🍔 */
    case 0x1f355: return true; /* 🍕 */
    case 0x1f356: return true; /* 🍖 */
    case 0x1f357: return true; /* 🍗 */
    case 0x1f358: return true; /* 🍘 */
    case 0x1f359: return true; /* 🍙 */
    case 0x1f35a: return true; /* 🍚 */
    case 0x1f35b: return true; /* 🍛 */
    case 0x1f35c: return true; /* 🍜 */
    case 0x1f35d: return true; /* 🍝 */
    case 0x1f35e: return true; /* 🍞 */
    case 0x1f35f: return true; /* 🍟 */
    case 0x1f360: return true; /* 🍠 */
    case 0x1f361: return true; /* 🍡 */
    case 0x1f362: return true; /* 🍢 */
    case 0x1f363: return true; /* 🍣 */
    case 0x1f364: return true; /* 🍤 */
    case 0x1f365: return true; /* 🍥 */
    case 0x1f366: return true; /* 🍦 */
    case 0x1f367: return true; /* 🍧 */
    case 0x1f368: return true; /* 🍨 */
    case 0x1f369: return true; /* 🍩 */
    case 0x1f36a: return true; /* 🍪 */
    case 0x1f36b: return true; /* 🍫 */
    case 0x1f36c: return true; /* 🍬 */
    case 0x1f36d: return true; /* 🍭 */
    case 0x1f36e: return true; /* 🍮 */
    case 0x1f36f: return true; /* 🍯 */
    case 0x1f370: return true; /* 🍰 */
    case 0x1f371: return true; /* 🍱 */
    case 0x1f372: return true; /* 🍲 */
    case 0x1f373: return true; /* 🍳 */
    case 0x1f374: return true; /* 🍴 */
    case 0x1f375: return true; /* 🍵 */
    case 0x1f376: return true; /* 🍶 */
    case 0x1f377: return true; /* 🍷 */
    case 0x1f378: return true; /* 🍸 */
    case 0x1f379: return true; /* 🍹 */
    case 0x1f37a: return true; /* 🍺 */
    case 0x1f37b: return true; /* 🍻 */
    case 0x1f37c: return true; /* 🍼 */
    case 0x1f37d: return true; /* 🍽 */
    case 0x1f37e: return true; /* 🍾 */
    case 0x1f37f: return true; /* 🍿 */
    case 0x1f380: return true; /* 🎀 */
    case 0x1f381: return true; /* 🎁 */
    case 0x1f382: return true; /* 🎂 */
    case 0x1f383: return true; /* 🎃 */
    case 0x1f384: return true; /* 🎄 */
    case 0x1f385: return true; /* 🎅 */
    case 0x1f386: return true; /* 🎆 */
    case 0x1f387: return true; /* 🎇 */
    case 0x1f388: return true; /* 🎈 */
    case 0x1f389: return true; /* 🎉 */
    case 0x1f38a: return true; /* 🎊 */
    case 0x1f38b: return true; /* 🎋 */
    case 0x1f38c: return true; /* 🎌 */
    case 0x1f38d: return true; /* 🎍 */
    case 0x1f38e: return true; /* 🎎 */
    case 0x1f38f: return true; /* 🎏 */
    case 0x1f390: return true; /* 🎐 */
    case 0x1f391: return true; /* 🎑 */
    case 0x1f392: return true; /* 🎒 */
    case 0x1f393: return true; /* 🎓 */
    case 0x1f394: return true; /* 🎔 */
    case 0x1f395: return true; /* 🎕 */
    case 0x1f396: return true; /* 🎖 */
    case 0x1f397: return true; /* 🎗 */
    case 0x1f398: return true; /* 🎘 */
    case 0x1f399: return true; /* 🎙 */
    case 0x1f39a: return true; /* 🎚 */
    case 0x1f39b: return true; /* 🎛 */
    case 0x1f39c: return true; /* 🎜 */
    case 0x1f39d: return true; /* 🎝 */
    case 0x1f39e: return true; /* 🎞 */
    case 0x1f39f: return true; /* 🎟 */
    case 0x1f3a0: return true; /* 🎠 */
    case 0x1f3a1: return true; /* 🎡 */
    case 0x1f3a2: return true; /* 🎢 */
    case 0x1f3a3: return true; /* 🎣 */
    case 0x1f3a4: return true; /* 🎤 */
    case 0x1f3a5: return true; /* 🎥 */
    case 0x1f3a6: return true; /* 🎦 */
    case 0x1f3a7: return true; /* 🎧 */
    case 0x1f3a8: return true; /* 🎨 */
    case 0x1f3a9: return true; /* 🎩 */
    case 0x1f3aa: return true; /* 🎪 */
    case 0x1f3ab: return true; /* 🎫 */
    case 0x1f3ac: return true; /* 🎬 */
    case 0x1f3ad: return true; /* 🎭 */
    case 0x1f3ae: return true; /* 🎮 */
    case 0x1f3af: return true; /* 🎯 */
    case 0x1f3b0: return true; /* 🎰 */
    case 0x1f3b1: return true; /* 🎱 */
    case 0x1f3b2: return true; /* 🎲 */
    case 0x1f3b3: return true; /* 🎳 */
    case 0x1f3b4: return true; /* 🎴 */
    case 0x1f3b5: return true; /* 🎵 */
    case 0x1f3b6: return true; /* 🎶 */
    case 0x1f3b7: return true; /* 🎷 */
    case 0x1f3b8: return true; /* 🎸 */
    case 0x1f3b9: return true; /* 🎹 */
    case 0x1f3ba: return true; /* 🎺 */
    case 0x1f3bb: return true; /* 🎻 */
    case 0x1f3bc: return true; /* 🎼 */
    case 0x1f3bd: return true; /* 🎽 */
    case 0x1f3be: return true; /* 🎾 */
    case 0x1f3bf: return true; /* 🎿 */
    case 0x1f3c0: return true; /* 🏀 */
    case 0x1f3c1: return true; /* 🏁 */
    case 0x1f3c2: return true; /* 🏂 */
    case 0x1f3c3: return true; /* 🏃 */
    case 0x1f3c4: return true; /* 🏄 */
    case 0x1f3c5: return true; /* 🏅 */
    case 0x1f3c6: return true; /* 🏆 */
    case 0x1f3c7: return true; /* 🏇 */
    case 0x1f3c8: return true; /* 🏈 */
    case 0x1f3c9: return true; /* 🏉 */
    case 0x1f3ca: return true; /* 🏊 */
    case 0x1f3cb: return true; /* 🏋 */
    case 0x1f3cc: return true; /* 🏌 */
    case 0x1f3cd: return true; /* 🏍 */
    case 0x1f3ce: return true; /* 🏎 */
    case 0x1f3cf: return true; /* 🏏 */
    case 0x1f3d0: return true; /* 🏐 */
    case 0x1f3d1: return true; /* 🏑 */
    case 0x1f3d2: return true; /* 🏒 */
    case 0x1f3d3: return true; /* 🏓 */
    case 0x1f3d4: return true; /* 🏔 */
    case 0x1f3d5: return true; /* 🏕 */
    case 0x1f3d6: return true; /* 🏖 */
    case 0x1f3d7: return true; /* 🏗 */
    case 0x1f3d8: return true; /* 🏘 */
    case 0x1f3d9: return true; /* 🏙 */
    case 0x1f3da: return true; /* 🏚 */
    case 0x1f3db: return true; /* 🏛 */
    case 0x1f3dc: return true; /* 🏜 */
    case 0x1f3dd: return true; /* 🏝 */
    case 0x1f3de: return true; /* 🏞 */
    case 0x1f3df: return true; /* 🏟 */
    case 0x1f3e0: return true; /* 🏠 */
    case 0x1f3e1: return true; /* 🏡 */
    case 0x1f3e2: return true; /* 🏢 */
    case 0x1f3e3: return true; /* 🏣 */
    case 0x1f3e4: return true; /* 🏤 */
    case 0x1f3e5: return true; /* 🏥 */
    case 0x1f3e6: return true; /* 🏦 */
    case 0x1f3e7: return true; /* 🏧 */
    case 0x1f3e8: return true; /* 🏨 */
    case 0x1f3e9: return true; /* 🏩 */
    case 0x1f3ea: return true; /* 🏪 */
    case 0x1f3eb: return true; /* 🏫 */
    case 0x1f3ec: return true; /* 🏬 */
    case 0x1f3ed: return true; /* 🏭 */
    case 0x1f3ee: return true; /* 🏮 */
    case 0x1f3ef: return true; /* 🏯 */
    case 0x1f3f0: return true; /* 🏰 */
    case 0x1f3f1: return true; /* 🏱 */
    case 0x1f3f2: return true; /* 🏲 */
    case 0x1f3f3: return true; /* 🏳 */
    case 0x1f3f4: return true; /* 🏴 */
    case 0x1f3f5: return true; /* 🏵 */
    case 0x1f3f6: return true; /* 🏶 */
    case 0x1f3f7: return true; /* 🏷 */
    case 0x1f3f8: return true; /* 🏸 */
    case 0x1f3f9: return true; /* 🏹 */
    case 0x1f3fa: return true; /* 🏺 */
    case 0x1f3fb: return true; /* 🏻 */
    case 0x1f3fc: return true; /* 🏼 */
    case 0x1f3fd: return true; /* 🏽 */
    case 0x1f3fe: return true; /* 🏾 */
    case 0x1f3ff: return true; /* 🏿 */
    case 0x1f400: return true; /* 🐀 */
    case 0x1f401: return true; /* 🐁 */
    case 0x1f402: return true; /* 🐂 */
    case 0x1f403: return true; /* 🐃 */
    case 0x1f404: return true; /* 🐄 */
    case 0x1f405: return true; /* 🐅 */
    case 0x1f406: return true; /* 🐆 */
    case 0x1f407: return true; /* 🐇 */
    case 0x1f408: return true; /* 🐈 */
    case 0x1f409: return true; /* 🐉 */
    case 0x1f40a: return true; /* 🐊 */
    case 0x1f40b: return true; /* 🐋 */
    case 0x1f40c: return true; /* 🐌 */
    case 0x1f40d: return true; /* 🐍 */
    case 0x1f40e: return true; /* 🐎 */
    case 0x1f40f: return true; /* 🐏 */
    case 0x1f410: return true; /* 🐐 */
    case 0x1f411: return true; /* 🐑 */
    case 0x1f412: return true; /* 🐒 */
    case 0x1f413: return true; /* 🐓 */
    case 0x1f414: return true; /* 🐔 */
    case 0x1f415: return true; /* 🐕 */
    case 0x1f416: return true; /* 🐖 */
    case 0x1f417: return true; /* 🐗 */
    case 0x1f418: return true; /* 🐘 */
    case 0x1f419: return true; /* 🐙 */
    case 0x1f41a: return true; /* 🐚 */
    case 0x1f41b: return true; /* 🐛 */
    case 0x1f41c: return true; /* 🐜 */
    case 0x1f41d: return true; /* 🐝 */
    case 0x1f41e: return true; /* 🐞 */
    case 0x1f41f: return true; /* 🐟 */
    case 0x1f420: return true; /* 🐠 */
    case 0x1f421: return true; /* 🐡 */
    case 0x1f422: return true; /* 🐢 */
    case 0x1f423: return true; /* 🐣 */
    case 0x1f424: return true; /* 🐤 */
    case 0x1f425: return true; /* 🐥 */
    case 0x1f426: return true; /* 🐦 */
    case 0x1f427: return true; /* 🐧 */
    case 0x1f428: return true; /* 🐨 */
    case 0x1f429: return true; /* 🐩 */
    case 0x1f42a: return true; /* 🐪 */
    case 0x1f42b: return true; /* 🐫 */
    case 0x1f42c: return true; /* 🐬 */
    case 0x1f42d: return true; /* 🐭 */
    case 0x1f42e: return true; /* 🐮 */
    case 0x1f42f: return true; /* 🐯 */
    case 0x1f430: return true; /* 🐰 */
    case 0x1f431: return true; /* 🐱 */
    case 0x1f432: return true; /* 🐲 */
    case 0x1f433: return true; /* 🐳 */
    case 0x1f434: return true; /* 🐴 */
    case 0x1f435: return true; /* 🐵 */
    case 0x1f436: return true; /* 🐶 */
    case 0x1f437: return true; /* 🐷 */
    case 0x1f438: return true; /* 🐸 */
    case 0x1f439: return true; /* 🐹 */
    case 0x1f43a: return true; /* 🐺 */
    case 0x1f43b: return true; /* 🐻 */
    case 0x1f43c: return true; /* 🐼 */
    case 0x1f43d: return true; /* 🐽 */
    case 0x1f43e: return true; /* 🐾 */
    case 0x1f43f: return true; /* 🐿 */
    case 0x1f440: return true; /* 👀 */
    case 0x1f441: return true; /* 👁 */
    case 0x1f442: return true; /* 👂 */
    case 0x1f443: return true; /* 👃 */
    case 0x1f444: return true; /* 👄 */
    case 0x1f445: return true; /* 👅 */
    case 0x1f446: return true; /* 👆 */
    case 0x1f447: return true; /* 👇 */
    case 0x1f448: return true; /* 👈 */
    case 0x1f449: return true; /* 👉 */
    case 0x1f44a: return true; /* 👊 */
    case 0x1f44b: return true; /* 👋 */
    case 0x1f44c: return true; /* 👌 */
    case 0x1f44d: return true; /* 👍 */
    case 0x1f44e: return true; /* 👎 */
    case 0x1f44f: return true; /* 👏 */
    case 0x1f450: return true; /* 👐 */
    case 0x1f451: return true; /* 👑 */
    case 0x1f452: return true; /* 👒 */
    case 0x1f453: return true; /* 👓 */
    case 0x1f454: return true; /* 👔 */
    case 0x1f455: return true; /* 👕 */
    case 0x1f456: return true; /* 👖 */
    case 0x1f457: return true; /* 👗 */
    case 0x1f458: return true; /* 👘 */
    case 0x1f459: return true; /* 👙 */
    case 0x1f45a: return true; /* 👚 */
    case 0x1f45b: return true; /* 👛 */
    case 0x1f45c: return true; /* 👜 */
    case 0x1f45d: return true; /* 👝 */
    case 0x1f45e: return true; /* 👞 */
    case 0x1f45f: return true; /* 👟 */
    case 0x1f460: return true; /* 👠 */
    case 0x1f461: return true; /* 👡 */
    case 0x1f462: return true; /* 👢 */
    case 0x1f463: return true; /* 👣 */
    case 0x1f464: return true; /* 👤 */
    case 0x1f465: return true; /* 👥 */
    case 0x1f466: return true; /* 👦 */
    case 0x1f467: return true; /* 👧 */
    case 0x1f468: return true; /* 👨 */
    case 0x1f469: return true; /* 👩 */
    case 0x1f46a: return true; /* 👪 */
    case 0x1f46b: return true; /* 👫 */
    case 0x1f46c: return true; /* 👬 */
    case 0x1f46d: return true; /* 👭 */
    case 0x1f46e: return true; /* 👮 */
    case 0x1f46f: return true; /* 👯 */
    case 0x1f470: return true; /* 👰 */
    case 0x1f471: return true; /* 👱 */
    case 0x1f472: return true; /* 👲 */
    case 0x1f473: return true; /* 👳 */
    case 0x1f474: return true; /* 👴 */
    case 0x1f475: return true; /* 👵 */
    case 0x1f476: return true; /* 👶 */
    case 0x1f477: return true; /* 👷 */
    case 0x1f478: return true; /* 👸 */
    case 0x1f479: return true; /* 👹 */
    case 0x1f47a: return true; /* 👺 */
    case 0x1f47b: return true; /* 👻 */
    case 0x1f47c: return true; /* 👼 */
    case 0x1f47d: return true; /* 👽 */
    case 0x1f47e: return true; /* 👾 */
    case 0x1f47f: return true; /* 👿 */
    case 0x1f480: return true; /* 💀 */
    case 0x1f481: return true; /* 💁 */
    case 0x1f482: return true; /* 💂 */
    case 0x1f483: return true; /* 💃 */
    case 0x1f484: return true; /* 💄 */
    case 0x1f485: return true; /* 💅 */
    case 0x1f486: return true; /* 💆 */
    case 0x1f487: return true; /* 💇 */
    case 0x1f488: return true; /* 💈 */
    case 0x1f489: return true; /* 💉 */
    case 0x1f48a: return true; /* 💊 */
    case 0x1f48b: return true; /* 💋 */
    case 0x1f48c: return true; /* 💌 */
    case 0x1f48d: return true; /* 💍 */
    case 0x1f48e: return true; /* 💎 */
    case 0x1f48f: return true; /* 💏 */
    case 0x1f490: return true; /* 💐 */
    case 0x1f491: return true; /* 💑 */
    case 0x1f492: return true; /* 💒 */
    case 0x1f493: return true; /* 💓 */
    case 0x1f494: return true; /* 💔 */
    case 0x1f495: return true; /* 💕 */
    case 0x1f496: return true; /* 💖 */
    case 0x1f497: return true; /* 💗 */
    case 0x1f498: return true; /* 💘 */
    case 0x1f499: return true; /* 💙 */
    case 0x1f49a: return true; /* 💚 */
    case 0x1f49b: return true; /* 💛 */
    case 0x1f49c: return true; /* 💜 */
    case 0x1f49d: return true; /* 💝 */
    case 0x1f49e: return true; /* 💞 */
    case 0x1f49f: return true; /* 💟 */
    case 0x1f4a0: return true; /* 💠 */
    case 0x1f4a1: return true; /* 💡 */
    case 0x1f4a2: return true; /* 💢 */
    case 0x1f4a3: return true; /* 💣 */
    case 0x1f4a4: return true; /* 💤 */
    case 0x1f4a5: return true; /* 💥 */
    case 0x1f4a6: return true; /* 💦 */
    case 0x1f4a7: return true; /* 💧 */
    case 0x1f4a8: return true; /* 💨 */
    case 0x1f4a9: return true; /* 💩 */
    case 0x1f4aa: return true; /* 💪 */
    case 0x1f4ab: return true; /* 💫 */
    case 0x1f4ac: return true; /* 💬 */
    case 0x1f4ad: return true; /* 💭 */
    case 0x1f4ae: return true; /* 💮 */
    case 0x1f4af: return true; /* 💯 */
    case 0x1f4b0: return true; /* 💰 */
    case 0x1f4b1: return true; /* 💱 */
    case 0x1f4b2: return true; /* 💲 */
    case 0x1f4b3: return true; /* 💳 */
    case 0x1f4b4: return true; /* 💴 */
    case 0x1f4b5: return true; /* 💵 */
    case 0x1f4b6: return true; /* 💶 */
    case 0x1f4b7: return true; /* 💷 */
    case 0x1f4b8: return true; /* 💸 */
    case 0x1f4b9: return true; /* 💹 */
    case 0x1f4ba: return true; /* 💺 */
    case 0x1f4bb: return true; /* 💻 */
    case 0x1f4bc: return true; /* 💼 */
    case 0x1f4bd: return true; /* 💽 */
    case 0x1f4be: return true; /* 💾 */
    case 0x1f4bf: return true; /* 💿 */
    case 0x1f4c0: return true; /* 📀 */
    case 0x1f4c1: return true; /* 📁 */
    case 0x1f4c2: return true; /* 📂 */
    case 0x1f4c3: return true; /* 📃 */
    case 0x1f4c4: return true; /* 📄 */
    case 0x1f4c5: return true; /* 📅 */
    case 0x1f4c6: return true; /* 📆 */
    case 0x1f4c7: return true; /* 📇 */
    case 0x1f4c8: return true; /* 📈 */
    case 0x1f4c9: return true; /* 📉 */
    case 0x1f4ca: return true; /* 📊 */
    case 0x1f4cb: return true; /* 📋 */
    case 0x1f4cc: return true; /* 📌 */
    case 0x1f4cd: return true; /* 📍 */
    case 0x1f4ce: return true; /* 📎 */
    case 0x1f4cf: return true; /* 📏 */
    case 0x1f4d0: return true; /* 📐 */
    case 0x1f4d1: return true; /* 📑 */
    case 0x1f4d2: return true; /* 📒 */
    case 0x1f4d3: return true; /* 📓 */
    case 0x1f4d4: return true; /* 📔 */
    case 0x1f4d5: return true; /* 📕 */
    case 0x1f4d6: return true; /* 📖 */
    case 0x1f4d7: return true; /* 📗 */
    case 0x1f4d8: return true; /* 📘 */
    case 0x1f4d9: return true; /* 📙 */
    case 0x1f4da: return true; /* 📚 */
    case 0x1f4db: return true; /* 📛 */
    case 0x1f4dc: return true; /* 📜 */
    case 0x1f4dd: return true; /* 📝 */
    case 0x1f4de: return true; /* 📞 */
    case 0x1f4df: return true; /* 📟 */
    case 0x1f4e0: return true; /* 📠 */
    case 0x1f4e1: return true; /* 📡 */
    case 0x1f4e2: return true; /* 📢 */
    case 0x1f4e3: return true; /* 📣 */
    case 0x1f4e4: return true; /* 📤 */
    case 0x1f4e5: return true; /* 📥 */
    case 0x1f4e6: return true; /* 📦 */
    case 0x1f4e7: return true; /* 📧 */
    case 0x1f4e8: return true; /* 📨 */
    case 0x1f4e9: return true; /* 📩 */
    case 0x1f4ea: return true; /* 📪 */
    case 0x1f4eb: return true; /* 📫 */
    case 0x1f4ec: return true; /* 📬 */
    case 0x1f4ed: return true; /* 📭 */
    case 0x1f4ee: return true; /* 📮 */
    case 0x1f4ef: return true; /* 📯 */
    case 0x1f4f0: return true; /* 📰 */
    case 0x1f4f1: return true; /* 📱 */
    case 0x1f4f2: return true; /* 📲 */
    case 0x1f4f3: return true; /* 📳 */
    case 0x1f4f4: return true; /* 📴 */
    case 0x1f4f5: return true; /* 📵 */
    case 0x1f4f6: return true; /* 📶 */
    case 0x1f4f7: return true; /* 📷 */
    case 0x1f4f8: return true; /* 📸 */
    case 0x1f4f9: return true; /* 📹 */
    case 0x1f4fa: return true; /* 📺 */
    case 0x1f4fb: return true; /* 📻 */
    case 0x1f4fc: return true; /* 📼 */
    case 0x1f4fd: return true; /* 📽 */
    case 0x1f4fe: return true; /* 📾 */
    case 0x1f4ff: return true; /* 📿 */
    case 0x1f500: return true; /* 🔀 */
    case 0x1f501: return true; /* 🔁 */
    case 0x1f502: return true; /* 🔂 */
    case 0x1f503: return true; /* 🔃 */
    case 0x1f504: return true; /* 🔄 */
    case 0x1f505: return true; /* 🔅 */
    case 0x1f506: return true; /* 🔆 */
    case 0x1f507: return true; /* 🔇 */
    case 0x1f508: return true; /* 🔈 */
    case 0x1f509: return true; /* 🔉 */
    case 0x1f50a: return true; /* 🔊 */
    case 0x1f50b: return true; /* 🔋 */
    case 0x1f50c: return true; /* 🔌 */
    case 0x1f50d: return true; /* 🔍 */
    case 0x1f50e: return true; /* 🔎 */
    case 0x1f50f: return true; /* 🔏 */
    case 0x1f510: return true; /* 🔐 */
    case 0x1f511: return true; /* 🔑 */
    case 0x1f512: return true; /* 🔒 */
    case 0x1f513: return true; /* 🔓 */
    case 0x1f514: return true; /* 🔔 */
    case 0x1f515: return true; /* 🔕 */
    case 0x1f516: return true; /* 🔖 */
    case 0x1f517: return true; /* 🔗 */
    case 0x1f518: return true; /* 🔘 */
    case 0x1f519: return true; /* 🔙 */
    case 0x1f51a: return true; /* 🔚 */
    case 0x1f51b: return true; /* 🔛 */
    case 0x1f51c: return true; /* 🔜 */
    case 0x1f51d: return true; /* 🔝 */
    case 0x1f51e: return true; /* 🔞 */
    case 0x1f51f: return true; /* 🔟 */
    case 0x1f520: return true; /* 🔠 */
    case 0x1f521: return true; /* 🔡 */
    case 0x1f522: return true; /* 🔢 */
    case 0x1f523: return true; /* 🔣 */
    case 0x1f524: return true; /* 🔤 */
    case 0x1f525: return true; /* 🔥 */
    case 0x1f526: return true; /* 🔦 */
    case 0x1f527: return true; /* 🔧 */
    case 0x1f528: return true; /* 🔨 */
    case 0x1f529: return true; /* 🔩 */
    case 0x1f52a: return true; /* 🔪 */
    case 0x1f52b: return true; /* 🔫 */
    case 0x1f52c: return true; /* 🔬 */
    case 0x1f52d: return true; /* 🔭 */
    case 0x1f52e: return true; /* 🔮 */
    case 0x1f52f: return true; /* 🔯 */
    case 0x1f530: return true; /* 🔰 */
    case 0x1f531: return true; /* 🔱 */
    case 0x1f532: return true; /* 🔲 */
    case 0x1f533: return true; /* 🔳 */
    case 0x1f534: return true; /* 🔴 */
    case 0x1f535: return true; /* 🔵 */
    case 0x1f536: return true; /* 🔶 */
    case 0x1f537: return true; /* 🔷 */
    case 0x1f538: return true; /* 🔸 */
    case 0x1f539: return true; /* 🔹 */
    case 0x1f53a: return true; /* 🔺 */
    case 0x1f53b: return true; /* 🔻 */
    case 0x1f53c: return true; /* 🔼 */
    case 0x1f53d: return true; /* 🔽 */
    case 0x1f53e: return true; /* 🔾 */
    case 0x1f53f: return true; /* 🔿 */
    case 0x1f540: return true; /* 🕀 */
    case 0x1f541: return true; /* 🕁 */
    case 0x1f542: return true; /* 🕂 */
    case 0x1f543: return true; /* 🕃 */
    case 0x1f544: return true; /* 🕄 */
    case 0x1f545: return true; /* 🕅 */
    case 0x1f546: return true; /* 🕆 */
    case 0x1f547: return true; /* 🕇 */
    case 0x1f548: return true; /* 🕈 */
    case 0x1f549: return true; /* 🕉 */
    case 0x1f54a: return true; /* 🕊 */
    case 0x1f54b: return true; /* 🕋 */
    case 0x1f54c: return true; /* 🕌 */
    case 0x1f54d: return true; /* 🕍 */
    case 0x1f54e: return true; /* 🕎 */
    case 0x1f54f: return true; /* 🕏 */
    case 0x1f550: return true; /* 🕐 */
    case 0x1f551: return true; /* 🕑 */
    case 0x1f552: return true; /* 🕒 */
    case 0x1f553: return true; /* 🕓 */
    case 0x1f554: return true; /* 🕔 */
    case 0x1f555: return true; /* 🕕 */
    case 0x1f556: return true; /* 🕖 */
    case 0x1f557: return true; /* 🕗 */
    case 0x1f558: return true; /* 🕘 */
    case 0x1f559: return true; /* 🕙 */
    case 0x1f55a: return true; /* 🕚 */
    case 0x1f55b: return true; /* 🕛 */
    case 0x1f55c: return true; /* 🕜 */
    case 0x1f55d: return true; /* 🕝 */
    case 0x1f55e: return true; /* 🕞 */
    case 0x1f55f: return true; /* 🕟 */
    case 0x1f560: return true; /* 🕠 */
    case 0x1f561: return true; /* 🕡 */
    case 0x1f562: return true; /* 🕢 */
    case 0x1f563: return true; /* 🕣 */
    case 0x1f564: return true; /* 🕤 */
    case 0x1f565: return true; /* 🕥 */
    case 0x1f566: return true; /* 🕦 */
    case 0x1f567: return true; /* 🕧 */
    case 0x1f568: return true; /* 🕨 */
    case 0x1f569: return true; /* 🕩 */
    case 0x1f56a: return true; /* 🕪 */
    case 0x1f56b: return true; /* 🕫 */
    case 0x1f56c: return true; /* 🕬 */
    case 0x1f56d: return true; /* 🕭 */
    case 0x1f56e: return true; /* 🕮 */
    case 0x1f56f: return true; /* 🕯 */
    case 0x1f570: return true; /* 🕰 */
    case 0x1f571: return true; /* 🕱 */
    case 0x1f572: return true; /* 🕲 */
    case 0x1f573: return true; /* 🕳 */
    case 0x1f574: return true; /* 🕴 */
    case 0x1f575: return true; /* 🕵 */
    case 0x1f576: return true; /* 🕶 */
    case 0x1f577: return true; /* 🕷 */
    case 0x1f578: return true; /* 🕸 */
    case 0x1f579: return true; /* 🕹 */
    case 0x1f57a: return true; /* 🕺 */
    case 0x1f57b: return true; /* 🕻 */
    case 0x1f57c: return true; /* 🕼 */
    case 0x1f57d: return true; /* 🕽 */
    case 0x1f57e: return true; /* 🕾 */
    case 0x1f57f: return true; /* 🕿 */
    case 0x1f580: return true; /* 🖀 */
    case 0x1f581: return true; /* 🖁 */
    case 0x1f582: return true; /* 🖂 */
    case 0x1f583: return true; /* 🖃 */
    case 0x1f584: return true; /* 🖄 */
    case 0x1f585: return true; /* 🖅 */
    case 0x1f586: return true; /* 🖆 */
    case 0x1f587: return true; /* 🖇 */
    case 0x1f588: return true; /* 🖈 */
    case 0x1f589: return true; /* 🖉 */
    case 0x1f58a: return true; /* 🖊 */
    case 0x1f58b: return true; /* 🖋 */
    case 0x1f58c: return true; /* 🖌 */
    case 0x1f58d: return true; /* 🖍 */
    case 0x1f58e: return true; /* 🖎 */
    case 0x1f58f: return true; /* 🖏 */
    case 0x1f590: return true; /* 🖐 */
    case 0x1f591: return true; /* 🖑 */
    case 0x1f592: return true; /* 🖒 */
    case 0x1f593: return true; /* 🖓 */
    case 0x1f594: return true; /* 🖔 */
    case 0x1f595: return true; /* 🖕 */
    case 0x1f596: return true; /* 🖖 */
    case 0x1f597: return true; /* 🖗 */
    case 0x1f598: return true; /* 🖘 */
    case 0x1f599: return true; /* 🖙 */
    case 0x1f59a: return true; /* 🖚 */
    case 0x1f59b: return true; /* 🖛 */
    case 0x1f59c: return true; /* 🖜 */
    case 0x1f59d: return true; /* 🖝 */
    case 0x1f59e: return true; /* 🖞 */
    case 0x1f59f: return true; /* 🖟 */
    case 0x1f5a0: return true; /* 🖠 */
    case 0x1f5a1: return true; /* 🖡 */
    case 0x1f5a2: return true; /* 🖢 */
    case 0x1f5a3: return true; /* 🖣 */
    case 0x1f5a4: return true; /* 🖤 */
    case 0x1f5a5: return true; /* 🖥 */
    case 0x1f5a6: return true; /* 🖦 */
    case 0x1f5a7: return true; /* 🖧 */
    case 0x1f5a8: return true; /* 🖨 */
    case 0x1f5a9: return true; /* 🖩 */
    case 0x1f5aa: return true; /* 🖪 */
    case 0x1f5ab: return true; /* 🖫 */
    case 0x1f5ac: return true; /* 🖬 */
    case 0x1f5ad: return true; /* 🖭 */
    case 0x1f5ae: return true; /* 🖮 */
    case 0x1f5af: return true; /* 🖯 */
    case 0x1f5b0: return true; /* 🖰 */
    case 0x1f5b1: return true; /* 🖱 */
    case 0x1f5b2: return true; /* 🖲 */
    case 0x1f5b3: return true; /* 🖳 */
    case 0x1f5b4: return true; /* 🖴 */
    case 0x1f5b5: return true; /* 🖵 */
    case 0x1f5b6: return true; /* 🖶 */
    case 0x1f5b7: return true; /* 🖷 */
    case 0x1f5b8: return true; /* 🖸 */
    case 0x1f5b9: return true; /* 🖹 */
    case 0x1f5ba: return true; /* 🖺 */
    case 0x1f5bb: return true; /* 🖻 */
    case 0x1f5bc: return true; /* 🖼 */
    case 0x1f5bd: return true; /* 🖽 */
    case 0x1f5be: return true; /* 🖾 */
    case 0x1f5bf: return true; /* 🖿 */
    case 0x1f5c0: return true; /* 🗀 */
    case 0x1f5c1: return true; /* 🗁 */
    case 0x1f5c2: return true; /* 🗂 */
    case 0x1f5c3: return true; /* 🗃 */
    case 0x1f5c4: return true; /* 🗄 */
    case 0x1f5c5: return true; /* 🗅 */
    case 0x1f5c6: return true; /* 🗆 */
    case 0x1f5c7: return true; /* 🗇 */
    case 0x1f5c8: return true; /* 🗈 */
    case 0x1f5c9: return true; /* 🗉 */
    case 0x1f5ca: return true; /* 🗊 */
    case 0x1f5cb: return true; /* 🗋 */
    case 0x1f5cc: return true; /* 🗌 */
    case 0x1f5cd: return true; /* 🗍 */
    case 0x1f5ce: return true; /* 🗎 */
    case 0x1f5cf: return true; /* 🗏 */
    case 0x1f5d0: return true; /* 🗐 */
    case 0x1f5d1: return true; /* 🗑 */
    case 0x1f5d2: return true; /* 🗒 */
    case 0x1f5d3: return true; /* 🗓 */
    case 0x1f5d4: return true; /* 🗔 */
    case 0x1f5d5: return true; /* 🗕 */
    case 0x1f5d6: return true; /* 🗖 */
    case 0x1f5d7: return true; /* 🗗 */
    case 0x1f5d8: return true; /* 🗘 */
    case 0x1f5d9: return true; /* 🗙 */
    case 0x1f5da: return true; /* 🗚 */
    case 0x1f5db: return true; /* 🗛 */
    case 0x1f5dc: return true; /* 🗜 */
    case 0x1f5dd: return true; /* 🗝 */
    case 0x1f5de: return true; /* 🗞 */
    case 0x1f5df: return true; /* 🗟 */
    case 0x1f5e0: return true; /* 🗠 */
    case 0x1f5e1: return true; /* 🗡 */
    case 0x1f5e2: return true; /* 🗢 */
    case 0x1f5e3: return true; /* 🗣 */
    case 0x1f5e4: return true; /* 🗤 */
    case 0x1f5e5: return true; /* 🗥 */
    case 0x1f5e6: return true; /* 🗦 */
    case 0x1f5e7: return true; /* 🗧 */
    case 0x1f5e8: return true; /* 🗨 */
    case 0x1f5e9: return true; /* 🗩 */
    case 0x1f5ea: return true; /* 🗪 */
    case 0x1f5eb: return true; /* 🗫 */
    case 0x1f5ec: return true; /* 🗬 */
    case 0x1f5ed: return true; /* 🗭 */
    case 0x1f5ee: return true; /* 🗮 */
    case 0x1f5ef: return true; /* 🗯 */
    case 0x1f5f0: return true; /* 🗰 */
    case 0x1f5f1: return true; /* 🗱 */
    case 0x1f5f2: return true; /* 🗲 */
    case 0x1f5f3: return true; /* 🗳 */
    case 0x1f5f4: return true; /* 🗴 */
    case 0x1f5f5: return true; /* 🗵 */
    case 0x1f5f6: return true; /* 🗶 */
    case 0x1f5f7: return true; /* 🗷 */
    case 0x1f5f8: return true; /* 🗸 */
    case 0x1f5f9: return true; /* 🗹 */
    case 0x1f5fa: return true; /* 🗺 */
    case 0x1f5fb: return true; /* 🗻 */
    case 0x1f5fc: return true; /* 🗼 */
    case 0x1f5fd: return true; /* 🗽 */
    case 0x1f5fe: return true; /* 🗾 */
    case 0x1f5ff: return true; /* 🗿 */
    case 0x1f600: return true; /* 😀 */
    case 0x1f601: return true; /* 😁 */
    case 0x1f602: return true; /* 😂 */
    case 0x1f603: return true; /* 😃 */
    case 0x1f604: return true; /* 😄 */
    case 0x1f605: return true; /* 😅 */
    case 0x1f606: return true; /* 😆 */
    case 0x1f607: return true; /* 😇 */
    case 0x1f608: return true; /* 😈 */
    case 0x1f609: return true; /* 😉 */
    case 0x1f60a: return true; /* 😊 */
    case 0x1f60b: return true; /* 😋 */
    case 0x1f60c: return true; /* 😌 */
    case 0x1f60d: return true; /* 😍 */
    case 0x1f60e: return true; /* 😎 */
    case 0x1f60f: return true; /* 😏 */
    case 0x1f610: return true; /* 😐 */
    case 0x1f611: return true; /* 😑 */
    case 0x1f612: return true; /* 😒 */
    case 0x1f613: return true; /* 😓 */
    case 0x1f614: return true; /* 😔 */
    case 0x1f615: return true; /* 😕 */
    case 0x1f616: return true; /* 😖 */
    case 0x1f617: return true; /* 😗 */
    case 0x1f618: return true; /* 😘 */
    case 0x1f619: return true; /* 😙 */
    case 0x1f61a: return true; /* 😚 */
    case 0x1f61b: return true; /* 😛 */
    case 0x1f61c: return true; /* 😜 */
    case 0x1f61d: return true; /* 😝 */
    case 0x1f61e: return true; /* 😞 */
    case 0x1f61f: return true; /* 😟 */
    case 0x1f620: return true; /* 😠 */
    case 0x1f621: return true; /* 😡 */
    case 0x1f622: return true; /* 😢 */
    case 0x1f623: return true; /* 😣 */
    case 0x1f624: return true; /* 😤 */
    case 0x1f625: return true; /* 😥 */
    case 0x1f626: return true; /* 😦 */
    case 0x1f627: return true; /* 😧 */
    case 0x1f628: return true; /* 😨 */
    case 0x1f629: return true; /* 😩 */
    case 0x1f62a: return true; /* 😪 */
    case 0x1f62b: return true; /* 😫 */
    case 0x1f62c: return true; /* 😬 */
    case 0x1f62d: return true; /* 😭 */
    case 0x1f62e: return true; /* 😮 */
    case 0x1f62f: return true; /* 😯 */
    case 0x1f630: return true; /* 😰 */
    case 0x1f631: return true; /* 😱 */
    case 0x1f632: return true; /* 😲 */
    case 0x1f633: return true; /* 😳 */
    case 0x1f634: return true; /* 😴 */
    case 0x1f635: return true; /* 😵 */
    case 0x1f636: return true; /* 😶 */
    case 0x1f637: return true; /* 😷 */
    case 0x1f638: return true; /* 😸 */
    case 0x1f639: return true; /* 😹 */
    case 0x1f63a: return true; /* 😺 */
    case 0x1f63b: return true; /* 😻 */
    case 0x1f63c: return true; /* 😼 */
    case 0x1f63d: return true; /* 😽 */
    case 0x1f63e: return true; /* 😾 */
    case 0x1f63f: return true; /* 😿 */
    case 0x1f640: return true; /* 🙀 */
    case 0x1f641: return true; /* 🙁 */
    case 0x1f642: return true; /* 🙂 */
    case 0x1f643: return true; /* 🙃 */
    case 0x1f644: return true; /* 🙄 */
    case 0x1f645: return true; /* 🙅 */
    case 0x1f646: return true; /* 🙆 */
    case 0x1f647: return true; /* 🙇 */
    case 0x1f648: return true; /* 🙈 */
    case 0x1f649: return true; /* 🙉 */
    case 0x1f64a: return true; /* 🙊 */
    case 0x1f64b: return true; /* 🙋 */
    case 0x1f64c: return true; /* 🙌 */
    case 0x1f64d: return true; /* 🙍 */
    case 0x1f64e: return true; /* 🙎 */
    case 0x1f64f: return true; /* 🙏 */
    case 0x1f650: return true; /* 🙐 */
    case 0x1f651: return true; /* 🙑 */
    case 0x1f652: return true; /* 🙒 */
    case 0x1f653: return true; /* 🙓 */
    case 0x1f654: return true; /* 🙔 */
    case 0x1f655: return true; /* 🙕 */
    case 0x1f656: return true; /* 🙖 */
    case 0x1f657: return true; /* 🙗 */
    case 0x1f658: return true; /* 🙘 */
    case 0x1f659: return true; /* 🙙 */
    case 0x1f65a: return true; /* 🙚 */
    case 0x1f65b: return true; /* 🙛 */
    case 0x1f65c: return true; /* 🙜 */
    case 0x1f65d: return true; /* 🙝 */
    case 0x1f65e: return true; /* 🙞 */
    case 0x1f65f: return true; /* 🙟 */
    case 0x1f660: return true; /* 🙠 */
    case 0x1f661: return true; /* 🙡 */
    case 0x1f662: return true; /* 🙢 */
    case 0x1f663: return true; /* 🙣 */
    case 0x1f664: return true; /* 🙤 */
    case 0x1f665: return true; /* 🙥 */
    case 0x1f666: return true; /* 🙦 */
    case 0x1f667: return true; /* 🙧 */
    case 0x1f668: return true; /* 🙨 */
    case 0x1f669: return true; /* 🙩 */
    case 0x1f66a: return true; /* 🙪 */
    case 0x1f66b: return true; /* 🙫 */
    case 0x1f66c: return true; /* 🙬 */
    case 0x1f66d: return true; /* 🙭 */
    case 0x1f66e: return true; /* 🙮 */
    case 0x1f66f: return true; /* 🙯 */
    case 0x1f670: return true; /* 🙰 */
    case 0x1f671: return true; /* 🙱 */
    case 0x1f672: return true; /* 🙲 */
    case 0x1f673: return true; /* 🙳 */
    case 0x1f674: return true; /* 🙴 */
    case 0x1f675: return true; /* 🙵 */
    case 0x1f676: return true; /* 🙶 */
    case 0x1f677: return true; /* 🙷 */
    case 0x1f678: return true; /* 🙸 */
    case 0x1f679: return true; /* 🙹 */
    case 0x1f67a: return true; /* 🙺 */
    case 0x1f67b: return true; /* 🙻 */
    case 0x1f67c: return true; /* 🙼 */
    case 0x1f67d: return true; /* 🙽 */
    case 0x1f67e: return true; /* 🙾 */
    case 0x1f67f: return true; /* 🙿 */
    case 0x1f680: return true; /* 🚀 */
    case 0x1f681: return true; /* 🚁 */
    case 0x1f682: return true; /* 🚂 */
    case 0x1f683: return true; /* 🚃 */
    case 0x1f684: return true; /* 🚄 */
    case 0x1f685: return true; /* 🚅 */
    case 0x1f686: return true; /* 🚆 */
    case 0x1f687: return true; /* 🚇 */
    case 0x1f688: return true; /* 🚈 */
    case 0x1f689: return true; /* 🚉 */
    case 0x1f68a: return true; /* 🚊 */
    case 0x1f68b: return true; /* 🚋 */
    case 0x1f68c: return true; /* 🚌 */
    case 0x1f68d: return true; /* 🚍 */
    case 0x1f68e: return true; /* 🚎 */
    case 0x1f68f: return true; /* 🚏 */
    case 0x1f690: return true; /* 🚐 */
    case 0x1f691: return true; /* 🚑 */
    case 0x1f692: return true; /* 🚒 */
    case 0x1f693: return true; /* 🚓 */
    case 0x1f694: return true; /* 🚔 */
    case 0x1f695: return true; /* 🚕 */
    case 0x1f696: return true; /* 🚖 */
    case 0x1f697: return true; /* 🚗 */
    case 0x1f698: return true; /* 🚘 */
    case 0x1f699: return true; /* 🚙 */
    case 0x1f69a: return true; /* 🚚 */
    case 0x1f69b: return true; /* 🚛 */
    case 0x1f69c: return true; /* 🚜 */
    case 0x1f69d: return true; /* 🚝 */
    case 0x1f69e: return true; /* 🚞 */
    case 0x1f69f: return true; /* 🚟 */
    case 0x1f6a0: return true; /* 🚠 */
    case 0x1f6a1: return true; /* 🚡 */
    case 0x1f6a2: return true; /* 🚢 */
    case 0x1f6a3: return true; /* 🚣 */
    case 0x1f6a4: return true; /* 🚤 */
    case 0x1f6a5: return true; /* 🚥 */
    case 0x1f6a6: return true; /* 🚦 */
    case 0x1f6a7: return true; /* 🚧 */
    case 0x1f6a8: return true; /* 🚨 */
    case 0x1f6a9: return true; /* 🚩 */
    case 0x1f6aa: return true; /* 🚪 */
    case 0x1f6ab: return true; /* 🚫 */
    case 0x1f6ac: return true; /* 🚬 */
    case 0x1f6ad: return true; /* 🚭 */
    case 0x1f6ae: return true; /* 🚮 */
    case 0x1f6af: return true; /* 🚯 */
    case 0x1f6b0: return true; /* 🚰 */
    case 0x1f6b1: return true; /* 🚱 */
    case 0x1f6b2: return true; /* 🚲 */
    case 0x1f6b3: return true; /* 🚳 */
    case 0x1f6b4: return true; /* 🚴 */
    case 0x1f6b5: return true; /* 🚵 */
    case 0x1f6b6: return true; /* 🚶 */
    case 0x1f6b7: return true; /* 🚷 */
    case 0x1f6b8: return true; /* 🚸 */
    case 0x1f6b9: return true; /* 🚹 */
    case 0x1f6ba: return true; /* 🚺 */
    case 0x1f6bb: return true; /* 🚻 */
    case 0x1f6bc: return true; /* 🚼 */
    case 0x1f6bd: return true; /* 🚽 */
    case 0x1f6be: return true; /* 🚾 */
    case 0x1f6bf: return true; /* 🚿 */
    case 0x1f6c0: return true; /* 🛀 */
    case 0x1f6c1: return true; /* 🛁 */
    case 0x1f6c2: return true; /* 🛂 */
    case 0x1f6c3: return true; /* 🛃 */
    case 0x1f6c4: return true; /* 🛄 */
    case 0x1f6c5: return true; /* 🛅 */
    case 0x1f6c6: return true; /* 🛆 */
    case 0x1f6c7: return true; /* 🛇 */
    case 0x1f6c8: return true; /* 🛈 */
    case 0x1f6c9: return true; /* 🛉 */
    case 0x1f6ca: return true; /* 🛊 */
    case 0x1f6cb: return true; /* 🛋 */
    case 0x1f6cc: return true; /* 🛌 */
    case 0x1f6cd: return true; /* 🛍 */
    case 0x1f6ce: return true; /* 🛎 */
    case 0x1f6cf: return true; /* 🛏 */
    case 0x1f6d0: return true; /* 🛐 */
    case 0x1f6d1: return true; /* 🛑 */
    case 0x1f6d2: return true; /* 🛒 */
    case 0x1f6d3: return true; /* 🛓 */
    case 0x1f6d4: return true; /* 🛔 */
    case 0x1f6d5: return true; /* 🛕 */
    case 0x1f6d6: return true; /* 🛖 */
    case 0x1f6d7: return true; /* 🛗 */
    case 0x1f6e0: return true; /* 🛠 */
    case 0x1f6e1: return true; /* 🛡 */
    case 0x1f6e2: return true; /* 🛢 */
    case 0x1f6e3: return true; /* 🛣 */
    case 0x1f6e4: return true; /* 🛤 */
    case 0x1f6e5: return true; /* 🛥 */
    case 0x1f6e6: return true; /* 🛦 */
    case 0x1f6e7: return true; /* 🛧 */
    case 0x1f6e8: return true; /* 🛨 */
    case 0x1f6e9: return true; /* 🛩 */
    case 0x1f6ea: return true; /* 🛪 */
    case 0x1f6eb: return true; /* 🛫 */
    case 0x1f6ec: return true; /* 🛬 */
    case 0x1f6f0: return true; /* 🛰 */
    case 0x1f6f1: return true; /* 🛱 */
    case 0x1f6f2: return true; /* 🛲 */
    case 0x1f6f3: return true; /* 🛳 */
    case 0x1f6f4: return true; /* 🛴 */
    case 0x1f6f5: return true; /* 🛵 */
    case 0x1f6f6: return true; /* 🛶 */
    case 0x1f6f7: return true; /* 🛷 */
    case 0x1f6f8: return true; /* 🛸 */
    case 0x1f6f9: return true; /* 🛹 */
    case 0x1f6fa: return true; /* 🛺 */
    case 0x1f6fb: return true; /* 🛻 */
    case 0x1f6fc: return true; /* 🛼 */
    case 0x1f700: return true; /* 🜀 */
    case 0x1f701: return true; /* 🜁 */
    case 0x1f702: return true; /* 🜂 */
    case 0x1f703: return true; /* 🜃 */
    case 0x1f704: return true; /* 🜄 */
    case 0x1f705: return true; /* 🜅 */
    case 0x1f706: return true; /* 🜆 */
    case 0x1f707: return true; /* 🜇 */
    case 0x1f708: return true; /* 🜈 */
    case 0x1f709: return true; /* 🜉 */
    case 0x1f70a: return true; /* 🜊 */
    case 0x1f70b: return true; /* 🜋 */
    case 0x1f70c: return true; /* 🜌 */
    case 0x1f70d: return true; /* 🜍 */
    case 0x1f70e: return true; /* 🜎 */
    case 0x1f70f: return true; /* 🜏 */
    case 0x1f710: return true; /* 🜐 */
    case 0x1f711: return true; /* 🜑 */
    case 0x1f712: return true; /* 🜒 */
    case 0x1f713: return true; /* 🜓 */
    case 0x1f714: return true; /* 🜔 */
    case 0x1f715: return true; /* 🜕 */
    case 0x1f716: return true; /* 🜖 */
    case 0x1f717: return true; /* 🜗 */
    case 0x1f718: return true; /* 🜘 */
    case 0x1f719: return true; /* 🜙 */
    case 0x1f71a: return true; /* 🜚 */
    case 0x1f71b: return true; /* 🜛 */
    case 0x1f71c: return true; /* 🜜 */
    case 0x1f71d: return true; /* 🜝 */
    case 0x1f71e: return true; /* 🜞 */
    case 0x1f71f: return true; /* 🜟 */
    case 0x1f720: return true; /* 🜠 */
    case 0x1f721: return true; /* 🜡 */
    case 0x1f722: return true; /* 🜢 */
    case 0x1f723: return true; /* 🜣 */
    case 0x1f724: return true; /* 🜤 */
    case 0x1f725: return true; /* 🜥 */
    case 0x1f726: return true; /* 🜦 */
    case 0x1f727: return true; /* 🜧 */
    case 0x1f728: return true; /* 🜨 */
    case 0x1f729: return true; /* 🜩 */
    case 0x1f72a: return true; /* 🜪 */
    case 0x1f72b: return true; /* 🜫 */
    case 0x1f72c: return true; /* 🜬 */
    case 0x1f72d: return true; /* 🜭 */
    case 0x1f72e: return true; /* 🜮 */
    case 0x1f72f: return true; /* 🜯 */
    case 0x1f730: return true; /* 🜰 */
    case 0x1f731: return true; /* 🜱 */
    case 0x1f732: return true; /* 🜲 */
    case 0x1f733: return true; /* 🜳 */
    case 0x1f734: return true; /* 🜴 */
    case 0x1f735: return true; /* 🜵 */
    case 0x1f736: return true; /* 🜶 */
    case 0x1f737: return true; /* 🜷 */
    case 0x1f738: return true; /* 🜸 */
    case 0x1f739: return true; /* 🜹 */
    case 0x1f73a: return true; /* 🜺 */
    case 0x1f73b: return true; /* 🜻 */
    case 0x1f73c: return true; /* 🜼 */
    case 0x1f73d: return true; /* 🜽 */
    case 0x1f73e: return true; /* 🜾 */
    case 0x1f73f: return true; /* 🜿 */
    case 0x1f740: return true; /* 🝀 */
    case 0x1f741: return true; /* 🝁 */
    case 0x1f742: return true; /* 🝂 */
    case 0x1f743: return true; /* 🝃 */
    case 0x1f744: return true; /* 🝄 */
    case 0x1f745: return true; /* 🝅 */
    case 0x1f746: return true; /* 🝆 */
    case 0x1f747: return true; /* 🝇 */
    case 0x1f748: return true; /* 🝈 */
    case 0x1f749: return true; /* 🝉 */
    case 0x1f74a: return true; /* 🝊 */
    case 0x1f74b: return true; /* 🝋 */
    case 0x1f74c: return true; /* 🝌 */
    case 0x1f74d: return true; /* 🝍 */
    case 0x1f74e: return true; /* 🝎 */
    case 0x1f74f: return true; /* 🝏 */
    case 0x1f750: return true; /* 🝐 */
    case 0x1f751: return true; /* 🝑 */
    case 0x1f752: return true; /* 🝒 */
    case 0x1f753: return true; /* 🝓 */
    case 0x1f754: return true; /* 🝔 */
    case 0x1f755: return true; /* 🝕 */
    case 0x1f756: return true; /* 🝖 */
    case 0x1f757: return true; /* 🝗 */
    case 0x1f758: return true; /* 🝘 */
    case 0x1f759: return true; /* 🝙 */
    case 0x1f75a: return true; /* 🝚 */
    case 0x1f75b: return true; /* 🝛 */
    case 0x1f75c: return true; /* 🝜 */
    case 0x1f75d: return true; /* 🝝 */
    case 0x1f75e: return true; /* 🝞 */
    case 0x1f75f: return true; /* 🝟 */
    case 0x1f760: return true; /* 🝠 */
    case 0x1f761: return true; /* 🝡 */
    case 0x1f762: return true; /* 🝢 */
    case 0x1f763: return true; /* 🝣 */
    case 0x1f764: return true; /* 🝤 */
    case 0x1f765: return true; /* 🝥 */
    case 0x1f766: return true; /* 🝦 */
    case 0x1f767: return true; /* 🝧 */
    case 0x1f768: return true; /* 🝨 */
    case 0x1f769: return true; /* 🝩 */
    case 0x1f76a: return true; /* 🝪 */
    case 0x1f76b: return true; /* 🝫 */
    case 0x1f76c: return true; /* 🝬 */
    case 0x1f76d: return true; /* 🝭 */
    case 0x1f76e: return true; /* 🝮 */
    case 0x1f76f: return true; /* 🝯 */
    case 0x1f770: return true; /* 🝰 */
    case 0x1f771: return true; /* 🝱 */
    case 0x1f772: return true; /* 🝲 */
    case 0x1f773: return true; /* 🝳 */
    case 0x1f780: return true; /* 🞀 */
    case 0x1f781: return true; /* 🞁 */
    case 0x1f782: return true; /* 🞂 */
    case 0x1f783: return true; /* 🞃 */
    case 0x1f784: return true; /* 🞄 */
    case 0x1f785: return true; /* 🞅 */
    case 0x1f786: return true; /* 🞆 */
    case 0x1f787: return true; /* 🞇 */
    case 0x1f788: return true; /* 🞈 */
    case 0x1f789: return true; /* 🞉 */
    case 0x1f78a: return true; /* 🞊 */
    case 0x1f78b: return true; /* 🞋 */
    case 0x1f78c: return true; /* 🞌 */
    case 0x1f78d: return true; /* 🞍 */
    case 0x1f78e: return true; /* 🞎 */
    case 0x1f78f: return true; /* 🞏 */
    case 0x1f790: return true; /* 🞐 */
    case 0x1f791: return true; /* 🞑 */
    case 0x1f792: return true; /* 🞒 */
    case 0x1f793: return true; /* 🞓 */
    case 0x1f794: return true; /* 🞔 */
    case 0x1f795: return true; /* 🞕 */
    case 0x1f796: return true; /* 🞖 */
    case 0x1f797: return true; /* 🞗 */
    case 0x1f798: return true; /* 🞘 */
    case 0x1f799: return true; /* 🞙 */
    case 0x1f79a: return true; /* 🞚 */
    case 0x1f79b: return true; /* 🞛 */
    case 0x1f79c: return true; /* 🞜 */
    case 0x1f79d: return true; /* 🞝 */
    case 0x1f79e: return true; /* 🞞 */
    case 0x1f79f: return true; /* 🞟 */
    case 0x1f7a0: return true; /* 🞠 */
    case 0x1f7a1: return true; /* 🞡 */
    case 0x1f7a2: return true; /* 🞢 */
    case 0x1f7a3: return true; /* 🞣 */
    case 0x1f7a4: return true; /* 🞤 */
    case 0x1f7a5: return true; /* 🞥 */
    case 0x1f7a6: return true; /* 🞦 */
    case 0x1f7a7: return true; /* 🞧 */
    case 0x1f7a8: return true; /* 🞨 */
    case 0x1f7a9: return true; /* 🞩 */
    case 0x1f7aa: return true; /* 🞪 */
    case 0x1f7ab: return true; /* 🞫 */
    case 0x1f7ac: return true; /* 🞬 */
    case 0x1f7ad: return true; /* 🞭 */
    case 0x1f7ae: return true; /* 🞮 */
    case 0x1f7af: return true; /* 🞯 */
    case 0x1f7b0: return true; /* 🞰 */
    case 0x1f7b1: return true; /* 🞱 */
    case 0x1f7b2: return true; /* 🞲 */
    case 0x1f7b3: return true; /* 🞳 */
    case 0x1f7b4: return true; /* 🞴 */
    case 0x1f7b5: return true; /* 🞵 */
    case 0x1f7b6: return true; /* 🞶 */
    case 0x1f7b7: return true; /* 🞷 */
    case 0x1f7b8: return true; /* 🞸 */
    case 0x1f7b9: return true; /* 🞹 */
    case 0x1f7ba: return true; /* 🞺 */
    case 0x1f7bb: return true; /* 🞻 */
    case 0x1f7bc: return true; /* 🞼 */
    case 0x1f7bd: return true; /* 🞽 */
    case 0x1f7be: return true; /* 🞾 */
    case 0x1f7bf: return true; /* 🞿 */
    case 0x1f7c0: return true; /* 🟀 */
    case 0x1f7c1: return true; /* 🟁 */
    case 0x1f7c2: return true; /* 🟂 */
    case 0x1f7c3: return true; /* 🟃 */
    case 0x1f7c4: return true; /* 🟄 */
    case 0x1f7c5: return true; /* 🟅 */
    case 0x1f7c6: return true; /* 🟆 */
    case 0x1f7c7: return true; /* 🟇 */
    case 0x1f7c8: return true; /* 🟈 */
    case 0x1f7c9: return true; /* 🟉 */
    case 0x1f7ca: return true; /* 🟊 */
    case 0x1f7cb: return true; /* 🟋 */
    case 0x1f7cc: return true; /* 🟌 */
    case 0x1f7cd: return true; /* 🟍 */
    case 0x1f7ce: return true; /* 🟎 */
    case 0x1f7cf: return true; /* 🟏 */
    case 0x1f7d0: return true; /* 🟐 */
    case 0x1f7d1: return true; /* 🟑 */
    case 0x1f7d2: return true; /* 🟒 */
    case 0x1f7d3: return true; /* 🟓 */
    case 0x1f7d4: return true; /* 🟔 */
    case 0x1f7d5: return true; /* 🟕 */
    case 0x1f7d6: return true; /* 🟖 */
    case 0x1f7d7: return true; /* 🟗 */
    case 0x1f7d8: return true; /* 🟘 */
    case 0x1f7e0: return true; /* 🟠 */
    case 0x1f7e1: return true; /* 🟡 */
    case 0x1f7e2: return true; /* 🟢 */
    case 0x1f7e3: return true; /* 🟣 */
    case 0x1f7e4: return true; /* 🟤 */
    case 0x1f7e5: return true; /* 🟥 */
    case 0x1f7e6: return true; /* 🟦 */
    case 0x1f7e7: return true; /* 🟧 */
    case 0x1f7e8: return true; /* 🟨 */
    case 0x1f7e9: return true; /* 🟩 */
    case 0x1f7ea: return true; /* 🟪 */
    case 0x1f7eb: return true; /* 🟫 */
    case 0x1f800: return true; /* 🠀 */
    case 0x1f801: return true; /* 🠁 */
    case 0x1f802: return true; /* 🠂 */
    case 0x1f803: return true; /* 🠃 */
    case 0x1f804: return true; /* 🠄 */
    case 0x1f805: return true; /* 🠅 */
    case 0x1f806: return true; /* 🠆 */
    case 0x1f807: return true; /* 🠇 */
    case 0x1f808: return true; /* 🠈 */
    case 0x1f809: return true; /* 🠉 */
    case 0x1f80a: return true; /* 🠊 */
    case 0x1f80b: return true; /* 🠋 */
    case 0x1f810: return true; /* 🠐 */
    case 0x1f811: return true; /* 🠑 */
    case 0x1f812: return true; /* 🠒 */
    case 0x1f813: return true; /* 🠓 */
    case 0x1f814: return true; /* 🠔 */
    case 0x1f815: return true; /* 🠕 */
    case 0x1f816: return true; /* 🠖 */
    case 0x1f817: return true; /* 🠗 */
    case 0x1f818: return true; /* 🠘 */
    case 0x1f819: return true; /* 🠙 */
    case 0x1f81a: return true; /* 🠚 */
    case 0x1f81b: return true; /* 🠛 */
    case 0x1f81c: return true; /* 🠜 */
    case 0x1f81d: return true; /* 🠝 */
    case 0x1f81e: return true; /* 🠞 */
    case 0x1f81f: return true; /* 🠟 */
    case 0x1f820: return true; /* 🠠 */
    case 0x1f821: return true; /* 🠡 */
    case 0x1f822: return true; /* 🠢 */
    case 0x1f823: return true; /* 🠣 */
    case 0x1f824: return true; /* 🠤 */
    case 0x1f825: return true; /* 🠥 */
    case 0x1f826: return true; /* 🠦 */
    case 0x1f827: return true; /* 🠧 */
    case 0x1f828: return true; /* 🠨 */
    case 0x1f829: return true; /* 🠩 */
    case 0x1f82a: return true; /* 🠪 */
    case 0x1f82b: return true; /* 🠫 */
    case 0x1f82c: return true; /* 🠬 */
    case 0x1f82d: return true; /* 🠭 */
    case 0x1f82e: return true; /* 🠮 */
    case 0x1f82f: return true; /* 🠯 */
    case 0x1f830: return true; /* 🠰 */
    case 0x1f831: return true; /* 🠱 */
    case 0x1f832: return true; /* 🠲 */
    case 0x1f833: return true; /* 🠳 */
    case 0x1f834: return true; /* 🠴 */
    case 0x1f835: return true; /* 🠵 */
    case 0x1f836: return true; /* 🠶 */
    case 0x1f837: return true; /* 🠷 */
    case 0x1f838: return true; /* 🠸 */
    case 0x1f839: return true; /* 🠹 */
    case 0x1f83a: return true; /* 🠺 */
    case 0x1f83b: return true; /* 🠻 */
    case 0x1f83c: return true; /* 🠼 */
    case 0x1f83d: return true; /* 🠽 */
    case 0x1f83e: return true; /* 🠾 */
    case 0x1f83f: return true; /* 🠿 */
    case 0x1f840: return true; /* 🡀 */
    case 0x1f841: return true; /* 🡁 */
    case 0x1f842: return true; /* 🡂 */
    case 0x1f843: return true; /* 🡃 */
    case 0x1f844: return true; /* 🡄 */
    case 0x1f845: return true; /* 🡅 */
    case 0x1f846: return true; /* 🡆 */
    case 0x1f847: return true; /* 🡇 */
    case 0x1f850: return true; /* 🡐 */
    case 0x1f851: return true; /* 🡑 */
    case 0x1f852: return true; /* 🡒 */
    case 0x1f853: return true; /* 🡓 */
    case 0x1f854: return true; /* 🡔 */
    case 0x1f855: return true; /* 🡕 */
    case 0x1f856: return true; /* 🡖 */
    case 0x1f857: return true; /* 🡗 */
    case 0x1f858: return true; /* 🡘 */
    case 0x1f859: return true; /* 🡙 */
    case 0x1f860: return true; /* 🡠 */
    case 0x1f861: return true; /* 🡡 */
    case 0x1f862: return true; /* 🡢 */
    case 0x1f863: return true; /* 🡣 */
    case 0x1f864: return true; /* 🡤 */
    case 0x1f865: return true; /* 🡥 */
    case 0x1f866: return true; /* 🡦 */
    case 0x1f867: return true; /* 🡧 */
    case 0x1f868: return true; /* 🡨 */
    case 0x1f869: return true; /* 🡩 */
    case 0x1f86a: return true; /* 🡪 */
    case 0x1f86b: return true; /* 🡫 */
    case 0x1f86c: return true; /* 🡬 */
    case 0x1f86d: return true; /* 🡭 */
    case 0x1f86e: return true; /* 🡮 */
    case 0x1f86f: return true; /* 🡯 */
    case 0x1f870: return true; /* 🡰 */
    case 0x1f871: return true; /* 🡱 */
    case 0x1f872: return true; /* 🡲 */
    case 0x1f873: return true; /* 🡳 */
    case 0x1f874: return true; /* 🡴 */
    case 0x1f875: return true; /* 🡵 */
    case 0x1f876: return true; /* 🡶 */
    case 0x1f877: return true; /* 🡷 */
    case 0x1f878: return true; /* 🡸 */
    case 0x1f879: return true; /* 🡹 */
    case 0x1f87a: return true; /* 🡺 */
    case 0x1f87b: return true; /* 🡻 */
    case 0x1f87c: return true; /* 🡼 */
    case 0x1f87d: return true; /* 🡽 */
    case 0x1f87e: return true; /* 🡾 */
    case 0x1f87f: return true; /* 🡿 */
    case 0x1f880: return true; /* 🢀 */
    case 0x1f881: return true; /* 🢁 */
    case 0x1f882: return true; /* 🢂 */
    case 0x1f883: return true; /* 🢃 */
    case 0x1f884: return true; /* 🢄 */
    case 0x1f885: return true; /* 🢅 */
    case 0x1f886: return true; /* 🢆 */
    case 0x1f887: return true; /* 🢇 */
    case 0x1f890: return true; /* 🢐 */
    case 0x1f891: return true; /* 🢑 */
    case 0x1f892: return true; /* 🢒 */
    case 0x1f893: return true; /* 🢓 */
    case 0x1f894: return true; /* 🢔 */
    case 0x1f895: return true; /* 🢕 */
    case 0x1f896: return true; /* 🢖 */
    case 0x1f897: return true; /* 🢗 */
    case 0x1f898: return true; /* 🢘 */
    case 0x1f899: return true; /* 🢙 */
    case 0x1f89a: return true; /* 🢚 */
    case 0x1f89b: return true; /* 🢛 */
    case 0x1f89c: return true; /* 🢜 */
    case 0x1f89d: return true; /* 🢝 */
    case 0x1f89e: return true; /* 🢞 */
    case 0x1f89f: return true; /* 🢟 */
    case 0x1f8a0: return true; /* 🢠 */
    case 0x1f8a1: return true; /* 🢡 */
    case 0x1f8a2: return true; /* 🢢 */
    case 0x1f8a3: return true; /* 🢣 */
    case 0x1f8a4: return true; /* 🢤 */
    case 0x1f8a5: return true; /* 🢥 */
    case 0x1f8a6: return true; /* 🢦 */
    case 0x1f8a7: return true; /* 🢧 */
    case 0x1f8a8: return true; /* 🢨 */
    case 0x1f8a9: return true; /* 🢩 */
    case 0x1f8aa: return true; /* 🢪 */
    case 0x1f8ab: return true; /* 🢫 */
    case 0x1f8ac: return true; /* 🢬 */
    case 0x1f8ad: return true; /* 🢭 */
    case 0x1f8b0: return true; /* 🢰 */
    case 0x1f8b1: return true; /* 🢱 */
    case 0x1f900: return true; /* 🤀 */
    case 0x1f901: return true; /* 🤁 */
    case 0x1f902: return true; /* 🤂 */
    case 0x1f903: return true; /* 🤃 */
    case 0x1f904: return true; /* 🤄 */
    case 0x1f905: return true; /* 🤅 */
    case 0x1f906: return true; /* 🤆 */
    case 0x1f907: return true; /* 🤇 */
    case 0x1f908: return true; /* 🤈 */
    case 0x1f909: return true; /* 🤉 */
    case 0x1f90a: return true; /* 🤊 */
    case 0x1f90b: return true; /* 🤋 */
    case 0x1f90c: return true; /* 🤌 */
    case 0x1f90d: return true; /* 🤍 */
    case 0x1f90e: return true; /* 🤎 */
    case 0x1f90f: return true; /* 🤏 */
    case 0x1f910: return true; /* 🤐 */
    case 0x1f911: return true; /* 🤑 */
    case 0x1f912: return true; /* 🤒 */
    case 0x1f913: return true; /* 🤓 */
    case 0x1f914: return true; /* 🤔 */
    case 0x1f915: return true; /* 🤕 */
    case 0x1f916: return true; /* 🤖 */
    case 0x1f917: return true; /* 🤗 */
    case 0x1f918: return true; /* 🤘 */
    case 0x1f919: return true; /* 🤙 */
    case 0x1f91a: return true; /* 🤚 */
    case 0x1f91b: return true; /* 🤛 */
    case 0x1f91c: return true; /* 🤜 */
    case 0x1f91d: return true; /* 🤝 */
    case 0x1f91e: return true; /* 🤞 */
    case 0x1f91f: return true; /* 🤟 */
    case 0x1f920: return true; /* 🤠 */
    case 0x1f921: return true; /* 🤡 */
    case 0x1f922: return true; /* 🤢 */
    case 0x1f923: return true; /* 🤣 */
    case 0x1f924: return true; /* 🤤 */
    case 0x1f925: return true; /* 🤥 */
    case 0x1f926: return true; /* 🤦 */
    case 0x1f927: return true; /* 🤧 */
    case 0x1f928: return true; /* 🤨 */
    case 0x1f929: return true; /* 🤩 */
    case 0x1f92a: return true; /* 🤪 */
    case 0x1f92b: return true; /* 🤫 */
    case 0x1f92c: return true; /* 🤬 */
    case 0x1f92d: return true; /* 🤭 */
    case 0x1f92e: return true; /* 🤮 */
    case 0x1f92f: return true; /* 🤯 */
    case 0x1f930: return true; /* 🤰 */
    case 0x1f931: return true; /* 🤱 */
    case 0x1f932: return true; /* 🤲 */
    case 0x1f933: return true; /* 🤳 */
    case 0x1f934: return true; /* 🤴 */
    case 0x1f935: return true; /* 🤵 */
    case 0x1f936: return true; /* 🤶 */
    case 0x1f937: return true; /* 🤷 */
    case 0x1f938: return true; /* 🤸 */
    case 0x1f939: return true; /* 🤹 */
    case 0x1f93a: return true; /* 🤺 */
    case 0x1f93b: return true; /* 🤻 */
    case 0x1f93c: return true; /* 🤼 */
    case 0x1f93d: return true; /* 🤽 */
    case 0x1f93e: return true; /* 🤾 */
    case 0x1f93f: return true; /* 🤿 */
    case 0x1f940: return true; /* 🥀 */
    case 0x1f941: return true; /* 🥁 */
    case 0x1f942: return true; /* 🥂 */
    case 0x1f943: return true; /* 🥃 */
    case 0x1f944: return true; /* 🥄 */
    case 0x1f945: return true; /* 🥅 */
    case 0x1f946: return true; /* 🥆 */
    case 0x1f947: return true; /* 🥇 */
    case 0x1f948: return true; /* 🥈 */
    case 0x1f949: return true; /* 🥉 */
    case 0x1f94a: return true; /* 🥊 */
    case 0x1f94b: return true; /* 🥋 */
    case 0x1f94c: return true; /* 🥌 */
    case 0x1f94d: return true; /* 🥍 */
    case 0x1f94e: return true; /* 🥎 */
    case 0x1f94f: return true; /* 🥏 */
    case 0x1f950: return true; /* 🥐 */
    case 0x1f951: return true; /* 🥑 */
    case 0x1f952: return true; /* 🥒 */
    case 0x1f953: return true; /* 🥓 */
    case 0x1f954: return true; /* 🥔 */
    case 0x1f955: return true; /* 🥕 */
    case 0x1f956: return true; /* 🥖 */
    case 0x1f957: return true; /* 🥗 */
    case 0x1f958: return true; /* 🥘 */
    case 0x1f959: return true; /* 🥙 */
    case 0x1f95a: return true; /* 🥚 */
    case 0x1f95b: return true; /* 🥛 */
    case 0x1f95c: return true; /* 🥜 */
    case 0x1f95d: return true; /* 🥝 */
    case 0x1f95e: return true; /* 🥞 */
    case 0x1f95f: return true; /* 🥟 */
    case 0x1f960: return true; /* 🥠 */
    case 0x1f961: return true; /* 🥡 */
    case 0x1f962: return true; /* 🥢 */
    case 0x1f963: return true; /* 🥣 */
    case 0x1f964: return true; /* 🥤 */
    case 0x1f965: return true; /* 🥥 */
    case 0x1f966: return true; /* 🥦 */
    case 0x1f967: return true; /* 🥧 */
    case 0x1f968: return true; /* 🥨 */
    case 0x1f969: return true; /* 🥩 */
    case 0x1f96a: return true; /* 🥪 */
    case 0x1f96b: return true; /* 🥫 */
    case 0x1f96c: return true; /* 🥬 */
    case 0x1f96d: return true; /* 🥭 */
    case 0x1f96e: return true; /* 🥮 */
    case 0x1f96f: return true; /* 🥯 */
    case 0x1f970: return true; /* 🥰 */
    case 0x1f971: return true; /* 🥱 */
    case 0x1f972: return true; /* 🥲 */
    case 0x1f973: return true; /* 🥳 */
    case 0x1f974: return true; /* 🥴 */
    case 0x1f975: return true; /* 🥵 */
    case 0x1f976: return true; /* 🥶 */
    case 0x1f977: return true; /* 🥷 */
    case 0x1f978: return true; /* 🥸 */
    case 0x1f97a: return true; /* 🥺 */
    case 0x1f97b: return true; /* 🥻 */
    case 0x1f97c: return true; /* 🥼 */
    case 0x1f97d: return true; /* 🥽 */
    case 0x1f97e: return true; /* 🥾 */
    case 0x1f97f: return true; /* 🥿 */
    case 0x1f980: return true; /* 🦀 */
    case 0x1f981: return true; /* 🦁 */
    case 0x1f982: return true; /* 🦂 */
    case 0x1f983: return true; /* 🦃 */
    case 0x1f984: return true; /* 🦄 */
    case 0x1f985: return true; /* 🦅 */
    case 0x1f986: return true; /* 🦆 */
    case 0x1f987: return true; /* 🦇 */
    case 0x1f988: return true; /* 🦈 */
    case 0x1f989: return true; /* 🦉 */
    case 0x1f98a: return true; /* 🦊 */
    case 0x1f98b: return true; /* 🦋 */
    case 0x1f98c: return true; /* 🦌 */
    case 0x1f98d: return true; /* 🦍 */
    case 0x1f98e: return true; /* 🦎 */
    case 0x1f98f: return true; /* 🦏 */
    case 0x1f990: return true; /* 🦐 */
    case 0x1f991: return true; /* 🦑 */
    case 0x1f992: return true; /* 🦒 */
    case 0x1f993: return true; /* 🦓 */
    case 0x1f994: return true; /* 🦔 */
    case 0x1f995: return true; /* 🦕 */
    case 0x1f996: return true; /* 🦖 */
    case 0x1f997: return true; /* 🦗 */
    case 0x1f998: return true; /* 🦘 */
    case 0x1f999: return true; /* 🦙 */
    case 0x1f99a: return true; /* 🦚 */
    case 0x1f99b: return true; /* 🦛 */
    case 0x1f99c: return true; /* 🦜 */
    case 0x1f99d: return true; /* 🦝 */
    case 0x1f99e: return true; /* 🦞 */
    case 0x1f99f: return true; /* 🦟 */
    case 0x1f9a0: return true; /* 🦠 */
    case 0x1f9a1: return true; /* 🦡 */
    case 0x1f9a2: return true; /* 🦢 */
    case 0x1f9a3: return true; /* 🦣 */
    case 0x1f9a4: return true; /* 🦤 */
    case 0x1f9a5: return true; /* 🦥 */
    case 0x1f9a6: return true; /* 🦦 */
    case 0x1f9a7: return true; /* 🦧 */
    case 0x1f9a8: return true; /* 🦨 */
    case 0x1f9a9: return true; /* 🦩 */
    case 0x1f9aa: return true; /* 🦪 */
    case 0x1f9ab: return true; /* 🦫 */
    case 0x1f9ac: return true; /* 🦬 */
    case 0x1f9ad: return true; /* 🦭 */
    case 0x1f9ae: return true; /* 🦮 */
    case 0x1f9af: return true; /* 🦯 */
    case 0x1f9b0: return true; /* 🦰 */
    case 0x1f9b1: return true; /* 🦱 */
    case 0x1f9b2: return true; /* 🦲 */
    case 0x1f9b3: return true; /* 🦳 */
    case 0x1f9b4: return true; /* 🦴 */
    case 0x1f9b5: return true; /* 🦵 */
    case 0x1f9b6: return true; /* 🦶 */
    case 0x1f9b7: return true; /* 🦷 */
    case 0x1f9b8: return true; /* 🦸 */
    case 0x1f9b9: return true; /* 🦹 */
    case 0x1f9ba: return true; /* 🦺 */
    case 0x1f9bb: return true; /* 🦻 */
    case 0x1f9bc: return true; /* 🦼 */
    case 0x1f9bd: return true; /* 🦽 */
    case 0x1f9be: return true; /* 🦾 */
    case 0x1f9bf: return true; /* 🦿 */
    case 0x1f9c0: return true; /* 🧀 */
    case 0x1f9c1: return true; /* 🧁 */
    case 0x1f9c2: return true; /* 🧂 */
    case 0x1f9c3: return true; /* 🧃 */
    case 0x1f9c4: return true; /* 🧄 */
    case 0x1f9c5: return true; /* 🧅 */
    case 0x1f9c6: return true; /* 🧆 */
    case 0x1f9c7: return true; /* 🧇 */
    case 0x1f9c8: return true; /* 🧈 */
    case 0x1f9c9: return true; /* 🧉 */
    case 0x1f9ca: return true; /* 🧊 */
    case 0x1f9cb: return true; /* 🧋 */
    case 0x1f9cd: return true; /* 🧍 */
    case 0x1f9ce: return true; /* 🧎 */
    case 0x1f9cf: return true; /* 🧏 */
    case 0x1f9d0: return true; /* 🧐 */
    case 0x1f9d1: return true; /* 🧑 */
    case 0x1f9d2: return true; /* 🧒 */
    case 0x1f9d3: return true; /* 🧓 */
    case 0x1f9d4: return true; /* 🧔 */
    case 0x1f9d5: return true; /* 🧕 */
    case 0x1f9d6: return true; /* 🧖 */
    case 0x1f9d7: return true; /* 🧗 */
    case 0x1f9d8: return true; /* 🧘 */
    case 0x1f9d9: return true; /* 🧙 */
    case 0x1f9da: return true; /* 🧚 */
    case 0x1f9db: return true; /* 🧛 */
    case 0x1f9dc: return true; /* 🧜 */
    case 0x1f9dd: return true; /* 🧝 */
    case 0x1f9de: return true; /* 🧞 */
    case 0x1f9df: return true; /* 🧟 */
    case 0x1f9e0: return true; /* 🧠 */
    case 0x1f9e1: return true; /* 🧡 */
    case 0x1f9e2: return true; /* 🧢 */
    case 0x1f9e3: return true; /* 🧣 */
    case 0x1f9e4: return true; /* 🧤 */
    case 0x1f9e5: return true; /* 🧥 */
    case 0x1f9e6: return true; /* 🧦 */
    case 0x1f9e7: return true; /* 🧧 */
    case 0x1f9e8: return true; /* 🧨 */
    case 0x1f9e9: return true; /* 🧩 */
    case 0x1f9ea: return true; /* 🧪 */
    case 0x1f9eb: return true; /* 🧫 */
    case 0x1f9ec: return true; /* 🧬 */
    case 0x1f9ed: return true; /* 🧭 */
    case 0x1f9ee: return true; /* 🧮 */
    case 0x1f9ef: return true; /* 🧯 */
    case 0x1f9f0: return true; /* 🧰 */
    case 0x1f9f1: return true; /* 🧱 */
    case 0x1f9f2: return true; /* 🧲 */
    case 0x1f9f3: return true; /* 🧳 */
    case 0x1f9f4: return true; /* 🧴 */
    case 0x1f9f5: return true; /* 🧵 */
    case 0x1f9f6: return true; /* 🧶 */
    case 0x1f9f7: return true; /* 🧷 */
    case 0x1f9f8: return true; /* 🧸 */
    case 0x1f9f9: return true; /* 🧹 */
    case 0x1f9fa: return true; /* 🧺 */
    case 0x1f9fb: return true; /* 🧻 */
    case 0x1f9fc: return true; /* 🧼 */
    case 0x1f9fd: return true; /* 🧽 */
    case 0x1f9fe: return true; /* 🧾 */
    case 0x1f9ff: return true; /* 🧿 */
    case 0x1fa00: return true; /* 🨀 */
    case 0x1fa01: return true; /* 🨁 */
    case 0x1fa02: return true; /* 🨂 */
    case 0x1fa03: return true; /* 🨃 */
    case 0x1fa04: return true; /* 🨄 */
    case 0x1fa05: return true; /* 🨅 */
    case 0x1fa06: return true; /* 🨆 */
    case 0x1fa07: return true; /* 🨇 */
    case 0x1fa08: return true; /* 🨈 */
    case 0x1fa09: return true; /* 🨉 */
    case 0x1fa0a: return true; /* 🨊 */
    case 0x1fa0b: return true; /* 🨋 */
    case 0x1fa0c: return true; /* 🨌 */
    case 0x1fa0d: return true; /* 🨍 */
    case 0x1fa0e: return true; /* 🨎 */
    case 0x1fa0f: return true; /* 🨏 */
    case 0x1fa10: return true; /* 🨐 */
    case 0x1fa11: return true; /* 🨑 */
    case 0x1fa12: return true; /* 🨒 */
    case 0x1fa13: return true; /* 🨓 */
    case 0x1fa14: return true; /* 🨔 */
    case 0x1fa15: return true; /* 🨕 */
    case 0x1fa16: return true; /* 🨖 */
    case 0x1fa17: return true; /* 🨗 */
    case 0x1fa18: return true; /* 🨘 */
    case 0x1fa19: return true; /* 🨙 */
    case 0x1fa1a: return true; /* 🨚 */
    case 0x1fa1b: return true; /* 🨛 */
    case 0x1fa1c: return true; /* 🨜 */
    case 0x1fa1d: return true; /* 🨝 */
    case 0x1fa1e: return true; /* 🨞 */
    case 0x1fa1f: return true; /* 🨟 */
    case 0x1fa20: return true; /* 🨠 */
    case 0x1fa21: return true; /* 🨡 */
    case 0x1fa22: return true; /* 🨢 */
    case 0x1fa23: return true; /* 🨣 */
    case 0x1fa24: return true; /* 🨤 */
    case 0x1fa25: return true; /* 🨥 */
    case 0x1fa26: return true; /* 🨦 */
    case 0x1fa27: return true; /* 🨧 */
    case 0x1fa28: return true; /* 🨨 */
    case 0x1fa29: return true; /* 🨩 */
    case 0x1fa2a: return true; /* 🨪 */
    case 0x1fa2b: return true; /* 🨫 */
    case 0x1fa2c: return true; /* 🨬 */
    case 0x1fa2d: return true; /* 🨭 */
    case 0x1fa2e: return true; /* 🨮 */
    case 0x1fa2f: return true; /* 🨯 */
    case 0x1fa30: return true; /* 🨰 */
    case 0x1fa31: return true; /* 🨱 */
    case 0x1fa32: return true; /* 🨲 */
    case 0x1fa33: return true; /* 🨳 */
    case 0x1fa34: return true; /* 🨴 */
    case 0x1fa35: return true; /* 🨵 */
    case 0x1fa36: return true; /* 🨶 */
    case 0x1fa37: return true; /* 🨷 */
    case 0x1fa38: return true; /* 🨸 */
    case 0x1fa39: return true; /* 🨹 */
    case 0x1fa3a: return true; /* 🨺 */
    case 0x1fa3b: return true; /* 🨻 */
    case 0x1fa3c: return true; /* 🨼 */
    case 0x1fa3d: return true; /* 🨽 */
    case 0x1fa3e: return true; /* 🨾 */
    case 0x1fa3f: return true; /* 🨿 */
    case 0x1fa40: return true; /* 🩀 */
    case 0x1fa41: return true; /* 🩁 */
    case 0x1fa42: return true; /* 🩂 */
    case 0x1fa43: return true; /* 🩃 */
    case 0x1fa44: return true; /* 🩄 */
    case 0x1fa45: return true; /* 🩅 */
    case 0x1fa46: return true; /* 🩆 */
    case 0x1fa47: return true; /* 🩇 */
    case 0x1fa48: return true; /* 🩈 */
    case 0x1fa49: return true; /* 🩉 */
    case 0x1fa4a: return true; /* 🩊 */
    case 0x1fa4b: return true; /* 🩋 */
    case 0x1fa4c: return true; /* 🩌 */
    case 0x1fa4d: return true; /* 🩍 */
    case 0x1fa4e: return true; /* 🩎 */
    case 0x1fa4f: return true; /* 🩏 */
    case 0x1fa50: return true; /* 🩐 */
    case 0x1fa51: return true; /* 🩑 */
    case 0x1fa52: return true; /* 🩒 */
    case 0x1fa53: return true; /* 🩓 */
    case 0x1fa60: return true; /* 🩠 */
    case 0x1fa61: return true; /* 🩡 */
    case 0x1fa62: return true; /* 🩢 */
    case 0x1fa63: return true; /* 🩣 */
    case 0x1fa64: return true; /* 🩤 */
    case 0x1fa65: return true; /* 🩥 */
    case 0x1fa66: return true; /* 🩦 */
    case 0x1fa67: return true; /* 🩧 */
    case 0x1fa68: return true; /* 🩨 */
    case 0x1fa69: return true; /* 🩩 */
    case 0x1fa6a: return true; /* 🩪 */
    case 0x1fa6b: return true; /* 🩫 */
    case 0x1fa6c: return true; /* 🩬 */
    case 0x1fa6d: return true; /* 🩭 */
    case 0x1fa70: return true; /* 🩰 */
    case 0x1fa71: return true; /* 🩱 */
    case 0x1fa72: return true; /* 🩲 */
    case 0x1fa73: return true; /* 🩳 */
    case 0x1fa74: return true; /* 🩴 */
    case 0x1fa78: return true; /* 🩸 */
    case 0x1fa79: return true; /* 🩹 */
    case 0x1fa7a: return true; /* 🩺 */
    case 0x1fa80: return true; /* 🪀 */
    case 0x1fa81: return true; /* 🪁 */
    case 0x1fa82: return true; /* 🪂 */
    case 0x1fa83: return true; /* 🪃 */
    case 0x1fa84: return true; /* 🪄 */
    case 0x1fa85: return true; /* 🪅 */
    case 0x1fa86: return true; /* 🪆 */
    case 0x1fa90: return true; /* 🪐 */
    case 0x1fa91: return true; /* 🪑 */
    case 0x1fa92: return true; /* 🪒 */
    case 0x1fa93: return true; /* 🪓 */
    case 0x1fa94: return true; /* 🪔 */
    case 0x1fa95: return true; /* 🪕 */
    case 0x1fa96: return true; /* 🪖 */
    case 0x1fa97: return true; /* 🪗 */
    case 0x1fa98: return true; /* 🪘 */
    case 0x1fa99: return true; /* 🪙 */
    case 0x1fa9a: return true; /* 🪚 */
    case 0x1fa9b: return true; /* 🪛 */
    case 0x1fa9c: return true; /* 🪜 */
    case 0x1fa9d: return true; /* 🪝 */
    case 0x1fa9e: return true; /* 🪞 */
    case 0x1fa9f: return true; /* 🪟 */
    case 0x1faa0: return true; /* 🪠 */
    case 0x1faa1: return true; /* 🪡 */
    case 0x1faa2: return true; /* 🪢 */
    case 0x1faa3: return true; /* 🪣 */
    case 0x1faa4: return true; /* 🪤 */
    case 0x1faa5: return true; /* 🪥 */
    case 0x1faa6: return true; /* 🪦 */
    case 0x1faa7: return true; /* 🪧 */
    case 0x1faa8: return true; /* 🪨 */
    case 0x1fab0: return true; /* 🪰 */
    case 0x1fab1: return true; /* 🪱 */
    case 0x1fab2: return true; /* 🪲 */
    case 0x1fab3: return true; /* 🪳 */
    case 0x1fab4: return true; /* 🪴 */
    case 0x1fab5: return true; /* 🪵 */
    case 0x1fab6: return true; /* 🪶 */
    case 0x1fac0: return true; /* 🫀 */
    case 0x1fac1: return true; /* 🫁 */
    case 0x1fac2: return true; /* 🫂 */
    case 0x1fad0: return true; /* 🫐 */
    case 0x1fad1: return true; /* 🫑 */
    case 0x1fad2: return true; /* 🫒 */
    case 0x1fad3: return true; /* 🫓 */
    case 0x1fad4: return true; /* 🫔 */
    case 0x1fad5: return true; /* 🫕 */
    case 0x1fad6: return true; /* 🫖 */
    case 0x1fb00: return true; /* 🬀 */
    case 0x1fb01: return true; /* 🬁 */
    case 0x1fb02: return true; /* 🬂 */
    case 0x1fb03: return true; /* 🬃 */
    case 0x1fb04: return true; /* 🬄 */
    case 0x1fb05: return true; /* 🬅 */
    case 0x1fb06: return true; /* 🬆 */
    case 0x1fb07: return true; /* 🬇 */
    case 0x1fb08: return true; /* 🬈 */
    case 0x1fb09: return true; /* 🬉 */
    case 0x1fb0a: return true; /* 🬊 */
    case 0x1fb0b: return true; /* 🬋 */
    case 0x1fb0c: return true; /* 🬌 */
    case 0x1fb0d: return true; /* 🬍 */
    case 0x1fb0e: return true; /* 🬎 */
    case 0x1fb0f: return true; /* 🬏 */
    case 0x1fb10: return true; /* 🬐 */
    case 0x1fb11: return true; /* 🬑 */
    case 0x1fb12: return true; /* 🬒 */
    case 0x1fb13: return true; /* 🬓 */
    case 0x1fb14: return true; /* 🬔 */
    case 0x1fb15: return true; /* 🬕 */
    case 0x1fb16: return true; /* 🬖 */
    case 0x1fb17: return true; /* 🬗 */
    case 0x1fb18: return true; /* 🬘 */
    case 0x1fb19: return true; /* 🬙 */
    case 0x1fb1a: return true; /* 🬚 */
    case 0x1fb1b: return true; /* 🬛 */
    case 0x1fb1c: return true; /* 🬜 */
    case 0x1fb1d: return true; /* 🬝 */
    case 0x1fb1e: return true; /* 🬞 */
    case 0x1fb1f: return true; /* 🬟 */
    case 0x1fb20: return true; /* 🬠 */
    case 0x1fb21: return true; /* 🬡 */
    case 0x1fb22: return true; /* 🬢 */
    case 0x1fb23: return true; /* 🬣 */
    case 0x1fb24: return true; /* 🬤 */
    case 0x1fb25: return true; /* 🬥 */
    case 0x1fb26: return true; /* 🬦 */
    case 0x1fb27: return true; /* 🬧 */
    case 0x1fb28: return true; /* 🬨 */
    case 0x1fb29: return true; /* 🬩 */
    case 0x1fb2a: return true; /* 🬪 */
    case 0x1fb2b: return true; /* 🬫 */
    case 0x1fb2c: return true; /* 🬬 */
    case 0x1fb2d: return true; /* 🬭 */
    case 0x1fb2e: return true; /* 🬮 */
    case 0x1fb2f: return true; /* 🬯 */
    case 0x1fb30: return true; /* 🬰 */
    case 0x1fb31: return true; /* 🬱 */
    case 0x1fb32: return true; /* 🬲 */
    case 0x1fb33: return true; /* 🬳 */
    case 0x1fb34: return true; /* 🬴 */
    case 0x1fb35: return true; /* 🬵 */
    case 0x1fb36: return true; /* 🬶 */
    case 0x1fb37: return true; /* 🬷 */
    case 0x1fb38: return true; /* 🬸 */
    case 0x1fb39: return true; /* 🬹 */
    case 0x1fb3a: return true; /* 🬺 */
    case 0x1fb3b: return true; /* 🬻 */
    case 0x1fb3c: return true; /* 🬼 */
    case 0x1fb3d: return true; /* 🬽 */
    case 0x1fb3e: return true; /* 🬾 */
    case 0x1fb3f: return true; /* 🬿 */
    case 0x1fb40: return true; /* 🭀 */
    case 0x1fb41: return true; /* 🭁 */
    case 0x1fb42: return true; /* 🭂 */
    case 0x1fb43: return true; /* 🭃 */
    case 0x1fb44: return true; /* 🭄 */
    case 0x1fb45: return true; /* 🭅 */
    case 0x1fb46: return true; /* 🭆 */
    case 0x1fb47: return true; /* 🭇 */
    case 0x1fb48: return true; /* 🭈 */
    case 0x1fb49: return true; /* 🭉 */
    case 0x1fb4a: return true; /* 🭊 */
    case 0x1fb4b: return true; /* 🭋 */
    case 0x1fb4c: return true; /* 🭌 */
    case 0x1fb4d: return true; /* 🭍 */
    case 0x1fb4e: return true; /* 🭎 */
    case 0x1fb4f: return true; /* 🭏 */
    case 0x1fb50: return true; /* 🭐 */
    case 0x1fb51: return true; /* 🭑 */
    case 0x1fb52: return true; /* 🭒 */
    case 0x1fb53: return true; /* 🭓 */
    case 0x1fb54: return true; /* 🭔 */
    case 0x1fb55: return true; /* 🭕 */
    case 0x1fb56: return true; /* 🭖 */
    case 0x1fb57: return true; /* 🭗 */
    case 0x1fb58: return true; /* 🭘 */
    case 0x1fb59: return true; /* 🭙 */
    case 0x1fb5a: return true; /* 🭚 */
    case 0x1fb5b: return true; /* 🭛 */
    case 0x1fb5c: return true; /* 🭜 */
    case 0x1fb5d: return true; /* 🭝 */
    case 0x1fb5e: return true; /* 🭞 */
    case 0x1fb5f: return true; /* 🭟 */
    case 0x1fb60: return true; /* 🭠 */
    case 0x1fb61: return true; /* 🭡 */
    case 0x1fb62: return true; /* 🭢 */
    case 0x1fb63: return true; /* 🭣 */
    case 0x1fb64: return true; /* 🭤 */
    case 0x1fb65: return true; /* 🭥 */
    case 0x1fb66: return true; /* 🭦 */
    case 0x1fb67: return true; /* 🭧 */
    case 0x1fb68: return true; /* 🭨 */
    case 0x1fb69: return true; /* 🭩 */
    case 0x1fb6a: return true; /* 🭪 */
    case 0x1fb6b: return true; /* 🭫 */
    case 0x1fb6c: return true; /* 🭬 */
    case 0x1fb6d: return true; /* 🭭 */
    case 0x1fb6e: return true; /* 🭮 */
    case 0x1fb6f: return true; /* 🭯 */
    case 0x1fb70: return true; /* 🭰 */
    case 0x1fb71: return true; /* 🭱 */
    case 0x1fb72: return true; /* 🭲 */
    case 0x1fb73: return true; /* 🭳 */
    case 0x1fb74: return true; /* 🭴 */
    case 0x1fb75: return true; /* 🭵 */
    case 0x1fb76: return true; /* 🭶 */
    case 0x1fb77: return true; /* 🭷 */
    case 0x1fb78: return true; /* 🭸 */
    case 0x1fb79: return true; /* 🭹 */
    case 0x1fb7a: return true; /* 🭺 */
    case 0x1fb7b: return true; /* 🭻 */
    case 0x1fb7c: return true; /* 🭼 */
    case 0x1fb7d: return true; /* 🭽 */
    case 0x1fb7e: return true; /* 🭾 */
    case 0x1fb7f: return true; /* 🭿 */
    case 0x1fb80: return true; /* 🮀 */
    case 0x1fb81: return true; /* 🮁 */
    case 0x1fb82: return true; /* 🮂 */
    case 0x1fb83: return true; /* 🮃 */
    case 0x1fb84: return true; /* 🮄 */
    case 0x1fb85: return true; /* 🮅 */
    case 0x1fb86: return true; /* 🮆 */
    case 0x1fb87: return true; /* 🮇 */
    case 0x1fb88: return true; /* 🮈 */
    case 0x1fb89: return true; /* 🮉 */
    case 0x1fb8a: return true; /* 🮊 */
    case 0x1fb8b: return true; /* 🮋 */
    case 0x1fb8c: return true; /* 🮌 */
    case 0x1fb8d: return true; /* 🮍 */
    case 0x1fb8e: return true; /* 🮎 */
    case 0x1fb8f: return true; /* 🮏 */
    case 0x1fb90: return true; /* 🮐 */
    case 0x1fb91: return true; /* 🮑 */
    case 0x1fb92: return true; /* 🮒 */
    case 0x1fb94: return true; /* 🮔 */
    case 0x1fb95: return true; /* 🮕 */
    case 0x1fb96: return true; /* 🮖 */
    case 0x1fb97: return true; /* 🮗 */
    case 0x1fb98: return true; /* 🮘 */
    case 0x1fb99: return true; /* 🮙 */
    case 0x1fb9a: return true; /* 🮚 */
    case 0x1fb9b: return true; /* 🮛 */
    case 0x1fb9c: return true; /* 🮜 */
    case 0x1fb9d: return true; /* 🮝 */
    case 0x1fb9e: return true; /* 🮞 */
    case 0x1fb9f: return true; /* 🮟 */
    case 0x1fba0: return true; /* 🮠 */
    case 0x1fba1: return true; /* 🮡 */
    case 0x1fba2: return true; /* 🮢 */
    case 0x1fba3: return true; /* 🮣 */
    case 0x1fba4: return true; /* 🮤 */
    case 0x1fba5: return true; /* 🮥 */
    case 0x1fba6: return true; /* 🮦 */
    case 0x1fba7: return true; /* 🮧 */
    case 0x1fba8: return true; /* 🮨 */
    case 0x1fba9: return true; /* 🮩 */
    case 0x1fbaa: return true; /* 🮪 */
    case 0x1fbab: return true; /* 🮫 */
    case 0x1fbac: return true; /* 🮬 */
    case 0x1fbad: return true; /* 🮭 */
    case 0x1fbae: return true; /* 🮮 */
    case 0x1fbaf: return true; /* 🮯 */
    case 0x1fbb0: return true; /* 🮰 */
    case 0x1fbb1: return true; /* 🮱 */
    case 0x1fbb2: return true; /* 🮲 */
    case 0x1fbb3: return true; /* 🮳 */
    case 0x1fbb4: return true; /* 🮴 */
    case 0x1fbb5: return true; /* 🮵 */
    case 0x1fbb6: return true; /* 🮶 */
    case 0x1fbb7: return true; /* 🮷 */
    case 0x1fbb8: return true; /* 🮸 */
    case 0x1fbb9: return true; /* 🮹 */
    case 0x1fbba: return true; /* 🮺 */
    case 0x1fbbb: return true; /* 🮻 */
    case 0x1fbbc: return true; /* 🮼 */
    case 0x1fbbd: return true; /* 🮽 */
    case 0x1fbbe: return true; /* 🮾 */
    case 0x1fbbf: return true; /* 🮿 */
    case 0x1fbc0: return true; /* 🯀 */
    case 0x1fbc1: return true; /* 🯁 */
    case 0x1fbc2: return true; /* 🯂 */
    case 0x1fbc3: return true; /* 🯃 */
    case 0x1fbc4: return true; /* 🯄 */
    case 0x1fbc5: return true; /* 🯅 */
    case 0x1fbc6: return true; /* 🯆 */
    case 0x1fbc7: return true; /* 🯇 */
    case 0x1fbc8: return true; /* 🯈 */
    case 0x1fbc9: return true; /* 🯉 */
    case 0x1fbca: return true; /* 🯊 */
    default: return false;
  }
  return false;
}

#endif
